using System;
using System.Collections.Generic;
using WingServerCommon.Utilities.Executors;
using WingServerCommon.Log;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Entities;

namespace WingMongoDBModule
{
    /// <summary>
    /// 转Pdf队列
    /// </summary>
    public class ConvertPdfExecutor
    {
        private readonly SequenceExecutor<ConvertPdfInfo> _sequenceExecutor;
        private string ConvertVidCode { get; }

        public ConvertPdfExecutor(string convertPdfCode)
        {
            ConvertVidCode = convertPdfCode;
            _sequenceExecutor = new SequenceExecutor<ConvertPdfInfo>
            (convertPdfCode, TimeSpan.FromMilliseconds(100));
        }

        public void Add(Func<ConvertPdfInfo, bool> doConvertVid, ConvertPdfInfo labFileInfo)
        {
            _sequenceExecutor.Add(doConvertVid, labFileInfo);
            Logger.WriteLineInfo($"ConvertVidExecutor add  success,CourseCode:{labFileInfo.CourseCode},CreatorCode:{labFileInfo.Entity.CreatorCode}");
        }

        public void AddRange(Func<ConvertPdfInfo, bool> doConvertVid, List<ConvertPdfInfo> labFileInfoList)
        {
            foreach (var labFileInfo in labFileInfoList)
            {
                Add(doConvertVid, labFileInfo);
            }
        }

        public bool WorkerIsEmpty()
        {
            return _sequenceExecutor.WorkerCount == 0;
        }

        public void Dispose()
        {
            _sequenceExecutor.Cancel();
        }
    }

    /// <summary>
    /// 转Pdf对象
    /// </summary>
    public class ConvertPdfInfo
    {
        /// <summary>
        /// 数据实体
        /// </summary>
        /// <value></value>
        public VideoInfoEntity Entity { get; set; }

        /// <summary>
        /// 关联课程编码
        /// </summary>
        /// <value></value>
        public string CourseCode { get; set; }

        /// <summary>
        /// 分类：1-课件；2-资料
        /// </summary>
        /// <value></value>
        public int Category { get; set; }
    }
}