using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace WingMongoDBModule.Common
{
    /// <summary>
    /// License验证类
    /// </summary>
    public class LicenseHelper
    {
        private const char _separator = '$';
        /// <summary>
        /// "Vinno.vCloud.CurrentLicense" two MD5
        /// </summary>
        private const string Key = "2A4A2B05EFD02D5BDEBB55711213BBBD";

        /// <summary>
        /// Get License from text file
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static License FromFile(string text)
        {
            License license = new License();
            var stringlist = text.Split(_separator);
            var dict = new Dictionary<string, string>();
            foreach (var s in stringlist)
            {
                var arry = s.Split('=');
                if (arry.Length > 1)
                {
                    dict.Add(arry[0], arry[1]);
                }
                if (arry.Length == 1)
                {
                    dict.Add(arry[0], "");
                }

            }
            var properties = license.GetType().GetProperties();
            foreach (var propertyInfo in properties)
            {
                dict.TryGetValue(propertyInfo.Name, out var value);
                if (string.IsNullOrEmpty(value)) continue;
                object obj;
                switch (propertyInfo.Name)
                {
                    case "MaxUsCount":
                        obj = Convert.ToInt32(value);
                        break;
                    case "ExpirationDay":
                        obj = Convert.ToInt32(value);
                        break;
                    case "ExpirationUtcDate":
                        obj = Convert.ToDateTime(value);
                        break;
                    case "MaxAccountCount":
                        obj = Convert.ToInt32(value);
                        break;
                    case "MaxDiskSpace":
                        obj = Convert.ToInt32(value);
                        break;
                    case "LicenseType":
                        obj = Enum.Parse(typeof(LicenseType), value);
                        break;
                    case "Identification":
                        obj = value.ToString();
                        break;
                    default:
                        obj = value;
                        break;

                }
                propertyInfo.SetValue(license, obj);
            }
            return license;
        }

        /// <summary>
        /// Gets Encrypt text
        /// </summary>
        /// <param name="inputText"></param>
        /// <returns></returns>
        public static string Encrypt(string inputText)
        {
            return AesEncrypt(inputText, Key);
        }
        /// <summary>
        ///  Gets Decrypt text
        /// </summary>
        /// <param name="inputText"></param>
        /// <returns></returns>
        public static string Decrypt(string inputText)
        {
            return AesDecrypt(inputText, Key);
        }
        /// <summary>
        /// AES algorithm  Encrypt string
        /// </summary>
        /// <param name="input"></param>
        /// <param name="key">AES Key </param>
        /// <returns></returns>
        private static string AesEncrypt(string input, string key)
        {
            var encryptKey = Encoding.UTF8.GetBytes(key);

            using (var aesAlg = Aes.Create())
            {
                using (var encryptor = aesAlg.CreateEncryptor(encryptKey, aesAlg.IV))
                {
                    using (var msEncrypt = new MemoryStream())
                    {
                        using (var csEncrypt = new CryptoStream(msEncrypt, encryptor,
                            CryptoStreamMode.Write))

                        using (var swEncrypt = new StreamWriter(csEncrypt))
                        {
                            swEncrypt.Write(input);
                        }

                        var iv = aesAlg.IV;

                        var decryptedContent = msEncrypt.ToArray();

                        var result = new byte[iv.Length + decryptedContent.Length];

                        Buffer.BlockCopy(iv, 0, result, 0, iv.Length);
                        Buffer.BlockCopy(decryptedContent, 0, result,
                            iv.Length, decryptedContent.Length);

                        return Convert.ToBase64String(result);
                    }
                }
            }
        }
        /// <summary>
        /// AES algorithm  Decrypt string
        /// </summary>
        /// <param name="cipherText"></param>
        /// <param name="keyString">AES Key</param>
        /// <returns></returns>
        private static string AesDecrypt(string cipherText, string keyString)
        {
            var fullCipher = Convert.FromBase64String(cipherText);

            byte[] iv = new byte[16];

            var cipher = new byte[fullCipher.Length - iv.Length];

            Buffer.BlockCopy(fullCipher, 0, iv, 0, iv.Length);
            Buffer.BlockCopy(fullCipher, iv.Length, cipher, 0, fullCipher.Length - iv.Length);
            var key = Encoding.UTF8.GetBytes(keyString);

            using (var aesAlg = Aes.Create())
            {
                using (var decryptor = aesAlg.CreateDecryptor(key, iv))
                {
                    string result;
                    using (var msDecrypt = new MemoryStream(cipher))
                    {
                        using (var csDecrypt = new CryptoStream(msDecrypt, decryptor, CryptoStreamMode.Read))
                        {
                            using (var srDecrypt = new StreamReader(csDecrypt))
                            {
                                result = srDecrypt.ReadToEnd();
                            }
                        }
                    }

                    return result;
                }
            }
        }
    }

    /// <summary>
    /// License info,it used the server.
    /// </summary>
    public class License
    {
        /// <summary>
        /// get or set version 
        /// </summary>
        public string Version { get; set; }
        /// <summary>
        ///  get or set license guid id 
        /// </summary>
        public string LicenseId { get; set; }
        /// <summary>
        /// get or set company name
        /// </summary>
        public string CompanyName { get; set; }
        /// <summary>
        /// get or set license type
        /// </summary>
        public LicenseType LicenseType { get; set; }
        /// <summary>
        /// get or set  license support  maximum  accounts
        /// </summary>
        public long MaxAccountCount { get; set; }
        /// <summary>
        ///  get or set  license support  maximum ultrasonic machines
        /// </summary>
        public long MaxUsCount { get; set; }
        /// <summary>
        ///  get or set  license support  maximum disk space
        /// </summary>
        public int MaxDiskSpace { get; set; }
        /// <summary>
        /// get or set  license  utc expiration date
        /// </summary>
        public DateTime ExpirationUtcDate { get; set; }
        /// <summary>
        /// availability day
        /// </summary>
        public int ExpirationDay { get; set; }
        /// <summary>
        /// get or set  license remarks
        /// </summary>
        public string Remarks { get; set; }
        /// <summary>
        /// get or set  license Identification
        /// </summary>
        public string Identification { get; set; }
        /// <summary>
        /// Reserved Info when compatible 
        /// </summary>
        public string ReservedInfo { get; set; }

        /// <summary>
        /// Functional Item Value
        /// </summary>
        public string FunctionalItemValue { get; set; }
        /// <summary>
        /// association key
        /// </summary>
        public string AssociationKey { get; set; }

    }

    public enum LicenseType
    {
        /// <summary>
        ///  For free trial user
        /// </summary>
        Trial,
        /// <summary>
        ///  Standard license
        /// </summary>
        Standard
    }
}