using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace WingMongoDBModule.Common
{
    public class RepositoryFactory
    {
        private static RepositoryFactory _factory;
        private readonly Dictionary<Type, object> _instances;

        public static RepositoryFactory Instance
        {
            get { return _factory ?? (_factory = new RepositoryFactory()); }
        }

        private RepositoryFactory()
        {
            _instances = new Dictionary<Type, object>();
        }

        public void Register(Type type, object repository)
        {
            if (_instances.ContainsKey(typeof(Type)))
            {
                //Logger.WriteLineError("Register duplicate type");
            }
            else
            {
                _instances.Add(type, repository);
            }
        }

        public T Get<T>()
        {
            object repository;
            if (_instances.TryGetValue(typeof(T), out repository))
            {
                return (T)repository;
            }
            throw new Exception("Invalid type");
        }
    }
}