using System;
using Newtonsoft.Json;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Log;
using WingMongoDBModule.Entities;
using System.Linq;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Net.Http;
using System.Text;
using System.Dynamic;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Security.Cryptography;
using System.Net.Http.Headers;
using System.Net;

namespace WingMongoDBModule.Common
{
    /// <summary>
    /// 银川项目对接对象存储，
    /// 天翼云对象存储，上传文件，把Key存储到银川对接平台数据表中
    /// </summary>
    public class YinChuanCos
    {
        private readonly static string accessKey = "leTSGnkEAWwnBbieiEkw";
        private readonly static string secretKey = "79bZIJaiVS6DQRSaZPXEkP9GVxUf9QhmIK08fAH3";
        // API 服务器
        private readonly static string OOS_SERVICE = "https://nxoss.xstore.ctyun.cn";
        private readonly static string bucketName = "ygt";
        private readonly static string rootdir = "60fa745e2ea22436c6d430e8";
        private IOOSHelper ctyunOOSHelper;
        public YinChuanCos()
        {
            ctyunOOSHelper = new CtyunOOSHttpHelper();

            ctyunOOSHelper.InitOOSHelper(OOS_SERVICE, accessKey, secretKey, bucketName, rootdir);
        }

        /// <summary>
        /// 上传对象存储
        /// </summary>
        /// <param name="filePath"></param>
        /// <returns></returns>
        internal string UploadFile(string filePath)
        {
            try
            {
                if (File.Exists(filePath))
                {
                    string msg = string.Empty;
                    string fileName = Path.GetFileName(filePath);
                    using (var fs = new FileStream(filePath, FileMode.Open, FileAccess.Read,
                            FileShare.ReadWrite))
                    {
                        var success = ctyunOOSHelper.UploadFile(fs, fileName, ref msg);
                        if (success)
                        {
                            var key = rootdir + "/" + fileName;
                            Logger.WriteLineError($"YinChuanCos upload file:key:{key}  , {msg},");
                            return key;
                        }
                        else
                        {
                            Logger.WriteLineWarn($"YinChuanCos upload file to ctyun failed: {msg}, filePath:{filePath}");
                        }
                    }
                }
                else
                {
                    Logger.WriteLineError($"YinChuanCos upload file not found {filePath}");
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"YinChuanCos upload file {filePath} error:{ex}");
            }

            return string.Empty;
        }

        internal string UploadFile(byte[] fileData, string fileName, int retryNum = 0)
        {
            try
            {
                string msg = string.Empty;
                var success = ctyunOOSHelper.UploadFile(fileData, fileName, ref msg);
                if (success)
                {
                    var key = rootdir + "/" + fileName;
                    return key;
                }
                else
                {
                    retryNum += 1;
                    Logger.WriteLineWarn($"YinChuanCos upload file to ctyun retryNum:{retryNum} failed: {msg}, fileName:{fileName}");
                    if (retryNum < 3)
                    {
                        var result = UploadFile(fileData, fileName, retryNum);
                        if (!string.IsNullOrEmpty(result))
                        {
                            retryNum = 4;
                        }
                        return result;
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"YinChuanCos upload file name:{fileName} error:{ex}");
            }

            return string.Empty;
        }

    }

    internal interface IOOSHelper
    {
        /// <summary>
        /// 设置配置信息
        /// </summary>
        /// <param name="serviceURL">云地址</param>
        /// <param name="accessKey">oos账号</param>
        /// <param name="secretKey">oos密码</param>
        /// <param name="bucketName">存储桶</param>
        void InitOOSHelper(string serviceURL, string accessKey, string secretKey, string bucketName, string root_dir);


        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="stream">文件流</param>
        /// <param name="objectname">例如：99999/1.JPG</param>
        /// <param name="msg">如果成功返回文件存储地址；失败返回错误原因</param>
        /// <returns></returns>
        bool UploadFile(Stream inputStream, string objectname, ref string msg);

        bool UploadFile(byte[] data, string objectname, ref string msg);
    }


    internal class CtyunOOSHttpHelper : IOOSHelper
    {
        private string _root_dir;
        /// <summary>
        /// 天翼云oos账号
        /// </summary>
        private string _accessKey;

        /// <summary>
        /// 天翼云oos密码
        /// </summary>
        private string _secretKey;

        /// <summary>
        /// 存储桶
        /// </summary>
        private string _bucketName;

        /// <summary>
        /// 天翼云地址
        /// </summary>
        private string _serviceURL;

        private CtyunOOSSignatureV2 signature;

        public void InitOOSHelper(string serviceURL, string accessKey, string secretKey, string bucketName, string rootdir)
        {
            this._serviceURL = serviceURL;
            this._accessKey = accessKey;
            this._secretKey = secretKey;
            this._bucketName = bucketName;
            _root_dir = rootdir;
            signature = new CtyunOOSSignatureV2(accessKey, secretKey);

        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="stream">文件流</param>
        /// <param name="objectname">商户号+文件名称，例如：99999/1.JPG</param>
        /// <param name="msg">如果成功返回文件存储地址；失败返回错误原因</param>
        /// <returns></returns>
        public bool UploadFile(Stream stream, string objectname, ref string msg)
        {
            bool success = true;

            int filelength = 0;

            filelength = (int)stream.Length; //获得文件长度

            byte[] data = new Byte[filelength]; //建立一个字节数组

            stream.Read(data, 0, filelength); //按字节流读取

            HttpRequestHelper httpReqHelper = new HttpRequestHelper(this._serviceURL);

            string uri = this._bucketName + "/" + _root_dir + "/" + objectname;

            string datetime = DateTime.Now.ToUniversalTime().ToString("R");

            var authorization = signature.AuthorizationSignature(HttpRequestHelper.HttpType.PUT.ToString(), "application/octet-stream", datetime, "/" + uri);

            Dictionary<string, string> headers = new Dictionary<string, string>();

            headers.Add("Date", datetime);
            headers.Add("Content-Type", "application/octet-stream");
            headers.Add("Authorization", authorization);
            try
            {
                httpReqHelper.AddRequestHeaders(headers);
                msg = httpReqHelper.HttpRequest(uri, HttpRequestHelper.HttpType.PUT, data);

                if (string.IsNullOrEmpty(msg))
                    msg = this._serviceURL + "/" + uri;

            }
            catch (Exception ex)
            {
                msg = ex.Message;
                success = false;
            }
            return success;
        }

        public bool UploadFile(byte[] data, string objectname, ref string msg)
        {
            bool success = true;


            HttpRequestHelper httpReqHelper = new HttpRequestHelper(this._serviceURL);

            string uri = this._bucketName + "/" + _root_dir + "/" + objectname;

            string datetime = DateTime.Now.ToUniversalTime().ToString("R");

            var authorization = signature.AuthorizationSignature(HttpRequestHelper.HttpType.PUT.ToString(), "application/octet-stream", datetime, "/" + uri);

            Dictionary<string, string> headers = new Dictionary<string, string>();

            headers.Add("Date", datetime);
            headers.Add("Content-Type", "application/octet-stream");
            headers.Add("Authorization", authorization);
            try
            {
                httpReqHelper.AddRequestHeaders(headers);
                msg = httpReqHelper.HttpRequest(uri, HttpRequestHelper.HttpType.PUT, data);

                if (string.IsNullOrEmpty(msg))
                    msg = this._serviceURL + "/" + uri;

            }
            catch (Exception ex)
            {
                msg = ex.Message;
                success = false;
            }
            return success;
        }
    }

    internal class HttpRequestHelper
    {
        private static string BaseUri;

        /// <summary>
        ///
        /// </summary>
        private Dictionary<string, string> headers;

        public HttpRequestHelper(string baseUri)
        {
            BaseUri = baseUri;
        }


        #region Delete方式
        private string Delete(string uri, byte[] data)
        {
            string serviceUrl = "";
            if (BaseUri == "" || BaseUri == null)
            {
                serviceUrl = uri;
            }
            else
            {
                serviceUrl = string.Format("{0}/{1}", BaseUri, uri);
            }
            return CommonHttpRequest(serviceUrl, "DELETE", data);
        }
        #endregion
        #region Put方式
        private string Put(string uri, byte[] data)
        {
            string serviceUrl = "";
            if (BaseUri == "" || BaseUri == null)
            {
                serviceUrl = uri;
            }
            else
            {
                serviceUrl = string.Format("{0}/{1}", BaseUri, uri);
            }
            // return CommonHttpRequest(serviceUrl, "PUT", data);
            return CommonHttpClientRequest(serviceUrl, HttpMethod.Put, data);
        }
        #endregion
        #region POST方式实现


        private string Post(string uri, byte[] data)
        {
            string serviceUrl = "";
            if (BaseUri == "" || BaseUri == null)
            {
                serviceUrl = uri;
            }
            else
            {
                serviceUrl = string.Format("{0}/{1}", BaseUri, uri);
            }
            return CommonHttpRequest(serviceUrl, "Post", data);
        }
        #endregion
        #region GET方式实现
        private string Get(string uri)
        {
            string serviceUrl = "";
            if (BaseUri == "" || BaseUri == null)
            {
                serviceUrl = uri;
            }
            else
            {
                serviceUrl = string.Format("{0}/{1}", BaseUri, uri);
            }
            return CommonHttpRequest(serviceUrl, "GET", null);
        }

        #endregion
        #region  私有方法
        private string CommonHttpRequest(string url, string reqType, byte[] data)
        {
            HttpWebRequest webRequest = null;
            Stream outstream = null;
            HttpWebResponse myResponse = null;
            StreamReader reader = null;
            try
            {
                //构造http请求的对象
                webRequest = (HttpWebRequest)WebRequest.Create(url);


                //设置
                webRequest.ProtocolVersion = HttpVersion.Version11;
                webRequest.Method = reqType;

                if (headers != null && headers.Count > 0)
                {
                    foreach (var header in headers)
                        webRequest.Headers.Add(header.Key, header.Value);
                }

                if (data != null && data.Length > 0)
                {
                    webRequest.ContentLength = data.Length;

                    //转成网络流
                    byte[] buf = data;

                    outstream = webRequest.GetRequestStream();
                    outstream.Flush();
                    outstream.Write(buf, 0, buf.Length);
                    outstream.Flush();
                    outstream.Close();
                }
                // 获得接口返回值
                myResponse = (HttpWebResponse)webRequest.GetResponse();
                reader = new StreamReader(myResponse.GetResponseStream(), Encoding.UTF8);
                string ReturnXml = reader.ReadToEnd();
                reader.Close();
                myResponse.Close();
                webRequest.Abort();
                return ReturnXml;
            }
            catch (Exception ex)
            {
                if (outstream != null) outstream.Close();
                if (reader != null) reader.Close();
                if (myResponse != null) myResponse.Close();
                if (webRequest != null) webRequest.Abort();
                throw ex;
            }
        }

        private static HttpClient _httpClient = new HttpClient();
        private string CommonHttpClientRequest(string url, HttpMethod reqType, byte[] data)
        {
            try
            {
                using (var requestHead = new HttpRequestMessage())
                {
                    requestHead.Method = reqType;
                    var contentType = MediaTypeHeaderValue.Parse("application/octet-stream");
                    requestHead.Version = HttpVersion.Version11;
                    requestHead.RequestUri = new Uri(url);
                    using (var content = new UploadContent(data, contentType))
                    {
                        if (headers != null && headers.Count > 0)
                        {
                            foreach (var header in headers)
                                requestHead.Headers.TryAddWithoutValidation(header.Key, header.Value);
                        }
                        requestHead.Content = content;
                        var responseTask = _httpClient.SendAsync(requestHead);
                        var response = responseTask.GetAwaiter().GetResult();
                        var resultStr = response.Content.ReadAsStringAsync().Result;
                        return resultStr;
                    }

                }
            }
            catch (Exception ex)
            {
                _httpClient.Dispose();
                _httpClient = new HttpClient();
                throw ex;
            }

        }
        #endregion


        #region 通用请求
        /// <summary>
        /// Http通用请求
        /// </summary>
        /// <param name="url"></param>
        /// <param name="type"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        public string HttpRequest(string url, HttpType type, byte[] data)
        {
            switch (type)
            {
                case HttpType.PUT:
                    return Put(url, data);
                case HttpType.GET:
                    return Get(url);
                case HttpType.POST:
                    return Post(url, data);
                case HttpType.DELETE:
                    return Delete(url, data);
                default:
                    break;
            }
            return "";
        }

        /// <summary>
        /// 添加请求头
        /// </summary>
        /// <param name="headers"></param>
        public void AddRequestHeaders(Dictionary<string, string> headers)
        {
            this.headers = headers;
        }

        /// <summary>
        /// Http通用请求
        /// </summary>
        /// <param name="ip"></param>
        /// <param name="port"></param>
        /// <param name="uri"></param>
        /// <param name="type"></param>
        /// <param name="data"></param>
        /// <returns></returns>
        public string HttpRequest(string ip, string port, string uri, HttpType type, byte[] data)
        {
            string url = "http://" + ip + ":" + port + uri;
            return HttpRequest(url, type, data);
        }


        #endregion


        public enum HttpType
        {
            PUT = 0,
            GET = 1,
            POST = 2,
            DELETE = 3


        }
    }

    /// <summary>
    /// 天翼云OOS签名V2版本
    /// </summary>
    internal class CtyunOOSSignatureV2
    {
        /// <summary>
        /// 账号
        /// </summary>
        private string secretkey;

        /// <summary>
        /// 秘钥
        /// </summary>
        private string accessKey;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="secretkey">秘钥</param>
        /// <param name="accessKey">账号</param>
        /// <param name="region"></param>
        public CtyunOOSSignatureV2(string accessKey, string secretkey)
        {
            this.secretkey = secretkey;
            this.accessKey = accessKey;
        }

        /// <summary>

        /// HMACSHA1算法加密并返回ToBase64string

        /// </summary>

        /// <param name="strText">签名参数字符串</param>

        /// <param name="strKey">密钥参数</param>

        /// <returns>返回一个签名值(即哈希值)</returns>

        private string ToBase64hmac(string strKey, string strText)
        {

            HMACSHA1 myHMACSHA1 = new HMACSHA1(Encoding.UTF8.GetBytes(strKey));

            byte[] byteText = myHMACSHA1.ComputeHash(Encoding.UTF8.GetBytes(strText));

            return System.Convert.ToBase64String(byteText);

        }


        private string GetstringToSign(string httpVerb, string contentType, string date, string uri)
        {
            string stringToSign = httpVerb + "\n\n" + contentType + "\n" + date + "\n" + uri;


            return stringToSign;
        }

        /// <summary>
        /// 获取认证签名信息
        /// </summary>
        /// <param name="httpVerb">请求方法</param>
        /// <param name="date">请求时间</param>
        /// <param name="uri">存储桶</param>
        /// <param name="objectname">存储对象名称</param>
        /// <returns></returns>
        public string AuthorizationSignature(string httpVerb, string contentType, string date, string uri)
        {

            var stringToSign = GetstringToSign(httpVerb, contentType, date, uri);

            var signature = ToBase64hmac(this.secretkey, stringToSign);

            return "AWS " + this.accessKey + ":" + signature;

        }
    }
}
