﻿using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Driver;
using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

namespace WingMongoDBModule.DBSync
{
    /// <summary>
    /// MongoDB collection proxy object.
    /// </summary>
    /// <typeparam name="TDocument"></typeparam>
    public class MongoCollectionProxy<TDocument> : IMongoCollection<TDocument> //TODO refactir ,we should implyment different type for distributed system or local system
    {
        /// <summary>
        /// MongoDB
        /// </summary>
        private IMongoDatabase _database = null;

        /// <summary>
        /// MongoDB collection
        /// </summary>
        private IMongoCollection<TDocument> _collection = null;
        
        //
        // 摘要:
        //     Gets the settings.
        public MongoCollectionSettings Settings { get { return _collection.Settings; } }
        //
        // 摘要:
        //     Gets the document serializer.
        public IBsonSerializer<TDocument> DocumentSerializer { get { return _collection.DocumentSerializer; } }
        //
        // 摘要:
        //     Gets the database.
        public IMongoDatabase Database { get { return _collection.Database; } }
        //
        // 摘要:
        //     Gets the namespace of the collection.
        public CollectionNamespace CollectionNamespace { get { return _collection.CollectionNamespace; } }

        //
        // 摘要:
        //     Gets the index manager.
        public IMongoIndexManager<TDocument> Indexes { get { return _collection.Indexes; } }

        public bool  IsSimple {get;set;}
        /// <summary>
        /// 构造器
        /// </summary>
        /// <param name="database">数据库对象</param>
        /// <param name="name">数据集名</param>
        /// <param name="settings">数据集设定</param>
        public MongoCollectionProxy(IMongoDatabase database, string name, MongoCollectionSettings settings)
        {
            this._database = database;
            this._collection = _database.GetCollection<TDocument>(name, settings);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TResult> Aggregate<TResult>(PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Aggregate(pipeline, options, cancellationToken);
        }


        //
        // 摘要:
        //     Runs an aggregation pipeline.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TResult>> AggregateAsync<TResult>(PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.AggregateAsync(pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TResult> Aggregate<TResult>(IClientSessionHandle session, PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Aggregate(session, pipeline, options, cancellationToken);
        }
        //
        // 摘要:
        //     Runs an aggregation pipeline.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TResult>> AggregateAsync<TResult>(IClientSessionHandle session, PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.AggregateAsync(session, pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline whose results are written to a collection.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        public void AggregateToCollection<TResult>(PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.AggregateToCollection<TResult>(pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline whose results are written to a collection.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task.
        public async Task AggregateToCollectionAsync<TResult>(PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.AggregateToCollectionAsync<TResult>(pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline whose results are written to a collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        public void AggregateToCollection<TResult>(IClientSessionHandle session, PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.AggregateToCollection<TResult>(session, pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Runs an aggregation pipeline whose results are written to a collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task.
        public async Task AggregateToCollectionAsync<TResult>(IClientSessionHandle session, PipelineDefinition<TDocument, TResult> pipeline, AggregateOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.AggregateToCollectionAsync<TResult>(session, pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Performs multiple write operations.
        //
        // 参数:
        //   requests:
        //     The requests.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of writing.
        public BulkWriteResult<TDocument> BulkWrite(IEnumerable<WriteModel<TDocument>> requests, BulkWriteOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.BulkWrite(requests, options, cancellationToken);
        }

        //
        // 摘要:
        //     Performs multiple write operations.
        //
        // 参数:
        //   requests:
        //     The requests.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of writing.
        public async Task<BulkWriteResult<TDocument>> BulkWriteAsync(IEnumerable<WriteModel<TDocument>> requests, BulkWriteOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.BulkWriteAsync(requests, options, cancellationToken);
        }

        //
        // 摘要:
        //     Performs multiple write operations.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   requests:
        //     The requests.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of writing.
        public BulkWriteResult<TDocument> BulkWrite(IClientSessionHandle session, IEnumerable<WriteModel<TDocument>> requests, BulkWriteOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.BulkWrite(session, requests, options, cancellationToken);
        }
        //
        // 摘要:
        //     Performs multiple write operations.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   requests:
        //     The requests.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of writing.
        public async Task<BulkWriteResult<TDocument>> BulkWriteAsync(IClientSessionHandle session, IEnumerable<WriteModel<TDocument>> requests, BulkWriteOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.BulkWriteAsync(session, requests, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        [Obsolete("Use CountDocuments or EstimatedDocumentCount instead.")]
        public long Count(FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Count(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        [Obsolete("Use CountDocumentsAsync or EstimatedDocumentCountAsync instead.")]
        public async Task<long> CountAsync(FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.CountAsync(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        [Obsolete("Use CountDocuments or EstimatedDocumentCount instead.")]
        public long Count(IClientSessionHandle session, FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Count(session, filter, options, cancellationToken);
        }
        //
        // 摘要:
        //     Counts the number of documents in the collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        [Obsolete("Use CountDocumentsAsync or EstimatedDocumentCountAsync instead.")]
        public async Task<long> CountAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.CountAsync(session, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection. For a fast estimate of the
        //     total documents in a collection see MongoDB.Driver.IMongoCollection`1.EstimatedDocumentCount(MongoDB.Driver.EstimatedDocumentCountOptions,System.Threading.CancellationToken).
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        //
        // 言论：
        //     Note: when migrating from Count to CountDocuments the following query operations
        //     must be replaced:
        //     +-------------+--------------------------------+ | Operator | Replacement | +=============+================================+
        //     | $where | $expr | +-------------+--------------------------------+ | $near |
        //     $geoWithin with $center | +-------------+--------------------------------+ |
        //     $nearSphere | $geoWithin with $centerSphere | +-------------+--------------------------------+
        public long CountDocuments(FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.CountDocuments(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection. For a fast estimate of the
        //     total documents in a collection see .
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        //
        // 言论：
        //     Note: when migrating from CountAsync to CountDocumentsAsync the following query
        //     operations must be replaced:
        //     +-------------+--------------------------------+ | Operator | Replacement | +=============+================================+
        //     | $where | $expr | +-------------+--------------------------------+ | $near |
        //     $geoWithin with $center | +-------------+--------------------------------+ |
        //     $nearSphere | $geoWithin with $centerSphere | +-------------+--------------------------------+
        public async Task<long> CountDocumentsAsync(FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.CountDocumentsAsync(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection. For a fast estimate of the
        //     total documents in a collection see MongoDB.Driver.IMongoCollection`1.EstimatedDocumentCount(MongoDB.Driver.EstimatedDocumentCountOptions,System.Threading.CancellationToken).
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        //
        // 言论：
        //     Note: when migrating from Count to CountDocuments the following query operations
        //     must be replaced:
        //     +-------------+--------------------------------+ | Operator | Replacement | +=============+================================+
        //     | $where | $expr | +-------------+--------------------------------+ | $near |
        //     $geoWithin with $center | +-------------+--------------------------------+ |
        //     $nearSphere | $geoWithin with $centerSphere | +-------------+--------------------------------+
        public long CountDocuments(IClientSessionHandle session, FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.CountDocuments(session, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Counts the number of documents in the collection. For a fast estimate of the
        //     total documents in a collection see MongoDB.Driver.IMongoCollection`1.EstimatedDocumentCountAsync(MongoDB.Driver.EstimatedDocumentCountOptions,System.Threading.CancellationToken).
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The number of documents in the collection.
        //
        // 言论：
        //     Note: when migrating from CountAsync to CountDocumentsAsync the following query
        //     operations must be replaced:
        //     +-------------+--------------------------------+ | Operator | Replacement | +=============+================================+
        //     | $where | $expr | +-------------+--------------------------------+ | $near |
        //     $geoWithin with $center | +-------------+--------------------------------+ |
        //     $nearSphere | $geoWithin with $centerSphere | +-------------+--------------------------------+
        public async Task<long> CountDocumentsAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, CountOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.CountDocumentsAsync(session, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteMany(FilterDefinition<TDocument> filter, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteMany(filter, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteMany(FilterDefinition<TDocument> filter, DeleteOptions options, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteMany(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteManyAsync(FilterDefinition<TDocument> filter, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteManyAsync(filter, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteManyAsync(FilterDefinition<TDocument> filter, DeleteOptions options, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteManyAsync(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteMany(IClientSessionHandle session, FilterDefinition<TDocument> filter, DeleteOptions options = null, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteMany(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Deletes multiple documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteManyAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, DeleteOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteManyAsync(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteMany(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteOne(FilterDefinition<TDocument> filter, DeleteOptions options, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteOne(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }
        //
        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteOne(FilterDefinition<TDocument> filter, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteOne(filter, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteOneAsync(FilterDefinition<TDocument> filter, DeleteOptions options, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteOneAsync(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteOneAsync(FilterDefinition<TDocument> filter, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteOneAsync(filter, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        public async Task<DeleteResult> DeleteOneWithoutWriteLogAsync(FilterDefinition<TDocument> filter, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteOneAsync(filter, cancellationToken);
            return task;
        }

        //
        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public DeleteResult DeleteOne(IClientSessionHandle session, FilterDefinition<TDocument> filter, DeleteOptions options = null, CancellationToken cancellationToken = default)
        {
            DeleteResult result = _collection.DeleteOne(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Deletes a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the delete operation.
        public async Task<DeleteResult> DeleteOneAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, DeleteOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.DeleteOneAsync(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateDeleteOne(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Gets the distinct values for a specified field.
        //
        // 参数:
        //   field:
        //     The field.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TField:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TField> Distinct<TField>(FieldDefinition<TDocument, TField> field, FilterDefinition<TDocument> filter, DistinctOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Distinct(field, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Gets the distinct values for a specified field.
        //
        // 参数:
        //   field:
        //     The field.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TField:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TField>> DistinctAsync<TField>(FieldDefinition<TDocument, TField> field, FilterDefinition<TDocument> filter, DistinctOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.DistinctAsync(field, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Gets the distinct values for a specified field.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   field:
        //     The field.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TField:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TField> Distinct<TField>(IClientSessionHandle session, FieldDefinition<TDocument, TField> field, FilterDefinition<TDocument> filter, DistinctOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Distinct(session, field, filter, options, cancellationToken);
        }
        //
        // 摘要:
        //     Gets the distinct values for a specified field.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   field:
        //     The field.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TField:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TField>> DistinctAsync<TField>(IClientSessionHandle session, FieldDefinition<TDocument, TField> field, FilterDefinition<TDocument> filter, DistinctOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.DistinctAsync(session, field, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Returns an estimate of the number of documents in the collection.
        //
        // 参数:
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     An estimate of the number of documents in the collection.
        public long EstimatedDocumentCount(EstimatedDocumentCountOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.EstimatedDocumentCount(options, cancellationToken);
        }

        //
        // 摘要:
        //     Returns an estimate of the number of documents in the collection.
        //
        // 参数:
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     An estimate of the number of documents in the collection.
        public async Task<long> EstimatedDocumentCountAsync(EstimatedDocumentCountOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.EstimatedDocumentCountAsync(options, cancellationToken);
        }

        //
        // 摘要:
        //     Finds the documents matching the filter.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TProjection>> FindAsync<TProjection>(FilterDefinition<TDocument> filter, FindOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.FindAsync<TProjection>(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Finds the documents matching the filter.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TProjection>> FindAsync<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, FindOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.FindAsync<TProjection>(session, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Finds a single document and deletes it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndDelete<TProjection>(FilterDefinition<TDocument> filter, FindOneAndDeleteOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndDelete<TProjection>(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndDelete<TDocument>(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return projection;
        }

        //
        // 摘要:
        //     Finds a single document and deletes it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndDeleteAsync<TProjection>(FilterDefinition<TDocument> filter, FindOneAndDeleteOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndDeleteAsync<TProjection>(filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndDelete<TDocument>(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Finds a single document and deletes it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndDelete<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, FindOneAndDeleteOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndDelete<TProjection>(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndDelete<TDocument>(_collection.CollectionNamespace.CollectionName, filter),IsSimple);
            return projection;
        }


        //
        // 摘要:
        //     Finds a single document and deletes it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndDeleteAsync<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, FindOneAndDeleteOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndDeleteAsync<TProjection>(session, filter, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndDelete<TDocument>(_collection.CollectionNamespace.CollectionName, filter, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Finds a single document and replaces it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndReplace<TProjection>(FilterDefinition<TDocument> filter, TDocument replacement, FindOneAndReplaceOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndReplace<TProjection>(filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndReplace<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return projection;
        }

        //
        // 摘要:
        //     Finds a single document and replaces it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndReplaceAsync<TProjection>(FilterDefinition<TDocument> filter, TDocument replacement, FindOneAndReplaceOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndReplaceAsync<TProjection>(filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndReplace<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return task;
        }


        //
        // 摘要:
        //     Finds a single document and replaces it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndReplace<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, FindOneAndReplaceOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndReplace<TProjection>(session, filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndReplace<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return projection;
        }

        //
        // 摘要:
        //     Finds a single document and replaces it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndReplaceAsync<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, FindOneAndReplaceOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndReplaceAsync<TProjection>(session, filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndReplace<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Finds a single document and updates it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndUpdate<TProjection>(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, FindOneAndUpdateOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndUpdate<TProjection>(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndUpdate<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return projection;
        }

        //
        // 摘要:
        //     Finds a single document and updates it atomically.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndUpdateAsync<TProjection>(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, FindOneAndUpdateOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndUpdateAsync<TProjection>(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndUpdate<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Finds a single document and updates it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public TProjection FindOneAndUpdate<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, FindOneAndUpdateOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            TProjection projection = _collection.FindOneAndUpdate<TProjection>(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndUpdate<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return projection;
        }

        //
        // 摘要:
        //     Finds a single document and updates it atomically.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     The returned document.
        public async Task<TProjection> FindOneAndUpdateAsync<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, FindOneAndUpdateOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.FindOneAndUpdateAsync<TProjection>(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateFindOneAndUpdate<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Finds the documents matching the filter.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TProjection> FindSync<TProjection>(FilterDefinition<TDocument> filter, FindOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            return _collection.FindSync<TProjection>(filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Finds the documents matching the filter.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TProjection:
        //     The type of the projection (same as TDocument if there is no projection).
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TProjection> FindSync<TProjection>(IClientSessionHandle session, FilterDefinition<TDocument> filter, FindOptions<TDocument, TProjection> options = null, CancellationToken cancellationToken = default)
        {
            return _collection.FindSync<TProjection>(session, filter, options, cancellationToken);
        }

        //
        // 摘要:
        //     Inserts many documents.
        //
        // 参数:
        //   documents:
        //     The documents.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        public void InsertMany(IEnumerable<TDocument> documents, InsertManyOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.InsertMany(documents, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertMany<TDocument>(_collection.CollectionNamespace.CollectionName, documents),IsSimple);
        }

        //
        // 摘要:
        //     Inserts many documents.
        //
        // 参数:
        //   documents:
        //     The documents.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertManyAsync(IEnumerable<TDocument> documents, InsertManyOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertManyAsync(documents, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertMany(_collection.CollectionNamespace.CollectionName, documents, true),IsSimple);
        }

        //
        // 摘要:
        //     Inserts many documents.
        //
        // 参数:
        //   documents:
        //     The documents.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertManyWithoutWriteLogAsync(IEnumerable<TDocument> documents, InsertManyOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertManyAsync(documents, options, cancellationToken);
        }

        //
        // 摘要:
        //     Inserts many documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   documents:
        //     The documents.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        public void InsertMany(IClientSessionHandle session, IEnumerable<TDocument> documents, InsertManyOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.InsertMany(session, documents, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertMany<TDocument>(_collection.CollectionNamespace.CollectionName, documents),IsSimple);
        }

        //
        // 摘要:
        //     Inserts many documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   documents:
        //     The documents.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertManyAsync(IClientSessionHandle session, IEnumerable<TDocument> documents, InsertManyOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertManyAsync(session, documents, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertMany(_collection.CollectionNamespace.CollectionName, documents, true),IsSimple);
        }

        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   document:
        //     The document.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        public void InsertOne(TDocument document, InsertOneOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.InsertOne(document, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne(_collection.CollectionNamespace.CollectionName, document),IsSimple);
        }

        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   document:
        //     The document.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertOneAsync(TDocument document, InsertOneOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertOneAsync(document, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne<TDocument>(_collection.CollectionNamespace.CollectionName, document, true),IsSimple);
        }

        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   document:
        //     The document.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertOneWithoutWriteLogAsync(TDocument document, InsertOneOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertOneAsync(document, options, cancellationToken);
        }

        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   document:
        //     The document.
        //
        //   _cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        [Obsolete("Use the new overload of InsertOneAsync with an InsertOneOptions parameter instead.")]
        public async Task InsertOneAsync(TDocument document, CancellationToken _cancellationToken)
        {
            await _collection.InsertOneAsync(document, _cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne<TDocument>(_collection.CollectionNamespace.CollectionName, document, true),IsSimple);
        }

        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   document:
        //     The document.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        public void InsertOne(IClientSessionHandle session, TDocument document, InsertOneOptions options = null, CancellationToken cancellationToken = default)
        {
            _collection.InsertOne(session, document, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne(_collection.CollectionNamespace.CollectionName, document),IsSimple);
        }
        //
        // 摘要:
        //     Inserts a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   document:
        //     The document.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the insert operation.
        public async Task InsertOneAsync(IClientSessionHandle session, TDocument document, InsertOneOptions options = null, CancellationToken cancellationToken = default)
        {
            await _collection.InsertOneAsync(session, document, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateInsertOne<TDocument>(_collection.CollectionNamespace.CollectionName, document, true),IsSimple);
        }

        //
        // 摘要:
        //     Executes a map-reduce command.
        //
        // 参数:
        //   map:
        //     The map function.
        //
        //   reduce:
        //     The reduce function.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TResult> MapReduce<TResult>(BsonJavaScript map, BsonJavaScript reduce, MapReduceOptions<TDocument, TResult> options = null, CancellationToken cancellationToken = default)
        {
            return _collection.MapReduce<TResult>(map, reduce, options, cancellationToken);
        }

        //
        // 摘要:
        //     Executes a map-reduce command.
        //
        // 参数:
        //   map:
        //     The map function.
        //
        //   reduce:
        //     The reduce function.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TResult>> MapReduceAsync<TResult>(BsonJavaScript map, BsonJavaScript reduce, MapReduceOptions<TDocument, TResult> options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.MapReduceAsync<TResult>(map, reduce, options, cancellationToken);
        }

        //
        // 摘要:
        //     Executes a map-reduce command.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   map:
        //     The map function.
        //
        //   reduce:
        //     The reduce function.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A cursor.
        public IAsyncCursor<TResult> MapReduce<TResult>(IClientSessionHandle session, BsonJavaScript map, BsonJavaScript reduce, MapReduceOptions<TDocument, TResult> options = null, CancellationToken cancellationToken = default)
        {
            return _collection.MapReduce<TResult>(session, map, reduce, options, cancellationToken);
        }

        //
        // 摘要:
        //     Executes a map-reduce command.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   map:
        //     The map function.
        //
        //   reduce:
        //     The reduce function.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A Task whose result is a cursor.
        public async Task<IAsyncCursor<TResult>> MapReduceAsync<TResult>(IClientSessionHandle session, BsonJavaScript map, BsonJavaScript reduce, MapReduceOptions<TDocument, TResult> options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.MapReduceAsync<TResult>(session, map, reduce, options, cancellationToken);
        }

        //
        // 摘要:
        //     Returns a filtered collection that appears to contain only documents of the derived
        //     type. All operations using this filtered collection will automatically use discriminators
        //     as necessary.
        //
        // 类型参数:
        //   TDerivedDocument:
        //     The type of the derived document.
        //
        // 返回结果:
        //     A filtered collection.
        public IFilteredMongoCollection<TDerivedDocument> OfType<TDerivedDocument>() where TDerivedDocument : TDocument
        {
            return _collection.OfType<TDerivedDocument>();
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        public ReplaceOneResult ReplaceOne(FilterDefinition<TDocument> filter, TDocument replacement, ReplaceOptions options = null, CancellationToken cancellationToken = default)
        {
            ReplaceOneResult result = _collection.ReplaceOne(filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        [Obsolete("Use the overload that takes a ReplaceOptions instead of an UpdateOptions.")]
        public ReplaceOneResult ReplaceOne(FilterDefinition<TDocument> filter, TDocument replacement, UpdateOptions options, CancellationToken cancellationToken = default)
        {
            ReplaceOneResult result = _collection.ReplaceOne(filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        public async Task<ReplaceOneResult> ReplaceOneAsync(FilterDefinition<TDocument> filter, TDocument replacement, ReplaceOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.ReplaceOneAsync(filter, replacement, options, cancellationToken);
             MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        [Obsolete("Use the overload that takes a ReplaceOptions instead of an UpdateOptions.")]
        public async Task<ReplaceOneResult> ReplaceOneAsync(FilterDefinition<TDocument> filter, TDocument replacement, UpdateOptions options, CancellationToken cancellationToken = default)
        {
            var task = await _collection.ReplaceOneAsync(filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        public ReplaceOneResult ReplaceOne(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, ReplaceOptions options = null, CancellationToken cancellationToken = default)
        {
            ReplaceOneResult result = _collection.ReplaceOne(session, filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return result;
        }
        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        [Obsolete("Use the overload that takes a ReplaceOptions instead of an UpdateOptions.")]
        public ReplaceOneResult ReplaceOne(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, UpdateOptions options, CancellationToken cancellationToken = default)
        {
            ReplaceOneResult result = _collection.ReplaceOne(session, filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        public async Task<ReplaceOneResult> ReplaceOneAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, ReplaceOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.ReplaceOneAsync(session, filter, replacement, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Replaces a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   replacement:
        //     The replacement.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the replacement.
        [Obsolete("Use the overload that takes a ReplaceOptions instead of an UpdateOptions.")]
        public async Task<ReplaceOneResult> ReplaceOneAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, TDocument replacement, UpdateOptions options, CancellationToken cancellationToken = default)
        {
            var task = await _collection.ReplaceOneAsync(session, filter, replacement, options, cancellationToken);
             MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateReplaceOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, replacement, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Updates many documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public UpdateResult UpdateMany(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            UpdateResult result = _collection.UpdateMany(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateMany<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Updates many documents.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public async Task<UpdateResult> UpdateManyAsync(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.UpdateManyAsync(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateMany<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Updates many documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public UpdateResult UpdateMany(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            UpdateResult result = _collection.UpdateMany(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateMany<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Updates many documents.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public async Task<UpdateResult> UpdateManyAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.UpdateManyAsync(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateMany<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Updates a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public UpdateResult UpdateOne(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            UpdateResult result = _collection.UpdateOne(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Updates a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public async Task<UpdateResult> UpdateOneAsync(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.UpdateOneAsync(filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Updates a single document.
        //
        // 参数:
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public async Task<UpdateResult> UpdateOneWithoutWriteLogAsync(FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            var task = await _collection.UpdateOneAsync(filter, update, options, cancellationToken);
            return task;
        }

        //
        // 摘要:
        //     Updates a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public UpdateResult UpdateOne(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            UpdateResult result = _collection.UpdateOne(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update),IsSimple);
            return result;
        }

        //
        // 摘要:
        //     Updates a single document.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   filter:
        //     The filter.
        //
        //   update:
        //     The update.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 返回结果:
        //     The result of the update operation.
        public async Task<UpdateResult> UpdateOneAsync(IClientSessionHandle session, FilterDefinition<TDocument> filter, UpdateDefinition<TDocument> update, UpdateOptions options = null, CancellationToken cancellationToken = default)
        {
            var task =  await _collection.UpdateOneAsync(session, filter, update, options, cancellationToken);
            MongoDBLogTracker.Instance.Record(MongoDBLogFactory.CreateUpdateOne<TDocument>(_collection.CollectionNamespace.CollectionName, filter, update, true),IsSimple);
            return task;
        }

        //
        // 摘要:
        //     Watches changes on the collection.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A change stream.
        public IChangeStreamCursor<TResult> Watch<TResult>(PipelineDefinition<ChangeStreamDocument<TDocument>, TResult> pipeline, ChangeStreamOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Watch<TResult>(pipeline, options, cancellationToken);
        }


        //
        // 摘要:
        //     Watches changes on the collection.
        //
        // 参数:
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A change stream.
        public async Task<IChangeStreamCursor<TResult>> WatchAsync<TResult>(PipelineDefinition<ChangeStreamDocument<TDocument>, TResult> pipeline, ChangeStreamOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.WatchAsync<TResult>(pipeline, options, cancellationToken);
        }


        //
        // 摘要:
        //     Watches changes on the collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A change stream.
        public IChangeStreamCursor<TResult> Watch<TResult>(IClientSessionHandle session, PipelineDefinition<ChangeStreamDocument<TDocument>, TResult> pipeline, ChangeStreamOptions options = null, CancellationToken cancellationToken = default)
        {
            return _collection.Watch<TResult>(session, pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Watches changes on the collection.
        //
        // 参数:
        //   session:
        //     The session.
        //
        //   pipeline:
        //     The pipeline.
        //
        //   options:
        //     The options.
        //
        //   cancellationToken:
        //     The cancellation token.
        //
        // 类型参数:
        //   TResult:
        //     The type of the result.
        //
        // 返回结果:
        //     A change stream.
        public async Task<IChangeStreamCursor<TResult>> WatchAsync<TResult>(IClientSessionHandle session, PipelineDefinition<ChangeStreamDocument<TDocument>, TResult> pipeline, ChangeStreamOptions options = null, CancellationToken cancellationToken = default)
        {
            return await _collection.WatchAsync<TResult>(session, pipeline, options, cancellationToken);
        }

        //
        // 摘要:
        //     Returns a new IMongoCollection instance with a different read concern setting.
        //
        // 参数:
        //   readConcern:
        //     The read concern.
        //
        // 返回结果:
        //     A new IMongoCollection instance with a different read concern setting.
        public IMongoCollection<TDocument> WithReadConcern(ReadConcern readConcern)
        {
            return _collection.WithReadConcern(readConcern);
        }

        //
        // 摘要:
        //     Returns a new IMongoCollection instance with a different read preference setting.
        //
        // 参数:
        //   readPreference:
        //     The read preference.
        //
        // 返回结果:
        //     A new IMongoCollection instance with a different read preference setting.
        public IMongoCollection<TDocument> WithReadPreference(ReadPreference readPreference)
        {
            return _collection.WithReadPreference(readPreference);
        }

        //
        // 摘要:
        //     Returns a new IMongoCollection instance with a different write concern setting.
        //
        // 参数:
        //   writeConcern:
        //     The write concern.
        //
        // 返回结果:
        //     A new IMongoCollection instance with a different write concern setting.
        public IMongoCollection<TDocument> WithWriteConcern(WriteConcern writeConcern)
        {
            return _collection.WithWriteConcern(writeConcern);
        }    
    }
}
