﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    public class AdminInfoDO : BaseDO
    {
        private string _headImageToken;
        /// <summary>
        /// 管理员编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string AdminCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 对应父级管理员编码
        /// </summary>
        [QueryItem]
        public string FatherCode { get; set; }

        /// <summary>
        /// 管理员名称
        /// </summary>
        [QueryItem]
        public string AdminName { get; set; }

        /// <summary>
        /// 密码
        /// </summary>
        public string SecretPassword { get; set; }

        /// <summary>
        /// 姓名
        /// </summary>
        [QueryItem]
        public string FullName { get; set; }

        /// <summary>
        /// 头像
        /// </summary>
        public string HeadImageToken
        {
            get
            {
                return DOShare.ChangeUrl(_headImageToken);
            }
            set
            {
                _headImageToken = value;
            }
        }
        /// <summary>
        /// 管理授权码
        /// </summary>
        public string LicenseKey { get; set; }

        /// <summary>
        /// 最后登录IP地址
        /// </summary>
        public string LastIP { get; set; }

        /// <summary>
        /// 管理员联系电话
        /// </summary>
        [QueryItem]
        public string Phone { get; set; }

        /// <summary>
        /// 管理员邮箱
        /// </summary>
        [QueryItem]
        public string Email { get; set; }

        /// <summary>
        /// 后台角色列表
        /// </summary>
        /// <value></value>
        public List<string> RoleCodes { get; set; } = new();

        /// <summary>
        ///分配的医院列表
        /// </summary>
        /// <value></value>
        public List<string> AssignedOrganizations { get; set; } = new();

        /// <summary>
        /// 登录失败信息 
        /// </summary>
        /// <value></value>
        public LoginLockInfoDO LoginLockInfo { get; set; }

        /// <summary>
        /// password update time
        /// </summary>
        public DateTime PasswordUpdateTime { get; set; }

        /// <summary>
        /// 密码记录列表
        /// </summary>
        public List<string> PasswordRecords { get; set; } = new();

        /// <summary>
        /// 清理日志实体列表
        /// </summary>
        public List<ClearLogOperateDO> ClearLogOperateList { get; set; } = new();

        /// <summary>
        /// 绑定用户code
        /// </summary>
        public string BindUserCode  { get; set; }

        /// <summary>
        /// 绑定用户名称
        /// </summary>
        public string BindUserName  { get; set; }
    }

    /// <summary>
    /// 清理日志实体
    /// </summary>
    public class ClearLogOperateDO
    {
        /// <summary>
        /// 目标服务器
        /// </summary>
        public string ServerHost { get; set; }

        /// <summary>
        /// 上次日志清理时间【可以根据daysAgo计算出来】
        /// </summary>
        public DateTime LastClearTime { get; set; }

        /// <summary>
        /// 上次清理操作时间
        /// </summary>
        public DateTime LastClearOperateTime { get; set; }
    }

    /// <summary>
    /// 自定义组件对象
    /// </summary>
    public class CustomReportTemplateModuleDO
    {
        private string _customModuleFile;
        /// <summary>
        /// 自定义组件唯一键（Guid）
        /// </summary>
        /// <value></value>
        public string CustomModuleCode { get; set; }

        /// <summary>
        /// 自定义组件名称
        /// </summary>
        /// <value></value>
        public string CustomModuleName { get; set; }

        /// <summary>
        /// 自定义组件类型
        /// </summary>
        /// <value></value>
        public string CustomModuleType { get; set; }

        /// <summary>
        /// 自定义组件json文件地址
        /// </summary>
        /// <value></value>
        public string CustomModuleFile
        {
            get
            {
                return DOShare.ChangeUrl(_customModuleFile);
            }
            set
            {
                _customModuleFile = value;
            }
        }
    }
}
