﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 管理员图表统计结果
    /// </summary>
    public class AdminStatisticRecordDO : BaseDO
    {
        /// <summary>
        /// 管理员图表统计结果编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string AdminStatisticRecordCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 图表配置编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string AdminStatisticCode { get; set; }

        /// <summary>
        /// 图表配置名
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string AdminStatisticName { get; set; }

        /// <summary>
        /// 管理员编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string AdminCode { get; set; }

        /// <summary>
        /// 统计维度-XAxis
        /// </summary>
        /// <value></value>
        public List<string> XAxisDatas { get; set; } = new();

        /// <summary>
        /// 统计结果-YAxis
        /// </summary>
        /// <value></value>
        public List<List<double>> YAxisDatas { get; set; } = new();

        /// <summary>
        /// 子集统计
        /// </summary>
        /// <value></value>
        public List<StatisticInfoDO> Children { get; set; }

        /// <summary>
        /// 服务器地址
        /// </summary>
        /// <value></value>
        public string ServerHost { get; set; }

        /// <summary>
        /// 最近更新时间
        /// </summary>
        /// <value></value>
        public DateTime LastestUpdateTime { get; set; }

        /// <summary>
        /// 下次更新时间
        /// </summary>
        /// <value></value>
        public DateTime NextHandleTime { get; set; }

        /// <summary>
        /// 统计数据json格式字符串
        /// </summary>
        public string StatisticJsonData { get; set; }

        /// <summary>
        /// 统计平台
        /// </summary>
        public StatisticsPlatformEnum StatisticsPlatform { get; set; } = StatisticsPlatformEnum.Management;
    }

    public class StatisticInfoDO
    {
        /// <summary>
        /// 统计维度-XAxis
        /// </summary>
        /// <value></value>
        public List<string> XAxisDatas { get; set; } = new();

        /// <summary>
        /// 统计结果-YAxis
        /// </summary>
        /// <value></value>
        public List<List<double>> YAxisDatas { get; set; } = new();

        /// <summary>
        /// 子集统计，数量与XAxisDatas保持一致
        /// </summary>
        /// <value></value>
        public List<StatisticInfoDO> Children { get; set; }

        /// <summary>
        /// 数据项实体
        /// </summary>
        /// <value></value>
        public DataItem DataItemInfo { get; set; }

        /// <summary>
        /// 统计数据json格式字符串
        /// </summary>
        public string StatisticJsonData { get; set; }
    }
}
