﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.LiveConsultation;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    public class DeviceInfoDO : BaseDO
    {
        private string _headPicUrl;
        /// <summary>
        /// 设备编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string DeviceCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 设备物理唯一码
        /// </summary>
        [UniqueItem]
        [QueryItem]
        public string SerialNumber { get; set; }

        /// <summary>
        /// 设备密码
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// 设备名称（Dongid或登录账号）
        /// </summary>
        [QueryItem]
        public string Name { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        [QueryItem]
        public string Description { get; set; }

        /// <summary>
        /// 设备型号
        /// </summary>
        [QueryItem]
        public string DeviceModel { get; set; }

        /// <summary>
        /// 设备类型
        /// </summary>
        [QueryItem]
        public string DeviceType { get; set; }

        /// <summary>
        /// 设备头图
        /// </summary>
        public string HeadPicUrl
        {
            get
            {
                return DOShare.ChangeUrl(_headPicUrl);
            }
            set
            {
                _headPicUrl = value;
            }
        }
        /// <summary>
        /// 设备软件版本
        /// </summary>
        public string DeviceSoftwareVersion { get; set; }

        /// <summary>
        /// 设备SDK版本
        /// </summary>
        public string SDKSoftwareVersion { get; set; }

        /// <summary>
        /// 设备所属组织
        /// </summary>
        [QueryItem]
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 组织下属科室
        /// </summary>
        [QueryItem]
        public string DepartmentCode { get; set; }

        /// <summary>
        /// 超声机动态唯一码
        /// </summary>
        [UniqueItem]
        [QueryItem]
        public string ShortCode { get; set; }

        /// <summary>
        /// 是否自动分享
        /// </summary>
        [QueryItem]
        public bool IsAutoShared { get; set; }

        /// <summary>
        /// 敏感信息加密显示
        /// </summary>
        [QueryItem]
        public bool IsEncryptedShow { get; set; }

        /// <summary>
        /// 最近登录时间
        /// </summary>
        public DateTime LastLoginTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 系统版本
        /// </summary>
        public string SystemVersion { get; set; } = string.Empty;

        /// <summary>
        /// CPU型号
        /// </summary>
        public string CPUModel { get; set; } = string.Empty;

        /// <summary>
        /// 系统语言
        /// </summary>
        public string SystemLanguage { get; set; } = string.Empty;

        /// <summary>
        /// AI应用集合
        /// </summary>
        public List<string> DiagnosisModules { get; set; } = new();

        /// <summary>
        /// 报告转发配置集合
        /// </summary>
        public List<string> ReportPosterCodes { get; set; } = new();

        /// <summary>
        /// 是否把多路视屏合流
        /// </summary>
        /// <value></value>
        public bool MergedChannel { get; set; } = true;

        /// <summary>
        /// 合流的画面宽度
        /// </summary>
        /// <value></value>
        public int MergedVideoOutputWidth { get; set; } = 1920;

        /// <summary>
        /// 合流的画面高度
        /// </summary>
        /// <value></value>
        public int MergedVideoOutputHeight { get; set; } = 1080;

        /// <summary>
        /// 视屏设备画面信息
        /// </summary>
        /// <value></value>
        public IList<VideoDeviceDO> VideoDeviceInfos { get; set; } = new List<VideoDeviceDO>();

        /// <summary>
        /// 设置下载模式
        /// </summary>
        /// <value></value>
        public DownloadModeSettingEnum DownloadModeSetting { get; set; }

        /// <summary>
        /// 直播开关，true=开启，false=关闭
        /// </summary>
        /// <value></value>
        public bool LiveOpened { get; set; }

        /// <summary>
        /// 是否支持Rtc
        /// </summary>
        /// <value></value>
        public bool SupportRtc { get; set; }

        /// <summary>
        /// 连接方式
        /// </summary>
        /// <value></value>
        public ProxyTypeEnum ProxyType { get; set; } = ProxyTypeEnum.Old;
        
        /// <summary>
        /// 设备属性列表 json数据格式，
        /// </summary>
        /// <value></value>
        public string DeviceAttributes { get; set; }

        /// <summary>
        /// 设备耗材属性列表 json数据格式，
        /// </summary>
        /// <value></value>
        public string DeviceConsumableAttributes { get; set; }

        /// <summary>
        /// iot设备升级指定版本
        /// </summary>
        /// <value></value>
        public string UpgradeVersionCode { get; set; }
    }

    public class VideoDeviceDO
    {
        /// <summary>
        /// 设备Id（比如超声机唯一码，用来表示这是超声机画面）
        /// </summary>
        /// <value></value>
        public string VideoDeviceId { get; set; }

        /// <summary>
        /// 视屏画面设备来源的类型（设备主屏或摄像头）
        /// </summary>
        /// <value></value>
        public VideoDeviceSourceTypeEnum VideoDeviceSourceType { get; set; }

        /// <summary>
        /// 设备设置的画面宽度，上报值
        /// </summary>
        /// <value></value>
        public int Width { get; set; }

        /// <summary>
        /// 设备设置的画面高度，上报值
        /// </summary>
        /// <value></value>
        public int Height { get; set; }

        /// <summary>
        /// 设备设置的画面宽度
        /// </summary>
        /// <value></value>
        public int OutputWidth { get; set; }

        /// <summary>
        /// 设备设置的画面高度
        /// </summary>
        /// <value></value>
        public int OutputHeight { get; set; }

        /// <summary>
        /// 帧率
        /// </summary>
        /// <value></value>
        public int VideoFps { get; set; } = 20;

        /// <summary>
        /// 码率
        /// </summary>
        /// <value></value>
        public int VideoBitrate { get; set; } = 2000;

        /// <summary>
        /// 最小码率
        /// </summary>
        /// <value></value>
        public int MinVideoBitrate { get; set; } = 1000;
    }
}
