using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 分布式服务器管理信息
    /// </summary>
    public class DistributedServerInfoDO : BaseDO
    {
        /// <summary>
        /// 编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string ServerCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }
        /// <summary>
        /// 服务器名称
        /// </summary>
        [QueryItem]
        public string Name { get; set; }
        /// <summary>
        /// 启用
        /// </summary>
        /// <value></value>
        [QueryItem]
        public bool Enable { get; set; }
        /// <summary>
        /// 功能区分 官方或私有服务器
        /// </summary>
        /// <value></value>
        [QueryItem]
        public DistributedServerTypeEnum ServerType { get; set; }
        /// <summary>
        /// 云服务地址
        /// </summary>
        [QueryItem]
        public string ServerUrl { get; set; }
        /// <summary>
        /// 描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }
        /// <summary>
        /// 是否主服务器
        /// </summary>
        /// <value></value>
        public bool IsMaster { get; set; }
        /// <summary>
        /// 纬度
        /// </summary>
        /// <value></value>
        public string Lat { get; set; }
        /// <summary>
        /// 经度
        /// </summary>
        /// <value></value>
        public string Lng { get; set; }
        /// <summary>
        /// 指定客户端IP访问此服务器
        /// </summary>
        /// <value></value>
        public List<IPAddressInfoDO> AssignClientIPList { get; set; }
        /// <summary>
        /// rtc 初始房间id
        /// </summary>
        /// <value></value>
        public int RTCStartingRoomId { get; set; }
        /// <summary>
        /// 分享前缀code
        /// </summary>
        /// <value></value>
        public string ShareCodePrefix { get; set; }
        /// <summary>
        /// 默认存储节点
        /// </summary>
        /// <value></value>
        public string DefaultStorageNode { get; set; }

        /// <summary>
        /// 存储服务配置
        /// </summary>
        /// <value></value>
        public string StorageServerConfig { get; set; }

        /// <summary>
        /// 局域网服务器地址包含端口
        /// </summary>
        /// <value></value>
        public string LANHost{ get; set; }
    }

    /// <summary>
    /// IP 字段信息
    /// </summary>
    public class IPAddressInfoDO
    {
        /// <summary>
        /// 起始IP段
        /// </summary>
        /// <value></value>
        public string StartIp { get; set; }
        /// <summary>
        /// 结束IP段
        /// </summary>
        /// <value></value>
        public string EndIp { get; set; }
        /// <summary>
        ///  起始IP段 Long类型
        /// </summary>
        /// <value></value>
        public long LongStartIP { get; set; }
        /// <summary>
        ///  结束IP段 Long类型
        /// </summary>
        /// <value></value>
        public long LongEndIP { get; set; }
    }

}