﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    public class OrganizationDO : BaseDO
    {
        private string  _logoUrl;
        /// <summary>
        /// 组织编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string OrganizationCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 组织名称
        /// </summary>
        [QueryItem]
        public string OrganizationName { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        public string Description { get; set; }

        [QueryItem]
        /// <summary>
        /// 最精确地区编码
        /// </summary>
        public string RegionCode { get; set; }

        /// <summary>
        /// 顶级区域编码
        /// </summary>
        public string RootRegionCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 父级编码
        /// </summary>
        public string ParentCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 顶级编码
        /// </summary>
        public string RootCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 组织类型
        /// </summary>
        /// <value></value>
        public OrganizationTypeEnum OrganizationType { get; set; }

        /// <summary>
        /// 所属管理组列表
        /// </summary>
        public List<string> AuthorityGroups { get; set; } = new();

        /// <summary>
        /// 经纬度
        /// </summary>
        public string Nautica { get; set; }

        /// <summary>
        /// Logo地址或者token
        /// </summary>
        public string LogoUrl 
         {
            get
            {
                return DOShare.ChangeUrl(_logoUrl);
            }
            set
            {
                _logoUrl = value;
            }
        }
        /// <summary>
        /// 审核状态
        /// </summary>
        /// <value></value>
        [QueryItem]
        public OrganizationStateEnum State { get; set; }

        /// <summary>
        /// 检查服务病人对象类型
        /// </summary>
        /// <value></value>
        public OrganizationPatientTypeEnum PatientType { get; set; }

        /// <summary>
        /// 机构负责人
        /// </summary>
        /// <value></value>
        public List<string> Directors { get; set; }

        /// <summary>
        /// 已分配的管理员列表
        /// </summary>
        /// <value></value>
        public List<string> AssignedAdmins { get; set; }

        /// <summary>
        /// 患者配置
        /// </summary>
        /// <value></value>
        public string PatientSettingJson { get; set; }

        /// <summary>
        /// 检查配置
        /// </summary>
        /// <value></value>
        public string ExamSettingJson { get; set; }

        /// <summary>
        /// 会诊配置
        /// </summary>
        /// <value></value>
        public string ConsultationSettingJson { get; set; }

        /// <summary>
        /// 患者配置版本
        /// </summary>
        /// <value></value>
        public string PatientSettingVersion { get; set; }

        /// <summary>
        /// 检查配置版本
        /// </summary>
        /// <value></value>
        public string ExamSettingVersion { get; set; }

        /// <summary>
        /// 会诊配置版本
        /// </summary>
        /// <value></value>
        public string ConsultationSettingVersion { get; set; }

        /// <summary>
        /// 是否是虚拟机构(废弃)
        /// </summary>
        /// <value></value>
        [QueryItem]
        public bool Isinvented { get; set; } = false;

        /// <summary>
        /// 配置列表
        /// </summary>
        /// <value></value>
        public List<SettingClass> Settings { get; set; } = new();

        /// <summary>
        /// AI应用集合
        /// </summary>
        public List<string> DiagnosisModules { get; set; } = new();

        /// <summary>
        /// 敏感信息加密显示
        /// </summary>
        /// <value></value>
        public bool IsEncryptedShow { get; set; } = true;

        /// <summary>
        /// 转诊超时时限
        /// </summary>
        public double ReferralLimitHours { get; set; } = 1;

        /// <summary>
        /// 可转诊机构
        /// </summary>
        public List<string> ReferralOrganizationCodes { get; set; }

        /// <summary>
        /// CMS模板code
        /// </summary>
        public string CMSTemplateCode { get; set; }

        /// <summary>
        /// 急诊医院编码
        /// </summary>
        /// <value></value>
        public string EmergencyOrganizationCode { get; set; }

        /// <summary>
        /// 急诊值班医生编码
        /// </summary>
        /// <value></value>
        public string EmergencyDoctorCode { get; set; }     

        /// <summary>
        /// 机构短码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string ShortCode { get; set; }       

        /// <summary>
        /// 默认词条库编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string DefaultThesaurusCode { get; set; }

        /// <summary>
        /// 默认报告模板编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string DefaultReportTemplateCode { get; set; }  

        /// <summary>
        /// 即时会诊是否需要填写病人信息
        /// </summary>
        /// <value></value>
        public bool EmergencyNeedFill { get; set; }

        /// <summary>
        /// 即时会诊配置版本号
        /// </summary>
        /// <value></value>
        public string EmergencySettingVersion { get; set; }

        /// <summary>
        /// 即时会诊病人信息
        /// </summary>
        /// <value></value>
        public string EmergencySettingJson { get; set; }

        /// <summary>
        /// 绑定资源包列表历史记录 
        /// </summary>
        /// <value></value>
        public List<BindChargeResourcePackageDO> BindChargeResourcePackageRecords { get; set; }

        /// <summary>
        /// 绑定资源包列表 
        /// </summary>
        /// <value></value>
        public List<BindChargeResourcePackageDO> BindChargeResourcePackages { get; set; }
    }

    public class SettingClass
    {
        /// <summary>
        /// 语言key
        /// </summary>
        /// <value>en-US</value>
        public string LangugeKey { get; set; }

        /// <summary>
        /// 配置模块
        /// </summary>
        /// <value>exam.json</value>
        public string SettingModule { get; set; }

        /// <summary>
        /// 自定义内容Json
        /// </summary>
        /// <value></value>
        public string CustomContent { get; set; }

        /// <summary>
        /// 完整内容（基础配置拼接自定义内容）
        /// </summary>
        /// <value></value>
        public string CompleteContent { get; set; }

        /// <summary>
        /// 基于哪个版本进行自定义的，对比用
        /// </summary>
        /// <value></value>
        public string BaseConfigVersion { get; set; }

        /// <summary>
        /// 版本
        /// </summary>
        /// <value></value>
        public string Version { get; set; }
    }
}
