using System.Collections.Generic;
using WingMongoDBModule.Mongodb.Driver;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Enum;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 病人信息
    /// </summary>
    public class PatientInfoDO : BaseDO
    {
        private string _headImgUrl;
        /// <summary>
        /// 病人编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string PatientCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }
        [QueryItem]
        /// <summary>
        /// 病人名称
        /// </summary>
        public string Name { get; set; }

        [QueryItem]
        /// <summary>
        /// 联系电话
        /// </summary>
        public string Phone { get; set; }

        [QueryItem]
        /// <summary>
        /// 身份证号码
        /// </summary>
        public string IdentityCard { get; set; }

        [QueryItem]
        /// <summary>
        /// 保险编号
        /// </summary>
        public string InsuranceCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 年龄- 废弃
        /// </summary>
        public string Age { get; set; }

        /// <summary>
        /// 性别
        /// </summary>
        [QueryItem]
        public int Gender { get; set; }

        [QueryItem]
        /// <summary>
        /// 是否已登记确认
        /// </summary>
        public bool IsValid { get; set; }

        [QueryItem]
        /// <summary>
        /// 创建人编码
        /// </summary>
        public string CreatorCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 机构编码
        /// </summary>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 分配用户code集合
        /// </summary>
        /// <value></value>
        public List<string> AssignmentUserCodes { get; set; }

        [QueryItem]
        /// <summary>
        ///设备Code(废弃)
        /// </summary>
        public string DeviceCode { get; set; }
        /// <summary>
        /// 病人信息字段 json 集合
        /// </summary>
        /// <value></value>
        public List<DataItemDTO> PatientData { get; set; }

        [QueryItem]
        /// <summary>
        /// 病人/宠物头像
        /// </summary>
        public string HeadImgUrl
        {
            get
            {
                return DOShare.ChangeUrl(_headImgUrl);
            }
            set
            {
                _headImgUrl = value;
            }
        }
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Person、Animals</value>
        public string PatientType { get; set; }

        /// <summary>
        /// 操作人列表
        /// </summary>
        /// <value>"4914352a75f645279f6b4ad6764cc288","4914352a75f645279f6b4ad6764cc281"</value>
        public List<string> UpdateUsers { get; set; } = new List<string>();

        [QueryItem]
        /// <summary>
        /// 是否转诊
        /// </summary>
        public bool IsReferral { get; set; }


        [QueryItem]
        /// <summary>
        /// 医院机构编码
        /// </summary>
        public string RootOrganizationCode { get; set; }

        /// <summary>
        /// 设备病人ID
        /// </summary>
        public List<string> DevicePatientIDs { get; set; } = new List<string>();

        [QueryItem]
        /// <summary>
        /// 是否dicom上传
        /// </summary>
        public bool IsFromDicom { get; set; } = false;

        [QueryItem]
        /// <summary>
        /// 所属项目
        /// </summary>
        /// <value></value>
        public string ProjectCode { get; set; }

        /// <summary>
        /// 是否URM
        /// </summary>
        public bool IsUrm { get; set; }
    }
}