﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;
using WingInterfaceLibrary.DTO.Record;

namespace WingMongoDBModule.DO
{
    public class RecordInfoDO : BaseDO
    {
        /// <summary>
        /// 检查编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string RecordCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 病人编码
        /// </summary>
        [QueryItem]
        public string PatientCode { get; set; }

        /// <summary>
        /// 设备编码
        /// </summary>
        [QueryItem]
        public string DeviceCode { get; set; }

        /// <summary>
        /// 检查状态
        /// </summary>
        [QueryItem]
        public RecordStatusEnum RecordStatus { get; set; }

        /// <summary>
        /// 检查医生
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string CreatorCode { get; set; }

        /// <summary>
        /// 处理标识，用来做特殊处理标识
        /// </summary>
        public string Tags { get; set; }

        /// <summary>
        /// 创建方式
        /// </summary>
        /// <value></value>
        [QueryItem]
        public RecordCreateTypeEnum CreateType { get; set; }

        /// <summary>
        /// 病人信息
        /// </summary>
        public List<DataItem> PatientInfo { get; set; }

        /// <summary>
        /// 病人检查信息列表
        /// </summary>
        /// <value></value>
        public List<PatientInfoExt> PatientInfoExtList { get; set; }

        /// <summary>
        /// 设备病人ID
        /// </summary>
        /// <value></value>
        public string DevicePatientID { get; set; }

        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Person、Animals</value>
        public string PatientType { get; set; }

        /// <summary>
        /// 已读用户列表
        /// </summary>
        /// <value>"test01","test02"</value>
        public List<string> ReadUsers { get; set; } = new();
        /// <summary>
        /// 根组织
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string RootOrganizationCode { get; set; }
        /// <summary>
        /// 组织
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 关联的超声机本地上传的检查code
        /// </summary>
        /// <value></value>
        public List<string> AssociatedExamCodes { get; set; } = new List<string>();

        /// <summary>
        /// AI诊断状态
        /// </summary>
        /// <value>2</value>
        public DiagnosisStatusEnum DiagnosisStatus { get; set; }

        /// <summary>
        /// AI诊断结果
        /// </summary>
        /// <value></value>
        public List<DiagnosisInfoDO> DiagnosisInfos { get; set; } = new List<DiagnosisInfoDO>();

        /// <summary>
        /// 是否采图中
        /// </summary>
        /// <value></value>
        public bool IsCollecting { get; set; }

        /// <summary>
        /// 开始采图时间
        /// </summary>
        /// <value></value>
        public DateTime StartCollectingTime { get; set; }

        /// <summary>
        /// 是否医生出了诊断报告
        /// </summary>
        /// <value></value>
        [QueryItem]
        public bool IsUserReport { get; set; }

        /// <summary>
        /// 是否转出
        /// </summary>
        /// <value></value>
        public bool IsReferral { get; set; }

        [QueryItem]
        /// <summary>
        /// 是否dicom上传
        /// </summary>
        public bool IsFromDicom { get; set; } = false;

        /// <summary>
        /// 国家脑卒中项目，掌超上传医生
        /// </summary>
        /// <value></value>
        public string CustomDoctor { get; set; }

        /// <summary>
        /// 国家脑卒中项目，掌超上传医院
        /// </summary>
        /// <value></value>
        public string CustomOrganzation { get; set; }

        /// <summary>
        /// 国家脑卒中项目，掌超上传设备SN
        /// </summary>
        /// <value></value>
        public string EquipmentSN { get; set; }

        /// <summary>
        /// 检查时间
        /// </summary>
        /// <value></value>
        public DateTime? ExamTime { get; set; }

        /// <summary>
        /// 设备端ExamId
        /// </summary>
        /// <value></value>
        public string DeviceExamId { get; set; }

        /// <summary>
        /// 图像是否已质控
        /// </summary>
        /// <value></value>
        public bool IsImgQualityControlled { get; set; } = false;

        /// <summary>
        /// 报告是否已质控
        /// </summary>
        /// <value></value>
        public bool IsReportQualityControlled { get; set; } = false;

        /// <summary>
        /// 图像质控分
        /// </summary>
        /// <value></value>
        public double ImgScore { get; set; }

        /// <summary>
        /// 报告质控分
        /// </summary>
        /// <value></value>
        public double ReportScore { get; set; }

        /// <summary>
        /// 体检机构编码
        /// </summary>
        /// <value></value>
        public string VitalOrganizationCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 科研项目Code
        /// </summary>
        /// <value></value>
        public string ProjectCode { get; set; }

    }

    /// <summary>
    /// 病人额外信息
    /// </summary>
    public class PatientInfoExt
    {
        /// <summary>
        /// 病人额外信息类型（病人扫查类型）
        /// </summary>
        public string PatientScanType { get; set; }

        /// <summary>
        /// 病人额外信息内容
        /// </summary>
        public List<DataItem> Content { get; set; }
    }

    public class DataItem
    {
        /// <summary>
        /// key
        /// </summary>
        /// <value></value>
        public string Key { get; set; }

        /// <summary>
        /// value
        /// </summary>
        /// <value></value>
        public string Value { get; set; }

        /// <summary>
        /// 类型
        /// </summary>
        /// <value>string、int</value>
        public string Type { get; set; }
    }

    /// <summary>
    /// AI诊断结论
    /// </summary>
    public class DiagnosisInfoDO
    {
        /// <summary>
        /// AI识别器官
        /// </summary>
        /// <value></value>
        public DiagnosisOrganEnum DiagnosisOrgan { get; set; }
        /// <summary>
        /// 诊断结论
        /// </summary>
        /// <value></value>
        public DiagnosisConclusionEnum Conclusion { get; set; }
        /// <summary>
        /// 报告状态
        /// </summary>
        /// <value></value>
        public DiagnosisReportStatusEnum ReportStatus { get; set; }
    }



}
