﻿using System;
using System.Collections.Generic;
using System.Linq;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.MeasureEnum;
using WingInterfaceLibrary.Request.Examine;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 扫查数据
    /// </summary>
    public class RemedicalDO : BaseDO
    {
        /// <summary>
        /// 编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string RemedicalCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        [QueryItem]
        /// <summary>
        /// DeviceCode
        /// </summary>
        public string DeviceCode { get; set; }

        [QueryItem]
        /// <summary>
        /// Record Code
        /// </summary>
        public string RecordCode { get; set; }

        /// <summary>
        /// 病人扫查类型
        /// </summary>
        public string PatientScanType { get; set; }

        /// <summary>
        /// ApplicationCategory
        /// </summary>
        /// <value>ABD</value>
        public string ApplicationCategory { get; set; }

        /// <summary>
        /// ApplicationName
        /// </summary>
        public string Application { get; set; }

        /// <summary>
        /// 设备上传图片
        /// </summary>
        public TerminalImageDO TerminalImages { get; set; } = new();

        /// <summary>
        /// 文件数据类型
        /// </summary>
        public RemedicalFileDataTypeEnum FileDataType { get; set; }

        /// <summary>
        /// AI识别部位及图像位置描述
        /// </summary>
        public ImageLocationDO ImageLocation { get; set; }

        /// <summary>
        /// AI诊断结论
        /// </summary>
        /// <value></value>
        public DiagnosisConclusionEnum DiagnosisConclusion { get; set; }

        /// <summary>
        /// AI诊断详情
        /// </summary>
        /// <value></value>
        public string DiagnosisResult { get; set; }

        /// <summary>
        /// AI识别的器官
        /// </summary>
        /// <value></value>
        public List<DiagnosisOrganEnum> DiagnosisOrgans { get; set; } = new List<DiagnosisOrganEnum>();
        /// <summary>
        /// 测量结果数据
        /// </summary>
        /// <value></value>
        public MeasuredResultsDO MeasuredResult { get; set; }
        /// <summary>
        /// 注释结果数据
        /// </summary>
        /// <value></value>
        public ScanImageDO CommentResult { get; set; }

        /// <summary>
        /// 颈动脉结果
        /// </summary>
        /// <value></value>
        public CarotidResultDO CarotidResult { get; set; }

        /// <summary>
        /// 业务类型
        /// </summary>
        /// <value></value>
        public BusinessTypeEnum BusinessType { get; set; }

        /// <summary>
        /// 器官识别结果
        /// </summary>
        /// <value></value>
        public List<string> OrganDiagnosisInfos { get; set; } = new List<string>();

        [QueryItem]
        public bool OldPlatformNeedAI { get; set; } = false;

        /// <summary>
        /// 质控结果
        /// </summary>
        /// <value></value>
        public ReportQualityControl QualityControlDatas { get; set; }

        [QueryItem]
        /// <summary>
        /// 科研项目code
        /// </summary>
        /// <value></value>
        public string ProjectCode { get; set; }

        /// <summary>
        /// URM分析参数
        /// </summary>
        /// <value></value>
        public Dictionary<string, string> UrmParameters { get; set; } = new Dictionary<string, string>();

        /// <summary>
        /// 科研上传图像集合
        /// </summary>
        /// <typeparam name="ImgItem"></typeparam>
        /// <returns></returns>
        public List<ImgItem> ImgList { get; set; } = new List<ImgItem>();

        /// <summary>
        /// URM分析Pos参数值
        /// </summary>
        /// <value></value>
        public string UrmPosition { get; set; }

        /// <summary>
        /// URM原图编码
        /// </summary>
        /// <value></value>
        public string UrmOriginalCode { get; set; }

        /// <summary>
        /// URM分析/测量图像编码
        /// </summary>
        /// <value></value>
        public string RemedicalOriginalCode { get; set; }

        /// <summary>
        /// URMSrcWidth
        /// </summary>
        /// <value></value>
        public int URMSrcWidth { get; set; }

        /// <summary>
        /// URMSrcHeight
        /// </summary>
        /// <value></value>
        public int URMSrcHeight { get; set; }

        /// <summary>
        /// URMRes
        /// </summary>
        /// <value></value>
        public double URMRes { get; set; }

    }

    public class CarotidResultDO
    {
        private string _surfaceFile;
        private string _cdnSurfaceFile;
        private string _mdlFile;
        private string _cdnMdlFile;
        public CarotidScanTypeEnum CarotidScanType { get; set; }
        public CarotidScanDirectionEnum CarotidScanDirection { get; set; }
        /// <summary>
        /// 源站SurfaceFile下载地址
        /// </summary>
        /// <value></value>
        public string SurfaceFile
        {
            get
            {
                return DOShare.ChangeUrl(_surfaceFile);
            }
            set
            {
                _surfaceFile = value;
            }
        }
        /// <summary>
        /// CDNSurfaceFile下载地址
        /// </summary>
        /// <value></value>
        public string CDNSurfaceFile
        {
            get
            {
                return DOShare.ChangeUrl(_cdnSurfaceFile);
            }
            set
            {
                _cdnSurfaceFile = value;
            }
        }

        /// <summary>
        /// SurfaceFile大小
        /// </summary>
        /// <value></value>
        public long SurfaceFileSize { get; set; }
        /// <summary>
        /// 源站MdlFile下载地址
        /// </summary>
        /// <value></value>
        public string MdlFile
        {
            get
            {
                return DOShare.ChangeUrl(_mdlFile);
            }
            set
            {
                _mdlFile = value;
            }
        }
        /// <summary>
        /// CDNMdlFile下载地址
        /// </summary>
        /// <value></value>
        public string CDNMdlFile
        {
            get
            {
                return DOShare.ChangeUrl(_cdnMdlFile);
            }
            set
            {
                _cdnMdlFile = value;
            }
        }
        /// <summary>
        ///MdlFile大小
        /// </summary>
        /// <value></value>
        public long MdlFileSize { get; set; }
        public List<MeasureImageFileDO> MeasureImageFiles { get; set; } = new List<MeasureImageFileDO>();
        public string MeasureResult { get; set; } = string.Empty;
        private List<string> _surfaceImageList = new List<string>();
        public List<string> SurfaceImageList
        {
            get
            {
                var urls = new List<string>();
                if (_surfaceImageList != null && _surfaceImageList.Any())
                {
                    foreach (var url in _surfaceImageList)
                    {
                        urls.Add(DOShare.ChangeUrl(url));
                    }
                }
                return urls;
            }
            set
            {
                _surfaceImageList = value ?? new List<string>();
            }
        }
    }

    public class MeasureImageFileDO
    {
        private string _imageFile;
        public CarotidAIImageTypeEnum ImageType { get; set; }
        public string ImageFile
        {
            get
            {
                return DOShare.ChangeUrl(_imageFile);
            }
            set
            {
                _imageFile = value;
            }
        }
    }

    /// <summary>
    /// 扫查图像数据
    /// </summary>
    public class TerminalImageDO
    {
        private string _previewUrl;
        private string _imageUrl;
        private string _coverImageUrl;
        private string _originImageUrl;
        private string _urmImageUrl;

        /// <summary>
        /// Preview url
        /// </summary>
        public string PreviewUrl
        {
            get
            {
                return DOShare.ChangeUrl(_previewUrl);
            }
            set
            {
                _previewUrl = value;
            }
        }

        /// <summary>
        /// Image url
        /// </summary>
        public string ImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_imageUrl);
            }
            set
            {
                _imageUrl = value;
            }
        }

        /// <summary>
        /// 第一帧大图url
        /// </summary>
        public string CoverImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_coverImageUrl);
            }
            set
            {
                _coverImageUrl = value;
            }
        }


        /// <summary>
        /// 源站高清图下载地址
        /// </summary>
        /// <value></value>
        public string OriginImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_originImageUrl);
            }
            set
            {
                _originImageUrl = value;
            }
        }

        /// <summary>
        /// 废弃
        /// </summary>
        /// <value></value>
        public string UrmImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_urmImageUrl);
            }
            set
            {
                _urmImageUrl = value;
            }

        }


        /// <summary>
        /// 图像大小
        /// </summary>
        /// <value></value>
        public long ImageSize { get; set; }
    }

    /// <summary>
    /// 图像位置数据
    /// </summary>
    public class ImageLocationDO
    {
        /// <summary>
        /// 扫查图片部位类型
        /// </summary>
        public string Group { get; set; }

        /// <summary>
        /// 定位信息
        /// </summary>
        public string Position { get; set; }

        /// <summary>
        /// 象限信息
        /// </summary>
        public string Quadrant { get; set; }
    }
    /// <summary>
    /// 测量模式
    /// </summary>
    public class MeasuredResultsDO
    {
        /// <summary>
        /// 版本号
        /// </summary>
        /// <value>1.0.0.1</value>
        public string Version { get; set; }

        /// <summary>
        /// 测量节点集合
        /// </summary>
        /// <value></value>
        public List<FetusNodeDO> FetusNodes { get; set; }


    }
    /// <summary>
    /// 节点
    /// </summary>
    public class FetusNodeDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Measure.FetusNode</value>
        public string TypeName { get; set; }
        /// <summary>
        /// 顺序
        /// </summary>
        /// <value>-1</value>
        ///  <code>0</code>
        public string FetusIndex { get; set; }
        /// <summary>
        /// 节点子项集合
        /// </summary>
        /// <value></value>
        public List<ChildrenFetusNodeDO> Children { get; set; }
    }
    /// <summary>
    /// 节点子项
    /// </summary>
    public class ChildrenFetusNodeDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Measure.FetusNode</value>
        public string TypeName { get; set; }
        /// <summary>
        ///测量文件夹名称
        /// </summary>
        /// <value>General</value>
        public string FolderName { get; set; }
        /// <summary>
        ///测量文件夹描述
        /// </summary>
        /// <value>General</value>
        /// <code>0</code>
        public string FolderDescription { get; set; }
        /// <summary>
        ///测量模式名称
        /// </summary>
        /// <value>2D</value>
        public string ModeName { get; set; }
        /// <summary>
        ///应用Id
        /// </summary>
        /// <value>Adult1</value>
        public string ApplicationId { get; set; }
        /// <summary>
        ///应用名称
        /// </summary>
        /// <value>Cardiac</value>
        /// <code>0</code>
        public string Application { get; set; }
        /// <summary>
        /// 测量子集合
        /// </summary>
        /// <value></value>
        public List<string> Children { get; set; }
    }
    /// <summary>
    /// 测量数据
    /// </summary>
    public class MeasuredFeatureDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Measure.StraightLineFeature</value>
        public string TypeName { get; set; }
        /// <summary>
        /// 创建者
        /// </summary>
        /// <value>Distance</value>
        /// <code>0</code>
        public string Creator { get; set; }
        /// <summary>
        /// Guid
        /// </summary>
        /// <value>7e9409ca-0418-4cd3-93c4-85832ba3520d</value>
        public string Guid { get; set; }
        /// <summary>
        /// 是否显示
        /// </summary>
        /// <value>true</value>
        /// <code>0</code>
        public bool ShowChildFeatures { get; set; }
        /// <summary>
        /// 时间
        /// </summary>
        /// <value></value>
        /// <code>0</code>
        public DateTime TimeStamp { get; set; }
        /// <summary>
        /// 是否自动测量
        /// </summary>
        /// <value>true</value>
        /// <code>0</code>
        public bool IsAutoCalculated { get; set; }
        /// <summary>
        /// 是否显示
        /// </summary>
        /// <value>true</value>
        public bool IsVisible { get; set; }
        /// <summary>
        /// 注解
        /// </summary>
        /// <value>IVC Insp</value>
        ///  <code>0</code>
        public string Annotation { get; set; }
        /// <summary>
        /// 简短注解
        /// </summary>
        /// <value>D</value>
        ///  <code>0</code>
        public string BriefAnnotation { get; set; }
        /// <summary>
        /// 基础类型名称
        /// </summary>
        /// <value>VerticalDistance</value>
        ///  <code>0</code>
        public string BaseType { get; set; }
        /// <summary>
        /// x轴单位
        /// </summary>
        /// <value>10</value>
        ///  <code>0</code>
        public Unit XUnit { get; set; }
        /// <summary>
        /// Y轴单位
        /// </summary>
        /// <value>10</value>
        ///  <code>0</code>
        public Unit YUnit { get; set; }
        /// <summary>
        /// 绘制类型
        /// </summary>
        /// <value>StraightLine</value>
        ///  <code>0</code>
        public string ShapeType { get; set; }
        /// <summary>
        /// 测量数值集合
        /// </summary>
        /// <value></value>
        public List<MeasuredValueDO> Values { get; set; }
        /// <summary>
        /// 测量坐标集合
        /// </summary>
        /// <value></value>
        public List<PointDO> Points { get; set; }

    }
    /// <summary>
    /// 测量数值
    /// </summary>
    public class MeasuredValueDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Measure.StraightLineFeature</value>
        public string TypeName { get; set; }

        /// <summary>
        /// 名称
        /// </summary>
        /// <value>Distance</value>
        ///  <code>0</code>
        public string Name { get; set; }

        /// <summary>
        /// 描述
        /// </summary>
        /// <value>Distance</value>
        ///  <code>0</code>
        public string Description { get; set; }

        /// <summary>
        /// 是否关联工作表
        /// </summary>
        /// <value></value>
        ///  <code>0</code>
        public bool WorksheetAffinity { get; set; }
        /// <summary>
        /// 描述简称
        /// </summary>
        /// <value>D</value>
        ///  <code>0</code>
        public string BriefDescription { get; set; }

        /// <summary>
        /// 数值
        /// </summary>
        /// <value>22.58</value>
        ///  <code>0</code>
        public string Value { get; set; }

        /// <summary>
        /// 单位
        /// </summary>
        /// <value>10</value>
        ///  <code>0</code>
        public Unit Unit { get; set; }

        /// <summary>
        ///目标单位
        /// </summary>
        /// <value>10</value>
        ///  <code>0</code>
        public Unit TargetUnit { get; set; }
    }
    /// <summary>
    /// 坐标点数值
    /// </summary>
    public class PointDO
    {
        /// <summary>
        /// x轴上数值
        /// </summary>
        /// <value>-11.8984573576828</value>
        ///  <code>0</code>
        public double x { get; set; }
        /// <summary>
        /// y轴上数值
        /// </summary>
        /// <value>2.80014851119276</value>
        ///  <code>0</code>
        public double y { get; set; }
    }
    /// <summary>
    /// 注释数据
    /// </summary>
    public class ScanImageDO
    {
        /// <summary>
        ///Visual
        /// </summary>
        /// <value></value>
        public VisualDO Visual { get; set; }
    }
    /// <summary>
    /// Visual
    /// </summary>
    public class VisualDO
    {
        /// <summary>
        /// Visuals
        /// </summary>
        /// <value></value>
        public List<VisualKeyDO> Children { get; set; }
    }
    /// <summary>
    /// VisualKeyDTO
    /// </summary>
    public class VisualKeyDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>Visual_A</value>
        public string VisualKeyTypeName { get; set; }
        /// <summary>
        /// VisualArea
        /// </summary>
        /// <value></value>
        public VisualAreaDO VisualArea { get; set; }

    }
    /// <summary>
    /// VisualArea
    /// </summary>
    public class VisualAreaDO
    {
        /// <summary>
        /// VisualAreas
        /// </summary>
        /// <value></value>
        public List<BaseAreaDO> Children { get; set; }
    }
    /// <summary>
    /// Area
    /// </summary>
    public class BaseAreaDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value>TissueArea</value>
        public string VisualAreaTypeName { get; set; }
        /// <summary>
        /// 注释集合
        /// </summary>
        /// <value></value>
        public List<AdornerDO> Adorner { get; set; }
    }
    /// <summary>
    /// 注释
    /// </summary>
    public class AdornerDO
    {
        /// <summary>
        /// 类型
        /// </summary>
        /// <value> TextAnnotation</value>
        public string AdornerTypeName { get; set; }
        /// <summary>
        /// 坐标
        /// </summary>
        /// <value></value>
        public PointDO TopLeft { get; set; }
        /// <summary>
        /// 注释文本
        /// </summary>
        /// <value>二尖瓣前叶</value>
        public string Content { get; set; }
    }

}
