﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 客户端测量图像数据
    /// </summary>
    public class RemedicalMeasuredInfoDO : BaseDO
    {
        /// <summary>
        /// 客户端测量图像数据编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string RemedicalMeasuredInfoCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 模式
        /// </summary>
        /// <value>Tissue</value>
        public string Model { get; set; } = string.Empty;

        /// <summary>
        /// 用户编码
        /// </summary>
        /// <value>UserDO_20220315125334n5j3A2</value>
        [QueryItem]
        public string UserCode { get; set; } = string.Empty;

        /// <summary>
        /// 扫查记录编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        [QueryItem]
        public string RecordCode { get; set; } = string.Empty;

        /// <summary>
        /// 扫查图像编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        [QueryItem]
        public string RemedicalCode { get; set; } = string.Empty;

        /// <summary>
        /// 第几帧，默认0
        /// </summary>
        /// <value>0</value>
        public int FrameIndex { get; set; }

        private string _measuredFileToken;
        private string _previewFileToken;
        /// <summary>
        /// 原图地址
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        public string MeasuredFileToken
        {
            get
            {
                return DOShare.ChangeUrl(_measuredFileToken);
            }
            set
            {
                _measuredFileToken = value;
            }
        }

        /// <summary>
        /// 缩略图地址
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        public string PreviewFileToken
        {
            get
            {
                return DOShare.ChangeUrl(_previewFileToken);
            }
            set
            {
                _previewFileToken = value;
            }
        }

        /// <summary>
        /// 测量图像数据详情
        /// </summary>
        /// <value></value>
        public string MeasuredData { get; set; } = string.Empty;

        /// <summary>
        /// URM原图编码
        /// </summary>
        /// <value></value>
        public string UrmOriginalCode { get; set; }

        /// <summary>
        /// URM分析/测量图像编码
        /// </summary>
        /// <value></value>
        public string RemedicalOriginalCode { get; set; }
    }
}
