﻿using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 报告
    /// </summary>
    public class ReportDO : BaseDO
    {
        /// <summary>
        /// 报告编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string ReportCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 检查编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string RecordCode { get; set; } = string.Empty;

        /// <summary>
        /// 报告医生
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string ReportUserCode { get; set; } = string.Empty;

        /// <summary>
        /// 设备编码
        /// </summary>
        [QueryItem]
        public string DeviceCode { get; set; } = string.Empty;

        /// <summary>
        /// 病人Code
        /// </summary>
        [QueryItem]
        public string PatientCode { get; set; } = string.Empty;

        /// <summary>
        /// 病人Id
        /// </summary>
        [QueryItem]
        public string PatientId { get; set; } = string.Empty;

        /// <summary>
        /// 报告模板Json
        /// </summary>
        public string ReportTemplateJson { get; set; } = string.Empty;

        /// <summary>
        /// 报告模版Code
        /// </summary>
        /// <value></value>
        public string ReportTemplateCode { get; set; }

        /// <summary>
        /// 报告内容Json
        /// </summary>
        public string ReportDatasJson { get; set; } = string.Empty;

        /// <summary>
        /// 报告测量项内容Json
        /// </summary>
        public string ReportMeasureDatasJson { get; set; } = string.Empty;

        /// <summary>
        /// 病人姓名
        /// </summary>
        public string PatientName { get; set; } = string.Empty;

        /// <summary>
        /// 报告标签
        /// </summary>
        public List<string> ReportLabels { get; set; } = new();

        /// <summary>
        /// 来源标签
        /// </summary>
        public List<string> PlatformLabels { get; set; } = new();

        /// <summary>
        /// 诊断标签
        /// </summary>
        public List<string> DiagnosisLabels { get; set; } = new();

        /// <summary>
        /// 检查器官
        /// </summary>
        /// <value>2</value>
        public DiagnosisOrganEnum ReportOrgan { get; set; }

        /// <summary>
        /// 报告预览图
        /// </summary>
        public List<ReportPreviewDO> ReportPreviewList { get; set; } = new List<ReportPreviewDO>();

        /// <summary>
        /// 转诊记录编码
        /// </summary>
        public string ReferralRecordCode { get; set; } = string.Empty;

        /// <summary>
        /// 业务类型
        /// </summary>
        /// <value>0</value>
        public ReportTypeEnum ReportType { get; set; } = ReportTypeEnum.RemoteDiagnosis;

        /// <summary>
        /// 是否转诊报告
        /// </summary>
        /// <value></value>
        public bool IsReferral { get; set; }

        /// <summary>
        /// 质控状态枚举
        /// </summary>
        /// <value></value>
        public QualityType QualityType { get; set; } = QualityType.None;

        /// <summary>
        /// 质控合格状态
        /// </summary>
        /// <value></value>
        public QualifiedState QualifiedState { get; set; } = QualifiedState.UnSet;

        /// <summary>
        /// 质控结果
        /// </summary>
        /// <value></value>
        public ReportQualityControl QualityControlDatas { get; set; }

        /// <summary>
        /// 设备报告id
        /// </summary>
        /// <value></value>
        public string DeviceReportId { get; set; }

        /// <summary>
        /// 提交报告的原始信息数据
        /// </summary>
        /// <value></value>
        public string ReportExamDataJson { get; set; }

        /// <summary>
        /// 健康诊断 0：默认；1：正常；2：异常
        /// </summary>
        /// <value></value>
        public int HealthDiagnosis { get; set; }
    }

    public class ReportPreviewDO
    {
        private string _fileToken;
        private string _thumbnailToken;
        /// <summary>
        /// 文件地址
        /// </summary>
        /// <value></value>
        public string FileToken
        {
            get
            {
                return DOShare.ChangeUrl(_fileToken);
            }
            set
            {
                _fileToken = value;
            }
        }

        /// <summary>
        /// 缩略图
        /// </summary>
        /// <value>http://*****.jpg</value>
        public string ThumbnailToken
        {
            get
            {
                return DOShare.ChangeUrl(_thumbnailToken);
            }
            set
            {
                _thumbnailToken = value;
            }
        }

        /// <summary>
        /// 语言，可为空-医师报告
        /// </summary>
        /// <value></value>
        public string Language { get; set; }

        /// <summary>
        /// 文件类型
        /// </summary>
        /// <value></value>
        public UploadFileTypeEnum FileType { get; set; }
    }
}
