﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    /// <summary>
    /// 词条库
    /// </summary>
    public class ThesaurusDO : BaseDO
    {
        /// <summary>
        /// 词库编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string ThesaurusCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        [QueryItem]
        /// <summary>
        /// 词库名称
        /// </summary>
        public string ThesaurusName { get; set; }

        [QueryItem]
        /// <summary>
        /// 词条发布状态
        /// </summary>
        /// <value></value>
        public ThesaurusStatusTypeEnum ThesaurusStateType { get; set; }

        [QueryItem]
        /// <summary>
        /// 词条类型
        /// </summary>
        /// <value></value>
        public ThesaurusTypeEnum ThesaurusType { get; set; }

        [QueryItem]
        /// <summary>
        /// 词条使用类型
        /// </summary>
        /// <value></value>
        public ThesaurusUsageTypeEnum ThesaurusUsageType { get; set; }

        [QueryItem]
        /// <summary>
        /// 限定所属组织
        /// </summary>
        /// <value></value>
        public string OrganizationCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 限定所属人
        /// </summary>
        /// <value></value>
        public string PersonalCode { get; set; }

        /// <summary>
        /// 词条项集合
        /// </summary>
        /// <value></value>
        public List<ThesaurusItemDO> ThesaurusItems { get; set; }

        [QueryItem]
        /// <summary>
        /// 拷贝时引用的词条编码
        /// </summary>
        /// <value></value>
        public string ReferenceThesaurusCode { get; set; }

        [BsonIgnore]
        [Obsolete]
        public List<string> ThesaurusUser { get; set; }

        [QueryItem]
        /// <summary>
        /// 是否默认
        /// </summary>
        /// <value></value>
        public bool IsDefault { get; set; }


        [QueryItem]
        /// <summary>
        /// 所属语言（代表是什么语言类型）
        /// </summary>
        /// <value></value>
        public string LanguageCode { get; set; }

        /// <summary>
        /// 词条业务类型
        /// </summary>
        /// <value></value>
        public ThesaurusBusinessTypeEnum ThesaurusBusinessType { get; set; } = ThesaurusBusinessTypeEnum.Ultrasonic;
    }

    /// <summary>
    /// 词条项（词条分类或词条内容）
    /// </summary>
    public class ThesaurusItemDO
    {
        /// <summary>
        /// 词条项唯一键（Guid）
        /// </summary>
        /// <value></value>
        public string ThesaurusItemCode { get; set; }

        /// <summary>
        /// 词条项类型
        /// </summary>
        /// <value></value>
        public ThesaurusItemTypeEnum ThesaurusItemType { get; set; }

        /// <summary>
        /// 词条项名称（自定义词条使用）
        /// </summary>
        /// <value></value>
        public string ThesaurusItemName { get; set; }

        /// <summary>
        /// 词条项描述
        /// </summary>
        /// <value></value>
        public string ThesaurusItemDescription { get; set; }

        /// <summary>
        /// 词条项结论
        /// </summary>
        /// <value></value>
        public string ThesaurusItemConclusion { get; set; }

        /// <summary>
        /// 父级词条项（为空则为顶级项）
        /// </summary>
        /// <value></value>
        public string ParentItemCode { get; set; }

        /// <summary>
        /// 是否有子节点
        /// </summary>
        /// <value></value>
        public bool HasChildren { get; set; }

        /// <summary>
        /// 排序
        /// </summary>
        /// <value></value>
        public int Sort { get; set; }
    }
}
