
using System.Collections.Generic;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    public class UserDefinedMeasureInfoDO : BaseDO
    {
        /// <summary>
        ///用户Id
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string UserId { get; set; }
        /// <summary>
        /// 用户自定义测量设置
        /// </summary>
        /// <value></value>
        public Dictionary<string, string> SystemSettings { get; set; } = new Dictionary<string, string>();
        /// <summary>
        /// 测量集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedMeasureApplicationDO> MeasureApplications { get; set; } = new List<UserDefinedMeasureApplicationDO>();
        /// <summary>
        /// 注释集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedCommentDataDO> CommentItems { get; set; } = new List<UserDefinedCommentDataDO>();

        /// <summary>
        /// 体标集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedBodyPatternDataDO> BodyPatterns { get; set; } = new List<UserDefinedBodyPatternDataDO>();
    }

    /// <summary>
    /// 体标
    /// </summary>
    public class UserDefinedBodyPatternDataDO
    {        
        /// <summary>
        /// 版本号
        /// </summary>
        /// <value>1.0.0.0</value>
        public string Version { get; set; }

        /// <summary>
        /// 分类名称
        /// </summary>
        /// <value>ABD</value>
        public string CategoryName { get; set; }

        /// <summary>
        /// 应用名称
        /// </summary>
        /// <value>Abdo Diff</value>
        public string ApplicationName { get; set; }

        /// <summary>
        /// 体标内容
        /// </summary>
        /// <value></value>
        public List<BodyPatternDO> BodyPatternItems { get; set; }
    }
    
    /// <summary>
    /// 体标内容
    /// </summary>
    public class BodyPatternDO
    {
        /// <summary>
        /// 版本号
        /// </summary>
        /// <value>1.0</value>
        public string Category { get; set; }

        /// <summary>
        /// 描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        /// <summary>
        /// 体标图像列表
        /// </summary>
        /// <value></value>
        public IList<string> ImageUrlList { get; set; } 
    }

    /// <summary>
    /// 测量项
    /// </summary>
    public class UserDefinedMeasureApplicationDO
    {
        /// <summary>
        ///测量应用Id
        /// </summary>
        /// <value>Abdo Diff</value>
        public string Id { get; set; }
        /// <summary>
        /// 版本号
        /// </summary>
        /// <value>1.0.0.1</value>
        public string Version { get; set; }

        /// <summary>
        ///测量应用分类
        /// </summary>
        /// <value>ABD</value>
        public string CategoryName { get; set; }

        /// <summary>
        ///测量工作模式集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedMeasureModeDO> Modes { get; set; }


    }
    /// <summary>
    /// 测量模式
    /// </summary>
    public class UserDefinedMeasureModeDO
    {
        /// <summary>
        /// 测量模式名称
        /// </summary>
        /// <value>Tissue</value>
        public string ModeName { get; set; }
        /// <summary>
        /// 测量配置组集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedMeasureGroupDO> WorkingGroups { get; set; } = new List<UserDefinedMeasureGroupDO>();

    }
    /// <summary>
    /// 测量组
    /// </summary>
    public class UserDefinedMeasureGroupDO
    {
        /// <summary>
        ///测量组名称
        /// </summary>
        /// <value>General</value>
        public string Name { get; set; }
        /// <summary>
        ///测量文件夹集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedMeasureFolderDO> Folders { get; set; }
    }

    /// <summary>
    /// 测量文件夹
    /// </summary>
    public class UserDefinedMeasureFolderDO
    {
        /// <summary>
        /// 测量文件夹名称
        /// </summary>
        /// <value>General</value>
        public string Name { get; set; }
        /// <summary>
        /// 工作测量配置项名称集合
        /// </summary>
        /// <value>"Area","Volume"</value>
        public List<string> WorkingItemNames { get; set; }

        /// <summary>
        /// 默认工作测量配置项
        /// </summary>
        /// <value>General</value>
        public string DefaultItem { get; set; }
        /// <summary>
        ///带有多个测量方法的测量项集合
        /// </summary>
        /// <value></value>
        public List<UserDefinedItemMetaDO> MultiMethodItemMetas { get; set; }

        /// <summary>
        /// 设置测量项数据
        /// </summary>
        /// <value></value>
        public string ItemMetaJson { get; set; }

    }
    /// <summary>
    /// 测量项
    /// </summary>
    public class UserDefinedItemMetaDO
    {
        /// <summary>
        ///测量名称
        /// </summary>
        /// <value>Area</value>
        public string Name { get; set; }

        /// <summary>
        ///当前测量方法名称
        /// </summary>
        /// <value>LW</value>
        public string WorkingMethodItem { get; set; }

    }

    /// <summary>
    /// 注释
    /// </summary>
    public class UserDefinedCommentDataDO
    {
        /// <summary>
        /// 版本号
        /// </summary>
        /// <value>1.0.0.0</value>
        public string Version { get; set; }
        /// <summary>
        /// 请求语言code
        /// </summary>
        /// <value>zh-CN</value>
        public string LanguageCode { get; set; }

        /// <summary>
        /// 分类名称
        /// </summary>
        /// <value>ABD</value>
        public string CategoryName { get; set; }

        /// <summary>
        /// 应用名称
        /// </summary>
        /// <value>Abdo Diff</value>
        public string ApplicationName { get; set; }

        /// <summary>
        /// 注释内容
        /// </summary>
        /// <value></value>
        public List<CommentItemDO> CommentItems { get; set; }
    }
    /// <summary>
    /// 注释内容
    /// </summary>
    public class CommentItemDO
    {
        /// <summary>
        /// 注释内容
        /// </summary>
        /// <value>"肝左叶"</value>
        public string Text { get; set; }
    }
}
