﻿using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DO.Enum;
using WingMongoDBModule.Mongodb.Driver;
using System;
namespace WingMongoDBModule.DO
{
    public class UserInfoDO : BaseDO
    {
        /// <summary>
        /// 用户编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string UserCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        [UniqueItem]
        [QueryItem]
        /// <summary>
        /// 用户名
        /// </summary>
        public string UserName { get; set; }

        /// <summary>
        /// 密码
        /// </summary>
        public string SecretPassword { get; set; }

        [QueryItem]
        /// <summary>
        /// 联系电话
        /// </summary>
        public string Phone { get; set; }

        [QueryItem]
        /// <summary>
        /// 邮箱
        /// </summary>
        public string Email { get; set; }

        /// <summary>
        /// 用户昵称
        /// </summary>
        public string NickName { get; set; }

        [QueryItem]
        /// <summary>
        /// 用户姓名
        /// </summary>
        public string FullName { get; set; }

        private string _headImageToken;

        /// <summary>
        /// 头像
        /// </summary>
        public string HeadImageToken
        {
            get
            {
                return DOShare.ChangeUrl(_headImageToken);
            }
            set
            {
                _headImageToken = value;
            }
        }

        [QueryItem]
        /// <summary>
        /// 所属组织
        /// </summary>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 所属权限组
        /// </summary>
        public List<string> AuthorityGroups { get; set; } = new();

        /// <summary>
        /// 绑定设备列表
        /// </summary>
        public List<string> BindDevices { get; set; } = new();

        /// <summary>
        /// 绑定设备机构列表
        /// </summary>
        public List<string> BindDeviceOrganizations { get; set; } = new();

        /// <summary>
        /// 登录次数
        /// </summary>
        /// <value></value>
        public int Logintimes { get; set; }

        /// <summary>
        /// 积分，计算等级
        /// </summary>
        public int Score { get; set; }

        /// <summary>
        /// 最后使用IP地址
        /// </summary>
        public string LastIP { get; set; }

        [QueryItem]
        /// <summary>
        /// 用户状态
        /// </summary>
        /// <value></value>
        public UserInfoStateEnum UserState { get; set; }

        /// <summary>
        /// 岗位身份列表
        /// </summary>
        /// <value></value>
        public List<string> PositionCodes { get; set; } = new();

        /// <summary>
        /// 用户认证角色列表
        /// </summary>
        /// <value></value>
        public List<string> RoleCodes { get; set; } = new();

        /// <summary>
        /// 用户职称列表
        /// </summary>
        /// <value></value>
        public List<string> RankCodes { get; set; } = new();

        /// <summary>
        /// 词条库
        /// </summary>
        /// <value></value>
        public List<string> ThesaurusCodes { get; set; } = new();

        /// <summary>
        /// 擅长领域
        /// </summary>
        /// <value>"test1","test2"</value>
        public List<string> FieldList { get; set; } = new List<string>();

        /// <summary>
        /// 删除的病人
        /// </summary>
        public List<string> DeletePatientCodes { get; set; } = new();

        /// <summary>
        /// 是否开启批量导出分时诊断数据
        /// </summary>
        public bool IsBatchExportDiagnoseData { get; set; }

        [QueryItem]
        /// <summary>
        /// 绑定的助理用户编码
        /// </summary>
        /// <value></value>
        public string BindAssistantUserCode { get; set; }

        /// <summary>
        /// 助理医师用户编码
        /// </summary>
        /// <value></value>
        [QueryItem]
        public string BindAssistantDoctorUserCode { get; set; }

        /// <summary>
        /// 登录失败信息 
        /// </summary>
        /// <value></value>
        public LoginLockInfoDO LoginLockInfo { get; set; }

        /// <summary>
        /// password update time
        /// </summary>
        public DateTime PasswordUpdateTime { get; set; }

        /// <summary>
        /// 密码记录列表
        /// </summary>
        public List<string> PasswordRecords { get; set; } = new();

        private string _signature;

        /// <summary>
        /// 签名
        /// </summary>
        public string Signature
        {
            get
            {
                return DOShare.ChangeUrl(_signature);
            }
            set
            {
                _signature = value;
            }
        }

        /// <summary>
        /// 语言
        /// </summary>
        public string Language { get; set; }

        /// <summary>
        /// 报告标签
        /// </summary>
        /// <value></value>
        public bool EnableReportLabel { get; set; }

        /// <summary>
        /// 第三方关联信息
        /// </summary>
        /// <value></value>
        public List<AssociatedInfo> AssociatedInfos { get; set; }

        /// <summary>
        /// 公共平台用户ID
        /// </summary>
        /// <value></value>
        public string CommonPlatformUserId { get; set; }

        /// <summary>
        /// 急诊设备编码
        /// </summary>
        /// <value></value>
        public string BindEmergencyDeviceCode { get; set; }

        /// <summary>
        /// 急诊专家编码
        /// </summary>
        /// <value></value>
        public string BindEmergencyExpertCode { get; set; }

        /// <summary>
        /// 配置Dashboard查询医院编码集合
        /// </summary>
        /// <value></value>
        public List<string> DashboardOrgCodes { get; set; }

        /// <summary>
        /// 是否代理商
        /// </summary>
        /// <value></value>
        public bool IsOldAgent { get; set; }

        /// <summary>
        /// 用户绑定权限列表
        /// </summary>
        /// <value></value>
        public List<string> UserFeatureCodes { get; set; }

        /// <summary>
        /// 微信OpenId
        /// </summary>
        /// <value></value>
        public string OpenId { get; set; }

        /// <summary>
        /// 生日
        /// </summary>
        /// <value></value>
        public string Birthday { get; set; } = string.Empty;

        /// <summary>
        /// 出生年月
        /// </summary>
        /// <value></value>
        public DateTime? BirthDate { get; set; } = DateTime.MinValue;

        /// <summary>
        /// 性别：0-未知；1-男；2-女
        /// </summary>
        /// <value></value>
        public int Gender { get; set; }

        /// <summary>
        /// 颈围 
        /// </summary>
        /// <value></value>
        public float NeckGirth { get; set; }
        /// <summary>
        /// 身高 
        /// </summary>
        /// <value></value>
        public float Height { get; set; }
        /// <summary>
        /// 体重 
        /// </summary>
        /// <value></value>
        public float Weight { get; set; }
        /// <summary>
        /// 体脂数 
        /// </summary>
        /// <value></value>
        public float BMI { get; set; }

        /// <summary>
        /// 科研项目Dongle 
        /// </summary>
        /// <value></value>
        public string LabDongle { get; set; }

        /// <summary>
        /// 绑定资源包列表历史记录 
        /// </summary>
        /// <value></value>
        public List<BindChargeResourcePackageDO> BindChargeResourcePackageRecords { get; set; }

        /// <summary>
        /// 绑定资源包列表 
        /// </summary>
        /// <value></value>
        public List<BindChargeResourcePackageDO> BindChargeResourcePackages { get; set; }
    }

    /// <summary>
    /// 用户绑定计费资源信息实体 
    /// </summary>
    /// <value></value>
    public class BindChargeResourcePackageDO
    {
        /// <summary>
        /// 绑定资源包列code 
        /// </summary>
        /// <value></value>
        public string ResourcePackageCode { get; set; }

        /// <summary>
        /// 业务类型code 
        /// </summary>
        /// <value></value>
        public string BussinessCode { get; set; }

        /// <summary>
        /// 计费资源包类型
        /// </summary>
        /// <value></value>
        public ChargeResourcePackageTypeEnum ResourcePackageType { get; set; }

        /// <summary>
        /// 数量 
        /// </summary>
        /// <value></value>
        public int Count { get; set; }
        
        /// <summary>
        /// 开始时间 
        /// </summary>
        /// <value></value>
        public DateTime BeginTime { get; set; }
        
        /// <summary>
        /// 结束时间 
        /// </summary>
        /// <value></value>
        public DateTime EndTime { get; set; }
    }

    /// <summary>
    ///登录失败信息 
    /// </summary>
    public class LoginLockInfoDO
    {
        public DateTime LoginDate { get; set; }

        public int Times { get; set; }
    }

    /// <summary>
    /// 第三方关联信息
    /// </summary>
    public class AssociatedInfo
    {
        /// <summary>
        /// Id
        /// </summary>
        /// <value></value>
        public string Id { get; set; }

        /// <summary>
        /// 关联用户名
        /// </summary>
        /// <value></value>
        public string RelationName { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        /// <value></value>
        public string Title { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        /// <value></value>
        public string CTitle { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        /// <value></value>
        public string ETitle { get; set; }

        private string _icon;
        /// <summary>
        /// Ico
        /// </summary>
        /// <value></value>
        public string Icon
        {
            get
            {
                return DOShare.ChangeUrl(_icon);
            }
            set
            {
                _icon = value;
            }
        }

        /// <summary>
        /// 描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        private string _url;
        /// <summary>
        /// 系统地址
        /// </summary>
        /// <value></value>
        public string Url
        {
            get
            {
                return DOShare.ChangeUrl(_url);
            }
            set
            {
                _url = value;
            }
        }

        /// <summary>
        /// 索引号
        /// </summary>
        /// <value></value>
        public int Index { get; set; }
    }
}
