﻿using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.DO
{
    public class UserRoleDO : BaseDO
    {
        /// <summary>
        /// 角色编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string RoleCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        [QueryItem]
        /// <summary>
        /// 角色名称
        /// </summary>
        public string RoleName { get; set; }

        /// <summary>
        /// 前端权限组
        /// </summary>
        /// <value></value>
        public string UserGroupCode { get; set; }

        [QueryItem]
        /// <summary>
        /// 角色类型 0-不展示，1-展示
        /// </summary>
        /// <value></value>
        public RoleShowTypeEnum RoleShowType { get; set; }

        [QueryItem]
        /// <summary>
        /// 组织编码（可空，空则为公开）
        /// </summary>
        /// <value></value>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 角色描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        private string _iconUrl;
        /// <summary>
        /// 角色icon
        /// </summary>
        /// <value></value>
        public string IConUrl
        {
            get
            {
                return DOShare.ChangeUrl(_iconUrl);
            }
            set
            {
                _iconUrl = value;
            }
        }

        /// <summary>
        /// 颜色配置起
        /// </summary>
        /// <value></value>
        public string ColorStart { get; set; }

        /// <summary>
        /// 颜色配置止
        /// </summary>
        /// <value></value>
        public string ColorEnd { get; set; }

        /// <summary>
        /// 资质文件类型
        /// </summary>
        /// <value></value>
        public RoleQualificationEnum RoleQualification { get; set; }

        /// <summary>
        /// 角色多语言配置
        /// </summary>
        /// <returns></returns>
        public List<UserRoleLanguageConfigDO> LanguageConfigs { get; set; } = new();

        [QueryItem]
        /// <summary>
        /// 擅长领域显示类型 0-不展示，1-展示
        /// </summary>
        /// <value></value>
        public RoleShowTypeEnum FieldShowType { get; set; }

    }

    /// <summary>
    /// 用户角色配置-语言
    /// </summary>
    public class UserRoleLanguageConfigDO
    {
        /// <summary>
        /// 语言
        /// </summary>
        /// <value></value>
        public string Language { get; set; }

        /// <summary>
        /// 角色名称
        /// </summary>
        public string RoleName { get; set; }

        /// <summary>
        /// 角色描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        /// <summary>
        /// 擅长领域
        /// </summary>
        /// <typeparam name="string"></typeparam>
        /// <returns></returns>
        public List<string> FieldList { get; set; } = new List<string>();
    }
}
