﻿using System;
using MongoDB.Driver;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;

namespace WingMongoDBModule.Mongodb.Driver
{
    public sealed class MongoClientFactory
    {
        private MongoClientFactory() { }

        private static int _maxConn = ConfigurationManager.GetParammeter<IntParameter>("Database", "MaxConn").Value;

        /// <summary>
        /// 创建一个 MongoClientSettings 对象，设置 ConnectTimeout 和 Server 属性，然后使用 MongoClientSettings 对象创建一个
        /// MongoClient 对象
        /// </summary>
        /// <param name="host">MongoDB 服务器的主机名。</param>
        /// <param name="port">MongoDB 实例的端口号。</param>
        /// <returns>
        /// 一个 MongoClient 对象。
        /// </returns>
        public static MongoClient CreateClient(string host, int port)
        {
            MongoClientSettings mongoClientSettings = new MongoClientSettings();
            mongoClientSettings.ConnectTimeout = TimeSpan.FromSeconds(60);
            mongoClientSettings.Server = new MongoServerAddress(host, port);
            mongoClientSettings.MaxConnectionPoolSize = _maxConn;
            var client = new MongoClient(mongoClientSettings);
            return client;
        }
    }
}