using System;
using System.Threading.Tasks;

namespace WingMongoDBModule.Mongodb.Driver
{
    public class SpinThread
    {
        private bool _keepRunning = false;
        private string _taskName { get; set; }
        private TimeSpan _waitTime;
        private Action<object> _action;
        private object _param;
        public SpinThread(string taskName, TimeSpan waitTime, Action<object> action, object param = null)
        {
            _taskName = taskName;
            _waitTime = waitTime;
            _action = action;
            _param = param;
        }

        /// <summary>
        /// 开始运行
        /// </summary>
        public void Start()
        {
            _keepRunning = true;
            Task.Run(async() =>
            {
                Console.WriteLine($"[ScheduledWork] {_taskName} start running");
                while (_keepRunning)
                {
                    try
                    {
                        _action?.Invoke(_param);
                        await Task.Delay(_waitTime);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine($"[ScheduledWork] {_taskName} error:{ex}");
                    }
                }
                Console.WriteLine($"[ScheduledWork] {_taskName} stoped");
            });
        }

        /// <summary>
        /// 停止运行
        /// </summary>
        public void Stop()
        {
            _keepRunning = false;
        }

        /// <summary>
        /// 析构时停止
        /// </summary>
        ~SpinThread()
        {
            Stop();
        }
    }
}