using System;
using System.Diagnostics;
using System.IO;
using System.Linq;

namespace WingMongoDBModule.Driver
{
    public class BackupDBJob
    {
        private BackupDBJob(){}
        
        /// <summary>
        /// > 执行 mongodump 命令备份数据库
        /// </summary>
        public static void Execute()
        {
            try
            {
                Console.WriteLine($"Mongodb backup job starting");
                var toolPath = string.Empty;
                var mongoPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Packages");
                if (Environment.OSVersion.Platform == PlatformID.Win32NT)
                {
                    toolPath = Path.Combine(mongoPath, "mongodump.exe");
                }
                else
                {
                    toolPath = Path.Combine(mongoPath, "mongodump");
                }
                var bakPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Mongodb", "Bak");
                Directory.CreateDirectory(bakPath);
                var directories = Directory.GetDirectories(bakPath);
                if (directories.Length > 10)
                {
                    var needDelInfos = directories.Select(f => new { path = f, createTime = Directory.GetCreationTime(f) });
                    var needDelPaths = needDelInfos.OrderBy(f => f.createTime).Take(5);
                    foreach (var delPath in needDelPaths)
                    {
                        Directory.Delete(delPath.path, true);
                    }
                }
                var needMakePath = Path.Combine(bakPath, DateTime.UtcNow.ToString("yyyyMMddHHmmss"));
                var mkdir = Directory.CreateDirectory(needMakePath);
                var arguments = $"-h 127.0.0.1:9100 -d WingCloudDB -o {mkdir} --quiet";
                var startInfo = new ProcessStartInfo
                {
                    FileName = toolPath,
                    Arguments = arguments,
                    RedirectStandardOutput = false,
                };
                using (var process = Process.Start(startInfo))
                {
                    Console.WriteLine($"Mongodb backup job completed");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Mongodb backup job error {ex}");
                throw;
            }
        }
    }
}
