using System.Collections.Generic;
using WingInterfaceLibrary.Enum;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 系统公告
    /// </summary>
    public class AnnouncementInfoEntity : BaseEntity
    {
        /// <summary>
        /// 公告类型
        ///【广播】 
        /// </summary>
        /// <value></value>
        public AnnouncementTypeEnum AnnouncementType { get; set; }

        // /// <summary>
        // /// 公告时间
        // /// 2023-01-20 09:00
        // /// </summary>
        // /// <value></value>
        // public DateTime AnnouncementTime { get; set; }

        // /// <summary>
        // /// 过期时间
        // /// 2023-11-20 09:00
        // /// </summary>
        // /// <value></value>
        // public DateTime ExpireTime { get; set; }

        /// <summary>
        /// 公告状态
        /// 已发布 未发布 
        /// </summary>
        /// <value>0</value>
        public AnnouncementStatusEnum AnnouncementStatus { get; set; }

        /// <summary>
        /// 图片
        /// </summary>
        /// <value></value>
        public string FileToken { get; set; }

        /// <summary>
        /// 排序状态
        /// </summary>
        /// <value>0</value>
        public int Sort { get; set; }

        /// <summary>
        /// 设备类型
        /// </summary>
        /// <value></value>
        public string DeviceType { get; set; }

        /// <summary>
        /// 发布内容的语言
        /// </summary>
        /// <value>zh-CN</value>
        public IList<AnnouncementLanguageConfigEntity> LanguageConfigs { get; set; }

        /// <summary>
        /// 视频地址
        /// </summary>
        /// <value>https://flyinsono-bj-1300984704.cos.ap-beijing.myqcloud.com/video.mp4</value>
        public string VideoToken { get; set; }

        /// <summary>
        /// 封面图
        /// </summary>
        /// <value>https://flyinsono-bj-1300984704.cos.ap-beijing.myqcloud.com/0B7B494B7A7044F3ADE0FCF8C0422700.png</value>
        public string Poster { get; set; }

        /// <summary>
        /// Vod云点播文件Id
        /// </summary>
        /// <value>12334e</value>
        public string VodFileId { get; set; }

        /// <summary>
        /// 课程时长【单位秒】
        /// </summary>
        /// <value>120</value>
        public int Duration { get; set; }

        /// <summary>
        /// 视频大小【MB】
        /// </summary>
        /// <value>120</value>
        public double VideoSize { get; set; }

    }
    
    /// <summary>
    /// 公告语言配置实体
    /// </summary>
    public class AnnouncementLanguageConfigEntity
    {
        /// <summary>
        /// 语言
        /// </summary>
        /// <value></value>
        public string Language { get; set; }

        /// <summary>
        /// 公告标题
        /// </summary>
        /// <value>杏聆荟将以进行系统维护，期间暂停服务，特此公告</value>
        public string Title { get; set; }

        /// <summary>
        /// 公告内容
        /// </summary>
        /// <value>平台将于2024年6月30日18：00-19：00进行系统更新。更新内容如下：【可用文本编辑器或者textarea】</value>
        public string Content { get; set; }
    }
}