﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;
using WingInterfaceLibrary.LiveConsultation;
using WingInterfaceLibrary.LiveConsultation.Enums;
using WingMongoDBModule.Entities;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 会诊记录表
    /// </summary>
    public class ConsultationRecordEntity : BaseEntity
    {
        /// <summary>
        /// 管理员编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string ConsultationCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 申请机构编码
        /// </summary>
        [QueryItem]
        public string ApplyOrganizationCode { get; set; }

        /// <summary>
        /// 申请科室编码
        /// </summary>
        [QueryItem]
        public string ApplyDepartmentCode { get; set; }

        /// <summary>
        /// 申请医生编码
        /// </summary>
        [QueryItem]
        public string ApplyUserCode { get; set; }

        /// <summary>
        /// 会诊机构编码
        /// </summary>
        [QueryItem]
        public string ExpertOrganizationCode { get; set; }

        /// <summary>
        /// 会诊科室编码
        /// </summary>
        [QueryItem]
        public string ExpertDepartmentCode { get; set; }

        /// <summary>
        /// 会诊专家编码
        /// </summary>
        [QueryItem]
        public string ExpertUserCode { get; set; }

        /// <summary>
        /// 设备编码
        /// </summary>
        [QueryItem]
        public string DeviceCode { get; set; }

        /// <summary>
        /// 病人编码
        /// </summary>
        [QueryItem]
        public string PatientCode { get; set; }

        /// <summary>
        /// 病人信息
        /// </summary>
        /// <value></value>
        public IList<DataItemDTO> PatientDatas { get; set; } = new List<DataItemDTO>();

        /// <summary>
        /// 服务对象：人用、兽用
        /// </summary>
        /// <value>0</value>
        [QueryItem]
        public OrganizationPatientTypeEnum PatientType { get; set; }

        /// <summary>
        /// 检查部位 废弃
        /// </summary>
        /// <value>0</value>
        [QueryItem]
        public string ScanPosition { get; set; }

        /// <summary>
        /// 检查部位：肝脏；胆囊；胎儿
        /// </summary>
        /// <value>肝脏 附件</value>        
        /// <code>0</code>
        [QueryItem]
        public List<string> ScanPositions { get; set; }

        /// <summary>
        /// 扫查医生
        /// </summary>
        /// <value>0</value>
        public string ScanUserCode { get; set; }

        /// <summary>
        /// 操作人
        /// </summary>
        /// <value>0</value>
        public string OperateUserCode { get; set; }

        /// <summary>
        /// 会诊开始时间
        /// </summary>
        /// <value>0</value>
        [QueryItem]
        public DateTime ConsultationTime { get; set; }

        /// <summary>
        /// 会诊截止时间
        /// </summary>
        /// <value></value>
        public DateTime ConsultationTimeEnd { get; set; }

        /// <summary>
        /// 会诊状态
        /// </summary>
        /// <value>0</value>
        [QueryItem]
        public TransactionStatusEnum ConsultationStatus { get; set; }

        // /// <summary>
        // /// 评价等级[废弃，请看ConsultationEvaluateEntity]
        // /// </summary>
        // /// <value>0</value>
        // [QueryItem]
        // public EvaluateGradeEnum EvaluateGrade { get; set; }

        // /// <summary>
        // /// 随访状态[废弃，请看FollowUpVisitEntity]
        // /// </summary>
        // /// <value>0</value>
        // [QueryItem]
        // public FollowUpVisitStatusEnum FollowUpVisitStatus { get; set; }

        /// <summary>
        /// 是否需要随访
        /// </summary>
        /// <value>0</value>
        [QueryItem]
        public bool NeedFollowUpVisit { get; set; }

        /// <summary>
        /// 会诊图像列表
        /// </summary>
        public IList<ConsultationFileDO> ConsultationFileList { get; set; }

        /// <summary>
        /// 拒绝申请会诊原因
        /// </summary>
        /// <value></value>
        public string RejectReason { get; set; } = string.Empty;

        /// <summary>
        /// 地点
        /// </summary>
        /// <value></value>
        public string Location { get; set; }

        /// <summary>
        /// 会诊参与方对象列表
        /// </summary>
        /// <value></value>
        public List<ConsultationMemberDO> ConsultationMembers { get; set; }

        /// <summary>
        /// 会诊描述
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        /// <summary>
        /// 会诊提醒集合
        /// </summary>
        /// <value></value>
        public List<ConsultationReminderDO> ConsultationReminders { get; set; }

        /// <summary>
        /// 审核人编码
        /// </summary>
        /// <value></value>
        public string ApproverCode { get; set; }

        /// <summary>
        /// 报告方式
        /// </summary>
        /// <value></value>
        public ConsultationReportMode ReportMode { get; set; }

        [QueryItem]
        /// <summary>
        /// 疾病名称
        /// </summary>
        /// <value></value>
        public string Diseases { get; set; }

        /// <summary>
        /// 初步诊断
        /// </summary>
        /// <value>初步诊断</value>
        [QueryItem]
        public string PrimaryDiagnosis { get; set; }

        /// <summary>
        /// 发起人编码
        /// </summary>
        /// <value></value>
        public string InitiatorCode { get; set; }

        /// <summary>
        /// 是否急诊标识
        /// </summary>
        /// <value></value>
        public bool IsEmergency { get; set; }

        /// <summary>
        /// 急诊code
        /// </summary>
        /// <value></value>
        public string EmergencyCode { get; set; }

        /// <summary>
        /// 急诊实时状态
        /// </summary>
        /// <value></value>
        public EmergencyConsultationStatus EmergencyStatus { get; set; }

        /// <summary>
        /// 是否已会诊
        /// </summary>
        /// <value></value>
        public bool IsConsulted { get; set; }

        /// <summary>
        /// 质控状态枚举
        /// </summary>
        /// <value></value>
        public QualityType QualityType { get; set; } = QualityType.None;

        /// <summary>
        /// 质控合格状态
        /// </summary>
        /// <value></value>
        public QualifiedState QualifiedState { get; set; } = QualifiedState.UnSet;
    }

    /// <summary>
    /// 参与成员对象
    /// </summary>
    public class ConsultationMemberDO
    {
        /// <summary>
        /// 参与人员编码
        /// </summary>
        /// <value></value>
        public string MemberCode { get; set; }

        /// <summary>
        /// 参与人员机构编码
        /// </summary>
        /// <value></value>
        public string MemberOrganizationCode { get; set; }

        /// <summary>
        /// 参与人员科室编码
        /// </summary>
        /// <value></value>
        public string MemberDepartmentCode { get; set; }

        /// <summary>
        /// 参与人员状态
        /// </summary>
        /// <value></value>
        public ConsultationMemeberAcceptStatus MemberStatus { get; set; }

        /// <summary>
        /// 参与人员操作时间
        /// </summary>
        /// <value></value>
        public DateTime MemberOperateTime { get; set; }

        /// <summary>
        /// 拒绝原因
        /// </summary>
        /// <value></value>
        public string RefusalCause { get; set; }
    }

    /// <summary>
    /// 会诊图像信息
    /// </summary>
    public class ConsultationFileDO
    {
        private string _sourceUrl;
        private string _previewImageUrl;
        private string _coverImageUrl;
        /// <summary>
        /// 源文件地址
        /// </summary>
        /// <value></value>
        public string SourceUrl
        {
            get
            {
                return DOShare.ChangeUrl(_sourceUrl);
            }
            set
            {
                _sourceUrl = value;
            }
        }
        /// <summary>
        /// 缩略图地址
        /// </summary>
        /// <value></value>
        public string PreviewImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_previewImageUrl);
            }
            set
            {
                _previewImageUrl = value;
            }
        }
        /// <summary>
        /// 海报图地址
        /// </summary>
        /// <value></value>
        public string CoverImageUrl
        {
            get
            {
                return DOShare.ChangeUrl(_coverImageUrl);
            }
            set
            {
                _coverImageUrl = value;
            }
        }
        /// <summary>
        /// 文件数据类型
        /// </summary>
        /// <value>0</value>
        public RemedicalFileDataTypeEnum FileDataType { get; set; }
        /// <summary>
        /// 创建时间
        /// </summary>
        /// <value></value>
        public DateTime CreateTime { get; set; }
        /// <summary>
        /// 创建人
        /// </summary>
        /// <value></value>
        public string CreatorCode { get; set; }
        /// <summary>
        /// 会诊图像操作类型枚举
        /// </summary>
        /// <value></value>
        public ConsultationFileTypeEnum ConsultationFileType { get; set; }
        /// <summary>
        /// 对应图像数据code
        /// </summary>
        /// <value></value>
        public string RemedicalCode { get; set; } = string.Empty;
        /// <summary>
        /// 对应测量数据code
        /// </summary>
        /// <value></value>
        public string RemedicalMeasureCode { get; set; } = string.Empty;
    }
}
