
using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 课程浏览观看记录列表
    /// </summary>
    public class CourseBrowsedRecordEntity : BaseEntity
    {
        /// <summary>
        /// 浏览记录code
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string CourseBrowsedRecordCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }
        
        /// <summary>
        /// 关联业务编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        public string RelevanceCode { get; set; }

        /// <summary>
        /// 用户编码
        /// </summary>
        public string UserCode { get; set; }

        /// <summary>
        /// 展示姓名
        /// </summary>
        public string DisplayName { get; set; }     

        /// <summary>
        /// 来源平台
        /// </summary>
        public string FromPlatform { get; set; }

        /// <summary>
        /// 浏览次数
        /// </summary>
        public int BrowsedCount { get; set; }

        /// <summary>
        /// 是否在线：true-在线；false-离线
        /// </summary>
        /// <value>false</value>
        public bool IsOnline { get; set; }

        /// <summary>
        /// 浏览用户类型枚举
        /// </summary>
        /// <value>0</value>
        public BrowsedUserTypeEnum BrowsedUserType { get; set; }       

        /// <summary>
        /// 浏览详情列表
        /// </summary>
        /// <value>0</value>
        public IList<BrowsedTimeRecordEntity> BrowsedTimeRecords { get; set; }       
    }

    /// <summary>
    /// 浏览详情
    /// </summary>
    /// <value>0</value>
    public class BrowsedTimeRecordEntity
    {        
        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime StartTime { get; set; } = DateTime.MinValue;

        /// <summary>
        /// 结束时间
        /// </summary>
        public DateTime EndTime { get; set; } = DateTime.MinValue;

        /// <summary>
        /// 时长 : 单位-分钟
        /// </summary>
        public int Duration { get; set; }
    }

    
}