using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.DTO.Education;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 课程信息表
    /// </summary>
    public class CourseInfoEntity : BaseEntity
    {
        private string _poster;
        private string _coursewareToken;
        private string  _courseToken;
        /// <summary>
        /// 课程名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 教师Code
        /// </summary>
        public string TeacherCode { get; set; }

        /// <summary>
        ///教师名称
        /// </summary>
        public string TeacherName { get; set; }

        /// <summary>
        /// 课程时长【单位秒】
        /// </summary>
        public int Duration { get; set; }

        /// <summary>
        /// 课程状态
        /// </summary>
        public CourseStatusEnum Status { get; set; }

        /// <summary>
        /// 可见范围：国内，国外，或者两者兼有
        /// </summary>
        public CourseViewRangeEnum ViewRange { get; set; }

        /// <summary>
        /// 课程价格
        /// </summary>
        public decimal Price { get; set; }

        /// <summary>
        /// 课程简介
        /// </summary>
        public string CourseIntro { get; set; }

        /// <summary>
        /// 面向类型
        /// </summary>
        public CourseAudienceTypeEnum AudienceType { get; set; }

        /// <summary>
        /// 封面图，课程海报
        /// </summary>
        public string Poster
         {
            get
            {
                return DOShare.ChangeUrl(_poster);
            }
            set
            {
                _poster = value;
            }
        }

        /// <summary>
        /// 创建人
        /// </summary>
        public string CreatorCode { get; set; }

        /// <summary>
        /// 创建机构code
        /// </summary>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 课程预计开始时间
        /// </summary>
        public DateTime StartTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 课程结束签到时间
        /// </summary>
        public DateTime Deadline { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 实际开始时间
        /// </summary>
        public DateTime ActualStartTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 实际结束时间
        /// </summary>
        public DateTime ActualEndTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 病例标签
        /// </summary>
        public IList<string> CaseLabelCodes { get; set; }

        // /// <summary>
        // /// 直播通道数据
        // /// </summary>
        // public IList<ChannelDataInfoEntity> ChannelDatas { get;private set; }

        /// <summary>
        /// 课程地址，邀请码
        /// </summary>
        public string CourseToken
        {
            get
            {
                return DOShare.ChangeUrl(_courseToken);
            }
            set
            {
                _courseToken = value;
            }
        }

        /// <summary>
        /// 课件地址
        /// </summary>
        public string CoursewareToken 
         {
            get
            {
                return DOShare.ChangeUrl(_coursewareToken);
            }
            set
            {
                _coursewareToken = value;
            }
        }

        /// <summary>
        /// 用户组（班级）
        /// </summary>
        public IList<string> UserGroupCodes { get; set; }

        /// <summary>
        /// 课程分类
        /// </summary>
        public CourseTypeEnum CourseType { get; set; }

        /// <summary>
        /// 是否仅为视频课
        /// </summary>
        public bool IsJustVideoCourse { get; set; } = false;

        /// <summary>
        /// 是否小班化
        /// </summary>
        /// <value></value>
        public bool IsSmallClass { get; set; }

        /// <summary>
        /// 助教列表
        /// </summary>
        public IList<StudentInfoEntity> Assistants { get; set; } = new List<StudentInfoEntity>();

        /// <summary>
        /// 参与互动专家
        /// </summary>
        public IList<StudentInfoEntity> Experts { get; set; } = new List<StudentInfoEntity>();

        /// <summary>
        /// 课程标签列表
        /// </summary>
        public IList<string> CourseLabelCodes { get; set; }

        /// <summary>
        /// 是否代理商课程
        /// </summary>
        public bool IsAgentCourse { get; set; }

        /// <summary>
        /// 是否置顶
        /// </summary>
        public bool IsStick { get; set; }

        /// <summary>
        /// 排序
        /// </summary>
        public int Sort { get; set; }

        /// <summary>
        /// 绑定试卷
        /// </summary>
        public IList<CourseExaminationEntity> BindExams { get; set; }

        /// <summary>
        /// 教培素材Code列表
        /// </summary>
        public IList<string> CourseVideoCodes { get; set; }

        /// <summary>
        /// 课程呈现类型
        /// </summary>
        /// <value></value>
        public CourseAppearTypeEnum CourseAppearType { get; set; }

        /// <summary>
        /// 所属专辑
        /// </summary>
        /// <value></value>
        public IList<string> CourseAlbumCodes { get; set; }

        /// <summary>
        /// 播放量
        /// </summary>
        /// <value>1</value>
        public int PlayCount { get; set; }

        /// <summary>
        /// 课程资料
        /// </summary>
        /// <value>1</value>
        public IList<string> CourseMaterialCodes { get; set; }


        public ShareInfoEntity ShareInfo { get; set; }

        /// <summary>
        /// 课程分享直播信息，RTMP
        /// </summary>
        /// <value></value>
        public LiveDataDO CourseChannel { get; set; }

        /// <summary>
        /// 其他互动人员共享屏幕直播信息，RTMP
        /// </summary>
        /// <value></value>
        public LiveDataDO OthersScreenSharingChannel { get; set; }

        /// <summary>
        /// 桌面共享RTC信息
        /// </summary>
        /// <value></value>
        public ScreenSharingChannelDO RtcScreenSharingChannel { get; set; }

        /// <summary>
        /// 数据来源：是否来自老平台
        /// </summary>
        /// <value></value>
        public bool IsOldPlatData { get; set; }

        /// <summary>
        /// 老平台数据分类：
        /// </summary>
        /// <value></value>
        public OldPlatCourseType OldCourseType { get; set; }

        /// <summary>
        /// 语言
        /// </summary>
        /// <value>zh-CN</value>
        public string LanguageCode { get; set; }
        
        /// <summary>
        /// 是否禁言：true-禁言；false-不禁言
        /// </summary>
        /// <value>false</value>
        public bool IsProhibition { get; set; }

        /// <summary>
        /// 统计信息
        /// </summary>
        /// <value>false</value>
        public CourseStatisticsInfoEntity StatisticsInfo { get; set; }
    }

    /// <summary>
    /// 课程统计信息
    /// </summary>
    public class CourseStatisticsInfoEntity
    {
        /// <summary>
        /// 报名人数
        /// </summary>
        /// <value>false</value>
        public int SignCount { get; set; }
        
        /// <summary>
        /// 观看数量
        /// </summary>
        /// <value>false</value>
        public int BrwosedCount { get; set; }
        
        /// <summary>
        /// 最多在线人数
        /// </summary>
        /// <value>false</value>
        public int MaxOnlineCount { get; set; }

        /// <summary>
        /// 统计详情信息
        /// </summary>
        /// <value>false</value>
        public List<CourseStatisticsOnlineDetailEntity> StatisticsDetailInfos { get; set; }
    }

    /// <summary>
    /// 在线课程在线统计详情信息
    /// </summary>
    public class CourseStatisticsOnlineDetailEntity 
    {
        /// <summary>
        /// 当前时间
        /// </summary>
        public DateTime CurrentTime { get; set; }

        /// <summary>
        /// 当前时间最多在线人数
        /// </summary>
        public int MaxOnlineCount { get; set; }
    }

    public class ShareInfoEntity
    {
        /// <summary>
        /// 当前课程共享状态
        /// </summary>
        /// <value>0</value>
        public CourseShareStates CurrentShareState { get; set; }

        /// <summary>
        /// 当前课程共享类型
        /// </summary>
        /// <value>0</value>
        public CourseShareType CurrentShareType { get; set; }

        /// <summary>
        /// 当前课程共享人或者机器编码
        /// </summary>
        /// <value>memberCode_1</value>
        public string CurrentShareMemberCode { get; set; }

        /// <summary>
        /// 当前课程共享人或者机器名称
        /// </summary>
        /// <value>member1</value>
        public string CurrentShareMemberName { get; set; }
    }

    /// <summary>
    /// 课程考试
    /// </summary>
    public class CourseExaminationQuestionEntity
    {
        /// <summary>
        /// 题目Code
        /// </summary>
        /// <value></value>
        public string Code { get; set; }

        /// <summary>
        /// 题干
        /// </summary>
        /// <value></value>
        public string Stem { get; set; }

        /// <summary>
        /// 题目类型
        /// </summary>
        /// <value></value>
        public QuestionTypeEnum QuestionType { get; set; }

        /// <summary>
        /// 题目选项
        /// </summary>
        /// <value></value>
        public IList<QuestionOptionDTO> QuestionOptionList { get; set; }

        /// <summary>
        /// 图像列表
        /// </summary>
        /// <value></value>
        public IList<QuestionFileDTO> FileList { get; set; }

        /// <summary>
        /// 判断题对错
        /// </summary>
        /// <value></value>
        public bool TrueOrFalse { get; set; } = false;

        /// <summary>
        /// 题目分数
        /// </summary>
        /// <value></value>
        public double Score { get; set; }
    }
    public class CourseExaminationEntity
    {
        /// <summary>
        /// 编码
        /// </summary>
        /// <value></value>
        public string Code { get; set; }

        /// <summary>
        /// 名称
        /// </summary>
        /// <value></value>
        public string Name { get; set; }

        /// <summary>
        /// 总分
        /// </summary>
        /// <value></value>
        public double TotalScore { get; set; }

        /// <summary>
        /// 题目列表
        /// </summary>
        /// <value></value>
        public List<CourseExaminationQuestionEntity> QuestionList { get; set; }

        /// <summary>
        /// 提交次数
        /// </summary>
        /// <value></value>
        public int SubmitLimitCount { get; set; }

        /// <summary>
        /// 是否已发布
        /// </summary>
        /// <value></value>
        public bool IsRelease { get; set; }

        /// <summary>
        /// 考试试卷类型
        /// </summary>
        /// <value></value>
        public CourseExaminationTypeEnum ExaminationType { get; set; }

        /// <summary>
        /// 考试开始时间
        /// </summary>
        /// <value></value>
        public DateTime StartTime { get; set; }

        /// <summary>
        /// 考试时长（分钟）
        /// </summary>
        /// <value></value>
        public int ExamDuration { get; set; }

        /// <summary>
        /// 及格分数
        /// </summary>
        /// <value></value>
        public double PassingScore { get; set; }


    }
    /// <summary>
    /// 学生信息实体
    /// </summary>
    public class StudentInfoEntity
    {
        /// <summary>
        /// 用户code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 是否支付：是-支付；否-未支付
        /// </summary>
        public bool IsPay { get; set; }

        /// <summary>
        /// 学生对于课程的状态
        /// </summary>
        ///  <value>0</value>
        public StudentCourseStatusEnum SignCourseStatus { get; set; }

    }

    public class ScreenSharingChannelDO
    {
        /// <summary>
        /// RTC房间号
        /// </summary>
        public int RtcRoomId { get; set; }

        /// <summary>
        /// SdkAppId
        /// </summary>
        public int SdkAppId { get; set; }

        /// <summary>
        /// 直播用户唯一码
        /// </summary>
        public string UserCode { get; set; }

        /// <summary>
        /// 用户签名
        /// </summary>
        public string UserSign { get; set; }

        /// <summary>
        /// 直播推拉流信息
        /// </summary>
        /// <value></value>
        public LiveDataDO LiveData { get; set; }
    }

    /// <summary>
    /// 老平台数据类型
    /// </summary>
    public enum OldPlatCourseType
    {
        LiveCourse = 0,
        PastVideo = 1
    }
}