using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 健康体检预约信息
    /// </summary>
    public class HealthExamBookingEntity : BaseEntity
    {
        /// <summary>
        /// 预约类型
        /// </summary>
        public BookingTypeEnum BookingType { get; set; }

        /// <summary>
        /// 体检名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 体检地点
        /// </summary>
        public string Location { get; set; }

        /// <summary>
        /// 体检开始日期
        /// </summary>
        /// <value></value>
        public DateTime StartDate { get; set; }

        /// <summary>
        /// 体检结束日期
        /// </summary>
        /// <value></value>
        public DateTime EndDate { get; set; }

        /// <summary>
        /// 体检对象
        /// </summary>
        /// <value></value>
        public string Subject { get; set; }

        /// <summary>
        /// 体检说明
        /// </summary>
        /// <value></value>
        public string Description { get; set; }

        /// <summary>
        /// 发起人编码
        /// </summary>
        /// <value></value>
        public string InitiatorCode { get; set; }

        /// <summary>
        /// 发起人机构编码
        /// </summary>
        /// <value></value>
        public string OrgCode { get; set; }

        /// <summary>
        /// 预约状态（待提交、已预约、已取消）
        /// </summary>
        /// <value></value>
        public BookingStatusEnum Status { get; set; }

        /// <summary>
        /// 体检人员集合
        /// </summary>
        /// <typeparam name="HealthExamPersonEntity"></typeparam>
        /// <returns></returns>
        public List<HealthExamPersonEntity> Persons { get; set; } = new List<HealthExamPersonEntity>();

        /// <summary>
        /// 检查项目
        /// </summary>
        /// <typeparam name="HealthExamPersonEntity"></typeparam>
        /// <returns></returns>
        public List<HealthExamItemEntity> ExamItems { get; set; } = new List<HealthExamItemEntity>();
    }

    public class HealthExamPersonEntity
    {
        /// <summary>
        /// 姓名
        /// </summary>
        /// <value></value>
        public string Name { get; set; }

        /// <summary>
        /// 性别
        /// </summary>
        /// <value></value>
        public PatientGenderEnum Gender { get; set; }

        /// <summary>
        /// 证件号
        /// </summary>
        /// <value></value>
        public string IdentityCard { get; set; }

        /// <summary>
        /// 手机号
        /// </summary>
        /// <value></value>
        public string Phone { get; set; }
    }

    /// <summary>
    /// 检查项目实体
    /// </summary>
    public class HealthExamItemEntity
    {
        /// <summary>
        /// 检查项目
        /// </summary>
        /// <value></value>
        public string Code { get; set; }

        /// <summary>
        /// 检查项目
        /// </summary>
        /// <value></value>
        public string Name { get; set; }

        /// <summary>
        /// 描述
        /// </summary>
        /// <value></value>
        public string Content { get; set; }
    }
}