using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.NotificationEnum;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 消息通知实体
    /// </summary>
    public class MessageInfoEntity : BaseEntity
    {
        /// <summary>
        /// 通知类型
        /// </summary>
        public NotificationTypeEnum NotificationType   { get; set; }

        /// <summary>
        /// 通知内容  json字符串
        /// </summary>
        public string Content { get; set; }

        /// <summary>
        /// 服务器地址【记录那台服务器触发的】
        /// </summary>
        public string ServerHost { get; set; }

        /// <summary>
        /// 通知时间
        /// </summary>
        public DateTime NotifyTime { get; set; }

        /// <summary>
        /// 接收方类型【客户端、设备端】
        /// </summary>
        public ApplicantTypeEnum ReceiverType { get; set; }

        /// <summary>
        /// 业务类型【会诊、聊天会话、公告】
        /// </summary>
        public TransactionTypeEnum TransactionType  { get; set; }

        /// <summary>
        /// 关联业务编码
        /// </summary>
        public string RelevanceCode  { get; set; }
        
        /// <summary>
        /// 关联客户信息列表
        /// </summary>
        public IList<ClientInfoEntity> ClientInfos  { get; set; }
    }

    /// <summary>
    /// 关联客户信息
    /// </summary>
    public class ClientInfoEntity
    {
        /// <summary>
        /// 客户id (User code , admin code or device code)
        /// </summary>
        public string ClientId { get; set; }

        /// <summary>
        /// 客户名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 是否已读：默认false-表示未读
        /// </summary>
        public bool IsReaded { get; set; }

        /// <summary>
        /// 送达时间
        /// </summary>
        public DateTime DeliveryTime { get; set; }

        /// <summary>
        /// 已读时间
        /// </summary>
        public DateTime ReadTime { get; set; }
    }
}