using System.Collections.Generic;
using WingInterfaceLibrary.Enum;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 对接项目配置
    /// </summary>
    public class ProjectInfoEntity : BaseEntity
    {
        /// <summary>
        /// 项目名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 推送类型，Default：标准通信方式(post+json)
        /// </summary>
        public ProjectPushTypeEnum PushType { get; set; }

        /// <summary>
        /// 是否中国大陆
        /// </summary>
        /// <value></value>
        public bool IsChinaMainland { get; set; }

        /// <summary>
        /// 联系人名称
        /// </summary>
        public string ContactName { get; set; }

        /// <summary>
        /// 联系方式
        /// </summary>
        public string ContactPhone { get; set; }

        /// <summary>
        /// 医院集合
        /// </summary>
        /// <typeparam name="ProjectOrganizationEntity"></typeparam>
        /// <returns></returns>
        public List<ProjectOrganizationEntity> OrganizationInfos { get; set; } = new List<ProjectOrganizationEntity>();

        /// <summary>
        /// 推送配置集合
        /// </summary>
        /// <typeparam name="ProjectPosterEntity"></typeparam>
        /// <returns></returns>
        public List<ProjectPosterEntity> PosterInfos { get; set; } = new List<ProjectPosterEntity>();
    }

    public class ProjectOrganizationEntity
    {
        /// <summary>
        /// 医院名称
        /// </summary>
        public string OrganizationName { get; set; }

        /// <summary>
        /// 医院编码
        /// </summary>
        public string OrganizationCode { get; set; }

        /// <summary>
        /// 第三方医院编码
        /// </summary>
        public string ThirdCode { get; set; }


        public override bool Equals(object obj)
        {
            if (obj == null || GetType() != obj.GetType())
                return false;

            ProjectOrganizationEntity other = (ProjectOrganizationEntity)obj;
            return OrganizationName == other.OrganizationName && OrganizationCode == other.OrganizationCode && ThirdCode == other.ThirdCode;
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int hash = 17;
                hash = hash * 23 + OrganizationName.GetHashCode();
                hash = hash * 23 + OrganizationCode.GetHashCode();
                hash = hash * 23 + ThirdCode.GetHashCode();
                return hash;
            }
        }
    }

    public class ProjectPosterEntity
    {
        /// <summary>
        /// 数据类型，医院、用户、设备等
        /// </summary>
        public ProjectDataTypeEnum DataType { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string ApiUrl { get; set; }

        /// <summary>
        /// 超时时间
        /// </summary>
        public int TimeoutSeconds { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string PushHeaders { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string PushTemplate { get; set; }


        public override bool Equals(object obj)
        {
            if (obj == null || GetType() != obj.GetType())
                return false;

            ProjectPosterEntity other = (ProjectPosterEntity)obj;
            return DataType == other.DataType && ApiUrl == other.ApiUrl && TimeoutSeconds == other.TimeoutSeconds;
        }

        public override int GetHashCode()
        {
            unchecked
            {
                int hash = 17;
                hash = hash * 23 + DataType.GetHashCode();
                hash = hash * 23 + ApiUrl.GetHashCode();
                hash = hash * 23 + TimeoutSeconds.GetHashCode();
                return hash;
            }
        }
    }
}