using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DO;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 项目数据推送记录
    /// </summary>
    public class ProjectPushRecordEntity : BaseEntity
    {
        public ProjectPushRecordEntity()
        {

        }

        public ProjectPushRecordEntity(ProjectInfoEntity projectInfo, ProjectPosterEntity posterInfo)
        {
            ProjectCode = projectInfo.Code;
            ProjectName = projectInfo.Name;
            PushType = projectInfo.PushType;
            IsChinaMainland = projectInfo.IsChinaMainland;
            DataType = posterInfo.DataType;
            ApiUrl = posterInfo.ApiUrl;
            TimeoutSeconds = posterInfo.TimeoutSeconds;
            CreateTime = DateTime.UtcNow;
            PushStatus = ProjectPushStatusEnum.NotSend;
            PushTemplate = posterInfo.PushTemplate;
        }

        /// <summary>
        /// 项目编码
        /// </summary>
        public string ProjectCode { get; set; }

        /// <summary>
        /// 项目名称
        /// </summary>
        [QueryItem]
        public string ProjectName { get; set; }

        /// <summary>
        /// 推送类型，Default：标准通信方式(post+json)
        /// </summary>
        public ProjectPushTypeEnum PushType { get; set; }

        /// <summary>
        /// 是否中国大陆
        /// </summary>
        /// <value></value>
        public bool IsChinaMainland { get; set; }

        /// <summary>
        /// 是否分布式处理
        /// </summary>
        /// <value></value>
        public bool IsDistributeHandle { get; set; }

        /// <summary>
        /// 数据类型，医院、用户、设备等
        /// </summary>
        public ProjectDataTypeEnum DataType { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string ApiUrl { get; set; }

        /// <summary>
        /// 超时时间
        /// </summary>
        public int TimeoutSeconds { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string PushHeaders { get; set; }

        /// <summary>
        /// 接口地址
        /// </summary>
        public string PushTemplate { get; set; }

        /// <summary>
        /// 数据名称
        /// </summary>
        [QueryItem]
        public string DataName { get; set; }

        /// <summary>
        /// 数据编码
        /// </summary>
        [QueryItem]
        public string DataCode { get; set; }

        /// <summary>
        /// 关联业务参数
        /// </summary>
        /// <value></value>
        public List<DataItem> RelationDatas { get; set; } = new List<DataItem>();

        /// <summary>
        /// 推送状态
        /// </summary>
        [QueryItem]
        public ProjectPushStatusEnum PushStatus { get; set; }

        /// <summary>
        /// 推送时间
        /// </summary>
        public DateTime PushTime { get; set; }

        /// <summary>
        /// 推送内容
        /// </summary>
        public string PushDatas { get; set; }

        /// <summary>
        /// 响应结果
        /// </summary>
        public string ResultContent { get; set; }
    }
}