﻿using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.Mongodb.Driver;
using WingInterfaceLibrary.LiveConsultation;
using WingInterfaceLibrary.LiveConsultation.Enums;
using WingMongoDBModule.Entities;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 检查关联的AI信息
    /// </summary>
    public class RemedicalAISelectedInfoEntity : BaseEntity
    {
        private string _orginalFileToken;
        private string _cdnFileToken;
        private string _aiFileToken;
        private string _aiCdnFileToken;
        private string _previewFileToken;
        /// <summary>
        /// 记录编码
        /// </summary>
        /// <value></value>
        [BsonIgnore]
        public string RemedicalAISelectedInfoCode
        {
            get
            {
                return Code;
            }
            set
            {
                Code = value;
            }
        }

        /// <summary>
        /// 病人编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        [QueryItem]
        public string PatientCode { get; set; } = string.Empty;

        /// <summary>
        /// 扫查记录编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        [QueryItem]
        public string RecordCode { get; set; } = string.Empty;

        /// <summary>
        /// 扫查图像编码
        /// </summary>
        /// <value>4914352a75f645279f6b4ad6764cc288</value>
        [QueryItem]
        public string RemedicalCode { get; set; } = string.Empty;

        /// <summary>
        /// 第几帧，默认0
        /// </summary>
        /// <value>0</value>
        public int FrameIndex { get; set; }

        /// <summary>
        /// 用户编码
        /// </summary>
        /// <value>UserDO_20220315125334n5j3A2</value>
        [QueryItem]
        public string UserCode { get; set; } = string.Empty;

        /// <summary>
        /// 原图-源站地址
        /// </summary>
        public string OrginalFileToken
        {
            get
            {
                return DOShare.ChangeUrl(_orginalFileToken);
            }
            set
            {
                _orginalFileToken = value;
            }
        }


        /// <summary>
        /// 原图-CDN地址
        /// </summary>
        /// <value></value>
        public string CdnFileToken 
           {
            get
            {
                return DOShare.ChangeUrl(_cdnFileToken);
            }
            set
            {
                _cdnFileToken = value;
            }
        }

        /// <summary>
        /// AI图-源站地址
        /// </summary>
        public string AIFileToken 
          {
            get
            {
                return DOShare.ChangeUrl(_aiFileToken);
            }
            set
            {
                _aiFileToken = value;
            }
        }
        /// <summary>
        /// AI图-CDN地址
        /// </summary>
        /// <value></value>
        public string AICdnFileToken 
        {
            get
            {
                return DOShare.ChangeUrl( _aiCdnFileToken);
            }
            set
            {
                 _aiCdnFileToken = value;
            }
        }

        /// <summary>
        /// 缩略图地址
        /// </summary>
        public string PreviewFileToken
        {
            get
            {
                return DOShare.ChangeUrl( _previewFileToken);
            }
            set
            {
                 _previewFileToken = value;
            }
        }

        /// <summary>
        /// AI诊断详情，不抽帧
        /// </summary>
        /// <value></value>
        public string DiagnosisData { get; set; } = string.Empty;

        /// <summary>
        /// AI诊断结论
        /// </summary>
        /// <value></value>
        public DiagnosisConclusionEnum DiagnosisConclusion { get; set; }

        /// <summary>
        /// AI识别的器官
        /// </summary>
        /// <value></value>
        public List<DiagnosisOrganEnum> DiagnosisOrgans { get; set; } = new List<DiagnosisOrganEnum>();
    }
}
