using System;
using System.Collections.Generic;
using System.Linq;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 学生考试答题情况
    /// </summary>
    public class StudentExaminationEntity : BaseEntity
    {
        /// <summary>
        /// 考试编码
        /// </summary>
        /// <value></value>
        public string ExaminationCode { get; set; }

        /// <summary>
        /// 答题人编码
        /// </summary>
        /// <value></value>
        public string StudentCode { get; set; }

        /// <summary>
        /// 答题人名称
        /// </summary>
        /// <value></value>
        public string StudentName { get; set; }

        /// <summary>
        ///总分数
        /// </summary>
        /// <value>100</value>
        public double TotalScore { get; set; }

        /// <summary>
        /// 关联业务类型
        /// </summary>
        /// <value></value>
        public RelevanceTypeEnum RelevanceType { get; set; }

        /// <summary>
        /// 关联业务编码
        /// </summary>
        /// <value></value>
        public string RelevanceCode { get; set; }

        /// <summary>
        /// 学生提交答案列表
        /// </summary>
        /// <value></value>
        public IList<ExaminationAnswerEntity> StudentAnswers { get; set; }

        /// <summary>
        /// 提交次数
        /// </summary>
        /// <value></value>
        public int SubmitCount { get; set; }
    }

    public class ExaminationAnswerEntity
    {
        /// <summary>
        /// 考试题目编码
        /// </summary>
        /// <value></value>
        public string QuestionCode { get; set; }

        /// <summary>
        /// 题目类型
        /// </summary>
        /// <value></value>
        public QuestionTypeEnum QuestionType { get; set; }

        /// <summary>
        /// 答案
        /// </summary>
        /// <value></value>
        public string Answer { get; set; }

        private List<string> _files = new List<string>();
        /// <summary>
        /// 答案文件地址
        /// </summary>
        /// <value></value>
        public IList<string> Files
        {
            get
            {
                var urls = new List<string>();
                if (_files != null && _files.Any())
                {
                    foreach (var url in _files)
                    {
                        urls.Add(DOShare.ChangeUrl(url));
                    }
                }
                return urls;
            }
            set
            {
                _files = value?.ToList() ?? new List<string>();
            }
        }

        /// <summary>
        /// 已选选项code
        /// </summary>
        /// <value></value>
        public IList<string> OptionCodeList { get; set; }

        /// <summary>
        /// 是否答题正确
        /// </summary>
        /// <value></value>
        public bool IsCorrect { get; set; } = false;

        /// <summary>
        /// 答题得分
        /// </summary>
        /// <value></value>
        public double Score { get; set; }

    }
}