using System;
using System.Collections.Generic;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 用户绑定设备扩展信息实体
    /// </summary>
    public class UserDeviceExtendInfoEntity : BaseEntity
    {
        /// <summary>
        /// 获取或设置 用户编号
        /// </summary>
        /// <value></value>
        public string UserCode { get; set; }

        /// <summary>
        /// 设备编码
        /// </summary>
        /// <value></value>
        public string DeviceCode { get; set; }

        /// <summary>
        /// 用户设备名称
        /// </summary>
        /// <value></value>
        public string UserDeviceName { get; set; }

        /// <summary>
        /// 无线网名称
        /// </summary>
        /// <value></value>
        public string WifiName { get; set; }

        /// <summary>
        /// 是否自动获取
        /// </summary>
        /// <value></value>
        public bool IsAuto { get; set; }

        /// <summary>
        /// 设备所在城市
        /// </summary>
        /// <value></value>
        public string AreaCode { get; set; }

        /// <summary>
        /// 机器Mac地址
        /// </summary>
        /// <value></value>
        public string MachineAddress { get; set; }

        /// <summary>
        /// 报告模式
        /// </summary>
        public int ModeType { get; set; }

        /// <summary>
        /// 设备类型
        /// </summary>
        public string DeviceType { get; set; }
        
        /// <summary>
        /// 设备SN号
        /// </summary>
        public string DeviceSN { get; set; }

        /// <summary>
        /// 设备mac地址
        /// </summary>
        public string DeviceMAC { get; set; }

        /// <summary>
        /// 数据文件列表
        /// </summary>
        public IList<IOTDeviceDataFolderEntity> FolderList { get; set; }
    }

    
    /// <summary>
    /// 设备数据文件夹记录实体
    /// </summary>
    public class IOTDeviceDataFolderEntity
    {
        /// <summary>
        /// The unique database table id
        /// </summary>
        /// <returns></returns>
        public string Id { get; set; } = Guid.NewGuid().ToString("N");

        /// <summary>
        /// 文件夹名称
        /// </summary>
        /// <returns></returns>
        public string FolderName { get; set; }

        /// <summary>
        /// 数据文件列表
        /// </summary>
        public IList<IOTDeviceFileDataRecordEntity> DataFileList { get; set; }

        /// <summary>
        /// Create time
        /// </summary>
        public DateTime CreateTime { get; set; } = DateTime.UtcNow;
    }

    /// <summary>
    /// 设备数据文件详情实体
    /// </summary>
    public class IOTDeviceFileDataRecordEntity
    {
        /// <summary>
        /// The unique database table id
        /// </summary>
        /// <returns></returns>
        public string Id { get; set; } = Guid.NewGuid().ToString("N");

        /// <summary>
        /// Create time
        /// </summary>
        public DateTime CreateTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 治疗时间
        /// </summary>
        public DateTime TreatTime { get; set; } = DateTime.UtcNow;

        /// <summary>
        /// 数据类型分类
        /// </summary>
        public int DataFileType { get; set; }

        /// <summary>
        /// 数据类型描述
        /// </summary>
        public string DataDesc { get; set; }

        /// <summary>
        /// 原始文件名，带后缀
        /// </summary>
        public string OriginalFileName { get; set; }

        /// <summary>
        /// 文件地址
        /// </summary>
        public string FileToken { get; set; }

        /// <summary>
        /// 是否完成
        /// </summary>
        public bool IsCompleted { get; set; }
    }
}