using System;
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 用户组
    /// </summary>
    public class UserGroupEntity : BaseEntity
    {
        /// <summary>
        /// 用户组名称
        /// </summary>
        [UniqueItem]
        public string Name { get; set; }

        /// <summary>
        /// 短码班级
        /// </summary>
        [UniqueItem]
        public string ShortCode { get; set; }

        /// <summary>
        /// 用户组描述
        /// </summary>
        public string Description { get; set; }

        /// <summary>
        /// 最大人数
        /// </summary>
        public int MaxPeople { get; set; }

        /// <summary>
        /// 学生列表
        /// </summary>
        public IList<LearnerEntity> Students { get; set; }

        /// <summary>
        /// 创建人Code
        /// </summary>
        public string CreatorCode { get; set; }

        /// <summary>
        /// 是否代理商，后台可以维护和创建
        /// </summary>
        public bool IsAgent { get; set; } = false;

        /// <summary>
        /// 是否老平台数据
        /// </summary>
        /// <value></value>
        public bool IsOldPlatformData { get; set; }

        /// <summary>
        /// 用户组类型，默认：班级-0
        /// </summary>
        /// <value></value>
        public UserGroupEnum UserGroupType { get; set; }

        /// <summary>
        /// 用户列表
        /// </summary>
        public IList<string> UserCodes { get; set; }

        /// <summary>
        /// 设备列表
        /// </summary>
        public IList<string> DeviceCodes { get; set; }
    }

    /// <summary>
    /// 班级学员实体
    /// </summary>
    public class LearnerEntity
    {
        /// <summary>
        /// 用户code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 学员状态枚举
        /// </summary>
        public LearnerStatusEnum LearnerStatus { get; set; }
    }
}