
using System.Collections.Generic;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Entities
{
    /// <summary>
    /// 素材信息表
    /// </summary>
    public class VideoInfoEntity : BaseEntity
    {
        /// <summary>
        /// 素材名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 素材上传人Code
        /// </summary>
        public string CreatorCode { get; set; }

        /// <summary>
        /// 素材上传人
        /// </summary>
        public string CreatorName { get; set; }

        /// <summary>
        /// 素材分类：视频、图片、文件
        /// </summary>
        public UploadFileTypeEnum FodderType { get; set; } 

        /// <summary>
        /// 素材使用：公开的、私有的
        /// </summary>
        public bool IsPublic { get; set; } = true;

        /// <summary>
        /// 可见范围：国内，国外，或者两者兼有 todo
        /// </summary>
        public CourseViewRangeEnum ViewRange { get; set; }

        private string _videoToken;
        /// <summary>
        /// 视频地址
        /// </summary>
        public string VideoToken
        {
            get
            {
                return DOShare.ChangeUrl(_videoToken);
            }
            set
            {
                _videoToken = value;
            }
        }

        /// <summary>
        /// Vod云点播文件Id
        /// </summary>
        public string VodFileId { get; set; }

        private string _poster;
        /// <summary>
        /// 封面图
        /// </summary>
        public string Poster
        {
            get
            {
                return DOShare.ChangeUrl(_poster);
            }
            set
            {
                _poster = value;
            }
        }

        /// <summary>
        /// 视频时长【秒】
        /// </summary>
        public int Duration { get; set; }

        /// <summary>
        /// 视频大小【MB】
        /// </summary>
        public double VideoSize { get; set; }

        /// <summary>
        /// 简介
        /// </summary>
        /// <value></value>
        public string Intro { get; set; }

        /// <summary>
        /// 是否自动创建
        /// </summary>
        /// <value></value>
        public bool AutoCreateCourse { get; set; }

        /// <summary>
        /// 用户组
        /// </summary>
        /// <value></value>
        public IList<string> UserGroupCodes { get; set; }

        /// <summary>
        /// 课程标签
        /// </summary>
        /// <value></value>
        public IList<string> CourseLabelCodes { get; set; }

        /// <summary>
        /// 课程规模
        /// </summary>
        /// <value></value>
        public CourseAudienceTypeEnum AudienceType { get; set; }

        /// <summary>
        /// 播放量
        /// </summary>
        /// <value>1</value>
        public int PlayCount { get; set; }

        private string _srcFileToken;
        /// <summary>
        /// 原始文件地址
        /// </summary>
        /// <value></value>
        public string SrcFileToken
        {
            get
            {
                return DOShare.ChangeUrl(_srcFileToken);
            }
            set
            {
                _srcFileToken = value;
            }
        }

        /// <summary>
        /// 是否管理员创建
        /// </summary>
        /// <value>1</value>
        public bool IsAdminCreated { get; set; }
    }
}