
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Log;

namespace WingMongoDBModule.Repositories
{
    public class AdminDBRepository : BaseDBRepository<AdminInfoDO>
    {
        public AdminDBRepository():base("Admins"){}
        /// <summary>
        /// > Find all admins where the full name or admin name contains the given name
        /// </summary>
        /// <param name="name">The name to search for</param>
        /// <returns>
        /// A list of AdminInfoDO objects.
        /// </returns>
        public async Task<List<AdminInfoDO>> FindAdminsByNameAsync(string name)
        {
            if (!string.IsNullOrWhiteSpace(name))
            {
                MongoCustomerFilter<MongoTable> filter = new();
                MongoCustomerFilter<MongoDataItem> subfilter = new();

                subfilter.AppendEq((item => item.Key, "FullName"));
                subfilter.AppendRegex(item => item.Value, name, true);
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                subfilter.ClearFilter();

                subfilter.AppendEq((item => item.Key, "AdminName"));
                subfilter.AppendRegex(item => item.Value, name, true);
                filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                subfilter.ClearFilter();

                return await FindAllAsync(filter.OrCombine());
            }
            return new List<AdminInfoDO>();
        }
    }
}
