using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;

namespace WingMongoDBModule.Repositories
{
    public class PositionDBRepository : BaseDBRepository<UserPositionDO>
    {
        public PositionDBRepository():base("UserPositions"){}

        /// <summary>
        /// > Update the position name and department code of the position with the specified code
        /// </summary>
        /// <param name="UpdatePositionNameDBRequest">The request parameter class, which is used to pass the
        /// parameters of the request to the method.</param>
        /// <returns>
        /// The return value is a boolean value.
        /// </returns>
        public async Task<bool> UpdatePositionName(UpdatePositionNameDBRequest request)
        {
            request.PositionName = request.PositionName.Trim();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> subFilter = new();
            //查找当前岗位身份
            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.PositionCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            var currentOrg = await FindOneAsync(customerFilter.AndCombine());
            //验证是否有重复名称
            subFilter.AppendEq((item => item.Key, "PositionName"));
            subFilter.AppendEq((item => item.Value, request.PositionName));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();
            var isFound = await FindOneAsync(customerFilter.AndCombine());
            if(isFound!=null && isFound.OrganizationCode == currentOrg.OrganizationCode && isFound.Code != request.PositionCode)
            {
                throw new RpcException((int)CustomerRpcCode.PositionNameDuplicate,"PositionName Duplicate","PositionName Duplicate");
            }

            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.PositionCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();

            MongoCustomerUpdate<MongoTable> update = new();
            subFilter.AppendEq((item => item.Key, nameof(request.PositionName)));
            subFilter.AppendEq((item => item.Key, nameof(request.FitDepartmentCode)));
            update.SetField("BaseData.PositionName", request.PositionName);
            update.SetField("BaseData.FitDepartmentCode", request.FitDepartmentCode);
            update.RemoveSubItemFilter(item => item.QueryData.QueryItems, subFilter.OrCombine());
            await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);

            update.ClearUpdate();
            update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
            {
                Key = nameof(request.PositionName),
                Value = request.PositionName
            });
            update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
            {
                Key = nameof(request.FitDepartmentCode),
                Value = request.FitDepartmentCode
            });
            return await UpdateOneAsync(customerFilter.AndCombine(), update.AllUpdateDefinition) > 0;
        }
    }
}