using System;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingServerCommon.Log;

namespace WingMongoDBModule.Repositories
{
    public class RankDBRepository : BaseDBRepository<UserRankDO>
    {
        public RankDBRepository():base("UserRanks"){}
        /// <summary>
        /// > Update the rank name of the specified rank code
        /// </summary>
        /// <param name="UpdateRankNameDBRequest">The request parameter class, which is used to pass the
        /// parameters of the request to the method.</param>
        /// <returns>
        /// A boolean value.
        /// </returns>
        public async Task<bool> UpdateRankName(UpdateRankNameDBRequest request)
        {
            request.RankName = request.RankName.Trim();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> subFilter = new();

            //查找当前职称
            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.RankCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            var currentOrg = await FindOneAsync(customerFilter.AndCombine());

            //验证是否有重复名称
            subFilter.AppendEq((item => item.Key, "RankName"));
            subFilter.AppendEq((item => item.Value, request.RankName));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();
            var isFound = await FindOneAsync(customerFilter.AndCombine());
            if(isFound!=null && currentOrg.OrganizationCode == isFound.OrganizationCode && isFound.Code != request.RankCode)
            {
                throw new RpcException((int)CustomerRpcCode.RankNameDuplicate,"RankName Duplicate","RankName Duplicate");
            }

            subFilter.AppendEq((item => item.Key, "Code"));
            subFilter.AppendEq((item => item.Value, request.RankCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            subFilter.ClearFilter();

            MongoCustomerUpdate<MongoTable> update = new();
            subFilter.AppendEq((item => item.Key, nameof(request.RankName)));
            update.SetField("BaseData.RankName", request.RankName);
            update.RemoveSubItemFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
            await UpdateAllAsync(customerFilter.AndCombine(), update.AllUpdateDefinition);

            update.ClearUpdate();
            update.AddSubItem(item => item.QueryData.QueryItems, new MongoDataItem
            {
                Key = nameof(request.RankName),
                Value = request.RankName
            });
            return await UpdateAllAsync(customerFilter.AndCombine(), update.AllUpdateDefinition) > 0;
        }
    }
}