﻿using System.Collections.Generic;
using System.Threading.Tasks;
using WingMongoDBModule.DO;

namespace WingMongoDBModule.Repositories
{
    public class StorageDBRepository : BaseDBRepository<StorageInfoDO>
    {
        public StorageDBRepository():base("StorageInfos"){}
        /// <summary>
        /// It deletes a record from the database based on the token.
        /// </summary>
        /// <param name="token">The token that was generated by the server.</param>
        /// <returns>
        /// The return type is a boolean.
        /// </returns>

        public async Task<bool> DeleteByTokenAsync(string token)
        {
            var storageInfo = await FindOneAsync("Token", token);
            return await base.DeleteOneAsync("Code", storageInfo.Code.ToString());
        }


        /// <summary>
        /// > Finds a single record in the database where the FileName column matches the name parameter
        /// </summary>
        /// <param name="name">The name of the file</param>
        /// <returns>
        /// A StorageInfoDO object.
        /// </returns>
        public async Task<StorageInfoDO> FindOneByFileNameAsync(string name)
        {
            return await base.FindOneAsync("FileName", name);
        }


        /// <summary>
        /// > Find all the StorageInfoDO objects where the FileName property is equal to the name parameter
        /// </summary>
        /// <param name="name">the name of the file</param>
        /// <returns>
        /// A list of StorageInfoDO objects.
        /// </returns>
        public async Task<List<StorageInfoDO>> FindAllByFileNameAsync(string name)
        {
            var dic = new Dictionary<string, string>();
            dic.Add("FileName", name);
            return await base.FindListAsync(dic);
        }


        /// <summary>
        /// > Finds the storage route for the given token
        /// </summary>
        /// <param name="token">The token that was generated by the client.</param>
        /// <returns>
        /// A StorageInfoDO object.
        /// </returns>
        public async Task<StorageInfoDO> FindStorageRouteAsync(string token)
        {
            var storageInfo = await base.FindOneAsync("Token", token);
            return storageInfo;
        }
    }
}
