using System.Threading.Tasks;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.DTO.AdminAuthorityGroup;
using WingInterfaceLibrary.Request;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminAuthorityGroupsDBService
    {
        /// <summary>
        /// 根据编码获取前端权限组
        /// </summary>
        /// <param name="code">权限组编码</param>
        /// <returns>权限组信息</returns>
        public async Task<AdminAuthorityGroupDTO> FindAdminAuthorityGroupByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var adminAuthorityGroupDO = await _adminAuthorityGroupDBRepository.FindOneAsync("Code", code);
                return adminAuthorityGroupDO.MappingTo<AdminAuthorityGroupDTO>();
            }
        }

        /// <summary>
        /// 根据名称查找前端权限组
        /// </summary>
        /// <param name="name">权限组名称</param>
        /// <returns>权限组信息</returns>
        public Task<AdminAuthorityGroupDTO> FindAdminAuthorityGroupByNameAsync(string name)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// 新增前端权限组
        /// </summary>
        /// <param name="adminAuthorityGroupDO">权限组信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns>Id</returns>
        public async Task<string> InsertAdminAuthorityGroupAsync(CreateAdminAuthorityGroupsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertAdminAuthorityGroupAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminAuthorityGroupDBRepository.InsertOneAsync(request.Data.MappingTo<AdminAuthorityGroupDO>(), request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 更新前端权限组
        /// </summary>
        /// <param name="code">组编码</param>
        /// <param name="data">权限组信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateAdminAuthorityGroupByCodeAsync(string code, AdminAuthorityGroupDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAdminAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminAuthorityGroupDBRepository.UpdateOneAsync("Code", code, data.MappingTo<AdminAuthorityGroupDO>(), extensionData);
            }
        }

        /// <summary>
        /// 删除前端权限组
        /// </summary>
        /// <param name="id">Id</param>
        /// <returns>是否成功</returns>
        public async Task<bool> DeleteAdminAuthorityGroupByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteAdminAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _adminAuthorityGroupDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 查找前端权限组（分页）
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns>权限组列表</returns>
        public async Task<PageCollection<AdminAuthorityGroupDTO>> FindAdminAuthorityGroupPagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindAdminAuthorityGroupPagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _adminAuthorityGroupDBRepository.GetPages(dbRequest);
                return pageInfo.MappingTo<PageCollection<AdminAuthorityGroupDTO>>();
            }
        }

    }
}