

using WingServerCommon.Service;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Message;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Utilities;
using System.Collections.Generic;
using WingMongoDBModule.Entities;
using MongoDB.Driver;
using WingServerCommon.Mapper;
using System.Linq;
using System;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IChatMessageDBService
    {
        public async Task<PageResult<ChatMessagesDTO>> GetChatMessagePagesDBAsync(GetChatMessagePagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetChatMessagePagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var relevanceCode = request.RelevanceCode;
                var filter = Builders<ChatMessageEntity>.Filter.Where(c => c.RelevanceCode == relevanceCode);
                PageResult<ChatMessagesDTO> result = new PageResult<ChatMessagesDTO>();
                result.PageIndex = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.TotalCount = (int)await _chatMessageDBRepository.CountDocumentsAsync(filter);
                SortDefinitionBuilder<ChatMessageEntity> builderSort = Builders<ChatMessageEntity>.Sort;
                SortDefinition<ChatMessageEntity> sort = builderSort.Descending(x => x.CreateTime);
                var dataList = await _chatMessageDBRepository.FindAllAsync(filter,
                 new FindOptions<ChatMessageEntity>
                 {
                     NoCursorTimeout = true,
                     Skip = (request.CurrentPage - 1) * request.PageSize,
                     Limit = request.PageSize,
                     Sort = sort
                 });
                result.PageData = dataList.MappingTo<List<ChatMessagesDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 添加讨论/评论记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddChatMessageDBAsync(ChatMessagesDTO request)
        {
            if (string.IsNullOrWhiteSpace(request.Code))
            {
                var entity = request.MappingTo<ChatMessageEntity>();
                entity.CreateTime = request.CreateTime;
                entity.UpdateTime = DateTime.UtcNow;
                var insertCode = await _chatMessageDBRepository.InsertOneAsync(entity);
                return insertCode;
            }
            return string.Empty;
        }
    }
}