using System.Collections.Immutable;
using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DTO.Management;
using WingServerCommon.Service;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Mapper;
using System.Collections.Generic;
using System.Linq;
using System;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.Patient;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.LiveConsultation;
using MongoDB.Bson;
using WingMongoDBModule.Entities;
using WingInterfaceLibrary.DTO.Common;
using WingServerCommon.Log;
using System.Text.RegularExpressions;
using WingInterfaceLibrary.DTO.DataCenter;
using System;
using WingInterfaceLibrary.LiveConsultation.Enums;
using WingInterfaceLibrary.LiveConsultation.Consultation;
using WingInterfaceLibrary.Request.DBCopy;
using WingInterfaceLibrary.DTO.Organization;
using WingMongoDBModule.Common;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using MongoDB.Driver.GeoJsonObjectModel;
using System.Text;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IConsultationRecordDBService
    {

        /// <summary>
        /// 新增会诊记录
        /// </summary>
        /// <param name="CreateAdminDBRequest">客户端发送给服务器的请求对象。</param>
        /// <returns>
        public async Task<string> InsertConsultationRecordAsync(CreateConsultationRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertConsultationRecordAsync execute", $"{this.GetType().Name}"))
            {
                var consultationRecordDO = request.Data.MappingTo<ConsultationRecordEntity>();
                var result = await _consultationRecordDBRepository.InsertOneAsync(consultationRecordDO);
                return result;
            }
        }

        public async Task<ConsultationDetailDTO> FindConsultationRecordByCodeAsync(ConsultationRecordCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindPatientByCodeAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.Code))
                {
                    return null;
                }
                var consultationRecordDO = await _consultationRecordDBRepository.FindOneAsync("Code", request.Code);
                if (consultationRecordDO == null)
                {
                    return null;
                }
                var result = consultationRecordDO.MappingTo<ConsultationDetailDTO>();
                PatientInfoDTO patientInfoDTO = null;
                if (!string.IsNullOrWhiteSpace(consultationRecordDO.PatientCode))
                {
                    var patientInfo = await FindPatientByCodeAsync(consultationRecordDO.PatientCode);
                    patientInfoDTO = patientInfo.MappingTo<PatientInfoDTO>();
                }
                var organizationCodes = new List<string> { result.ApplyOrganizationCode, result.ExpertOrganizationCode };
                var organizations = await FindOrganizationDOListByCodesAsync(organizationCodes);
                var userCodes = new List<string> { result.ApplyUserCode, result.ExpertUserCode, result.OperateUserCode, result.ScanUserCode };
                var users = await FindUserListByUserCodesAsync(userCodes);
                result.ApplyOrganizationName = organizations.FirstOrDefault(o => o.OrganizationCode == result.ApplyOrganizationCode)?.OrganizationName;
                var applyUser = users.FirstOrDefault(u => u.UserCode == result.ApplyUserCode);
                var expertUser = users.FirstOrDefault(u => u.UserCode == result.ExpertUserCode);
                var assistantUser = users.FirstOrDefault(u => u.UserCode == expertUser?.BindAssistantUserCode);
                var assistantDoctorUser = users.FirstOrDefault(u => u.UserCode == expertUser?.BindAssistantDoctorUserCode);
                var operateUser = users.FirstOrDefault(u => u.UserCode == result.OperateUserCode);
                var scanUser = users.FirstOrDefault(u => u.UserCode == result.ScanUserCode);
                result.ApplyUserName = string.IsNullOrWhiteSpace(applyUser?.FullName) ? applyUser?.UserName : applyUser?.FullName;
                result.ExpertUserName = string.IsNullOrWhiteSpace(expertUser?.FullName) ? expertUser?.UserName : expertUser?.FullName;
                result.AssistantUserCode = assistantUser?.UserCode;
                result.AssistantUserName = string.IsNullOrWhiteSpace(assistantUser?.FullName) ? assistantUser?.UserName : assistantUser?.FullName;
                result.ExpertOrganizationName = organizations.FirstOrDefault(o => o.OrganizationCode == result.ExpertOrganizationCode)?.OrganizationName;
                result.OperateUserName = string.IsNullOrWhiteSpace(operateUser?.FullName) ? operateUser?.UserName : operateUser?.FullName;
                result.ScanUserName = string.IsNullOrWhiteSpace(scanUser?.FullName) ? scanUser?.UserName : scanUser?.FullName;
                result.AssistantDoctorUserCode = assistantDoctorUser?.UserCode;
                result.AssistantDoctorUserName = string.IsNullOrWhiteSpace(assistantDoctorUser?.FullName) ? assistantDoctorUser?.UserName : assistantDoctorUser?.FullName; ;
                if (!string.IsNullOrWhiteSpace(result.DeviceCode))
                {
                    var deviceInfo = await FindDeviceInfoByCodeAsync(result.DeviceCode);
                    result.DeviceName = deviceInfo?.Name;
                    result.DisplayName = deviceInfo?.DisplayName;
                }
                if (patientInfoDTO != null)
                {
                    result.PatientDatas = patientInfoDTO?.PatientData;
                    result.PatientName = patientInfoDTO?.Name;
                    result.Sex = patientInfoDTO.PatientData.FirstOrDefault(x => !string.IsNullOrWhiteSpace(x.Key) && x.Key.Contains("Sex"))?.Value;
                }
                return result;
            }
        }

        public async Task<IList<ConsultationDetailDTO>> FindConsultationRecordByCodesAsync(ConsultationRecordCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindConsultationRecordByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var codes = request.Codes;
                var filter = Builders<ConsultationRecordEntity>.Filter.Where(c => codes.Contains(c.Code));
                var entities = await _consultationRecordDBRepository.FindAllAsync(filter);
                var result = entities.MappingTo<List<ConsultationDetailDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 查询会诊预约记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ConsultationDetailDTO>> FindConsultationRecordPagesAsync(GetConsultationRecordPagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindConsultationRecordPagesAsync execute", $"{this.GetType().Name}"))
            {
                var allFilter = await GetConsultationRecordsFilter(request);
                PageResult<ConsultationDetailDTO> result = new PageResult<ConsultationDetailDTO>();
                result.PageIndex = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.TotalCount = (int)await _consultationRecordDBRepository.CountDocumentsAsync(allFilter);
                List<ConsultationRecordEntity> consultationRecordList = null;
                if (request.ConsultationStatus == QueryConsultationStatusEnum.All)
                {
                    var consultationStatusOrder = new Dictionary<TransactionStatusEnum, int>
                    {
                        { TransactionStatusEnum.Applied, 1 },
                        { TransactionStatusEnum.ToStart, 2 },
                        { TransactionStatusEnum.InProgress, 3 },
                        { TransactionStatusEnum.PendingReport, 4 },
                        { TransactionStatusEnum.Expired, 5 },
                        { TransactionStatusEnum.Rejected, 6 },
                        { TransactionStatusEnum.Withdrawn, 7 },
                        { TransactionStatusEnum.End, 8 }
                    };

                    consultationRecordList = await _consultationRecordDBRepository.FindAllAsync(allFilter);
                    consultationRecordList = consultationRecordList
                    .OrderBy(x =>
                    {
                        if (consultationStatusOrder.ContainsKey(x.ConsultationStatus))
                        {
                            return consultationStatusOrder[x.ConsultationStatus];
                        }
                        else
                        {
                            return 9;
                        }
                    })
                    .ThenByDescending(x => x.CreateTime)
                    .Skip((request.CurrentPage - 1) * request.PageSize).Take(request.PageSize)
                    .ToList();
                }
                else
                {
                    SortDefinitionBuilder<ConsultationRecordEntity> builderSort = Builders<ConsultationRecordEntity>.Sort;
                    SortDefinition<ConsultationRecordEntity> sort = builderSort.Descending(x => x.CreateTime);//默认按创建时间倒序 
                    if (request.ConsultationStatus == QueryConsultationStatusEnum.Applied
                    || request.ConsultationStatus == QueryConsultationStatusEnum.ToStart)
                    {
                        sort = builderSort.Ascending(x => x.ConsultationTime);//按会诊时间正序
                    }
                    consultationRecordList = await _consultationRecordDBRepository.FindAllAsync(allFilter,
                    new FindOptions<ConsultationRecordEntity>
                    {
                        NoCursorTimeout = true,
                        Skip = (request.CurrentPage - 1) * request.PageSize,
                        Limit = request.PageSize,
                        Sort = sort
                    });
                }
                result.PageData = consultationRecordList.MappingTo<List<ConsultationDetailDTO>>();
                if (result.PageData.Count > 0)
                {
                    var patientCodes = result.PageData.Select(x => x.PatientCode).ToList();
                    var patients = await FindPatientInfoListByCodeListAsync(patientCodes);
                    foreach (var item in result.PageData)
                    {
                        var patient = patients.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                        item.PatientDatas = patient?.PatientData;
                    }
                }
                return result;
            }
        }


        /// <summary>
        /// 查询会诊预约记录过滤条件
        /// </summary>
        private async Task<FilterDefinition<ConsultationRecordEntity>> GetConsultationRecordsFilter(GetConsultationRecordPagesDBRequest request)
        {
            var runStep = new StringBuilder("Start invoke------>GetConsultationRecordsFilter");
            try
            {
                var userInfo = await FindUserByCodeAsync(request.UserCode);
                if (userInfo == null)
                {
                    ThrowCustomerException(CustomerRpcCode.UserNotFind, "UserNotFind");
                }
                //有质控权限
                var orgCodeList = new List<string>();
                if (userInfo.UserFeatureCodes.Contains("QualityControl"))
                {
                    orgCodeList = new List<string> { userInfo.RootOrganizationCode };
                    //下级医院
                    var orgList = await GetAllChildOrganizations(userInfo.RootOrganizationCode);
                    if (orgList != null && orgList.Count > 0)
                    {
                        orgCodeList.AddRange(orgList.Select(x => x.OrganizationCode));
                    }
                    //上级医院
                    var parents = await GetParentEntities(new List<string> { userInfo.RootOrganizationCode });
                    if (parents != null && parents.Count > 0)
                    {
                        orgCodeList.AddRange(parents.Select(x => x.OrganizationCode));
                    }
                }
                var relationUsers = new List<string> { userInfo.UserCode };
                FilterDefinition<ConsultationRecordEntity> keywordFilter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code != "");//关键字
                FilterDefinition<ConsultationRecordEntity> routineFilter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code != "");//常规条件
                FilterDefinition<ConsultationRecordEntity> typeFilter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code != "");//查询类型条件
                //关键字查询
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    //keyword
                    var keywordFilterDefinitions = new List<FilterDefinition<ConsultationRecordEntity>>();
                    var keyword = request.Keyword.Trim();
                    //病人名称、手机号、年龄、性别、会诊专家、申请机构、既往病史、疾病名称、初步诊断
                    var keywordDictionary = new Dictionary<string, string>();
                    var tempKeyWord = CryptionHelper.EncryptOneByOne(keyword);
                    keywordDictionary.Add("Name", tempKeyWord);
                    keywordDictionary.Add("Phone", tempKeyWord);
                    keywordDictionary.Add("Age", keyword);
                    keywordDictionary.Add("Gender", keyword);
                    keywordDictionary.Add("Anamnesis", keyword);
                    var pageFilterRequest = new GetPatientsByOrgPageRequest()
                    {
                        KeywordOrFilter = keywordDictionary,
                        IsFuzzy = true
                    };
                    runStep.Append("------>GetPatientsDBAsync");
                    var patientInfoList = await GetPatientsDBAsync(pageFilterRequest);
                    if (patientInfoList != null && patientInfoList.Count > 0)
                    {
                        var patientCodes = patientInfoList.Select(p => p.PatientCode).ToList();
                        keywordFilterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.PatientCode, patientCodes));
                    }
                    //申请机构
                    runStep.Append("------>SearchOrganizationsAsync");
                    var orgInfoList = await SearchOrganizationsAsync(keyword, "", "", OrganizationTypeEnum.Corporation);
                    if (orgInfoList != null && orgInfoList.Count > 0)
                    {
                        var orgCodes = orgInfoList.Select(p => p.OrganizationCode).ToList();
                        keywordFilterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ApplyOrganizationCode, orgCodes));
                    }
                    //会诊专家
                    runStep.Append("------>FindUsersByName");
                    var userList = await FindUsersByName(keyword);
                    if (userList != null && userList.Count > 0)
                    {
                        var userCodes = userList.Select(p => p.Code).ToList();
                        keywordFilterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, userCodes));
                    }
                    //疾病名称
                    keywordFilterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Diseases.Contains(keyword)));
                    //初步诊断
                    keywordFilterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.PrimaryDiagnosis.Contains(keyword)));
                    //汇总关键字or条件
                    if (keywordFilterDefinitions.Count == 0)
                    {
                        keywordFilter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1");
                    }
                    else
                    {
                        keywordFilter = Builders<ConsultationRecordEntity>.Filter.Or(keywordFilterDefinitions);
                    }
                    if (keyword == "未填写" || keyword.ToLower() == "Not Filled In".ToLower())
                    {
                        var otherFilter = Builders<ConsultationRecordEntity>.Filter.Where(c => c.PatientCode == null);
                        keywordFilter = keywordFilter | otherFilter;
                    }
                }
                //常规查询
                var filterDefinitions = new List<FilterDefinition<ConsultationRecordEntity>>();
                filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.IsDelete == false));
                //进行中之前的状态不显示
                filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => !(x.IsEmergency == true && x.ConsultationStatus < TransactionStatusEnum.InProgress)));
                //时间范围
                if (request.StartDate > DateTime.MinValue)
                {
                    request.StartDate = Convert.ToDateTime(request.StartDate.ToString("yyyy-MM-dd"));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ConsultationTime >= request.StartDate));
                }
                if (request.EndDate > DateTime.MinValue)
                {
                    request.EndDate = Convert.ToDateTime(request.EndDate.ToString("yyyy-MM-dd"));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ConsultationTime < request.EndDate.AddDays(1)));
                }
                //状态
                if (request.ConsultationStatus != QueryConsultationStatusEnum.All)
                {
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ConsultationStatus == Enum.Parse<TransactionStatusEnum>(request.ConsultationStatus.ToString())));
                }
                //评价
                if (request.EvaluateGrade != QueryEvaluateGradeEnum.All)
                {
                    //未设置
                    if (request.EvaluateGrade == QueryEvaluateGradeEnum.UnSet)
                    {
                        runStep.Append("------>GetAllConsultationEvaluatesAsync");
                        var consultationEvaluates = await GetAllConsultationEvaluatesAsync();
                        if (consultationEvaluates.Count > 0)
                        {
                            var codes = consultationEvaluates.Select(x => x.ConsultationRecordCode);
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Nin(x => x.Code, codes));
                        }
                    }
                    else
                    {
                        runStep.Append("------>GetConsultationEvaluatesAsync");
                        var consultationEvaluates = await GetConsultationEvaluatesAsync((EvaluateGradeEnum)(int)request.EvaluateGrade - 1);
                        if (consultationEvaluates.Count > 0)
                        {
                            var codes = consultationEvaluates.Select(x => x.ConsultationRecordCode);
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.Code, codes));
                        }
                        else
                        {
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1"));
                        }
                    }
                }
                //高级查询 会诊专家、申请机构、会诊机构、性别、年龄、疾病、初步诊断
                //找到把我设置为助理医师的医生
                runStep.Append("------>GetBindExpertsByAssistantDoctorAsync");
                var bindExpertsByAssistantDoctor = await GetBindExpertsByAssistantDoctorAsync(request.UserCode);
                if (bindExpertsByAssistantDoctor != null && bindExpertsByAssistantDoctor.Count > 0)
                {
                    relationUsers.AddRange(bindExpertsByAssistantDoctor.Select(s => s.UserCode));
                }
                var expertCodes = request.ExpertCodes;//查询专家
                if (expertCodes != null && expertCodes.Count > 0)
                {
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, expertCodes));
                }
                var applyOrganizationCodes = request.ApplyOrganizationCodes;//查询申请机构
                if (applyOrganizationCodes != null && applyOrganizationCodes.Count > 0)
                {
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ApplyOrganizationCode, applyOrganizationCodes));
                }
                var expertOrganizationCodes = request.ExpertOrganizationCodes;//查询会诊机构
                if (expertOrganizationCodes != null && expertOrganizationCodes.Count > 0)
                {
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertOrganizationCode, expertOrganizationCodes));
                }
                var patientDiseases = request.PatientDiseases;//查询病人疾病
                if (!string.IsNullOrEmpty(patientDiseases))
                {
                    if (patientDiseases == "未填写" || patientDiseases.ToLower() == "Not Filled In".ToLower())
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => string.IsNullOrEmpty(x.Diseases)));
                    }
                    else
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Diseases.Contains(patientDiseases)));
                    }
                }
                var patientPrimaryDiagnosis = request.PatientPrimaryDiagnosis;//查询初步诊断
                if (!string.IsNullOrEmpty(patientPrimaryDiagnosis))
                {
                    if (patientPrimaryDiagnosis == "未填写" || patientPrimaryDiagnosis.ToLower() == "Not Filled In".ToLower())
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => string.IsNullOrEmpty(x.PrimaryDiagnosis)));
                    }
                    else
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.PrimaryDiagnosis.Contains(patientPrimaryDiagnosis)));
                    }
                }
                // //高级查询：病人性别和年龄
                // var patientAgeLimit = request.PatientAgeLimit;
                // var patientSex = request.PatientSex;
                // var isCustomPatientFilter = (patientAgeLimit != null && (patientAgeLimit.MinAge != 0 || patientAgeLimit.MaxAge != 0)) || !string.IsNullOrWhiteSpace(patientSex);
                // if (isCustomPatientFilter)
                // {
                //     var orgPageFilterRequest = new GetPatientsByOrgPageRequest()
                //     {
                //         CurrentPage = 1,
                //         PageSize = 999999,
                //         IsFuzzy = true
                //     };
                //     var orgPatientInfoList = await GetPatientsDBAsync(orgPageFilterRequest);
                //     if (orgPatientInfoList != null && orgPatientInfoList.Count > 0)
                //     {
                //         if (patientAgeLimit != null && (patientAgeLimit.MinAge != 0 || patientAgeLimit.MaxAge != 0))
                //         {
                //             orgPatientInfoList = orgPatientInfoList.Where(x => IsCheckInAge(x.PatientData, patientAgeLimit.MinAge, patientAgeLimit.MaxAge, patientAgeLimit.Unit))?.ToList();
                //         }
                //         if (!string.IsNullOrWhiteSpace(patientSex))
                //         {
                //             orgPatientInfoList = orgPatientInfoList.Where(x => x.Gender == Convert.ToInt32(GetGender(patientSex))).ToList();
                //         }
                //         if (orgPatientInfoList.Count > 0)
                //         {
                //             var patientCodes = orgPatientInfoList.Select(x => x.PatientCode);
                //             filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.PatientCode, patientCodes));
                //         }
                //         else
                //         {
                //             filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1"));
                //         }
                //     }
                //     else
                //     {
                //         filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1"));
                //     }
                // }
                //汇总常规and条件
                routineFilter = Builders<ConsultationRecordEntity>.Filter.And(filterDefinitions);
                //类型查询
                filterDefinitions.Clear();
                var filterTypeDefinitions = new List<FilterDefinition<ConsultationRecordEntity>>();
                if (request.ConsultationQueryType == ConsultationQueryTypeEnum.All)
                {
                    //质控员查询的会诊记录为所在机构上下级的已报告的会诊单
                    if (orgCodeList.Count > 0)
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ApplyOrganizationCode, orgCodeList));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertOrganizationCode, orgCodeList));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.In(c => c.MemberOrganizationCode, orgCodeList)));
                    }
                    else if (userInfo.IsDirector)
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyOrganizationCode == userInfo.RootOrganizationCode));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertOrganizationCode == userInfo.RootOrganizationCode));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberOrganizationCode, userInfo.RootOrganizationCode)));
                    }
                    else if (userInfo.PositionCodes.Contains("PositionCode01"))//主管
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyDepartmentCode == userInfo.OrganizationCode));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertDepartmentCode == userInfo.OrganizationCode));
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberDepartmentCode, userInfo.OrganizationCode)));
                    }
                    else if (userInfo.RoleCodes.Contains("Role_ExpertAssistant"))//专家助理
                    {
                        runStep.Append("------>GetBindExpertsAsync1");
                        var bindExperts = await GetBindExpertsAsync(userInfo.UserCode);
                        if (bindExperts?.Count > 0)
                        {
                            //有绑定，看到指定的专家
                            var bindExportCode = bindExperts.Select(c => c.UserCode).Distinct().ToList();
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ApplyUserCode, bindExportCode));
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, bindExportCode));
                        }
                        //未指定会诊专家的单子 所有该会诊机构的专家助理都能看到
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertOrganizationCode == userInfo.RootOrganizationCode && (x.ExpertUserCode == null || x.ExpertUserCode == "")));
                    }
                    //自己的
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberCode, userInfo.UserCode)));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyUserCode == userInfo.UserCode));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, relationUsers));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.OperateUserCode == userInfo.UserCode));
                }
                else if (request.ConsultationQueryType == ConsultationQueryTypeEnum.MyApply)
                {
                    if (userInfo.IsDirector)
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyOrganizationCode == userInfo.RootOrganizationCode));
                    }
                    else if (userInfo.PositionCodes.Contains("PositionCode01"))//主管
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyDepartmentCode == userInfo.OrganizationCode));
                    }
                    else if (userInfo.RoleCodes.Contains("Role_ExpertAssistant"))//专家助理
                    {
                        runStep.Append("------>GetBindExpertsAsync2");
                        var bindExperts = await GetBindExpertsAsync(userInfo.UserCode);
                        if (bindExperts?.Count > 0)
                        {
                            //有绑定，只能看到指定的专家
                            var bindExportCode = bindExperts.Select(c => c.UserCode).Distinct().ToList();
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ApplyUserCode, bindExportCode));
                        }
                        else
                        {
                            //未绑定专家
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1"));
                        }
                    }
                    //自己的
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyUserCode == userInfo.UserCode));
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.OperateUserCode == userInfo.UserCode));
                }
                else if (request.ConsultationQueryType == ConsultationQueryTypeEnum.MyArrange)
                {
                    if (userInfo.IsDirector)
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertOrganizationCode == userInfo.RootOrganizationCode));
                    }
                    else if (userInfo.PositionCodes.Contains("PositionCode01"))//主管
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertDepartmentCode == userInfo.OrganizationCode));
                    }
                    else if (userInfo.RoleCodes.Contains("Role_ExpertAssistant"))//专家助理
                    {
                        runStep.Append("------>GetBindExpertsAsync3");
                        var bindExperts = await GetBindExpertsAsync(userInfo.UserCode);
                        if (bindExperts?.Count > 0)
                        {
                            //有绑定,可以看到绑定专家的单子
                            var bindExportCode = bindExperts.Select(c => c.UserCode).Distinct().ToList();
                            filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, bindExportCode));
                        }
                        //未指定会诊专家的单子所有专家助理都能看到
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertOrganizationCode == userInfo.RootOrganizationCode && (x.ExpertUserCode == null || x.ExpertUserCode == "")));
                    }
                    //自己的
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.In(x => x.ExpertUserCode, relationUsers));
                }
                else//我参与的
                {
                    if (userInfo.IsDirector)
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberOrganizationCode, userInfo.RootOrganizationCode)));
                    }
                    else if (userInfo.PositionCodes.Contains("PositionCode01"))//主管
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberDepartmentCode, userInfo.OrganizationCode)));
                    }
                    else if (userInfo.RoleCodes.Contains("Role_ExpertAssistant"))//专家助理
                    {
                        filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.Code == "-1"));
                    }
                    //自己的
                    filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberCode, userInfo.UserCode)));
                }
                typeFilter = Builders<ConsultationRecordEntity>.Filter.Or(filterDefinitions);
                FilterDefinition<ConsultationRecordEntity> customerFilter = Builders<ConsultationRecordEntity>.Filter.And(keywordFilter, routineFilter, typeFilter);//最终条件
                runStep.Append("------>Return filter");
                return customerFilter;
            }
            catch(Exception ex)
            {
                Logger.WriteLineError($"StepLink:{runStep},error:{ex}");
                ThrowCustomerException(CustomerRpcCode.Unknow, $"error:{ex}");
                return FilterDefinition<ConsultationRecordEntity>.Empty;
            }
        }

        private bool IsCheckInAge(List<DataItemDTO> patientData, int minAge, int maxAge, QueryAgeUnitsEnum queryAgeEnum)
        {
            try
            {
                var ageYear = GetPatientAgeValue(patientData, "AgeYear");
                var ageMonth = GetPatientAgeValue(patientData, "AgeMonth");
                var ageWeek = GetPatientAgeValue(patientData, "AgeWeek");
                var ageDay = GetPatientAgeValue(patientData, "AgeDay");
                if (ageYear == 0 && ageMonth == 0 && ageWeek == 0 && ageDay == 0)
                {
                    return false;
                }
                var checkAge = 0;
                if (queryAgeEnum == QueryAgeUnitsEnum.Year)
                {
                    if (ageYear == maxAge && ageMonth > 0 && maxAge > 0)
                    {
                        return false;
                    }
                    checkAge = ageYear;
                }
                else if (queryAgeEnum == QueryAgeUnitsEnum.Month)
                {
                    if (ageYear > 0)
                    {
                        return false;
                    }
                    if (ageMonth == maxAge && ageWeek > 0 && maxAge > 0)
                    {
                        return false;
                    }
                    checkAge = ageMonth;
                }
                else//周
                {
                    if (ageYear > 0 || ageMonth > 0)
                    {
                        return false;
                    }
                    if (ageWeek == maxAge && ageDay > 0 && maxAge > 0)
                    {
                        return false;
                    }
                    checkAge = ageWeek;
                }
                return (checkAge >= minAge || minAge == 0) && (checkAge <= maxAge || maxAge == 0);
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"IConsultationRecordDBService IsCheckInAge Err:{ex}");
                return false;
            }
        }
        private int GetPatientAgeValue(List<DataItemDTO> source, string key)
        {
            try
            {
                var dataItem = source.FirstOrDefault(x => x.Key == key);
                if (dataItem == null)
                {
                    return 0;
                }
                if (string.IsNullOrWhiteSpace(dataItem.Value))
                {
                    return 0;
                }
                if (Regex.IsMatch(dataItem.Value, @"^\d+$"))// 判断字符串是否为数字 的正则表达式
                {
                    return Convert.ToInt32(dataItem.Value);
                }
                else
                {
                    return 0;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"IConsultationRecordDBService GetPatientAgeValue Err:{ex}");
                return 0;
            }
        }


        /// <summary>
        /// 会诊大列表导出
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<ExportConsultationDataResult> ExportConsultationsDBAsync(ExportConsultationRecordsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} ExportConsultationsDBAsync execute", $"{this.GetType().Name}"))
            {
                string language = request.Language;
                var result = new ExportConsultationDataResult();
                var dbRequest = new GetConsultationRecordPagesDBRequest();
                dbRequest.UserCode = request.UserCode;
                dbRequest.Keyword = request.Keyword;
                dbRequest.StartDate = request.StartDate;
                dbRequest.EndDate = request.EndDate;
                dbRequest.ConsultationStatus = request.ConsultationStatus;
                dbRequest.ConsultationQueryType = request.ConsultationQueryType;
                dbRequest.EvaluateGrade = request.EvaluateGrade;
                dbRequest.ExpertCodes = request.ExpertCodes;
                dbRequest.ApplyOrganizationCodes = request.ApplyOrganizationCodes;
                dbRequest.ExpertOrganizationCodes = request.ExpertOrganizationCodes;
                dbRequest.PatientSex = request.PatientSex;
                dbRequest.PatientAgeLimit = request.PatientAgeLimit;
                dbRequest.PatientDiseases = request.PatientDiseases;
                dbRequest.PatientPrimaryDiagnosis = request.PatientPrimaryDiagnosis;
                var allFilter = await GetConsultationRecordsFilter(dbRequest);
                var consultationRecordList = await _consultationRecordDBRepository.FindAllAsync(allFilter);
                if (consultationRecordList == null || consultationRecordList.Count == 0)
                {
                    return result;
                }
                var patientCodes = consultationRecordList.Select(x => x.PatientCode).ToList();
                var patients = await FindPatientInfoListByCodeListAsync(patientCodes);
                var consultationRecordCodes = consultationRecordList.Select(x => x.Code).ToList();
                //数据解密
                patients = patients.Where(p => !string.IsNullOrWhiteSpace(p.Name)).ToList();
                foreach (var item in patients)
                {
                    SetEncryptedShow(item.PatientType, item.PatientData);
                    item.Name = CryptionHelper.NameEncrypt(item.Name, false);
                }
                //无PatientCode的会诊单（急诊）
                if (consultationRecordList.Any(x => string.IsNullOrWhiteSpace(x.PatientCode)))
                {
                    patients.Add(new PatientInfoDTO { PatientCode = "", Name = GetNotFilled(language), PatientData = new List<DataItemDTO>() });
                }
                if (request.IsExportConsultation)//导出会诊
                {
                    foreach (var item in patients)
                    {
                        var consultationExportData = new ConsultationExportData();
                        consultationExportData.PatientCode = item.PatientCode;
                        consultationExportData.PatientName = item.Name;
                        consultationExportData.ConsultationItemList = new List<ConsultationItem>();
                        var entitys = consultationRecordList.Where(r => r.PatientCode == item.PatientCode).ToList();
                        if (item.PatientCode == "")
                        {
                            entitys = consultationRecordList.Where(r => string.IsNullOrWhiteSpace(r.PatientCode)).ToList();
                        }
                        if (entitys != null && entitys.Count > 0)
                        {

                            var organizationCodes = entitys.Select(x => x.ApplyOrganizationCode).ToList();
                            organizationCodes.AddRange(entitys.Select(x => x.ExpertOrganizationCode));
                            var organizations = await FindOrganizationDOListByCodesAsync(organizationCodes);
                            var userCodes = entitys.Select(x => x.ApplyUserCode).ToList();
                            userCodes.AddRange(entitys.Select(x => x.ExpertUserCode));
                            var users = await FindUserListByUserCodesAsync(userCodes);
                            foreach (var consultationRecordEntity in entitys)
                            {
                                var applyUser = users.FirstOrDefault(u => u.UserCode == consultationRecordEntity.ApplyUserCode);
                                var expertUser = users.FirstOrDefault(u => u.UserCode == consultationRecordEntity.ExpertUserCode);
                                var detail = new ConsultationItem();
                                detail.PatientName = string.IsNullOrWhiteSpace(item.Name) ? GetNotFilled(language) : item.Name;
                                detail.Age = GetPatientAge(item.PatientData, language);
                                detail.Sex = GetPatientGender(item.PatientData, language);
                                detail.ApplyOrganizationName = organizations.FirstOrDefault(o => o.OrganizationCode == consultationRecordEntity.ApplyOrganizationCode)?.OrganizationName;
                                detail.ApplyUserName = string.IsNullOrWhiteSpace(applyUser?.FullName) ? applyUser?.UserName : applyUser?.FullName;
                                detail.ExpertUserName = string.IsNullOrWhiteSpace(expertUser?.FullName) ? expertUser?.UserName : expertUser?.FullName;
                                detail.ExpertOrganizationName = organizations.FirstOrDefault(o => o.OrganizationCode == consultationRecordEntity.ExpertOrganizationCode)?.OrganizationName;
                                detail.ConsultationTime = consultationRecordEntity.ConsultationTime;
                                detail.ConsultationTimeEnd = consultationRecordEntity.ConsultationTimeEnd;
                                detail.Diseases = consultationRecordEntity.Diseases;
                                detail.PrimaryDiagnosis = consultationRecordEntity.PrimaryDiagnosis;
                                detail.ConsultationStatus = consultationRecordEntity.ConsultationStatus;
                                detail.ScanPositions = consultationRecordEntity.ScanPositions;
                                consultationExportData.ConsultationItemList.Add(detail);
                            }
                        }
                        result.ConsultationList.Add(consultationExportData);
                    }
                }
                if (request.IsExportReport)//导出报告
                {
                    var reports = await GetReportsByRecordCodesAsync(consultationRecordCodes);
                    foreach (var item in patients)
                    {
                        var reportData = new ConsultationReportData();
                        reportData.PatientCode = item.PatientCode;
                        reportData.PatientName = item.Name;
                        var itemReportList = reports.Where(r => r.PatientCode == item.PatientCode);
                        reportData.ReportItemList = itemReportList.Select(x => new ConsultationReportItem { ConsultationCode = x.RecordCode, SubmissionTime = x.ReportTime, FileToken = x.ReportPreviewList.FirstOrDefault(x => x.FileType == UploadFileTypeEnum.PDF)?.FileToken }).ToList();
                        if (reportData.ReportItemList.Any())
                        {
                            result.ReportList.Add(reportData);
                        }
                    }
                }
                if (request.IsExportConsultationFileData)//导出图像
                {
                    foreach (var item in patients)
                    {

                        var consultationRecords = consultationRecordList.Where(x => x.PatientCode == item.PatientCode);
                        var consultationFileData = new ConsultationFileData();
                        consultationFileData.PatientCode = item.PatientCode;
                        consultationFileData.PatientName = item.Name;
                        consultationFileData.ConsultationFileItemList = new List<ConsultationFileItem>();
                        foreach (var consultationRecord in consultationRecords)
                        {
                            var fileList = consultationRecord.ConsultationFileList;
                            if (fileList != null && fileList.Count > 0)
                            {
                                fileList = fileList.Where(x => !string.IsNullOrWhiteSpace(x.SourceUrl)).ToList();
                                if (fileList != null && fileList.Count > 0)
                                {
                                    consultationFileData.ConsultationFileItemList.AddRange(fileList.Select(x => new ConsultationFileItem { ConsultationCode = consultationRecord.Code, SubmissionTime = x.CreateTime, FileToken = x.SourceUrl, FileDataType = x.FileDataType, ConsultationFileType = x.ConsultationFileType }).ToList());
                                }
                            }
                        }
                        if (consultationFileData.ConsultationFileItemList.Any())
                        {
                            result.ConsultationFileDatalList.Add(consultationFileData);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 获取未填写
        /// </summary>
        /// <param name="patientInfo"></param>
        /// <returns></returns>
        private string GetNotFilled(string language)
        {
            var res = LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientNotFilled").Value ?? string.Empty;
            return res;
        }

        /// <summary>
        /// 获取年龄
        /// </summary>
        /// <param name="patientInfo"></param>
        /// <returns></returns>
        private string GetPatientAge(List<DataItemDTO> patientData, string language)
        {
            var ageStr = "";
            var ageYear = GetPatientValue(patientData, "AgeYear");
            var ageMonth = GetPatientValue(patientData, "AgeMonth");
            var ageWeek = GetPatientValue(patientData, "AgeWeek");
            var ageDay = GetPatientValue(patientData, "AgeDay");
            var ageUnits = GetPatientValue(patientData, "AgeUnits");
            if (!string.IsNullOrEmpty(ageYear))
            {
                ageStr += ageYear + LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientAgeYear").Value;
            }
            if (!string.IsNullOrEmpty(ageMonth))
            {
                ageStr += ageYear + LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientAgeMonth").Value;
            }
            if (!string.IsNullOrEmpty(ageWeek))
            {
                ageStr += ageYear + LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientAgeWeek").Value;
            }
            if (!string.IsNullOrEmpty(ageDay))
            {
                ageStr += ageYear + LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientAgeDay").Value;
            }
            if (string.IsNullOrEmpty(ageStr))//兼容老数据
            {
                var oldAge = GetPatientValue(patientData, "Age");
                ageStr += oldAge;
            }
            return ageStr;
        }

        private string GetPatientGender(List<DataItemDTO> patientData, string language)
        {
            var genderStr = string.Empty;
            var gender = GetPatientValue(patientData, "Sex");
            if (gender == PatientGenderEnum.Male.ToString())
            {
                genderStr = LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientGenderMale").Value;
            }
            else if (gender == PatientGenderEnum.Female.ToString())
            {
                genderStr = LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientGenderFemale").Value;
            }
            else
            {
                genderStr = LanguageSetting.GetParammeter<StringParameter>(language, "Language", "PatientNotFilled").Value;
            }
            return genderStr;
        }

        private string GetPatientValue(List<DataItemDTO> source, string key)
        {
            var dataItem = source.FirstOrDefault(x => x.Key == key);
            return dataItem?.Value ?? string.Empty;
        }



        public async Task<bool> DeleteConsultationRecordByCodeAsync(ConsultationRecordCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteConsultationRecordByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _consultationRecordDBRepository.DeleteOneAsync("Code", request.Code);
                return result;
            }
        }

        public async Task<bool> UpdateConsultationRecordAsync(ConsultationDetailDTO data, bool canNull = false)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateConsultationRecordAsync execute", $"{this.GetType().Name}"))
            {
                var oldDO = await _consultationRecordDBRepository.FindOneAsync("Code", data.ConsultationCode);

                var consultationRecordDO = data.MappingTo<ConsultationRecordEntity>();
                consultationRecordDO.UpdateTime = DateTime.UtcNow;
                var result = await _consultationRecordDBRepository.UpdateOneAsync("Code", consultationRecordDO.Code, consultationRecordDO, canNull);
                if (oldDO.ConsultationStatus != TransactionStatusEnum.End && consultationRecordDO.ConsultationStatus == TransactionStatusEnum.End)
                {
                    await AddPushConsultationAsync(consultationRecordDO);
                }
                return result;
            }
        }

        public async Task<bool> AddFollowUpVisitInfoAsync(AddFollowUpVisitInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddFollowUpVisitInfoDBRequest execute", $"{this.GetType().Name}"))
            {
                var followUpVisit = new FollowUpVisitEntity();
                followUpVisit.ClinicalSituation = request.ClinicalSituation;
                followUpVisit.ConsultationRecordCode = request.ConsultationRecordCode;
                followUpVisit.CreateTime = DateTime.UtcNow;
                followUpVisit.DoctorCode = request.DoctorCode;
                followUpVisit.GeneralCase = request.GeneralCase;
                followUpVisit.OccurredTime = request.OccurredTime;
                followUpVisit.PatientName = request.PatientName;
                followUpVisit.PatientPhone = request.PatientPhone;
                var result = await _followUpVisitDBRepository.InsertOneAsync(followUpVisit);
                return !string.IsNullOrWhiteSpace(result);
            }
        }

        public async Task<bool> DeleteFollowUpVisitInfoAsync(DeleteFollowUpVisitInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteFollowUpVisitInfoAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.FollowUpVisitCode))
                {
                    var result = await _followUpVisitDBRepository.DeleteOneAsync("Code", request.FollowUpVisitCode);
                    return result;
                }
                return false;
            }
        }

        public async Task<bool> UpdateFollowUpVisitInfoAsync(UpdateFollowUpVisitInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateFollowUpVisitInfoAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.FollowUpVisitCode))
                {
                    var update = Builders<FollowUpVisitEntity>.Update
                    .Set(c => c.ClinicalSituation, request.ClinicalSituation)
                    .Set(c => c.GeneralCase, request.GeneralCase)
                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                    var filter = Builders<FollowUpVisitEntity>.Filter.Where(c =>
                    c.Code == request.FollowUpVisitCode && c.IsDelete == false);
                    return await _followUpVisitDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return false;
            }
        }

        public async Task<List<FollowUpVisitDTO>> GetFollowUpVisitInfoAsync(GetFollowUpVisitInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetFollowUpVisitInfoAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<FollowUpVisitDTO>();
                if (!string.IsNullOrWhiteSpace(request.ConsultationRecordCode))
                {
                    var filter = Builders<FollowUpVisitEntity>.Filter.Where(x => x.ConsultationRecordCode == request.ConsultationRecordCode);
                    var dbResult = await _followUpVisitDBRepository.FindAllAsync(filter);
                    var doctorCodes = dbResult.Select(f => f.DoctorCode);
                    MongoCustomerFilter<MongoTable> doctorFilter = new MongoCustomerFilter<MongoTable>();
                    doctorFilter.AppendIn("BaseData.Code", doctorCodes.ToList());
                    var doctors = await _userDBRepository.FindAllAsync(doctorFilter.AndCombine());
                    foreach (var entity in dbResult)
                    {
                        var doctorName = "";
                        var doctor = doctors.FirstOrDefault(x => x.Code == entity.DoctorCode);
                        if (doctor != null && !string.IsNullOrWhiteSpace(doctor.Code))
                        {
                            doctorName = string.IsNullOrWhiteSpace(doctor.FullName) ? doctor.UserName : doctor.FullName;
                        }
                        var detail = new FollowUpVisitDTO
                        {
                            ClinicalSituation = entity.ClinicalSituation,
                            ConsultationRecordCode = entity.ConsultationRecordCode,
                            FollowUpVisitCode = entity.Code,
                            OccurredTime = entity.OccurredTime,
                            DoctorCode = entity.DoctorCode,
                            DoctorName = doctorName,
                            GeneralCase = entity.GeneralCase,
                            PatientName = entity.PatientName,
                            PatientPhone = entity.PatientPhone
                        };
                        result.Add(detail);
                    }
                }
                return result;
            }
        }

        public async Task<FollowUpVisitDTO> GetFollowUpVisitInfoDetailAsync(GetFollowUpVisitInfoDetailDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetFollowUpVisitInfoDetailAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.FollowUpVisitCode))
                {
                    var filter = Builders<FollowUpVisitEntity>.Filter.Where(x => x.Code == request.FollowUpVisitCode);
                    var result = await _followUpVisitDBRepository.FindOneAsync(filter);
                    MongoCustomerFilter<MongoTable> doctorFilter = new MongoCustomerFilter<MongoTable>();
                    doctorFilter.AppendEq(("Code", result.DoctorCode));
                    var doctor = await _userDBRepository.FindOneAsync(doctorFilter.AndCombine());
                    var doctorName = "";
                    if (doctor != null && !string.IsNullOrWhiteSpace(doctor.Code))
                    {
                        doctorName = string.IsNullOrWhiteSpace(doctor.FullName) ? doctor.UserName : doctor.FullName;
                    }
                    return new FollowUpVisitDTO
                    {
                        ClinicalSituation = result.ClinicalSituation,
                        ConsultationRecordCode = result.ConsultationRecordCode,
                        FollowUpVisitCode = result.Code,
                        OccurredTime = result.OccurredTime,
                        DoctorCode = result.DoctorCode,
                        DoctorName = doctorName,
                        GeneralCase = result.GeneralCase,
                        PatientName = result.PatientName,
                        PatientPhone = result.PatientPhone
                    };
                }
                return null;
            }
        }

        public async Task<bool> SetNeedFollowUpVisitAsync(SetNeedFollowUpVisitDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SetNeedFollowUpVisitAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.ConsultationRecordCode))
                {
                    var update = Builders<ConsultationRecordEntity>.Update
                    .Set(c => c.NeedFollowUpVisit, request.IsNeed);
                    var filter = Builders<ConsultationRecordEntity>.Filter.Where(c =>
                    c.Code == request.ConsultationRecordCode && c.IsDelete == false);
                    return await _consultationRecordDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return false;
            }
        }

        public async Task<bool> AddConsultationEvaluateAsync(AddConsultationEvaluateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddConsultationEvaluateDBRequest execute", $"{this.GetType().Name}"))
            {
                var consultationEvaluateDO = new ConsultationEvaluateEntity();
                if (request.EvaluateScore > 0 && request.EvaluateScore <= 1.5)
                {
                    consultationEvaluateDO.EvaluateGrade = EvaluateGradeEnum.Dissatisfaction;
                }
                else if (request.EvaluateScore > 1.5 && request.EvaluateScore <= 3.5)
                {
                    consultationEvaluateDO.EvaluateGrade = EvaluateGradeEnum.General;
                }
                else if (request.EvaluateScore > 3.5)
                {
                    consultationEvaluateDO.EvaluateGrade = EvaluateGradeEnum.Satisfaction;
                }
                else
                {
                    consultationEvaluateDO.EvaluateGrade = EvaluateGradeEnum.UnSet;
                }
                consultationEvaluateDO.EvaluateScore = request.EvaluateScore;
                consultationEvaluateDO.ConsultationRecordCode = request.ConsultationRecordCode;
                consultationEvaluateDO.CreateTime = DateTime.UtcNow;
                consultationEvaluateDO.Feedback = request.Feedback;
                var result = await _consultationEvaluateDBRepository.InsertOneAsync(consultationEvaluateDO);
                return !string.IsNullOrWhiteSpace(result);
            }
        }

        public async Task<bool> DeleteConsultationEvaluateAsync(DeleteConsultationEvaluateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteConsultationEvaluateAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.ConsultationEvaluateCode))
                {
                    var result = await _consultationEvaluateDBRepository.DeleteOneAsync("Code", request.ConsultationEvaluateCode);
                    return result;
                }
                return false;
            }
        }

        public async Task<bool> UpdateConsultationEvaluateAsync(UpdateConsultationEvaluateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateConsultationEvaluateAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.ConsultationEvaluateCode))
                {
                    var evaluateGrade = EvaluateGradeEnum.UnSet;
                    if (request.EvaluateScore > 0 && request.EvaluateScore < 1.5)
                    {
                        evaluateGrade = EvaluateGradeEnum.Dissatisfaction;
                    }
                    else if (request.EvaluateScore >= 1.5 && request.EvaluateScore < 3.5)
                    {
                        evaluateGrade = EvaluateGradeEnum.General;
                    }
                    else if (request.EvaluateScore >= 3.5)
                    {
                        evaluateGrade = EvaluateGradeEnum.Satisfaction;
                    }
                    var update = Builders<ConsultationEvaluateEntity>.Update
                    .Set(c => c.EvaluateGrade, evaluateGrade)
                    .Set(c => c.EvaluateScore, request.EvaluateScore)
                    .Set(c => c.Feedback, request.Feedback);
                    var filter = Builders<ConsultationEvaluateEntity>.Filter.Where(c =>
                    c.Code == request.ConsultationEvaluateCode);
                    var result = await _consultationEvaluateDBRepository.UpdateOneAsync(filter, update);
                    return result > 0;
                }
                return false;
            }
        }

        public async Task<ConsultationEvaluateDTO> GetConsultationEvaluateDetailAsync(GetConsultationEvaluateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetConsultationEvaluateDetailAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.ConsultationRecordCode))
                {
                    var result = await _consultationEvaluateDBRepository.FindOneAsync("ConsultationRecordCode", request.ConsultationRecordCode);
                    if (result != null)
                    {
                        return new ConsultationEvaluateDTO
                        {
                            ConsultationEvaluateCode = result.Code,
                            ConsultationRecordCode = result.ConsultationRecordCode,
                            EvaluateGrade = result.EvaluateGrade,
                            EvaluateScore = result.EvaluateScore,
                            Feedback = result.Feedback
                        };
                    }
                }
                return null;
            }
        }

        private async Task<List<ConsultationEvaluateEntity>> GetConsultationEvaluatesAsync(EvaluateGradeEnum evaluateGrade)
        {
            using (new Performance(_performanceThreshold, "{0} GetConsultationEvaluateDetailAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<ConsultationEvaluateEntity>.Filter.Where(x => x.EvaluateGrade == evaluateGrade);
                var result = await _consultationEvaluateDBRepository.FindAllAsync(filter);
                return result;
            }
        }

        private async Task<List<ConsultationEvaluateEntity>> GetAllConsultationEvaluatesAsync()
        {
            using (new Performance(_performanceThreshold, "{0} GetConsultationEvaluateDetailAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _consultationEvaluateDBRepository.FindAllAsync();
                return result;
            }
        }

        /// <summary>
        /// 根据code列表获取评价列表
        /// </summary>
        /// <param name="request">移除设备请求实体</param>
        /// <returns></returns>
        public async Task<List<ConsultationEvaluateDTO>> GetAllConsultationEvaluatesByCodesAsync(List<string> codeList)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllConsultationEvaluatesByCodesAsync execute", $"{this.GetType().Name}"))
            {
                var filter = new Dictionary<string, List<string>>();
                filter.Add("ConsultationRecordCode", codeList);
                var dataList = await _consultationEvaluateDBRepository.FindInAsync(filter);
                return dataList.MappingTo<List<ConsultationEvaluateDTO>>();
            }
        }


        public async Task<bool> SetMemberStatusAsync(SetMemberStatusDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SetMemberInfo execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(request.ConsultationRecordCode))
                {
                    var result = await _consultationRecordDBRepository.FindOneAsync("Code", request.ConsultationRecordCode);
                    if (result != null)
                    {
                        if (result.ConsultationStatus == TransactionStatusEnum.ToStart || (result.ConsultationStatus == TransactionStatusEnum.Expired && result.ConsultationTimeEnd > DateTime.UtcNow.AddDays(-30)))
                        {
                            var members = result.ConsultationMembers;
                            members.ForEach(f =>
                            {
                                if (f.MemberCode == request.MemberCode && request.MemberStatus != ConsultationMemeberAcceptStatus.Default)
                                {
                                    f.MemberOperateTime = DateTime.UtcNow;
                                    f.MemberStatus = request.MemberStatus;
                                    f.RefusalCause = request.RefusalCause;
                                }
                            });
                            var update = Builders<ConsultationRecordEntity>.Update
                            .Set(c => c.ConsultationMembers, members);
                            var filter = Builders<ConsultationRecordEntity>.Filter.Where(c =>
                            c.Code == request.ConsultationRecordCode);
                            return await _consultationRecordDBRepository.UpdateAllAsync(filter, update) > 0;
                        }
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 查询本人会诊预约记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<List<ConsultationDetailDTO>> FindConsultationRecordsByUserCodeAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindConsultationRecordsByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filterDefinitions = new List<FilterDefinition<ConsultationRecordEntity>>();
                //自己的
                filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.ElemMatch(x => x.ConsultationMembers, Builders<ConsultationMemberDO>.Filter.Eq(c => c.MemberCode, userCode)));
                filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ApplyUserCode == userCode));
                filterDefinitions.Add(Builders<ConsultationRecordEntity>.Filter.Where(x => x.ExpertUserCode == userCode));
                var filter = Builders<ConsultationRecordEntity>.Filter.Or(filterDefinitions);
                var consultationRecordList = await _consultationRecordDBRepository.FindAllAsync(filter);
                var result = consultationRecordList.MappingTo<List<ConsultationDetailDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 查询待开始的会诊记录列表
        /// </summary>
        /// <returns></returns>
        public async Task<IList<ConsultationDetailDTO>> FindToStartConsultationRecordListAsync()
        {
            using (new Performance(_performanceThreshold, "{0} FindToStartConsultationRecordListAsync execute", $"{this.GetType().Name}"))
            {
                List<ConsultationDetailDTO> result = new List<ConsultationDetailDTO>();
                var filter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.ConsultationStatus == TransactionStatusEnum.ToStart);
                var consultationRecordList = await _consultationRecordDBRepository.FindAllAsync(filter);
                result = consultationRecordList.MappingTo<List<ConsultationDetailDTO>>();
                if (result != null && result.Count > 0)
                {
                    var patientCodes = result.Select(x => x.PatientCode).ToList();
                    var patientInfos = await FindPatientInfoListByCodeListAsync(patientCodes);
                    var expertUserCodes = result.Select(x => x.ExpertUserCode).ToList();
                    var users = await FindUserListByUserCodesAsync(expertUserCodes);
                    foreach (var item in result)
                    {
                        var expertInfo = users.FirstOrDefault(u => u.UserCode == item.ExpertUserCode);
                        if (!string.IsNullOrWhiteSpace(expertInfo?.FullName))
                        {
                            item.ExpertUserName = expertInfo?.FullName;
                        }
                        else if (!string.IsNullOrWhiteSpace(expertInfo?.UserName))
                        {
                            item.ExpertUserName = expertInfo?.UserName;
                        }
                        var patientInfo = patientInfos.FirstOrDefault(x => x.PatientCode == item.PatientCode);
                        if (patientInfo != null)
                        {
                            var patientInfoDTO = patientInfo.MappingTo<PatientInfoDTO>();
                            if (patientInfoDTO != null)
                            {
                                item.PatientName = CryptionHelper.NameEncrypt(patientInfo.Name, false);
                                item.Sex = patientInfo.PatientData.FirstOrDefault(x => !string.IsNullOrWhiteSpace(x.Key) && x.Key.Contains("Sex"))?.Value;
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 根据状态查询会诊记录
        /// </summary>
        /// <returns></returns>
        public async Task<IList<ConsultationDetailDTO>> GetConsultationRecordsByStatusAsync(GetConsultationRecordsByStatusDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetConsultationRecordsByStatusAsync execute", $"{this.GetType().Name}"))
            {
                var status = request.Status;
                List<ConsultationDetailDTO> result = new List<ConsultationDetailDTO>();
                var filter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.ConsultationStatus == status);
                var entities = await _consultationRecordDBRepository.FindAllAsync(filter);
                return entities.MappingTo<List<ConsultationDetailDTO>>();
            }
        }

        /// <summary>
        /// 更新会诊发起人编码
        /// </summary>
        /// <param name="recordId"></param>
        /// <param name="initatorCode"></param>
        /// <returns></returns>
        public async Task<bool> UpdateInitatorCodeAsync(string recordId, string initatorCode)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateInitatorCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _consultationRecordDBRepository.FindOneAsync("Code", recordId);
                if (entity != null)
                {
                    if (entity.InitiatorCode != initatorCode)
                    {
                        entity.InitiatorCode = initatorCode;
                        return await _consultationRecordDBRepository.UpdateOneAsync("Code", recordId, entity, true);
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// 更新会诊状态
        /// </summary>
        /// <param name="consultationCode"></param>
        /// <param name="status"></param>
        /// <returns></returns>
        public async Task<bool> UpdateConsultationStatusAsync(string consultationCode, TransactionStatusEnum status)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateConsultationStatusAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _consultationRecordDBRepository.FindOneAsync("Code", consultationCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (status == TransactionStatusEnum.Expired && entity.ConsultationStatus != TransactionStatusEnum.ToStart)
                    {
                        return false;
                    }
                    if (entity.ConsultationStatus != status && entity.ConsultationStatus != TransactionStatusEnum.End)
                    {
                        entity.ConsultationStatus = status;
                        if (entity.IsConsulted && entity.ConsultationStatus == TransactionStatusEnum.ToStart)
                        {
                            entity.ConsultationStatus = TransactionStatusEnum.PendingReport;
                        }
                        if (!entity.IsConsulted && entity.ConsultationStatus == TransactionStatusEnum.PendingReport)
                        {
                            entity.IsConsulted = true;
                        }
                        var result = await _consultationRecordDBRepository.UpdateOneAsync("Code", consultationCode, entity, true);
                        //修改日程状态
                        var updateScheduleStateRequest = new UpdateScheduleStateDBRequest
                        {
                            RelevanceCode = entity.Code,
                            Status = entity.ConsultationStatus,
                        };
                        await UpdateScheduleStateDBAsync(updateScheduleStateRequest);
                        return result;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// 更新会诊邀请信息
        /// </summary>
        /// <param name="consultationCode"></param>
        /// <param name="consultationReminders"></param>
        /// <returns></returns>
        public async Task<bool> UpdateConsultationRemindersAsync(string consultationCode, List<ConsultationReminderDTO> consultationReminders)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateConsultationRemindersAsync execute", $"{this.GetType().Name}"))
            {
                if (consultationReminders != null)
                {
                    var consultationRemindersDO = consultationReminders.MappingTo<List<ConsultationReminderDO>>();
                    var filter = Builders<ConsultationRecordEntity>.Filter.Eq(f => f.Code, consultationCode);
                    var update = Builders<ConsultationRecordEntity>.Update.Set(f => f.ConsultationReminders, consultationRemindersDO);
                    return await _consultationRecordDBRepository.UpdateOneAsync(filter, update) > 0;
                }
            }
            return false;
        }

        /// <summary>
        /// 获取急诊订单号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> GetEmergencyOrderAsync(GetEmergencyOrderDBRequest request)
        {
            DateTime zeroTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, 0, 0, 0);
            var filter1 = Builders<ConsultationRecordEntity>.Filter.Eq(f => f.DeviceCode, request.EmergencyDeviceCode);
            var filter2 = Builders<ConsultationRecordEntity>.Filter.Gte(f => f.CreateTime, zeroTime.ToUniversalTime());
            var filter3 = Builders<ConsultationRecordEntity>.Filter.Eq(f => f.IsEmergency, true);
            var orderCount = await _consultationRecordDBRepository.CountDocumentsAsync(Builders<ConsultationRecordEntity>.Filter.And(filter1, filter2, filter3));
            long.TryParse(zeroTime.ToString("yyyyMMdd"), out long zeroTimeNumber);
            return (zeroTimeNumber * 10000 + orderCount + 1).ToString();
        }

        /// <summary>
        /// 根据急诊单号和设备号获取急诊单
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ConsultationDetailDTO> GetRecordByEmergencyCodeAndDeviceCodeAsync(GetRecordByEmergencyCodeAndDeviceCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetRecordByEmergencyCodeAndDeviceCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<ConsultationRecordEntity>.Filter.Where(x => x.EmergencyCode == request.EmergencyCode && x.DeviceCode == request.EmergencyDeviceCode);
                var list = await _consultationRecordDBRepository.FindAllAsync(filter);
                if (list != null && list.Count > 0)
                {
                    var result = list.FirstOrDefault().MappingTo<ConsultationDetailDTO>();
                    return result;
                }
                return null;
            }
        }

        /// <summary>
        /// 更新会诊专家编码
        /// </summary>
        /// <param name="recordId"></param>
        /// <param name="expertCode"></param>
        /// <returns></returns>
        public async Task<bool> UpdateExpertCodeAsync(string recordId, string expertCode)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateExpertCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _consultationRecordDBRepository.FindOneAsync("Code", recordId);
                if (entity != null)
                {
                    if (entity.ExpertUserCode == expertCode)
                    {
                        return true;
                    }
                    var expertEntity = await FindUserByCodeAsync(expertCode);
                    if (expertEntity != null && !string.IsNullOrWhiteSpace(expertEntity.UserCode))
                    {
                        if (entity.ConsultationMembers == null)
                        {
                            entity.ConsultationMembers = new List<ConsultationMemberDO>();
                        }
                        if (entity.IsEmergency && expertEntity.BindAssistantDoctorUserCode != null && !entity.ConsultationMembers.Any(a => a.MemberCode == expertEntity.BindAssistantDoctorUserCode))
                        {
                            entity.ConsultationMembers.Add(new ConsultationMemberDO
                            {
                                MemberCode = expertEntity.BindAssistantDoctorUserCode,
                                MemberOrganizationCode = expertEntity.RootOrganizationCode,
                                MemberDepartmentCode = expertEntity.OrganizationCode,
                                MemberOperateTime = DateTime.UtcNow,
                                MemberStatus = ConsultationMemeberAcceptStatus.Accepted
                            });
                        }
                        entity.ExpertUserCode = expertEntity.UserCode;
                        entity.ExpertOrganizationCode = expertEntity.RootOrganizationCode;
                        if (expertEntity.OrganizationCode != expertEntity.RootOrganizationCode)
                        {
                            entity.ExpertDepartmentCode = expertEntity.OrganizationCode;
                        }
                        else
                        {
                            entity.ExpertDepartmentCode = "";
                        }
                        return await _consultationRecordDBRepository.UpdateOneAsync("Code", recordId, entity, true);
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// 根据会诊单号查询会诊中可直接切换的会诊单列表
        /// 查询同一家申请医院&&绑定同一台会诊设备&&会诊日期是同一天&&会诊记录==待开始&&主管专家相同的会诊单列表
        /// </summary>
        /// <returns></returns>
        public async Task<IList<ConsultationDetailDTO>> FindCanSwitchConsultationsDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindCanSwitchConsultationsDBAsync execute", $"{this.GetType().Name}"))
            {
                var consultationRecordDO = await _consultationRecordDBRepository.FindOneAsync("Code", code);
                consultationRecordDO.ConsultationMembers = consultationRecordDO.ConsultationMembers?.ToList() ?? new List<ConsultationMemberDO>();

                var builder = Builders<ConsultationRecordEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false
                && c.ConsultationStatus == TransactionStatusEnum.ToStart
                && c.ApplyOrganizationCode == consultationRecordDO.ApplyOrganizationCode//同一申请机构
                && c.DeviceCode == consultationRecordDO.DeviceCode//同一设备
                && c.ExpertUserCode == consultationRecordDO.ExpertUserCode//同一汇总专家
                );//同一基层医生
                var findList = await _consultationRecordDBRepository.FindAllAsync(filter);
                var result = new List<ConsultationDetailDTO>();
                if (findList != null && findList.Count > 0)
                {
                    foreach (var item in findList)
                    {
                        //同一天
                        if (item.ConsultationTime.ToUniversalTime().Date != consultationRecordDO.ConsultationTime.ToUniversalTime().Date)
                        {
                            continue;
                        }
                        result.Add(item.MappingTo<ConsultationDetailDTO>());
                    }
                    result = result.OrderBy(x => x.ConsultationTime).ToList();//以会诊开始时间排序
                }
                return result;
            }
        }

        /// <summary>
        /// 更新急诊实时状态
        /// </summary>
        /// <param name="recordId"></param>
        /// <param name="expertCode"></param>
        /// <returns></returns>
        public async Task<bool> UpdateEmergencyStatusAsync(string recordId, EmergencyConsultationStatus status)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateExpertCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _consultationRecordDBRepository.FindOneAsync("Code", recordId);
                if (entity != null)
                {
                    entity.EmergencyStatus = status;
                    return await _consultationRecordDBRepository.UpdateOneAsync("Code", recordId, entity, true);
                }
            }
            return false;
        }


        public async Task<bool> SyncBatchConsultationDBAsync(SyncBatchConsultationRequest syncBatchConsultationRequest)
        {
            try
            {
                SyncDBEnum syncType = syncBatchConsultationRequest.SyncType;
                var insertList = new List<ConsultationRecordEntity>();
                var updateList = new List<ConsultationRecordEntity>();
                var orgs = new List<OrganizationDO>();
                var orgCodes = syncBatchConsultationRequest?.SyncConsultations?.Select(x => x.ApplyOrganizationCode).ToList();
                if (orgCodes != null && orgCodes.Count() > 0)
                {
                    MongoCustomerFilter<MongoTable> filter = new();
                    filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", orgCodes);
                    orgs = await _organizationDBRepository.FindAllAsync(filter.AndCombine());
                }
                var consultationCodes = syncBatchConsultationRequest.SyncConsultations.Select(x => x.ConsultationCode);
                var filterConsultation = Builders<ConsultationRecordEntity>.Filter.Where(c => consultationCodes.Contains(c.Code));
                var consultationEntities = await _consultationRecordDBRepository.FindAllAsync(filterConsultation);
                Dictionary<string, List<string>> filterPatient = new Dictionary<string, List<string>>();
                filterPatient.Add("RootOrganizationCode", orgCodes);
                var patients = await _patientInfoDBRepository.FindInAsync(filterPatient);
                var userFilter = new Dictionary<string, List<string>>();
                userFilter.Add("OrganizationCode", orgCodes);
                var userList = await _userDBRepository.FindInAsync(userFilter);
                foreach (var request in syncBatchConsultationRequest.SyncConsultations)
                {
                    try
                    {
                        var isUpdate = false;
                        var consultationCode = request.ConsultationCode;

                        var consultationRecordEntity = consultationEntities.FirstOrDefault(x => x.Code == request.ConsultationCode);
                        if (consultationRecordEntity == null || string.IsNullOrWhiteSpace(consultationRecordEntity.Code))
                        {
                            consultationRecordEntity = new ConsultationRecordEntity();
                            consultationRecordEntity.Code = consultationCode;
                        }
                        else
                        {
                            isUpdate = true;
                        }
                        consultationRecordEntity = request.MappingTo<ConsultationRecordEntity>();
                        if (consultationRecordEntity.ConsultationFileList?.Count > 0)
                        {
                            foreach (var fileItem in consultationRecordEntity.ConsultationFileList)
                            {
                                fileItem.SourceUrl = fileItem.SourceUrl.ToUrlToken();
                                fileItem.PreviewImageUrl = fileItem.PreviewImageUrl.ToUrlToken();
                                fileItem.CoverImageUrl = fileItem.CoverImageUrl.ToUrlToken();
                            }
                        }
                        if (request.Evaluates != null && !string.IsNullOrEmpty(request.Evaluates.Code))
                        {
                            //找到相同的
                            var builder = Builders<ConsultationEvaluateEntity>.Filter;
                            var filter = builder.Eq(c => c.Code, request.Evaluates.Code);
                            var consultationEvaluateInfo = await _consultationEvaluateDBRepository.FindOneAsync(filter);
                            //求同存异
                            if (!string.IsNullOrEmpty(consultationEvaluateInfo?.Code))
                            {
                                var data = request.Evaluates.MappingTo<ConsultationEvaluateEntity>();
                                data.Code = request.Evaluates.Code.Trim();
                                var res = await _consultationEvaluateDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data);

                            }
                            else
                            {
                                //新增
                                var data = request.Evaluates.MappingTo<ConsultationEvaluateEntity>();
                                data.Code = request.Evaluates.Code.Trim();
                                var res = await _consultationEvaluateDBRepository.InsertOneForAsyncDBAsync(data);
                            }
                        }
                        var org = orgs.FirstOrDefault(x => x.OrganizationCode == consultationRecordEntity.ApplyOrganizationCode);
                        if (org == null || string.IsNullOrWhiteSpace(org.OrganizationCode))
                        {
                            continue;
                        }
                        else
                        {
                            if (org.PatientType == OrganizationPatientTypeEnum.Animals)
                            {
                                var patientData = request.PatientInfo;
                                if (patientData.FirstOrDefault(x => x.Key == "Name") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Name").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Name").Key = "AnimalInfoName";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "Phone") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Phone").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Phone").Key = "AnimalInfoPhone";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "ID") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "ID").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "ID").Key = "AnimalInfoID";
                                }
                                if (patientData.FirstOrDefault(x => x.Key == "Sex") != null && !string.IsNullOrWhiteSpace(patientData.FirstOrDefault(x => x.Key == "Sex").Value))
                                {
                                    patientData.FirstOrDefault(x => x.Key == "Sex").Key = "AnimalInfoSex";
                                    patientData.FirstOrDefault(x => x.Key == "AnimalInfoSex").Value = "AnimalInfo" + patientData.FirstOrDefault(x => x.Key == "AnimalInfoSex").Value;
                                }
                            }
                        }
                        //注意加密和创建时间、修改时间
                        SetEncryptedData(request.PatientInfo);
                        consultationRecordEntity.PatientCode = await AddOrUpdatePatientForSync(patients, request.PatientInfo, request.DeviceCode, org, request.CreateTime, request.UpdateTime); //使用姓名+病人ID+机构ID来识别病人唯一性
                        consultationRecordEntity.PatientDatas = request.PatientInfo;//存储病人快照信息       
                                                                                    //扫查医生
                        var scanUser = request.ScanUser;
                        if (!string.IsNullOrWhiteSpace(scanUser))
                        {
                            consultationRecordEntity.ScanUserCode = userList.FirstOrDefault(x => x.FullName.Contains(scanUser) || x.UserName.Contains(scanUser))?.Code;
                        }
                        if (isUpdate)
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                updateList.Add(consultationRecordEntity);
                            }
                            else
                            {
                                await _consultationRecordDBRepository.UpdateOneForSyncDBAsync("Code", consultationRecordEntity.Code, consultationRecordEntity);
                            }

                        }
                        else
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                insertList.Add(consultationRecordEntity);
                            }
                            else
                            {
                                await _consultationRecordDBRepository.InsertOneForAsyncDBAsync(consultationRecordEntity);
                            }

                        }
                        if (request.SyncFollowUpVisitList?.Count > 0)
                        {
                            //找到相同的
                            var ids = request.SyncFollowUpVisitList.Select(c => c.FollowUpVisitCode).Distinct().ToList();
                            var builder = Builders<FollowUpVisitEntity>.Filter;
                            var filter = builder.In(c => c.Code, ids);
                            var courseLabelList = await _followUpVisitDBRepository.FindAllAsync(filter);
                            //求同存异
                            if (courseLabelList?.Count > 0)
                            {
                                var existCodes = courseLabelList.Select(c => c.Code).Distinct().ToList();
                                var updateData = request.SyncFollowUpVisitList.Where(c => existCodes.Contains(c.FollowUpVisitCode));
                                var insertData = request.SyncFollowUpVisitList.Where(c => !existCodes.Contains(c.FollowUpVisitCode));
                                if (insertData?.Count() > 0)
                                {
                                    //新增
                                    var req = insertData.Select(
                                        f =>
                                        {
                                            var data = f.MappingTo<FollowUpVisitEntity>();
                                            data.Code = f.FollowUpVisitCode.Trim();
                                            if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Synchronize)
                                            {
                                                var r = _followUpVisitDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                            }
                                            return data;
                                        }
                                    ).ToList();
                                    if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Migrate)
                                    {
                                        var res = await _followUpVisitDBRepository.InsertAllWithoutOplogLogAsync(req);
                                    }
                                }
                                if (updateData?.Count() > 0)
                                {
                                    var req = updateData.Select(
                                        f =>
                                        {
                                            var data = f.MappingTo<FollowUpVisitEntity>();
                                            data.Code = f.FollowUpVisitCode.Trim();
                                            if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Synchronize)
                                            {
                                                var r = _followUpVisitDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                            }
                                            return data;
                                        }
                                    ).ToList();
                                    if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Migrate)
                                    {
                                        var res = await _followUpVisitDBRepository.UpdateAllWithoutOplogLogAsync(req);
                                    }
                                }
                            }
                            else
                            {
                                //新增
                                var req = request.SyncFollowUpVisitList.Select(
                                    f =>
                                    {
                                        var data = f.MappingTo<FollowUpVisitEntity>();
                                        data.Code = f.FollowUpVisitCode.Trim();
                                        if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Synchronize)
                                        {
                                            var r = _followUpVisitDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                        }
                                        return data;
                                    }
                                ).ToList();
                                if (syncBatchConsultationRequest.SyncType == SyncDBEnum.Migrate)
                                {
                                    var res = await _followUpVisitDBRepository.InsertAllWithoutOplogLogAsync(req);
                                }
                            }
                        }

                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn("foreach SyncBatchConsultationDBAsync," + ex);
                    }
                }
                //迁移or同步
                if (syncType == SyncDBEnum.Migrate)
                {
                    if (updateList.Count > 0)
                    {
                        await _consultationRecordDBRepository.UpdateAllWithoutOplogLogAsync(updateList);
                    }
                    if (insertList.Count > 0)
                    {
                        await _consultationRecordDBRepository.InsertAllWithoutOplogLogAsync(insertList);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn("SyncBatchConsultationDBAsync," + ex);
            }
            return true;
        }
        /// <summary>
        /// 查询会诊增量数据
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        /// <returns></returns>
        public async Task<IList<LiveConsultationDTO>> FindConsultationsDataCenterAsync(DateTime startTime, DateTime endTime)
        {
            var result = new List<LiveConsultationDTO>();
            try
            {
                var customerFilter = Builders<ConsultationRecordEntity>.Filter.Where(
                    c => (c.CreateTime >= startTime && c.CreateTime <= endTime)
                    || (c.UpdateTime >= startTime && c.UpdateTime <= endTime));
                var recordResult = await _consultationRecordDBRepository.FindAllAsync(customerFilter);
                if (recordResult != null && recordResult.Count > 0)
                {
                    //获取医院信息 
                    var rootOrganizationCodes = recordResult.Select(v => v.ApplyOrganizationCode).ToList();
                    rootOrganizationCodes.AddRange(recordResult.Select(v => v.ExpertOrganizationCode).ToList());
                    var filter = new Dictionary<string, List<string>>();
                    filter.Add("Code", rootOrganizationCodes.ToList());
                    var organizationList = await _organizationDBRepository.FindInAsync(filter);
                    //获取设备信息
                    var deviceCodeCodes = recordResult.Select(v => v.DeviceCode).ToList();
                    filter = new Dictionary<string, List<string>>();
                    filter.Add("Code", deviceCodeCodes);
                    var deviceList = await _deviceInfoDBRepository.FindInAsync(filter);
                    //用户信息
                    var userCodes = recordResult.Select(v => v.ApplyUserCode).ToList();
                    userCodes.AddRange(recordResult.Select(v => v.ExpertUserCode).ToList());
                    filter = new Dictionary<string, List<string>>();
                    filter.Add("Code", userCodes);
                    var userList = await _userDBRepository.FindInAsync(filter);

                    var evaluateList = await GetAllConsultationEvaluatesAsync();
                    if (evaluateList == null)
                    {
                        evaluateList = new List<ConsultationEvaluateEntity>();
                    }
                    var followUpVisitList = await _followUpVisitDBRepository.FindAllAsync();
                    if (followUpVisitList == null)
                    {
                        followUpVisitList = new List<FollowUpVisitEntity>();
                    }
                    foreach (var item in recordResult)
                    {
                        var applyOrganization = organizationList.FirstOrDefault(c => c.Code == item.ApplyOrganizationCode);
                        var expertOrganization = organizationList.FirstOrDefault(c => c.Code == item.ExpertOrganizationCode);
                        var device = deviceList.FirstOrDefault(c => c.Code == item.DeviceCode);
                        var applyUser = userList.FirstOrDefault(c => c.Code == item.ApplyUserCode);
                        var expertUser = userList.FirstOrDefault(c => c.Code == item.ExpertUserCode);

                        var curFollowUpVisitList = followUpVisitList?.Where(c => c.ConsultationRecordCode == item.ConsultationCode) ?? new List<FollowUpVisitEntity>();
                        var curEvaluateInfo = evaluateList?.FirstOrDefault(c => c.ConsultationRecordCode == item.ConsultationCode) ?? new ConsultationEvaluateEntity();
                        result.Add(new LiveConsultationDTO()
                        {
                            Code = item.ConsultationCode,
                            CreateTime = item.CreateTime,
                            UpdateTime = item.UpdateTime,
                            EmergencyCode = item.EmergencyCode,
                            IsEmergency = item.IsEmergency,
                            InitiatorCode = item.InitiatorCode,
                            PrimaryDiagnosis = item.PrimaryDiagnosis,
                            Diseases = item.Diseases,
                            ReportMode = item.ReportMode,
                            ApproverCode = item.ApproverCode,
                            ConsultationReminders = item.ConsultationReminders?.MappingTo<List<ConsultationReminderDTO>>(),
                            Description = item.Description,
                            ConsultationMembers = item.ConsultationMembers?.MappingTo<List<ConsultationMemberDTO>>(),
                            Location = item.Location,
                            RejectReason = item.RejectReason,
                            ConsultationStatus = item.ConsultationStatus,
                            ConsultationTimeEnd = item.ConsultationTimeEnd,
                            ConsultationTime = item.ConsultationTime,
                            // OperateUserName = item.OperateUserName,
                            OperateUserCode = item.OperateUserCode,
                            //  ScanUserName = item.ScanUserName,
                            ScanUserCode = item.ScanUserCode,
                            ScanPositions = item.ScanPositions,
                            PatientType = item.PatientType,
                            PatientDatas = item.PatientDatas,
                            // EvaluateGrade = item.EvaluateGrade,
                            IsNeedFollowUpVisit = item.NeedFollowUpVisit,
                            // FollowUpVisitStatus = item.FollowUpVisitStatus,
                            Evaluates = curEvaluateInfo.MappingTo<SyncEvaluatesRequest>(),
                            SyncFollowUpVisitList = curFollowUpVisitList.MappingTo<List<SyncFollowUpVisitRequest>>(),
                            PrimaryHosptial = new OrganizationView()
                            {
                                Code = applyOrganization?.Code,
                                Name = applyOrganization?.OrganizationName,
                                Description = applyOrganization?.Description,
                                DepartmentCode = item.ApplyDepartmentCode,
                                //DepartmentName 
                                LogoUrl = applyOrganization?.LogoUrl
                                //    CountryCode =
                                //    CountryName =
                                //    ProvinceCode =
                                //    ProvinceName =
                                //    CityCode =
                                //    CityName =
                            },
                            CentralHospital = new OrganizationView()
                            {
                                Code = expertOrganization?.Code,
                                Name = expertOrganization?.OrganizationName,
                                Description = expertOrganization?.Description,
                                DepartmentCode = item.ExpertDepartmentCode,
                                //DepartmentName 
                                LogoUrl = expertOrganization?.LogoUrl
                                //    CountryCode =
                                //    CountryName =
                                //    ProvinceCode =
                                //    ProvinceName =
                                //    CityCode =
                                //    CityName =
                            },
                            DeviceInfo = new DeviceView()
                            {
                                Code = device?.Code,
                                Name = device?.Name,
                                Description = device?.Description,
                                UniqueCode = device?.ShortCode,
                                OrganizationCode = device?.OrganizationCode,
                                // OrganizationName 
                                DeviceType = device?.DeviceType,
                                DeviceModel = device?.DeviceModel,
                            },
                            DoctorInfo = new UserView()
                            {
                                Code = applyUser?.Code,
                                Name = applyUser?.FullName,
                                OrganizationCode = applyUser?.OrganizationCode,
                                OrganizationName = organizationList?.FirstOrDefault(c => c.Code == applyUser?.OrganizationCode)?.OrganizationName
                            },
                            ExpertInfo = new UserView()
                            {
                                Code = expertUser?.Code,
                                Name = expertUser?.FullName,
                                OrganizationCode = expertUser?.OrganizationCode,
                                OrganizationName = organizationList?.FirstOrDefault(c => c.Code == expertUser?.OrganizationCode)?.OrganizationName
                            },
                            FileInfos = item.ConsultationFileList?.Select(v => new LiveConsultationFileView()
                            {
                                DataType = v.FileDataType,
                                OriginalUrl = v.SourceUrl,
                                // CdnUrl = v.TerminalImages?.ImageUrl,
                                PreviewUrl = v.PreviewImageUrl,
                                ConverUrl = v.CoverImageUrl,
                                ConsultationFileType = v.ConsultationFileType,
                                //Type = v.FileDataType,TODO
                                RemedicalCode = v.RemedicalCode,
                                RemedicalMeasureCode = v.RemedicalMeasureCode,
                                UserCode = v.CreatorCode,

                            }).ToList()
                            // ReportConclusionInfos =
                        });
                    }
                }

            }
            catch (System.Exception ex)
            {
                Logger.WriteLineError($"FindConsultationsDataCenterAsync error:{ex}");
            }
            return result;


        }
    }
}