using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Result;
using MongoDB.Driver;
using System.Linq;
using System;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Courses;
using WingMongoDBModule.Entities;
using MongoDB.Bson;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request.Courses;
using System.Collections.Generic;
using WingServerCommon.Interfaces.Cache;
using System.IO;
using WingServerCommon.Log;
using System.Net.Http;
using System.Net;
using System.Data;
using WingServerCommon.Interfaces.FileTransfer;
using Syncfusion.Pdf;
using Syncfusion.Presentation;
using Syncfusion.PresentationToPdfConverter;
using SkiaSharp;
using WingInterfaceLibrary.DTO.DataCenter;
using WingMongoDBModule.DO;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver.GeoJsonObjectModel;

namespace WingMongoDBModule.Service
{
    /// <summary>
    /// 课程服务
    /// </summary>
    public partial class DatabaseService : JsonRpcService, ICourseDBService
    {
        /// <summary>
        /// 创建新课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateCourseAsync(CreateCourseDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateCourseAsync execute", "DatabaseService"))
            {
                var entity = request.Data.MappingTo<CourseInfoEntity>();
                entity.CreateTime = DateTime.UtcNow;
                var courseCode = await _courseInfoDBRepository.InsertOneAsync(entity);
                if (!string.IsNullOrEmpty(courseCode))
                {
                    if (request.Data.CourseAlbumCodes?.Count > 0)
                    {
                        var res = await UpdateAlbumCourseListDBAsync(request.Data.CourseAlbumCodes, null, courseCode);
                    }
                }
                return courseCode;
            }
        }

        /// <summary>
        /// /// 查询课程记录列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CourseInfoDetailDTO>> FindCoursesByCodesAsync(FindCourseByCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindCoursesByCodesAsync execute", "DatabaseService"))
            {
                var result = new List<CourseInfoDetailDTO>();
                if (request.Codes?.Count > 0)
                {
                    var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false);
                    var inFilter = Builders<CourseInfoEntity>.Filter.In(c => c.Code, request.Codes);
                    filter = filter & inFilter;
                    var findList = await _courseInfoDBRepository.FindAllAsync(filter);
                    result = findList.MappingTo<List<CourseInfoDetailDTO>>();
                    if (findList?.Count > 0)
                    {
                        for (var i = 0; i < findList.Count; i++)
                        {
                            var findEntity = findList[i];
                            if (findEntity?.CaseLabelCodes?.Count > 0 || findEntity?.CourseLabelCodes?.Count > 0)
                            {
                                //查询CASE的标签
                                var caseRequest = new QueryCourseLabelDBRequest()
                                {
                                    LanguageCode = request.LanguageCode,
                                    IsNeedChild = true,
                                    LabelCodes = findEntity.CaseLabelCodes,
                                    Type = LabelTypeEnum.CaseLabel
                                };
                                var caseLabelList = await GetCourseLabelDBByCodesAsync(caseRequest);
                                if (caseLabelList?.Count > 0)
                                {
                                    result[i].CaseLabels = caseLabelList.Select(findCaseLabelInfo =>
                                        new BaseLabelInfoDTO()
                                        {
                                            Code = findCaseLabelInfo.Code,
                                            Name = findCaseLabelInfo.Name,
                                            ParentCode = findCaseLabelInfo.ParentCode
                                        }).ToList();
                                }
                                //查询COURSE的标签
                                var courseRequest = new QueryCourseLabelDBRequest()
                                {
                                    LanguageCode = request.LanguageCode,
                                    IsNeedChild = true,
                                    LabelCodes = findEntity.CourseLabelCodes,
                                    Type = LabelTypeEnum.CourseLabel
                                };
                                var courseLabelList = await GetCourseLabelDBByCodesAsync(courseRequest);
                                if (courseLabelList?.Count > 0)
                                {
                                    result[i].CourseLabels = courseLabelList.Select(findCourseLabelInfo =>
                                        new BaseLabelInfoDTO()
                                        {
                                            Code = findCourseLabelInfo.Code,
                                            Name = findCourseLabelInfo.Name,
                                            ParentCode = findCourseLabelInfo.ParentCode
                                        }).ToList();
                                }
                            }
                            if (findEntity?.UserGroupCodes?.Count > 0)
                            {
                                //查询所有的用户组
                                var allUserGroupList = await GetUserGroupListsDBByUserGroupCodesAsync(findEntity.UserGroupCodes);
                                if (allUserGroupList?.Count > 0)
                                {
                                    result[i].UserGroups = allUserGroupList.Select(c =>
                                        new BaseUserGroupDTO()
                                        {
                                            Code = c.Code,
                                            Name = c.Name
                                        }).ToList();
                                }
                            }
                            if (findEntity?.CourseAlbumCodes?.Count > 0)
                            {
                                var albumCodesList = await FindAllAlbumListByAlbumCodesDBAsync(findEntity.CourseAlbumCodes);
                                if (albumCodesList?.Count > 0)
                                {
                                    result[i].CourseAlbums = albumCodesList.Select(c =>
                                        new BaseCourseAlbumDTO()
                                        {
                                            Code = c.Code,
                                            Name = c.Name
                                        }).ToList();
                                }
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 查询课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseInfoDetailDTO> FindCourseByCodeAsync(FindCourseByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindCourseByCodeAsync execute", "DatabaseService"))
            {
                var dataList = await GetCourseInfoDetailDBByCodesAsync(new List<string>() { request.Code }, request.LanguageCode);
                var result = dataList?.FirstOrDefault() ?? new CourseInfoDetailDTO();
                return result;
            }
        }

        /// <summary>
        /// 封装课程记录
        /// </summary>
        /// <param name="request"></param>
        private async Task<IList<CourseInfoDetailDTO>> GetCourseInfoDetailDBByCodesAsync(IList<string> codes, string languageCode = "en-US")
        {
            IList<CourseInfoDetailDTO> result = new List<CourseInfoDetailDTO>();
            var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false);
            var inFilter = Builders<CourseInfoEntity>.Filter.In(c => c.Code, codes);
            filter = filter & inFilter;
            var findList = await _courseInfoDBRepository.FindAllAsync(filter);
            if (findList?.Count > 0)
            {
                var videoCodes = new List<string>();
                var materialCodes = new List<string>();
                foreach (var findItem in findList)
                {
                    if (findItem.CourseVideoCodes?.Count > 0)
                    {
                        videoCodes.AddRange(findItem.CourseVideoCodes);
                    }
                    if (findItem.CourseMaterialCodes?.Count > 0)
                    {
                        materialCodes.AddRange(findItem.CourseMaterialCodes);
                    }
                }
                //查询video
                var videoList = await FindVideosByCodesDBAsync(videoCodes);
                var materialList = await FindVideosByCodesDBAsync(materialCodes);
                //查询已报名学员数量
                var signInList = await FindSignInListDBByCourseCodesAsync(codes);
                result = findList.MappingTo<List<CourseInfoDetailDTO>>();
                foreach (var findEntity in result)
                {
                    if (findEntity.CourseVideoCodes?.Count > 0 && videoList?.Count > 0)
                    {
                        findEntity.CourseVideos = videoList.Where(c => findEntity.CourseVideoCodes.Contains(c.Code))?.ToList() ?? new List<VideoInfoDTO>();
                        findEntity.DefaultVideoToken = findEntity.CourseVideos.FirstOrDefault()?.VideoToken;
                    }
                    if (findEntity.CourseMaterialCodes?.Count > 0 && materialList?.Count > 0)
                    {
                        findEntity.CourseMaterials = materialList.Where(c => findEntity.CourseMaterialCodes.Contains(c.Code))?.ToList() ?? new List<VideoInfoDTO>();
                    }
                    if (signInList?.Count > 0)
                    {
                        findEntity.SignInCount = signInList.Where(c => c.CourseOrAlbumCode == findEntity.Code && (findEntity.Price <= 0 || c.IsPay))?.Count() ?? 0;
                    }
                    if (findEntity?.CaseLabelCodes?.Count > 0 || findEntity?.CourseLabelCodes?.Count > 0)
                    {
                        //查询CASE的标签
                        var caseRequest = new QueryCourseLabelDBRequest()
                        {
                            LanguageCode = languageCode,
                            IsNeedChild = true,
                            LabelCodes = findEntity.CaseLabelCodes,
                            Type = LabelTypeEnum.CaseLabel
                        };
                        var caseLabelList = await GetCourseLabelDBByCodesAsync(caseRequest);
                        if (caseLabelList?.Count > 0)
                        {
                            findEntity.CaseLabels = caseLabelList.Select(findCaseLabelInfo =>
                                new BaseLabelInfoDTO()
                                {
                                    Code = findCaseLabelInfo.Code,
                                    Name = findCaseLabelInfo.Name,
                                    ParentCode = findCaseLabelInfo.ParentCode
                                }).ToList();
                        }
                        //查询COURSE的标签
                        var courseRequest = new QueryCourseLabelDBRequest()
                        {
                            LanguageCode = languageCode,
                            IsNeedChild = true,
                            LabelCodes = findEntity.CourseLabelCodes,
                            Type = LabelTypeEnum.CourseLabel
                        };
                        var courseLabelList = await GetCourseLabelDBByCodesAsync(courseRequest);
                        if (courseLabelList?.Count > 0)
                        {
                            findEntity.CourseLabels = courseLabelList.Select(findCourseLabelInfo =>
                                new BaseLabelInfoDTO()
                                {
                                    Code = findCourseLabelInfo.Code,
                                    Name = findCourseLabelInfo.Name,
                                    ParentCode = findCourseLabelInfo.ParentCode
                                }).ToList();
                        }
                    }
                    if (findEntity?.UserGroupCodes?.Count > 0)
                    {
                        //查询所有的用户组
                        var allUserGroupList = await GetUserGroupListsDBByUserGroupCodesAsync(findEntity.UserGroupCodes);
                        if (allUserGroupList?.Count > 0)
                        {
                            findEntity.UserGroups = allUserGroupList.Select(c =>
                                new BaseUserGroupDTO()
                                {
                                    Code = c.Code,
                                    Name = c.Name
                                }).ToList();
                        }
                    }
                    if (findEntity.Experts?.Count > 0)
                    {
                        foreach (var expertItem in findEntity.Experts)
                        {
                            var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Where(c => c.Code == expertItem.Code)?.FirstOrDefault() ?? new CacheUserDTO();
                            expertItem.Name = !string.IsNullOrEmpty(userInfo.FullName) ? userInfo.FullName : userInfo.UserName;
                        }
                    }
                    if (findEntity.Assistants?.Count > 0)
                    {
                        foreach (var assistantItem in findEntity.Assistants)
                        {
                            var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Where(c => c.Code == assistantItem.Code)?.FirstOrDefault() ?? new CacheUserDTO();
                            assistantItem.Name = !string.IsNullOrEmpty(userInfo.FullName) ? userInfo.FullName : userInfo.UserName;
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 查询管理后台课程列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<BaseCourseInfoDTO>> FindAdminCoursePagesAsync(FindCoursePagesDBRequest request)
        {
            var pageResult = await GetAdminCoursesPagesByCondition(request);
            return PageResultConvert<CourseInfoEntity, BaseCourseInfoDTO>(pageResult);
        }

        /// <summary>
        /// 管理后台分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<PageCollection<CourseInfoEntity>> GetAdminCoursesPagesByCondition(FindCoursePagesDBRequest request)
        {
            var keyword = request.Keyword;
            var participantCode = request.ParticipantCode;//参与者
            var teacherCode = request.TeacherCode;//老师
            var userCode = request.UserCode;
            var builder = Builders<CourseInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }
            if (((int)request.CourseType) > 0)
            {
                filter = filter & builder.Where(c => c.CourseType == request.CourseType);
            }
            if (((int)request.Status) > 0)
            {
                filter = filter & builder.Where(c => c.Status == request.Status);
            }
            if (!string.IsNullOrEmpty(teacherCode))
            {
                filter = filter & builder.Where(c => c.TeacherCode == teacherCode);
            }
            if (!string.IsNullOrWhiteSpace(participantCode))//参与专家查询列表
            {
                var filterDefinitions = new List<FilterDefinition<CourseInfoEntity>>();
                filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Assistants, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode)));
                filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Experts, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode)));
                filter = filter & Builders<CourseInfoEntity>.Filter.Or(filterDefinitions);
                //filter = filter & Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Experts, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode));
            }
            if (request.CourseApprovalStatus != QueryCourseApprovalStatusEnum.All)
            {
                if (request.CourseApprovalStatus == QueryCourseApprovalStatusEnum.NotApproval)
                {
                    filter = filter & builder.Where(c => c.Status == CourseStatusEnum.NoApproval || c.Status == CourseStatusEnum.RejectApproval);
                }
                else
                {
                    filter = filter & builder.Where(c => c.Status != CourseStatusEnum.NoApproval && c.Status != CourseStatusEnum.RejectApproval);
                }
            }
            if (request.Status != CourseStatusEnum.Unknown)
            {
                filter = filter & builder.Where(c => c.Status == request.Status);
            }
            if (request.CourseType != CourseTypeEnum.Unknown)
            {
                filter = filter & builder.Where(c => c.CourseType == request.CourseType);
            }
            if (request.StartTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.CreateTime >= request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.CreateTime < request.EndTime.AddDays(1));
            }
            if (request.CourseLabels != null && request.CourseLabels.Count > 0)
            {
                filter = filter & builder.Where(c => c.CourseLabelCodes.Any(w => request.CourseLabels.Contains(w)));
            }
            //课程规模类型
            var audienceType = request.AudienceType;
            if (audienceType == CourseAudienceTypeEnum.PublicClass)
            {
                //查公开课
                filter = filter & builder.Where(c => c.AudienceType == audienceType);
            }
            else if (audienceType == CourseAudienceTypeEnum.PrivateClass)
            {
                filter = filter & builder.Where(c => c.AudienceType == audienceType);
                //不是老师查询创建或专家查询参与的情况（即学生或助教查询私有课）
                //需判断是否在班级内或在助教列表内
                if (!string.IsNullOrWhiteSpace(userCode) && string.IsNullOrWhiteSpace(teacherCode) && string.IsNullOrWhiteSpace(participantCode))
                {
                    var filterDefinitions = new List<FilterDefinition<CourseInfoEntity>>();
                    // //助教
                    // filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Assistants, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, userCode)));
                    //学生
                    var userGroups = await GetUserGroupsByStudentCode(userCode);
                    var userGroupCodes = userGroups.Select(x => x.Code);
                    if (userGroupCodes != null && userGroupCodes.Count() > 0)
                    {
                        filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.Where(c => c.UserGroupCodes.Any(w => userGroupCodes.Contains(w))));
                    }
                    filter = filter & Builders<CourseInfoEntity>.Filter.Or(filterDefinitions);
                }
            }
            else
            {
                //不是老师查询创建或专家查询参与的情况（即学生或助教查询所有课）
                //需判断是否在班级内或在助教列表内
                if (!string.IsNullOrWhiteSpace(userCode) && string.IsNullOrWhiteSpace(teacherCode) && string.IsNullOrWhiteSpace(participantCode))
                {
                    var filterDefinitions = new List<FilterDefinition<CourseInfoEntity>>();
                    filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.Where(c => c.AudienceType == CourseAudienceTypeEnum.PublicClass));
                    // //助教
                    // filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Assistants, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, userCode)));
                    var userGroups = await GetUserGroupsByStudentCode(userCode);
                    var userGroupCodes = userGroups.Select(x => x.Code);
                    if (userGroupCodes != null && userGroupCodes.Count() > 0)
                    {
                        filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.Where(c => c.UserGroupCodes.Any(w => userGroupCodes.Contains(w))));
                    }
                    filter = filter & Builders<CourseInfoEntity>.Filter.Or(filterDefinitions);
                }
            }
            var sort = Builders<CourseInfoEntity>.Sort.Descending(v => v.CreateTime).Ascending(v => (int)v.Status).Descending(v => v.IsStick).Descending(v => v.Sort);
            var dbPage = new DBPageRequest<CourseInfoEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            dbPage.Sort = sort;
            var pageResult = await _courseInfoDBRepository.GetPages(dbPage);
            return pageResult;
        }

        /// <summary>
        /// 通用分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<PageCollection<CourseInfoEntity>> GetCoursesPagesByCondition(FindCoursePagesDBRequest request)
        {
            var keyword = request.Keyword;
            var participantCode = request.ParticipantCode;//参与者
            var teacherCode = request.TeacherCode;//老师
            var userCode = request.UserCode;
            var builder = Builders<CourseInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }
            // if (((int)request.CourseType) > 0)
            // {
            //     filter = filter & builder.Where(c => c.CourseType == request.CourseType);
            // }
            // if (((int)request.Status) > 0)
            // {
            //     filter = filter & builder.Where(c => c.Status == request.Status);
            // }
            if (!string.IsNullOrEmpty(teacherCode))
            {
                filter = filter & builder.Where(c => c.TeacherCode == teacherCode);
            }
            if (!string.IsNullOrWhiteSpace(participantCode))//参与专家查询列表
            {
                var filterDefinitions = new List<FilterDefinition<CourseInfoEntity>>();
                filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Assistants, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode)));
                filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Experts, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode)));
                filter = filter & Builders<CourseInfoEntity>.Filter.Or(filterDefinitions);
                //filter = filter & Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Experts, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, participantCode));
            }
            if (request.CourseApprovalStatus != QueryCourseApprovalStatusEnum.All)
            {
                if (request.CourseApprovalStatus == QueryCourseApprovalStatusEnum.NotApproval)
                {
                    filter = filter & builder.Where(c => c.Status == CourseStatusEnum.NoApproval || c.Status == CourseStatusEnum.RejectApproval);
                }
                else
                {
                    filter = filter & builder.Where(c => c.Status != CourseStatusEnum.NoApproval && c.Status != CourseStatusEnum.RejectApproval);
                }
            }
            if (request.Status != CourseStatusEnum.Unknown)
            {
                filter = filter & builder.Where(c => c.Status == request.Status);
            }
            if (request.CourseType != CourseTypeEnum.Unknown)
            {
                filter = filter & builder.Where(c => c.CourseType == request.CourseType);
            }
            if (request.StartTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.StartTime >= request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.StartTime < request.EndTime.AddDays(1));
            }
            if (request.CourseLabels != null && request.CourseLabels.Count > 0)
            {
                filter = filter & builder.Where(c => c.CourseLabelCodes.Any(w => request.CourseLabels.Contains(w)));
            }
            else
            {
                //没有选择标签，查询全部
                //1.根据语言查询所有标签
                //2.查询所有有标签的课程或者没有标签的课程
                var queryInfo = new QueryCourseLabelDBRequest()
                {
                    IsNeedChild = true,
                    LanguageCode = request.LanguageCode,
                    Type = LabelTypeEnum.CourseLabel
                };
                var allCourseLables = await GetCourseLabelDBByCodesAsync(queryInfo);
                if (allCourseLables?.Count > 0)
                {
                    var findCourseLabels = allCourseLables.Where(c => c.IsShow == true)?.Select(c => c.Code)?.Distinct().ToList() ?? new List<string>();
                    if (findCourseLabels?.Count > 0)
                    {
                        filter = filter & builder.Where(c => (c.CourseLabelCodes != null && c.CourseLabelCodes.Count > 0 && c.CourseLabelCodes.Any(w => findCourseLabels.Contains(w))) || 
                        c.CourseLabelCodes == null || c.CourseLabelCodes.Count <= 0);
                    }
                }
            }
            // 是否仅查往期课程, todo
            if (request.QueryCourseType == QueryCourseTypeEnum.VideoCourse)
            {
                filter = filter & builder.Where(c => c.CourseType == CourseTypeEnum.Multimedia && c.IsJustVideoCourse == true);
            }
            else if (request.QueryCourseType == QueryCourseTypeEnum.LiveCourse)
            {
                filter = filter & builder.Where(c => c.CourseType == CourseTypeEnum.LiveCourse);
                if (string.IsNullOrEmpty(teacherCode) && string.IsNullOrEmpty(participantCode))
                {
                    //表示首页查询，过滤掉过期的课程，过期课程是指，starttime开始时间小于一天的
                    filter = filter & builder.Where(c => c.StartTime >= DateTime.UtcNow.AddDays(-1));
                }
            }
            else
            {
                if (request.JustPastVideo) 
                {
                    filter = filter & builder.Where(c => c.CourseType == CourseTypeEnum.Multimedia);
                }
            }
          
            //不是老师查询创建或专家查询参与的情况（即学生或助教查询所有课）
            //需判断是否在班级内或在助教列表内
            if (!string.IsNullOrWhiteSpace(userCode) && string.IsNullOrWhiteSpace(teacherCode) && string.IsNullOrWhiteSpace(participantCode))
            {
                var filterDefinitions = new List<FilterDefinition<CourseInfoEntity>>();
                filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.Where(c => c.AudienceType == CourseAudienceTypeEnum.PublicClass));
                // //助教
                // filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.ElemMatch(x => x.Assistants, Builders<StudentInfoEntity>.Filter.Eq(c => c.Code, userCode)));
                var userGroups = await GetUserGroupsByStudentCode(userCode);
                var userGroupCodes = userGroups.Select(x => x.Code);
                if (userGroupCodes != null && userGroupCodes.Count() > 0)
                {
                    filterDefinitions.Add(Builders<CourseInfoEntity>.Filter.Where(c => c.UserGroupCodes.Any(w => userGroupCodes.Contains(w))));
                }
                filter = filter & Builders<CourseInfoEntity>.Filter.Or(filterDefinitions);
            }
            var sort = Builders<CourseInfoEntity>.Sort.Descending(v => v.IsStick).Descending(v => v.Sort).Descending(v => v.CreateTime);
            var dbPage = new DBPageRequest<CourseInfoEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            dbPage.Sort = sort;
            var pageResult = await _courseInfoDBRepository.GetPages(dbPage);
            return pageResult;
        }

        /// <summary>
        /// 查询课程列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<BaseCourseInfoDTO>> FindCoursePagesAsync(FindCoursePagesDBRequest request)
        {
            var pageResult = await GetCoursesPagesByCondition(request);
            return PageResultConvert<CourseInfoEntity, BaseCourseInfoDTO>(pageResult);
        }

        /// <summary>
        /// 根据课程状态查询直播课
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<BaseCourseInfoDTO>> FindLiveCoursesByStatusDBAsync(List<CourseStatusEnum> statuList)
        {
            using (new Performance(_performanceThreshold, "{0} FindLiveCoursesByStatusDBAsync execute", "DatabaseService"))
            {
                var dataList = new List<BaseCourseInfoDTO>();
                if (statuList?.Count > 0)
                {
                    var builder = Builders<CourseInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.CourseType == CourseTypeEnum.LiveCourse && statuList.Contains(c.Status));
                    var result = await _courseInfoDBRepository.FindAllAsync(filter);
                    if (result?.Count > 0)
                    {
                        dataList = result.MappingTo<List<BaseCourseInfoDTO>>();
                    }
                }
                return dataList;
            }
        }

        /// <summary>
        /// 删除课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseByCodeAsync(DeleteCourseByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteCourseByCodeAsync execute", "DatabaseService"))
            {
                var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var update = Builders<CourseInfoEntity>.Update.Set(x => x.IsDelete, true).Set(c => c.UpdateTime, DateTime.UtcNow);
                    var res = await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                    if (res)
                    {
                        if (findEntity.CourseAlbumCodes?.Count > 0)
                        {
                            var albumFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.IsDelete == false && findEntity.CourseAlbumCodes.Contains(c.Code));
                            var findAlbumList = await _courseAlbumDBRepository.FindAllAsync(albumFilter);
                            if (findAlbumList?.Count > 0)
                            {
                                foreach(var item in findAlbumList)
                                {
                                    if (item.CourseCodes.Contains(findEntity.Code))
                                    {
                                        item.CourseCodes.Remove(findEntity.Code);
                                        await _courseAlbumDBRepository.UpdateOneAsync("Code", item.Code, item, true);
                                    }
                                }
                            }
                        }
                    }
                    return res;
                }
                return false;
            }
        }

        /// <summary>
        /// 设置课程禁言
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetCourseLiveProhibitionByDBAsync(CourseLiveProhibitionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseAsync execute", "DatabaseService"))
            {
                var res = false;
                var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var update = Builders<CourseInfoEntity>.Update
                    .Set(x => x.IsProhibition, request.IsProhibition)
                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                    res = await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return res;
            }
        }

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="data"></param>
        /// <param name="canNull"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseAsync(UpdateCourseDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseAsync execute", "DatabaseService"))
            {
                var res = false;
                var updateData = request.Data;
                var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var courseAlbumList = await GetCourseAlbumListByCourseCodeDBAsync(findEntity.Code);
                    var experts = updateData.Experts?.MappingTo<List<StudentInfoEntity>>();
                    var assistants = updateData.Assistants?.MappingTo<List<StudentInfoEntity>>();
                    var update = Builders<CourseInfoEntity>.Update
                    .Set(x => x.Name, updateData.Name)
                    .Set(x => x.CourseIntro, updateData.CourseIntro)
                    .Set(x => x.TeacherCode, updateData.TeacherCode)
                    .Set(x => x.StartTime, updateData.StartTime)
                    .Set(x => x.Duration, updateData.Duration)
                    .Set(x => x.CourseType, updateData.CourseType)
                    .Set(x => x.AudienceType, updateData.AudienceType)
                    .Set(x => x.TeacherName, updateData.TeacherName)
                    .Set(x => x.CoursewareToken, updateData.CoursewareToken)
                    .Set(x => x.Poster, updateData.Cover)
                    .Set(x => x.ViewRange, updateData.ViewRange)
                    .Set(x => x.CaseLabelCodes, updateData.CaseLabelCodes)
                    .Set(x => x.CourseLabelCodes, updateData.CourseLabelCodes)
                    .Set(x => x.UserGroupCodes, updateData.UserGroupCodes)
                    .Set(x => x.Price, updateData.Price)
                    .Set(x => x.Experts, experts)
                    .Set(x => x.Assistants, assistants)
                    .Set(x => x.IsAgentCourse, updateData.IsAgentCourse)
                    .Set(x => x.CourseAppearType, updateData.CourseAppearType)
                    .Set(x => x.CourseAlbumCodes, updateData.CourseAlbumCodes)
                    .Set(x => x.Sort, updateData.Sort)
                    .Set(x => x.IsStick, updateData.IsStick)
                    .Set(c => c.UpdateTime, DateTime.UtcNow)
                    .Set(c => c.CourseVideoCodes, updateData.CourseVideoCodes)
                    .Set(c => c.IsSmallClass, updateData.IsSmallClass)
                    .Set(c => c.CourseMaterialCodes, updateData.CourseMaterialCodes)
                    .Set(c => c.IsJustVideoCourse, updateData.IsJustVideoCourse);
                    if (findEntity.Status == CourseStatusEnum.RejectApproval)
                    {
                        update = update.Set(c => c.Status, CourseStatusEnum.NoApproval);
                    }
                    //.Set(c => c.BindExams, updateData.BindExams);
                    res = await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                    if (res)
                    {
                        var removeList = courseAlbumList?.ToList()?.Select(c => c.Code).ToList();
                        await UpdateAlbumCourseListDBAsync(request.Data.CourseAlbumCodes, removeList, findEntity.Code);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 已报名课程或者专辑人数
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<int> FindSignInListCountAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindSignInListCountAsync execute", "DatabaseService"))
            {
                var builder = Builders<SignStudentEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.CourseOrAlbumCode == code);
                var result = await _signStudentDBRepository.FindAllAsync(filter);
                return result.Count;
            }
        }
        /// <summary>
        /// 查询学生状态信息
        /// </summary>
        /// <param name="courseOrAlbumCodes"></param>
        /// <returns></returns>
        public async Task<IList<SignStudentDTO>> FindSignInListAsync(IList<string> courseOrAlbumCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindSignInListAsync execute", "DatabaseService"))
            {
                IList<SignStudentDTO> list = new List<SignStudentDTO>();
                var builder = Builders<SignStudentEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (courseOrAlbumCodes.Count > 0)
                {
                    var newFilter = builder.In(c => c.CourseOrAlbumCode, courseOrAlbumCodes);
                    filter = filter & newFilter;
                    var result = await _signStudentDBRepository.FindAllAsync(filter);
                    if (result != null && result.Count > 0)
                    {
                        foreach (var studentInfo in result)
                        {
                            list.Add(new SignStudentDTO
                            {
                                StudentCode = studentInfo.StudentCode,
                                CourseOrAlbumCode = studentInfo.CourseOrAlbumCode,
                                CourseAppearType = studentInfo.CourseAppearType,
                                IsPay = studentInfo.IsPay,
                                SignCourseStatus = studentInfo.SignCourseStatus
                            });
                        }
                    }
                }
                return list;
            }
        }

        /// <summary>
        /// 查询报名学生列表通过课程或者专辑codes
        /// </summary>
        /// <param name="courseOrAlbumCodes"></param>
        /// <returns></returns>
        private async Task<IList<SignStudentEntity>> FindSignInListDBByCourseCodesAsync(IList<string> courseOrAlbumCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindSignInListDBByCourseCodesAsync execute", "DatabaseService"))
            {
                IList<SignStudentEntity> list = new List<SignStudentEntity>();
                var builder = Builders<SignStudentEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (courseOrAlbumCodes.Count > 0)
                {
                    var newFilter = builder.In(c => c.CourseOrAlbumCode, courseOrAlbumCodes);
                    filter = filter & newFilter;
                    list = await _signStudentDBRepository.FindAllAsync(filter);
                }
                return list;
            }
        }

        /// <summary>
        /// 查询学生课程列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<MyCoursePageDTO>> FindMyCoursePagesDBAsync(FindMyCoursePagesDBRequest request)
        {
            var userCode = request.StudentCode;
            var queryStatus = request.QueryStatus;
            var queryCourseType = request.CourseAppearType;
            var builder = Builders<SignStudentEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.StudentCode == userCode);
            if ((int)queryCourseType > 0)
            {
                var queryCourseTypeFilter = builder.Eq(c => c.CourseAppearType, queryCourseType);
                filter = filter & queryCourseTypeFilter;
            }
            request.Keyword = ReplaceSpecialCharacter(request.Keyword);
            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {
                var subFilter1 = Builders<CourseInfoEntity>.Filter.Or(
                    Builders<CourseInfoEntity>.Filter.Regex(f => f.Name, new BsonRegularExpression($".*{request.Keyword}.*", "ig")),
                    Builders<CourseInfoEntity>.Filter.Regex(f => f.TeacherName, new BsonRegularExpression($".*{request.Keyword}.*", "ig"))
                );
                List<string> codeList = new List<string>();
                var courses = await _courseInfoDBRepository.FindAllAsync(subFilter1, new FindOptions<CourseInfoEntity>
                {
                    Projection = Builders<CourseInfoEntity>.Projection.Include(c => c.Code)
                });
                if (courses != null && courses.Count > 0)
                {
                    var courseCodes = courses.Select(s => s.Code).ToList();
                    codeList.AddRange(courseCodes);
                }
                var subFilter2 = Builders<CourseAlbumEntity>.Filter.Or(
                    Builders<CourseAlbumEntity>.Filter.Regex(f => f.Name, new BsonRegularExpression($".*{request.Keyword}.*", "ig")),
                    Builders<CourseAlbumEntity>.Filter.Regex(f => f.TeacherName, new BsonRegularExpression($".*{request.Keyword}.*", "ig"))
                );
                var courseAlbums = await _courseAlbumDBRepository.FindAllAsync(subFilter2, new FindOptions<CourseAlbumEntity>
                {
                    Projection = Builders<CourseAlbumEntity>.Projection.Include(c => c.Code)
                });
                if (courseAlbums != null && courseAlbums.Count > 0)
                {
                    var courseAlbumCodes = courseAlbums.Select(s => s.Code).ToList();
                    codeList.AddRange(courseAlbumCodes);
                }
                if (codeList.Count > 0)
                {
                    var newFilter = builder.In(c => c.CourseOrAlbumCode, codeList);
                    filter = filter & newFilter;
                }
                else
                {
                    var newFilter = builder.Eq(c => c.CourseOrAlbumCode, "");
                    filter = filter & newFilter;
                }
            }
            if ((int)queryStatus > 0)
            {
                var queryStatusFilter = builder.Eq(c => c.SignCourseStatus, queryStatus);
                filter = filter & queryStatusFilter;
            }
            var sort = Builders<SignStudentEntity>.Sort.Descending(v => v.CreateTime);
            var dbPage = new DBPageRequest<SignStudentEntity>();
            dbPage.CurrentPage = 1;
            dbPage.PageSize = 1000;
            dbPage.Filter = filter;
            dbPage.Sort = sort;
            var pageResult = await _signStudentDBRepository.GetPages(dbPage);

            return PageResultConvert<SignStudentEntity, MyCoursePageDTO>(pageResult);
        }

        /// <summary>
        /// 更新课程状态记录
        /// </summary>
        /// <param name="data"></param>
        /// <param name="canNull"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseStatusAsync(UpdateCourseStatusDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseStatusAsync execute", "DatabaseService"))
            {
                var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var update = Builders<CourseInfoEntity>.Update
                    .Set(c => c.Status, request.Status)
                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                    if (request.Status == CourseStatusEnum.Started)
                    {
                        update = update.Set(c => c.ActualStartTime, DateTime.UtcNow);
                        var browsedRecordDBReq = new CourseBrowsedRecordRequest()
                        {
                            RelevanceCode = findEntity.Code,
                            UserCode = findEntity.TeacherCode,
                            DisplayName = findEntity.TeacherName,
                            IsOnline = false,
                            BrowsedUserType = BrowsedUserTypeEnum.Teacher
                        };
                        await SaveCourseBrowsedRecordByDBAsync(browsedRecordDBReq);
                    }
                    if (request.Status == CourseStatusEnum.HasEnded)
                    {
                        update = update.Set(c => c.ActualEndTime, DateTime.UtcNow);
                        await GetCourseBrowsedStatisticsDataAsync(findEntity, false);
                        update = update.Set(c => c.StatisticsInfo, findEntity.StatisticsInfo);
                    }
                    if (request.Status == CourseStatusEnum.Approved)
                    {
                        if (findEntity.CourseChannel == null && request.CourseChannel != null)
                        {
                            update = update.Set(c => c.CourseChannel, request.CourseChannel.MappingTo<LiveDataDO>());
                        }
                        if (findEntity.OthersScreenSharingChannel == null && request.OthersScreenSharingChannel != null)
                        {
                            update = update.Set(c => c.OthersScreenSharingChannel, request.OthersScreenSharingChannel.MappingTo<LiveDataDO>());
                        }
                    }
                    if (request.RtcScreenSharingChannel != null)
                    {
                        update = update.Set(c => c.RtcScreenSharingChannel, request.RtcScreenSharingChannel.MappingTo<ScreenSharingChannelDO>());
                    }
                    var res = await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                    return res;
                }
                return false;
            }
        }

        /// <summary>
        /// 创建视频
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateVideoDBAsync(SaveVideoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateVideoDBAsync execute", "DatabaseService"))
            {
                var entity = request.MappingTo<VideoInfoEntity>();
                if (request.AutoCreateCourse)
                {
                    request.Category = 1;
                    //创建课程
                    var courseDB = new CreateCourseDBRequest()
                    {
                        Data = new CourseInfoDBRequest() 
                        {
                            Name = request.Name,
                            CourseIntro = request.Intro,
                            TeacherCode = request.CreatorCode,
                            TeacherName = request.CreatorName,
                            Cover = request.Poster,
                            Duration = request.Duration,
                            StartTime = DateTime.UtcNow,
                            CourseAppearType = CourseAppearTypeEnum.Independent,
                            CourseType = CourseTypeEnum.Multimedia,
                            AudienceType = request.AudienceType,
                            Status = CourseStatusEnum.Approved,
                            ViewRange = request.ViewRange,
                            CreatorCode = request.CreatorCode,
                            UserGroupCodes = request.UserGroupCodes,                           
                            CourseLabelCodes = request.CourseLabelCodes,
                            IsJustVideoCourse = true
                        }
                    };
                    var coureseId = await CreateCourseAsync(courseDB);
                    request.CourseCode = coureseId;
                }
                entity.SrcFileToken = entity.VideoToken;
                var convertInfo = new ConvertPdfInfo();
                convertInfo.Entity = entity;
                convertInfo.CourseCode = request.CourseCode;
                convertInfo.Category = request.Category;
                _convertPdfExecutor.Add(DoConvertPdf, convertInfo);
                return await Task.FromResult(true);
            }
        }

        /// <summary>
        /// 更新素材名称
        /// </summary>
        /// <param name="videoCode">素材编码</param>
        /// <param name="newName">新名称</param>
        /// <returns></returns>
        public async Task<bool> ChangeVideoNameDBAsync(string videoCode, string newName)
        {
            using (new Performance(_performanceThreshold, "{0} ChangeVideoDBNameAsync execute", "DatabaseService"))
            {
                var filter = Builders<VideoInfoEntity>.Filter.Eq(f => f.Code, videoCode);
                var update = Builders<VideoInfoEntity>.Update.Set(f => f.Name, newName);
                return await _videoInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            }
        }

        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="fileInfo"></param>
        /// <returns></returns>
        private bool DoConvertPdf(ConvertPdfInfo convertInfo)
        {
            try
            {
                string tempFilePath = string.Empty, outFilePath = string.Empty, imageFilePath = string.Empty;
                if (convertInfo.Entity.FodderType == UploadFileTypeEnum.PPTX)
                {
                    var downloadFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Temp", "Downloads");
                    if (!Directory.Exists(downloadFolder))
                    {
                        Directory.CreateDirectory(downloadFolder);
                    }
                    tempFilePath = DownloadAsync(downloadFolder, convertInfo.Entity.SrcFileToken).Result;
                    if (File.Exists(tempFilePath))
                    {
                        outFilePath = tempFilePath.Replace(".pptx", ".pdf");
                        imageFilePath = tempFilePath.Replace(".pptx", ".jpeg");
                        //Open the file as Stream.
                        using (FileStream docStream = new FileStream(tempFilePath, FileMode.Open, FileAccess.Read))
                        {
                            //Loads file stream into Word document.
                            using (var pptxDoc = Presentation.Open(docStream))
                            {
                                PresentationToPdfConverterSettings pdfConverterSettings = new PresentationToPdfConverterSettings();
                                //Enable the portable rendering
                                pdfConverterSettings.ShowHiddenSlides = true;
                                pdfConverterSettings.EmbedFonts = true;
                                pdfConverterSettings.EmbedCompleteFonts = true;
                                //Use a sets of default FallbackFont collection to IPresentation
                                pptxDoc.FontSettings.InitializeFallbackFonts();
                                //Converts Word document into PDF document
                                using (PdfDocument pdfDoc = PresentationToPdfConverter.Convert(pptxDoc, pdfConverterSettings))
                                {
                                    using (FileStream outStream = new FileStream(outFilePath, FileMode.OpenOrCreate, FileAccess.Write))
                                    {
                                        pdfDoc.EnableMemoryOptimization = true;
                                        pdfDoc.Compression = PdfCompressionLevel.Best;
                                        //Save the PDF file
                                        pdfDoc.Save(outStream);
                                    }
                                    //Load the first page
                                    using (var image = pptxDoc.Slides[0].ConvertToImage(ExportImageFormat.Jpeg))
                                    {
                                        using (BinaryReader reader = new BinaryReader(image))
                                        {
                                            var data = reader.ReadBytes((int)image.Length);
                                            using (var imageStream = new FileStream(imageFilePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                                            {
                                                imageStream.Write(data, 0, data.Length);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (convertInfo.Entity.FodderType == UploadFileTypeEnum.PDF)
                {
                    var downloadFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Temp", "Downloads");
                    if (!Directory.Exists(downloadFolder))
                    {
                        Directory.CreateDirectory(downloadFolder);
                    }
                    tempFilePath = DownloadAsync(downloadFolder, convertInfo.Entity.SrcFileToken).Result;
                    if (File.Exists(tempFilePath))
                    {
                        if (string.IsNullOrWhiteSpace(imageFilePath))
                        {
                            imageFilePath = Path.ChangeExtension(tempFilePath, "jpg");
                        }
                        using (var imageStream = new FileStream(imageFilePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                        {
                            using var pdfDocument = new PDFiumSharp.PdfDocument(tempFilePath);
                            var firstPage = pdfDocument.Pages[0];
                            using var pageBitmap = new PDFiumSharp.PDFiumBitmap(400, 300, true);
                            firstPage.Render(pageBitmap);
                            var image = SKBitmap.Decode(pageBitmap.AsBmpStream());
                            image.Encode(imageStream, SKEncodedImageFormat.Jpeg, 0);
                        }
                    }
                }
                if (File.Exists(imageFilePath))
                {
                    string fileToken = UpLoadFile(imageFilePath);
                    if (!string.IsNullOrWhiteSpace(fileToken))
                    {
                        convertInfo.Entity.Poster = fileToken;
                    }
                }
                if (File.Exists(outFilePath))
                {
                    string fileToken = UpLoadFile(outFilePath);
                    if (string.IsNullOrWhiteSpace(fileToken)) return false;
                    convertInfo.Entity.VideoToken = fileToken;
                }
                var result = _videoInfoDBRepository.InsertOneAsync(convertInfo.Entity).Result;
                if (!string.IsNullOrEmpty(result))
                {
                    if (!string.IsNullOrEmpty(convertInfo.CourseCode))
                    {
                        var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == convertInfo.CourseCode && c.IsDelete == false);
                        var findEntity = _courseInfoDBRepository.FindOneAsync(filter).Result;
                        if (!string.IsNullOrEmpty(findEntity?.Name))
                        {
                            if (convertInfo.Category == 1)
                            {
                                if (findEntity.CourseVideoCodes == null)
                                {
                                    findEntity.CourseVideoCodes = new List<string>();
                                }
                                findEntity.CourseVideoCodes.Add(result);
                            }
                            else if (convertInfo.Category == 2)
                            {
                                if (findEntity.CourseMaterialCodes == null)
                                {
                                    findEntity.CourseMaterialCodes = new List<string>();
                                }
                                findEntity.CourseMaterialCodes.Add(result);
                            }
                            var res = _courseInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity).Result;
                            return res;
                        }
                    }
                    if (File.Exists(tempFilePath)) File.Delete(tempFilePath);
                    if (File.Exists(outFilePath)) File.Delete(outFilePath);
                    if (File.Exists(outFilePath)) File.Delete(imageFilePath);
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"DoConvertPdf error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 上传文件（不分块上传）
        /// </summary>
        /// <param name="filePath"></param>
        /// <returns></returns>
        private string UpLoadFile(string filePath)
        {
            if (File.Exists(filePath))
            {
                var uploadFile = GetFileTransferByStorageAsync(filePath, isNeedPartUpload: false).GetAwaiter().GetResult();
                if (uploadFile == null || uploadFile.Count <= 0)
                {
                    Logger.WriteLineInfo($"Upload File {filePath} Fail");
                    return "";
                }
                else
                {
                    return uploadFile.FirstOrDefault();
                }
            }
            return "";
        }

        /// <summary>
        /// 下载文件
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        private async Task<string> DownloadAsync(string downloadFolder, string url)
        {
            if (string.IsNullOrEmpty(url))
            {
                throw new NoNullAllowedException("url");
            }
            if (!Directory.Exists(downloadFolder))
            {
                Directory.CreateDirectory(downloadFolder);
            }
            var fileName = Path.GetFileName(url);
            var targetPath = Path.Combine(downloadFolder, fileName);
            long fileSize = 0;
            using (var request = new HttpRequestMessage())
            {
                request.RequestUri = new Uri(url);
                request.Method = HttpMethod.Get;
                var response = await _httpClient.SendAsync(request);
                if (response != null && response.StatusCode == HttpStatusCode.OK)
                {
                    var contentLength = response.Content.Headers.ContentLength;
                    fileSize = contentLength == null ? 0 : contentLength.Value;
                }
            }
            if (fileSize <= 0)
            {
                throw new NotSupportedException($"fileSize is {fileSize}");
            }
            byte[] bytes = await _httpClient.GetByteArrayAsync(url);
            File.WriteAllBytes(targetPath, bytes);
            return targetPath;
        }

        /// <summary>
        /// 根据视频codes查询所属视频
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<IList<VideoInfoDTO>> FindVideosByCodesDBAsync(IList<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} FindVideosByCodesDBAsync execute", "DatabaseService"))
            {
                var resultList = new List<VideoInfoDTO>();
                if (codes?.Count > 0)
                {
                    var filter = Builders<VideoInfoEntity>.Filter.Where(c => c.IsDelete == false);
                    filter = filter & Builders<VideoInfoEntity>.Filter.In(c => c.Code, codes);
                    var videoList = await _videoInfoDBRepository.FindAllAsync(filter);
                    resultList = videoList.MappingTo<List<VideoInfoDTO>>();
                }
                return resultList;
            }
        }

        /// <summary>
        /// 根据课程code查询所属视频
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<VideoInfoDTO>> FindVideosByCourseCodeDBAsync(FindVideosByCourseCodeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindVideosByCourseCodeDBAsync execute", "DatabaseService"))
            {
                var resultList = new List<VideoInfoDTO>();
                if (string.IsNullOrEmpty(request.CourseCode))
                {
                    var filter = Builders<VideoInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
                    var videoEntity = await _videoInfoDBRepository.FindOneAsync(filter);
                    resultList.Add(videoEntity.MappingTo<VideoInfoDTO>());
                }
                else
                {
                    var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.CourseCode && c.IsDelete == false);
                    var courseEntity = await _courseInfoDBRepository.FindOneAsync(courseFilter);
                    if (courseEntity?.CourseVideoCodes?.Count > 0)
                    {
                        var videoFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.IsDelete == false);
                        var inFilter = Builders<VideoInfoEntity>.Filter.In(c => c.Code, courseEntity.CourseVideoCodes);
                        videoFilter = videoFilter & inFilter;
                        var dataList = await _videoInfoDBRepository.FindAllAsync(videoFilter);
                        resultList.AddRange(dataList.MappingTo<List<VideoInfoDTO>>());
                    }
                }
                return resultList;
            }
        }

        /// <summary>
        /// 根据课程类型查询课程文件分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideoFilesCourseTypeByPageDBAsync(FindVideoFilesCourseTypeByPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindVideoFilesCourseTypeByPageDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<VideoInfoEntity>.Filter;
                var userCode = request.UserCode;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    var teacherFilter = builder.Regex(c => c.CreatorName, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (nameFilter | teacherFilter);
                }
                if (request.FileTypes.Count > 0)
                {
                    var fodderTypeFilter = builder.Where(c => request.FileTypes.Contains(c.FodderType));
                    filter = filter & fodderTypeFilter;
                }
                //自己上传的
                var teacherOtherFilter = builder.Where(c => c.CreatorCode == userCode);
                //其他人公共的开放的
                var publicOtherFilter = builder.Where(c => c.CreatorCode != userCode && c.IsPublic == true);
                filter = filter & (teacherOtherFilter | publicOtherFilter);
                var dbPage = new DBPageRequest<VideoInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _videoInfoDBRepository.GetPages(dbPage);
                return PageResultConvert<VideoInfoEntity, VideoInfoDTO>(pageResult);
            }
        }
        /// <summary>
        /// 查询所有视频列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideosPagesDBAsync(FindVideoPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindVideosPagesDBAsync execute", $"{this.GetType().Name}"))
            {

                var builder = Builders<VideoInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    var teacherFilter = builder.Regex(c => c.CreatorName, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (nameFilter | teacherFilter);
                }
                if (!string.IsNullOrEmpty(request.CourseCode))
                {
                    var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.CourseCode && c.IsDelete == false);
                    var courseEntity = await _courseInfoDBRepository.FindOneAsync(courseFilter);
                    if (request.Category == 1)
                    {
                        if (courseEntity?.CourseVideoCodes?.Count > 0)
                        {
                            var inFilter = Builders<VideoInfoEntity>.Filter.In(c => c.Code, courseEntity.CourseVideoCodes);
                            filter = filter & inFilter;
                        }
                        else
                        {
                            var notFindFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.Code == Guid.NewGuid().ToString());
                            filter = filter & notFindFilter;
                        }
                    }
                    else if (request.Category == 2)
                    {
                        if (courseEntity?.CourseMaterialCodes?.Count > 0)
                        {
                            var inFilter = Builders<VideoInfoEntity>.Filter.In(c => c.Code, courseEntity.CourseMaterialCodes);
                            filter = filter & inFilter;
                        }
                        else
                        {
                            var notFindFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.Code == Guid.NewGuid().ToString());
                            filter = filter & notFindFilter;
                        }
                    }
                }
                if (((int)request.FodderType) > 0)
                {
                    var fodderTypeFilter = builder.Where(c => c.FodderType == request.FodderType);
                    filter = filter & fodderTypeFilter;
                }
                var teacherCode = "";
                var tokens = CacheMaintenance.Instance.Get<ITokensManager>().Where(cb => cb.Code == request.Token && cb.AccountType == 1);
                if (tokens?.Count > 0)
                {
                    teacherCode = tokens.FirstOrDefault()?.ClientId ?? string.Empty;
                }
                if (!string.IsNullOrEmpty(teacherCode))
                {
                    if (request.IsJustQueryMyUpload)
                    {
                        var teacherFilter = builder.Where(c => c.CreatorCode == teacherCode);
                        filter = filter & teacherFilter;
                    }
                    else
                    {
                        //自己上传的
                        var teacherOtherFilter = builder.Where(c => c.CreatorCode == teacherCode);
                        //其他人公共的开放的
                        var publicOtherFilter = builder.Where(c => c.CreatorCode != teacherCode && c.IsPublic == true);
                        filter = filter & (teacherOtherFilter | publicOtherFilter);
                    }
                }
                var dbPage = new DBPageRequest<VideoInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _videoInfoDBRepository.GetPages(dbPage);
                return PageResultConvert<VideoInfoEntity, VideoInfoDTO>(pageResult);
            }
        }

        /// <summary>
        /// 管理端查询所有视频列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideosPagesByManagementDBAsync(FindVideoPagesManagementDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindVideosPagesByManagementDBAsync execute", $"{this.GetType().Name}"))
            {

                var builder = Builders<VideoInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    var teacherFilter = builder.Regex(c => c.CreatorName, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (nameFilter | teacherFilter);
                }
                if (!string.IsNullOrEmpty(request.CourseCode))
                {
                    var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.CourseCode && c.IsDelete == false);
                    var courseEntity = await _courseInfoDBRepository.FindOneAsync(courseFilter);
                    if (request.Category == 1)
                    {
                        if (courseEntity?.CourseVideoCodes?.Count > 0)
                        {
                            var inFilter = Builders<VideoInfoEntity>.Filter.In(c => c.Code, courseEntity.CourseVideoCodes);
                            filter = filter & inFilter;
                        }
                        else
                        {
                            var notFindFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.Code == Guid.NewGuid().ToString());
                            filter = filter & notFindFilter;
                        }
                    }
                    else if (request.Category == 2)
                    {
                        if (courseEntity?.CourseMaterialCodes?.Count > 0)
                        {
                            var inFilter = Builders<VideoInfoEntity>.Filter.In(c => c.Code, courseEntity.CourseMaterialCodes);
                            filter = filter & inFilter;
                        }
                        else
                        {
                            var notFindFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.Code == Guid.NewGuid().ToString());
                            filter = filter & notFindFilter;
                        }
                    }
                }
                if (((int)request.FodderType) > 0)
                {
                    var fodderTypeFilter = builder.Where(c => c.FodderType == request.FodderType);
                    filter = filter & fodderTypeFilter;
                }
                var teacherCode = "";
                var tokens = CacheMaintenance.Instance.Get<ITokensManager>().Where(cb => cb.Code == request.Token && cb.AccountType == 1);
                if (tokens?.Count > 0)
                {
                    teacherCode = tokens.FirstOrDefault()?.ClientId ?? string.Empty;
                }
                if (!string.IsNullOrEmpty(teacherCode))
                {
                    if (request.IsJustQueryMyUpload)
                    {
                        var teacherFilter = builder.Where(c => c.CreatorCode == teacherCode);
                        filter = filter & teacherFilter;
                    }
                    else
                    {
                        //自己上传的
                        var teacherOtherFilter = builder.Where(c => c.CreatorCode == teacherCode);
                        //其他人公共的开放的
                        var publicOtherFilter = builder.Where(c => c.CreatorCode != teacherCode && c.IsPublic == true);
                        filter = filter & (teacherOtherFilter | publicOtherFilter);
                    }
                }
                var dbPage = new DBPageRequest<VideoInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _videoInfoDBRepository.GetPages(dbPage);
                return PageResultConvert<VideoInfoEntity, VideoInfoDTO>(pageResult);
            }
        }

        /// <summary>
        /// 删除视频，同步删除课程管理视频code
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteVideoByCodeDBAsync(DeleteVideoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteVideoByCodeDBAsync execute", "DatabaseService"))
            {
                var result = await _videoInfoDBRepository.DeleteOneAsync("Code", request.Code);
                if (result)
                {
                    if (!string.IsNullOrEmpty(request.CourseCode))
                    {
                        var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.CourseCode && c.IsDelete == false);
                        var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                        if (findEntity?.CourseVideoCodes?.Count > 0)
                        {
                            if (findEntity.CourseVideoCodes.Contains(request.Code))
                            {
                                findEntity.CourseVideoCodes.Remove(request.Code);
                            }
                            var res = await _courseInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                            return res;
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseBindExamsAsync(UpdateCourseBindExamsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseAsync execute", "DatabaseService"))
            {
                var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.CourseCode && c.IsDelete == false);
                var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var bindExamEntities = request.BindExams?.MappingTo<List<CourseExaminationEntity>>();
                    var update = Builders<CourseInfoEntity>.Update
                    .Set(c => c.UpdateTime, DateTime.UtcNow)
                    .Set(c => c.BindExams, bindExamEntities);
                    return await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return false;
            }
        }

        /// <summary>
        /// 学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddStudentInCourseUserGroupDBAsync(AddStudentInCourseUserGroupDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.UserGroupCode))
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            if (request.StudentCodes == null || request.StudentCodes.Count == 0)
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupStudentsIsEmpty, "UserGroupStudentsIsEmpty");
            }
            var filter = Builders<UserGroupEntity>.Filter.Eq(f => f.Code, request.UserGroupCode);
            var groupInfo = await _userGroupDBRepository.FindOneAsync(filter);
            if (groupInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupNotExists, "UserGroupNotExists");
            }
            UpdateDefinition<UserGroupEntity> update = null;
            var learnerList = request.StudentCodes.Select(c => new LearnerEntity()
            {
                Code = c,
                LearnerStatus = LearnerStatusEnum.NoApproval
            }).ToList();
            if (groupInfo.Students?.Any() == true)
            {
                if (request.StudentCodes != null && request.StudentCodes.Count > 0)
                {
                    update = Builders<UserGroupEntity>.Update.PullFilter(p => p.Students, Builders<LearnerEntity>.Filter.In(f => f.Code, request.StudentCodes));
                    await _userGroupDBRepository.UpdateOneAsync(filter, update);
                    groupInfo.Students = groupInfo.Students.Where(s => !request.StudentCodes.Contains(s.Code)).Distinct(new LearnerEntityComparer()).ToList();
                }
                foreach (var item in learnerList)
                {
                    if (!groupInfo.Students.ToList().Exists(c => c.Code == item.Code))
                    {
                        groupInfo.Students.Add(item);
                    }
                }
                update = Builders<UserGroupEntity>.Update.Set(p => p.Students, groupInfo.Students);
            }
            else
            {
                update = Builders<UserGroupEntity>.Update.AddToSetEach(p => p.Students, learnerList);
            }
            return await _userGroupDBRepository.UpdateOneAsync(filter, update) > 0;
        }

        // 实现一个比较器来比较LearnerEntity对象是否相等
        public class LearnerEntityComparer : IEqualityComparer<LearnerEntity>
        {
            public bool Equals(LearnerEntity x, LearnerEntity y)
            {
                // 比较Code和LearnerStatus属性是否相等
                return x.Code == y.Code;
            }

            public int GetHashCode(LearnerEntity obj)
            {
                // 计算Code和LearnerStatus属性的哈希码
                return obj.Code.GetHashCode();
            }
        }


        /// <summary>
        /// 学生退出班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> StudentLeaveCourseUserGroupDBAsync(StudentLeaveCourseUserGroupDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.UserGroupCode))
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            if (request.StudentCodes == null || request.StudentCodes.Count == 0)
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupStudentsIsEmpty, "UserGroupStudentsIsEmpty");
            }
            var filter = Builders<UserGroupEntity>.Filter.Eq(f => f.Code, request.UserGroupCode);
            var groupInfo = await _userGroupDBRepository.FindOneAsync(filter);
            if (groupInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserGroupNotExists, "UserGroupNotExists");
            }
            UpdateDefinition<UserGroupEntity> update = null;
            if (groupInfo.Students?.Any() == true)
            {
                var pullFilter = Builders<LearnerEntity>.Filter.In(e => e.Code, request.StudentCodes);
                update = Builders<UserGroupEntity>.Update.PullFilter(f => f.Students, pullFilter);
            }
            return await _userGroupDBRepository.UpdateOneAsync(filter, update) > 0;
        }

        /// <summary>
        /// 学习结束处理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> StudyCompletedDBAsync(StudyCompletedDBRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.CourseCode))
            {
                ThrowCustomerException(CustomerRpcCode.CourseCodeEmpty, "CourseCodeEmpty");
            }
            if (string.IsNullOrWhiteSpace(request.StudentCode))
            {
                ThrowCustomerException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var filter = Builders<CourseInfoEntity>.Filter.Eq(f => f.Code, request.CourseCode);
            var courseInfo = await _courseInfoDBRepository.FindOneAsync(filter);
            if (courseInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.CourseNoFound, "CourseNoFound");
            }
            var studentFilter = Builders<SignStudentEntity>.Filter.Where(w => w.StudentCode == request.StudentCode && w.CourseOrAlbumCode == request.CourseCode && w.IsDelete == false);
            var signStudent = await _signStudentDBRepository.FindOneAsync(studentFilter);
            if (signStudent != null)
            {
                if (signStudent.SignCourseStatus != StudentCourseStatusEnum.Ended && signStudent.SignCourseStatus != StudentCourseStatusEnum.NoSignUp)
                {
                    UpdateDefinition<SignStudentEntity> subUpdate = Builders<SignStudentEntity>.Update.Set(p => p.SignCourseStatus, StudentCourseStatusEnum.Ended);
                    return await _signStudentDBRepository.UpdateOneAsync(studentFilter, subUpdate) > 0;
                }
            }
            return false;
        }

        /// <summary>
        /// 添加课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddCourseAlbumDBAsync(AddCourseAlbumDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateCourseAsync execute", "DatabaseService"))
            {
                var entity = request.Data.MappingTo<CourseAlbumEntity>();
                entity.CreateTime = DateTime.UtcNow;
                var courseAlbumCode = await _courseAlbumDBRepository.InsertOneAsync(entity);
                return courseAlbumCode;
            }
        }

        /// <summary>
        /// 根据code查询专辑列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CourseAlbumDTO>> FindAllAlbumListByAlbumCodesDBAsync(IList<string> courseAlbumCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseAlbumListByCourseCodeDBAsync execute", "DatabaseService"))
            {
                IList<CourseAlbumDTO> list = new List<CourseAlbumDTO>();
                var builder = Builders<CourseAlbumEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var secondFilter = builder.In(c => c.Code, courseAlbumCodes);
                filter = filter & secondFilter;
                var findEntity = await _courseAlbumDBRepository.FindAllAsync(filter);
                if (findEntity != null)
                {
                    var sort = findEntity.OrderByDescending(v => v.IsStick).ThenByDescending(v => v.Sort).ThenByDescending(v => v.UpdateTime).ToList();
                    list = sort.MappingTo<List<CourseAlbumDTO>>();
                }
                return list;
            }
        }

        /// <summary>
        /// 更新专辑中的课程列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<bool> UpdateAlbumCourseListDBAsync(IList<string> addCourseAlbumCodes, IList<string> removeCourseAlbumCodes, string courseCode)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateAlbumCourseListDBAsync execute", "DatabaseService"))
            {
                if (removeCourseAlbumCodes?.Count > 0)
                {
                    foreach (var removeCourseAlbumCode in removeCourseAlbumCodes)
                    {
                        var removeFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.Code == removeCourseAlbumCode && c.IsDelete == false);
                        var removeEntity = await _courseAlbumDBRepository.FindOneAsync(removeFilter);
                        if (removeEntity != null)
                        {
                            if (removeEntity.CourseCodes.Contains(courseCode))
                            {
                                removeEntity.CourseCodes.Remove(courseCode);
                                //更新db
                                var update = Builders<CourseAlbumEntity>.Update
                                    .Set(x => x.CourseCodes, removeEntity.CourseCodes)
                                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                                var removeResult = await _courseAlbumDBRepository.UpdateOneAsync(removeFilter, update) > 0;
                            }
                        }
                    }
                }
                if (addCourseAlbumCodes?.Count > 0)
                {
                    foreach (var addCourseAlbumCode in addCourseAlbumCodes)
                    {
                        var addFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.Code == addCourseAlbumCode && c.IsDelete == false);
                        var addEntity = await _courseAlbumDBRepository.FindOneAsync(addFilter);
                        if (addEntity != null)
                        {
                            if (!addEntity.CourseCodes.Contains(courseCode))
                            {
                                addEntity.CourseCodes.Add(courseCode);
                                //更新db
                                var update = Builders<CourseAlbumEntity>.Update
                                    .Set(x => x.CourseCodes, addEntity.CourseCodes)
                                    .Set(c => c.UpdateTime, DateTime.UtcNow);
                                var addResult = await _courseAlbumDBRepository.UpdateOneAsync(addFilter, update) > 0;
                            }
                        }
                    }
                }
                return true;
            }
        }

        /// <summary>
        /// 根据课程code查询所有的专辑列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CourseAlbumDTO>> GetCourseAlbumListByCourseCodeDBAsync(string courseCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseAlbumListByCourseCodeDBAsync execute", "DatabaseService"))
            {
                IList<CourseAlbumDTO> list = new List<CourseAlbumDTO>();
                var builder = Builders<CourseAlbumEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var secondFilter = builder.AnyIn(c => c.CourseCodes, new List<string>() { courseCode });
                filter = filter & secondFilter;
                var findEntity = await _courseAlbumDBRepository.FindAllAsync(filter);
                if (findEntity != null)
                {
                    var sort = findEntity.OrderByDescending(v => v.IsStick).ThenByDescending(v => v.Sort).ThenByDescending(v => v.UpdateTime).ToList();
                    list = sort.MappingTo<List<CourseAlbumDTO>>();
                }
                return list;
            }
        }

        /// <summary>
        /// 根据讲师code查询所有的专辑列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CourseAlbumDTO>> GetCourseAlbumListByTeacherCodeDBAsync(string teacherCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseAlbumListByTeacherCodeDBAsync execute", "DatabaseService"))
            {
                IList<CourseAlbumDTO> list = new List<CourseAlbumDTO>();
                var builder = Builders<CourseAlbumEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(teacherCode))
                {
                    var teacherFilter = builder.Where(c => c.TeacherCode == teacherCode);
                    filter = filter & teacherFilter;
                }
                var findEntity = await _courseAlbumDBRepository.FindAllAsync(filter);
                if (findEntity != null)
                {
                    var sort = findEntity.OrderByDescending(v => v.IsStick).ThenByDescending(v => v.Sort).ThenByDescending(v => v.UpdateTime).ToList();
                    list = sort.MappingTo<List<CourseAlbumDTO>>();
                }
                return list;
            }
        }

        /// <summary>
        /// 更新课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseAlbumDBAsync(UpdateCourseAlbumDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseAsync execute", "DatabaseService"))
            {
                var filter = Builders<CourseAlbumEntity>.Filter.Where(c => c.Code == request.CourseAlbumCode && c.IsDelete == false);
                var findEntity = await _courseAlbumDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    if (request.AddCourseCodes != null && request.AddCourseCodes.Count > 0)
                    {
                        foreach (var addCode in request.AddCourseCodes)
                        {
                            if (!findEntity.CourseCodes.Contains(addCode))
                            {
                                findEntity.CourseCodes.Add(addCode);
                            }
                        }
                    }
                    if (request.RemoveCourseCodes != null && request.RemoveCourseCodes.Count > 0)
                    {
                        foreach (var addCode in request.RemoveCourseCodes)
                        {
                            if (findEntity.CourseCodes.Contains(addCode))
                            {
                                findEntity.CourseCodes.Remove(addCode);
                            }
                        }
                    }
                    if (request.AddCourseLabelCodes != null && request.AddCourseLabelCodes.Count > 0)
                    {
                        foreach (var addCode in request.AddCourseLabelCodes)
                        {
                            if (!findEntity.CourseLabelCodes.Contains(addCode))
                            {
                                findEntity.CourseLabelCodes.Add(addCode);
                            }
                        }
                    }
                    else
                    {
                        findEntity.CourseLabelCodes = new List<string>();
                    }
                    if (!string.IsNullOrWhiteSpace(request.Introduction))
                    {
                        findEntity.Introduction = request.Introduction;
                    }
                    if (!string.IsNullOrWhiteSpace(request.Cover))
                    {
                        findEntity.Cover = request.Cover;
                    }
                    if (request.IsClientSubmit)
                    {
                        var update = Builders<CourseAlbumEntity>.Update
                        .Set(x => x.CourseCodes, findEntity.CourseCodes)
                        .Set(x => x.Introduction, findEntity.Introduction)
                        .Set(x => x.Cover, findEntity.Cover)
                        .Set(x => x.CourseLabelCodes, findEntity.CourseLabelCodes)
                        .Set(x => x.Name, request.Name)
                        .Set(x => x.TeacherCode, request.TeacherCode)
                        .Set(x => x.TeacherName, request.TeacherName)
                        .Set(x => x.LanguageCode, request.LanguageCode)
                        .Set(x => x.Price, request.Price)
                        .Set(c => c.UpdateTime, DateTime.UtcNow);
                        return await _courseAlbumDBRepository.UpdateOneAsync(filter, update) > 0;
                    }
                    else
                    {
                        var update = Builders<CourseAlbumEntity>.Update
                        .Set(x => x.CourseCodes, findEntity.CourseCodes)
                        .Set(x => x.Introduction, findEntity.Introduction)
                        .Set(x => x.Cover, findEntity.Cover)
                        .Set(x => x.CourseLabelCodes, findEntity.CourseLabelCodes)
                        .Set(x => x.Name, request.Name)
                        .Set(x => x.TeacherCode, request.TeacherCode)
                        .Set(x => x.TeacherName, request.TeacherName)
                        .Set(x => x.ViewRange, request.ViewRange)
                        .Set(x => x.Price, request.Price)
                        .Set(x => x.Sort, request.Sort)
                        .Set(x => x.LanguageCode, request.LanguageCode)
                        .Set(x => x.IsStick, request.IsStick)
                        .Set(c => c.UpdateTime, DateTime.UtcNow);
                        return await _courseAlbumDBRepository.UpdateOneAsync(filter, update) > 0;
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 删除课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseAlbumDBAsync(DeleteCourseAlbumDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteCourseAlbumDBAsync execute", "DatabaseService"))
            {
                var filter = Builders<CourseAlbumEntity>.Filter.Where(c => c.Code == request.CourseAlbumCode && c.IsDelete == false);
                var findEntity = await _courseAlbumDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var update = Builders<CourseAlbumEntity>.Update.Set(x => x.IsDelete, true).Set(c => c.UpdateTime, DateTime.UtcNow);
                    return await _courseAlbumDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return false;
            }
        }

        /// <summary>
        /// 查找课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CourseAlbumDTO>> FindCourseAlbumPageDBAsync(FindCourseAlbumPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindCourseAlbumPageDBAsync execute", "DatabaseService"))
            {
                var pageResult = await GetCourseAlbumPageByCondition(request);
                var result = PageResultConvert<CourseAlbumEntity, CourseAlbumDTO>(pageResult);
                return result;
            }
        }

        /// <summary>
        /// 课程专辑分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<PageCollection<CourseAlbumEntity>> GetCourseAlbumPageByCondition(FindCourseAlbumPageDBRequest request)
        {
            var keyword = request.Keyword;
            var builder = Builders<CourseAlbumEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            //var userCode = request.TeacherCode;
            //var queryStatus = request.QueryStatus;
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var introductionFilter = builder.Regex(c => c.Introduction, new BsonRegularExpression(keyword, "i"));
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                var teacherNameFilter = builder.Regex(c => c.TeacherName, new BsonRegularExpression(keyword, "i"));
                filter = filter & (introductionFilter | nameFilter | teacherNameFilter);
            }
            if (request.CourseLabelCodes != null && request.CourseLabelCodes.Count > 0)
            {
                filter = filter & builder.Where(c => c.CourseLabelCodes.Any(w => request.CourseLabelCodes.Contains(w)));
            }
            if (!string.IsNullOrEmpty(request.TeacherCode))
            {
                filter = filter & builder.Where(c => c.TeacherCode == request.TeacherCode);
            }
            if (request.StartTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.CreateTime >= request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                filter = filter & builder.Where(c => c.CreateTime < request.EndTime.AddDays(1));
            }
            var sort = Builders<CourseAlbumEntity>.Sort.Descending(v => v.IsStick).Descending(v => v.Sort).Descending(v => v.UpdateTime);
            var dbPage = new DBPageRequest<CourseAlbumEntity>();
            dbPage.CurrentPage = request.PageIndex;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = filter;
            dbPage.Sort = sort;
            var pageResult = await _courseAlbumDBRepository.GetPages(dbPage);
            return pageResult;
        }

        /// <summary>
        /// 报名课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> SignUpCourseDBAsync(SignUpCourseDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SignUpCourseDBAsync execute", "DatabaseService"))
            {
                if (string.IsNullOrWhiteSpace(request.CourseOrAlbumCode))
                {
                    if (request.CourseAppearType == CourseAppearTypeEnum.Album)
                    {
                        ThrowCustomerException(CustomerRpcCode.CourseAlbumCodeEmpty, "CourseAlbum code is empty");
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
                    }
                }
                else
                {
                    if (request.CourseAppearType == CourseAppearTypeEnum.Album)
                    {
                        var courseAlbumInfo = await _courseAlbumDBRepository.FindOneAsync("Code", request.CourseOrAlbumCode);
                        if (courseAlbumInfo == null)
                        {
                            ThrowCustomerException(CustomerRpcCode.CourseNoFound, "CourseNoFound");
                        }
                    }
                    else
                    {
                        var courseInfo = await _courseInfoDBRepository.FindOneAsync("Code", request.CourseOrAlbumCode);
                        if (courseInfo != null)
                        {
                            if (courseInfo.Status == CourseStatusEnum.NoApproval || courseInfo.Status == CourseStatusEnum.RejectApproval)
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseNotReviewed, "CourseNotReviewed");
                            }
                            if (courseInfo.Experts != null && courseInfo.Experts.Count > 0 && courseInfo.Experts.Select(x => x.Code).Contains(request.StudentCode))
                            {
                                ThrowCustomerException(CustomerRpcCode.AssistantCantSignUp, "AssistantCantSignUp");
                            }
                            if (courseInfo.Assistants != null && courseInfo.Assistants.Count > 0 && courseInfo.Assistants.Select(x => x.Code).Contains(request.StudentCode))
                            {
                                ThrowCustomerException(CustomerRpcCode.AssistantCantSignUp, "AssistantCantSignUp");
                            }
                        }
                        else
                        {
                            ThrowCustomerException(CustomerRpcCode.CourseNoFound, "CourseNoFound");
                        }
                    }
                }
                var studentCode = request.StudentCode;
                if (string.IsNullOrWhiteSpace(request.StudentCode))
                {
                    ThrowCustomerException(CustomerRpcCode.StudentCodeEmpty, "StudentCodeEmpty");
                }
                var filter = Builders<SignStudentEntity>.Filter.Where(c => c.CourseOrAlbumCode == request.CourseOrAlbumCode && c.StudentCode == studentCode && c.IsDelete == false);
                var findEntity = await _signStudentDBRepository.FindOneAsync(filter);
                if (findEntity == null)
                {
                    var code = await _signStudentDBRepository.InsertOneAsync(new SignStudentEntity
                    {
                        StudentCode = studentCode,
                        CourseOrAlbumCode = request.CourseOrAlbumCode,
                        CourseAppearType = request.CourseAppearType,
                        IsPay = false,
                        SignCourseStatus = StudentCourseStatusEnum.SignUp
                    });
                    if (!string.IsNullOrEmpty(code))
                    {
                        //报名成功
                        //新增学生记录
                        var cacheUserInfo = await _userDBRepository.FindOneAsync("Code", studentCode);
                        var browsedRecordDBReq = new CourseBrowsedRecordRequest()
                        {
                            RelevanceCode = request.CourseOrAlbumCode,
                            UserCode = studentCode,
                            DisplayName = string.IsNullOrWhiteSpace(cacheUserInfo.FullName) ? cacheUserInfo.UserName : cacheUserInfo.FullName,
                            IsOnline = false,
                            BrowsedUserType = BrowsedUserTypeEnum.Student
                        };
                        var res = await SaveCourseBrowsedRecordByDBAsync(browsedRecordDBReq);
                    }
                    return code;
                }
                return string.Empty;
            }
        }

        /// <summary>
        /// 取消报名课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CancelSignUpCourseDBAsync(CancelSignUpCourseDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CancelSignUpCourseDBAsync execute", "DatabaseService"))
            {
                var res = false;
                if (string.IsNullOrWhiteSpace(request.CourseOrAlbumCode))
                {
                    ThrowCustomerException(CustomerRpcCode.CourseCodeEmpty, "CourseCodeEmpty");
                }
                if (string.IsNullOrWhiteSpace(request.StudentCode))
                {
                    ThrowCustomerException(CustomerRpcCode.StudentCodeEmpty, "StudentCodeEmpty");
                }
                var filter = Builders<SignStudentEntity>.Filter.Where(c => c.CourseOrAlbumCode == request.CourseOrAlbumCode && c.StudentCode == request.StudentCode && c.IsDelete == false);
                var findEntity = await _signStudentDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    res = await _signStudentDBRepository.DeleteOneAsync("Code", findEntity.Code);
                    if (res)
                    {
                        var delReq = new CourseBrowsedRecordRequest()
                        {
                            RelevanceCode = findEntity.CourseOrAlbumCode,
                            UserCode = findEntity.StudentCode
                        };
                        res = await DeleteCourseBrowsedRecordByDBAsync(delReq);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 查找学生所在班级的列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<UserGroupDTO>> FindUserGroupByStudentDBAsync(FindUserGroupByStudentDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserGroupByStudentDBAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.Keyword;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class);
                var codeFilter = Builders<LearnerEntity>.Filter.Eq(d => d.Code, request.StudentCode);
                var statusFilter = Builders<LearnerEntity>.Filter.Eq(d => d.LearnerStatus, LearnerStatusEnum.Accept);
                var secondFilter = builder.ElemMatch(c => c.Students, (codeFilter & statusFilter));
                filter = filter & secondFilter;
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    filter = filter & nameFilter;
                }
                var sort = Builders<UserGroupEntity>.Sort.Descending(v => v.CreateTime);
                var dbPage = new DBPageRequest<UserGroupEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _userGroupDBRepository.GetPages(dbPage);
                return PageResultConvert<UserGroupEntity, UserGroupDTO>(pageResult);
            }
        }

        /// <summary>
        /// 课程专辑详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseAlbumDetailDTO> FindCourseAlbumByCodeDBAsync(FindCourseAlbumByCodeDBRequest request)
        {
            var courseAlbumCode = request.CourseAlbumCode;
            if (string.IsNullOrEmpty(courseAlbumCode))
            {
                ThrowCustomerException(CustomerRpcCode.CourseCodeEmpty, "CourseCodeEmpty");
            }
            var languageCode = request.LanguageCode;
            if (string.IsNullOrWhiteSpace(languageCode))
            {
                languageCode = "en-US";
            }
            var filter = Builders<CourseAlbumEntity>.Filter.Where(w => w.Code == courseAlbumCode && w.IsDelete == false);
            var findEntity = await _courseAlbumDBRepository.FindOneAsync(filter);
            var detail = findEntity.MappingTo<CourseAlbumDetailDTO>();
            if (findEntity?.CourseCodes != null && findEntity?.CourseCodes.Count > 0)
            {
                detail.CourseInfos = await GetCourseInfoDetailDBByCodesAsync(findEntity?.CourseCodes, languageCode);
            }
            return detail;
        }

        /// <summary>
        /// 根据时间范围查询在线培训数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<OnlineTrainingDTO>> FindCourseByDateCenterDBAsync(FindOnlineTrainingByDateCenterDBRequest request)
        {
            var startTime = request.StartTime;
            var endTime = request.EndTime;
            var resultDatas = new List<OnlineTrainingDTO>();
            var filter = Builders<CourseInfoEntity>.Filter.Where(c => (c.CreateTime >= startTime && c.CreateTime <= endTime) || (c.UpdateTime >= startTime && c.UpdateTime <= endTime));
            var courses = await _courseInfoDBRepository.FindAllAsync(filter);
            if (courses != null && courses.Any())
            {
                var courseLabelIds = courses.SelectMany(x => x.CourseLabelCodes ?? new List<string>())
                    .Union(courses.SelectMany(x => x.CaseLabelCodes ?? new List<string>()))
                    .Distinct().ToList();
                var courseLabels = await GetCourseLabelsByCodes(courseLabelIds);
                var userGroupIds = courses.SelectMany(x => x.UserGroupCodes ?? new List<string>()).Distinct().ToList();
                var userGroups = await GetUserGroupsByCodes(userGroupIds);
                foreach (var course in courses)
                {
                    var onlineTraining = course.MappingTo<OnlineTrainingDTO>();
                    onlineTraining.Code = course.Code;
                    onlineTraining.CreateTime = course.CreateTime;
                    onlineTraining.UpdateTime = course.UpdateTime;

                    if (course.CaseLabelCodes != null && course.CaseLabelCodes.Any())
                    {
                        var caseLabelInfos = courseLabels.Where(x => course.CaseLabelCodes.Contains(x.Code))?.ToList() ?? new List<CourseLabelEntity>();
                        onlineTraining.CaseLabelInfos = caseLabelInfos.MappingTo<List<CourseLabelView>>();
                    }
                    if (course.CourseLabelCodes != null && course.CourseLabelCodes.Any())
                    {
                        var caseLabelInfos = courseLabels.Where(x => course.CourseLabelCodes.Contains(x.Code))?.ToList() ?? new List<CourseLabelEntity>();
                        onlineTraining.CourseLabelInfos = caseLabelInfos.MappingTo<List<CourseLabelView>>();
                    }
                    if (course.UserGroupCodes != null && course.UserGroupCodes.Any())
                    {
                        var userGroupInfos = userGroups.Where(x => course.UserGroupCodes.Contains(x.Code))?.ToList() ?? new List<UserGroupEntity>();
                        onlineTraining.UserGroupInfos = userGroupInfos.MappingTo<List<GroupView>>();
                    }
                    var studentFilter = Builders<SignStudentEntity>.Filter.Where(w => w.CourseOrAlbumCode == course.Code && w.IsDelete == false);
                    var signStudents = await _signStudentDBRepository.FindAllAsync(studentFilter);
                    if (signStudents != null && signStudents.Any())
                    {
                        MongoCustomerFilter<MongoTable> findAllStudent = new MongoCustomerFilter<MongoTable>();
                        findAllStudent.AppendAnyIn("Code", signStudents.Select(s => s.StudentCode).ToList());
                        var allStudentInfo = await _userDBRepository.FindAllAsync(findAllStudent.AndCombine());
                        onlineTraining.Students = new List<SignStudentView>();
                        foreach (var item in signStudents)
                        {
                            var student = allStudentInfo.FirstOrDefault(w => w.Code == item.StudentCode);
                            if (student == null)
                            {
                                continue;
                            }
                            onlineTraining.Students.Add(new SignStudentView
                            {
                                StudentCode = student.Code,
                                StudentName = !string.IsNullOrEmpty(student.FullName) ? student.FullName : student.UserName,
                                SignCourseStatus = item.SignCourseStatus,
                                IsPay = item.IsPay,
                                CourseAppearType = item.CourseAppearType
                            });
                        }
                    }
                    if (course.Assistants != null && course.Assistants.Any())
                    {
                        onlineTraining.Assistants = new List<SignStudentView>();
                        foreach (var item in course.Assistants)
                        {
                            var student = item.MappingTo<SignStudentView>();
                            student.StudentCode = item.Code;
                            onlineTraining.Assistants.Add(student);
                        }
                    }
                    if (course.Experts != null && course.Experts.Any())
                    {
                        onlineTraining.Experts = new List<SignStudentView>();
                        foreach (var item in course.Experts)
                        {
                            var student = item.MappingTo<SignStudentView>();
                            student.StudentCode = item.Code;
                            onlineTraining.Experts.Add(student);
                        }
                    }
                    resultDatas.Add(onlineTraining);
                }
            }
            return resultDatas;
        }

        /// <summary>
        /// 根据时间范围查询在线视频数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<OnlineVideoDTO>> FindVideoByDateCenterDBAsync(FindOnlineVideoByDateCenterDBRequest request)
        {
            var startTime = request.StartTime;
            var endTime = request.EndTime;

            var resultDatas = new List<OnlineVideoDTO>();
            var filter = Builders<VideoInfoEntity>.Filter.Where(c => (c.CreateTime >= startTime && c.CreateTime <= endTime) || (c.UpdateTime >= startTime && c.UpdateTime <= endTime));
            var videos = await _videoInfoDBRepository.FindAllAsync(filter);
            if (videos != null && videos.Any())
            {
                foreach (var video in videos)
                {
                    var videoInfo = video.MappingTo<OnlineVideoDTO>();
                    resultDatas.Add(videoInfo);
                }
            }
            return resultDatas;
        }

        /// <summary>
        /// 根据时间范围查询教培试卷数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<ExamPaperDTO>> FindExaminationPapeByDateCenterDBAsync(FindExaminationPaperByDateCenterDBRequest request)
        {
            var startTime = request.StartTime;
            var endTime = request.EndTime;

            var resultDatas = new List<ExamPaperDTO>();
            var filter = Builders<ExaminationPaperEntity>.Filter.Where(c => (c.CreateTime >= startTime && c.CreateTime <= endTime) || (c.UpdateTime >= startTime && c.UpdateTime <= endTime));
            var examPapers = await _examinationPaperDBRepository.FindAllAsync(filter);
            if (examPapers != null && examPapers.Any())
            {
                var courseLabelIds = examPapers.SelectMany(x => x.CourseLabelCodes ?? new List<string>())
                    .Union(examPapers.SelectMany(x => x.CaseLabelCodes ?? new List<string>()))
                    .Distinct().ToList();
                var courseLabels = await GetCourseLabelsByCodes(courseLabelIds);
                var questionIds = new List<string>();
                foreach (var item in examPapers)
                {
                    questionIds.AddRange(item.QuestionList?.Select(x => x.Code)?.ToList() ?? new List<string>());
                }
                var questions = await GetExaminationQuestionByCodes(questionIds);

                foreach (var examPaper in examPapers)
                {

                    var examPaperInfo = examPaper.MappingTo<ExamPaperDTO>();

                    if (examPaper.CaseLabelCodes != null && examPaper.CaseLabelCodes.Any())
                    {
                        var caseLabelInfos = courseLabels.Where(x => examPaper.CaseLabelCodes.Contains(x.Code))?.ToList() ?? new List<CourseLabelEntity>();
                        examPaperInfo.CaseLabels = caseLabelInfos.MappingTo<List<CourseLabelDTO>>();
                    }
                    if (examPaper.CourseLabelCodes != null && examPaper.CourseLabelCodes.Any())
                    {
                        var caseLabelInfos = courseLabels.Where(x => examPaper.CourseLabelCodes.Contains(x.Code))?.ToList() ?? new List<CourseLabelEntity>();
                        examPaperInfo.CourseLabels = caseLabelInfos.MappingTo<List<CourseLabelDTO>>();
                    }
                    if (examPaperInfo.QuestionList != null && examPaperInfo.QuestionList.Any())
                    {
                        foreach (var item in examPaperInfo.QuestionList)
                        {
                            var question = questions.FirstOrDefault(x => x.Code == item.Code);
                            if (question != null)
                            {
                                item.Stem = question.Stem;
                                item.QuestionType = question.QuestionType;
                                item.TrueOrFalse = question.TrueOrFalse;
                                item.QuestionOptionList = question.QuestionOptionList?.ToList() ?? new List<QuestionOptionDTO>();
                                item.FileList = question.FileList?.ToList() ?? new List<QuestionFileDTO>();
                            }
                        }
                    }
                    resultDatas.Add(examPaperInfo);

                }
            }
            return resultDatas;
        }

        /// <summary>
        /// 根据时间范围查询课程专辑数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<CourseAlbumView>> FindCourseAlbumByDateCenterDBAsync(FindCourseAlbumByDateCenterDBRequest request)
        {
            var startTime = request.StartTime;
            var endTime = request.EndTime;

            var resultDatas = new List<CourseAlbumView>();
            var filter = Builders<CourseAlbumEntity>.Filter.Where(c => (c.CreateTime >= startTime && c.CreateTime <= endTime) || (c.UpdateTime >= startTime && c.UpdateTime <= endTime));
            var courseAlbums = await _courseAlbumDBRepository.FindAllAsync(filter);
            if (courseAlbums != null && courseAlbums.Any())
            {
                var courseIds = courseAlbums.SelectMany(x => x.CourseCodes ?? new List<string>()).Distinct().ToList();
                var courses = await GetCoursesByCodes(courseIds);
                var courseLabelIds = courseAlbums.SelectMany(x => x.CourseLabelCodes ?? new List<string>()).Distinct().ToList();
                var courseLabels = await GetCourseLabelsByCodes(courseLabelIds);

                foreach (var courseAlbum in courseAlbums)
                {
                    var courseAlbumInfo = courseAlbum.MappingTo<CourseAlbumView>();

                    if (courseAlbum.CourseLabelCodes != null && courseAlbum.CourseLabelCodes.Any())
                    {
                        var caseLabelInfos = courseLabels.Where(x => courseAlbum.CourseLabelCodes.Contains(x.Code))?.ToList() ?? new List<CourseLabelEntity>();
                        courseAlbumInfo.CourseLabels = caseLabelInfos.MappingTo<List<CourseLabelDTO>>();
                    }
                    if (courseAlbum.CourseCodes != null && courseAlbum.CourseCodes.Any())
                    {
                        var courseInfos = courses.Where(x => courseAlbum.CourseCodes.Contains(x.Code))?.ToList() ?? new List<CourseInfoEntity>();
                        courseAlbumInfo.CourseInfos = courseInfos.Select(x => new BaseInfoView
                        {
                            Code = x.Code,
                            Name = x.Name,
                        }).ToList();
                    }
                    if (courseAlbum.Students != null && courseAlbum.Students.Any())
                    {
                        courseAlbumInfo.Students = courseAlbum.Students.MappingTo<List<StudentView>>();
                    }
                    resultDatas.Add(courseAlbumInfo);
                }
            }
            return resultDatas;
        }

        /// <summary>
        /// 升级课程标签，多语言
        /// </summary>
        private async Task<bool> UpgradeCourseLabelForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeCourseLabelForESDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var builder = Builders<CourseLabelEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false);
                    var findList = await _courseLabelDBRepository.FindAllAsync(filter); 
                    if (findList?.Count > 0)
                    {
                        foreach(var item in findList)
                        {
                            if(item.LabelLanguageConfigs?.Any(c => c.LanguageCode == "es-ES") == false || item.LabelLanguageConfigs?.Any(c => c.LanguageCode == "ru-RU") == false)
                            {
                                //表示不存在
                                if(item.LabelLanguageConfigs.Any(c => c.Name == "银杏知声"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Ginkgo biloba conoce el sonido",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Серебро знает звуки",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "超声技术功能小课堂"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Pequeña clase de función de la tecnología de ultrasonido",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Небольшой класс по ультразвуковой технологии",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "江苏省第八次疼痛医学学术年会"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Octava Conferencia Anual académica de medicina del dolor en la provincia de Jiangsu",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Восьмая ежегодная конференция по болевой медицине в провинции Цзянсу",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "2023 Webinar"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Seminario en línea 2023",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Сетевой семинар 2023",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "NT-e Abdomen"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Nt - e abdomen",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "NT - e брюшная полость",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "V series"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Serie V",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Серия V",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "VINNO Training"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Formación vinno",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Обучение VINNO",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "Tutorial in Dutch"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Tutorial en neerlandés",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Курсы голландского языка",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "NT-e MSK"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Tutorial en neerlandés",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Курсы голландского языка",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "A series"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Una serie",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Серия",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "Deutsch"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Deutsch",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Дойч",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "依诺有声网络超声规培学员小课堂"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Flyinsono audible Network ultrasonido formación de estudiantes pequeña clase",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Flyinsono Звуковая сеть Ультразвуковые курсы",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "犬猫心脏进阶系列课程"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Serie avanzada de cursos de corazón de perro y gato",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Серия курсов Сердце кошки",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "犬猫超声规范化操作系列课程（腹超）"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Serie de cursos de operación estandarizada por ultrasonido para perros y gatos (ecografía abdominal)",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Серия курсов по ультразвуковой стандартизации собак и кошек (гиперживот) ",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "毛孩子云学院"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Maohaiziyun College",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Академия Мао Цзыцзы",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "青草互学"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Aprendizaje mutuo de hierba y hierba",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Взаимное обучение травам",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "内部课程"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Cursos internos",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Внутренние курсы",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "肝脏"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Hígado",
                                        IsShow = true,
                                        LanguageCode = "es-ES"
                                    });
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Печень",
                                        IsShow = true,
                                        LanguageCode = "ru-RU"
                                    });
                                }
                            }
                            foreach(var languageItem in item.LabelLanguageConfigs)
                            {
                                languageItem.IsShow = true;
                                if (languageItem.Name == "银杏知声")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Ginkgo biloba knows the sound";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "超声技术功能小课堂")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Ultrasound Technology Function Classroom";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "江苏省第八次疼痛医学学术年会")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "The 8th Academic Annual Conference on Pain Medicine in Jiangsu Province";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "2023 Webinar")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "2023 Webinar";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "NT-e Abdomen")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "NT-e Abdomen";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "V series")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "V series";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "VINNO Training")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "VINNO Training";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "Tutorial in Dutch")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Tutorial in Dutch";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "NT-e MSK")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "NT-e MSK";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "A series")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "A series";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "Deutsch")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Deutsch";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "依诺有声网络超声规培学员小课堂")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Flyisono Audio Network Ultrasound Training Trainee Classroom";
                                    }
                                    else if (languageItem.LanguageCode == "zh-CN") 
                                    {
                                        languageItem.Name = "飞依诺有声网络超声规培学员小课堂";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "犬猫心脏进阶系列课程")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Dog and Cat Heart Advanced Series Courses";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "犬猫超声规范化操作系列课程（腹超）")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Series of Courses on Standardized Ultrasound Operations for Dogs and Cats (Abdominal Ultrasound)";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "毛孩子云学院")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Mao Haizi Cloud Academy";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "青草互学")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Green Grass Mutual Learning";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "内部课程")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "Internal courses";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                                else if (languageItem.Name == "肝脏")
                                {
                                    if (languageItem.LanguageCode == "en-US") 
                                    {
                                        languageItem.Name = "liver";
                                    }
                                    else
                                    {
                                        if (string.IsNullOrEmpty(languageItem.Name))
                                        {
                                            languageItem.Name = "";
                                        }
                                    }
                                }
                            }
                            res = await _courseLabelDBRepository.UpdateForSync("Code", item.Code, item, true, false);
                        }
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    Logger.WriteLineWarn($"CourseDBService UpgradeCourseLabelForESDBAsync Error: {ex}");
                    return res;
                }
            }
        }

        public async Task<bool> AddCoursePlayCountDBAsync(AddCoursePlayCountDBRequest request)
        {
            var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.Code == request.Code && c.IsDelete == false);
            var findEntity = await _courseInfoDBRepository.FindOneAsync(filter);
            if (findEntity != null)
            {
                findEntity.PlayCount += request.AddPlayCount;
                var update = Builders<CourseInfoEntity>.Update
                .Set(c => c.UpdateTime, DateTime.UtcNow)
                .Set(c => c.PlayCount, findEntity.PlayCount);
                return await _courseInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            }
            return false;
        }

        private async Task<List<CourseLabelEntity>> GetCourseLabelsByCodes(IList<string> labelCodes)
        {
            var filter = Builders<CourseLabelEntity>.Filter.Where(c => labelCodes.Contains(c.Code));
            var entities = await _courseLabelDBRepository.FindAllAsync(filter);
            return entities?.ToList() ?? new List<CourseLabelEntity>();
        }
        private async Task<List<UserGroupEntity>> GetUserGroupsByCodes(IList<string> groupCodes)
        {
            var filter = Builders<UserGroupEntity>.Filter.Where(c => groupCodes.Contains(c.Code));
            var entities = await _userGroupDBRepository.FindAllAsync(filter);
            return entities?.ToList() ?? new List<UserGroupEntity>();
        }
        private async Task<List<UserGroupEntity>> GetUserGroupsByStudentCode(string studentCode)
        {
            var filterDefinitions = new List<FilterDefinition<UserGroupEntity>>();
            filterDefinitions.Add(Builders<UserGroupEntity>.Filter.ElemMatch(x => x.Students, Builders<LearnerEntity>.Filter.Where(c => c.Code == studentCode && c.LearnerStatus != LearnerStatusEnum.Reject)));
            var filter = Builders<UserGroupEntity>.Filter.Or(filterDefinitions);
            var entities = await _userGroupDBRepository.FindAllAsync(filter);
            if (entities != null && entities.Any())
            {
                entities = entities.Where(x => x.UserGroupType == UserGroupEnum.Class)?.ToList();
            }
            return entities?.ToList() ?? new List<UserGroupEntity>();
        }
        private async Task<List<CourseInfoEntity>> GetCoursesByCodes(IList<string> courseCodes)
        {
            var filter = Builders<CourseInfoEntity>.Filter.Where(c => courseCodes.Contains(c.Code));
            var entities = await _courseInfoDBRepository.FindAllAsync(filter);
            return entities?.ToList() ?? new List<CourseInfoEntity>();
        }
        private async Task<List<ExaminationQuestionDTO>> GetExaminationQuestionByCodes(IList<string> questionCodes)
        {
            if (questionCodes == null || !questionCodes.Any())
            {
                return new List<ExaminationQuestionDTO>();
            }
            var result = await FindExaminationQuestionsByCodesDBAsync(new FindExaminationQuestionsByCodesDBRequest
            {
                Codes = questionCodes,
            });
            return result?.ToList() ?? new List<ExaminationQuestionDTO>();
        }

        /// <summary>
        /// 升级课程标签，多语言
        /// </summary>
        private async Task<bool> UpgradeCourseLabelForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeCourseLabelForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var builder = Builders<CourseLabelEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false);
                    var findList = await _courseLabelDBRepository.FindAllAsync(filter); 
                    if (findList?.Count > 0)
                    {
                        foreach(var item in findList)
                        {
                            if(item.LabelLanguageConfigs?.Any(c => c.LanguageCode == "ro-RO") == false)
                            {
                                //表示不存在
                                if(item.LabelLanguageConfigs.Any(c => c.Name == "银杏知声"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Ginkgo biloba cunoaşte sunetul",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "超声技术功能小课堂"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Sala de clasă a funcţiei tehnologiei ultrasunetelor",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "江苏省第八次疼痛医学学术年会"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "A 8-a Conferință Anuală Academică privind Medicina Durerii în provincia Jiangsu",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "2023 Webinar"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Webinar 2023",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "NT-e Abdomen"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "NT-e Abdomenul",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "V series"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Serie V",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "VINNO Training"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Instruire VINNO",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "Tutorial in Dutch"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Tutorial în limba olandeză",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "NT-e MSK"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "NT-e MSK",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "A series"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "O serie",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "Deutsch"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Deutsch",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "飞依诺有声网络超声规培学员小课堂"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Flyinsono Audio Network Antrenament cu ultrasunete Trainee Clasă",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "犬猫心脏进阶系列课程"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Cursuri avansate pentru inima câinilor și pisicilor",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "犬猫超声规范化操作系列课程（腹超）"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Seria de cursuri privind operațiunile standardizate cu ultrasunete pentru câini și pisici (ultrasunete abdominale)",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "毛孩子云学院"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Mao Haizi Cloud Academy",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "青草互学"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Învățarea reciprocă a ierbii verzi",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "内部课程"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "Cursuri interne",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                                else if(item.LabelLanguageConfigs.Any(c => c.Name == "肝脏"))
                                {
                                    item.LabelLanguageConfigs.Add(new LabelLanguageConfigEntity() {
                                        Name = "ficat",
                                        IsShow = true,
                                        LanguageCode = "ro-RO"
                                    });
                                }
                            }
                            res = await _courseLabelDBRepository.UpdateForSync("Code", item.Code, item, true, false);
                        }
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    Logger.WriteLineWarn($"CourseDBService UpgradeCourseLabelForRODBAsync Error: {ex}");
                    return res;
                }
            }
        }

    }
}