using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingMongoDBModule.DBSync;
using WingMongoDBModule.Mongodb.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.DistributedServerInfo;
using WingMongoDBModule.DO;
using WingMongoDBModule.Request;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Result;
using MongoDB.Driver;
using System.Linq;
using WingServerCommon.Log;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.FastestServer;
using System;
using System.Net;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Common;

namespace WingMongoDBModule.Service
{

    public partial class DatabaseService : JsonRpcService, IDistributedServerInfoDBService
    {

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<DistributedServerInfoDTO>> GetDistributedServerInfoPagesAsync(GetDistributedServerInfoDBPagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDistributedServerInfoPagesAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    regexFilters.Add(("Name", request.Keyword, true, false));
                }
                if (regexFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.Or, regexFilters.ToArray());
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _distributedServerInfoDBRepository.GetPages(dbPage);
                return PageResultConvert<DistributedServerInfoDO, DistributedServerInfoDTO>(pageResult);
            }
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDistributedServerInfoAsync(DeleteDistributedServerInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteDistributedServerInfoAsync execute", $"{this.GetType().Name}"))
            {
                return await _distributedServerInfoDBRepository.DeleteOneAsync("Code", request.Code);
            }
        }
        /// <summary>
        /// 增加
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDistributedServerInfoAsync(AddDistributedServerInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddDistributedServerInfoAsync execute", $"{this.GetType().Name}"))
            {
                var distributedServerInfoDO = request.Data.MappingTo<DistributedServerInfoDO>();
                var resultcode = await _distributedServerInfoDBRepository.InsertOneAsync(distributedServerInfoDO, request.PlatformDatas, request.ExtensionData);
                var res = !string.IsNullOrEmpty(resultcode);
                if (res) 
                {
                    await LoadDistributedServerConfig();
                }
                return res;
            }
        }
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDistributedServerInfoAsync(UpdateDistributedServerInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDistributedServerInfoAsync execute", $"{this.GetType().Name}"))
            {
                var distributedServerInfoDO = request.Data.MappingTo<DistributedServerInfoDO>();
                var res = await _distributedServerInfoDBRepository.UpdateOneAsync("Code", distributedServerInfoDO.Code, distributedServerInfoDO, request.ExtensionData, true);
                await LoadDistributedServerConfig();
                return res;
            }
        }

        /// <summary>
        /// 查询codes
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<DistributedServerInfoDTO>> GetDistributedServerInfoDBByCodesAsync(GetDistributedServerInfoDBByCodesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDistributedServerInfoDBByCodesAsync execute", $"{this.GetType().Name}"))
            {
                Dictionary<string, List<string>> filter = new Dictionary<string, List<string>>();
                filter.Add("Code", request.Codes);
                var servers = await _distributedServerInfoDBRepository.FindInAsync(filter);
                return servers.MappingTo<List<DistributedServerInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询服务器信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DistributedServerInfoDTO> GetDistributedServerInfoDBByServerUrlAsync(string serverUrl)
        {
            using (new Performance(_performanceThreshold, "{0} GetDistributedServerInfoDBByServerUrlAsync execute", $"{this.GetType().Name}"))
            {
                Dictionary<string, string> filter = new Dictionary<string, string>();
                var serverInfo = await _distributedServerInfoDBRepository.FindOneAsync("ServerUrl", serverUrl);
                var resp = new DistributedServerInfoDTO()
                {
                    ServerCode = serverInfo.ServerCode,
                    IsMaster = serverInfo.IsMaster,
                    Name = serverInfo.Name,
                    Description = serverInfo.Description,
                    Enable = serverInfo.Enable,
                    ServerType = serverInfo.ServerType,
                    ServerUrl = serverInfo.ServerUrl,
                    Lat = serverInfo.Lat,
                    Lng = serverInfo.Lng,
                    DefaultStorageNode = serverInfo.DefaultStorageNode,
                    RTCStartingRoomId = serverInfo.RTCStartingRoomId,
                    ShareCodePrefix = serverInfo.ShareCodePrefix,
                    LANHost = serverInfo.LANHost
                };
                return resp;
            }
        }

        /// <summary>
        /// 查询单个
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DistributedServerInfoDTO> GetDistributedServerInfoDBByCodeAsync(GetDistributedServerInfoDBByCodeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDistributedServerInfoDBByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var server = await _distributedServerInfoDBRepository.FindOneAsync("Code", request.Code);
                return server.MappingTo<DistributedServerInfoDTO>();
            }
        }


        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDistributedServerIPListAsync(UpdateServerInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDistributedServerIPListAsync execute", $"{this.GetType().Name}"))
            {
                var serverInfo = await _distributedServerInfoDBRepository.FindOneAsync("Name", request.Name);
                var tempList = request.IpRangeList.MappingTo<List<IPAddressInfoDO>>();
                serverInfo.AssignClientIPList = tempList;
                return await _distributedServerInfoDBRepository.UpdateOneAsync("Code", serverInfo.Code, serverInfo, null);
            }
        }

        /// <summary>
        /// 查询服务器配置信息IP列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<IPAddressInfoDTO>> GetDistributedServerIpListDBAsync(GetDistributedServerIpListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDistributedServerIpListDBAsync execute", $"{this.GetType().Name}"))
            {
                var server = await _distributedServerInfoDBRepository.FindOneAsync("Code", request.Code);
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    var longIP = IpToLong(request.Keyword);
                    server.AssignClientIPList = server.AssignClientIPList.Where(a => a.StartIp == request.Keyword || a.EndIp == request.Keyword || (a.LongStartIP <= longIP && longIP <= a.LongEndIP)).ToList();
                }
                var result = server.AssignClientIPList.Skip((request.CurrentPage - 1) * request.PageSize).Take(request.PageSize).MappingTo<List<IPAddressInfoDTO>>();
                PageResult<IPAddressInfoDTO> pageResult = new PageResult<IPAddressInfoDTO>();
                pageResult.PageIndex = request.CurrentPage;
                pageResult.PageSize = request.PageSize;
                pageResult.PageData = result;
                pageResult.TotalCount = server.AssignClientIPList.Count;
                return pageResult;
            }
        }

        /// <summary>
        /// Ip转Long
        /// </summary>
        /// <param name="ip"></param>
        /// <returns></returns>
        private long IpToLong(string ip)
        {
            try
            {
                if (System.Net.IPAddress.TryParse(ip, out IPAddress ipAddress))
                {
                    char[] separator = new char[] { '.' };
                    string[] items = ip.Split(separator);
                    var ipLong = long.Parse(items[0]) << 24 | long.Parse(items[1]) << 16 | long.Parse(items[2]) << 8 | long.Parse(items[3]);
                    return ipLong;
                }
                // else
                // {
                //     Logger.WriteLineWarn($"IpToLong failed, err ip formatter, ip: {ip}");
                // }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"IpToLong failed, ip: {ip}, ex:{ex}");
            }
            return 0;
        }
    }
}