using System;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.DTO.DiagnosisResult;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request.DBRequest;
using WingMongoDBModule.Entities;
using WingServerCommon.Service;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IDiagnosisResultDBService
    {
        /// <summary>
        /// 保存AI诊断结果
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDiagnosisResultInfosAsync(AddDiagnosisResultInfosDBRequest request)
        {
            if (!string.IsNullOrWhiteSpace(request.FileUrl) && request.DiagnosisResultInfos != null && request.DiagnosisResultInfos.Any())
            {
                var entities = request.DiagnosisResultInfos.Select(x => new DiagnosisResultEntity
                {
                    RelationCode = request.RelationCode,
                    FileUrl = request.FileUrl,
                    Index = x.Index,
                    DiagnosisResult = x.DiagnosisResult,
                    CreateTime = request.CreateTime != null ? (DateTime)request.CreateTime : DateTime.UtcNow,
                    UpdateTime = request.UpdateTime != null ? (DateTime)request.UpdateTime : DateTime.UtcNow,
                }).ToList();
                if (request.SyncOthers)
                {
                    foreach(var item in entities)
                    {
                        await _diagnosisResultDBRepository.InsertOneForAsyncDBAsync(item);
                    }
                    return true;
                }
                else
                {
                    return await _diagnosisResultDBRepository.InsertAllWithoutOplogLogAsync(entities);
                }
            }
            return false;
        }

        /// <summary>
        /// 查询图像AI原始结果详情
        /// </summary>
        /// <returns></returns>
        public async Task<DiagnosisResultDTO> GetDiagnosisResultInfoByCodeAsync(GetDiagnosisResultInfoDBRequest request)
        {
            var relatedCode = request.RelationCode;
            var index = request.Index;
            using (new Performance(_performanceThreshold, "{0} GetDiagnosisResultInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<DiagnosisResultEntity>.Filter.Where(x => x.RelationCode == relatedCode && x.Index == index);
                var entity = await _diagnosisResultDBRepository.FindOneAsync(filter);
                if (entity != null)
                {
                    return new DiagnosisResultDTO
                    {
                        Index = entity.Index,
                        DiagnosisResult = entity.DiagnosisResult,
                    };
                }
                return null;
            }
        }

    }
}