using System.Data.Common;
using System.Net.Cache;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.DBRequest;
using WingServerCommon.Utilities;
using MongoDB.Driver;
using WingMongoDBModule.Entities;
using System;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Mapper;
using System.Linq;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Request;
using MongoDB.Bson;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingServerCommon.Log;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.DTO.LiveRoom;
using WingServerCommon.Interfaces.Cache;
using System.Text.RegularExpressions;
using Syncfusion.Pdf.Graphics;
using WingInterfaceLibrary.DTO.DataCenter;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.Result.Courses;
using Newtonsoft.Json;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IEducationDBService
    {
        #region CourseLabel模块
        /// <summary>
        /// 获取标签列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IList<CourseLabelDTO>> GetCourseLabelsDBAsync(QueryCourseLabelListRequest request, bool isFilterShow = false)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseLabelsDBAsync execute", $"{this.GetType().Name}"))
            {
                IList<CourseLabelDTO> result = new List<CourseLabelDTO>();
                var builder = Builders<CourseLabelEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (request.Type > 0)
                {
                    var typeFilter = builder.Where(c => c.Type == request.Type);
                    filter = filter & typeFilter;
                }
                if (!string.IsNullOrEmpty(request.ParentCode))
                {
                    var firstFilter = builder.Where(c => c.Code == request.ParentCode);
                    var secondFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(d => d.Code, request.ParentCode));
                    var thirdFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(e => e.Code, request.ParentCode)));
                    var fourthFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(e => e.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(f => f.Code, request.ParentCode))));
                    filter = filter & (firstFilter | secondFilter | thirdFilter | fourthFilter);
                }
                if (!string.IsNullOrEmpty(request.LabelCode))
                {
                    var firstFilter = builder.Where(c => c.Code == request.LabelCode);
                    var secondFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(d => d.Code, request.LabelCode));
                    var thirdFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(e => e.Code, request.LabelCode)));
                    var fourthFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(e => e.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(f => f.Code, request.LabelCode))));
                    filter = filter & (firstFilter | secondFilter | thirdFilter | fourthFilter);
                }
                var findList = await _courseLabelDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    if (string.IsNullOrEmpty(request.ParentCode))
                    {
                        //查所有
                        result = MappingCourseLabels(findList, request.LanguageCode, isFilterShow: isFilterShow);
                    }
                    else
                    {
                        //找到了，那就查询找到本身的所有子集 
                        //最多四级
                        //第一级
                        foreach (var firstItem in findList)
                        {
                            if (firstItem == null)
                            {
                                continue;
                            }
                            //第二级
                            if (firstItem.ChildLabels?.Count > 0 && firstItem.Code == request.ParentCode)
                            {
                                result = MappingCourseLabels(firstItem.ChildLabels, request.LanguageCode, firstItem.Code, isFilterShow: isFilterShow);
                                return result;
                            }
                            else
                            {
                                foreach (var secondItem in firstItem.ChildLabels)
                                {
                                    if (secondItem == null)
                                    {
                                        continue;
                                    }
                                    //第三级
                                    if (secondItem.ChildLabels?.Count > 0 && secondItem.Code == request.ParentCode)
                                    {
                                        result = MappingCourseLabels(secondItem.ChildLabels, request.LanguageCode, secondItem.Code, isFilterShow: isFilterShow);
                                        return result;
                                    }
                                    else
                                    {
                                        foreach (var thirdItem in secondItem.ChildLabels)
                                        {
                                            if (thirdItem == null)
                                            {
                                                continue;
                                            }
                                            //第四级
                                            if (thirdItem.ChildLabels?.Count > 0 && thirdItem.Code == request.ParentCode)
                                            {
                                                result = MappingCourseLabels(thirdItem.ChildLabels, request.LanguageCode, thirdItem.Code, true, isFilterShow: isFilterShow);
                                                return result;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 迭代查询课程标签【递归】
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private IList<CourseLabelDTO> MappingCourseLabels(IList<CourseLabelEntity> findList, string languageCode, string parentCode = "", bool isLastLevel = false, int level = 1, bool isFilterShow = false)
        {
            List<CourseLabelDTO> result = new List<CourseLabelDTO>();
            //最多四级
            //第一级
            foreach (var findItem in findList)
            {
                var curLevel = level;
                if (findItem != null)
                {
                    CourseLabelDTO tempItem = GetNewCourseLabelDTO(findItem, languageCode, parentCode, isLastLevel, isFilterShow);
                    if (!string.IsNullOrEmpty(tempItem?.Code) && !string.IsNullOrEmpty(tempItem?.Name))
                    {
                        //第二级或者下一级
                        if (findItem.ChildLabels?.Count > 0)
                        {
                            curLevel++;
                            if (curLevel >= 4)
                            {
                                isLastLevel = true;
                            }
                            else
                            {
                                isLastLevel = false;
                            }
                            var dataList = MappingCourseLabels(findItem.ChildLabels, languageCode, findItem.Code, isLastLevel, curLevel, isFilterShow);
                            tempItem.ChildLabels = dataList;
                        }
                        result.Add(tempItem);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 迭代查询课程标签【递归】
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private void SetIterationCourseLabelParentCode(CourseLabelDTO findItem)
        {
            if (findItem.ChildLabels?.Count > 0)
            {
                foreach (var nextItem in findItem.ChildLabels)
                {
                    //第二级
                    nextItem.ParentCode = findItem.Code;
                    if (nextItem.ChildLabels?.Count > 0)
                    {
                        SetIterationCourseLabelParentCode(nextItem);
                    }
                }
            }
        }

        /// <summary>
        /// 获取所有标签列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IList<CourseLabelDTO>> GetAllCourseLabelDBAsync(QueryCourseLabelDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetAllCourseLabelDBAsync execute", $"{this.GetType().Name}"))
            {
                IList<CourseLabelDTO> result = new List<CourseLabelDTO>();
                var builder = Builders<CourseLabelEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var findList = await _courseLabelDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    //最多四级
                    //第一级
                    result = IterationAllCourseLabels(findList, request.LanguageCode);
                }
                return result;
            }
        }

        /// <summary>
        /// 迭代查询课程标签【递归】
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private IList<CourseLabelDTO> IterationAllCourseLabels(IList<CourseLabelEntity> findList, string languageCode)
        {
            List<CourseLabelDTO> result = new List<CourseLabelDTO>();
            //最多四级
            //第一级
            foreach (var findItem in findList)
            {
                if (findItem != null)
                {
                    CourseLabelDTO tempItem = GetNewCourseLabelDTO(findItem, languageCode);
                    result.Add(tempItem);
                    //第二级或者下一级
                    if (findItem.ChildLabels?.Count > 0)
                    {
                        var dataList = IterationAllCourseLabels(findItem.ChildLabels, languageCode);
                        result.AddRange(dataList);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 获取标签列表【可获取子标签】
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IList<CourseLabelDTO>> GetCourseLabelDBByCodesAsync(QueryCourseLabelDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseLabelDBByCodesAsync execute", $"{this.GetType().Name}"))
            {
                IList<CourseLabelDTO> result = new List<CourseLabelDTO>();
                var builder = Builders<CourseLabelEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Type == request.Type);
                var findList = await _courseLabelDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    CourseLabelDTO tempItem = null;
                    if (!request.IsNeedChild)
                    {
                        if (request.LabelCodes?.Count > 0) 
                        {
                            var findResult = findList.FindAll(c => request.LabelCodes.Contains(c.Code));
                            if (findResult?.Count > 0)
                            {
                                foreach (var c in findResult)
                                {
                                    tempItem = GetNewCourseLabelDTO(c, request.LanguageCode);
                                    result.Add(tempItem);
                                }
                            }
                        }
                        else
                        {
                            foreach (var c in findList)
                            {
                                tempItem = GetNewCourseLabelDTO(c, request.LanguageCode);
                                result.Add(tempItem);
                            }
                        }
                    }
                    else
                    {
                        //最多四级
                        //第一级
                        result = IterationCourseLabels(findList, request.LabelCodes, request.LanguageCode);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 迭代查询课程标签【递归】
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private IList<CourseLabelDTO> IterationCourseLabels(IList<CourseLabelEntity> findList, IList<string> labelCodes, string languageCode)
        {
            List<CourseLabelDTO> result = new List<CourseLabelDTO>();
            //最多四级
            //第一级
            foreach (var findItem in findList)
            {
                if (findItem != null)
                {
                    if (labelCodes?.Count > 0 && labelCodes.Contains(findItem.Code))
                    {
                        CourseLabelDTO tempItem = GetNewCourseLabelDTO(findItem, languageCode);
                        result.Add(tempItem);
                    }
                    else
                    {
                        CourseLabelDTO tempItem = GetNewCourseLabelDTO(findItem, languageCode);
                        result.Add(tempItem);
                    }
                    //第二级或者下一级
                    if (findItem.ChildLabels?.Count > 0)
                    {
                        var dataList = IterationCourseLabels(findItem.ChildLabels, labelCodes, languageCode);
                        result.AddRange(dataList);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 封装新标签实体
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private CourseLabelDTO GetNewCourseLabelDTO(CourseLabelEntity c, string languageCode, string parentCode = "", bool isLastLevel = false, bool isFilterShow = false)
        {
            LabelLanguageConfigEntity labelConfigInfo;
            if (isFilterShow) 
            {
                labelConfigInfo = c.LabelLanguageConfigs?.FirstOrDefault(x => x.LanguageCode == languageCode && x.IsShow == true) ?? new LabelLanguageConfigEntity();
            }
            else
            {
                labelConfigInfo = c.LabelLanguageConfigs?.FirstOrDefault(x => x.LanguageCode == languageCode) ?? new LabelLanguageConfigEntity();
            }
            var tempItem = new CourseLabelDTO()
            {
                Code = c.Code,
                Name = labelConfigInfo.Name,
                UseObjectType = c.UseObjectType,
                LanguageCode = languageCode,
                IsShow = labelConfigInfo.IsShow,
                Type = c.Type,
                ParentCode = parentCode,
                IsLastLevel = isLastLevel,
                LabelLanguageConfigs = c.LabelLanguageConfigs?.MappingTo<List<LabelLanguageConfigDTO>>() ?? new List<LabelLanguageConfigDTO>()
            };
            return tempItem;
        }

        /// <summary>
        /// 新增标签
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddCourseLabelDBAsync(CourseLabelRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddCourseLabelDBAsync execute", $"{this.GetType().Name}"))
            {
                var allLabelList = await GetAllCourseLabelDBAsync(new QueryCourseLabelDBRequest() { LanguageCode = "zh-CN" }) ?? new List<CourseLabelDTO>();
                if (allLabelList?.Count > 0 && request.LabelLanguageConfigs?.Count > 0)
                {
                    foreach(var curLangItem in request.LabelLanguageConfigs)
                    {
                        var exist = allLabelList.Any(c => c.LabelLanguageConfigs.Any(dx => dx.LanguageCode == curLangItem.LanguageCode && dx.Name == curLangItem.Name));
                        if (exist) {
                            if (curLangItem.LanguageCode == "zh-CN")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelCNNameRepeat, "Course label cn name repeat");
                            }
                            else if (curLangItem.LanguageCode == "ru-RU")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelRUNameRepeat, "Course label ru name repeat");
                            }
                            else if (curLangItem.LanguageCode == "es-ES")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelESNameRepeat, "Course label es name repeat");
                            }
                            else if (curLangItem.LanguageCode == "ro-RO")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelRONameRepeat, "Course label ro name repeat");
                            }
                            else
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelENNameRepeat, "Course label en name repeat");
                            }
                        }
                    }
                }
                var result = false;
                CourseLabelEntity newLabelInfo = null;
                if (!string.IsNullOrWhiteSpace(request.ParentCode))
                {
                    var findEntity = await FindOneCourseLabelInfoAsync(request.ParentCode);
                    if (findEntity != null)
                    {
                        if (findEntity.Code == request.ParentCode)
                        {
                            newLabelInfo = CreateNewCourseLabelInfo(request);
                            if (findEntity.ChildLabels == null)
                            {
                                findEntity.ChildLabels = new List<CourseLabelEntity>()
                                {
                                    newLabelInfo
                                };
                            }
                            else
                            {
                                findEntity.ChildLabels.Add(newLabelInfo);
                            }
                        }
                        else
                        {
                            var firstItems = findEntity.ChildLabels;
                            if (firstItems?.Count > 0)
                            {
                                foreach (var secondItem in firstItems)
                                {
                                    if (secondItem != null)
                                    {
                                        if (secondItem.Code == request.ParentCode)
                                        {
                                            newLabelInfo = CreateNewCourseLabelInfo(request);
                                            if (secondItem.ChildLabels == null)
                                            {
                                                secondItem.ChildLabels = new List<CourseLabelEntity>()
                                                {
                                                    newLabelInfo
                                                };
                                            }
                                            else
                                            {
                                                secondItem.ChildLabels.Add(newLabelInfo);
                                            }
                                            break;
                                        }
                                        //第三级
                                        if (secondItem.ChildLabels?.Count > 0)
                                        {
                                            foreach (var thirdItem in secondItem.ChildLabels)
                                            {
                                                if (thirdItem != null)
                                                {
                                                    if (thirdItem.Code == request.ParentCode)
                                                    {
                                                        newLabelInfo = CreateNewCourseLabelInfo(request);
                                                        if (thirdItem.ChildLabels == null)
                                                        {
                                                            thirdItem.ChildLabels = new List<CourseLabelEntity>()
                                                            {
                                                                newLabelInfo
                                                            };
                                                        }
                                                        else
                                                        {
                                                            thirdItem.ChildLabels.Add(newLabelInfo);
                                                        }
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        result = await _courseLabelDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                    }
                    else
                    {
                        //没找到，直接新增
                        newLabelInfo = CreateNewCourseLabelInfo(request);
                        var newId = await _courseLabelDBRepository.InsertOneAsync(newLabelInfo);
                        if (!string.IsNullOrEmpty(newId))
                        {
                            result = true;
                        }
                    }
                }
                else
                {
                    newLabelInfo = CreateNewCourseLabelInfo(request);
                    var newId = await _courseLabelDBRepository.InsertOneAsync(newLabelInfo);
                    if (!string.IsNullOrEmpty(newId))
                    {
                        request.Code = newId;
                        result = true;
                    }
                }
                if (request.IsMove && !string.IsNullOrEmpty(request.Code))
                {
                    result = await ChangeCourseLabelDBAsync(request.ParentCode, request.Code, request.Type);
                }
                return result;
            }
        }

        /// <summary>
        /// 迁移课程标签
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private async Task<bool> ChangeCourseLabelDBAsync(string parentCode, string code, LabelTypeEnum type)
        {
            //删除相关课程标签
            var result = false;
            var albumFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == parentCode));
            var albumList = await _courseAlbumDBRepository.FindAllAsync(albumFilter);
            if (albumList?.Count > 0)
            {
                foreach (var albumItem in albumList)
                {
                    if (albumItem.CourseLabelCodes.Contains(parentCode))
                    {
                        albumItem.CourseLabelCodes.Remove(parentCode);
                    }
                    if (type == LabelTypeEnum.CourseLabel)
                    {
                        albumItem.CourseLabelCodes.Add(code);
                    }
                    albumItem.CourseLabelCodes = albumItem.CourseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    result = await _courseAlbumDBRepository.UpdateOneAsync("Code", albumItem.Code, albumItem, true);
                }
            }

            var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == parentCode) || c.CaseLabelCodes.Any(dx => dx == parentCode)));
            var courseList = await _courseInfoDBRepository.FindAllAsync(courseFilter);
            if (courseList?.Count > 0)
            {
                foreach (var courseItem in courseList)
                {
                    if (courseItem.CourseLabelCodes.Contains(parentCode))
                    {
                        courseItem.CourseLabelCodes.Remove(parentCode);
                    }
                    if (courseItem.CaseLabelCodes.Contains(parentCode))
                    {
                        courseItem.CaseLabelCodes.Remove(parentCode);
                    }
                    if (type == LabelTypeEnum.CourseLabel)
                    {
                        courseItem.CourseLabelCodes.Add(code);
                    }
                    else if (type == LabelTypeEnum.CaseLabel)
                    {
                        courseItem.CaseLabelCodes.Add(code);
                    }
                    courseItem.CourseLabelCodes = courseItem.CourseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    courseItem.CaseLabelCodes = courseItem.CaseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    result = await _courseInfoDBRepository.UpdateOneAsync("Code", courseItem.Code, courseItem, true);
                }
            }

            var videoFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == parentCode));
            var videoList = await _videoInfoDBRepository.FindAllAsync(videoFilter);
            if (videoList?.Count > 0)
            {
                foreach (var videoItem in videoList)
                {
                    if (videoItem.CourseLabelCodes.Contains(parentCode))
                    {
                        videoItem.CourseLabelCodes.Remove(parentCode);
                    }
                    if (type == LabelTypeEnum.CourseLabel)
                    {
                        videoItem.CourseLabelCodes.Add(code);
                    }
                    videoItem.CourseLabelCodes = videoItem.CourseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    result = await _videoInfoDBRepository.UpdateOneAsync("Code", videoItem.Code, videoItem, true);
                }
            }

            var questionFilter = Builders<ExaminationQuestionEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == parentCode) || c.CaseLabelCodes.Any(dx => dx == parentCode)));
            var questionList =  await _examinationQuestionDBRepository.FindAllAsync(questionFilter);
            if (questionList?.Count > 0)
            {
                foreach (var questionItem in questionList)
                {
                    if (questionItem.CourseLabelCodes.Contains(parentCode))
                    {
                        questionItem.CourseLabelCodes.Remove(parentCode);
                    }
                    if (questionItem.CaseLabelCodes.Contains(parentCode))
                    {
                        questionItem.CaseLabelCodes.Remove(parentCode);
                    }
                    if (type == LabelTypeEnum.CourseLabel)
                    {
                        questionItem.CourseLabelCodes.Add(code);
                    }
                    else if (type == LabelTypeEnum.CaseLabel)
                    {
                        questionItem.CaseLabelCodes.Add(code);
                    }
                    questionItem.CourseLabelCodes = questionItem.CourseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    questionItem.CaseLabelCodes = questionItem.CaseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    result = await _examinationQuestionDBRepository.UpdateOneAsync("Code", questionItem.Code, questionItem, true);
                }
            }

            var paperFilter = Builders<ExaminationPaperEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == parentCode) || c.CaseLabelCodes.Any(dx => dx == parentCode)));
            var paperList =  await  _examinationPaperDBRepository.FindAllAsync(paperFilter);
            if (paperList?.Count > 0)
            {
                foreach (var paperItem in paperList)
                {
                    if (paperItem.CourseLabelCodes.Contains(parentCode))
                    {
                        paperItem.CourseLabelCodes.Remove(parentCode);
                    }
                    if (paperItem.CaseLabelCodes.Contains(parentCode))
                    {
                        paperItem.CaseLabelCodes.Remove(parentCode);
                    }
                    if (type == LabelTypeEnum.CourseLabel)
                    {
                        paperItem.CourseLabelCodes.Add(code);
                    }
                    else if (type == LabelTypeEnum.CaseLabel)
                    {
                        paperItem.CaseLabelCodes.Add(code);
                    }
                    paperItem.CourseLabelCodes = paperItem.CourseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    paperItem.CaseLabelCodes = paperItem.CaseLabelCodes?.Distinct()?.ToList() ?? new List<string>();
                    result = await _examinationPaperDBRepository.UpdateOneAsync("Code", paperItem.Code, paperItem, true);
                }
            }
            return result;
        }

        /// <summary>
        /// 创建标签
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        private CourseLabelEntity CreateNewCourseLabelInfo(CourseLabelRequest request)
        {
            var labelInfo = new CourseLabelEntity()
            {
                Code = Guid.NewGuid().ToString("N"),
                LabelLanguageConfigs = request.LabelLanguageConfigs.MappingTo<List<LabelLanguageConfigEntity>>(),
                UseObjectType = request.UseObjectType,
                Type = request.Type,
                ChildLabels = new List<CourseLabelEntity>()
            };
            request.Code = labelInfo.Code;
            return labelInfo;
        }

        /// <summary>
        /// 修改标签
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseLabelDBAsync(CourseLabelRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseLabelDBAsync execute", $"{this.GetType().Name}"))
            {
                var allLabelList = await GetAllCourseLabelDBAsync(new QueryCourseLabelDBRequest() { LanguageCode = "zh-CN" }) ?? new List<CourseLabelDTO>();
                if (allLabelList?.Count > 0 && request.LabelLanguageConfigs?.Count > 0)
                {
                    allLabelList = allLabelList.Where(c => c.Code != request.Code)?.ToList() ?? new List<CourseLabelDTO>();
                    foreach(var curLangItem in request.LabelLanguageConfigs)
                    {
                        var exist = allLabelList.Any(c => c.LabelLanguageConfigs.Any(dx => dx.LanguageCode == curLangItem.LanguageCode && dx.Name == curLangItem.Name));
                        if (exist) {
                            if (curLangItem.LanguageCode == "zh-CN")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelCNNameRepeat, "Course label cn name repeat");
                            }
                            else if (curLangItem.LanguageCode == "ru-RU")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelRUNameRepeat, "Course label ru name repeat");
                            }
                            else if (curLangItem.LanguageCode == "es-ES")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelESNameRepeat, "Course label es name repeat");
                            }
                            else if (curLangItem.LanguageCode == "ro-RO")
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelRONameRepeat, "Course label ro name repeat");
                            }
                            else
                            {
                                ThrowCustomerException(CustomerRpcCode.CourseLabelENNameRepeat, "Course label en name repeat");
                            }
                        }
                    }
                }
                var result = false;
                if (!string.IsNullOrWhiteSpace(request.Code))
                {
                    var findEntity = await FindOneCourseLabelInfoAsync(request.Code);
                    if (findEntity != null)
                    {
                        if (findEntity.Code == request.Code)
                        {
                            findEntity.UpdateTime = DateTime.UtcNow;
                            findEntity.Type = request.Type;
                            findEntity.UseObjectType = request.UseObjectType;
                            findEntity.LabelLanguageConfigs = request.LabelLanguageConfigs.MappingTo<List<LabelLanguageConfigEntity>>();
                        }
                        else
                        {
                            var firstItems = findEntity.ChildLabels;
                            if (firstItems?.Count > 0)
                            {
                                foreach (var secondItem in firstItems)
                                {
                                    if (secondItem != null)
                                    {
                                        if (secondItem.Code == request.Code)
                                        {
                                            secondItem.UpdateTime = DateTime.UtcNow;
                                            secondItem.LabelLanguageConfigs = request.LabelLanguageConfigs.MappingTo<List<LabelLanguageConfigEntity>>();
                                            break;
                                        }
                                        //第三级
                                        if (secondItem.ChildLabels?.Count > 0)
                                        {
                                            foreach (var thirdItem in secondItem.ChildLabels)
                                            {
                                                if (thirdItem != null)
                                                {
                                                    if (thirdItem.Code == request.Code)
                                                    {
                                                        thirdItem.UpdateTime = DateTime.UtcNow;
                                                        thirdItem.LabelLanguageConfigs = request.LabelLanguageConfigs.MappingTo<List<LabelLanguageConfigEntity>>();
                                                        break;
                                                    }
                                                    //第四级
                                                    if (thirdItem.ChildLabels?.Count > 0)
                                                    {
                                                        foreach (var fourthItem in thirdItem.ChildLabels)
                                                        {
                                                            if (fourthItem != null)
                                                            {
                                                                if (fourthItem.Code == request.Code)
                                                                {
                                                                    fourthItem.UpdateTime = DateTime.UtcNow;
                                                                    fourthItem.LabelLanguageConfigs = request.LabelLanguageConfigs.MappingTo<List<LabelLanguageConfigEntity>>();
                                                                    break;
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        result = await _courseLabelDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 通过code删除标签
        /// </summary> 
        /// <param name="code">标签code</param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseLabelDBByCodeAsync(string code, LabelTypeEnum labelType)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteCourseLabelDBByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                if (!string.IsNullOrWhiteSpace(code))
                {
                    var findEntity = await FindOneCourseLabelInfoAsync(code);
                    if (findEntity != null)
                    {
                        if (findEntity.Code == code)
                        {
                            findEntity.IsDelete = true;
                        }
                        else
                        {
                            var firstItems = findEntity.ChildLabels;
                            if (firstItems?.Count > 0)
                            {
                                CourseLabelEntity removeItems = null;
                                foreach (var secondItem in firstItems)
                                {
                                    if (secondItem != null)
                                    {
                                        if (secondItem.Code == code)
                                        {
                                            removeItems = secondItem;
                                            break;
                                        }
                                        //第三级
                                        if (secondItem.ChildLabels?.Count > 0)
                                        {
                                            foreach (var thirdItem in secondItem.ChildLabels)
                                            {
                                                if (thirdItem != null)
                                                {
                                                    if (thirdItem.Code == code)
                                                    {
                                                        removeItems = thirdItem;
                                                        break;
                                                    }
                                                    //第四级
                                                    if (thirdItem.ChildLabels?.Count > 0)
                                                    {
                                                        foreach (var fourthItem in thirdItem.ChildLabels)
                                                        {
                                                            if (fourthItem != null)
                                                            {
                                                                if (fourthItem.Code == code)
                                                                {
                                                                    removeItems = fourthItem;
                                                                    break;
                                                                }
                                                            }
                                                        }
                                                        if (removeItems != null)
                                                        {
                                                            thirdItem.ChildLabels.Remove(removeItems);
                                                            removeItems = null;
                                                        }
                                                    }
                                                }
                                            }
                                            if (removeItems != null)
                                            {
                                                secondItem.ChildLabels.Remove(removeItems);
                                                removeItems = null;
                                            }
                                        }
                                    }
                                }
                                if (removeItems != null)
                                {
                                    firstItems.Remove(removeItems);
                                    removeItems = null;
                                }
                            }
                        }
                        result = await _courseLabelDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                    }
                    if (result)
                    {
                        //删除相关课程标签
                        var albumFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == code));
                        var albumList = await _courseAlbumDBRepository.FindAllAsync(albumFilter);
                        if (albumList?.Count > 0)
                        {
                            foreach (var albumItem in albumList)
                            {
                                if (albumItem.CourseLabelCodes.Contains(code))
                                {
                                    albumItem.CourseLabelCodes.Remove(code);
                                }
                                result = await _courseAlbumDBRepository.UpdateOneAsync("Code", albumItem.Code, albumItem, true);
                            }
                        }

                        var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == code) || c.CaseLabelCodes.Any(dx => dx == code)));
                        var courseList = await _courseInfoDBRepository.FindAllAsync(courseFilter);
                        if (courseList?.Count > 0)
                        {
                            foreach (var courseItem in courseList)
                            {
                                if (courseItem.CourseLabelCodes.Contains(code))
                                {
                                    courseItem.CourseLabelCodes.Remove(code);
                                }
                                if (courseItem.CaseLabelCodes.Contains(code))
                                {
                                    courseItem.CaseLabelCodes.Remove(code);
                                }
                                result = await _courseInfoDBRepository.UpdateOneAsync("Code", courseItem.Code, courseItem, true);
                            }
                        }

                        var videoFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == code));
                        var videoList = await _videoInfoDBRepository.FindAllAsync(videoFilter);
                        if (videoList?.Count > 0)
                        {
                            foreach (var videoItem in videoList)
                            {
                                if (videoItem.CourseLabelCodes.Contains(code))
                                {
                                    videoItem.CourseLabelCodes.Remove(code);
                                }
                                result = await _videoInfoDBRepository.UpdateOneAsync("Code", videoItem.Code, videoItem, true);
                            }
                        }

                        var questionFilter = Builders<ExaminationQuestionEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == code) || c.CaseLabelCodes.Any(dx => dx == code)));
                        var questionList =  await _examinationQuestionDBRepository.FindAllAsync(questionFilter);
                        if (questionList?.Count > 0)
                        {
                            foreach (var questionItem in questionList)
                            {
                                if (questionItem.CourseLabelCodes.Contains(code))
                                {
                                    questionItem.CourseLabelCodes.Remove(code);
                                }
                                if (questionItem.CaseLabelCodes.Contains(code))
                                {
                                    questionItem.CaseLabelCodes.Remove(code);
                                }
                                result = await _examinationQuestionDBRepository.UpdateOneAsync("Code", questionItem.Code, questionItem, true);
                            }
                        }

                        var paperFilter = Builders<ExaminationPaperEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == code) || c.CaseLabelCodes.Any(dx => dx == code)));
                        var paperList =  await  _examinationPaperDBRepository.FindAllAsync(paperFilter);
                        if (paperList?.Count > 0)
                        {
                            foreach (var paperItem in paperList)
                            {
                                if (paperItem.CourseLabelCodes.Contains(code))
                                {
                                    paperItem.CourseLabelCodes.Remove(code);
                                }
                                if (paperItem.CaseLabelCodes.Contains(code))
                                {
                                    paperItem.CaseLabelCodes.Remove(code);
                                }
                                result = await _examinationPaperDBRepository.UpdateOneAsync("Code", paperItem.Code, paperItem, true);
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 通过code查询标签
        /// </summary> 
        /// <param name="code">标签code</param>
        /// <returns></returns>
        private async Task<CourseLabelEntity> FindOneCourseLabelInfoAsync(string code)
        {
            CourseLabelEntity resultEntity = null;
            var builder = Builders<CourseLabelEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            var firstFilter = builder.Where(c => c.Code == code);
            var secondFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(d => d.Code, code));
            var thirdFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(e => e.Code, code)));
            var fourthFilter = builder.ElemMatch(c => c.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(d => d.ChildLabels, Builders<CourseLabelEntity>.Filter.ElemMatch(e => e.ChildLabels, Builders<CourseLabelEntity>.Filter.Eq(f => f.Code, code))));
            filter = filter & (firstFilter | secondFilter | thirdFilter | fourthFilter);
            resultEntity = await _courseLabelDBRepository.FindOneAsync(filter);
            return resultEntity;
        }
        #endregion 

        #region UserGroup用户组
        /// <summary>
        /// 生成唯一 shortCode
        /// </summary>
        /// <returns></returns>
        private string GeneralShortCode()
        {
            try
            {
                lock (_createShortCodeLocker)
                {
                    var uniqueId = new byte[6];
                    var rand = new Random((int)(DateTime.Now.Ticks % 1000000));
                    for (int i = 0; i < 6; i++)
                    {
                        int randCode;
                        do
                        {
                            randCode = rand.Next(50, 90);
                            uniqueId[i] = Convert.ToByte(randCode);
                        } while (randCode >= 58 && randCode <= 64 || randCode == 79 || randCode == 73);
                    }

                    return System.Text.Encoding.ASCII.GetString(uniqueId);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"GeneralShortCode error {ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 客户端根据创建人获取创建的用户组
        /// </summary> 
        /// <param name="creatorCode">创建人code</param>
        /// <returns></returns>
        public async Task<IList<UserGroupDTO>> GetUserGroupListsDBByCreatorCodeAsync(string creatorCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserGroupListsDBByCreatorCodeAsync execute", $"{this.GetType().Name}"))
            {
                IList<UserGroupDTO> result = new List<UserGroupDTO>();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.CreatorCode == creatorCode && c.UserGroupType == UserGroupEnum.Class);
                var findList = await _userGroupDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    result = await TransferUserGroupList(findList);
                }
                return result;
            }
        }

        /// <summary>
        /// 根据用户组列表查询用户组信息
        /// </summary> 
        /// <param name="creatorCode">创建人code</param>
        /// <returns></returns>
        public async Task<IList<UserGroupDTO>> GetUserGroupListsDBByUserGroupCodesAsync(IList<string> userGroupCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserGroupListsDBByUserGroupCodesAsync execute", $"{this.GetType().Name}"))
            {
                IList<UserGroupDTO> result = new List<UserGroupDTO>();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class);
                if (userGroupCodes?.Count > 0)
                {
                    var infilter = builder.In(c => c.Code, userGroupCodes);
                    filter = filter & infilter;
                }
                var findList = await _userGroupDBRepository.FindAllAsync(filter);
                if (findList?.Count > 0)
                {
                    foreach (var item in findList)
                    {
                        var newUserGroup = new UserGroupDTO()
                        {
                            Code = item.Code,
                            Name = item.Name,
                            MaxPeople = item.MaxPeople,
                            CreatorCode = item.CreatorCode,
                            CreateTime = item.CreateTime,
                            UpdateTime = item.UpdateTime,
                            CreatorName = "",
                            ShortCode = item.ShortCode,
                            Students = new List<StudentInfoDTO>()
                        };
                        result.Add(newUserGroup);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 根据班级短码查询班级
        /// </summary> 
        /// <param name="creatorCode">创建人code</param>
        /// <returns></returns>
        public async Task<UserGroupDTO> GetUserGroupDBByShortCodeAsync(string shortCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserGroupDBByShortCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = new UserGroupDTO();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.ShortCode == shortCode);
                var item = await _userGroupDBRepository.FindOneAsync(filter);
                if (item != null)
                {
                    result = new UserGroupDTO()
                    {
                        Code = item.Code,
                        Name = item.Name,
                        MaxPeople = item.MaxPeople,
                        CreatorCode = item.CreatorCode,
                        CreateTime = item.CreateTime,
                        UpdateTime = item.UpdateTime,
                        CreatorName = "",
                        ShortCode = item.ShortCode,
                        Students = new List<StudentInfoDTO>()
                    };
                }
                return result;
            }
        }


        /// <summary>
        /// 查询用户组分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<UserGroupDTO>> FindUserGroupsPagesDBAsync(QueryUserGroupPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserGroupsPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class);
                if (!string.IsNullOrWhiteSpace(request.TeacherCode))
                {
                    if (request.IsManager)
                    {
                        var teacherFilter = builder.Where(e => e.CreatorCode == request.TeacherCode || e.CreatorCode == "");
                        filter = filter & teacherFilter;
                    }
                    else
                    {
                        var teacherFilter = builder.Where(e => e.CreatorCode == request.TeacherCode);
                        filter = filter & teacherFilter;
                    }
                }
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    FilterDefinition<UserGroupEntity> regexFilter;
                    var userCodeList = new List<string>();
                    var userList = await GetGroupUserListsByKeyword(keyword);
                    if (userList?.Count > 0)
                    {
                        userCodeList = userList.Select(c => c.Code).Distinct().ToList();
                        var creatorFilter = builder.In(c => c.CreatorCode, userCodeList);
                        regexFilter = creatorFilter | nameFilter;
                    }
                    else
                    {
                        regexFilter = nameFilter;
                    }
                    filter = filter & regexFilter;
                }
                if (request.IsOldPlatformData != null)
                {
                    if (request.IsOldPlatformData == true)
                    {
                        var oldDataFilter = builder.Where(e => e.IsOldPlatformData == true || e.IsAgent == true);
                        filter = filter & oldDataFilter;
                    }
                    else
                    {
                        var oldDataFilter = builder.Where(e => e.IsOldPlatformData != true && e.IsAgent != true);
                        filter = filter & oldDataFilter;
                    }
                }
                if (request.IsManager)
                {
                    var defaultFilter = builder.Where(e => e.Name != "DefaultDistributorGroup");
                    filter = filter & defaultFilter;
                }
                var dbPage = new DBPageRequest<UserGroupEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _userGroupDBRepository.GetPages(dbPage);
                PageResult<UserGroupDTO> convertResult = new();
                convertResult.PageIndex = pageResult.CurrentPage;
                convertResult.PageSize = pageResult.PageSize;
                convertResult.TotalCount = pageResult.DataCount;
                convertResult.PageData = await TransferUserGroupList(pageResult.PageData);
                return convertResult;
            }
        }

        // <summary>
        /// 审核用户组学员
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AuditUserGroupLearnerDBAsync(UserGroupRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AuditUserGroupLearner execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var findEntity = await _userGroupDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code) && findEntity.Students?.Count > 0 && request.Students?.Count > 0)
                {
                    foreach (var item in request.Students)
                    {
                        var entity = new LearnerEntity()
                        {
                            Code = item.Code,
                            LearnerStatus = item.LearnerStatus
                        };
                        var studentInfo = findEntity.Students.FirstOrDefault(c => c.Code == item.Code);
                        if (studentInfo == null || string.IsNullOrWhiteSpace(studentInfo.Code))
                        {
                            findEntity.Students.Add(entity);
                        }
                        else
                        {
                            studentInfo.LearnerStatus = item.LearnerStatus;
                        }
                    }
                    result = await _userGroupDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                }
                return result;
            }
        }

        /// <summary>
        /// 用户组数据转换
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<List<UserGroupDTO>> TransferUserGroupList(IList<UserGroupEntity> userGroupList, string studentKeyword = "")
        {
            var resultList = new List<UserGroupDTO>();
            if (userGroupList?.Count > 0)
            {
                foreach (var item in userGroupList)
                {
                    int courseCount = 0, practiceCount = 0, examCount = 0;
                    var findCourseFilter = Builders<CourseInfoEntity>.Filter.AnyEq(w => w.UserGroupCodes, item.Code);
                    var courses = await _courseInfoDBRepository.FindAllAsync(findCourseFilter);
                    if (courses != null && courses.Count > 0)
                    {
                        courseCount = courses.Count;
                        practiceCount = courses.Sum(c => c.BindExams != null ? c.BindExams.Count(b => b.ExaminationType == CourseExaminationTypeEnum.Practice) : 0);
                        examCount = courses.Sum(c => c.BindExams != null ? c.BindExams.Count(b => b.ExaminationType == CourseExaminationTypeEnum.Exam) : 0);
                    }
                    var userInfo = await FindUserDTOByCodeAsync(item.CreatorCode, false) ?? new UserDTO();
                    var userCodes = item.Students.Select(c => c.Code).ToList();
                    var newUserGroup = new UserGroupDTO()
                    {
                        Code = item.Code,
                        Name = item.Name,
                        MaxPeople = item.MaxPeople,
                        CreatorCode = item.CreatorCode,
                        CreateTime = item.CreateTime,
                        UpdateTime = item.UpdateTime,
                        CreatorName = string.IsNullOrEmpty(userInfo.FullName) ? userInfo.UserName : userInfo.FullName,
                        Students = await GetUserInfoByCodeFromCache(userCodes, studentKeyword, item.Students),
                        CourseCount = courseCount,
                        PracticeCount = practiceCount,
                        ExamCount = examCount,
                        ShortCode = item.ShortCode,
                        IsAgent = item.IsAgent,
                        Description = item.Description
                    };
                    newUserGroup.StudentCount = newUserGroup.Students.Count(c => c.LearnerStatus == LearnerStatusEnum.Accept);
                    resultList.Add(newUserGroup);
                }
            }
            return resultList;
        }

        /// <summary>
        /// 用户信息转换
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<IList<StudentInfoDTO>> GetUserInfoByCodeFromCache(IList<string> userCodes, string studentKeyword = "", IList<LearnerEntity> students = null)
        {
            IList<StudentInfoDTO> studentList = new List<StudentInfoDTO>();
            if (userCodes?.Count > 0)
            {
                var userList = await FindUserListByUserCodesAsync(userCodes.ToList());
                if (userList?.Count > 0)
                {
                    foreach (var item in userList)
                    {
                        string studentName = string.IsNullOrEmpty(item.FullName) ? item.UserName : item.FullName;
                        var newStudent = new StudentInfoDTO()
                        {
                            Code = item.UserCode,
                            Name = studentName,
                            Phone = item.Phone
                        };
                        if (students?.Count > 0)
                        {
                            newStudent.LearnerStatus = students.FirstOrDefault(c => c.Code == newStudent.Code)?.LearnerStatus ?? LearnerStatusEnum.NoApproval;
                        }
                        if (!string.IsNullOrEmpty(studentKeyword))
                        {
                            if (studentName.Contains(studentKeyword) || item.Phone.Contains(studentKeyword))
                            {
                                studentList.Add(newStudent);
                            }
                        }
                        else
                        {
                            studentList.Add(newStudent);
                        }
                    }
                }
            }
            return studentList;
        }

        /// <summary>
        /// 查询用户关键词
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<IList<UserInfoDO>> GetGroupUserListsByKeyword(string keyword)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            if (!string.IsNullOrWhiteSpace(keyword))
            {
                var filters = ManagementUserFilter(keyword);
                MongoCustomerFilter<MongoDataItem> queryAll = new();
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryAll.OrCombine(filters.ToArray()));
            }
            var userInfoList = await _userDBRepository.FindAllAsync(customerFilter.AndCombine());
            return userInfoList;
        }

        /// <summary>
        /// 根据学员code查询所有的用户组
        /// </summary>
        /// <param name="userCode"></param>
        /// <returns></returns>
        private async Task<IList<string>> FindUserGroupCodesByStudentCodeDBAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserGroupCodesByStudentCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<string>();
                if (!string.IsNullOrEmpty(userCode))
                {
                    var builder = Builders<UserGroupEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class);//&& c.Students.Contains(userCode)
                    var secondFilter = builder.ElemMatch(c => c.Students, Builders<LearnerEntity>.Filter.Eq(d => d.Code, userCode));
                    filter = filter & secondFilter;
                    var data = await _userGroupDBRepository.FindAllAsync(filter);
                    if (data?.Count > 0)
                    {
                        result = data.Select(c => c.Code).Distinct().ToList();
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 可以单独查询用户组，也可以查询用户组下具体的学员信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<UserGroupDTO> FindUserGroupsByGroupCodeDBAsync(string userGroupCode, string studentKeyword)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserGroupsPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new UserGroupDTO();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == userGroupCode);
                //可以单独查询用户组，也可以查询用户组下具体的学员信息
                if (!string.IsNullOrEmpty(studentKeyword))
                {
                    var userCodeList = new List<string>();
                    var userList = await GetGroupUserListsByKeyword(studentKeyword);
                    if (userList?.Count > 0)
                    {
                        userCodeList = userList.Select(c => c.Code).Distinct().ToList();
                        var keywordFitler = builder.ElemMatch(c => c.Students, Builders<LearnerEntity>.Filter.In(c => c.Code, userCodeList));
                        filter = filter & keywordFitler;
                    }
                }
                var data = await _userGroupDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(data?.Code))
                {
                    var resultList = await TransferUserGroupList(new List<UserGroupEntity>() { data }, studentKeyword);
                    if (resultList?.Count > 0)
                    {
                        result = resultList.FirstOrDefault() ?? new UserGroupDTO();
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 新增用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddUserGroupDBAsync(UserGroupRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddUserGroupDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Name == request.Name && c.UserGroupType == UserGroupEnum.Class);
                var isExist = await _userGroupDBRepository.Exists(filter);
                if (!isExist)
                {
                    var insertEntity = request.MappingTo<UserGroupEntity>();
                    var shortCode = "";
                    for (var i = 0; i < 5; i++)
                    {
                        shortCode = GeneralShortCode();
                        if (!string.IsNullOrWhiteSpace(shortCode))
                        {
                            var userGroup = await GetUserGroupDBByShortCodeAsync(shortCode);
                            if (userGroup == null || string.IsNullOrWhiteSpace(userGroup.Code))
                            {
                                break;
                            }
                        }
                        shortCode = string.Empty;
                    }
                    insertEntity.ShortCode = shortCode;
                    var res = await _userGroupDBRepository.InsertOneAsync(insertEntity);
                    if (!string.IsNullOrEmpty(res))
                    {
                        result = true;
                    }
                }
                else
                {
                    if (!request.NotValidateRepeat)
                    {
                        ThrowCustomerException(CustomerRpcCode.UserGroupNameNotRepeat, "UserGroupNameNotRepeat");
                    }
                    else
                    {
                        result = true;
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 修改用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> UpdateUserGroupDBAsync(UserGroupRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserGroupDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var findEntity = await _userGroupDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    var existFilter = builder.Where(c => c.IsDelete == false && c.Name == request.Name);
                    var existEntity = await _userGroupDBRepository.FindOneAsync(existFilter);
                    //用户组名称不存在，或者跟当前code是同一个才能修改
                    if (existEntity == null || existEntity.Code == findEntity.Code)
                    {
                        findEntity.UpdateTime = DateTime.UtcNow;
                        findEntity.Name = request.Name;
                        findEntity.MaxPeople = request.MaxPeople;
                        findEntity.CreatorCode = request.CreatorCode;
                        findEntity.IsAgent = request.IsAgent;
                        result = await _userGroupDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity, true);
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 删除用户组
        /// </summary> 
        /// <param name="code">用户组code</param>
        /// <returns></returns>
        public async Task<bool> DeleteUserGroupDBByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserGroupDBByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _userGroupDBRepository.DeleteOneAsync("Code", code);
                return result;
            }
        }

        /// <summary>
        /// 用户组新增用户
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddStudentsByGroupCodeDBAsync(UserGroupRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddStudentsByGroupCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = true;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                if (request.Students?.Count > 0)
                {
                    var findEntity = await _userGroupDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(findEntity?.Code))
                    {
                        bool isAdd = false;
                        foreach (var item in request.Students)
                        {
                            if (!findEntity.Students.ToList().Exists(c => c.Code == item.Code))
                            {
                                findEntity.Students.Add(new LearnerEntity()
                                {
                                    Code = item.Code,
                                    LearnerStatus = item.LearnerStatus
                                });
                                isAdd = true;
                            }
                        }
                        if (request.RemoveStudentCodes?.Count > 0)
                        {
                            foreach (var removeItem in request.RemoveStudentCodes)
                            {
                                var removeStudent = findEntity.Students.ToList().Find(c => c.Code == removeItem);
                                if (removeStudent != null && !string.IsNullOrEmpty(removeStudent.Code))
                                {
                                    findEntity.Students.Remove(removeStudent);
                                    isAdd = true;
                                }
                            }
                        }

                        var addCount = findEntity.Students.Count(c => c.LearnerStatus == LearnerStatusEnum.Accept);
                        if (addCount > findEntity.MaxPeople)
                        {
                            //超出最大人数
                            return false;
                        }
                        if (isAdd)
                        {
                            result = await _userGroupDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 用户组移除用户
        /// </summary>
        /// <param name="userGroupCode">用户组code</param>
        /// <param name="studentCodes">学员code列表</param>
        /// <returns></returns>
        public async Task<bool> RemoveStudentsByGroupCodeDBAsync(string userGroupCode, IList<string> studentCodes)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveStudentsByGroupCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == userGroupCode);
                if (studentCodes?.Count > 0)
                {
                    var findEntity = await _userGroupDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(findEntity?.Code))
                    {
                        bool isUpdate = false;
                        foreach (var item in studentCodes)
                        {
                            var removeStudent = findEntity.Students.ToList().Find(c => c.Code == item);
                            if (removeStudent != null && !string.IsNullOrEmpty(removeStudent.Code))
                            {
                                findEntity.Students.Remove(removeStudent);
                                isUpdate = true;
                            }
                        }
                        if (isUpdate)
                        {
                            result = await _userGroupDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddExaminationPaperAsync(AddExaminationPaperDBRequest request)
        {
            var result = await _examinationPaperDBRepository.InsertOneAsync(new ExaminationPaperEntity
            {
                Name = request.Name,
                TotalScore = request.TotalScore,
                QuestionList = request.QuestionList.MappingTo<List<QuestionDetail>>(),
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            });
            return !string.IsNullOrWhiteSpace(result);
        }

        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteExaminationPaperAsync(DeleteExaminationPaperDBRequest request)
        {
            return await _examinationPaperDBRepository.DeleteOneAsync("Code", request.ExaminationPaperCode);
        }

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateExaminationPaperAsync(UpdateExaminationPaperDBRequest request)
        {
            var filter = Builders<ExaminationPaperEntity>.Filter.Eq(f => f.Code, request.ExaminationPaperCode);
            List<UpdateDefinition<ExaminationPaperEntity>> updates = new();
            var update = Builders<ExaminationPaperEntity>.Update;
            if (request.Name != null)
            {
                updates.Add(update.Set(f => f.Name, request.Name));
            }
            if (request.QuestionList != null)
            {
                updates.Add(update.Set(f => f.QuestionList, request.QuestionList.MappingTo<List<QuestionDetail>>()));
            }
            if (request.TotalScore > 0)
            {
                updates.Add(update.Set(f => f.TotalScore, request.TotalScore));
            }
            if (request.CaseLabelCodes != null)
            {
                updates.Add(update.Set(f => f.CaseLabelCodes, request.CaseLabelCodes));
            }
            if (request.CourseLabelCodes != null)
            {
                updates.Add(update.Set(f => f.CourseLabelCodes, request.CourseLabelCodes));
            }
            var combineUpdates = update.Combine(updates.ToArray());
            var result = await _examinationPaperDBRepository.UpdateOneAsync(filter, combineUpdates);
            return result > 0;
        }

        /// <summary>
        /// 查询试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ExaminationPaperDTO> GetExaminationPaperAsync(GetExaminationPaperDBRequest request)
        {
            var filter = Builders<ExaminationPaperEntity>.Filter.Where(f => !f.IsDelete && f.Code == request.ExaminationPaperCode);
            var result = await _examinationPaperDBRepository.FindOneAsync(filter);
            return result.MappingTo<ExaminationPaperDTO>();
        }

        /// <summary>
        /// 查询试卷分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ExaminationPaperDTO>> GetExaminationPaperPagesAsync(GetExaminationPaperPagesDBRequest request)
        {
            var dbPage = new DBPageRequest<ExaminationPaperEntity>();
            dbPage.CurrentPage = request.CurrentPage;
            dbPage.PageSize = request.PageSize;
            dbPage.Filter = Builders<ExaminationPaperEntity>.Filter.Eq(f => f.IsDelete, false);
            if (request.Filter != null && request.Filter.ContainsKey("Keyword"))
            {
                dbPage.Filter = dbPage.Filter & Builders<ExaminationPaperEntity>.Filter.Regex(f => f.Name, $".*{request.Filter["Keyword"]}.*");
            }
            if (request.CaseLabelCodes?.Count > 0)
            {
                var caseFilter = Builders<ExaminationPaperEntity>.Filter.AnyIn(f => f.CaseLabelCodes, request.CaseLabelCodes.ToList());
                dbPage.Filter = dbPage.Filter & caseFilter;
            }
            if (request.CourseLabelCodes?.Count > 0)
            {
                var courseFilter = Builders<ExaminationPaperEntity>.Filter.AnyIn(f => f.CourseLabelCodes, request.CourseLabelCodes.ToList());
                dbPage.Filter = dbPage.Filter & courseFilter;
            }
            var pageResult = await _examinationPaperDBRepository.GetPages(dbPage);
            return PageResultConvert<ExaminationPaperEntity, ExaminationPaperDTO>(pageResult);
        }
        #endregion

        /// <summary>
        /// 题目列表分页查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ExaminationQuestionDTO>> FindExaminationQuestionPagesDBAsync(FindExaminationQuestionPagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindExaminationQuestionPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.SearchKey;
                var caseLabelCodes = request.CaseLabelCodes;
                var courseLabelCodes = request.CourseLabelCodes;
                var codes = request.Codes;
                var notInCodes = request.NotInCodes;
                var builder = Builders<ExaminationQuestionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Stem, new BsonRegularExpression(keyword, "i"));
                    filter = filter & nameFilter;
                }
                if (caseLabelCodes != null && caseLabelCodes.Count > 0)
                {
                    filter = filter & builder.AnyIn(x => x.CaseLabelCodes, caseLabelCodes);
                }
                if (courseLabelCodes != null && courseLabelCodes.Count > 0)
                {
                    filter = filter & builder.AnyIn(x => x.CourseLabelCodes, courseLabelCodes);
                }
                if (codes != null && codes.Count > 0)
                {
                    filter = filter & builder.In(x => x.Code, codes);
                }
                if (notInCodes != null && notInCodes.Count > 0)
                {
                    filter = filter & builder.Nin(x => x.Code, notInCodes);
                }
                var dbPage = new DBPageRequest<ExaminationQuestionEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _examinationQuestionDBRepository.GetPages(dbPage);
                return PageResultConvert<ExaminationQuestionEntity, ExaminationQuestionDTO>(pageResult);
            }
        }

        /// <summary>
        /// 根据codes查询题目列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<ExaminationQuestionDTO>> FindExaminationQuestionsByCodesDBAsync(FindExaminationQuestionsByCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindExaminationQuestionsByCodesDBAsync execute", $"{this.GetType().Name}"))
            {
                var codes = request.Codes;
                var builder = Builders<ExaminationQuestionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (codes != null && codes.Count > 0)
                {
                    filter = filter & builder.In(x => x.Code, codes);
                }

                var dbResult = await _examinationQuestionDBRepository.FindAllAsync(filter);
                var result = dbResult.MappingTo<List<ExaminationQuestionDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 新增题目
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateExaminationQuestionDBAsync(ExaminationQuestionDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateExaminationQuestionDBAsync execute", $"{this.GetType().Name}"))
            {
                var examinationQuestionEntity = request.MappingTo<ExaminationQuestionEntity>();
                var result = await _examinationQuestionDBRepository.InsertOneAsync(examinationQuestionEntity);
                return result;
            }
        }

        /// <summary>
        /// 详情
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<ExaminationQuestionDTO> FindExaminationQuestionByCodeDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindExaminationQuestionByCodeDBAsync execute", $"{this.GetType().Name}"))
            {

                var entity = await _examinationQuestionDBRepository.FindOneAsync("Code", code);
                var result = entity.MappingTo<ExaminationQuestionDTO>();
                return result;
            }
        }

        /// <summary>
        /// 修改题目
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateExaminationQuestionDBAsync(ExaminationQuestionDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateExaminationQuestionDBAsync execute", $"{this.GetType().Name}"))
            {
                var examinationQuestionEntity = request.MappingTo<ExaminationQuestionEntity>();
                var entity = await _examinationQuestionDBRepository.FindOneAsync("Code", examinationQuestionEntity.Code);
                examinationQuestionEntity.CreateTime = entity.CreateTime;
                var result = await _examinationQuestionDBRepository.UpdateOneAsync("Code", examinationQuestionEntity.Code, examinationQuestionEntity);
                return result;
            }
        }

        /// <summary>
        /// 删除题目
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> DeleteExaminationQuestionDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteExaminationQuestionDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _examinationQuestionDBRepository.DeleteOneAsync("Code", code);
                return result;
            }
        }

        /// <summary>
        /// 根据题目code查询试卷列表
        /// </summary>
        /// <param name="questionCode"></param>
        /// <returns></returns>
        public async Task<IList<ExaminationPaperDTO>> GetExaminationPaperPagesByQuestionCodeAsync(string questionCode)
        {
            var builder = Builders<ExaminationPaperEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            var secondFilter = builder.ElemMatch(c => c.QuestionList, Builders<QuestionDetail>.Filter.Eq(d => d.Code, questionCode));
            filter = filter & secondFilter;
            var dbResult = await _examinationPaperDBRepository.FindAllAsync(filter);
            var result = dbResult.MappingTo<List<ExaminationPaperDTO>>();
            return result;
        }

        /// <summary>
        /// 查询远程考核上传图像数据列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<RemoteExaminationPageDTO>> FindRemoteExaminationListDBAsync(FindRemoteExaminationListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindRemoteExaminationListDBAsync execute", $"{this.GetType().Name}"))
            {

                var builder = Builders<RemoteExaminationDataEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                filter = filter & builder.Where(x => x.UserCode == request.UserCode);
                var dbResult = await _remoteExaminationDataDBRepository.FindAllAsync(filter);
                var result = dbResult.MappingTo<List<RemoteExaminationPageDTO>>();
                if (result != null && result.Count > 0)
                {
                    var devices = await FindDeviceInfoListByCodeListAsync(result.Select(x => x.DeviceCode).ToList());
                    foreach (var item in result)
                    {
                        item.DeviceName = devices.FirstOrDefault(x => x.DeviceCode == item.DeviceCode)?.Name;
                    }
                }
                return result;
            }
        }
        /// <summary>
        /// 创建远程考核上传图像数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateRemoteExaminationDTODBAsync(RemoteExaminationDTO request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateRemoteExaminationDTODBAsync execute", $"{this.GetType().Name}"))
            {
                var entity = request.MappingTo<RemoteExaminationDataEntity>();
                var result = await _remoteExaminationDataDBRepository.InsertOneAsync(entity);
                return result;
            }
        }

        /// <summary>
        /// 创建学生答题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> CreateStudentExaminationDBAsync(AddStudentExaminationDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateStudentExaminationDBAsync execute", $"{this.GetType().Name}"))
            {
                var entity = request.StudentExamination.MappingTo<StudentExaminationEntity>();
                //姓名
                var user = await FindUserByCodeAsync(entity.StudentCode);
                entity.StudentName = string.IsNullOrWhiteSpace(user?.FullName) ? user?.UserName : user?.FullName;
                var result = await _studentExaminationDBRepository.InsertOneAsync(entity);
                return result;
            }
        }

        /// <summary>
        /// 更新学生答题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateStudentExaminationDBAsync(UpdateStudentExaminationDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateStudentExaminationDBAsync execute", $"{this.GetType().Name}"))
            {
                var entity = request.StudentExamination.MappingTo<StudentExaminationEntity>();
                var oldEntity = await _studentExaminationDBRepository.FindOneAsync("Code", entity.Code);
                entity.CreateTime = oldEntity.CreateTime;
                //姓名
                var user = await FindUserByCodeAsync(entity.StudentCode);
                entity.StudentName = string.IsNullOrWhiteSpace(user?.FullName) ? user?.UserName : user?.FullName;
                var result = await _studentExaminationDBRepository.UpdateOneAsync("Code", entity.Code, entity);
                return result;
            }
        }

        /// <summary>
        /// 查询学生答题列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<StudentExaminationDTO>> FindStudentExaminationPagesDBAsync(FindStudentExaminationPagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindStudentExaminationPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.Keyword;
                var userCode = request.UserCode;
                var relevanceType = request.RelevanceType;
                var relevanceCode = request.RelevanceCode;
                var builder = Builders<StudentExaminationEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.StudentName, new BsonRegularExpression(keyword, "i"));
                    filter = filter & nameFilter;
                }
                filter = filter & builder.Where(x => x.RelevanceType == relevanceType);
                filter = filter & builder.Where(x => x.RelevanceCode == relevanceCode);
                var dbPage = new DBPageRequest<StudentExaminationEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _studentExaminationDBRepository.GetPages(dbPage);
                return PageResultConvert<StudentExaminationEntity, StudentExaminationDTO>(pageResult);
            }
        }
        /// <summary>
        /// 查询学生答题详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<StudentExaminationDTO> FindStudentExaminationByCodeDBAsync(FindStudentExaminationByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindStudentExaminationByCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<StudentExaminationEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                var entity = await _studentExaminationDBRepository.FindTableOneAsync(filter);
                var result = entity.MappingTo<StudentExaminationDTO>();
                return result;
            }
        }

        /// <summary>
        /// 删除学生答题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteStudentExaminationByCodeDBAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteStudentExaminationByCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<StudentExaminationEntity>.Filter.Where(c => c.Code == code && c.IsDelete == false);
                var findEntity = await _studentExaminationDBRepository.FindOneAsync(filter);
                if (findEntity != null)
                {
                    var update = Builders<StudentExaminationEntity>.Update.Set(x => x.IsDelete, true).Set(c => c.UpdateTime, DateTime.UtcNow);
                    return await _studentExaminationDBRepository.UpdateOneAsync(filter, update) > 0;
                }
                return false;
            }
        }

        /// <summary>
        /// 同意学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> JoinClassApprovedDBAsync(JoinClassApprovedDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} JoinClassApprovedDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.UserGroupCode && c.CreatorCode == request.TeacherCode);
                var userGroupInfo = await _userGroupDBRepository.FindOneAsync(filter);
                if (userGroupInfo != null)
                {
                    foreach (var student in userGroupInfo.Students)
                    {
                        if (request.Students.Any(a => a == student.Code))
                        {
                            student.LearnerStatus = LearnerStatusEnum.Accept;
                        }
                    }
                    var addCount = userGroupInfo.Students.Count(c => c.LearnerStatus == LearnerStatusEnum.Accept);
                    if (addCount > userGroupInfo.MaxPeople)
                    {
                        //超出最大人数
                        return false;
                    }
                    result = await _userGroupDBRepository.UpdateOneAsync("Code", request.UserGroupCode, userGroupInfo);
                }
                return result;
            }
        }

        /// <summary>
        /// 拒绝学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> JoinClassRejectedDBAsync(JoinClassRejectedDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} JoinClassRejectedDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = false;
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.UserGroupCode && c.CreatorCode == request.TeacherCode);
                var userGroupInfo = await _userGroupDBRepository.FindOneAsync(filter);
                if (userGroupInfo != null)
                {
                    foreach (var student in userGroupInfo.Students)
                    {
                        if (request.Students.Any(a => a == student.Code))
                        {
                            student.LearnerStatus = LearnerStatusEnum.Reject;
                        }
                    }
                    result = await _userGroupDBRepository.UpdateOneAsync("Code", request.UserGroupCode, userGroupInfo);
                }
                return result;
            }
        }

        /// <summary>
        /// 查询学生列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<UserGroupStudentsDTO>> FindStudentListDBAsync(FindStudentListDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindStudentListDBAsync execute", $"{this.GetType().Name}"))
            {
                var students = new List<UserGroupStudentsDTO>();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.UserGroupCode && c.CreatorCode == request.TeacherCode);
                var userGroupInfos = await _userGroupDBRepository.FindOneAsync(filter);
                if (userGroupInfos != null && userGroupInfos.Students != null && userGroupInfos.Students.Count > 0)
                {
                    var userCodes = new List<string>();
                    if (request.LearnerStatus == LearnerStatusEnum.Unknown)
                    {
                        userCodes = userGroupInfos.Students.Select(s => s.Code).ToList();
                    }
                    else
                    {
                        userCodes = userGroupInfos.Students.Where(s => s.LearnerStatus == request.LearnerStatus).Select(s => s.Code).ToList();
                    }
                    if (userCodes != null && userCodes.Count > 0)
                    {
                        MongoCustomerFilter<MongoTable> findUsers = new MongoCustomerFilter<MongoTable>();
                        findUsers.AppendIn("BaseData.Code", userCodes);
                        var users = await _userDBRepository.FindAllAsync(findUsers.AndCombine());
                        var orgCodes = users.Select(c => c.OrganizationCode).Distinct().ToList();
                        MongoCustomerFilter<MongoTable> findOrgs = new MongoCustomerFilter<MongoTable>();
                        findOrgs.AppendIn("BaseData.Code", orgCodes);
                        var orgs = await _organizationDBRepository.FindAllAsync(findOrgs.AndCombine());
                        foreach (var student in users)
                        {
                            var name = string.IsNullOrWhiteSpace(student.FullName) ? student.UserName : student.FullName;
                            if (Regex.IsMatch(name, "[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(student.Phone))
                            {
                                name = student.Phone;
                            }
                            if (Regex.IsMatch(name, "[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(student.Email))
                            {
                                name = student.Email;
                            }
                            var userGroupInfo = userGroupInfos.Students.FirstOrDefault(f => f.Code == student.Code);
                            students.Add(new UserGroupStudentsDTO
                            {
                                Code = student.Code,
                                Name = name,
                                Phone = student.Phone,
                                IsPay = false, // todo
                                LearnerStatus = userGroupInfo.LearnerStatus,
                                UserGroupCode = request.UserGroupCode,
                                OrganizationName = orgs.Find(c => c.Code == student?.OrganizationCode)?.OrganizationName ?? string.Empty
                            });
                        }
                    }
                }
                return students;
            }
        }

        #region Migrate
        /// <summary>
        /// 批量新增课程标签数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertCourseLabelDataDBAsync(BatchInsertCourseLabelRequest request)
        {
            bool res = false;
            if (request?.CourseLabelMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.CourseLabelMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<CourseLabelEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var courseLabelList = await _courseLabelDBRepository.FindAllAsync(filter);
                //求同存异
                if (courseLabelList?.Count > 0)
                {
                    var existCodes = courseLabelList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.CourseLabelMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.CourseLabelMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<CourseLabelEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _courseLabelDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _courseLabelDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(
                            f =>
                            {
                                var data = f.MappingTo<CourseLabelEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _courseLabelDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _courseLabelDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.CourseLabelMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<CourseLabelEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _courseLabelDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _courseLabelDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 批量新增用户组数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertUserGroupDataDBAsync(BatchInsertUserGroupRequest request)
        {
            bool res = false;
            if (request?.UserGroupMigratorys?.Count > 0)
            {
                var defaultGroup = GetUserGroupInfoByAgent();
                //找到相同的
                var ids = request.UserGroupMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<UserGroupEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var userGroupList = await _userGroupDBRepository.FindAllAsync(filter);
                //求同存异
                if (userGroupList?.Count > 0)
                {
                    var existCodes = userGroupList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.UserGroupMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.UserGroupMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<UserGroupEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _userGroupDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _userGroupDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var updateList = new List<UserGroupEntity>();
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<UserGroupEntity>();
                            updateEntity.Code = f.Code.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            var curUserGroupInfo = userGroupList.FirstOrDefault(c => c.Code == f.Code);
                            curUserGroupInfo.Students = curUserGroupInfo.Students ?? new List<LearnerEntity>();
                            foreach (var item in updateEntity.Students)
                            {
                                //var findEntity = curUserGroupInfo.Students.FirstOrDefault(c => c.Code == item.Code);
                                if (curUserGroupInfo.Students.Any(c => c.Code == item.Code))
                                {
                                    //newStudents.Add(findEntity);
                                }
                                else
                                {
                                    curUserGroupInfo.Students.Add(item);
                                }
                            }
                            updateEntity.Students = curUserGroupInfo.Students;
                            updateList.Add(updateEntity);
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _userGroupDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity).Result;
                            }
                        }
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _userGroupDBRepository.UpdateAllWithoutOplogLogAsync(updateList);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.UserGroupMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<UserGroupEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _userGroupDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _userGroupDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 批量新增课程数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertCourseDataDBAsync(BatchInsertCourseRequest request)
        {
            bool res = false;
            if (request?.VideoMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.VideoMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<VideoInfoEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var videoList = await _videoInfoDBRepository.FindAllAsync(filter);
                //求同存异
                if (videoList?.Count > 0)
                {
                    var existCodes = videoList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.VideoMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.VideoMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<VideoInfoEntity>();
                                data.Code = f.Code.Trim();
                                data.IsPublic = false;
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _videoInfoDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _videoInfoDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(
                            f =>
                            {
                                var data = f.MappingTo<VideoInfoEntity>();
                                data.Code = f.Code.Trim();
                                data.IsPublic = false;
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _videoInfoDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _videoInfoDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.VideoMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<VideoInfoEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _videoInfoDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _videoInfoDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }

            if (request?.CourseMigratorys?.Count > 0)
            {
                //课程,包含视频和直播
                //报名
                //找到相同的
                var ids = request.CourseMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<CourseInfoEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var courseList = await _courseInfoDBRepository.FindAllAsync(filter);
                //求同存异
                if (courseList?.Count > 0)
                {
                    var existCodes = courseList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.CourseMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.CourseMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<CourseInfoEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                data.Assistants = f.Assistants.MappingTo<List<StudentInfoEntity>>();
                                data.Experts = f.Experts.MappingTo<List<StudentInfoEntity>>();
                                data.BindExams = f.BindExams.MappingTo<List<CourseExaminationEntity>>();//待验证                                
                                if (data.IsAgentCourse)
                                {
                                    data.UserGroupCodes = new List<string>() { GetUserGroupInfoByAgent().Code };
                                }
                                //保存报名学生
                                var signInStudentResult = SaveSignUpStudentData(f.Students, data.Code, request.SyncType);
                                var liveRoomResult = CreateLiveRoom(f);
                                var examResult = BindExamResult(f);
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _courseInfoDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _courseInfoDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        //新增
                        var req = updateData.Select(
                            f =>
                            {
                                var data = f.MappingTo<CourseInfoEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                data.Assistants = f.Assistants.MappingTo<List<StudentInfoEntity>>();
                                data.Experts = f.Experts.MappingTo<List<StudentInfoEntity>>();
                                data.BindExams = f.BindExams.MappingTo<List<CourseExaminationEntity>>();//待验证                                
                                if (data.IsAgentCourse)
                                {
                                    data.UserGroupCodes = new List<string>() { GetUserGroupInfoByAgent().Code };
                                }
                                //保存报名学生
                                var signInStudentResult = SaveSignUpStudentData(f.Students, data.Code, request.SyncType);
                                var liveRoomResult = CreateLiveRoom(f);
                                var examResult = BindExamResult(f);
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _courseInfoDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _courseInfoDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.CourseMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<CourseInfoEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            data.Assistants = f.Assistants.MappingTo<List<StudentInfoEntity>>();
                            data.Experts = f.Experts.MappingTo<List<StudentInfoEntity>>();
                            data.BindExams = f.BindExams.MappingTo<List<CourseExaminationEntity>>();//待验证
                            if (data.IsAgentCourse)
                            {
                                var userGroupInfo = GetUserGroupInfoByAgent();
                                if (!string.IsNullOrWhiteSpace(userGroupInfo?.Code))
                                {
                                    data.UserGroupCodes = new List<string>() { userGroupInfo.Code };
                                }
                                else
                                {
                                    data.UserGroupCodes = new List<string>();
                                }
                            }
                            //保存报名学生
                            var liveRoomResult = CreateLiveRoom(f);
                            var signInStudentResult = SaveSignUpStudentData(f.Students, data.Code, request.SyncType);
                            var examResult = BindExamResult(f);
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _courseInfoDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _courseInfoDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }

            //个人上传图像
            if (request.RemoteExaminationMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.RemoteExaminationMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<RemoteExaminationDataEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var remoteExaminationList = await _remoteExaminationDataDBRepository.FindAllAsync(filter);
                //求同存异
                if (remoteExaminationList?.Count > 0)
                {
                    var existCodes = remoteExaminationList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.RemoteExaminationMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.RemoteExaminationMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<RemoteExaminationDataEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _remoteExaminationDataDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _remoteExaminationDataDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(
                            f =>
                            {
                                var data = f.MappingTo<RemoteExaminationDataEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _remoteExaminationDataDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _remoteExaminationDataDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.RemoteExaminationMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<RemoteExaminationDataEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _remoteExaminationDataDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _remoteExaminationDataDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }

            //考试，试卷绑定，学生考核结果
            if (request.ExamMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.ExamMigratorys.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<ExaminationPaperEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var examPaperList = await _examinationPaperDBRepository.FindAllAsync(filter);
                //求同存异
                if (examPaperList?.Count > 0)
                {
                    var existCodes = examPaperList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.ExamMigratorys.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.ExamMigratorys.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<ExaminationPaperEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                data.QuestionList = TransferQuestionDetails(f.QuestionList);
                                //批量新增question
                                var questionResult = BatchAddQuestionDatas(f.ExaminationQuestions);
                                //批量新增student答题情况
                                var answerResult = BatchAddStudentExaminationDatas(f.StudentExamination);
                                if (request.SyncType == SyncDBEnum.Synchronize)
                                {
                                    var r = _examinationPaperDBRepository.InsertOneForAsyncDBAsync(data).Result;
                                }
                                return data;
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _examinationPaperDBRepository.InsertAllWithoutOplogLogAsync(req);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(
                            f =>
                            {
                                try
                                {
                                    var data = f.MappingTo<ExaminationPaperEntity>();
                                    data.Code = f.Code.Trim();
                                    data.IsDelete = f.IsDelete;

                                    data.QuestionList = TransferQuestionDetails(f.QuestionList);
                                    //批量新增question
                                    var questionResult = BatchAddQuestionDatas(f.ExaminationQuestions);
                                    //批量新增student答题情况
                                    var answerResult = BatchAddStudentExaminationDatas(f.StudentExamination);
                                    if (request.SyncType == SyncDBEnum.Synchronize)
                                    {
                                        var r = _examinationPaperDBRepository.UpdateOneForSyncDBAsync("Code", data.Code, data).Result;
                                    }
                                    return data;
                                }
                                catch (Exception et)
                                {
                                    Console.WriteLine(et.Message);
                                    return new ExaminationPaperEntity();
                                }
                            }
                        ).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _examinationPaperDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                    }
                }
                else
                {
                    //新增
                    var req = request.ExamMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<ExaminationPaperEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            data.QuestionList = TransferQuestionDetails(f.QuestionList);
                            //批量新增question
                            var questionResult = BatchAddQuestionDatas(f.ExaminationQuestions);
                            //批量新增student答题情况
                            var answerResult = BatchAddStudentExaminationDatas(f.StudentExamination);
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _examinationPaperDBRepository.InsertOneForAsyncDBAsync(data).Result;
                            }
                            return data;
                        }
                    ).ToList();
                    if (request.SyncType == SyncDBEnum.Migrate)
                    {
                        res = await _examinationPaperDBRepository.InsertAllWithoutOplogLogAsync(req);
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 反向更新考试结果
        /// </summary>
        private bool BindExamResult(CourseMigratoryInfo courseInfo)
        {
            var res = false;
            if (courseInfo?.BindExams?.Count > 0)
            {
                if (courseInfo.UserGroupCodes?.Count > 0)
                {
                    var builder = Builders<UserGroupEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class);
                    var inFilter = builder.In(c => c.Code, courseInfo.UserGroupCodes);
                    filter = filter & inFilter;
                    var userGroupList = _userGroupDBRepository.FindAllAsync(filter).Result;
                    var studentList = new List<string>();
                    if (userGroupList?.Count > 0)
                    {
                        foreach (var item in userGroupList)
                        {
                            if (item.Students?.Count > 0)
                            {
                                foreach (var studentInfo in item.Students)
                                {
                                    if (!studentList.Contains(studentInfo.Code))
                                    {
                                        studentList.Add(studentInfo.Code);
                                    }
                                }
                            }
                        }
                    }
                    var examCodes = courseInfo.BindExams.Select(c => c.Code).Distinct().ToList();
                    var examBuilder = Builders<StudentExaminationEntity>.Filter;
                    var examFilter = examBuilder.Where(c => c.IsDelete == false);
                    var examInFilter = examBuilder.In(c => c.ExaminationCode, examCodes);
                    examFilter = examFilter & examInFilter;
                    var studentExaminationList = _studentExaminationDBRepository.FindAllAsync(examFilter).Result;
                    if (studentExaminationList?.Count > 0)
                    {
                        int count = 0;
                        foreach (var studentExam in studentExaminationList)
                        {
                            if (studentList.Contains(studentExam.StudentCode))
                            {
                                studentExam.RelevanceCode = courseInfo.Code;
                                var r = _studentExaminationDBRepository.UpdateOneAsync("Code", studentExam.Code, studentExam).Result;
                                count++;
                            }
                        }
                        res = count > 0 ? true : false;
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 获取代理商用户组
        /// </summary>
        private UserGroupEntity GetUserGroupInfoByAgent()
        {
            var builder = Builders<UserGroupEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.UserGroupType == UserGroupEnum.Class && c.IsAgent == true && c.Name == "DefaultDistributorGroup");
            var userGroupEntity = _userGroupDBRepository.FindOneAsync(filter).Result ?? new UserGroupEntity();
            if (userGroupEntity == null || string.IsNullOrEmpty(userGroupEntity.Code))
            {
                //新增一个
                var shortCode = "";
                for (var i = 0; i < 5; i++)
                {
                    shortCode = GeneralShortCode();
                    if (!string.IsNullOrWhiteSpace(shortCode))
                    {
                        var userGroup = GetUserGroupDBByShortCodeAsync(shortCode).Result;
                        if (userGroup == null || string.IsNullOrWhiteSpace(userGroup.Code))
                        {
                            break;
                        }
                    }
                    shortCode = string.Empty;
                }
                userGroupEntity = new UserGroupEntity()
                {
                    Name = "DefaultDistributorGroup",
                    ShortCode = shortCode,
                    MaxPeople = 1000000,
                    Students = new List<LearnerEntity>(),
                    CreatorCode = "",
                    IsAgent = true
                };
                var identity = _userGroupDBRepository.InsertOneAsync(userGroupEntity).Result;
                userGroupEntity.Code = identity;
            }
            return userGroupEntity;
        }

        /// <summary>
        /// 同步学生答题情况
        /// </summary>
        private bool BatchAddStudentExaminationDatas(IList<StudentExaminationDTO> studentExamination)
        {
            long res = 0;
            if (studentExamination?.Count > 0)
            {
                //找到相同的
                var ids = studentExamination.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<StudentExaminationEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var studentExaminationList = _studentExaminationDBRepository.FindAllAsync(filter).Result;
                //求同存异
                if (studentExaminationList?.Count > 0)
                {
                    var existCodes = studentExaminationList.Select(c => c.Code).Distinct().ToList();
                    var updateData = studentExamination.Where(c => existCodes.Contains(c.Code));
                    var insertData = studentExamination.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<StudentExaminationEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                data.StudentAnswers = f.StudentAnswers.MappingTo<List<ExaminationAnswerEntity>>();
                                return data;
                            }
                        ).ToList();
                        res = _studentExaminationDBRepository.InsertAllAsync(req).Result;
                    }
                    if (updateData?.Count() > 0)
                    {
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<StudentExaminationEntity>();
                            updateEntity.Code = f.Code.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            updateEntity.StudentAnswers = f.StudentAnswers.MappingTo<List<ExaminationAnswerEntity>>();
                            var r = _studentExaminationDBRepository.UpdateOneAsync("Code", updateEntity.Code, updateEntity).Result;
                        }
                    }
                }
                else
                {
                    //新增
                    var req = studentExamination.Select(
                        f =>
                        {
                            var data = f.MappingTo<StudentExaminationEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            data.StudentAnswers = f.StudentAnswers.MappingTo<List<ExaminationAnswerEntity>>();
                            return data;
                        }
                    ).ToList();
                    res = _studentExaminationDBRepository.InsertAllAsync(req).Result;
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 批量新增question数据
        /// </summary>
        private bool BatchAddQuestionDatas(IList<ExaminationQuestionDTO> questionList)
        {
            long res = 0;
            if (questionList?.Count > 0)
            {
                //找到相同的
                var ids = questionList.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<ExaminationQuestionEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var examQuestionList = _examinationQuestionDBRepository.FindAllAsync(filter).Result;
                //求同存异
                if (examQuestionList?.Count > 0)
                {
                    var existCodes = examQuestionList.Select(c => c.Code).Distinct().ToList();
                    var updateData = questionList.Where(c => existCodes.Contains(c.Code));
                    var insertData = questionList.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<ExaminationQuestionEntity>();
                                data.Code = f.Code.Trim();
                                data.IsDelete = f.IsDelete;
                                return data;
                            }
                        ).ToList();
                        res = _examinationQuestionDBRepository.InsertAllAsync(req).Result;
                    }
                    if (updateData?.Count() > 0)
                    {
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<ExaminationQuestionEntity>();
                            updateEntity.Code = f.Code.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            var r = _examinationQuestionDBRepository.UpdateOneAsync("Code", updateEntity.Code, updateEntity).Result;
                        }
                    }
                }
                else
                {
                    //新增
                    var req = questionList.Select(
                        f =>
                        {
                            var data = f.MappingTo<ExaminationQuestionEntity>();
                            data.Code = f.Code.Trim();
                            data.IsDelete = f.IsDelete;
                            return data;
                        }
                    ).ToList();
                    res = _examinationQuestionDBRepository.InsertAllAsync(req).Result;
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 考试详情数据转换
        /// </summary>
        private List<QuestionDetail> TransferQuestionDetails(List<QuestionDetailDTO> questionList)
        {
            var result = new List<QuestionDetail>();
            if (questionList?.Count > 0)
            {
                result = questionList.Select(c => new QuestionDetail()
                {
                    Code = c.Code,
                    Score = c.Score
                }).ToList();
            }
            return result;
        }

        /// <summary>
        /// 直播间拉起，且开始时间大于当前时间48小时，则默认课程已结束
        /// </summary>
        private bool CreateLiveRoom(CourseMigratoryInfo courseInfo)
        {
            var userCodes = new List<string>();
            if (courseInfo.Experts?.Count > 0)
            {
                var expertUsers = courseInfo.Experts.Select(c => c.Code).Distinct().ToList();
                userCodes.AddRange(expertUsers);
            }
            if (courseInfo.Assistants?.Count > 0)
            {
                var assistantsUsers = courseInfo.Assistants.Select(c => c.Code).Distinct().ToList();
                userCodes.AddRange(assistantsUsers);
            }
            //发起人
            var initiatorCode = courseInfo.TeacherCode;
            if (!string.IsNullOrWhiteSpace(initiatorCode) && !userCodes.Contains(initiatorCode))
            {
                userCodes.Add(initiatorCode);
            }
            var isNeedPay = courseInfo.Price > 0 ? true : false;
            //观看人
            var viewerCodes = new List<string>();
            if (isNeedPay)
            {
                var selectPayUsers = courseInfo.Students.Where(c => c.IsPay).Select(c => c.Code).Distinct().ToList();
                viewerCodes.AddRange(selectPayUsers);
            }
            else
            {
                var selectUsers = courseInfo.Students.Select(c => c.Code).Distinct().ToList();
                viewerCodes.AddRange(selectUsers);
            }
            var users = FindUserListByUserCodesAsync(userCodes).Result;
            //生成会诊用户/设备信息
            var tokenManager = CacheMaintenance.Instance.Get<ITokensManager>();
            var liveMembers = new List<LiveMemberDTO>();
            foreach (var user in users)
            {
                var userCode = user.UserCode;
                //var isInitiator = userCode == initiatorCode;
                var userName = !string.IsNullOrWhiteSpace(user.FullName) ? user.FullName : user.UserName;
                var liveMember = new LiveMemberDTO
                {
                    Code = userCode,
                    Name = userName,
                    MemberType = LiveMemberEnum.User,
                    HeadImageToken = user.HeadImageUrl,
                    Status = LiveMemberStatus.Default,
                    MuteOpened = false,
                    VideoOpened = true,
                };
                liveMembers.Add(liveMember);
            }
            var room = new LiveRoomDTO
            {
                LiveRoomCode = courseInfo.Code,
                Name = courseInfo.Name,
                RelatedCode = courseInfo.Code,
                BusinessModule = BusinessModuleEnum.LivingCourse,
                Status = LiveRoomStatus.Default,
                UserInfos = liveMembers.Where(x => x.MemberType == LiveMemberEnum.User)?.ToList() ?? new List<LiveMemberDTO>(),
                DeviceInfos = liveMembers.Where(x => x.MemberType == LiveMemberEnum.Device)?.ToList() ?? new List<LiveMemberDTO>(),
            };
            var req = new AddOrUpdateLiveRoomDBRequest()
            {
                Data = room
            };
            var res = AddOrUpdateLiveRoomAsync(req).Result;
            if (!string.IsNullOrEmpty(res))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 保存学生报名数据
        /// </summary>
        private bool SaveSignUpStudentData(IList<StudentInfoDTO> students, string courseCode, SyncDBEnum syncType)
        {
            bool res = false;
            if (students?.Count > 0)
            {
                var signInStudents = students.Where(c => !string.IsNullOrEmpty(c.Code) && c.SignCourseStatus == StudentCourseStatusEnum.SignUp)
                .Select(c => new SignStudentEntity()
                {
                    Code = Guid.NewGuid().ToString("N"),
                    CreateTime = DateTime.UtcNow,
                    UpdateTime = DateTime.UtcNow,
                    IsDelete = false,
                    StudentCode = c.Code,
                    CourseOrAlbumCode = courseCode,
                    CourseAppearType = CourseAppearTypeEnum.Independent,
                    IsPay = c.IsPay,
                    SignCourseStatus = StudentCourseStatusEnum.SignUp
                }).ToList();
                //查找是否有报过名的
                var ids = signInStudents.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<SignStudentEntity>.Filter;
                var inFilter = builder.In(c => c.StudentCode, ids);
                var filter = builder.Where(c => c.CourseOrAlbumCode == courseCode);
                filter = filter & inFilter;
                var signInList = _signStudentDBRepository.FindAllAsync(filter).Result;
                if (signInList?.Count > 0)
                {
                    var existCodes = signInList.Select(c => c.Code).Distinct().ToList();
                    var updateData = signInStudents.Where(c => existCodes.Contains(c.Code));
                    var insertData = signInStudents.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        if (syncType == SyncDBEnum.Migrate)
                        {
                            res = _signStudentDBRepository.InsertAllWithoutOplogLogAsync(insertData.ToList()).Result;
                        }
                        else
                        {
                            var r = _signStudentDBRepository.InsertAllAsync(insertData.ToList()).Result;
                            if (r > 0)
                            {
                                res = true;
                            }
                            else
                            {
                                res = false;
                            }
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var updateList = new List<SignStudentEntity>();
                        //修改
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<SignStudentEntity>();
                            updateEntity.Code = f.Code.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            updateEntity.UpdateTime = DateTime.UtcNow;
                            if (syncType == SyncDBEnum.Synchronize)
                            {
                                var r = _signStudentDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity).Result;
                            }
                            updateList.Add(updateEntity);
                        }
                        if (syncType == SyncDBEnum.Migrate)
                        {
                            var r = _signStudentDBRepository.UpdateAllWithoutOplogLogAsync(updateList).Result;
                        }
                    }
                }
                else
                {
                    //新增
                    if (syncType == SyncDBEnum.Migrate)
                    {
                        res = _signStudentDBRepository.InsertAllWithoutOplogLogAsync(signInStudents).Result;
                    }
                    else
                    {
                        var r = _signStudentDBRepository.InsertAllAsync(signInStudents).Result;
                        if (r > 0)
                        {
                            res = true;
                        }
                        else
                        {
                            res = false;
                        }
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 更新课程状态
        /// </summary>
        /// <param name="courseCode"></param>
        /// <param name="status"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseStatusEnumAsync(string courseCode, CourseStatusEnum status)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseStatusEnumAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _courseInfoDBRepository.FindOneAsync("Code", courseCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (entity.Status != status && entity.Status != CourseStatusEnum.HasEnded)
                    {
                        entity.Status = status;
                        var result = await _courseInfoDBRepository.UpdateOneAsync("Code", courseCode, entity, true);
                        // //修改日程状态
                        // var updateScheduleStateRequest = new UpdateScheduleStateDBRequest
                        // {
                        //     RelevanceCode = entity.Code,
                        //     Status = entity.Status,
                        // };
                        // await UpdateScheduleStateDBAsync(updateScheduleStateRequest);
                        return result;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// 检查课程标签是否存在课程相关信息
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> CheckCourseLableExistCourseInfoByDBAsync(CourseLabelRequest request)
        {
            var res = false;
            var albumFilter = Builders<CourseAlbumEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == request.Code));
            res =  await  _courseAlbumDBRepository.Exists(albumFilter);
            if (res)
            {
                return res;
            }
            var courseFilter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == request.Code) || c.CaseLabelCodes.Any(dx => dx == request.Code)));
            res =  await  _courseInfoDBRepository.Exists(courseFilter);
            if (res)
            {
                return res;
            }

            var videoFilter = Builders<VideoInfoEntity>.Filter.Where(c => c.IsDelete == false && c.CourseLabelCodes.Any(dx => dx == request.Code));
            res =  await  _videoInfoDBRepository.Exists(videoFilter);
            if (res)
            {
                return res;
            }

            var questionFilter = Builders<ExaminationQuestionEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == request.Code) || c.CaseLabelCodes.Any(dx => dx == request.Code)));
            res =  await  _examinationQuestionDBRepository.Exists(questionFilter);
            if (res)
            {
                return res;
            }

            var paperFilter = Builders<ExaminationPaperEntity>.Filter.Where(c => c.IsDelete == false && (c.CourseLabelCodes.Any(dx => dx == request.Code) || c.CaseLabelCodes.Any(dx => dx == request.Code)));
            res =  await  _examinationPaperDBRepository.Exists(paperFilter);
            if (res)
            {
                return res;
            }
            return res;
        }
        #endregion

        #region update education db
        /// <summary>
        /// 更新用户权限针对杏聆荟学院
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateUserFeatureForEducationDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForEducationDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    //FlyinsonoCollege
                    var featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "FlyinsonoCollegeCourseIndex");
                    if (string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        var addRequest = new List<(UserFeatureDO adminFeatureDO, Dictionary<string, string> platformDatas, string extensionData)>();
                        var model = new UserFeatureDO//对应  service
                        {
                            FeatureCode = "FlyinsonoCollegeCourseIndex",
                            FatherCode = "FlyinsonoCollege",
                            FeatureName = "课程首页",
                            UniqueCode = "FlyinsonoCollegeCourseIndex",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "课程首页"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Курсы"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Active Courses"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Curso de actividades"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new UserFeatureDO//对应  service
                        {
                            FeatureCode = "FlyinsonoCollegeMyCourses",
                            FatherCode = "FlyinsonoCollege",
                            FeatureName = "我的课程",
                            UniqueCode = "FlyinsonoCollegeMyCourses",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "我的课程"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Мои курсы."
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "My Courses"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Mi curso"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));

                        model = new UserFeatureDO//对应  service
                        {
                            FeatureCode = "FlyinsonoCollegeVideoClass",
                            FatherCode = "FlyinsonoCollege",
                            FeatureName = "视频课堂",
                            UniqueCode = "FlyinsonoCollegeVideoClass",//审核申请
                            LanguageConfigs = new List<UserFeatureLanguageConfigDO>() {
                                new UserFeatureLanguageConfigDO() {
                                    Language = "zh-CN",
                                    FeatureName = "视频课堂"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ru-RU",
                                    FeatureName = "Видео Класс"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "en-US",
                                    FeatureName = "Video Class"
                                },
                                new UserFeatureLanguageConfigDO() {
                                    Language = "es-ES",
                                    FeatureName = "Clase de video"
                                }
                            }
                        };
                        addRequest.Add((model, new Dictionary<string, string>(), ""));
                        await _userFeatureDBRepository.InsertAllAsync(addRequest, false);
                    }

                    //查找所有的角色
                    var userRoleList = await _roleDBRepository.FindAllAsync();
                    if (userRoleList?.Count > 0)
                    {
                        userRoleList = userRoleList.Where(c => c.IsDelete == false)?.ToList() ?? new List<UserRoleDO>();
                        foreach(var roleInfo in userRoleList)
                        {
                            if (!string.IsNullOrEmpty(roleInfo?.UserGroupCode))
                            {
                                var userAuthorityGroupDO = await _userAuthorityGroupDBRepository.FindOneAsync("Code", roleInfo?.UserGroupCode);
                                if (userAuthorityGroupDO.Features?.Count > 0)
                                {
                                    userAuthorityGroupDO.Features.Add("FlyinsonoCollege");
                                    userAuthorityGroupDO.Features.Add("FlyinsonoCollegeCourseIndex");
                                    userAuthorityGroupDO.Features.Add("FlyinsonoCollegeMyCourses");
                                    userAuthorityGroupDO.Features.Add("FlyinsonoCollegeVideoClass");
                                    userAuthorityGroupDO.Features = userAuthorityGroupDO.Features.Distinct().ToList();
                                }
                                else
                                {
                                    userAuthorityGroupDO.Features = new List<string>() { "JPGL", "JPCL", "KCZJLB", "JPUGL", "JPCLL", "KCSPGL", "TMLB", "SJGL", "FlyinsonoCollege", "FlyinsonoCollegeCourseIndex", "FlyinsonoCollegeMyCourses", "FlyinsonoCollegeVideoClass" };
                                }
                                res = await _userAuthorityGroupDBRepository.UpdateOneForSyncDBAsync("Code", userAuthorityGroupDO.Code, userAuthorityGroupDO, "", true, false);
                            }
                        }
                        
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService UpdateUserFeatureForEducationDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }
    
        /// <summary>
        /// 更新用户权限针对杏聆荟学院
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateUserFeatureForROEducationDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserFeatureForROEducationDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    //FlyinsonoCollege
                    var featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "FlyinsonoCollegeCourseIndex");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ro-RO",
                                    FeatureName = "Curs acasă"
                                });
                        res =  await _userFeatureDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "FlyinsonoCollegeMyCourses");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ro-RO",
                                    FeatureName = "Cursul meu"
                                });
                        res =  await _userFeatureDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }

                    featureInfo = await _userFeatureDBRepository.FindOneAsync("Code", "FlyinsonoCollegeVideoClass");
                    if (!string.IsNullOrEmpty(featureInfo?.Code))
                    {
                        featureInfo.LanguageConfigs.Add(
                                new UserFeatureLanguageConfigDO() {
                                    Language = "ro-RO",
                                    FeatureName = "Clasa video"
                                });
                        res =  await _userFeatureDBRepository.UpdateOneAsync("Code", featureInfo.Code, featureInfo, string.Empty, true, false);
                    }
                   
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService UpdateUserFeatureForROEducationDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 更新多媒体课程默认是否只包含视频课
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateCourseVideoTypeDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateCourseVideoTypeDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.IsDelete == false && c.CourseType == CourseTypeEnum.Multimedia);
                    var courseList = await _courseInfoDBRepository.FindAllAsync(filter);
                    if (courseList?.Count > 0)
                    {
                        foreach(var courseInfo in courseList)
                        {
                            courseInfo.IsJustVideoCourse = true;
                            res = await _courseInfoDBRepository.UpdateOneForSyncDBAsync("Code", courseInfo.Code, courseInfo, true, false);
                        }
                        
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService UpdateUserFeatureForEducationDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }
        #endregion 

        /// <summary>
        /// 保存用户每次课程浏览记录-用户离线
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> VisitorBrowsedLeaveByDBAsync(CourseBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} VisitorBrowsedLeaveByDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    var filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.UserCode == request.UserCode);
                    var courseBrowsedRecordInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(courseBrowsedRecordInfo?.Code))
                    {
                        WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService VisitorBrowsedLeaveByDBAsync 0, Code: {courseBrowsedRecordInfo.Code}");
                        //已经找到，则更新时间和次数
                        courseBrowsedRecordInfo.UpdateTime = DateTime.UtcNow;
                        courseBrowsedRecordInfo.IsOnline = false;
                        
                        if (courseBrowsedRecordInfo.BrowsedTimeRecords == null || courseBrowsedRecordInfo.BrowsedTimeRecords.Count <= 0)
                        {
                            var model = new BrowsedTimeRecordEntity() 
                            {
                                StartTime = courseBrowsedRecordInfo.CreateTime,
                                EndTime = DateTime.UtcNow,
                                Duration = (int)Math.Ceiling((DateTime.UtcNow - courseBrowsedRecordInfo.CreateTime).TotalMinutes)
                            };
                             if (model.Duration <= 0)
                            {
                                model.Duration = 1;
                            }
                            courseBrowsedRecordInfo.BrowsedTimeRecords = new List<BrowsedTimeRecordEntity>() { model };
                        }
                        else
                        {
                            var findEntity = courseBrowsedRecordInfo.BrowsedTimeRecords.OrderByDescending(c => c.StartTime).FirstOrDefault();
                            findEntity.EndTime = DateTime.UtcNow;
                            findEntity.Duration = (int)Math.Ceiling((DateTime.UtcNow - findEntity.StartTime).TotalMinutes);
                            if (findEntity.Duration <= 0)
                            {
                                findEntity.Duration = 1;
                            }
                        }
                        res = await _courseBrowsedRecordDBRepository.UpdateOneAsync("Code", courseBrowsedRecordInfo.Code, courseBrowsedRecordInfo, true);
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService VisitorBrowsedLeaveByDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 删除课程浏览记录，针对学生取消报名
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> DeleteCourseBrowsedRecordByDBAsync(CourseBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteCourseBrowsedRecordByDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    var filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.UserCode == request.UserCode);
                    var courseBrowsedRecordInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(courseBrowsedRecordInfo?.Code) && courseBrowsedRecordInfo.BrowsedUserType == BrowsedUserTypeEnum.Student) 
                    {
                        res = await _courseBrowsedRecordDBRepository.DeleteOneAsync("Code", courseBrowsedRecordInfo.Code);
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService DeleteCourseBrowsedRecordByDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }

        /// <summary>
        /// 保存课程浏览记录
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<CourseBrowsedRecordDTO> SaveCourseBrowsedRecordByDBAsync(CourseBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveCourseBrowsedRecordByDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = new CourseBrowsedRecordDTO();
                try
                {
                    var filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.UserCode == request.UserCode);
                    var courseBrowsedRecordInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                    if (request.IsHeartRateRequest == true)
                    {
                        if (string.IsNullOrEmpty(courseBrowsedRecordInfo?.Code))
                        {
                            WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync aa, courseBrowsedRecordInfo: { JsonConvert.SerializeObject(courseBrowsedRecordInfo) }");
                            //表示游客第一次进来，既要创建记录也要记录第一次的时间
                            var model = request.MappingTo<CourseBrowsedRecordEntity>();
                            model.CreateTime = DateTime.UtcNow;
                            model.UpdateTime = DateTime.UtcNow;
                            model.BrowsedCount = 1;
                            model.BrowsedTimeRecords = new List<BrowsedTimeRecordEntity>();
                            if (request.IsOnline)
                            {
                                model.BrowsedTimeRecords.Add(new BrowsedTimeRecordEntity() 
                                {
                                    StartTime = DateTime.UtcNow
                                });
                            }
                            var result = await _courseBrowsedRecordDBRepository.InsertOneAsync(model);
                            model.Code = result;
                            res = model.MappingTo<CourseBrowsedRecordDTO>();
                        }
                        else
                        {
                            //已经存在用户，需要更新时间段
                            if (courseBrowsedRecordInfo?.BrowsedTimeRecords?.Count > 0)
                            {
                                WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync bb, courseBrowsedRecordInfo: { JsonConvert.SerializeObject(courseBrowsedRecordInfo) }, request.IsOnline: {request.IsOnline}");
                            
                                if (courseBrowsedRecordInfo.IsOnline && !request.IsOnline)
                                {
                                    //,老的在线，新的不在线，当离线处理
                                    var findEntity = courseBrowsedRecordInfo.BrowsedTimeRecords.OrderByDescending(c => c.StartTime).FirstOrDefault();
                                    if (findEntity.EndTime == DateTime.MinValue) 
                                    {
                                        findEntity.EndTime = DateTime.UtcNow;
                                        findEntity.Duration = (int)Math.Ceiling((DateTime.UtcNow - findEntity.StartTime).TotalMinutes);
                                        if (findEntity.Duration <= 0)
                                        {
                                            findEntity.Duration = 1;
                                        }
                                    }
                                }
                                else if (!courseBrowsedRecordInfo.IsOnline && request.IsOnline)
                                {
                                    //,新的在线，老的不在线，当在线处理
                                    var findEntity = courseBrowsedRecordInfo.BrowsedTimeRecords.OrderByDescending(c => c.StartTime).FirstOrDefault();
                                    if (findEntity.EndTime == DateTime.MinValue) 
                                    {
                                        findEntity.EndTime = DateTime.UtcNow;
                                        findEntity.Duration = (int)Math.Ceiling((DateTime.UtcNow - findEntity.StartTime).TotalMinutes);
                                        if (findEntity.Duration <= 0)
                                        {
                                            findEntity.Duration = 1;
                                        }
                                    }
                                    courseBrowsedRecordInfo.BrowsedTimeRecords.Add(new BrowsedTimeRecordEntity() 
                                    {
                                        StartTime = DateTime.UtcNow
                                    });
                                    courseBrowsedRecordInfo.BrowsedCount += 1;
                                }
                                else
                                {
                                    //老的在线，新的在线,继续保持，不处理
                                    //老的不在线，新的也不在线，不处理
                                }
                            }
                            else
                            {
                                WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync cc, courseBrowsedRecordInfo: { JsonConvert.SerializeObject(courseBrowsedRecordInfo) }, request.IsOnline: {request.IsOnline}");
                            
                                if (request.IsOnline)
                                {
                                    courseBrowsedRecordInfo.BrowsedTimeRecords = new List<BrowsedTimeRecordEntity>();
                                    courseBrowsedRecordInfo.BrowsedTimeRecords.Add(new BrowsedTimeRecordEntity() 
                                    {
                                        StartTime = DateTime.UtcNow
                                    });
                                    courseBrowsedRecordInfo.BrowsedCount += 1;
                                }
                            }
                            courseBrowsedRecordInfo.UpdateTime = DateTime.UtcNow;
                            courseBrowsedRecordInfo.IsOnline = request.IsOnline;
                            var result = await _courseBrowsedRecordDBRepository.UpdateOneAsync("Code", courseBrowsedRecordInfo.Code, courseBrowsedRecordInfo, true);
                            if (result)
                            {
                                res = courseBrowsedRecordInfo.MappingTo<CourseBrowsedRecordDTO>();
                            }
                        }
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(courseBrowsedRecordInfo?.Code))
                        {
                            //表示已添加记录，无需处理
                            WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 0, RelevanceCode: {request.RelevanceCode}");
                            res =  courseBrowsedRecordInfo.MappingTo<CourseBrowsedRecordDTO>();
                        }
                        else
                        {
                            if (!string.IsNullOrEmpty(request.DisplayName))
                            {
                                filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.DisplayName == request.DisplayName);
                                courseBrowsedRecordInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                                if (!string.IsNullOrEmpty(courseBrowsedRecordInfo?.Code))
                                {
                                    WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 1, Code: {courseBrowsedRecordInfo.Code}");
                                    res = courseBrowsedRecordInfo.MappingTo<CourseBrowsedRecordDTO>();
                                    res.ErrorCode = CustomerRpcCode.CourseVisitorNameIsUsed;
                                    return res;
                                }
                                else
                                {
                                    if (request.BrowsedUserType == BrowsedUserTypeEnum.Teacher)
                                    {
                                        //先删除其他的老师
                                        filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.BrowsedUserType == BrowsedUserTypeEnum.Teacher);
                                        var teacherInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                                        
                                        if (!string.IsNullOrEmpty(teacherInfo?.Code))
                                        {
                                            WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 2, Code: {teacherInfo.Code}");
                                            await _courseBrowsedRecordDBRepository.DeleteOneAsync("Code", teacherInfo.Code);
                                        }
                                        else 
                                        {
                                            WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 3, RelevanceCode: {request.RelevanceCode}");
                                        }
                                    }
                                    var model = request.MappingTo<CourseBrowsedRecordEntity>();
                                    model.CreateTime = DateTime.UtcNow;
                                    model.UpdateTime = DateTime.UtcNow;
                                    model.BrowsedCount = 1;
                                    model.BrowsedTimeRecords = new List<BrowsedTimeRecordEntity>();
                                    var result = await _courseBrowsedRecordDBRepository.InsertOneAsync(model);
                                    WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 4, Code: {result}, RelevanceCode: {request.RelevanceCode}");
                                    model.Code = result;
                                    res = model.MappingTo<CourseBrowsedRecordDTO>();
                                }
                            }
                            else
                            {
                                var model = request.MappingTo<CourseBrowsedRecordEntity>();
                                model.CreateTime = DateTime.UtcNow;
                                model.UpdateTime = DateTime.UtcNow;
                                model.BrowsedCount = 1;
                                model.BrowsedTimeRecords = new List<BrowsedTimeRecordEntity>();
                                var result = await _courseBrowsedRecordDBRepository.InsertOneAsync(model);
                                WingServerCommon.Log.Logger.WriteLineInfo($"EducationDBService SaveCourseBrowsedRecordByDBAsync 5, Code: {result}, RelevanceCode: {request.RelevanceCode}");
                                model.Code = result;
                                res = model.MappingTo<CourseBrowsedRecordDTO>();
                            }
                        }
                    }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService SaveCourseBrowsedRecordByDBAsync error, ex: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 根据课程code查询课程浏览记录
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<CourseBrowsedRecordDTO> GetCourseBrowsedRecordByRelevanceCodeDBAsync(QueryBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetCourseBrowsedRecordByRelevanceCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && c.UserCode == request.UserCode);
                    if (!string.IsNullOrEmpty(request.DisplayName))
                    {
                        var nameFilter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.DisplayName == request.DisplayName);
                        filter = filter & nameFilter;
                    } 
                    var courseBrowsedRecordInfo = await _courseBrowsedRecordDBRepository.FindOneAsync(filter);
                    var result = courseBrowsedRecordInfo.MappingTo<CourseBrowsedRecordDTO>();
                    return result;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService GetCourseBrowsedRecordByRelevanceCodeDBAsync error, ex: {ex}");
                    return new CourseBrowsedRecordDTO();
                }
            }
        }

        /// <summary>
        /// 查询课程浏览记录
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<PageResult<CourseBrowsedRecordDTO>> QueryCourseBrowsedRecordsByDBAsync(QueryBrowsedRecordPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryCourseBrowsedRecordsByDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var builder = Builders<CourseBrowsedRecordEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode);
                    if (!string.IsNullOrEmpty(request.DisplayName))
                    {
                        var keyword = EscapeExprSpecialWord(request.DisplayName);//字符串转义
                        var nameFilter = builder.Regex(c => c.DisplayName, new BsonRegularExpression(keyword, "i"));
                        filter = filter & nameFilter;
                    } 
                    //查询分类0-默认，所有；1-除去老师，助教，互动嘉宾；2-报名人数
                    if (request.Category == 1)
                    {
                        filter = filter & builder.Where(c => c.BrowsedUserType == BrowsedUserTypeEnum.Student || c.BrowsedUserType == BrowsedUserTypeEnum.Visitor);
                        //filter = filter & builder.Where(c => (c.BrowsedUserType == BrowsedUserTypeEnum.Student && c.BrowsedTimeRecords != null && c.BrowsedTimeRecords.Any()) || c.BrowsedUserType == BrowsedUserTypeEnum.Visitor);
                    }
                    else if (request.Category == 2)
                    {
                        filter = filter & builder.Where(c => c.BrowsedUserType == BrowsedUserTypeEnum.Student);
                    }
                    var sort = Builders<CourseBrowsedRecordEntity>.Sort.Ascending(v => v.BrowsedUserType).Descending(v => v.IsOnline);
                    var dbPage = new DBPageRequest<CourseBrowsedRecordEntity>();
                    dbPage.CurrentPage = request.PageIndex;
                    dbPage.PageSize = request.PageSize;
                    dbPage.Filter = filter;
                    dbPage.Sort = sort;
                    var pageResult = await _courseBrowsedRecordDBRepository.GetPages(dbPage);
                    var convertResult = PageResultConvert<CourseBrowsedRecordEntity, CourseBrowsedRecordDTO>(pageResult);
                    return convertResult;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService QueryCourseBrowsedRecordsByDBAsync error, ex: {ex}");
                    return new PageResult<CourseBrowsedRecordDTO>();
                }
            }
        }
    
        /// <summary>
        /// 获取课程观看人数和在线人数
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<CourseVisitorsStatisticResult> QueryCourseBrowsedCountByDBAsync(QueryBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryCourseBrowsedCountByDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new CourseVisitorsStatisticResult();
                var builder = Builders<CourseBrowsedRecordEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode);
                result.VisitorCount = (int)(await _courseBrowsedRecordDBRepository.CountDocumentsAsync(filter));

                var onlineFilter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == request.RelevanceCode && (c.BrowsedUserType == BrowsedUserTypeEnum.Visitor || c.BrowsedUserType == BrowsedUserTypeEnum.Student || c.BrowsedUserType == BrowsedUserTypeEnum.Assistant) && c.IsOnline);
                result.OnlineCount = (int)(await _courseBrowsedRecordDBRepository.CountDocumentsAsync(onlineFilter));
                return result;
            }
        }

        /// <summary>
        /// 获取课程观看人数和最多在线人数统计
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<CourseVisitorsStatisticResult> QueryCourseBrowsedStatisticsByDBAsync(QueryBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryCourseBrowsedStatisticsByDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new CourseVisitorsStatisticResult();
                try
                {
                    var courseInfo = await _courseInfoDBRepository.FindOneAsync("Code", request.RelevanceCode);
                    if (!string.IsNullOrEmpty(courseInfo?.Code))
                    {
                        if (courseInfo.StatisticsInfo != null) 
                        {
                            if (courseInfo.Status == CourseStatusEnum.HasEnded)
                            {
                                //实际只要结束了，就不用在重新统计了
                                result.VisitorCount = courseInfo.StatisticsInfo.BrwosedCount;
                                result.SignStudentCount = courseInfo.StatisticsInfo.SignCount;
                                result.MaxOnlineCount = courseInfo.StatisticsInfo.MaxOnlineCount;
                            }
                            else
                            {
                                result = await GetCourseBrowsedStatisticsDataAsync(courseInfo);
                            }
                        }
                        else
                        {
                            result = await GetCourseBrowsedStatisticsDataAsync(courseInfo);
                        }
                    }
                    return result;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService QueryCourseBrowsedStatisticsByDBAsync error, ex: {ex}");
                    return result;
                }
            }
        }

        /// <summary>
        /// 在线课程统计数据
        /// </summary>
        public async Task<CourseVisitorsStatisticResult> GetCourseBrowsedStatisticsDataAsync(CourseInfoEntity courseInfo, bool isNeedUpdataData = true)
        {
            var result = new CourseVisitorsStatisticResult();
            //计算最多同时在线人数
            var list = new List<CourseStatisticsOnlineDetailEntity>();
            try
            {
                var builder = Builders<CourseBrowsedRecordEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == courseInfo.Code && c.BrowsedUserType == BrowsedUserTypeEnum.Student);
                //var filter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == courseInfo.Code && c.BrowsedUserType == BrowsedUserTypeEnum.Student && c.BrowsedTimeRecords != null && c.BrowsedTimeRecords.Any());
                result.SignStudentCount = (int)(await _courseBrowsedRecordDBRepository.CountDocumentsAsync(filter));

                var visitorFilter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == courseInfo.Code && (c.BrowsedUserType == BrowsedUserTypeEnum.Visitor || c.BrowsedUserType == BrowsedUserTypeEnum.Student));
                //var visitorFilter = builder.Where(c => c.IsDelete == false && c.RelevanceCode == courseInfo.Code && (c.BrowsedUserType == BrowsedUserTypeEnum.Visitor || (c.BrowsedUserType == BrowsedUserTypeEnum.Student) && c.BrowsedTimeRecords != null && c.BrowsedTimeRecords.Any()));
                result.VisitorCount = (int)(await _courseBrowsedRecordDBRepository.CountDocumentsAsync(visitorFilter));

                //实际开始时间-当前时间
                var endTime = DateTime.UtcNow;
                if ((courseInfo.Status == CourseStatusEnum.HasEnded || courseInfo.Status == CourseStatusEnum.Overdue) && courseInfo.ActualEndTime > DateTime.MinValue)
                {
                    endTime = courseInfo.ActualEndTime;
                }
                if (courseInfo.ActualStartTime > DateTime.UtcNow.AddYears(-100) && courseInfo.ActualStartTime < endTime)
                {
                    var d = endTime - courseInfo.ActualStartTime;
                    if (d.TotalDays <= 20)
                    {
                        //切割时间，按照每3分钟拆分
                        var startTime = DateTime.Parse(courseInfo.ActualStartTime.ToString());
                        do
                        {
                            var detailInfo = new CourseStatisticsOnlineDetailEntity();
                            detailInfo.CurrentTime = DateTime.Parse(startTime.ToString());
                            list.Add(detailInfo);
                            startTime = startTime.AddMinutes(3);
                        } while (endTime.Subtract(startTime).TotalMinutes >= 0);
                        if (list?.Count > 0)
                        {
                            var recordList = await _courseBrowsedRecordDBRepository.FindAllAsync(visitorFilter);
                            if (recordList?.Count > 0)
                            {
                                foreach (var item in list)
                                {
                                    foreach(var recordItem in recordList)
                                    {
                                        bool existRecord = recordItem?.BrowsedTimeRecords?.Any(record => (record.StartTime <= item.CurrentTime && item.CurrentTime.AddMinutes(3) <= record.EndTime) ||
                                        (record.StartTime <= item.CurrentTime && record.EndTime >= item.CurrentTime) || 
                                        // (record.StartTime <= item.CurrentTime && record.EndTime <= item.CurrentTime.AddMinutes(1)) ||
                                        // (record.StartTime >= item.CurrentTime && record.EndTime >= item.CurrentTime.AddMinutes(1)) || 
                                        (record.StartTime <= item.CurrentTime.AddMinutes(3) && record.EndTime >= item.CurrentTime.AddMinutes(3))) ?? false;
                                        if (existRecord)
                                        {
                                            item.MaxOnlineCount++;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (list?.Count > 0) 
                {
                    result.MaxOnlineCount = list.Max(c => c.MaxOnlineCount);
                }
                //保存到db
                var statisticInfo = new CourseStatisticsInfoEntity()
                {
                    SignCount = result.SignStudentCount,
                    BrwosedCount = result.VisitorCount,
                    MaxOnlineCount = result.MaxOnlineCount,
                    StatisticsDetailInfos = list
                };
                courseInfo.StatisticsInfo = statisticInfo;
                if (isNeedUpdataData) 
                {
                    var res = await _courseInfoDBRepository.UpdateOneAsync("Code", courseInfo.Code, courseInfo, true);
                }
                return result;
            }
            catch(Exception ex)
            {
                WingServerCommon.Log.Logger.WriteLineError($"EducationDBService GetCourseBrowsedStatisticsDataAsync error, ex: {ex}");
                return result;
            }
        }

        /// <summary>
        /// 获取所有的在线游客信息
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<IList<QueryBrowsedRecordRequest>> QueryAllOnlineVisitorInfoByDBAsync()
        {
            var result = new List<QueryBrowsedRecordRequest>();
            var builder = Builders<CourseBrowsedRecordEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false && c.BrowsedUserType == BrowsedUserTypeEnum.Visitor && c.IsOnline == true);
            var dataList = await _courseBrowsedRecordDBRepository.FindAllAsync(filter);
            if (dataList?.Count > 0)
            {
                result = dataList.Select(c => new QueryBrowsedRecordRequest() {
                    RelevanceCode = c.RelevanceCode,
                    UserCode = c.UserCode,
                    DisplayName = c.DisplayName
                })?.ToList() ?? new List<QueryBrowsedRecordRequest>();
            }
            return result;
        }
    
        /// <summary>
        /// 同步批量处理浏览记录 
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> BatchSaveCourseBrowsedRecordDBAsync(CourseMigrateBrowsedRecordRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} BatchSaveCourseBrowsedRecordDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var res = false;
                    long result = 0;
                    if (request.CourseBrowsedRecords?.Count > 0)
                    {
                        var codes = request.CourseBrowsedRecords.Select(c => c.CourseBrowsedRecordCode).Distinct().ToList();
                        var filter = Builders<CourseBrowsedRecordEntity>.Filter.Where(c => c.IsDelete == false && codes.Contains(c.Code));
                        var courseBrowsedRecordInfos = await _courseBrowsedRecordDBRepository.FindAllAsync(filter);
                        if (courseBrowsedRecordInfos?.Count > 0)
                        {
                            //已经找到不存在的，插入，存在的不处理
                            var existRecords = courseBrowsedRecordInfos.Select(c => c.Code).Distinct().ToList();
                            var waitInsertData = request.CourseBrowsedRecords.Where(c => !existRecords.Contains(c.CourseBrowsedRecordCode));
                            if (waitInsertData?.Count() > 0)
                            {
                                var insertData = request.CourseBrowsedRecords.MappingTo<List<CourseBrowsedRecordEntity>>();
                                result = await _courseBrowsedRecordDBRepository.InsertAllAsync(insertData, true);
                            }
                        }
                        else
                        {
                            var insertData = request.CourseBrowsedRecords.MappingTo<List<CourseBrowsedRecordEntity>>();
                            result = await _courseBrowsedRecordDBRepository.InsertAllAsync(insertData, true);
                        }
                    }
                if (result > 0)
                {
                    res = true;
                }
                    return res;
                }
                catch(Exception ex)
                {
                    WingServerCommon.Log.Logger.WriteLineError($"EducationDBService VisitorBrowsedLeaveByDBAsync error, ex: {ex}");
                    return false;
                }
            }
        }
    }
}