using WingServerCommon.Service;
using WingInterfaceLibrary.Request.DBRequest;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.Patient;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingServerCommon.Mapper;
using JsonRpcLite.Services;
using WingMongoDBModule.DO;
using MongoDB.Driver;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DB.Request;
using MongoDB.Bson;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Record;
using WingMongoDBModule.Request;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IPatientInfoDBService
    {
        /// <summary>
        /// 创建病人
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<string> CreatePatientAsync(CreatePatientsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreatePatientAsync execute", $"{this.GetType().Name}"))
            {
                var patientInfoDO = request.Data.MappingTo<PatientInfoDO>();
                var result = await _patientInfoDBRepository.InsertOneAsync(patientInfoDO, request.PlatformDatas, request.ExtensionData);
                // await AddPushPatientAsync(patientInfoDO);
                return result;
            }
        }

        /// <summary>
        /// 创建病人
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<string> CreatePatientByUnregisteredAsync(CreatePatientByUnregisteredDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreatePatientByUnregisteredAsync execute", $"{this.GetType().Name}"))
            {
                var creatorCode = request.CreatorCode;
                var unregisteredPatientCode = request.UnregisteredPatientCode;
                var name = request.PatientName;
                var user = await FindUserByCodeAsync(creatorCode);
                OrganizationDO organization = null;
                if (!string.IsNullOrEmpty(user.RootOrganizationCode))
                {
                    organization = await _organizationDBRepository.FindOneAsync("Code", user.RootOrganizationCode);
                }
                if (string.IsNullOrWhiteSpace(unregisteredPatientCode))
                {
                    var newPatient = new PatientInfoDO();
                    newPatient.IsValid = true;
                    newPatient.CreateTime = DateTime.UtcNow;
                    newPatient.UpdateTime = DateTime.UtcNow;
                    newPatient.Name = name;
                    newPatient.CreatorCode = user.UserCode;
                    newPatient.OrganizationCode = user.OrganizationCode;
                    newPatient.RootOrganizationCode = user.RootOrganizationCode;
                    newPatient.PatientType = organization?.PatientType.ToString() ?? OrganizationPatientTypeEnum.Person.ToString();
                    newPatient.PatientData = new List<WingInterfaceLibrary.DTO.Common.DataItemDTO>();
                    if (newPatient.PatientType == OrganizationPatientTypeEnum.Person.ToString())
                    {
                        newPatient.PatientData.Add(new WingInterfaceLibrary.DTO.Common.DataItemDTO { Key = "Name", Value = name });
                    }
                    else
                    {
                        newPatient.PatientData.Add(new WingInterfaceLibrary.DTO.Common.DataItemDTO { Key = "AnimalInfoName", Value = name });
                    }
                    var insertPatientResult = await _patientInfoDBRepository.InsertOneAsync(newPatient, new Dictionary<string, string>(), string.Empty);
                    // await AddPushPatientAsync(newPatient);
                    return insertPatientResult;
                }
                var unregisteredPatient = await _patientInfoDBRepository.FindOneAsync("Code", unregisteredPatientCode);
                if (unregisteredPatient == null || string.IsNullOrWhiteSpace(unregisteredPatient.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.UnregisteredPatientNotExisted, "Unregistered patient not existed.");
                }
                if (unregisteredPatient.IsValid)
                {
                    ThrowCustomerException(CustomerRpcCode.PatientWasRegistered, "Patient was registered.");
                }
                if (user != null && !string.IsNullOrWhiteSpace(user.RootOrganizationCode))
                {
                    unregisteredPatient.CreatorCode = user.UserCode;
                    unregisteredPatient.OrganizationCode = user.OrganizationCode;
                    unregisteredPatient.RootOrganizationCode = user.RootOrganizationCode;
                    unregisteredPatient.PatientType = organization?.PatientType.ToString() ?? OrganizationPatientTypeEnum.Person.ToString();
                }
                unregisteredPatient.Code = string.Empty;
                unregisteredPatient.AssignmentUserCodes = new List<string>();
                unregisteredPatient.DevicePatientIDs = new List<string>();
                unregisteredPatient.UpdateUsers = new List<string>();
                unregisteredPatient.IsValid = true;
                unregisteredPatient.CreateTime = DateTime.UtcNow;
                unregisteredPatient.UpdateTime = DateTime.UtcNow;
                unregisteredPatient.Name = name;
                foreach (var item in unregisteredPatient.PatientData)
                {
                    if (item.Key == "AnimalInfoName" || item.Key == "Name")
                    {
                        item.Value = name;
                    }
                }
                var result = await _patientInfoDBRepository.InsertOneAsync(unregisteredPatient, new Dictionary<string, string>(), string.Empty);
                // await AddPushPatientAsync(unregisteredPatient);
                return result;
            }
        }

        /// <summary>
        /// 修改病人
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> UpdatePatientsAsync(string code, PatientInfoDTO data, string extensionData, bool canNull = false)
        {
            using (new Performance(_performanceThreshold, "{0} UpdatePatientsAsync execute", $"{this.GetType().Name}"))
            {
                if (!string.IsNullOrWhiteSpace(data.Name) || !string.IsNullOrWhiteSpace(data.Phone))
                {
                    MongoCustomerFilter<MongoTable> filter = new();
                    filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Name", data.Name), ("Phone", data.Phone), ("RootOrganizationCode", data.RootOrganizationCode));
                    var existedPatient = await _patientInfoDBRepository.FindOneAsync(filter.AndCombine());
                    if (existedPatient != null && existedPatient.Code != code)
                    {
                        ThrowCustomerException(CustomerRpcCode.PhoneAndNameExistError, "PhoneAndNameExistError");
                    }
                }
                var patientDO = data.MappingTo<PatientInfoDO>();
                patientDO.UpdateTime = DateTime.UtcNow;
                var result = await _patientInfoDBRepository.UpdateOneAsync("Code", code, patientDO, extensionData, canNull);
                // await AddPushPatientAsync(patientDO);
                return result;
            }
        }

        /// <summary>
        /// 分页查询增量患者集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<PatientInfoDTO>> FindPatientByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询

            customerFilter.AppendGt("BaseData.UpdateTime", startTime);
            var filter = customerFilter.AndCombine();
            var pagedResult = await _patientInfoDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = Builders<MongoTable>.Sort.Ascending("BaseData.UpdateTime"),
            }, onlyNotDeleted: false);
            var result = new PageResult<PatientInfoDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = pagedResult.PageData?.MappingTo<List<PatientInfoDTO>>() ?? new List<PatientInfoDTO>();
            return result;
        }

        /// <summary>
        /// 保存患者记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SavePatientInfoDBAsync(PatientInfoDTO request)
        {
            var code = request.PatientCode;
            var patientDO = request.MappingTo<PatientInfoDO>();
            var entity = await _patientInfoDBRepository.FindOneAsync("Code", code);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                await _patientInfoDBRepository.UpdateOneAsync("Code", code, patientDO, "");
            }
            else
            {
                await _patientInfoDBRepository.InsertOneAsync(patientDO, new Dictionary<string, string>(), "");
            }
            return true;
        }

        /// <summary>
        /// 分页查询病人信息
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        public async Task<PageCollection<PatientInfoDTO>> GetPatientsAsync(GetPatientsByOrgPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientsAsync execute", $"{this.GetType().Name}"))
            {
                var allFilter = await GetPatientsFilter(request);
                PageCollection<PatientInfoDTO> result = new PageCollection<PatientInfoDTO>();
                result.CurrentPage = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.DataCount = (int)await _patientInfoDBRepository.CountDocumentsAsync(allFilter);
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Ascending("BaseData.IsValid").Descending("BaseData.UpdateTime");
                var patientList = await _patientInfoDBRepository.FindAllAsync(allFilter,
                 new FindOptions<MongoTable>
                 {
                     NoCursorTimeout = true,
                     Skip = (request.CurrentPage - 1) * request.PageSize,
                     Limit = request.PageSize,
                     Sort = sort
                 });
                result.PageData = patientList.MappingTo<List<PatientInfoDTO>>();
                return result;
            }
        }

        /// <summary>
        /// 分页查询病人DB信息
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        internal async Task<IList<PatientInfoDO>> GetPatientsDBAsync(GetPatientsByOrgPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientsDBAsync execute", $"{this.GetType().Name}"))
            {
                var allFilter = await GetPatientsFilter(request);
                //IList<PatientInfoDTO> result = new List<PatientInfoDTO>();
                //SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                //SortDefinition<MongoTable> sort = builderSort.Ascending("BaseData.IsValid").Descending("BaseData.UpdateTime");
                var patientList = await _patientInfoDBRepository.FindAllAsync(allFilter,
                 new FindOptions<MongoTable>
                 {
                     NoCursorTimeout = true,
                     Skip = (request.CurrentPage - 1) * request.PageSize,
                     Limit = request.PageSize,
                     //Sort = sort
                 });
                return patientList;
            }
        }

        /// <summary>
        /// 查询病人信息过滤条件
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        private async Task<FilterDefinition<MongoTable>> GetPatientsFilter(GetPatientsByOrgPageRequest request)
        {
            //可以查自己创建的和本机构病人，如果是机构管理员，则还可以查看所有子机构的数据
            //可以查询分配的设备的检查对应的病人
            MongoCustomerFilter<MongoTable> keywordOrFilter = new();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> itemFilter = new();

            MongoCustomerFilter<MongoTable> customerOrFilter = new();
            MongoCustomerFilter<MongoDataItem> itemOrFilter = new();

            if (request.KeywordOrFilter != null)
            {
                MongoCustomerFilter<MongoDataItem> sumItemFilter = new();
                var filterDefinitions = new List<FilterDefinition<MongoDataItem>>();
                foreach (var kv in request.KeywordOrFilter)
                {
                    if (string.IsNullOrWhiteSpace(kv.Value))
                    {
                        continue;
                    }
                    var value = kv.Value;
                    if (kv.Key == "Gender")
                    {
                        value = GetGender(kv.Value);
                        if (string.IsNullOrWhiteSpace(value))
                        {
                            continue;
                        }
                    }
                    //年龄查询
                    else if (kv.Key == "Age")
                    {
                        var ageRegularExp = new BsonRegularExpression($".*{kv.Value}.*", "i");
                        var subAgeFilter = Builders<DataItemDTO>.Filter.Where
                        (f =>
                            (f.Key == "AgeYear") ||
                            (f.Key == "AgeMonth") ||
                            (f.Key == "AgeWeek") ||
                            (f.Key == "AgeDay") ||
                            (f.Key == "AgeUnits")
                        ) & Builders<DataItemDTO>.Filter.Regex(item => item.Value, ageRegularExp);
                        keywordOrFilter.SubFilter("BaseData.PatientData", subAgeFilter);
                        continue;
                    }
                    else if (kv.Key == "Anamnesis")
                    {
                        var ageRegularExp = new BsonRegularExpression($".*{kv.Value}.*", "i");
                        var subAgeFilter = Builders<DataItemDTO>.Filter.Where
                        (f => (f.Key == "Anamnesis")) & Builders<DataItemDTO>.Filter.Regex(item => item.Value, ageRegularExp);
                        keywordOrFilter.SubFilter("BaseData.PatientData", subAgeFilter);
                        continue;
                    }
                    else if (kv.Key == "DevicePatientID")
                    {
                        keywordOrFilter.AppendArrayEq("BaseData.DevicePatientIDs", kv.Value);
                        continue;
                    }
                    sumItemFilter.AppendEq((item => item.Key, kv.Key));
                    sumItemFilter.AppendRegex(item => item.Value, value, true, false);
                    var andCombineFilter = sumItemFilter.AndCombine();
                    filterDefinitions.Add(andCombineFilter);
                }
                if (filterDefinitions.Count > 0)
                {
                    var sumFilter = sumItemFilter.OrCombine(filterDefinitions.ToArray());
                    keywordOrFilter.SubFilter(item => item.QueryData.QueryItems, sumFilter);
                }
            }
            itemFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            if (request.IsValid.GetHashCode() != 0)
            {
                if (request.IsValid == WingInterfaceLibrary.Enum.PatientValidStatusEnum.CheckOut)
                {
                    itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "False"));
                }
                else
                {
                    itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "True"));
                }
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            if (!string.IsNullOrEmpty(request.PatientCode))
            {
                itemFilter.AppendEq((item => item.Key, "Code"), (item => item.Value, request.PatientCode));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            else
            {
                if (!string.IsNullOrWhiteSpace(request.UserCode))
                {
                    var userDTO = await FindUserByCodeAsync(request.UserCode);
                    //属于VINNO杏聆荟平台的用户，不应该看到该医院下所有设备的分时诊断病人，应只能看到自己名下设备的病人
                    if (userDTO.RootOrganizationCode != "VINNO_FIS")
                    {
                        var organizationCodes = new List<string> { userDTO.RootOrganizationCode };
                        //如果是机构负责人
                        if (userDTO.IsDirector)
                        {
                            var org = await _organizationDBRepository.FindOneAsync("Code", userDTO.RootOrganizationCode);
                            //查到该机构所有的子机构
                            var needOrganizations = new List<OrganizationDO>();
                            GetAllChildOrganizationDO(org, ref needOrganizations);
                            if (needOrganizations.Count > 0)
                            {
                                organizationCodes.AddRange(needOrganizations.Select(x => x.Code));
                            }
                        }
                        itemOrFilter.ClearFilter();
                        itemOrFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                        itemOrFilter.AppendIn(item => item.Value, organizationCodes);
                        customerOrFilter.SubFilter(item => item.QueryData.QueryItems, itemOrFilter.AndCombine());
                    }
                    //自己创建的病人
                    itemOrFilter.ClearFilter();
                    itemOrFilter.AppendEq((item => item.Key, "CreatorCode"), (item => item.Value, request.UserCode));
                    customerOrFilter.SubFilter(item => item.QueryData.QueryItems, itemOrFilter.AndCombine());
                    if (userDTO.BindDevices != null && userDTO.BindDevices.Count > 0)//分配自己的设备和机构对应的检查的病人
                    {
                        MongoCustomerFilter<MongoTable> recordFilter = new();
                        MongoCustomerFilter<MongoDataItem> recordItemFilter = new();
                        recordItemFilter.AppendEq((item => item.Key, "DeviceCode"));
                        recordItemFilter.AppendIn(item => item.Value, userDTO.BindDevices.ToList());
                        recordFilter.SubFilter(item => item.QueryData.QueryItems, recordItemFilter.AndCombine());
                        recordItemFilter.ClearFilter();
                        recordItemFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                        recordItemFilter.AppendIn(item => item.Value, userDTO.BindDeviceOrganizations.ToList());
                        recordFilter.SubFilter(item => item.QueryData.QueryItems, recordItemFilter.AndCombine());
                        var recordList = await _recordInfoDBRepository.FindAllAsync(recordFilter.AndCombine());
                        if (recordList != null && recordList.Count > 0)
                        {
                            var patientCodes = recordList.Select(x => x.PatientCode).Distinct().Where(x => !string.IsNullOrWhiteSpace(x)).ToList();
                            itemOrFilter.ClearFilter();
                            itemOrFilter.AppendEq((item => item.Key, "Code"));
                            itemOrFilter.AppendIn(item => item.Value, patientCodes);
                            customerOrFilter.SubFilter(item => item.QueryData.QueryItems, itemOrFilter.AndCombine());
                        }
                    }
                    if (userDTO?.DeletePatientCodes.Any() == true)
                    {
                        itemFilter.AppendEq((item => item.Key, "Code"));
                        itemFilter.AppendNotIn(item => item.Value, userDTO.DeletePatientCodes);
                        customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                    }
                    //查询指定设备的数据
                    var deviceCodes = request.DeviceCodes;
                    if (deviceCodes != null && deviceCodes.Count() > 0)
                    {
                        MongoCustomerFilter<MongoTable> recordFilter = new();
                        MongoCustomerFilter<MongoDataItem> recordItemFilter = new();
                        recordItemFilter.AppendEq((item => item.Key, "DeviceCode"));
                        recordItemFilter.AppendIn(item => item.Value, deviceCodes.ToList());
                        recordFilter.SubFilter(item => item.QueryData.QueryItems, recordItemFilter.AndCombine());
                        var recordList = await _recordInfoDBRepository.FindAllAsync(recordFilter.AndCombine());
                        if (recordList != null && recordList.Count > 0)
                        {
                            var patientCodes = recordList.Select(x => x.PatientCode).Distinct().Where(x => !string.IsNullOrWhiteSpace(x)).ToList();
                            itemFilter.ClearFilter();
                            itemFilter.AppendEq((item => item.Key, "Code"));
                            itemFilter.AppendIn(item => item.Value, patientCodes);
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                        }
                    }
                }
                else
                {
                    itemOrFilter.ClearFilter();
                    itemOrFilter.AppendEq((item => item.Key, "IsDelete"), (item => item.Value, "False"));
                    customerOrFilter.SubFilter(item => item.QueryData.QueryItems, itemOrFilter.AndCombine());
                }
            }
            //查询指定机构的数据
            var rootOrganizationCodes = request.RootOrganizationCodes;
            if (rootOrganizationCodes != null && rootOrganizationCodes.Count() > 0)
            {
                itemFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
                itemFilter.AppendIn(item => item.Value, rootOrganizationCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            if (!string.IsNullOrEmpty(request.NoBindOrgUserCode))
            {
                itemFilter.AppendEq((item => item.Key, "CreatorCode"));
                itemFilter.AppendEq((item => item.Value, request.NoBindOrgUserCode));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            if (request.StartTime > DateTime.MinValue)
            {
                customerFilter.AppendGte("BaseData.CreateTime", request.StartTime);
            }
            if (request.EndTime > DateTime.MinValue)
            {
                customerFilter.AppendLt("BaseData.CreateTime", request.EndTime.AddDays(1));
            }
            if (!string.IsNullOrWhiteSpace(request.ProjectCode))
            {
                itemFilter.AppendEq((item => item.Key, "ProjectCode"), (item => item.Value, request.ProjectCode));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            if (request.ProjectCodes?.Count > 0)
            {
                itemFilter.AppendEq((item => item.Key, "ProjectCode"));
                itemFilter.AppendIn(item => item.Value, request.ProjectCodes.ToList());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            }
            return customerFilter.AndCombine() & keywordOrFilter.OrCombine() & customerOrFilter.OrCombine();
        }

        public string GetGender(string gender)
        {
            string genderValue = "";
            if (!string.IsNullOrEmpty(gender))
            {
                switch (gender.ToLower())
                {
                    case "男":
                    case "男性":
                    case "male":
                        genderValue = PatientGenderEnum.Male.ToString("D");
                        break;
                    case "女":
                    case "女性":
                    case "female":
                        genderValue = PatientGenderEnum.Female.ToString("D");
                        break;
                    case "未填写":
                    case "not filled in":
                        genderValue = PatientGenderEnum.NotFilled.ToString("D");
                        break;
                }
            }
            return genderValue.ToString();
        }

        public async Task<List<PatientInfoDTO>> GetFindValidPatientsByNameAsync(FindValidPatientsDBbyNameRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetFindValidPatientsByNameAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                itemFilter.AppendEq((item => item.Key, "Name"));
                itemFilter.AppendIn(item => item.Value, new List<string> { request.Name });
                var itemFilters = itemFilter.AndCombine();
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilters);
                itemFilter.ClearFilter();
                //限制必须是组织内
                if (!string.IsNullOrWhiteSpace(request.RootOrganizationCode))
                {
                    itemFilter.AppendEq((item => item.Key, "RootOrganizationCode"), (item => item.Value, request.RootOrganizationCode));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                }
                itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "True"));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                itemFilter.ClearFilter();

                var patientList = await _patientInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
                return patientList.MappingTo<List<PatientInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询病人列表（分页）
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        public async Task<PageCollection<PatientInfoDTO>> GetPatientListByQueryDataAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientListByQueryDataAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _patientInfoDBRepository.GetPages(dbRequest);
                return pageInfo.MappingTo<PageCollection<PatientInfoDTO>>();
            }
        }

        /// <summary>
        /// 根据code列表获取病人列表
        /// </summary>
        /// <param name="request">移除设备请求实体</param>
        /// <returns></returns>
        public async Task<List<PatientInfoDTO>> FindPatientInfoListByCodeListAsync(List<string> codeList)
        {
            using (new Performance(_performanceThreshold, "{0} FindPatientInfoListByCodeListAsync execute", $"{this.GetType().Name}"))
            {
                if (codeList == null || codeList.Count == 0)
                {
                    return new List<PatientInfoDTO>();
                }
                var filter = new Dictionary<string, List<string>>();
                filter.Add("Code", codeList);
                //查询设备基础信息
                var dataList = await _patientInfoDBRepository.FindInAsync(filter);
                return dataList.MappingTo<List<PatientInfoDTO>>();
            }
        }


        /// <summary>
        /// 获取病人详细信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> FindPatientByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindPatientByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _patientInfoDBRepository.FindOneAsync("Code", code);
                return result.MappingTo<PatientInfoDTO>();
            }
        }
        /// <summary>
        /// 删除病人
        /// </summary>
        /// <param name="codes"></param>
        /// <returns></returns>
        public async Task<bool> DeletePatientsAsync(IReadOnlyList<string> codes)
        {
            using (new Performance(_performanceThreshold, "{0} DeletePatientsAsync execute", $"{this.GetType().Name}"))
            {
                var list = codes.Select(f => ("Code", f)).ToList();
                await _patientInfoDBRepository.DeleteAllAsync(list);
                return true;
            }
        }

        /// <summary>
        /// 更改指定设备传到虚拟机构的病人的医院
        /// </summary>
        /// <param name="deviceCode">设备编码</param>
        /// <param name="newOrgCode">机构编码</param>
        /// <returns></returns>
        public async Task<bool> UpdatePatientWithOrgCodeAsync(List<string> deviceCodes, string newOrgCode)
        {
            using (new Performance(_performanceThreshold, "{0} UpdatePatientWithOrgCodeAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfos = new Dictionary<string, List<string>>();
                deviceInfos.Add("DeviceCode", deviceCodes);
                var patientInfos = await _patientInfoDBRepository.FindInAsync(deviceInfos);
                patientInfos = patientInfos.Where(p => p.RootOrganizationCode == "VINNO_FIS").ToList();//只迁移设备在虚拟机构产生的数据
                var orgRequest = new GetOrganizationDBRequest();
                orgRequest.OrganizationCode = newOrgCode;
                var newOrg = await GetOrganizationDetailAsync(orgRequest);
                if (patientInfos != null)
                {
                    foreach (var item in patientInfos)
                    {
                        item.OrganizationCode = newOrg?.OrganizationCode;
                        item.RootOrganizationCode = newOrg?.RootCode;
                    }
                }
                if (patientInfos.Count > 0)
                {
                    var req = patientInfos.Select(f => ("Code", f.Code, f, "")).ToList();
                    return await _patientInfoDBRepository.UpdateAllAsync(req) > 0;
                }
                else
                {
                    return true;
                }
            }
        }

        /// <summary>
        /// 修改病人操作时间
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task UpdatePatientUpdateTime(string code)
        {
            var patientInfo = await _patientInfoDBRepository.FindOneAsync("Code", code);
            patientInfo.UpdateTime = DateTime.UtcNow;
            await _patientInfoDBRepository.UpdateOneAsync("Code", patientInfo.Code, patientInfo, null);
            // await AddPushPatientAsync(patientInfo);
        }

        /// <summary>
        /// 根据机构ID、姓名、设备病人ID获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> GetPatientByDevicePatientIDAsync(GetPatientByDevicePatientIDRequest request)
        {
            MongoCustomerFilter<MongoDataItem> itemFilter = new();
            string name = request.Name;
            string devicePatientID = request.DevicePatientID;
            string organizationCode = request.OrganizationCode;
            bool isScreenshotVersion = request.IsScreenshotVersion;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            if (isScreenshotVersion)
            {
                itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "False"));
            }
            else
            {
                itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "True"));
            }
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            itemFilter.AppendEq((item => item.Key, "Name"), (item => item.Value, name));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            itemFilter.AppendEq((item => item.Key, "RootOrganizationCode"), (item => item.Value, organizationCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            customerFilter.AppendArrayEq("BaseData.DevicePatientIDs", devicePatientID);
            var model = await _patientInfoDBRepository.FindOneAsync(customerFilter.AndCombine());
            var result = model.MappingTo<PatientInfoDTO>();
            return result;
        }

        /// <summary>
        /// 根据项目Code、姓名、设备病人ID获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> GetPatientByDevicePatientIDAndProjectCodeAsync(GetPatientByDevicePatientIDAndProjectCodeRequest request)
        {
            MongoCustomerFilter<MongoDataItem> itemFilter = new();
            string name = request.Name;
            string devicePatientID = request.DevicePatientID;
            string projectCode = request.ProjectCode;
            bool isScreenshotVersion = request.IsScreenshotVersion;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            if (isScreenshotVersion)
            {
                itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "False"));
            }
            else
            {
                itemFilter.AppendEq((item => item.Key, "IsValid"), (item => item.Value, "True"));
            }
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            itemFilter.AppendEq((item => item.Key, "Name"), (item => item.Value, name));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            itemFilter.AppendEq((item => item.Key, "ProjectCode"), (item => item.Value, projectCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            customerFilter.AppendArrayEq("BaseData.DevicePatientIDs", devicePatientID);
            var model = await _patientInfoDBRepository.FindOneAsync(customerFilter.AndCombine());
            var result = model.MappingTo<PatientInfoDTO>();
            return result;
        }

        /// <summary>
        /// 根据机构ID、设备ID、设备病人ID获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> GetPatientByPatientIdDBAsync(GetPatientByPatientIdDBRequest request)
        {
            var organizationCode = request.OrganizationCode;
            var deviceCode = request.DeviceCode;
            var patientId = request.DevicePatientID;
            var dic = new Dictionary<string, string>();
            dic.Add("RootOrganizationCode", organizationCode);
            dic.Add("DeviceCode", deviceCode);
            var entities = await _patientInfoDBRepository.FindListAsync(dic);
            if (entities != null && entities.Any())
            {
                var patientDO = entities.FirstOrDefault(x => x.DevicePatientIDs != null && x.DevicePatientIDs.Contains(patientId));
                if (patientDO != null)
                {
                    return patientDO.MappingTo<PatientInfoDTO>();
                }
            }
            return null;
        }


        /// <summary>
        /// 根据项目Code、设备ID、设备病人ID获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> GetPatientByProjectCodeDBAsync(GetPatientByProjectCodeDBRequest request)
        {
            var projectCode = request.ProjectCode;
            var deviceCode = request.DeviceCode;
            var patientId = request.DevicePatientID;
            var dic = new Dictionary<string, string>();
            dic.Add("ProjectCode", projectCode);
            dic.Add("DeviceCode", deviceCode);
            var entities = await _patientInfoDBRepository.FindListAsync(dic);
            if (entities != null && entities.Any())
            {
                var patientDO = entities.FirstOrDefault(x => x.DevicePatientIDs != null && x.DevicePatientIDs.Contains(patientId));
                if (patientDO != null)
                {
                    return patientDO.MappingTo<PatientInfoDTO>();
                }
            }
            return null;
        }

        /// <summary>
        /// 根据机构ID、姓名、设备病人ID获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<PatientInfoDTO>> GetPatientsByOperatorAsync(GetPatientsByOperatorRequest request)
        {
            string userCode = request.UserCode;
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendEq(("BaseData.CreatorCode", userCode));
            customerFilter.AppendArrayEq("BaseData.UpdateUsers", userCode);
            var list = await _patientInfoDBRepository.FindAllAsync(customerFilter.OrCombine());
            var result = list.MappingTo<List<PatientInfoDTO>>();
            return result;
        }

        /// <summary>
        /// 根据体检号获取病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PatientInfoDTO> GetPatientsByPhysicalExamNumberDBAsync(GetPatientsByPhysicalExamNumberDBRequest request)
        {
            var physicalExamNumber = request.PhysicalExamNumber;
            if (!string.IsNullOrWhiteSpace(physicalExamNumber))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendEq(("BaseData.BaseData.DevicePatientID", physicalExamNumber));
                var entities = await _patientInfoDBRepository.FindAllAsync(customerFilter.OrCombine());
                if (entities?.Any() ?? false)
                {
                    return entities.OrderByDescending(x => x.CreateTime).FirstOrDefault().MappingTo<PatientInfoDTO>();
                }
            }
            return null;
        }

        /// <summary>
        /// 根据userCode查询病人列表
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        public async Task<long> GetPatientsByUserCodeAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientsByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoTable> baseDataOrFilter = new();
                MongoCustomerFilter<MongoDataItem> itemFilter = new();
                itemFilter.AppendEq((item => item.Key, "CreatorCode"), (item => item.Value, userCode));
                customerFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
                if (!string.IsNullOrWhiteSpace(userCode))//操作人包括当前人
                {
                    baseDataOrFilter.AppendArrayEq("BaseData.UpdateUsers", userCode);
                    var userDO = await _userDBRepository.FindOneAsync("Code", userCode);
                    if (userDO?.DeletePatientCodes.Any() == true)
                    {
                        baseDataOrFilter.AppendNotIn("BaseData.Code", userDO.DeletePatientCodes);
                    }
                }
                var allFilter = customerFilter.OrCombine(baseDataOrFilter.AndCombine());
                var count = await _patientInfoDBRepository.CountDocumentsAsync(allFilter);
                return count;
            }
        }



        /// <summary>
        /// 分页查询病人信息
        /// </summary>
        /// <param name="request">请求参数</param>
        /// <returns></returns>
        async Task<PageCollection<PatientInfoDTO>> GetPatientPagesAsync(GetPatientsByOrgPageRequest request, SortDefinition<MongoTable> sort)
        {
            using (new Performance(_performanceThreshold, "{0} GetPatientPagesAsync execute", $"{this.GetType().Name}"))
            {
                var allFilter = await GetPatientsFilter(request);
                PageCollection<PatientInfoDTO> result = new PageCollection<PatientInfoDTO>();
                result.CurrentPage = request.CurrentPage;
                result.PageSize = request.PageSize;
                result.DataCount = (int)await _patientInfoDBRepository.CountDocumentsAsync(allFilter);
                var patientList = await _patientInfoDBRepository.FindAllAsync(allFilter,
                 new FindOptions<MongoTable>
                 {
                     NoCursorTimeout = true,
                     Skip = (request.CurrentPage - 1) * request.PageSize,
                     Limit = request.PageSize,
                     Sort = sort
                 });
                result.PageData = patientList.MappingTo<List<PatientInfoDTO>>();
                return result;
            }
        }



    }
}