using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Bson;
using MongoDB.Driver;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Request;
using WingServerCommon.Mapper;
using WingServerCommon.Service;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IRegisterPersonInfoDBService
    {
        public async Task<string> AddPersonInfoAsync(PersonInfoFullDTO request)
        {
            var entity = request.MappingTo<RegisterPersonInfoEntity>();
            // var entity = new RegisterPersonInfoEntity
            // {
            //     Code = request.Code,
            //     Name = request.Name,
            //     HeadImg = request.HeadImg,
            //     Sex = request.Sex,
            //     Age = request.Age,
            //     CardID = request.CardID,
            //     Remarks = request.Remarks,
            // };
            var result = await _registerPersonInfoDBRepository.InsertOneAsync(entity);
            return result;
        }

        public async Task<bool> DeletePersonAsync(string code)
        {
            var result = await _registerPersonInfoDBRepository.DeleteOneAsync("Code", code);
            return result;
        }

        public async Task<PersonInfoFullDTO> FindPersonInfoByCodeAsync(string code)
        {
            var filter = Builders<RegisterPersonInfoEntity>.Filter.Eq(f => f.Code, code);
            var entity = await _registerPersonInfoDBRepository.FindOneAsync(filter);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                var dto = entity.MappingTo<PersonInfoFullDTO>();
                // var dto = new PersonInfoFullDTO
                // {
                //     Code = entity.Code,
                //     Name = entity.Name,
                //     HeadImg = entity.HeadImg,
                //     Sex = entity.Sex,
                //     Age = entity.Age,
                //     CardID = entity.CardID,
                //     Remarks = entity.Remarks,
                //     PersonRegisterRecordList = entity.PersonRegisterRecordList?.Select(x => new PersonRegisterDTO
                //     {
                //         IsNormal = x.IsNormal,
                //         RegisterTime = x.RegisterTime
                //     })?.ToList() ?? new List<PersonRegisterDTO>(),
                // };
                return dto;
            }
            return null;
        }

        public async Task<PageCollection<PersonInfoFullDTO>> GetPersonPageAsync(GetPersonPageRequest request)
        {
            var keyword = request.Keyword;

            var builder = Builders<RegisterPersonInfoEntity>.Filter;
            var filter = builder.Where(c => c.IsDelete == false);
            if(!string.IsNullOrWhiteSpace(request.Government))
            {
                filter = filter & builder.Eq(c => c.Government, request.Government);
            }
            if (!string.IsNullOrEmpty(keyword))
            {
                keyword = EscapeExprSpecialWord(keyword);//字符串转义
                var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                filter = filter & nameFilter;
            }

            var pageFilter = new DBPageRequest<RegisterPersonInfoEntity>
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                Sort = Builders<RegisterPersonInfoEntity>.Sort.Descending(x => x.CreateTime)
            };
            var result = await _registerPersonInfoDBRepository.GetPages(pageFilter);
            return result.MappingTo<PageCollection<PersonInfoFullDTO>>();
        }

        public async Task<bool> UpdatePersonInfoAsync(PersonInfoFullDTO personFullInfo)
        {
            var persons = new List<PersonRegisterEntity>();
            if (personFullInfo?.PersonRegisterRecordList?.Any() ?? false)
            {
                foreach (var person in personFullInfo.PersonRegisterRecordList)
                {
                    persons.Add(new PersonRegisterEntity
                    {
                        IsNormal = person.IsNormal,
                        RegisterTime = person.RegisterTime,
                    });
                }
            }
            var filter = Builders<RegisterPersonInfoEntity>.Filter.Eq(f => f.Code, personFullInfo.Code);
            var update = Builders<RegisterPersonInfoEntity>.Update
                .Set(f => f.Age, personFullInfo.Age)
                .Set(f => f.CardID, personFullInfo.CardID)
                .Set(f => f.HeadImg, personFullInfo.HeadImg)
                .Set(f => f.Name, personFullInfo.Name)
                .Set(f => f.Remarks, personFullInfo.Remarks)
                .Set(f => f.Sex, personFullInfo.Sex)
                .Set(f => f.Organization, personFullInfo.Organization)
                .Set(f => f.Government, personFullInfo.Government)
                .Set(f => f.Nautical, personFullInfo.Nautical)
                .Set(f => f.PersonRegisterRecordList, persons)
                .Set(f => f.UpdateTime, DateTime.UtcNow);
            var result = await _registerPersonInfoDBRepository.UpdateOneAsync(filter, update) > 0;
            return result;
        }
    }
}