using System.Net.Cache;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingServerCommon.Mapper;
using System.Linq;
using System;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Statistic;
using WingInterfaceLibrary.DTO.Dictionary;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.CMSStatistic;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.LiveConsultation;
using System.Linq;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IStatisticDBService
    {

        /// <summary>
        /// 查询统计配置集合
        /// </summary>
        /// <returns></returns>
        public async Task<IList<StatisticDTO>> GetStatisticAsync(GetStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetStatisticAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("Enabled", bool.TrueString);
                var entities = await _statisticDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<StatisticDTO>>();
            }
        }

        /// <summary>
        /// 保存统计结果
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveStatisticRecordAsync(SaveStatisticRecordDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveStatisticRecordAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("UserCode", request.UserCode);
                dic.Add("StatisticCode", request.StatisticCode);
                var entities = await _statisticRecordDBRepository.FindListAsync(dic);
                if (entities.Count == 1)
                {
                    var updateEntity = entities.FirstOrDefault();
                    updateEntity.StatisticCode = request.StatisticCode;
                    updateEntity.StatisticName = request.StatisticName;
                    updateEntity.XAxisDatas = request.XAxisDatas;
                    updateEntity.YAxisDatas = request.YAxisDatas;
                    updateEntity.LastestUpdateTime = DateTime.UtcNow;
                    updateEntity.NextHandleTime = request.NextHandleTime;
                    updateEntity.ServerHost = request.ServerHost;
                    updateEntity.StatisticsPlatform = request.StatisticsPlatform;
                    updateEntity.StatisticJsonData = request.StatisticJsonData;
                    if (request.Children != null)
                    {
                        updateEntity.Children = request.Children.MappingTo<List<StatisticInfoDO>>();
                    }
                    return await _statisticRecordDBRepository.UpdateOneAsync("Code", updateEntity.Code, updateEntity, string.Empty);
                }
                else if (entities.Count > 1)
                {
                    var list = entities.Select(f => ("Code", f.Code)).ToList();
                    await _statisticRecordDBRepository.RealDeleteAllAsync(list);
                }
                var insertEntity = new StatisticRecordDO();
                insertEntity.UserCode = request.UserCode;
                insertEntity.StatisticCode = request.StatisticCode;
                insertEntity.StatisticName = request.StatisticName;
                insertEntity.XAxisDatas = request.XAxisDatas;
                insertEntity.YAxisDatas = request.YAxisDatas;
                insertEntity.LastestUpdateTime = DateTime.UtcNow;
                insertEntity.NextHandleTime = request.NextHandleTime;
                insertEntity.ServerHost = request.ServerHost;
                insertEntity.StatisticsPlatform = request.StatisticsPlatform;
                insertEntity.StatisticJsonData = request.StatisticJsonData;
                if (request.Children != null)
                {
                    insertEntity.Children = request.Children.MappingTo<List<StatisticInfoDO>>();
                }
                await _statisticRecordDBRepository.InsertOneAsync(insertEntity, new Dictionary<string, string>(), string.Empty);
                return true;
            }
        }


        /// <summary>
        /// 查询已过期的统计数据
        /// </summary>
        /// <returns></returns>
        public async Task<IList<StatisticRecordDTO>> GetExpiredStatisticRecordsAsync(GetStatisticDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetExpiredStatisticRecordsAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendLt("BaseData.NextHandleTime", DateTime.UtcNow);
                var filter = customerFilter.AndCombine();
                var entities = await _statisticRecordDBRepository.FindAllAsync(filter);
                return entities.MappingTo<List<StatisticRecordDTO>>();
            }
        }


        /// <summary>
        /// 用户编码查询统计结果
        /// </summary>
        /// <returns></returns>
        public async Task<IList<StatisticRecordDTO>> GetStatisticRecordsDBByUserAsync(GetStatisticRecordsDBByUserRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetStatisticRecordsDBByUserAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("UserCode", request.UserCode);
                var entities = await _statisticRecordDBRepository.FindListAsync(dic);
                return entities.MappingTo<List<StatisticRecordDTO>>();
            }
        }
        /// <summary>
        /// 删除统计数据
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteStatisticRecordAsync(DeleteStatisticRecordDBRequest request)
        {
            return await _statisticRecordDBRepository.DeleteOneAsync("Code", request.StatisticRecordCode);
        }

        /// <summary>
        /// 删除统计数据
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteStatisticRecordsByUserCodeAsync(DeleteStatisticRecordsByUserCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteStatisticRecordsByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("UserCode", request.UserCode);
                var entities = await _statisticRecordDBRepository.FindListAsync(dic);
                if (entities.Any())
                {
                    var list = entities.Select(f => ("Code", f.Code)).ToList();
                    await _statisticRecordDBRepository.RealDeleteAllAsync(list);
                }
                return true;
            }
        }

        /// <summary>
        /// 我的待办
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<StatisticMyPendingDTO>> GetMyPendingStatisticAsync(GetMyPendingStatisticDBRequest request)
        {
            var userCode = request.UserCode;
            var myPendingList = new List<StatisticMyPendingDTO>();
            var recordFilter = new Dictionary<string, string>();
            recordFilter.Add("CreatorCode", userCode);
            var recordList = await FindRecordInfoPagesAsync(new FindRecordInfoPagesRequest { CurrentPage = 1, PageSize = 999999, Filter = recordFilter, IsFuzzy = true });
            var recordCount = recordList.PageData.Count == 0 ? 1 : recordList.PageData.Count;
            //待撰写的报告
            var waitReport = new StatisticMyPendingDTO();
            waitReport.Title = "cmsWaitReport";
            waitReport.Rate = Math.Ceiling(recordList.PageData.Count(x => x.RecordStatus == RecordStatusEnum.NotReport) * (double)100 / recordCount);
            myPendingList.Add(waitReport);
            //待完成的扫查
            var notCompleted = new StatisticMyPendingDTO();
            notCompleted.Title = "cmsNotScanned";
            notCompleted.Rate = Math.Ceiling(recordList.PageData.Count(x => x.RecordStatus == RecordStatusEnum.NotScanned || x.RecordStatus == RecordStatusEnum.Uploaded) * (double)100 / recordCount);
            myPendingList.Add(notCompleted);
            //待参加的会诊(我申请的、我组织的、我参与的  未拒绝的 未开始的)
            var consultationList = await FindConsultationRecordsByUserCodeAsync(userCode);
            var consultationCount = consultationList.Count();
            var waitConsultationList = consultationList.Where(x => x.ConsultationStatus == TransactionStatusEnum.Applied || x.ConsultationStatus == TransactionStatusEnum.ToStart || (x.ConsultationStatus == TransactionStatusEnum.Expired && x.ConsultationTimeEnd > DateTime.UtcNow.AddDays(-30))).ToList();
            var myJoinList = waitConsultationList.Where(x => x.ConsultationMembers.Any(c => c.MemberCode == userCode && c.MemberStatus != LiveConsultationMemberStatus.Rejected));
            var notJoinList = waitConsultationList.Where(x => x.ExpertUserCode == userCode || x.ApplyUserCode == userCode);
            var waitconsultationCount = notJoinList.Count() + myJoinList.Count();
            var waitConsult = new StatisticMyPendingDTO();
            waitConsult.Title = "cmsWaitConsult";
            waitConsult.Rate = Math.Ceiling(waitconsultationCount * (double)100 / (consultationCount == 0 ? 1 : consultationCount));
            myPendingList.Add(waitConsult);
            return myPendingList;
        }


        /// <summary>
        /// 我的战绩
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<StatisticMyStandingDTO>> GetMyStandingStatisticAsync(GetMyStandingStatisticDBRequest request)
        {
            var userCode = request.UserCode;
            var myStandingList = new List<StatisticMyStandingDTO>();
            //已诊断病人（我创建的 我操作的病人）
            var patientCount = await GetPatientsByUserCodeAsync(userCode);
            var patientStanding = new StatisticMyStandingDTO();
            patientStanding.Title = "cmsDiagnosisPatients";
            patientStanding.Data = (int)patientCount;
            patientStanding.Unit = "";
            myStandingList.Add(patientStanding);
            //已完成报告
            var reportList = await GetReportsByUserCodeAsync(userCode);
            var reportStanding = new StatisticMyStandingDTO();
            reportStanding.Title = "cmsCompleteReport";
            reportStanding.Data = reportList.Count();
            reportStanding.Unit = "";
            myStandingList.Add(reportStanding);
            //已参与会诊(我申请的、我组织的、我参与的  待报告+已结束)
            var consultationList = await FindConsultationRecordsByUserCodeAsync(userCode);
            consultationList = consultationList.Where(x => x.ConsultationStatus == TransactionStatusEnum.PendingReport || x.ConsultationStatus == TransactionStatusEnum.End).ToList();
            var myJoinList = consultationList.Where(x => x.ConsultationMembers.Any(c => c.MemberCode == userCode && c.MemberStatus != LiveConsultationMemberStatus.Rejected));
            var notJoinList = consultationList.Where(x => x.ExpertUserCode == userCode || x.ApplyUserCode == userCode).ToList();
            var consultStanding = new StatisticMyStandingDTO();
            consultStanding.Title = "cmsJoinedConsult";
            consultStanding.Data = myJoinList.Count() + notJoinList.Count();
            consultStanding.Unit = "";
            myStandingList.Add(consultStanding);
            return myStandingList;
        }


        /// <summary>
        /// 实时诊断数据统计查询接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<StatisticRealtimeDataDTO>> GetRealtimeDataStatisticAsync(GeGetRealtimeDataStatisticDBRequest request)
        {
            var reports = new List<StatisticRealtimeDataDTO>();
            var orgCodes = new Dictionary<string, List<string>>();
            orgCodes.Add("Code", request.OrgCodes.ToList());
            var orgInfoList = await _organizationDBRepository.FindInAsync(orgCodes);//医院信息集合

            var orgs = new Dictionary<string, List<string>>();
            //通过医院数据查询检查数据
            var recordOrgs = new Dictionary<string, string>();
            orgs.Add("RootOrganizationCode", request.OrgCodes.ToList());
            var records = await _recordInfoDBRepository.FindInAsync(orgs);
            foreach (var item in records)
            {
                recordOrgs.TryAdd(item.RecordCode, item.RootOrganizationCode);
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("RecordCode", recordOrgs.Keys.ToList());
            var entities = await _reportDBRepository.FindInAsync(dic);
            entities.ForEach(report =>
              {
                  report.ReportDatasJson = string.Empty;
                  report.ReportTemplateJson = string.Empty;
              });
            entities = entities.Where(r => r.IsDelete == false).OrderByDescending(r => r.CreateTime).Take(request.GetMaxCount).ToList();
            foreach (var item in entities)
            {
               // var doctorName = await _userDBRepository.FindOneAsync("Code", item.ReportUserCode);
                recordOrgs.TryGetValue(item.RecordCode, out var orgCode);
                reports.Add(new StatisticRealtimeDataDTO()
                {
                    OrganizationCode = orgCode,
                    OrganizationName = orgInfoList.FirstOrDefault(c => c.Code == orgCode)?.OrganizationName,
                    CreateTime = item.CreateTime,
                    DoctorCode = item.ReportUserCode,
                    DoctorName =""
                });
            }
            return reports;
        }

        /// <summary>
        /// 提交报告统计查询接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<StatisticSubmittedReportsDTO>> GetSubmittedReportsStatisticAsync(GetGetSubmittedReportsStatisticDBRequest request)
        {
            var orgCodes = new Dictionary<string, List<string>>();
            orgCodes.Add("Code", request.OrgCodes.ToList());
            var orgInfoList = await _organizationDBRepository.FindInAsync(orgCodes);//医院信息集合
            var reports = new List<StatisticSubmittedReportsDTO>();
            var orgs = new Dictionary<string, List<string>>();
            //通过医院数据查询检查数据
            var recordOrgs = new Dictionary<string, string>();
            orgs.Add("RootOrganizationCode", request.OrgCodes.ToList());
            var records = await _recordInfoDBRepository.FindInAsync(orgs);
            foreach (var item in records)
            {
                recordOrgs.TryAdd(item.RecordCode, item.RootOrganizationCode);
            }
            var dic = new Dictionary<string, List<string>>();
            dic.Add("RecordCode", recordOrgs.Keys.ToList());
            var entities = await _reportDBRepository.FindInAsync(dic);
            entities.ForEach(report =>
               {
                   report.ReportDatasJson = string.Empty;
                   report.ReportTemplateJson = string.Empty;
               });
            foreach (var item in entities)
            {
                recordOrgs.TryGetValue(item.RecordCode, out var orgCode);
                if (!string.IsNullOrWhiteSpace(orgCode))
                {
                    var report = reports.FirstOrDefault(v => v.OrganizationCode == orgCode);
                    if (report == null)
                    {
                        report = new StatisticSubmittedReportsDTO()
                        {
                            OrganizationCode = orgCode,
                            OrganizationName = orgInfoList.FirstOrDefault(v => v.Code == orgCode)?.OrganizationName,
                            TotalCount = 1
                        };
                        reports.Add(report);
                    }
                    else
                    {
                        report.TotalCount = +1;
                    }

                }
            }
            return reports;
        }


        /// <summary>
        /// 诊断数据数量统计查询接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<StatisticDiagnosisDataDTO>> GetDiagnosisStatisticAsync(GetDiagnosisStatisticDBRequest request)
        {
            var result = new List<StatisticDiagnosisDataDTO>();
            // MongoCustomerFilter<MongoTable> customerFilter = new();
            // MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            // queryItemsFilter.AppendEq((item => item.Key, "RootOrganizationCode"));
            // queryItemsFilter.AppendIn(item => item.Value, request.OrgCodes);
            // if (queryItemsFilter.HasFilter)
            // {
            //     customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            // }
            // queryItemsFilter = new();
            // queryItemsFilter.AppendEq((item => item.Key, "RecordStatus"));
            // queryItemsFilter.AppendEq(item => item.Value, request.RecordStatus);
            // if (queryItemsFilter.HasFilter)
            // {
            //     customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            // }
            // var records = await _recordInfoDBRepository.FindAllAsync(customerFilter.AndCombine());//查询
            // foreach (var item in records)
            // {
            //     var record = result.FirstOrDefault(v => v.OrganizationCode == item.RootOrganizationCode && v.CreateTime.Date == item.CreateTime.Date);
            //     if (record == null)
            //     {
            //         result.Add(new StatisticDiagnosisDataDTO()
            //         {
            //             OrganizationCode = item.RootOrganizationCode,
            //             CreateTime = item.CreateTime.Date,
            //             TotalCount = 1
            //         });
            //     }
            //     else
            //     {
            //         record.TotalCount = +1;
            //     }
            // }
            return result;
        }
    }
}