using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DTO.Management;
using WingServerCommon.Service;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Mapper;
using System.Collections.Generic;
using System.Linq;
using MongoDB.Driver;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IThirdPartyInfoDBService
    {


        public async Task<string> InsertThirdPartyInfoAsync(CreateThirdPartyInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertThirdPartyInfoAsync execute", $"{this.GetType().Name}"))
            {
                var thirdPartyInfoDO = request.Data.MappingTo<ThirdPartyInfoDO>();
                return await _thirdPartyInfoDBRepository.InsertOneAsync(thirdPartyInfoDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        public async Task<ThirdPartyInfoDTO> FindThirdPartyInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindThirdPartyInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var thirdPartyInfoDO = await _thirdPartyInfoDBRepository.FindOneAsync("Code", code);
                var thirdPartyInfoDTO = thirdPartyInfoDO.MappingTo<ThirdPartyInfoDTO>();
                var tokenInfo = await _tokensDBRepository.FindOneAsync("ClientId", code);
                thirdPartyInfoDTO.Token = tokenInfo?.Code;
                return thirdPartyInfoDTO;
            }
        }

        public async Task<ThirdPartyInfoDTO> FindThirdPartyInfoByTokenAsync(string token)
        {
            using (new Performance(_performanceThreshold, "{0} FindThirdPartyInfoByTokenAsync execute", $"{this.GetType().Name}"))
            {
                var tokenInfo = await _tokensDBRepository.FindOneAsync("Code", token);
                var thirdPartyInfoDO = await _thirdPartyInfoDBRepository.FindOneAsync("Code", tokenInfo?.ClientId);
                var thirdPartyInfoDTO = thirdPartyInfoDO.MappingTo<ThirdPartyInfoDTO>();
                thirdPartyInfoDTO.Token = tokenInfo?.Code;
                return thirdPartyInfoDTO;
            }
        }

        public async Task<PageResult<ThirdPartyInfoDTO>> FindThirdPartyInfoPagesAsync(GetThirdPartyInfoPagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindThirdPartyInfoPagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageResult = await _thirdPartyInfoDBRepository.GetPages(dbRequest);
                var convertResult = new PageResult<ThirdPartyInfoDTO>();
                convertResult.PageIndex = pageResult.CurrentPage;
                convertResult.PageSize = pageResult.PageSize;
                convertResult.TotalCount = pageResult.DataCount;
                convertResult.PageData = pageResult.PageData.Select(f => f.MappingTo<ThirdPartyInfoDTO>()).ToList();
                if (convertResult.PageData.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("ClientId", convertResult.PageData.Select(c => c.Code).ToList());
                    var tokenInfos = await _tokensDBRepository.FindInAsync(dic);
                    foreach (var item in convertResult.PageData)
                    {
                        item.Token = tokenInfos.FirstOrDefault(t => t.ClientId == item.Code)?.Code;
                    }
                }
                return convertResult;
            }
        }

        public async Task<bool> DeleteThirdPartyInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteThirdPartyInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _thirdPartyInfoDBRepository.DeleteOneAsync("Code", code);
            }
        }

        public async Task<bool> UpdateThirdPartyInfoByCodeAsync(string code, ThirdPartyInfoDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateThirdPartyInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _thirdPartyInfoDBRepository.UpdateOneAsync("Code", code, data.MappingTo<ThirdPartyInfoDO>(), extensionData);
            }
        }
    }
}