using System.Threading.Tasks;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Service;
using WingInterfaceLibrary.DTO.UserAuthorityGroup;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request;
using WingServerCommon.Utilities;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IUserAuthorityGroupsDBService
    {
        /// <summary>
        /// 根据编码获取前端权限组
        /// </summary>
        /// <param name="code">权限组编码</param>
        /// <returns>权限组信息</returns>
        public async Task<UserAuthorityGroupDTO> FindUserAuthorityGroupByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var userAuthorityGroupDO = await _userAuthorityGroupDBRepository.FindOneAsync("Code", code);
                return userAuthorityGroupDO.MappingTo<UserAuthorityGroupDTO>();
            }
        }

        /// <summary>
        /// 根据名称查找前端权限组
        /// </summary>
        /// <param name="name">权限组名称</param>
        /// <returns>权限组信息</returns>
        public Task<UserAuthorityGroupDTO> FindUserAuthorityGroupByNameAsync(string name)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// 新增前端权限组
        /// </summary>
        /// <param name="userAuthorityGroupDO">权限组信息</param>
        /// <param name="extensionData">扩展信息</param>
        /// <returns>Id</returns>
        public async Task<string> InsertUserAuthorityGroupAsync(CreateFrontAuthorityGroupsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertUserAuthorityGroupAsync execute", $"{this.GetType().Name}"))
            {
                return await _userAuthorityGroupDBRepository.InsertOneAsync(request.Data.MappingTo<FrontAuthorityGroupDO>(), request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// 更新前端权限组
        /// </summary>
        /// <param name="code">组编码</param>
        /// <param name="data">权限组信息</param>
        /// <returns>是否成功</returns>
        public async Task<bool> UpdateUserAuthorityGroupByCodeAsync(string code, UserAuthorityGroupDTO data, string extensionData)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateUserAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _userAuthorityGroupDBRepository.UpdateOneAsync("Code", code, data.MappingTo<FrontAuthorityGroupDO>(), extensionData);
            }
        }

        /// <summary>
        /// 删除前端权限组
        /// </summary>
        /// <param name="id">Id</param>
        /// <returns>是否成功</returns>
        public async Task<bool> DeleteUserAuthorityGroupByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserAuthorityGroupByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _userAuthorityGroupDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 查找前端权限组（分页）
        /// </summary>
        /// <param name="request">分页信息</param>
        /// <returns>权限组列表</returns>
        public async Task<PageCollection<UserAuthorityGroupDTO>> FindUserAuthorityGroupPagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindUserAuthorityGroupPagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbRequest = GetDBPageRequest(request);
                var pageInfo = await _userAuthorityGroupDBRepository.GetPages(dbRequest);
                return PageCollectionConvert<FrontAuthorityGroupDO, UserAuthorityGroupDTO>(pageInfo);
            }
        }
    }
}