using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using MongoDB.Driver;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Common;
using WingMongoDBModule.DBSync;
using WingMongoDBModule.DO;
using WingMongoDBModule.Entity;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Repositories;
using WingMongoDBModule.Request;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using WingServerCommon.Mapper;
using WingServerCommon.Service;
using WingInterfaceLibrary.OpLog;
using WingInterfaceLibrary.Enum;
using WingMongoDBModule.DB.DO;
using WingInterfaceLibrary.DTO.Dictionary;
using WingMongoDBModule.Upgrades;
using WingMongoDBModule.Entities;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.CMSStatistic;
using WingInterfaceLibrary.Internal.Interface;
using Newtonsoft.Json;
using WingServerCommon.Utilities.Executors;
using WingInterfaceLibrary.Interface;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.DTO.Report;
using System.Collections.Concurrent;
using WingInterfaceLibrary.DTO.Common;
using System.Text.Json.Nodes;
using System.Diagnostics;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IAdminDBService
    {
        private System.Net.Http.HttpClient _httpClient = new System.Net.Http.HttpClient();
        private ConvertPdfExecutor _convertPdfExecutor = new ConvertPdfExecutor("ConvertPdfExecutor");
        private int _performanceThreshold => EnvironmentConfigManager.GetParammeter<IntParameter>("Log", "DBPerformance").Value;
        private string _server => EnvironmentConfigManager.GetParammeter<StringParameter>("Database", "Server").Value;
        private int _port => EnvironmentConfigManager.GetParammeter<IntParameter>("Database", "Port").Value;
        private string _dbName => EnvironmentConfigManager.GetParammeter<StringParameter>("Database", "DBName").Value;
        private bool _isDistributed => EnvironmentConfigManager.GetParammeter<BoolParameter>("General", "IsDistributed").Value;
        private bool _isMaster => EnvironmentConfigManager.GetParammeter<BoolParameter>("General", "IsMaster").Value;
        private string _serverID => EnvironmentConfigManager.GetParammeter<StringParameter>("General", "ServerID").Value;
        private string _storageServer => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "StorageServer").Value;
        private string _serverUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "CFSServerUrl").Value;
        private string _defaultCFSServerUrl = ConfigurationManager.StorageSettings.CFSServerUrl;
        private string _storageDisk => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "VCSStorageDisk").Value;
        private string _curCDNHost => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "CDNServerUrl").Value;
        private string _domains => EnvironmentConfigManager.GetParammeter<StringParameter>("Gateway", "Domains").Value;
        private string _httpService => EnvironmentConfigManager.GetParammeter<StringParameter>("Services", "JsonRpcHttp").Value;
        private bool _isResearchEditionService => EnvironmentConfigManager.GetParammeter<BoolParameter>("General", "IsResearchEditionService").Value;
        //是否体检系统本地工作站
        private bool _isLocalHost => GetStringParams("Vital", "CloudServerUrl") != "";
        private int _downloadOriginalLowestValue = 1048576;
        private static Mutex _dbVersionLocker = new Mutex();
        private readonly SequenceExecutor<string> _archiveDeletionExecutor;
        private IMasterInteractionCenterService _masterInteractionCenterService;
        private AdminAuthorityGroupDBRepository _adminAuthorityGroupDBRepository { get; set; }
        private AdminDBRepository _adminDBRepository { get; set; }
        private AdminFeaturesDBRepository _adminFeaturesDBRepository { get; set; }
        private AdminRoleDBRepository _adminRoleDBRepository { get; set; }
        private DeviceInfoDBRepository _deviceInfoDBRepository { get; set; }
        private DictionaryDBRepository _dictionaryDBRepository { get; set; }
        private IdentityApplyDBRepository _identityApplyDBRepository { get; set; }
        private OrganizationDBRepository _organizationDBRepository { get; set; }
        private PatientInfoDBRepository _patientInfoDBRepository { get; set; }
        private PositionDBRepository _positionDBRepository { get; set; }
        private RankDBRepository _rankDBRepository { get; set; }
        private RecordInfoDBRepository _recordInfoDBRepository { get; set; }
        private RemedicalDBRepository _remedicalDBRepository { get; set; }
        private RoleDBRepository _roleDBRepository { get; set; }
        private StorageDBRepository _storageDBRepository { get; set; }
        private UserAuthorityGroupDBRepository _userAuthorityGroupDBRepository { get; set; }
        private UserDBRepository _userDBRepository { get; set; }
        private UserFeatureDBRepository _userFeatureDBRepository { get; set; }
        private ThesaurusDBRepository _thesaurusDBRepository { get; set; }
        private ReportTemplateDBRepository _reportTemplateDBRepository { get; set; }
        private ReportDBRepository _reportDBRepository { get; set; }
        private ReportLabelDBRepository _reportLabelDBRepository { get; set; }
        private ShareInfoDBRepository _shareInfoDBRepository { get; set; }
        private DiagnosisModuleDBRepository _diagnosisModuleDBRepository { get; set; }
        private TokensDBRepository _tokensDBRepository { get; set; }
        private ThirdPartyInfoDBRepository _thirdPartyInfoDBRepository { get; set; }
        private DbVersionRepository _dbVersionRepository { get; set; }
        private UserDefinedMeasureInfoDBRepository _measureInfoRepository { get; set; }
        private UserExtendsDBRepository _userExtendsDBRepository { get; set; }
        private ReportPosterDBRepository _reportPosterDBRepository { get; set; }
        private ReferralRecordSettingDBRepository _referralRecordSettingDBRepository { get; set; }
        private ReportPushRecordsDBRepository _reportPushRecordsDBRepository { get; set; }
        private DistributedServerInfoDBRepository _distributedServerInfoDBRepository { get; set; }
        private ReferralRecordDBRepository _referralRecordDBRepository { get; set; }
        private AdminStatisticDBRepository _adminStatisticDBRepository { get; set; }
        private AdminStatisticRecordDBRepository _adminStatisticRecordDBRepository { get; set; }
        private RemedicalMeasuredInfoDBRepository _remedicalMeasuredInfoDBRepository { get; set; }
        private ConsultationRecordDBRepository _consultationRecordDBRepository { get; set; }
        private ConsultationEvaluateDBRepository _consultationEvaluateDBRepository { get; set; }
        private FollowUpVisitDBRepository _followUpVisitDBRepository { get; set; }
        private LiveRoomDBRepository _liveRoomDBRepository { get; set; }
        private CMSTemplateDBRepository _cmsTemplateDBRepository { get; set; }
        private StatisticDBRepository _statisticDBRepository { get; set; }
        private StatisticRecordDBRepository _statisticRecordDBRepository { get; set; }
        private GradingProtectionConfigDBRepository _gradingProtectionConfigDBRepository { get; set; }
        private ScheduleDBRepository _scheduleDBRepository { get; set; }
        private MessageInfoDBRepository _messageInfoDBRepository { get; set; }
        private AnnouncementInfoDBRepository _announcementInfoDBRepository { get; set; }
        private CourseLabelDBRepository _courseLabelDBRepository { get; set; }
        private CourseInfoDBRepository _courseInfoDBRepository { get; set; }
        private UserGroupDBRepository _userGroupDBRepository { get; set; }
        private ExaminationQuestionDBRepository _examinationQuestionDBRepository { get; set; }
        private ExaminationPaperDBRepository _examinationPaperDBRepository { get; set; }
        private VideoInfoDBRepository _videoInfoDBRepository { get; set; }
        private OrderDBRepository _orderDBRepository { get; set; }
        private PaymentRecordDBRepository _paymentRecordDBRepository { get; set; }
        private ProductDBRepository _productDBRepository { get; set; }
        private RemoteExaminationDataDBRepository _remoteExaminationDataDBRepository { get; set; }
        private StudentExaminationDBRepository _studentExaminationDBRepository { get; set; }
        private DiagnosisResultDBRepository _diagnosisResultDBRepository { get; set; }
        private UserDeviceExtendInfoDBRepository _userDeviceExtendInfoDBRepository { get; set; }
        private DeviceOutputConfigDBRepository _deviceOutputConfigDBRepository { get; set; }
        private BrandModelOutputConfigDBRepository _brandModelOutputConfigDBRepository { get; set; }
        private MeasureExportRecordDBRepository _measureExportRecordDBRepository { get; set; }
        private LabFolderDBRepository _labFolderDBRepository { get; set; }
        private LabUploadRecordRepository _labUploadRecordRepository { get; set; }
        private LabConvertStateDBRepository _labConvertStateDBRepository { get; set; }
        private DevicePatchDBRepository _devicePatchDBRepository { get; set; }
        private DevicePatchVersionDBRepository _devicePatchVersionDBRepository { get; set; }
        private DevicePrinterDBRepository _devicePrinterDBRepository { get; set; }
        private RegisterPersonInfoDBRepository _registerPersonInfoDBRepository { get; set; }
        private HealthExamBookingDBRepository _healthExamBookingDBRepository { get; set; }
        private RemedicalAISelectedInfoDBRepository _remedicalAISelectedInfoDBRepository { get; set; }
        private SystemConfigDBRepository _SystemConfigDBRepository { get; set; }
        private CourseAlbumDBRepository _courseAlbumDBRepository { get; set; }
        private SignStudentDBRepository _signStudentDBRepository { get; set; }
        private MigrateRecordDBRepository _migrateRecordDBRepository { get; set; }
        private PayUrlDBRepository _payUrlDBRepository { get; set; }
        private ChatMessageDBRepository _chatMessageDBRepository { get; set; }

        private ReferralDataDBRepository _referralDataDBRepository { get; set; }
        private PackagesDBRepository _packagesDBRepository { get; set; }
        private ProjectInfoDBRepository _projectInfoDBRepository { get; set; }
        private ProjectPushRecordDBRepository _projectPushRecordDBRepository { get; set; }
        private IOTDeviceVersionDBRepository _iotDeviceVersionDBRepository { get; set; }
        private DeviceReportDBRepository _deviceReportDBRepository { get; set; }
        private RegisterInfoDBRepository _registerInfoDBRepository;
        private CourseBrowsedRecordDBRepository _courseBrowsedRecordDBRepository { get; set; }
        private ElectrocardiogramDBRepository _electrocardiogramDBRepository;
        private CommonDictionaryDBRepository _commonDictionaryDBRepository;
        private FaceRecognitionDBRepository _faceRecognitionDBRepository;
        private ResearchProjectDBRepository _researchProjectDBRepository;
        private TaskDBRepository _taskDBRepository;
        private TaskMainDBRepository _taskMainDBRepository;
        private readonly object _createShortCodeLocker = new object();
        private IAIDiagnosisService _aiDiagnosisService;
        private IRemedicalService _remedicalService;
        private IStorageService _storageService;
        private bool _isCarotidOn = true;
        private ProjectPosterManager _projectPosterManager;
        private LicenseInfoDBRepository _licenseInfoDBRepository;
        private DongleInfoDBRepository _dongleInfoDBRepository;
        private ChargeResourcePackageDBRepository _chargeResourcePackageDBRepository;
        private MultiPartUploadInfoDBRepository _multiPartUploadInfoDBRepository;
        private UserUrmConfigDBRepository _userUrmConfigDBRepository;
        private bool isContainURMAuth = false;
        private bool isContainVETAuth = false;

        public DatabaseService()
        {
            Syncfusion.Licensing.SyncfusionLicenseProvider.RegisterLicense("MjE0OTMxMEAzMTM5MmUzNDJlMzBhUmxBUEx3MlcvU0V0MDJMNmVCanN1NWdEZTB3MjhPTUlQQTVscnFad2NvPQ==");
            MongoRepertoryCenter.Instance.Init(_server, _port, _dbName, _isDistributed);
            _adminAuthorityGroupDBRepository = new AdminAuthorityGroupDBRepository();
            _adminDBRepository = new AdminDBRepository();
            _adminFeaturesDBRepository = new AdminFeaturesDBRepository();
            _adminRoleDBRepository = new AdminRoleDBRepository();
            _deviceInfoDBRepository = new DeviceInfoDBRepository();
            _dictionaryDBRepository = new DictionaryDBRepository();
            _identityApplyDBRepository = new IdentityApplyDBRepository();
            _organizationDBRepository = new OrganizationDBRepository();
            _patientInfoDBRepository = new PatientInfoDBRepository();
            _positionDBRepository = new PositionDBRepository();
            _rankDBRepository = new RankDBRepository();
            _recordInfoDBRepository = new RecordInfoDBRepository();
            _remedicalDBRepository = new RemedicalDBRepository();
            _roleDBRepository = new RoleDBRepository();
            _storageDBRepository = new StorageDBRepository();
            _userAuthorityGroupDBRepository = new UserAuthorityGroupDBRepository();
            _userDBRepository = new UserDBRepository();
            _userFeatureDBRepository = new UserFeatureDBRepository();
            _thesaurusDBRepository = new ThesaurusDBRepository();
            _reportTemplateDBRepository = new ReportTemplateDBRepository();
            _reportDBRepository = new ReportDBRepository();
            _reportLabelDBRepository = new ReportLabelDBRepository();
            _shareInfoDBRepository = new ShareInfoDBRepository();
            _diagnosisModuleDBRepository = new DiagnosisModuleDBRepository();
            _tokensDBRepository = new TokensDBRepository();
            _thirdPartyInfoDBRepository = new ThirdPartyInfoDBRepository();
            _dbVersionRepository = new DbVersionRepository();
            _measureInfoRepository = new UserDefinedMeasureInfoDBRepository();
            _userExtendsDBRepository = new UserExtendsDBRepository();
            _reportPosterDBRepository = new ReportPosterDBRepository();
            _referralRecordSettingDBRepository = new ReferralRecordSettingDBRepository();
            _reportPushRecordsDBRepository = new ReportPushRecordsDBRepository();
            _distributedServerInfoDBRepository = new DistributedServerInfoDBRepository();
            _referralRecordDBRepository = new ReferralRecordDBRepository();
            _adminStatisticDBRepository = new AdminStatisticDBRepository();
            _adminStatisticRecordDBRepository = new AdminStatisticRecordDBRepository();
            _remedicalMeasuredInfoDBRepository = new RemedicalMeasuredInfoDBRepository();
            _consultationRecordDBRepository = new ConsultationRecordDBRepository();
            _followUpVisitDBRepository = new FollowUpVisitDBRepository();
            _consultationEvaluateDBRepository = new ConsultationEvaluateDBRepository();
            _liveRoomDBRepository = new LiveRoomDBRepository();
            _cmsTemplateDBRepository = new CMSTemplateDBRepository();
            _statisticDBRepository = new StatisticDBRepository();
            _statisticRecordDBRepository = new StatisticRecordDBRepository();
            _gradingProtectionConfigDBRepository = new GradingProtectionConfigDBRepository();
            _scheduleDBRepository = new ScheduleDBRepository();
            _messageInfoDBRepository = new MessageInfoDBRepository();
            _announcementInfoDBRepository = new AnnouncementInfoDBRepository();
            _courseInfoDBRepository = new CourseInfoDBRepository();
            _courseLabelDBRepository = new CourseLabelDBRepository();
            _userGroupDBRepository = new UserGroupDBRepository();
            _examinationQuestionDBRepository = new ExaminationQuestionDBRepository();
            _examinationPaperDBRepository = new ExaminationPaperDBRepository();
            _videoInfoDBRepository = new VideoInfoDBRepository();
            _orderDBRepository = new OrderDBRepository();
            _paymentRecordDBRepository = new PaymentRecordDBRepository();
            _remoteExaminationDataDBRepository = new RemoteExaminationDataDBRepository();
            _studentExaminationDBRepository = new StudentExaminationDBRepository();
            _productDBRepository = new ProductDBRepository();
            _diagnosisResultDBRepository = new DiagnosisResultDBRepository();
            _deviceOutputConfigDBRepository = new DeviceOutputConfigDBRepository();
            _brandModelOutputConfigDBRepository = new BrandModelOutputConfigDBRepository();
            _measureExportRecordDBRepository = new MeasureExportRecordDBRepository();
            _labFolderDBRepository = new LabFolderDBRepository();
            _labUploadRecordRepository = new LabUploadRecordRepository();
            _labConvertStateDBRepository = new LabConvertStateDBRepository();
            _devicePatchDBRepository = new DevicePatchDBRepository();
            _devicePatchVersionDBRepository = new DevicePatchVersionDBRepository();
            _devicePrinterDBRepository = new DevicePrinterDBRepository();
            _registerPersonInfoDBRepository = new RegisterPersonInfoDBRepository();
            _healthExamBookingDBRepository = new HealthExamBookingDBRepository();
            _remedicalAISelectedInfoDBRepository = new RemedicalAISelectedInfoDBRepository();
            _SystemConfigDBRepository = new SystemConfigDBRepository();
            _courseAlbumDBRepository = new CourseAlbumDBRepository();
            _signStudentDBRepository = new SignStudentDBRepository();
            _payUrlDBRepository = new PayUrlDBRepository();
            _migrateRecordDBRepository = new MigrateRecordDBRepository();
            _chatMessageDBRepository = new ChatMessageDBRepository();
            _referralDataDBRepository = new ReferralDataDBRepository();
            _packagesDBRepository = new PackagesDBRepository();
            _projectPushRecordDBRepository = new ProjectPushRecordDBRepository();
            _userDeviceExtendInfoDBRepository = new UserDeviceExtendInfoDBRepository();
            _deviceReportDBRepository = new DeviceReportDBRepository();
            _projectInfoDBRepository = new ProjectInfoDBRepository();
            _iotDeviceVersionDBRepository = new IOTDeviceVersionDBRepository();
            _courseBrowsedRecordDBRepository = new CourseBrowsedRecordDBRepository();
            _registerInfoDBRepository = new RegisterInfoDBRepository();
            _electrocardiogramDBRepository = new ElectrocardiogramDBRepository();
            _commonDictionaryDBRepository = new CommonDictionaryDBRepository();
            _faceRecognitionDBRepository = new FaceRecognitionDBRepository();
            _researchProjectDBRepository = new ResearchProjectDBRepository();
            _taskDBRepository = new TaskDBRepository();
            _taskMainDBRepository = new TaskMainDBRepository();
            _licenseInfoDBRepository = new LicenseInfoDBRepository();
            _dongleInfoDBRepository = new DongleInfoDBRepository();
            _chargeResourcePackageDBRepository = new ChargeResourcePackageDBRepository();
            _multiPartUploadInfoDBRepository = new MultiPartUploadInfoDBRepository();
            _userUrmConfigDBRepository = new UserUrmConfigDBRepository();
            DOShare.ForceHttps = EnvironmentConfigManager.GetParammeter<BoolParameter>("Storage", "ForceHttps").Value;
            _isCarotidOn = GetIsCarotidOn();
            RegisterRepository();
            var databaseExist = MongoRepertoryCenter.Instance.DatabaseExist();
            if (!databaseExist)
            {
                InitCollectionIndex();
                //Init
                InitDBData().Wait();
            }
            if (_httpService.Contains("VitalMixtureService"))
            {
                CheckAndInsertDefaultThesaurus().Wait();
                CheckAndInsertDefaultReportTemplate().Wait();
                CheckAndInsertBaseOrgInfo().Wait();
            }
            //DB 版本号
            //判断是否主服务器 后者 自建服务器
            DoVersion().Wait();

            //日志索引
            LogIndexes().Wait();
            MongoDBLogTracker.Instance.Initialize(SendOpLogToMaster);
            LoadEnvironmentConfig().Wait();
            LoadPatientInfoLanguageConfig().Wait();
            var archiveDeletionDays = EnvironmentConfigManager.GetParammeter<IntParameter>("Database", "ArchiveDeletionDays").Value;
            if (archiveDeletionDays > 0)
            {
                _archiveDeletionExecutor = new SequenceExecutor<string>("ArchiveDeletion", new TimeSpan(archiveDeletionDays, 0, 0, 0));
                _archiveDeletionExecutor.Add(OnArchiveDeletion, DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
            }
            MongoRepertoryCenter.Instance.ExecuteOpLogEvent += OnExecuteOpLogEvent;
            MongoRepertoryCenter.Instance.TokenRemove += OnTokenRemove;
            _projectPosterManager = new ProjectPosterManager(GetPushDatas, SavePushResult);
            SendProjectPushRecordByDBAsync().Wait();
            InitYinChuanProjectAsync().Wait();
            WatchingMongoDbServer();
        }



        /// <summary>
        /// 加载语言配置信息
        /// </summary>
        public async Task LoadPatientInfoLanguageConfig()
        {
            LanguageSetting.AddOrUpdateLanguageConfig();
        }

        /// <summary>
        /// 加载环境配置
        /// </summary>
        public async Task LoadEnvironmentConfig()
        {
            try
            {
                var builder = Builders<SystemConfigEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                var dataList = await _SystemConfigDBRepository.FindAllAsync(filter);
                if (dataList?.Count > 0)
                {
                    var dic = new Dictionary<string, object>();
                    //替换原先的值                
                    foreach (var item in dataList)
                    {
                        var obj = JsonConvert.DeserializeObject<object>(item.ConfigContent);
                        if (!dic.ContainsKey(item.ConfigKey))
                        {
                            dic.Add(item.ConfigKey, obj);
                        }
                    }
                    var dicStr = Newtonsoft.Json.JsonConvert.SerializeObject(dic);
                    EnvironmentConfigManager.AddOrUpdateDBConfig(dicStr);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService LoadEnvironmentConfig error, ex:{ex}");
            }
        }

        /// <summary>
        /// 加载云存储配置
        /// </summary>
        public async Task LoadDistributedServerConfig()
        {
            try
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                var filter = mongoCustomerFilter.AndCombine();
                var dataList = await _distributedServerInfoDBRepository.FindAllAsync(filter);
                if (dataList?.Count > 0)
                {
                    //替换原先的值  
                    if (!string.IsNullOrEmpty(_domains))
                    {
                        var curServerInfo = dataList.FirstOrDefault(c => _domains.Contains(c.Name)) ?? new DistributedServerInfoDO();
                        var values = new List<DataItemParameter>();
                        if (!string.IsNullOrEmpty(curServerInfo?.Code) && !string.IsNullOrEmpty(curServerInfo.StorageServerConfig))
                        {
                            var isUseBackupsStorageUrl = false;
                            var backupsCFSServerUrl = "";
                            JObject jsonObject = JObject.Parse(curServerInfo.StorageServerConfig);
                            var dataItem = new DataItemParameter();
                            dataItem.Key = "IsUseBackupsStorageUrl";
                            dataItem.Value = jsonObject["IsUseBackupsStorageUrl"].ToString();
                            isUseBackupsStorageUrl = Convert.ToBoolean(dataItem.Value);
                            values.Add(dataItem);
                            dataItem = new DataItemParameter();
                            dataItem.Key = "BackupsCFSServerUrl";
                            dataItem.Value = backupsCFSServerUrl = jsonObject["BackupsCFSServerUrl"].ToString();
                            values.Add(dataItem);
                            dataItem = new DataItemParameter();
                            dataItem.Key = "IsCopyServiceUseProdEnv";
                            dataItem.Value = jsonObject["IsCopyServiceUseProdEnv"].ToString();
                            values.Add(dataItem);
                            dataItem = new DataItemParameter();
                            dataItem.Key = "CopyServiceUrl";
                            dataItem.Value = jsonObject["CopyServiceUrl"].ToString();
                            values.Add(dataItem);
                            dataItem = new DataItemParameter();
                            dataItem.Key = "ManagementUseBackupsStorageUrl";
                            dataItem.Value = jsonObject["ManagementUseBackupsStorageUrl"].ToString();
                            values.Add(dataItem);
                            LanguageSetting.AddOrUpdateServerStorageConfig(_domains, values);
                            if (isUseBackupsStorageUrl)
                            {
                                ConfigurationManager.SetStorageCFSServerUrl(backupsCFSServerUrl);
                                IParameter value = new StringParameter()
                                {
                                    Section = "Storage",
                                    Name = "CFSServerUrl",
                                    Value = backupsCFSServerUrl
                                };
                                EnvironmentConfigManager.SetParammeter("Storage", "CFSServerUrl", value);
                                WriteBackupsStorageSetting(backupsCFSServerUrl);
                            }
                            else
                            {
                                // var configFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "appsettings.json");
                                // var appsettingsJson = File.ReadAllText(configFilePath);
                                // JObject obj = JObject.Parse(appsettingsJson);    
                                // var cfsUrl = obj["Storage"]["CFSServerUrl"].ToString();
                                ConfigurationManager.SetStorageCFSServerUrl(_defaultCFSServerUrl);
                                IParameter value = new StringParameter()
                                {
                                    Section = "Storage",
                                    Name = "CFSServerUrl",
                                    Value = _defaultCFSServerUrl
                                };
                                EnvironmentConfigManager.SetParammeter("Storage", "CFSServerUrl", value);
                                WriteBackupsStorageSetting(_defaultCFSServerUrl);
                            }
                        }
                        else
                        {
                            LanguageSetting.AddOrUpdateServerStorageConfig(_domains, values);
                            ConfigurationManager.SetStorageCFSServerUrl(_defaultCFSServerUrl);
                            IParameter value = new StringParameter()
                            {
                                Section = "Storage",
                                Name = "CFSServerUrl",
                                Value = _defaultCFSServerUrl
                            };
                            EnvironmentConfigManager.SetParammeter("Storage", "CFSServerUrl", value);
                            WriteBackupsStorageSetting(_defaultCFSServerUrl);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService LoadEnvironmentConfig error, ex:{ex}");
            }
        }

        /// <summary>
        /// 写入配置
        /// </summary>
        private void WriteBackupsStorageSetting(string cfsUrl)
        {
            var fileName = "BackupsStorageSetting.json";
            var message = "{ \"Storage\": { \"BackupsCFSServerUrl\": \"" + cfsUrl + "\" } }";
            var logPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Upgrade", fileName);
            using (StreamWriter streamWriter = new StreamWriter(logPath, false))
            {
                streamWriter.WriteLine(message);
            }
        }

        /// <summary>
        /// 注册Repository
        /// </summary>
        private void RegisterRepository()
        {
            RepositoryFactory.Instance.Register(typeof(ScheduleDBRepository), _scheduleDBRepository);
            RepositoryFactory.Instance.Register(typeof(GradingProtectionConfigDBRepository), _gradingProtectionConfigDBRepository);
        }
        /// <summary>
        /// 抛出自定义错误码
        /// </summary>
        /// <param name="customerRpcCodeEnum"></param>
        /// <param name="msg"></param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg)
        {
            ThrowRpcException((int)customerRpcCodeEnum, customerRpcCodeEnum.ToString());
        }
        /// <summary>
        /// 操作版本
        /// </summary>
        /// <returns></returns>
        private async Task DoVersion()
        {
            var currentVersion = await GetCurrentVersion();
            Logger.WriteLineInfo($"mongoDB currentVersion: {currentVersion}.");
            if (string.IsNullOrWhiteSpace(currentVersion))
            {
                SetNewDBVersion("1.0").Wait();
                currentVersion = "1.0";
            }
            //db版本升级是全部都要执行的。
            if (Convert.ToDouble(currentVersion) < 1.1d)//升级DB版本 
            {
                try
                {
                    //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.0 to 1.1, please wait.");

                    // 1.用户角色【普通用户、医师】名称、权限数据更新；
                    await UpdateUserRoleDBAsync();
                    await UpdateUserAuthorityGroupsDBAsync();
                    // 2.用户菜单数据更新；
                    await UpdateUserFeatureDBAsync();
                    // // 3.更新机构短码；批量更新5000以上的数据，opLog大概率会丢失
                    // await UpdateOrganizationsDBAsync();                        
                    // 4.词条和报告模板数据清洗；
                    await UpgradeThesaurusDataDBAsync();
                    await UpgradeReportTemplateDataDBAsync();
                    await UpgradeDefaultTemplateDataDBAsync();
                    // 5.更新反馈系统小图片
                    await UpdateUserAssociatedIconDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.1").Wait();
                        currentVersion = "1.1";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.1");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.1, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.2d)//升级DB版本 
            {
                try
                {
                    //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.1 to 1.2, please wait.");

                    // 1.管理员多语言数据更新；
                    await UpdateAdminAuthDBAsync();
                    await UpdateAdminRoleDBAsync();

                    // 2.用户多语言数据更新；
                    await UpdateUserAuthDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.2").Wait();
                        currentVersion = "1.2";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.2");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.2, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.3d)//升级DB版本 
            {
                try
                {
                    //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.2 to 1.3, please wait.");

                    // 1.添加默认图像；
                    await UpdateSystemDefaultAuthDBAsync();
                    await UpdateUserAgentAssociatedIconDBAsync();
                    await CopyDefaultLoadingFileToVCSDBAsync();
                    // await ClearDeviceModelDBAsync();
                    //await InitDeviceModelDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.3").Wait();
                        currentVersion = "1.3";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.3");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.3, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.4d)//升级DB版本 
            {
                try
                {
                    //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.3 to 1.4, please wait.");

                    // 1.添加杏聆荟管理后台 杏聆荟学院 相关权限；
                    await UpdateAdminFeatureDBAsync();
                    // 2.添加用户权限相关的西班牙语配置
                    await UpdateUserFeatureForESDBAsync();
                    // 3.添加管理权限相关的西班牙语配置

                    // 4.添加词条的西班牙语配置
                    await UpdateThesaurusForESDBAsync();
                    // 5.添加报告模板的西班牙配置
                    await UpdateReportTemplateForESDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.4").Wait();
                        currentVersion = "1.4";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.4");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.4, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.5d)//升级DB版本 
            {
                try
                {
                    //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.4 to 1.5, please wait.");

                    // 1.添加杏聆荟管理后台西班牙语相关权限；
                    await UpdateAdminRoleForESDBAsync();
                    // 2.添加用户权限相关的西班牙语配置
                    await UpdateUserRoleForESDBAsync();
                    // 3.更新设备型号相关西班牙语配置
                    await UpgradeDeviceModelForESDBAsync();
                    // 4.课程标签
                    await UpgradeCourseLabelForESDBAsync();
                    // 5.更新vod配置
                    await UpdateSystemConfigVodConfigDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.5").Wait();
                        currentVersion = "1.5";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.5");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.5, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.6d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.5 to 1.6, please wait.");

                    // 1.添加质控权限；
                    await UpdateQualityControlDBAsync();
                    // 2.更新杏聆荟学院相关权限，并同步更新到相关角色上
                    await UpdateUserFeatureForEducationDBAsync();
                    await UpdateCourseVideoTypeDBAsync();
                    // 3.添加认证讲师权限
                    await UpdateUserFeatureForCertifiedInstructorDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.6").Wait();
                        currentVersion = "1.6";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.6");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.6, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.7d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.6 to 1.7, please wait.");

                    // 1.添加设备类型和IOT默认数据；
                    await UpdateIOTDeviceTypeDBAsync();
                    // await AddIOTDefaultDeviceDATADBAsync();
                    // await AddIOTDefaultDeviceReportDBAsync();
                    // await DeleteIOTDeviceInfoDBAsync();
                    // 2.更新管理端权限列表
                    await UpdateManagementAuthDBAsync();
                    // 3.更新代理商权限数据
                    await UpdateUserFeatureForAgentDBAsync();
                    // 4.更新系统配置
                    await UpdateSystemConfigWXOpenApiDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.7").Wait();
                        currentVersion = "1.7";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.7");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.7, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.8d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.7 to 1.8, please wait.");

                    await UpdateAdminFeatureForRODBAsync();
                    await UpdateAdminRoleForRODBAsync();
                    await UpdateManagementAuthForRODBAsync();
                    await UpgradeCourseLabelForRODBAsync();
                    await UpgradeDeviceModelForRODBAsync();
                    await UpdateUserFeatureForROEducationDBAsync();
                    await UpdateReportTemplateForRODBAsync();
                    await InsertTemplateFromTemplateForRODBAsync();
                    await UpdateUserFeatureForRODBAsync();
                    await UpdateUserRoleForRODBAsync();
                    await UpdateQualityControlForRODBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.8").Wait();
                        currentVersion = "1.8";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.8");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.8, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 1.9d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.8 to 1.9, please wait.");

                    await InsertTemplateFromTemplateForSignBAsync();
                    await UpdateUserFeatureForResearchProjectDBAsync();
                    await UpdateAdminFeatureForResearchProjectDBAsync();
                    await UpdateDongleFeatureForResearchProjectDBAsync();
                    await UpdateAdminRoleForResearchProjectDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("1.9").Wait();
                        currentVersion = "1.9";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 1.9");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 1.9, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 2.0d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 1.9 to 2.0, please wait.");

                    await DeleteUserFeatureForResearchProjectDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("2.0").Wait();
                        currentVersion = "2.0";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 2.0");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 2.0, error:{ex}");
                }
            }
            if (Convert.ToDouble(currentVersion) < 2.1d)//升级DB版本 
            {
                try
                { //以下更新数据库版本号时升级具体内容
                    Logger.WriteLineInfo($"mongoDB is upgrading from 2.0 to 2.1, please wait.");

                    await DeleteTwoUserFeatureForResearchProjectDBAsync();
                    //更新版本
                    var isSuccess = await UpgradeToV11.DoUpgradeAsync();
                    if (isSuccess)
                    {
                        //注：下次正式线上发布需要升级DB版本号
                        SetNewDBVersion("2.1").Wait();
                        currentVersion = "2.1";
                    }
                    Logger.WriteLineInfo($"mongoDB upgrade successful, now the version is 2.1");
                }
                catch (Exception ex)
                {
                    Logger.WriteLineInfo($"mongoDB upgrade failed, now the version is 2.1, error:{ex}");
                }
            }
        }
        /// <summary>
        /// 获取当前DB版本
        /// </summary>
        /// <returns></returns>
        private async Task<string> GetCurrentVersion()
        {
            var dbVersions = await _dbVersionRepository.FindAllAsync();
            if (dbVersions != null && dbVersions.Count > 0)
            {
                var dbVersion = dbVersions.FirstOrDefault(v => v.Version != null);
                if (dbVersion != null)
                {
                    return dbVersion.Version;
                }
            }
            return "";
        }
        /// <summary>
        /// 设置数据库版本
        /// </summary>
        /// <param name="ver"></param>
        /// <returns></returns>
        private async Task SetNewDBVersion(string ver)
        {
            var dbVersions = await _dbVersionRepository.FindAllAsync();
            if (dbVersions != null && dbVersions.Count > 0)
            {
                var dbVersion = dbVersions.FirstOrDefault(v => v.Version != null);
                if (dbVersion != null && dbVersion.Version != ver)
                {
                    dbVersion.Version = ver;
                    dbVersion.UpdateTime = DateTime.UtcNow;
                    await _dbVersionRepository.UpdateOneForSyncDBAsync("Code", dbVersion.Code, dbVersion, "", true, false);
                }
            }
            else
            {
                var data = new DbVersionDO()
                {
                    Version = ver,
                    Code = "Version"
                };
                await _dbVersionRepository.InsertOneForAsyncDBAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        bool SendOpLogToMaster(OperationLog operationLog)
        {
            var syncOpLogToMasterRequest = new SyncOpLogToMasterRequest()
            {
                ActionType = operationLog.ActionType,
                BsonContent = operationLog.BsonContent,
                CollectionName = operationLog.CollectionName,
                FilterContent = operationLog.FilterContent,
                CreateTime = operationLog.CreateTime,
                Code = operationLog.Code,
                SourceUrl = _domains,
                ServerID = _serverID,
                IsSimple = operationLog.IsSimple
            };

            return _masterInteractionCenterService.SyncOpLogToMasterAsync(syncOpLogToMasterRequest).ConfigureAwait(false).GetAwaiter().GetResult();
        }

        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _masterInteractionCenterService = GetProxy<IMasterInteractionCenterService>();
        }

        public override async void LoadAfterRegister()
        {
            try
            {
                if (ConfigurationManager.IsDistributed && !ConfigurationManager.IsMaster)
                {
                    StartSyncDBFromMasterServer(5000, OnSyncDBFromMaster).ConfigureAwait(false).GetAwaiter();
                }
                _aiDiagnosisService = GetProxy<IAIDiagnosisService>();
                _remedicalService = GetProxy<IRemedicalService>();
                _storageService = GetProxy<IStorageService>();
                await LoadDistributedServerConfig();
                ResearchEditionServiceAuth().Wait();
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService LoadAfterRegister err, {ex}");
            }
        }

        /// <summary>
        /// 调用storage服务上传文件
        /// </summary>
        /// <returns></returns>
        private async Task<List<string>> GetFileTransferByStorageAsync(string sourceFilePath, bool isRechristen = false, bool isNeedPartUpload = false, bool isCopy = false)
        {
            var request = new WingInterfaceLibrary.Request.Storage.UploadFileTransferRequest()
            {
                SourceFilePath = sourceFilePath,
                IsRechristen = isRechristen,
                IsNeedPartUpload = isNeedPartUpload,
                IsCopy = isCopy,
            };
            var resList = await _storageService.UploadFileByServerConfigAsync(request);
            return resList;
        }

        /// <summary>
        /// 初始化数据
        /// </summary>
        /// <returns></returns>
        private async Task InitDBData()
        {
            //设备输出配置DeviceOutputConfig
            var filter = Builders<DeviceOutputConfig>.Filter.Empty;
            var isExists = await _deviceOutputConfigDBRepository.Exists(filter);
            if (!isExists)
            {
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1920x1080",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.MergedVideo,
                    VideoWidth = 1920,
                    VideoHeight = 1080
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1280x720",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.MergedVideo,
                    VideoWidth = 1280,
                    VideoHeight = 720
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "640x480",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Camera,
                    VideoWidth = 640,
                    VideoHeight = 480
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "320x240",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Camera,
                    VideoWidth = 320,
                    VideoHeight = 240
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1080x960",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Camera,
                    VideoWidth = 1080,
                    VideoHeight = 960
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1920x1080",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Camera,
                    VideoWidth = 1920,
                    VideoHeight = 1080
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1280x1080",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Desktop,
                    VideoWidth = 1280,
                    VideoHeight = 1080
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1600x1080",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Desktop,
                    VideoWidth = 1600,
                    VideoHeight = 1080
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1080x960",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Desktop,
                    VideoWidth = 1080,
                    VideoHeight = 960
                });
                await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                {
                    Name = "1920x1080",
                    DeviceVideoOutputType = DeviceVideoOutputTypeEnum.Desktop,
                    VideoWidth = 1920,
                    VideoHeight = 1080
                });
            }
            var allAdmin = await _adminDBRepository.FindAllAsync();
            var updateAdminList = new List<string>();
            foreach (var admin in allAdmin)
            {
                if (admin.IsDelete == false && !admin.SecretPassword.StartsWith("Symmetry_"))
                {
                    admin.SecretPassword = "Symmetry_mk9xah8N";//abc123
                    await _adminDBRepository.UpdateOneAsync("Code", admin.Code, admin, "", isWriteOperationLog: false);
                }
            }
            //超级管理员admin
            var find = await _adminDBRepository.FindOneAsync("Code", "PrimaryAdmin01");
            if (find == null)
            {
                var data = new AdminInfoDO
                {
                    Code = "PrimaryAdmin01",
                    AdminName = "Admin",
                    FatherCode = string.Empty,
                    SecretPassword = "Symmetry_mk9xah8N",//abc123
                    RoleCodes = new List<string> { "AdminRoleDO01" },
                };
                var id = await _adminDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }
            else
            {
                if (!find.RoleCodes.Contains("AdminRoleDO01"))
                {
                    find.RoleCodes.Add("AdminRoleDO01");
                    await _adminDBRepository.UpdateOneAsync("Code", find.Code, find, "", isWriteOperationLog: false);
                }
            }
            //超级管理员角色
            var findRole = await _adminRoleDBRepository.FindOneAsync("Code", "AdminRoleDO01");
            if (findRole == null)
            {
                var data = new AdminRoleDO
                {
                    Code = "AdminRoleDO01",
                    RoleName = "超级管理员",
                    Description = "拥有管理后台最高权限",
                    LanguageConfigs = new List<AdminRoleLanguageConfigDO> { new AdminRoleLanguageConfigDO { Language = "zh-CN", RoleName = "超级管理员", Description = "拥有管理后台最高权限" }, new AdminRoleLanguageConfigDO { Language = "en-US", RoleName = "Super administrator", Description = "Has the highest authority to manage the backend" }, new AdminRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Суперадминистратор", Description = "Обладает максимальными правами администрирования" } }
                };
                var id = await _adminRoleDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }
            //管理员权限
            var adminFeatureCount = await _adminFeaturesDBRepository.CountDocumentsAsync();
            if (adminFeatureCount == 0)
            {
                var adminFeatureList = new List<AdminFeatureDO>();
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "JGGL", FatherCode = "", FeatureName = "机构管理", UniqueCode = "JGGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "OrgDevices", FatherCode = "JGGL", FeatureName = "机构设备", UniqueCode = "OrgDevices" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "JGLB", FatherCode = "JGGL", FeatureName = "机构列表", UniqueCode = "JGLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "SBGL", FatherCode = "", FeatureName = "设备管理", UniqueCode = "SBGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "SBLXLB", FatherCode = "SBGL", FeatureName = "设备类型列表", UniqueCode = "SBLXLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "PPXHLB", FatherCode = "SBGL", FeatureName = "设备品牌型号列表", UniqueCode = "PPXHLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "SBLB", FatherCode = "SBGL", FeatureName = "设备列表", UniqueCode = "SBLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "HTXTGL", FatherCode = "", FeatureName = "后台系统管理", UniqueCode = "HTXTGL", });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "HTQXLB", FatherCode = "HTXTGL", FeatureName = "后台权限列表", UniqueCode = "HTQXLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "HTGLYJS", FatherCode = "HTXTGL", FeatureName = "后台管理员角色", UniqueCode = "HTGLYJS" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "HTGLYLB", FatherCode = "HTXTGL", FeatureName = "后台管理员列表", UniqueCode = "HTGLYLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "YHGL", FatherCode = "", FeatureName = "用户管理", UniqueCode = "YHGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "QXLB", FatherCode = "YHGL", FeatureName = "权限列表", UniqueCode = "QXLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "PTJS", FatherCode = "YHGL", FeatureName = "平台角色", UniqueCode = "PTJS" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "YHLB", FatherCode = "YHGL", FeatureName = "用户列表", UniqueCode = "YHLB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "MBGL", FatherCode = "", FeatureName = "模板管理", UniqueCode = "MBGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "RPMGI", FatherCode = "MBGL", FeatureName = "报告模板首页", UniqueCode = "RPMGI" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "BGBQ", FatherCode = "MBGL", FeatureName = "报告标签", UniqueCode = "BGBQ" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "CTKMB", FatherCode = "MBGL", FeatureName = "词库列表", UniqueCode = "CTKMB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "CLWH", FatherCode = "MBGL", FeatureName = "词条列表", UniqueCode = "CLWH" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "AIGL", FatherCode = "", FeatureName = "AI诊断", UniqueCode = "AIGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "AILB", FatherCode = "AIGL", FeatureName = "AI应用列表", UniqueCode = "AILB" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "HTGL", FatherCode = "", FeatureName = "后台管理", UniqueCode = "HTGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "XTRZ", FatherCode = "HTGL", FeatureName = "系统日志", UniqueCode = "XTRZ" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "DSFGL", FatherCode = "HTGL", FeatureName = "第三方管理", UniqueCode = "DSFGL" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "ServerPZ", FatherCode = "HTGL", FeatureName = "Server配置", UniqueCode = "ServerPZ" });
                adminFeatureList.Add(new AdminFeatureDO { FeatureCode = "ZXZH", FatherCode = "HTGL", FeatureName = "在线账号", UniqueCode = "ZXZH" });

                var languages = new List<string>();
                DirectoryInfo theFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge"));
                DirectoryInfo[] dirInfo = theFolder.GetDirectories();//获取所在目录的文件夹
                foreach (var item in dirInfo)
                {
                    languages.Add(item.Name);
                }
                var dicLanguage = new Dictionary<string, JObject>();
                foreach (var langaugeItem in languages)
                {
                    var jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(langaugeItem + "authority");
                    if (string.IsNullOrWhiteSpace(jsonContent))
                    {
                        ThrowCustomerException(CustomerRpcCode.ExceptionError, "Not find Authority Config");
                    }
                    JObject json = JObject.Parse(jsonContent);
                    dicLanguage.Add(langaugeItem, json);
                }
                foreach (var item in adminFeatureList)
                {
                    if (item != null && !string.IsNullOrWhiteSpace(item.FeatureCode) && (item.LanguageConfigs == null || item.LanguageConfigs.Count <= 0))
                    {
                        var tempList = new List<UserFeatureLanguageConfigDO>();
                        foreach (var langaugeItem in languages)
                        {
                            var jsonData = dicLanguage[langaugeItem];
                            var entity = new UserFeatureLanguageConfigDO()
                            {
                                Language = langaugeItem,
                                FeatureName = jsonData["Admin"][item.UniqueCode].ToString()
                            };
                            tempList.Add(entity);
                        }
                        item.LanguageConfigs = tempList;
                    }
                }

                await _adminFeaturesDBRepository.InsertAllAsync(adminFeatureList.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList(), false);
            }
            //普通管理员角色
            var findRole2 = await _adminRoleDBRepository.FindOneAsync("Code", "AdminRoleDO02");
            if (findRole2 == null)
            {
                var data = new AdminRoleDO
                {
                    Code = "AdminRoleDO02",
                    RoleName = "管理员",
                    Description = "一般管理员，只管理自己区域内的机构、设备、用户数据",
                    LanguageConfigs = new List<AdminRoleLanguageConfigDO> { new AdminRoleLanguageConfigDO { Language = "zh-CN", RoleName = "管理员", Description = "一般管理员，只管理自己区域内的机构、设备、用户数据" }, new AdminRoleLanguageConfigDO { Language = "en-US", RoleName = "administrator", Description = "A general administrator who only manages institutions, devices, and user data within their own area" }, new AdminRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Администратор", Description = "Общий администратор, который управляет только учреждениями, оборудованием, пользовательскими данными в своем регионе" } },
                    AdminGroupCode = await _adminAuthorityGroupDBRepository.InsertOneAsync(new AdminAuthorityGroupDO
                    {
                        AdminGroupCode = CodeCreator.CreateCode("AdminGroupCode"),
                        Features = new List<string> {
                    "JGGL", "OrgDevices", "JGLB","SBGL","SBLB","YHGL","PTJS","YHLB","MBGL","RPMGI","BGBQ","CTKMB","CLWH","AIGL","AILB"}
                    },
                     new Dictionary<string, string>(), null, false),
                };
                var id = await _adminRoleDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }

            //用户权限
            var userFeatureCount = await _userFeatureDBRepository.CountDocumentsAsync();
            if (userFeatureCount == 0)
            {
                var userFeatureList = new List<UserFeatureDO>();
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UserCenter", FatherCode = "", FeatureName = "个人中心", UniqueCode = "UserCenter" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "OrgPersonnel", FatherCode = "UserCenter", FeatureName = "组织架构", UniqueCode = "OrgPersonnel" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UserTemplate", FatherCode = "UserCenter", FeatureName = "个人模板", UniqueCode = "UserTemplate" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "Password", FatherCode = "UserCenter", FeatureName = "密码", UniqueCode = "Password" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "Setting", FatherCode = "UserCenter", FeatureName = "设置", UniqueCode = "Setting" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "PersonalSetting", FatherCode = "Setting", FeatureName = "个性化设置", UniqueCode = "PersonalSetting" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UserProfile", FatherCode = "UserCenter", FeatureName = "账号", UniqueCode = "UserProfile" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UserEntry", FatherCode = "UserCenter", FeatureName = "个人词条", UniqueCode = "UserEntry" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UserDevices", FatherCode = "UserCenter", FeatureName = "个人设备", UniqueCode = "UserDevices" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "LogOut", FatherCode = "UserCenter", FeatureName = "退出登录", UniqueCode = "LogOut" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "OrgEntries", FatherCode = "UserCenter", FeatureName = "机构词条", UniqueCode = "OrgEntries" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "OrgDevices", FatherCode = "UserCenter", FeatureName = "机构设备", UniqueCode = "OrgDevices" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "OrgReportTemplate", FatherCode = "UserCenter", FeatureName = "机构模板", UniqueCode = "OrgReportTemplate" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "OrgInfo", FatherCode = "UserCenter", FeatureName = "机构信息", UniqueCode = "OrgInfo" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "Messages", FatherCode = "", FeatureName = "消息", UniqueCode = "Messages" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "Consultation", FatherCode = "", FeatureName = "预约会诊", UniqueCode = "Consultation" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "EmergencyConsultation", FatherCode = "Consultation", FeatureName = "急诊", UniqueCode = "EmergencyConsultation" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "ApplyConsultationList", FatherCode = "Consultation", FeatureName = "申请记录列表", UniqueCode = "ApplyConsultationList" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "ConsultationPage", FatherCode = "Consultation", FeatureName = "实时会诊页面", UniqueCode = "ConsultationPage" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "ApplyConsultation", FatherCode = "ApplyConsultationList", FeatureName = "创建申请", UniqueCode = "ApplyConsultation" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "UpdateApplyConsultation", FatherCode = "ApplyConsultationList", FeatureName = "修改或撤回申请", UniqueCode = "UpdateApplyConsultation" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "FindApplyConsultation", FatherCode = "ApplyConsultationList", FeatureName = "查看申请", UniqueCode = "FindApplyConsultation" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "ApplyUserField", FatherCode = "ApplyConsultationList", FeatureName = "申请人字段", UniqueCode = "ApplyUserField" });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "AuditApplication",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "同意或者拒绝申请",
                    UniqueCode = "AuditApplication"//审核申请
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "AuditInvitation",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "同意或者拒绝邀请",
                    UniqueCode = "AuditInvitation"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "SelectAssistant",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "设置助理",
                    UniqueCode = "SelectAssistant"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "StartConsultation",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "开始会诊",
                    UniqueCode = "StartConsultation"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "EditReport",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "撰写或者修改报告（会诊意见单）",
                    UniqueCode = "EditReport"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "FindConsultationReport",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "查看会诊报告或会诊意见单",
                    UniqueCode = "FindConsultationReport"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "CreateFollowups",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "创建随访",
                    UniqueCode = "CreateFollowups"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "DataExport",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "数据导出",
                    UniqueCode = "DataExport"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "Appraise",
                    FatherCode = "ApplyConsultationList",
                    FeatureName = "评价",
                    UniqueCode = "Appraise"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "Line",
                    FatherCode = "ConsultationPage",
                    FeatureName = "线段",
                    UniqueCode = "Line"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "Pen",
                    FatherCode = "ConsultationPage",
                    FeatureName = "画笔",
                    UniqueCode = "Pen"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "Clear",
                    FatherCode = "ConsultationPage",
                    FeatureName = "清除",
                    UniqueCode = "Clear"
                });
                // userFeatureList.Add(new UserFeatureDO
                // {
                //     FeatureCode = "Screenshot",
                //     FatherCode = "ConsultationPage",
                //     FeatureName = "截图（云端保存）",
                //     UniqueCode = "Screenshot"
                // });
                // userFeatureList.Add(new UserFeatureDO
                // {
                //     FeatureCode = "Recording",
                //     FatherCode = "ConsultationPage",
                //     FeatureName = "超声画面录制（云端保存）",
                //     UniqueCode = "Recording"
                // });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "SaveImage",
                    FatherCode = "ConsultationPage",
                    FeatureName = "存图",
                    UniqueCode = "SaveImage"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "CloseConsultation",
                    FatherCode = "ConsultationPage",
                    FeatureName = "结束会诊",
                    UniqueCode = "CloseConsultation"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "StartRecording",
                    FatherCode = "ConsultationPage",
                    FeatureName = "全程录制（开始/暂停）",
                    UniqueCode = "StartRecording"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "CloseRecording",
                    FatherCode = "ConsultationPage",
                    FeatureName = "结束录制",
                    UniqueCode = "CloseRecording"
                });
                userFeatureList.Add(new UserFeatureDO
                {
                    FeatureCode = "RemoteParameter",
                    FatherCode = "ConsultationPage",
                    FeatureName = "远程调参",
                    UniqueCode = "RemoteParameter"
                });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "Diagnosis", FatherCode = "", FeatureName = "分时诊断", UniqueCode = "Diagnosis" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "ImagePool", FatherCode = "Diagnosis", FeatureName = "图像池", UniqueCode = "ImagePool" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "PatientLists", FatherCode = "Diagnosis", FeatureName = "病人列表", UniqueCode = "PatientLists" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "DiagnosisEditReport", FatherCode = "Diagnosis", FeatureName = "撰写/修改报告", UniqueCode = "DiagnosisEditReport" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "EditReportTag", FatherCode = "Diagnosis", FeatureName = "添加标签", UniqueCode = "EditReportTag" });
                userFeatureList.Add(new UserFeatureDO { FeatureCode = "FlyinsonoCollege", FatherCode = "", FeatureName = "杏聆荟学院", UniqueCode = "FlyinsonoCollege" });

                var languages = new List<string>();
                DirectoryInfo theFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge"));
                DirectoryInfo[] dirInfo = theFolder.GetDirectories();//获取所在目录的文件夹
                foreach (var item in dirInfo)
                {
                    languages.Add(item.Name);
                }
                var dicLanguage = new Dictionary<string, JObject>();
                foreach (var langaugeItem in languages)
                {
                    var jsonContent = WingServerCommon.ResourceManage.ResourceManager.GetSetting(langaugeItem + "authority");
                    if (string.IsNullOrWhiteSpace(jsonContent))
                    {
                        ThrowCustomerException(CustomerRpcCode.ExceptionError, "Not find Authority Config");
                    }
                    JObject json = JObject.Parse(jsonContent);
                    dicLanguage.Add(langaugeItem, json);
                }
                foreach (var item in userFeatureList)
                {
                    if (item != null && !string.IsNullOrWhiteSpace(item.FeatureCode) && (item.LanguageConfigs == null || item.LanguageConfigs.Count <= 0))
                    {
                        var tempList = new List<UserFeatureLanguageConfigDO>();
                        foreach (var langaugeItem in languages)
                        {
                            var jsonData = dicLanguage[langaugeItem];
                            var entity = new UserFeatureLanguageConfigDO()
                            {
                                Language = langaugeItem,
                                FeatureName = jsonData["User"][item.UniqueCode].ToString()
                            };
                            tempList.Add(entity);
                        }
                        item.LanguageConfigs = tempList;
                    }
                }

                await _userFeatureDBRepository.InsertAllAsync(userFeatureList.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList(), false);
            }
            //用户角色
            var userRoleCount = await _roleDBRepository.CountDocumentsAsync();
            if (userRoleCount == 0)
            {
                var userRoleList = new List<UserRoleDO>();
                userRoleList.Add(new UserRoleDO
                {
                    RoleCode = "Role_GeneralUser",
                    RoleName = "普通用户",
                    UserGroupCode = await _userAuthorityGroupDBRepository.InsertOneAsync(new FrontAuthorityGroupDO
                    {
                        UserGroupCode = CodeCreator.CreateCode("UserGroupCode"),
                        Features = new List<string> {
                    "Password", "Setting", "UserProfile",
                    "LogOut"}
                    }, new Dictionary<string, string>(), null, false),
                    Description = "系统默认角色",
                    RoleShowType = RoleShowTypeEnum.NotShow,
                    LanguageConfigs = new List<UserRoleLanguageConfigDO> { new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "普通用户", Description = "系统默认角色" }, new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "general user", Description = "System default role" }, new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Обычный пользователь", Description = "Роль по умолчанию в системе" } }
                });
                userRoleList.Add(new UserRoleDO
                {
                    RoleCode = "Role_InternshipDoctor",
                    RoleName = "实习医生",
                    UserGroupCode = await _userAuthorityGroupDBRepository.InsertOneAsync(new FrontAuthorityGroupDO
                    {
                        UserGroupCode = CodeCreator.CreateCode("UserGroupCode"),
                        Features = new List<string> {
                     "UserTemplate", "Password", "Setting",
                      "UserProfile", "UserEntry",
                      "UserDevices",
                       "LogOut", "PatientLists",
                    "FindApplyConsultation", "AuditApplication","AuditInvitation","FindConsultationReport","FlyinsonoCollege" }
                    }, new Dictionary<string, string>(), null, false),
                    Description = "以学生的角色上课\n以学生身份参与实时会诊\n以学生身份参与远程会诊",
                    RoleShowType = RoleShowTypeEnum.ISShow,
                    LanguageConfigs = new List<UserRoleLanguageConfigDO> { new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "实习医生", Description = "以学生的角色上课\n以学生身份参与实时会诊\n以学生身份参与远程会诊" }, new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Intern", Description = "Taking course as a student\nParticipating in online consultation as a student\nParticipating in remote consultation as a student" }, new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Стажер-врач", Description = "Учиться в качестве студента\nУчастие в реальном времени в консультации в качестве студента\nУчастие в удаленной консультации в качестве студента" } },
                    IConUrl = "https://cdn-bj.fis.plus/bd8cbd443d3842029453870eb2656136.png",
                    ColorStart = "F2F9FF",
                    ColorEnd = "A4CDFD",
                    RoleQualification = RoleQualificationEnum.ID
                });
                userRoleList.Add(new UserRoleDO
                {
                    RoleCode = "Role_ExpertAssistant",
                    RoleName = "专家助理",
                    UserGroupCode = await _userAuthorityGroupDBRepository.InsertOneAsync(new FrontAuthorityGroupDO
                    {
                        UserGroupCode = CodeCreator.CreateCode("UserGroupCode"),
                        Features = new List<string> {
                    "UserTemplate", "Password", "Setting",
                     "UserProfile", "UserEntry",
                      "UserDevices",
                       "LogOut", "PatientLists",
                       "ApplyConsultation" ,"UpdateApplyConsultation" ,"FindApplyConsultation", "AuditApplication","AuditInvitation",
                       "FindConsultationReport","CreateFollowups","DataExport","ApplyUserField","FlyinsonoCollege" }
                    }, new Dictionary<string, string>(), null, false),
                    Description = "以助理的角色参与专家课堂\n以助理的角色参与实时会诊\n开展远程会诊",
                    RoleShowType = RoleShowTypeEnum.ISShow,
                    LanguageConfigs = new List<UserRoleLanguageConfigDO> { new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "专家助理", Description = "以助理的角色参与专家课堂\n以助理的角色参与实时会诊\n开展远程会诊" }, new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Expert assistant", Description = "Teaching with course content reviewed\nStarting a remote consultation" }, new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Помощник эксперта", Description = "Участие в экспертном классе в качестве ассистента\nУчастие в консультации в режиме реального времени в качестве ассистента\nПроведение удаленной консультации" } },
                    IConUrl = "https://cdn-bj.fis.plus/220dd2675da14986a142ee4bfdf51951.png",
                    ColorStart = "FBEDE5",
                    ColorEnd = "D6906D",
                    RoleQualification = RoleQualificationEnum.ID
                });
                userRoleList.Add(new UserRoleDO
                {
                    RoleCode = "Role_CertifiedPhysician",
                    RoleName = "认证医师",
                    UserGroupCode = await _userAuthorityGroupDBRepository.InsertOneAsync(new FrontAuthorityGroupDO
                    {
                        UserGroupCode = CodeCreator.CreateCode("UserGroupCode"),
                        Features = new List<string> {
                     "UserTemplate", "Password",
                     "Setting", "UserProfile",
                      "UserEntry", "UserDevices",
                     "LogOut",
                       "PatientLists","ApplyConsultation","DiagnosisEditReport","EditReportTag",
                       "UpdateApplyConsultation" ,"FindApplyConsultation",
                        "AuditApplication","AuditInvitation",
                        "EditReport","FindConsultationReport",
                        "CreateFollowups","Appraise",
                        "Line", "Pen",
                         "Clear", "SaveImage", "CloseConsultation",
                           "StartRecording", "CloseRecording",
                           "RemoteParameter", "FlyinsonoCollege"}
                    }, new Dictionary<string, string>(), null, false),
                    Description = "培训授课\n培训课程需后台审核\n开展远程会诊",
                    RoleShowType = RoleShowTypeEnum.ISShow,
                    LanguageConfigs = new List<UserRoleLanguageConfigDO> { new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "认证医师", Description = "培训授课\n培训课程需后台审核\n开展远程会诊" }, new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Physician", Description = "Teaching with course content reviewed\nStarting a remote consultation" }, new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Сертифицированный врач", Description = "Преподавание в рамках обучения\nКурсы обучения требуют проверки администратором\nПроведение дистанционной консультации" } },
                    IConUrl = "https://cdn-bj.fis.plus/86a8df6521b44c07ae199dde5ee9c25d.png",
                    ColorStart = "FEFEFE",
                    ColorEnd = "BBBBBB",
                    RoleQualification = RoleQualificationEnum.Both
                });
                userRoleList.Add(new UserRoleDO
                {
                    RoleCode = "Role_CertifiedExpert",
                    RoleName = "认证专家",
                    UserGroupCode = await _userAuthorityGroupDBRepository.InsertOneAsync(new FrontAuthorityGroupDO
                    {
                        UserGroupCode = CodeCreator.CreateCode("UserGroupCode"),
                        Features = new List<string> {
                     "UserTemplate", "Password",
                     "Setting", "UserProfile",
                      "UserEntry", "UserDevices",
                       "LogOut",
                       "PatientLists","ApplyConsultation",
                       "UpdateApplyConsultation" ,"FindApplyConsultation","DiagnosisEditReport","EditReportTag",
                        "AuditApplication","AuditInvitation",
                         "SelectAssistant" ,"StartConsultation",
                         "EditReport","FindConsultationReport",
                         "CreateFollowups","DataExport",
                         "Appraise","Line",
                         "Pen", "Clear",
                         "SaveImage",
                         "CloseConsultation", "StartRecording",
                         "CloseRecording", "RemoteParameter","FlyinsonoCollege"
                        }
                    }, new Dictionary<string, string>(), null, false),
                    Description = "培训授课\n培训课程无需审核\n开展专家会诊\n开展远程会诊",
                    RoleShowType = RoleShowTypeEnum.ISShow,
                    LanguageConfigs = new List<UserRoleLanguageConfigDO> { new UserRoleLanguageConfigDO { Language = "zh-CN", RoleName = "认证专家", Description = "培训授课\n培训课程无需审核\n开展专家会诊\n开展远程会诊" }, new UserRoleLanguageConfigDO { Language = "en-US", RoleName = "Certified Expert", Description = "Teaching without course content reviewed\nStarting a professional consultation\nStarting a remote consultation" }, new UserRoleLanguageConfigDO { Language = "ru-RU", RoleName = "Сертифицированный специалист", Description = "Обучение и преподавание\nКурсы обучения не требуют проверки\nПроведение консультации экспертов\nПроведение удаленной консультации" } },
                    IConUrl = "https://cdn-bj.fis.plus/23d3fc20d0344fd583bcafd7c6251b36.png",
                    ColorStart = "FFF5DF",
                    ColorEnd = "C3A773",
                    RoleQualification = RoleQualificationEnum.Both
                });
                await _roleDBRepository.InsertAllAsync(userRoleList.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList(), false);
            }
            //岗位身份
            var position01 = await _positionDBRepository.FindOneAsync("Code", "PositionCode01");
            if (findRole == null)
            {
                var data = new UserPositionDO
                {
                    Code = "PositionCode01",
                    PositionName = "主管"
                };
                var id = await _positionDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }
            var position02 = await _positionDBRepository.FindOneAsync("Code", "PositionCode02");
            if (findRole == null)
            {
                var data = new UserPositionDO
                {
                    Code = "PositionCode02",
                    PositionName = "普通员工"
                };
                var id = await _positionDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, false);
            }
            //设备类型
            var dictionaryNum = await _dictionaryDBRepository.CountDocumentsAsync();
            if (dictionaryNum == 0)
            {
                var typeCode = await _dictionaryDBRepository.InsertOneAsync(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceType, DictionaryCode = "US", Value = "超声机", LanguageConfigs = new List<DictionaryLanguageConfigDTO> { new DictionaryLanguageConfigDTO { Language = "zh-CN", Value = "超声机" }, new DictionaryLanguageConfigDTO { Language = "en-US", Value = "US" }, new DictionaryLanguageConfigDTO { Language = "ru-RU", Value = "US" } } }, new Dictionary<string, string>(), string.Empty, false);

                // var dictionaryDOList = new List<DictionaryDO>();
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "E30" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "V1" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "V2" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "V3" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "V4" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = typeCode, Value = "V5" });
                // await _dictionaryDBRepository.InsertAllAsync(dictionaryDOList.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList(), false);

                var sonopostCode = await _dictionaryDBRepository.InsertOneAsync(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceType, DictionaryCode = "sonopost", Value = "超声魔盒", LanguageConfigs = new List<DictionaryLanguageConfigDTO> { new DictionaryLanguageConfigDTO { Language = "zh-CN", Value = "超声魔盒" }, new DictionaryLanguageConfigDTO { Language = "en-US", Value = "sonopost" }, new DictionaryLanguageConfigDTO { Language = "ru-RU", Value = "sonopost" } } }, new Dictionary<string, string>(), string.Empty, false);
                // dictionaryDOList = new List<DictionaryDO>();
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = sonopostCode, Value = "V1" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = sonopostCode, Value = "V2" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = sonopostCode, Value = "V3" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = sonopostCode, Value = "V4" });
                // dictionaryDOList.Add(new DictionaryDO { DictionaryType = (int)DictionaryTypeEnum.DeviceModel, ParentCode = sonopostCode, Value = "SP-0002" });
                // await _dictionaryDBRepository.InsertAllAsync(dictionaryDOList.Select(x => (x, new Dictionary<string, string>(), string.Empty)).ToList(), false);
            }
            //词条
            await CheckAndInsertDefaultThesaurus();
            //系统环境配置
            var systemConfigNum = await _SystemConfigDBRepository.CountDocumentsAsync();
            if (systemConfigNum == 0)
            {
                await InsertDefaultSystemConfig();
            }
            var cmsTemplateNum = await _cmsTemplateDBRepository.CountDocumentsAsync();
            if (cmsTemplateNum == 0)
            {
                //首页默认模板设计
                await AddCMSTemplatesDBAsync(new AddCMSTemplatesDBRequest
                {
                    Data = new CMSTemplateDTO
                    {
                        CMSTemplateName = "默认首页",
                        CMSTemplateCode = "mrmb",
                        CMSTemplateJson = "{\"id\":\"mrmb\",\"customizeTranslation\":[{\"keyword\":\"cmsPlatActivityModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"平台活动\"},{\"key\":\"English\",\"value\":\"Plat Activity\"}]},{\"keyword\":\"cmsUpdatesCourseModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"已订阅的课程更新\"},{\"key\":\"English\",\"value\":\"Updates Course\"}]},{\"keyword\":\"cmsDevicesModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"产品展示\"},{\"key\":\"English\",\"value\":\"Devices\"}]},{\"keyword\":\"cmsMyPendingModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"我的待办\"},{\"key\":\"English\",\"value\":\"My Pending\"}]},{\"keyword\":\"cmsMyStandingModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"我的战绩\"},{\"key\":\"English\",\"value\":\"My Standing\"}]},{\"keyword\":\"cmsNewsModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"消息盒子\"},{\"key\":\"English\",\"value\":\"News\"}]},{\"keyword\":\"cmsMessageModule\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"系统公告\"},{\"key\":\"English\",\"value\":\"Announces\"}]},{\"keyword\":\"cmsAppDownload\",\"type\":\"module\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"APP下载\"},{\"key\":\"English\",\"value\":\"App Download\"}]},{\"keyword\":\"cmsWaitReport\",\"type\":\"pending\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"待撰写的报告\"},{\"key\":\"English\",\"value\":\"Wait Report\"}]},{\"keyword\":\"cmsNotScanned\",\"type\":\"pending\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"待完成的扫查\"},{\"key\":\"English\",\"value\":\"Not Scanned\"}]},{\"keyword\":\"cmsWaitConsult\",\"type\":\"pending\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"待参加的会诊\"},{\"key\":\"English\",\"value\":\"Wait Consult\"}]},{\"keyword\":\"cmsDiagnosisPatients\",\"type\":\"standing\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"已诊断病人\"},{\"key\":\"English\",\"value\":\"Diagnosis Patients\"}]},{\"keyword\":\"cmsCompleteReport\",\"type\":\"standing\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"已完成报告\"},{\"key\":\"English\",\"value\":\"Complete Report\"}]},{\"keyword\":\"cmsJoinedConsult\",\"type\":\"standing\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"已参与会诊\"},{\"key\":\"English\",\"value\":\"Joined Consult\"}]},{\"keyword\":\"cmsUnitPerson\",\"type\":\"unit\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"人\"},{\"key\":\"English\",\"value\":\"Person\"}]},{\"keyword\":\"cmsUnitCopies\",\"type\":\"unit\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"份\"},{\"key\":\"English\",\"value\":\"Copies\"}]},{\"keyword\":\"cmsUnitPlace\",\"type\":\"unit\",\"languageDatas\":[{\"key\":\"Chinese\",\"value\":\"场\"},{\"key\":\"English\",\"value\":\"Place\"}]}],\"tasks\":[{\"icon\":\"sc-icon-card\",\"label\":\"cmsTable\",\"id\":\"2f4895ba-9f30-437e-93b4-30251d704604\",\"name\":\"table\",\"data\":{},\"result\":{\"parentId\":\"\",\"parentName\":\"\",\"tasks\":[{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"14dd3ea1-1864-4cbd-bf1f-3a996069c960\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"2f4895ba-9f30-437e-93b4-30251d704604\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-card\",\"label\":\"cmsTable\",\"id\":\"0c915edb-0b6c-4912-99b7-302130ff53e0\",\"name\":\"table\",\"data\":{},\"result\":{\"parentId\":\"14dd3ea1-1864-4cbd-bf1f-3a996069c960\",\"parentName\":\"cell\",\"tasks\":[{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"61ecb066-a7b2-46c5-975e-b5d979451920\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"0c915edb-0b6c-4912-99b7-302130ff53e0\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-picture\",\"label\":\"cmsImage\",\"id\":\"b6c52932-2ea8-4aa1-bd52-a7b8b096a4e0\",\"name\":\"image\",\"data\":{},\"result\":{\"parentId\":\"61ecb066-a7b2-46c5-975e-b5d979451920\",\"parentName\":\"cell\",\"grandParentId\":\"0c915edb-0b6c-4912-99b7-302130ff53e0\",\"grandParentName\":\"table\",\"property\":{\"fit\":\"fill\",\"id\":\"13db9d80-c70a-4c77-af65-858de5a23a72\",\"name\":\"首图\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/C21FD66E214045B28E25149EDA7A3503.png\",\"language\":\"zh-CN\",\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/0479EE993205473EBBF7122635750E68.png\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/7D0AD70D4F954CB8A3A58D7A0304B770.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/289109C19BC94D8DB1636F834A913152.png\",\"transferUrl\":\"\",\"type\":\"image\"}}}],\"property\":{\"cellMinRowHeight\":34.5,\"cellColumnWidth\":100,\"cellRowHeight\":100,\"rowStartIndex\":1,\"colStartIndex\":1,\"rowEndIndex\":2,\"colEndIndex\":2,\"rowSpan\":1,\"colSpan\":1,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}}],\"property\":{\"allMinHeight\":34.5,\"isWrapBreak\":true,\"cellMinRowHeightList\":[34.5],\"cellMinRowHeightListStr\":\"34.5\",\"isRedraw\":false,\"backgroundColor\":\"#FFFFFF\",\"columnWidthListStr\":\"100\",\"rowHeightListStr\":\"100\",\"columnWidthList\":[100],\"rowHeightList\":[100],\"isSetInterval\":false,\"rowCount\":1,\"columnCount\":1,\"columnInterval\":5,\"rowInterval\":5,\"isInitFlex\":false,\"isShowHeader\":false,\"headerTitle\":\"\",\"headerIcon\":\"\",\"isUseCarousel\":false,\"carouselAutoPlay\":true,\"carouselInterval\":3000,\"carouselDirection\":\"horizontal\",\"isUseDynamicData\":false,\"defaultData\":[{\"id\":\"13db9d80-c70a-4c77-af65-858de5a23a72\",\"name\":\"首图\",\"transferUrl\":\"\",\"imgUrl\":\"https://cdn-bj.fis.plus/C21FD66E214045B28E25149EDA7A3503.png\",\"type\":\"image\",\"autoFitBackground\":false,\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/0479EE993205473EBBF7122635750E68.png\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/7D0AD70D4F954CB8A3A58D7A0304B770.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/289109C19BC94D8DB1636F834A913152.png\",\"language\":\"zh-CN\"}],\"appointDataSource\":\"\",\"defaultDataSource\":\"\",\"operateType\":\"\",\"selectedField\":\"image\"}}}],\"property\":{\"cellMinRowHeight\":34.81,\"cellColumnWidth\":100,\"cellRowHeight\":\"65\",\"rowStartIndex\":1,\"colStartIndex\":1,\"rowEndIndex\":2,\"colEndIndex\":3,\"rowSpan\":1,\"colSpan\":2,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}},{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"9be6b67a-a6ec-4fc9-817e-debe14e2ea23\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"2f4895ba-9f30-437e-93b4-30251d704604\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-card\",\"label\":\"cmsTable\",\"id\":\"178119eb-9816-4c74-855d-410232def7aa\",\"name\":\"table\",\"data\":{},\"result\":{\"parentId\":\"9be6b67a-a6ec-4fc9-817e-debe14e2ea23\",\"parentName\":\"cell\",\"tasks\":[{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"2a352f8e-1759-4d73-ab9d-05a7c94761cf\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"178119eb-9816-4c74-855d-410232def7aa\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-picture\",\"label\":\"cmsImage\",\"id\":\"b9e692e8-3c7b-4b81-a7f1-7970d929bf4c\",\"name\":\"image\",\"data\":{},\"result\":{\"parentId\":\"2a352f8e-1759-4d73-ab9d-05a7c94761cf\",\"parentName\":\"cell\",\"grandParentId\":\"178119eb-9816-4c74-855d-410232def7aa\",\"grandParentName\":\"table\",\"property\":{\"fit\":\"fill\",\"id\":\"5a413da9-8a80-4d07-9691-729e9e95b8e8\",\"name\":\"中文魔盒\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/B9678C78576340D597180CA346AD3000.png\",\"transferUrl\":\"\",\"type\":\"image\",\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/BF98D1FBB828401AB73D8751536D1CFE.png\",\"language\":\"zh-CN\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/3EF15C18FB52490B8281140C057E6391.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/5C07E66080954CCD87C5937434B3BB37.png\"}}}],\"property\":{\"cellMinRowHeight\":21.75,\"cellColumnWidth\":100,\"cellRowHeight\":100,\"rowStartIndex\":1,\"colStartIndex\":1,\"rowEndIndex\":2,\"colEndIndex\":2,\"rowSpan\":1,\"colSpan\":1,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}}],\"property\":{\"allMinHeight\":21.75,\"isWrapBreak\":true,\"cellMinRowHeightList\":[21.75],\"cellMinRowHeightListStr\":\"21.75\",\"isRedraw\":false,\"backgroundColor\":\"#FFFFFF\",\"columnWidthListStr\":\"100\",\"rowHeightListStr\":\"100\",\"columnWidthList\":[100],\"rowHeightList\":[100],\"isSetInterval\":false,\"rowCount\":1,\"columnCount\":1,\"columnInterval\":5,\"rowInterval\":5,\"isInitFlex\":false,\"isShowHeader\":false,\"headerTitle\":\"\",\"headerIcon\":\"\",\"isUseCarousel\":false,\"carouselAutoPlay\":true,\"carouselInterval\":3000,\"carouselDirection\":\"horizontal\",\"isUseDynamicData\":false,\"defaultData\":[{\"fit\":\"fill\",\"id\":\"5a413da9-8a80-4d07-9691-729e9e95b8e8\",\"name\":\"中文魔盒\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/B9678C78576340D597180CA346AD3000.png\",\"transferUrl\":\"\",\"type\":\"image\",\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/BF98D1FBB828401AB73D8751536D1CFE.png\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/3EF15C18FB52490B8281140C057E6391.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/5C07E66080954CCD87C5937434B3BB37.png\",\"language\":\"zh-CN\"}],\"appointDataSource\":\"\",\"defaultDataSource\":\"\",\"operateType\":\"\",\"selectedField\":\"image\"}}}],\"property\":{\"cellMinRowHeight\":21.75,\"cellColumnWidth\":50,\"cellRowHeight\":\"35\",\"rowStartIndex\":2,\"colStartIndex\":1,\"rowEndIndex\":3,\"colEndIndex\":2,\"rowSpan\":1,\"colSpan\":1,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}},{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"3eed3464-b35a-4958-82d8-5aa26cee6bec\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"2f4895ba-9f30-437e-93b4-30251d704604\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-card\",\"label\":\"cmsTable\",\"id\":\"e0c273e9-31b0-49d0-8cd7-9dd698c35410\",\"name\":\"table\",\"data\":{},\"result\":{\"parentId\":\"3eed3464-b35a-4958-82d8-5aa26cee6bec\",\"parentName\":\"cell\",\"tasks\":[{\"icon\":\"icon-cell\",\"label\":\"cmsCell\",\"id\":\"f9d57595-04a0-42fe-9187-e97f67edfc3d\",\"name\":\"cell\",\"data\":{},\"result\":{\"parentId\":\"e0c273e9-31b0-49d0-8cd7-9dd698c35410\",\"parentName\":\"table\",\"tasks\":[{\"icon\":\"sc-icon-picture\",\"label\":\"cmsImage\",\"id\":\"68655866-59b6-4894-b469-5f4b2c1bbc27\",\"name\":\"image\",\"data\":{},\"result\":{\"parentId\":\"f9d57595-04a0-42fe-9187-e97f67edfc3d\",\"parentName\":\"cell\",\"grandParentId\":\"e0c273e9-31b0-49d0-8cd7-9dd698c35410\",\"grandParentName\":\"table\",\"property\":{\"fit\":\"fill\",\"id\":\"16b11100-ead8-4109-bec8-f0ade873a6e6\",\"name\":\"中文APP\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/6002C8A616BB47FFA064D761F5E165A5.png\",\"transferUrl\":\"\",\"type\":\"image\",\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/741F019280F04E0293E6B0952CF46EC8.png\",\"language\":\"zh-CN\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/C11829F8AC164FC5A426AF088064BE23.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/00B2A1D8523F4330849C0991DF297C21.png\"}}}],\"property\":{\"cellMinRowHeight\":21.75,\"cellColumnWidth\":100,\"cellRowHeight\":100,\"rowStartIndex\":1,\"colStartIndex\":1,\"rowEndIndex\":2,\"colEndIndex\":2,\"rowSpan\":1,\"colSpan\":1,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}}],\"property\":{\"allMinHeight\":21.75,\"isWrapBreak\":true,\"cellMinRowHeightList\":[21.75],\"cellMinRowHeightListStr\":\"21.75\",\"isRedraw\":false,\"backgroundColor\":\"#FFFFFF\",\"columnWidthListStr\":\"100\",\"rowHeightListStr\":\"100\",\"columnWidthList\":[100],\"rowHeightList\":[100],\"isSetInterval\":false,\"rowCount\":1,\"columnCount\":1,\"columnInterval\":5,\"rowInterval\":5,\"isInitFlex\":false,\"isShowHeader\":false,\"headerTitle\":\"\",\"headerIcon\":\"\",\"isUseCarousel\":false,\"carouselAutoPlay\":true,\"carouselInterval\":3000,\"carouselDirection\":\"horizontal\",\"isUseDynamicData\":false,\"defaultData\":[{\"fit\":\"fill\",\"id\":\"16b11100-ead8-4109-bec8-f0ade873a6e6\",\"name\":\"中文APP\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/6002C8A616BB47FFA064D761F5E165A5.png\",\"transferUrl\":\"\",\"type\":\"image\",\"imgPhoneUrl\":\"https://cdn-bj.fis.plus/741F019280F04E0293E6B0952CF46EC8.png\",\"imgUrlEN\":\"https://cdn-bj.fis.plus/C11829F8AC164FC5A426AF088064BE23.png\",\"imgPhoneUrlEN\":\"https://cdn-bj.fis.plus/00B2A1D8523F4330849C0991DF297C21.png\",\"language\":\"zh-CN\"}],\"appointDataSource\":\"\",\"defaultDataSource\":\"\",\"operateType\":\"\",\"selectedField\":\"image\"}}}],\"property\":{\"cellMinRowHeight\":21.75,\"cellColumnWidth\":50,\"cellRowHeight\":\"35\",\"rowStartIndex\":2,\"colStartIndex\":2,\"rowEndIndex\":3,\"colEndIndex\":3,\"rowSpan\":1,\"colSpan\":1,\"mergeBottomCell\":false,\"mergeRightCell\":false,\"backgroundColor\":\"#FFFFFF\",\"spliceCell\":false}}}],\"property\":{\"allMinHeight\":78.31,\"isWrapBreak\":true,\"cellMinRowHeightList\":[34.81,21.75,21.75],\"cellMinRowHeightListStr\":\"34.81,21.75,21.75\",\"isRedraw\":false,\"backgroundColor\":\"#FFFFFF\",\"columnWidthListStr\":\"50,50\",\"rowHeightListStr\":\"65,35\",\"columnWidthList\":[50,50],\"rowHeightList\":[\"65\",\"35\"],\"isSetInterval\":false,\"rowCount\":2,\"columnCount\":2,\"columnInterval\":10,\"rowInterval\":5,\"isInitFlex\":true,\"isShowHeader\":false,\"headerTitle\":\"\",\"headerIcon\":\"\",\"isUseCarousel\":false,\"carouselAutoPlay\":true,\"carouselInterval\":3000,\"carouselDirection\":\"horizontal\",\"isUseDynamicData\":false,\"defaultData\":[{\"fit\":\"fill\",\"id\":\"f14adb1f-16ea-4a01-8e73-34f2f35a61de\",\"name\":\"默认图片\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/000520dfc7a2431aa700027a39ca71f6.jpg\",\"transferUrl\":\"\",\"type\":\"image\"},{\"fit\":\"fill\",\"id\":\"1461eff1-f105-4784-87a8-5647b5854543\",\"name\":\"默认图片\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/000520dfc7a2431aa700027a39ca71f6.jpg\",\"transferUrl\":\"\",\"type\":\"image\"},{\"fit\":\"fill\",\"id\":\"b32c7fb2-7bb9-4b54-a8b2-fe6cca8a8188\",\"name\":\"默认图片\",\"autoFitBackground\":false,\"imgUrl\":\"https://cdn-bj.fis.plus/000520dfc7a2431aa700027a39ca71f6.jpg\",\"transferUrl\":\"\",\"type\":\"image\"}],\"appointDataSource\":\"\",\"defaultDataSource\":\"\",\"operateType\":\"\"}}}]}",
                        IsDefault = true
                    }
                });
            }
            //管理端统计
            var adminStatisticNum = await _adminStatisticDBRepository.CountDocumentsAsync();
            if (adminStatisticNum == 0)
            {
                await InsertAdminStatisticAsync("UserActivate", "认证用户", "0 0/3 * * * ?", "GetUserActivateInfoStatisticAsync");
                await InsertAdminStatisticAsync("UserApply", "我的待办", "0 0/3 * * * ?", "GetUserActivateLogStatisticAsync");
                await InsertAdminStatisticAsync("UserRole", "认证角色", "0 0/3 * * * ?", "GetUserRoleStatisticAsync");
                await InsertAdminStatisticAsync("OrganizationUsage", "机构分布", "0 0/3 * * * ?", "GetOrganizationStatisticAsync");
                await InsertAdminStatisticAsync("DeviceType", "机型分布，设备型号分布", "0 0/3 * * * ?", "GetDeviceStatisticAsync");

            }
            //统计配置执行统计
            var statisticNum = await _statisticDBRepository.CountDocumentsAsync();
            if (statisticNum == 0)
            {
                //CMS 模块我的代办
                await InsertCMSStatisticAsync("MyPending", "我的待办", "0 0/3 * * * ?", "GetMyPendingStatisticAsync");
                await InsertCMSStatisticAsync("MyStanding", "我的战绩", "0 0/3 * * * ?", "GetMyStandingStatisticAsync");
                await InsertCMSStatisticAsync("StatisticMessages", "消息盒子", "0 0/3 * * * ?", "GetStatisticMessagesAsync");
                await InsertCMSStatisticAsync("SystemAnnounce", "系统公告", "0 0/3 * * * ?", "GetSystemAnnounceAsync");
                // await InsertCMSStatisticAsync("CheckPlace", "检查部位", "0 0/3 * * * ?", "GetCheckPlaceStatisticAsync");
                // await InsertCMSStatisticAsync("SubmittedReports ", "提交报告数量", "0 0/3 * * * ?", "GetSubmittedReportsStatisticAsync");
                // await InsertCMSStatisticAsync("RealtimeData", "实时数据", "0 0/3 * * * ?", "GetRealtimeDataStatisticAsync");
                // await InsertCMSStatisticAsync("QualityControlReport", "报告质控", "0 0/3 * * * ?", "GetQualityControlReportStatisticAsync");
                // await InsertCMSStatisticAsync("QualityControlImage", "图像质控", "0 0/3 * * * ?", "GetQualityControlImageStatisticAsync");
                // await InsertCMSStatisticAsync("Diagnosis", "诊断统计", "0 0/3 * * * ?", "GetDiagnosisStatisticAsync");
                // await InsertCMSStatisticAsync("PositiveCase", "阳性病例", "0 0/3 * * * ?", "GetPositiveCaseStatisticAsync");
            }
            var gradingProtectionConfigNum = await _gradingProtectionConfigDBRepository.CountDocumentsAsync();
            if (gradingProtectionConfigNum == 0)
            {
                //密码错误限制
                await InsertGradingProtectionConfigAsync("PasswordErrorLimit01", "PasswordErrorLimit", "", GradingConfigTypeEnum.Switch, true.ToString());
                //密码过期限制
                await InsertGradingProtectionConfigAsync("PasswordOverdueLimit01", "PasswordOverdueLimit", "", GradingConfigTypeEnum.Switch, true.ToString());
                //密码规则
                await InsertGradingProtectionConfigAsync("Rule", "PasswordRule", "", GradingConfigTypeEnum.Text, "6,1,1,1,1");
            }
            //创建VINNO杏聆荟平台（可绑定所有设备、可查看所有数据）
            //统计配置执行统计
            await CheckAndInsertBaseOrgInfo();
            await InsertDefaultReportTemplate();
            var allTemplate = await _reportTemplateDBRepository.FindAllAsync();
            if (allTemplate != null)
            {
                var updateTemplateCodes = new List<string>();
                foreach (var template in allTemplate)
                {
                    if (string.IsNullOrWhiteSpace(template.LanguageCode))
                    {
                        updateTemplateCodes.Add(template.Code);
                    }
                }
                if (updateTemplateCodes.Count > 0)
                {
                    var reportTemplateFilter = new MongoCustomerFilter<MongoTable>();
                    reportTemplateFilter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "Code", updateTemplateCodes);
                    var reportTemplateUpdate = new MongoCustomerUpdate<MongoTable>();
                    reportTemplateUpdate.SetField("BaseData.LanguageCode", "zh-CN");
                    await _reportTemplateDBRepository.UpdateAllAsync(reportTemplateFilter.AndCombine(), reportTemplateUpdate.AllUpdateDefinition);
                }
            }
            var allExtends = await _userExtendsDBRepository.FindAllAsync();
            if (allExtends != null)
            {
                var updateExtends = new List<string>();
                foreach (var extend in allExtends)
                {
                    if (string.IsNullOrWhiteSpace(extend.LanguageCode))
                    {
                        extend.LanguageCode = "zh-CN";
                        await _userExtendsDBRepository.UpdateOneAsync("Code", extend.Code, extend, string.Empty);
                    }
                }
            }
        }

        private async Task CheckAndInsertDefaultThesaurus()
        {
            var thesaurusNum = await _thesaurusDBRepository.CountDocumentsAsync();
            if (thesaurusNum == 0)
            {
                await AddThesaurusDO("人用超声通用词库", "HumanGeneral-CN.json", "zh-CN", ThesaurusUsageTypeEnum.People, "EA6089158A524FEFBB3F3F33F96871FE");
                //await AddThesaurusDO("兽用超声通用词库", "PetAbdomen.json", "zh-CN", ThesaurusUsageTypeEnum.Animal);
                await AddThesaurusDO("Human General", "HumanGeneral.json", "en-US", ThesaurusUsageTypeEnum.People, "DD6C8473E9CA41EB999453C4EE5100BB");
                await AddThesaurusDO("универсальный словарь ультразвукового исследования у человека", "HumanGeneral-RU.json", "ru-RU", ThesaurusUsageTypeEnum.People, "D6BBC80FABA846F2A37D457FAC55D093");
                // await AddThesaurusDO("兽用超声通用词库", "兽用超声通用词库.json", "zh-CN", ThesaurusUsageTypeEnum.Animal, "0565594460904F7D923F0C74AF250FEC");
                // await AddThesaurusDO("Animal General", "Animal General.json", "en-US", ThesaurusUsageTypeEnum.Animal, "675EFE1EB0F3412587407245565323ED");
                // await AddThesaurusDO("Обзор животных", "Обзор животных.json", "ru-RU", ThesaurusUsageTypeEnum.Animal, "E357A0982ABB4472BE50135C448C8191");
            }
            //Electrocardiogram.json
            //Electrocardiogram-CN.json
            MongoCustomerFilter<MongoTable> electrocardiogramThesaurusFilter = new();
            MongoCustomerFilter<MongoDataItem> itemFilter = new();
            itemFilter.AppendEq((item => item.Key, "Code"), (item => item.Value, "B941E5E0DC2D429F839CCF6191925A79"));
            electrocardiogramThesaurusFilter.SubFilter(item => item.QueryData.QueryItems, itemFilter.AndCombine());
            var electrocardiogramthesaurusNum = await _thesaurusDBRepository.CountDocumentsAsync(electrocardiogramThesaurusFilter.AndCombine());
            if (electrocardiogramthesaurusNum == 0)
            {
                // await AddThesaurusDO("兽用心电通用词库", "Electrocardiogram-CN.json", "zh-CN", ThesaurusUsageTypeEnum.Animal, "B941E5E0DC2D429F839CCF6191925A78", ThesaurusBusinessTypeEnum.Electrocardiogram);
                await AddThesaurusDO("人用心电通用词库", "Electrocardiogram-CN.json", "zh-CN", ThesaurusUsageTypeEnum.People, "B941E5E0DC2D429F839CCF6191925A79", ThesaurusBusinessTypeEnum.Electrocardiogram);
            }
            MongoCustomerFilter<MongoTable> electrocardiogramENThesaurusFilter = new();
            MongoCustomerFilter<MongoDataItem> itemENFilter = new();
            itemENFilter.AppendEq((item => item.Key, "Code"), (item => item.Value, "B941E5E0DC2D429F839CCF6191925A77"));
            electrocardiogramENThesaurusFilter.SubFilter(item => item.QueryData.QueryItems, itemENFilter.AndCombine());
            var electrocardiogramENThesaurusNum = await _thesaurusDBRepository.CountDocumentsAsync(electrocardiogramENThesaurusFilter.AndCombine());
            if (electrocardiogramENThesaurusNum == 0)
            {
                // await AddThesaurusDO("Animal Electrocardiogram", "Electrocardiogram.json", "en-US", ThesaurusUsageTypeEnum.Animal, "B941E5E0DC2D429F839CCF6191925A76", ThesaurusBusinessTypeEnum.Electrocardiogram);
                await AddThesaurusDO("Human Electrocardiogram", "Electrocardiogram.json", "en-US", ThesaurusUsageTypeEnum.People, "B941E5E0DC2D429F839CCF6191925A77", ThesaurusBusinessTypeEnum.Electrocardiogram);
            }
        }

        /// <summary>
        /// 检查并插入基础平台配置数据
        /// </summary>
        private async Task CheckAndInsertBaseOrgInfo()
        {
            var basicOrganization = await _organizationDBRepository.FindOneAsync("Code", "VINNO_FIS");
            if (basicOrganization == null || string.IsNullOrWhiteSpace(basicOrganization.Code))
            {
                basicOrganization = new OrganizationDO();
                basicOrganization.Code = "VINNO_FIS";
                basicOrganization.OrganizationName = "VINNO FIS";
                basicOrganization.RootCode = "VINNO_FIS";
                basicOrganization.State = OrganizationStateEnum.Audited;
                basicOrganization.IsEncryptedShow = false;
                basicOrganization.ShortCode = "SEPVWG";
                basicOrganization.DiagnosisModules = new List<string> { "BreastLesionDetectBIRADS", "LiverLesionDetect", "ThyroidLesionDetect", "CarotidPlaqueDetect" };
                await _organizationDBRepository.InsertOneAsync(basicOrganization, new Dictionary<string, string>(), "", isWriteOperationLog: false);
            }
        }

        private async Task CheckAndInsertDefaultReportTemplate()
        {
            var thesaurusNum = await _reportTemplateDBRepository.CountDocumentsAsync();
            if (thesaurusNum == 0)
            {
                var reportTemplateU = new ReportTemplateDO();
                reportTemplateU.Code = "2e1441e904c2e2cbea7ba7d692e5e98f32";
                reportTemplateU.ReportTemplateName = "HEIUltrasonic";
                reportTemplateU.IsDefault = true;
                var templatePathU = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIUltrasonic.json");
                reportTemplateU.ReportTemplateJson = File.ReadAllText(templatePathU);
                reportTemplateU.ReportDatasJson = "";
                reportTemplateU.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                reportTemplateU.ReportTemplateType = ReportTemplateTypeEnum.Default;
                reportTemplateU.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
                reportTemplateU.LanguageCode = "zh-CN";
                MongoCustomerFilter<MongoTable> filter1 = new MongoCustomerFilter<MongoTable>();
                filter1.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "2e1441e904c2e2cbea7ba7d692e5e98f32"));
                if (!await _reportTemplateDBRepository.Exists(filter1.AndCombine()))
                {
                    await _reportTemplateDBRepository.InsertOneAsync(reportTemplateU, new Dictionary<string, string>(), string.Empty, false);
                }
            }

            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "38ab39f05188433a820527997f064a310b3d";
            reportTemplate.ReportTemplateName = "HEIECG(Vertical)";
            reportTemplate.IsDefault = true;
            var templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIECG.json");
            reportTemplate.ReportTemplateJson = File.ReadAllText(templatePath);
            reportTemplate.ReportDatasJson = "";
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";
            MongoCustomerFilter<MongoTable> filter2 = new MongoCustomerFilter<MongoTable>();
            filter2.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "38ab39f05188433a820527997f064a310b3d"));
            if (!await _reportTemplateDBRepository.Exists(filter2.AndCombine()))
            {
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "38ab39f05186daca820527997f064a310b3f";
            reportTemplate.ReportTemplateName = "HEIECG(Horizontal)";
            reportTemplate.IsDefault = true;
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIECG(Horizontal).json");
            reportTemplate.ReportTemplateJson = File.ReadAllText(templatePath);
            reportTemplate.ReportDatasJson = "";
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";
            MongoCustomerFilter<MongoTable> filter3 = new MongoCustomerFilter<MongoTable>();
            filter3.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "38ab39f05186daca820527997f064a310b3f"));
            if (!await _reportTemplateDBRepository.Exists(filter3.AndCombine()))
            {
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

        }

        /// <summary>
        /// 插入配置数据
        /// </summary>
        private async Task InsertGradingProtectionConfigAsync(string moduleCode, string name, string description, GradingConfigTypeEnum configType, string value)
        {
            //
            var config = new GradingProtectionConfigEntity
            {
                ModuleCode = moduleCode,
                Name = name,
                Description = description,
                ConfigType = (int)configType,
                Value = value
            };
            await _gradingProtectionConfigDBRepository.InsertOneAsync(config);
        }

        /// <summary>
        /// 插入默认环境配置
        /// </summary>
        /// <returns></returns>
        private async Task InsertDefaultSystemConfig()
        {
            var entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultCMS",
                ConfigKey = "CMS",
                ConfigContent = "{\"CMSUrl\":\"\"}",
                ConfigPageControls = ""
            };
            var r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultStorage",
                ConfigKey = "Storage",
                ConfigContent = "{\"AutoSliceSizeForUpload\":5242880,\"ManualDivisionForUpload\":209715200,\"OtherNodes\":\"flyinsono-bj-1300984704|ap-beijing;flyinsono-hk-1300984704|ap-hongkong;flyinsono-fra-1300984704|eu-frankfurt\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultVodPlayer",
                ConfigKey = "VodPlayer",
                ConfigContent = "{\"AppKey\":\"\",\"AppSecret\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultReport",
                ConfigKey = "Report",
                ConfigContent = "{\"MessageUrl\":\"\",\"ReportUrl\":\"\",\"ReportMessageTemplateId\":\"\",\"ShareCodePrefix\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultLiveConsultation",
                ConfigKey = "LiveConsultation",
                ConfigContent = "{\"ReminderSmsTemplateId\":\"\",\"InviteSmsTemplateId\":\"\",\"SubmitSmsTemplateId\":\"\",\"DefaultQueryDays\":30,\"EmergencyTimeout\":15}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultAlipay",
                ConfigKey = "Alipay",
                ConfigContent = "{\"GatewayHost\":\"\",\"AppId\":\"\",\"MerchantPrivateKey\":\"\",\"AlipayPublicKey\":\"\",\"NotifyUrl\":\"\",\"SignType\":\"\",\"PayShortBaseUrl\":\"\",\"PayReturnUrl\":\"\",\"Enable\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultASR",
                ConfigKey = "ASR",
                ConfigContent = "{\"AppKey\":\"\",\"AppSecret\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultCourse",
                ConfigKey = "Course",
                ConfigContent = "{\"CourseUrl\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultLab",
                ConfigKey = "Lab",
                ConfigContent = "{\"TemplateFileUrl\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultShare",
                ConfigKey = "Share",
                ConfigContent = "{\"ShareExamUrl\":\"\",\"DeviceLiveUrl\":\"\",\"CourseLiveUrl\":\"\",\"CourseShareUrl\":\"\",\"ShareExamImagesUrl\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultRtmp",
                ConfigKey = "Rtmp",
                ConfigContent = "{\"RTMPType\":\"Embedded\",\"RTMPAppId\":\"\",\"RTMPAppKey\":\"\",\"RTMPPlayUrl\":\"\",\"RTMPPushUrl\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultGeneral",
                ConfigKey = "General",
                ConfigContent = "{\"LiveProtocol\":\"\",\"AssociatedSystemIcon\":\"http://cdn-bj.fis.plus/EA34BFF115404DE1BDA789001ECC1249.jpg\",\"AssociatedSystemPosition\":\"MainMenu\", \"AgentAssessmentSystemIcon\":\"http://cdn-bj.fis.plus/EA34BFF115404DE1BDA789001ECC1249.jpg\", \"AgentAssessmentSystemPosition\":\"PersonCenter\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultRtc",
                ConfigKey = "Rtc",
                ConfigContent = "{\"RTCType\":\"TRTC\",\"RTCAppId\":\"\",\"RTCAppKey\":\"\",\"RTCBizid\":0,\"RTCPlayUrl\":\"\",\"RTCPushKey\":\"\",\"RTCPushUrl\":\"\",\"RTCStartingRoomId\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
            entity = new SystemConfigEntity()
            {
                Code = "SystemConfigDefaultWeChat",
                ConfigKey = "WeChat",
                ConfigContent = "{\"GatewayHost\":\"\",\"AppId\":\"\",\"ServerIP\":\"\",\"MchId\":\"\",\"Key\":\"\",\"NotifyUrl\":\"\",\"PayShortBaseUrl\":\"\",\"Enable\":\"\"}",
                ConfigPageControls = ""
            };
            r = await _SystemConfigDBRepository.InsertOneForAsyncDBAsync(entity);
        }

        /// <summary>
        /// 插入默认报告模版
        /// </summary>
        /// <returns></returns>
        private async Task InsertDefaultReportTemplate()
        {
            var storageFold = "";
            var vcsStorageUrlPrefix = "";
            if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
            {
                var secondDirectory = "";
                var tempServerUrl = "";
                if (!string.IsNullOrEmpty(_serverUrl))
                {
                    tempServerUrl = _serverUrl.TrimEnd('/');
                    if (tempServerUrl.Split('/').Length > 0)
                    {
                        var arrayList = tempServerUrl.Split('/');
                        secondDirectory = arrayList[(arrayList.Length - 1)];
                    }
                }
                var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                if (!Directory.Exists(storageFold))
                {
                    Directory.CreateDirectory(storageFold);
                }
                vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
            }
            var defaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";
            var animalDefaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f";
            reportTemplate.ReportTemplateName = "默认中文模版(人用)";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Default.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";

            MongoCustomerFilter<MongoTable> filter1 = new MongoCustomerFilter<MongoTable>();
            filter1.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "2e1441e904c2e2cbea7ba7d692e5e98f"));
            if (!await _reportTemplateDBRepository.Exists(filter1.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Default");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/0512202F156C46E4A5887B661D41A860.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/E97C650122CE4D6A8AE57470B640EA3B.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "3E9E0DBDAF3E4C65933D3EF37AF52470";
            reportTemplate.ReportTemplateName = "По умолчанию(для людей)";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Default_ru.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "ru-RU";

            MongoCustomerFilter<MongoTable> filterDefaultRu = new MongoCustomerFilter<MongoTable>();
            filterDefaultRu.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "3E9E0DBDAF3E4C65933D3EF37AF52470"));
            if (!await _reportTemplateDBRepository.Exists(filterDefaultRu.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "умолчанию_Preview_RU.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "умолчанию_Thumbnail.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ru-RU"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }


            reportTemplate.Code = "6f90e258b31f447dabde67f2160d70925";
            reportTemplate.ReportTemplateName = "Животные";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_ru.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            reportTemplate.LanguageCode = "ru-RU";

            MongoCustomerFilter<MongoTable> filterPetRu = new MongoCustomerFilter<MongoTable>();
            filterPetRu.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "6f90e258b31f447dabde67f2160d70925"));
            if (!await _reportTemplateDBRepository.Exists(filterPetRu.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_Preview_RU.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_ru_Thumbnail.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ru-RU"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "03DED5E023C5419696F34CB1DF9F53FC";
            reportTemplate.ReportTemplateName = "ИИ(русский)";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI_ru.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "ru-RU";

            MongoCustomerFilter<MongoTable> filterAI = new MongoCustomerFilter<MongoTable>();
            filterAI.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "03DED5E023C5419696F34CB1DF9F53FC"));
            if (!await _reportTemplateDBRepository.Exists(filterAI.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "ИИ_Preview_RU.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "ИИ_Thumbnail.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ru-RU"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "372bd4c55ae9ceff5291e7c0d91ffaca";
            reportTemplate.ReportTemplateName = "默认中文模版(兽用)";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet.json"));
            reportTemplate.ReportDatasJson = animalDefaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            reportTemplate.LanguageCode = "zh-CN";

            MongoCustomerFilter<MongoTable> filter2 = new MongoCustomerFilter<MongoTable>();
            filter2.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "372bd4c55ae9ceff5291e7c0d91ffaca"));
            if (!await _reportTemplateDBRepository.Exists(filter2.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Pet");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/4B0F60E385D146A2B50164D30891B9C0.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/92570B8D59B741C1A78E457A6271999A.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/DDC5FBF14819451FA4F3A0DF4F902768.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/FA5D6AFFF91A4CCFB288BD67FB250458.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "2017965c544811964710ebe6562700bf";
            reportTemplate.ReportTemplateName = "DefaultEnglishTemplate(human)";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Default_en.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter3 = new MongoCustomerFilter<MongoTable>();
            filter3.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "2017965c544811964710ebe6562700bf"));
            if (!await _reportTemplateDBRepository.Exists(filter3.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Default_en");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/CC8B8ACD842945719C6E617EFB801B42.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/75033FC2AEE247C0A3AF756FB339CAA8.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/1059971273674ABD910719A25D1009B5.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/C7F2226E35C74B3AA04316E96D6C1F50.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "3927dfbc00dcc5e60f378cb7666ddb21";
            reportTemplate.ReportTemplateName = "DefaultEnglishTemplate(animal)";
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_en.json"));
            reportTemplate.ReportDatasJson = animalDefaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter4 = new MongoCustomerFilter<MongoTable>();
            filter4.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "3927dfbc00dcc5e60f378cb7666ddb21"));
            if (!await _reportTemplateDBRepository.Exists(filter4.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Pet_en");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/DD637D7BFA7F48AEA5387AB0393C74E5.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/94C0E89779A74480BE8452F587BDCBB5.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/EB5161DDEE104D7D999035DAF09EF3C1.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/A2A9C879C25E43BDA366D19EA8C8E2C5.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "e9d763614fd6a01cf35653c0cdae3d51";
            reportTemplate.ReportTemplateName = "AI中文";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter5 = new MongoCustomerFilter<MongoTable>();
            filter5.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "e9d763614fd6a01cf35653c0cdae3d51"));
            if (!await _reportTemplateDBRepository.Exists(filter5.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "AI");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/A79DAD84D03246119AA59D4476677D87.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/D5747F04B6D243D8A9EB45ABE8446C3F.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/3E1DB516859A4AC4B607E24A12C51E61.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/592C2352CCDF43349169D7AA7D7724F8.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "c21a2ff88dfc51859918ad0c085c653a";
            reportTemplate.ReportTemplateName = "AIEnglish";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI_en.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter6 = new MongoCustomerFilter<MongoTable>();
            filter6.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "c21a2ff88dfc51859918ad0c085c653a"));
            if (!await _reportTemplateDBRepository.Exists(filter6.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "AI_en");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/A79DAD84D03246119AA59D4476677D87.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/D5747F04B6D243D8A9EB45ABE8446C3F.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/3E1DB516859A4AC4B607E24A12C51E61.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/592C2352CCDF43349169D7AA7D7724F8.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "540d93e2ad626e7da63e5a6dcbad9781";
            reportTemplate.ReportTemplateName = "ImageOnly";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "ImageOnly.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter7 = new MongoCustomerFilter<MongoTable>();
            filter7.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "540d93e2ad626e7da63e5a6dcbad9781"));
            if (!await _reportTemplateDBRepository.Exists(filter7.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "ImageOnly");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/39960BDA474A4F20B8F47CFCE52B0750.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/F7837318F2584A4BA8C672F12EE6D0C6.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/50ABA5BADD0146169F99E4906B09F417.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/C75697A0DE89454B99245C734B2AB7C1.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "3e1534b13211898c882dada62273413a";
            reportTemplate.ReportTemplateName = "Kidney";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Kidney.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter8 = new MongoCustomerFilter<MongoTable>();
            filter8.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "3e1534b13211898c882dada62273413a"));
            if (!await _reportTemplateDBRepository.Exists(filter8.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Kidney");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/CF85727CB74E4002862F1D4794373784.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/572D53509A7C44E3801373C322FDEC50.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/5A9C2D44CCE74FBCB37CF5D4B4C581FF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/33FB570905644F3B96A508BBCA78B7E6.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "7cd11cef5194f8f1dc3e977c0562d18c";
            reportTemplate.ReportTemplateName = "OB";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "OB.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter9 = new MongoCustomerFilter<MongoTable>();
            filter9.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "7cd11cef5194f8f1dc3e977c0562d18c"));
            if (!await _reportTemplateDBRepository.Exists(filter9.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "OB");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/E93EBBA948DF43A095E87B1BA2768DF1.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/570F5D93A1A7456CAE75CA30BAF008E5.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/17FD63EF35D04215B108CF057379C4D2.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/B46D68973D0D45E0A8AF055D803779E3.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }

            reportTemplate.Code = "aa3f47c3cddbd2fe529c13448ce9fbef";
            reportTemplate.ReportTemplateName = "Repro";
            reportTemplate.IsDefault = false;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Repro.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter10 = new MongoCustomerFilter<MongoTable>();
            filter10.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "aa3f47c3cddbd2fe529c13448ce9fbef"));
            if (!await _reportTemplateDBRepository.Exists(filter10.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "Repro", true);
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/71777237205142B98A10E62BABE8AC31.jpg;https://cdn-bj.fis.plus/AA49671BE12A4C348776C04F08FE6E47.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/EBA94BA2A0D8436B8AEEA886AB61C9AD.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/F2B5A3F2511E42F4B48950D90142B271.jpg;https://cdn-bj.fis.plus/AA08CA705B5F41E9AFB62E2118B086F1.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/8085E91080234BE7B474BE5CF6C07129.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 读取并上传图片
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        private async Task<Tuple<string, string, string, string>> UploadReportTemplatePreviewImage(string storageFold, string vcsStorageUrlPrefix, string fileName, bool isPaged = false)
        {
            var fileCNPreviewUrl = "";
            var fileENPreviewUrl = "";
            var fileCNThumbnailUrl = "";
            var fileENThumbnailUrl = "";

            var fileCNPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview.jpg");
            var fileENPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview_EN.jpg");
            var fileCNThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Thumbnail.jpg");
            var fileENThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Thumbnail_EN.jpg");

            var fileCNPreviewUrl2 = "";
            var fileENPreviewUrl2 = "";
            var fileCNPreviewPath2 = "";
            var fileENPreviewPath2 = "";
            if (isPaged)
            {
                fileCNPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview_1.jpg");
                fileENPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview_1_EN.jpg");
                fileCNPreviewPath2 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview_2.jpg");
                fileENPreviewPath2 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", fileName + "_Preview_2_EN.jpg");
            }
            var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
            if (File.Exists(fileCNPreviewPath))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileCNPreviewPath);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileCNPreviewPath);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileCNPreviewPath, waitCopyFilePath, true);
                    fileCNPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileCNPreviewPath, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileCNPreviewPath} Fail");
                    }
                    fileCNPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileCNPreviewUrl))
                    {
                        fileCNPreviewUrl = fileCNPreviewUrl.ToUrlToken();
                    }
                }
            }
            if (File.Exists(fileENPreviewPath))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileENPreviewPath);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileENPreviewPath);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileENPreviewPath, waitCopyFilePath, true);
                    fileENPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileENPreviewPath, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileENPreviewPath} Fail");
                    }
                    fileENPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileENPreviewUrl))
                    {
                        fileENPreviewUrl = fileENPreviewUrl.ToUrlToken();
                    }
                }
            }
            if (File.Exists(fileCNThumbnailPath))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileCNThumbnailPath);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileCNThumbnailPath);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileCNThumbnailPath, waitCopyFilePath, true);
                    fileCNThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileCNThumbnailPath, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileCNThumbnailPath} Fail");
                    }
                    fileCNThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileCNThumbnailUrl))
                    {
                        fileCNThumbnailUrl = fileCNThumbnailUrl.ToUrlToken();
                    }
                }
            }
            if (File.Exists(fileENThumbnailPath))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileENThumbnailPath);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileENThumbnailPath);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileENThumbnailPath, waitCopyFilePath, true);
                    fileENThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileENThumbnailPath, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileENThumbnailPath} Fail");
                    }
                    fileENThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileENThumbnailUrl))
                    {
                        fileENThumbnailUrl = fileENThumbnailUrl.ToUrlToken();
                    }
                }
            }
            if (File.Exists(fileCNPreviewPath2))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileCNPreviewPath2);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileCNPreviewPath2);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileCNPreviewPath2, waitCopyFilePath, true);
                    fileCNPreviewUrl2 = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileCNPreviewPath2, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileCNPreviewPath2} Fail");
                    }
                    fileCNPreviewUrl2 = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileCNPreviewUrl2))
                    {
                        fileCNPreviewUrl2 = fileCNPreviewUrl2.ToUrlToken();
                    }
                }
            }
            if (File.Exists(fileENPreviewPath2))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    string oriFilename = Path.GetFileNameWithoutExtension(fileENPreviewPath2);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileENPreviewPath2);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileENPreviewPath2, waitCopyFilePath, true);
                    fileENPreviewUrl2 = vcsStorageUrlPrefix + (factName + factNameExtension);
                }
                else
                {
                    var fileList = _fileTransfer.FileTransferAsync(fileENPreviewPath2, false).GetAwaiter().GetResult();
                    if (fileList == null || fileList.Count <= 0)
                    {
                        Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileENPreviewPath2} Fail");
                    }
                    fileENPreviewUrl2 = fileList.FirstOrDefault() ?? string.Empty;
                    if (!string.IsNullOrEmpty(fileENPreviewUrl2))
                    {
                        fileENPreviewUrl2 = fileENPreviewUrl2.ToUrlToken();
                    }
                }
            }
            if (isPaged)
            {
                var previewUrl1 = fileCNPreviewUrl + ";" + fileCNPreviewUrl2;
                var previewUrl2 = fileENPreviewUrl + ";" + fileENPreviewUrl2;
                return Tuple.Create<string, string, string, string>(previewUrl1, fileCNThumbnailUrl, previewUrl2, fileENThumbnailUrl);
            }
            else
            {
                return Tuple.Create<string, string, string, string>(fileCNPreviewUrl, fileCNThumbnailUrl, fileENPreviewUrl, fileENThumbnailUrl);
            }
        }

        /// <summary>
        /// 查找等保配置
        /// </summary>
        /// <param name="moduleCode">Code</param>
        /// <param name="name">配置项名称</param>
        /// <returns></returns>
        public async Task<string> FindGradingProtectionConfigAsync(string moduleCode, string name)
        {
            var filter = Builders<GradingProtectionConfigEntity>.Filter.Where(u => u.Code == moduleCode && u.Name == name && u.IsDelete == false);
            var result = await _gradingProtectionConfigDBRepository.FindOneAsync(filter);
            return result.Value;
        }

        /// <summary>
        /// 创建索引
        /// </summary>
        /// <returns></returns>
        private void InitCollectionIndex()
        {
            _userDBRepository.CreateIndex();
            _organizationDBRepository.CreateIndex();
            _patientInfoDBRepository.CreateIndex();
            _recordInfoDBRepository.CreateIndex();
            _tokensDBRepository.CreateIndex();
            _deviceInfoDBRepository.CreateIndex();
            _remedicalMeasuredInfoDBRepository.CreateIndex();
            _reportDBRepository.CreateIndex();
            _scheduleDBRepository.CreateIndex();
            _remedicalDBRepository.CreateIndex();
            _messageInfoDBRepository.CreateIndex();
            _chatMessageDBRepository.CreateIndex();
            _referralDataDBRepository.CreateIndex();
        }

        /// <summary>
        /// 统计配置数据
        /// </summary>
        /// <param name="name"></param>
        /// <param name="description"></param>
        /// <param name="intervalSetting"></param>
        /// <param name="handleClass"></param>
        /// <returns></returns>
        private async Task InsertCMSStatisticAsync(string name, string description, string intervalSetting, string handleClass)
        {
            var entity = new StatisticDO();
            entity.Name = name;
            entity.Description = description;
            entity.IntervalSetting = intervalSetting;
            entity.HandleClass = handleClass;
            entity.Enabled = true;
            entity.StatisticsPlatform = StatisticsPlatformEnum.CMS;
            await _statisticDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
        }
        private async Task InsertAdminStatisticAsync(string name, string description, string intervalSetting, string handleClass)
        {
            var entity = new AdminStatisticDO();
            entity.Name = name;
            entity.Description = description;
            entity.IntervalSetting = intervalSetting;
            entity.HandleClass = handleClass;
            entity.Enabled = true;
            await _adminStatisticDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
        }

        private async Task AddThesaurusDO(string thesaurusName, string fileName, string languageCode, ThesaurusUsageTypeEnum thesaurusUsageTypeEnum, string thesaurusCode, ThesaurusBusinessTypeEnum thesaurusBusinessType = ThesaurusBusinessTypeEnum.Ultrasonic)
        {
            var thesaurusDO = new ThesaurusDO();
            thesaurusDO.Code = thesaurusCode;
            thesaurusDO.IsDefault = true;
            thesaurusDO.ThesaurusName = thesaurusName;
            thesaurusDO.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
            thesaurusDO.ThesaurusUsageType = thesaurusUsageTypeEnum;
            thesaurusDO.OrganizationCode = "";
            thesaurusDO.PersonalCode = "";
            thesaurusDO.LanguageCode = languageCode;
            thesaurusDO.ThesaurusBusinessType = thesaurusBusinessType;
            thesaurusDO.ThesaurusItems = new List<ThesaurusItemDO>();
            var json = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Thesaurus", fileName));
            var thesaurusDataList = ConvertHelper.JsonToObj<List<ThesaurusInfoDTO>>(json);
            foreach (var item in thesaurusDataList)
            {
                var itemDO = new ThesaurusItemDO();
                itemDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                itemDO.ThesaurusItemType = ThesaurusItemTypeEnum.Classification;
                itemDO.ThesaurusItemName = item.Name;
                itemDO.ThesaurusItemDescription = "";
                itemDO.ThesaurusItemConclusion = "";
                itemDO.ParentItemCode = "";
                thesaurusDO.ThesaurusItems.Add(itemDO);
                foreach (var application in item.Applications)
                {
                    var applicationDO = new ThesaurusItemDO();
                    applicationDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                    applicationDO.ThesaurusItemType = ThesaurusItemTypeEnum.Classification;
                    applicationDO.ThesaurusItemName = application.Name;
                    applicationDO.ThesaurusItemDescription = "";
                    applicationDO.ThesaurusItemConclusion = "";
                    applicationDO.ParentItemCode = itemDO.ThesaurusItemCode;
                    thesaurusDO.ThesaurusItems.Add(applicationDO);
                    foreach (var applicationItem in application.Items)
                    {
                        var applicationItemDO = new ThesaurusItemDO();
                        applicationItemDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                        applicationItemDO.ThesaurusItemType = ThesaurusItemTypeEnum.Contents;
                        applicationItemDO.ThesaurusItemName = "";
                        applicationItemDO.ParentItemCode = applicationDO.ThesaurusItemCode;
                        applicationItemDO.ThesaurusItemDescription = applicationItem.Description;
                        applicationItemDO.ThesaurusItemConclusion = applicationItem.Summary;
                        thesaurusDO.ThesaurusItems.Add(applicationItemDO);
                    }
                }
            }
            await _thesaurusDBRepository.InsertOneForAsyncDBAsync(thesaurusDO, new Dictionary<string, string>(), string.Empty, false);
        }

        /// <summary>
        /// 日志索引
        /// </summary>
        /// <returns></returns>
        private async Task LogIndexes()
        {
            var db = MongoRepertoryCenter.Instance.GetDatabase(_logDBName);
            var collection = db.GetCollection<LogEntry>(_collectionName);
            //创建索引
            var indexs = await collection.Indexes.ListAsync();
            var indexsList = await indexs.ToListAsync();
            var indexsStringList = indexsList.Select(x => x.ToString());
            if (!indexsStringList.Any(x => x.Contains("Timestamp")))
            {
                collection.Indexes.CreateOne(new CreateIndexModel<LogEntry>(Builders<LogEntry>.IndexKeys.Descending(t => t.Timestamp)));
            }
            if (!indexsStringList.Any(x => x.Contains("Level")))
            {
                collection.Indexes.CreateOne(new CreateIndexModel<LogEntry>(Builders<LogEntry>.IndexKeys.Ascending(t => t.Level)));
            }
            if (!indexsStringList.Any(x => x.Contains("RenderedMessage")))
            {
                collection.Indexes.CreateOne(new CreateIndexModel<LogEntry>(Builders<LogEntry>.IndexKeys.Ascending(t => t.RenderedMessage)));
            }
        }

        /// <summary>
        /// 生成分页查询条件
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        internal DBPageRequest GetDBPageRequest(PageFilterRequest request)
        {
            var customerFilter = new MongoCustomerFilter<MongoTable>();
            var mongoDataItemFilter = new MongoCustomerFilter<MongoDataItem>();
            foreach (var para in request.Filter)
            {
                if (request.IsFuzzy)
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key));
                    mongoDataItemFilter.AppendRegex(item => item.Value, para.Value, request.IsFuzzy);
                }
                else
                {
                    mongoDataItemFilter.AppendEq((item => item.Key, para.Key), (item => item.Value, para.Value));
                }
                customerFilter.SubFilter(item => item.QueryData.QueryItems, mongoDataItemFilter.AndCombine());
                mongoDataItemFilter.ClearFilter();
            }
            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var dbPageRequest = new DBPageRequest();
            dbPageRequest.CurrentPage = request.CurrentPage;
            dbPageRequest.PageSize = request.PageSize;
            dbPageRequest.Sort = sort;
            dbPageRequest.Filter = customerFilter.AndCombine();
            return dbPageRequest;
        }

        /// <summary>
        /// 分页数据转换
        /// </summary>
        /// <param name="pageResult"></param>
        /// <typeparam name="T"></typeparam>
        /// <typeparam name="F"></typeparam>
        /// <returns></returns>
        internal PageResult<T> PageResultConvert<S, T>(PageCollection<S> pageResult)
        {
            PageResult<T> convertResult = new();
            convertResult.PageIndex = pageResult.CurrentPage;
            convertResult.PageSize = pageResult.PageSize;
            convertResult.TotalCount = pageResult.DataCount;
            convertResult.PageData = pageResult.PageData.Select(f => f.MappingTo<T>()).ToList();
            return convertResult;
        }

        /// <summary>
        /// 分页数据转换
        /// </summary>
        /// <param name="pageResult"></param>
        /// <typeparam name="S"></typeparam>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        internal PageCollection<T> PageCollectionConvert<S, T>(PageCollection<S> pageResult)
        {
            PageCollection<T> convertResult = new();
            convertResult.CurrentPage = pageResult.CurrentPage;
            convertResult.PageSize = pageResult.PageSize;
            convertResult.DataCount = pageResult.DataCount;
            convertResult.PageData = pageResult.PageData.Select(f => f.MappingTo<T>()).ToList();
            return convertResult;
        }

        /// <summary>
        /// 归档删除冷数据
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        private bool OnArchiveDeletion(string key)
        {
            try
            {
                Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion starting, key:{key}");
                var minDate = DateTime.UtcNow.Date.AddYears(-1);
                {
                    //分时诊断
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    customerFilter.AppendLte("BaseData.UpdateTime", minDate);
                    var entities = _recordInfoDBRepository.FindAllAsync(customerFilter.AndCombine()).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteRecordIds = entities.Select(f => ("Code", f.Code)).ToList();
                        var deleteRecordResult = _recordInfoDBRepository.RealDeleteAllAsync(deleteRecordIds).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle record successfully, totalCount:{entities.Count}");
                        //云端图像
                        var recordIds = entities.Select(x => x.Code).ToList();
                        MongoCustomerFilter<MongoTable> customerFilter_remedical = new();
                        customerFilter_remedical.AppendIn("BaseData.RecordCode", recordIds);
                        var remedicals = _remedicalDBRepository.FindAllAsync(customerFilter_remedical.AndCombine()).Result;
                        var deleteRemedicalIds = remedicals.Select(f => ("Code", f.Code)).ToList();
                        var deleteRemedicalResult = _remedicalDBRepository.RealDeleteAllAsync(deleteRemedicalIds).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle remedical successfully, totalCount:{remedicals.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle record end, filter entities empty.");
                    }
                }
                {
                    //实时会诊
                    var filter = Builders<ConsultationRecordEntity>.Filter.Where(c => c.UpdateTime <= minDate);
                    var entities = _consultationRecordDBRepository.FindAllAsync(filter).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteList = entities.Select(f => ("Code", f.Code)).ToList();
                        var result = _consultationRecordDBRepository.RealDeleteAllAsync(deleteList).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle consultation successfully, totalCount:{entities.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle consultation end, filter entities empty.");
                    }
                }
                {
                    //教培-在线培训
                    var filter = Builders<CourseInfoEntity>.Filter.Where(c => c.UpdateTime <= minDate);
                    var entities = _courseInfoDBRepository.FindAllAsync(filter).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteList = entities.Select(f => ("Code", f.Code)).ToList();
                        var result = _courseInfoDBRepository.RealDeleteAllAsync(deleteList).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle course successfully, totalCount:{entities.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle course end, filter entities empty.");
                    }
                }
                {
                    //教培-在线视频
                    var filter = Builders<VideoInfoEntity>.Filter.Where(c => c.UpdateTime <= minDate);
                    var entities = _videoInfoDBRepository.FindAllAsync(filter).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteList = entities.Select(f => ("Code", f.Code)).ToList();
                        var result = _videoInfoDBRepository.RealDeleteAllAsync(deleteList).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle video successfully, totalCount:{entities.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle video end, filter entities empty.");
                    }
                }
                {
                    //教培-教培考试
                    var filter = Builders<ExaminationPaperEntity>.Filter.Where(c => c.UpdateTime <= minDate);
                    var entities = _examinationPaperDBRepository.FindAllAsync(filter).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteList = entities.Select(f => ("Code", f.Code)).ToList();
                        var result = _examinationPaperDBRepository.RealDeleteAllAsync(deleteList).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle exam paper successfully, totalCount:{entities.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle exam paper end, filter entities empty.");
                    }
                }
                {
                    //教培-课程专辑
                    var filter = Builders<CourseAlbumEntity>.Filter.Where(c => c.UpdateTime <= minDate);
                    var entities = _courseAlbumDBRepository.FindAllAsync(filter).Result;
                    if (entities != null && entities.Any())
                    {
                        var deleteList = entities.Select(f => ("Code", f.Code)).ToList();
                        var result = _courseAlbumDBRepository.RealDeleteAllAsync(deleteList).Result;
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle course album successfully, totalCount:{entities.Count}");
                    }
                    else
                    {
                        Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion, handle course album end, filter entities empty.");
                    }
                }

                Logger.WriteLineInfo($"DatabaseService OnArchiveDeletion successfully, key:{key}");

                _archiveDeletionExecutor?.Add(OnArchiveDeletion, DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DatabaseService OnArchiveDeletion err, ex:{ex}");
            }
            return true;
        }

        private bool ArraysAreEqual<T>(List<T> array1, List<T> array2)
        {
            if (array1.Count != array2.Count)
            {
                return false;
            }

            for (int i = 0; i < array1.Count; i++)
            {
                if (!array1[i].Equals(array2[i]))
                {
                    return false;
                }
            }

            return true;
        }

        private bool GetIsCarotidOn()
        {
            var isCarotidOn = false;
            try
            {
                isCarotidOn = EnvironmentConfigManager.GetParammeter<BoolParameter>("AI", "IsCarotidOn").Value;

            }
            catch (Exception ex)
            {

            }
            return isCarotidOn;
        }

        private void WatchingMongoDbServer()
        {
            Task.Run(async () =>
            {
                while (true)
                {
                    await Task.Delay(60 * 1000);
                    var isWatchingDB = GetBoolParams("Database", "IsWatchingDB");
                    if (isWatchingDB)
                    {
                        RestartLocalMongoDbServer();
                    }
                }
            });
        }

        private static bool GetBoolParams(string node, string item)
        {
            try
            {
                return EnvironmentConfigManager.GetParammeter<BoolParameter>(node, item).Value;
            }
            catch
            {
                return false;
            }
        }

        private static string GetStringParams(string node, string item)
        {
            try
            {
                return EnvironmentConfigManager.GetParammeter<StringParameter>(node, item).Value;
            }
            catch
            {
                return "";
            }
        }

        private void RestartLocalMongoDbServer()
        {
            var serverIsRunning = MongoRepertoryCenter.Instance.MongodServerIsRunning();
            if (serverIsRunning)
            {
                return;
            }
            try
            {
                Console.WriteLine("restart Mongodb server.");
                StartDB();
                Console.WriteLine("Mongodb server restarted.");
            }
            catch (Exception exception)
            {
                Console.WriteLine($"Start mongod server process filed,error:{exception}");
            }
        }



        private void StartDB()
        {
            var _dependencies = ConfigurationManager.GetParammeter<StringParameter>("Dependencies", "Apps").Value;
            if (!string.IsNullOrWhiteSpace(_dependencies))
            {
                var dependenciesApps = _dependencies.Split(',');
                foreach (var item in dependenciesApps)
                {
                    if (!string.IsNullOrEmpty(item))
                    {
                        if (!item.StartsWith("startmongodb"))
                        {
                            continue;
                        }
                        var appItems = item.Split('|');
                        var appItem = "";//application 
                        var paramsValue = "";
                        if (appItems.Length > 0)
                        {
                            appItem = appItems[0];
                            for (int i = 1; i < appItems.Length; i++)
                            {
                                paramsValue += " " + appItems[i];
                            }
                        }
                        if (string.IsNullOrEmpty(appItem))
                        {
                            throw new Exception(" load appsetting:Dependencies.Apps is null");
                        }
                        var directory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Packages");
                        var appPath = Path.Combine(directory, appItem);
                        if (!File.Exists(appPath))
                        {
                            string[] files = System.IO.Directory.GetFiles(directory, "", System.IO.SearchOption.TopDirectoryOnly);
                            appPath = files.FirstOrDefault(v => v.Contains(appItem));
                        }
                        Process process = new Process();
                        process.StartInfo.WorkingDirectory = AppDomain.CurrentDomain.BaseDirectory;
                        process.StartInfo.FileName = appPath;
                        process.StartInfo.Arguments = paramsValue;
                        process.StartInfo.RedirectStandardOutput = true; //重定向输出，
                        process.StartInfo.RedirectStandardError = true;
                        process.StartInfo.UseShellExecute = false;
                        process.Start();
                    }
                }
            }
        }

        private async Task ResearchEditionServiceAuth()
        {
            try
            {
                var currentSoftwareLicenseInfoJson = EnvironmentConfigManager.GetParammeter<StringParameter>("Services", "CurrentSoftwareLicenseInfo").Value;
                LicenseInfoDTO licenseInfoDTO = new LicenseInfoDTO();
                if (!string.IsNullOrEmpty(currentSoftwareLicenseInfoJson))
                {
                    licenseInfoDTO = JsonConvert.DeserializeObject<LicenseInfoDTO>(currentSoftwareLicenseInfoJson);
                }
                if (!string.IsNullOrEmpty(licenseInfoDTO?.AuthorityInfos?.Trim()))
                {
                    var dongleAuth = licenseInfoDTO.AuthorityInfos.Split('丨')?.ToList();
                    if (dongleAuth.Count > 0)
                    {
                        isContainURMAuth = dongleAuth.Exists(c => c == "URMAuthority");
                        isContainVETAuth = dongleAuth.Exists(c => c == "VETAuthority");
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn(ex.ToString());
            }
            if (isContainVETAuth)
            {
                var builder = Builders<ResearchProjectEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == "BaseVetProject");
                var existEntity = await _researchProjectDBRepository.Exists(filter);
                if (!existEntity)
                {
                    var entity = new ResearchProjectEntity();
                    entity.Code = "BaseVetProject";
                    entity.CreateTime = DateTime.UtcNow;
                    entity.UpdateTime = DateTime.UtcNow;
                    entity.OperateTime = DateTime.UtcNow;
                    entity.OperatorName = "系统";
                    entity.CreatorName = "系统";
                    entity.PatientType = OrganizationPatientTypeEnum.Animals;
                    entity.Name = "系统默认";
                    entity.Introduction = "系统默认，用于转移数据";
                    await _researchProjectDBRepository.InsertOneAsync(entity);

                }
            }
        }
    }
}