using System.Data.Common;
using System.IO;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.DTO.Device;
using WingServerCommon.Service;
using WingInterfaceLibrary.DTO.Dictionary;
using WingMongoDBModule.Common;
using JsonRpcLite.Services;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingServerCommon.Mapper;
using WingMongoDBModule.DB.DO;
using System.Linq;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.DTO.Management;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using MongoDB.Driver;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request;
using WingMongoDBModule.Request;
using WingServerCommon.Utilities;
using WingServerCommon.Interfaces.Cache;
using WingMongoDBModule.Entities;
using MongoDB.Bson;
using WingInterfaceLibrary.Request.DBRequest;
using WingServerCommon.Log;
using Newtonsoft.Json;
using WingInterfaceLibrary.Request.Device;
using MongoDB.Driver.GeoJsonObjectModel;
using WingInterfaceLibrary.Request.Education;
using MongoDB.Driver.GeoJsonObjectModel.Serializers;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.Request.IOTDevice;
using WingInterfaceLibrary.Result.IOTDevice;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Request.Comment;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IDeviceInfoDBService
    {

        /// <summary>
        /// > 函数接收请求对象，映射到数据对象，然后调用repository将数据对象插入数据库
        /// </summary>
        /// <param name="CreateDictionaryItemDBRequest">这是传递给服务的请求对象。</param>
        /// <returns>
        /// 返回值是一个字符串。
        /// </returns>
        public async Task<string> CreateDictionaryItemAsync(CreateDictionaryItemDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CreateDictionaryItemAsync execute", $"{this.GetType().Name}"))
            {
                var dictionaryDO = request.Data.MappingTo<DictionaryDO>();
                return await _dictionaryDBRepository.InsertOneAsync(dictionaryDO, request.PlatformDatas, request.ExtensionData);
            }
        }

        /// <summary>
        /// > 通过代码更新字典项
        /// </summary>
        /// <param name="DictionaryDTO">用于在客户端和服务器之间传输数据的数据传输对象。</param>
        /// <param name="extensionData">需要更新的数据，JSON格式的字符串。</param>
        /// <param name="canNull">是否允许更新空值</param>
        /// <returns>
        /// 返回值是一个布尔值。
        /// </returns>
        public async Task<bool> UpdateDictionaryItemAsync(DictionaryDTO dictionaryDTO, string extensionData, bool canNull = false)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDictionaryItemAsync execute", $"{this.GetType().Name}"))
            {
                var dictionaryDO = dictionaryDTO.MappingTo<DictionaryDO>();
                return await _dictionaryDBRepository.UpdateOneAsync("Code", dictionaryDO.Code, dictionaryDO, extensionData, canNull);
            }
        }

        /// <summary>
        /// 查询设备类型（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<DictionaryDTO>> FindDictionaryPagesAsync(GetDeviceTypePagesRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDictionaryPagesAsync execute", $"{this.GetType().Name}"))
            {
                PageResult<DictionaryDTO> result = new PageResult<DictionaryDTO>();
                result.PageIndex = request.PageIndex;
                result.PageSize = request.PageSize;
                if (request.DictionaryType == DictionaryTypeEnum.DeviceType)
                {
                    //查询language
                    var queryDic = new Dictionary<string, string>();
                    queryDic.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceType).ToString());
                    var dictionaryDatas = await _dictionaryDBRepository.FindListAsync(queryDic) ?? new List<DictionaryDO>();
                    if (dictionaryDatas?.Count > 0)
                    {
                        dictionaryDatas = dictionaryDatas.OrderByDescending(c => c.CreateTime).ToList();
                        if (!string.IsNullOrWhiteSpace(request.Keyword))
                        {
                            var filterDatas = dictionaryDatas.Where(dicData =>
                            {
                                if (dicData.LanguageConfigs == null || !dicData.LanguageConfigs.Any())
                                {
                                    return dicData.Value.Contains(request.Keyword, StringComparison.OrdinalIgnoreCase);
                                }
                                else
                                {
                                    return dicData.LanguageConfigs.Any(x => x.Value.Contains(request.Keyword, StringComparison.OrdinalIgnoreCase));
                                }
                            })?.ToList() ?? new();
                            var pageData = filterDatas.Skip((request.PageIndex - 1) * request.PageSize).Take(request.PageSize)?.ToList() ?? new List<DictionaryDO>();
                            var dicDTOList = pageData.MappingTo<List<DictionaryDTO>>();
                            result.TotalCount = filterDatas.Count;
                            result.PageData = dicDTOList;
                        }
                        else
                        {
                            var pageData = dictionaryDatas.Skip((request.PageIndex - 1) * request.PageSize).Take(request.PageSize)?.ToList() ?? new List<DictionaryDO>();
                            var dicDTOList = pageData.MappingTo<List<DictionaryDTO>>();
                            result.TotalCount = dictionaryDatas.Count;
                            result.PageData = dicDTOList;
                        }
                    }
                }
                else
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

                    queryItemsFilter.AppendEq((item => item.Key, "DictionaryType"));
                    queryItemsFilter.AppendEq((item => item.Value, ((int)DictionaryTypeEnum.DeviceModel).ToString()));
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    queryItemsFilter = new();
                    if (!string.IsNullOrWhiteSpace(request.Keyword))
                    {
                        queryItemsFilter.AppendEq((item => item.Key, "Value"));
                        queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                        customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    }
                    var filter = customerFilter.AndCombine();
                    result.TotalCount = (int)await _dictionaryDBRepository.CountDocumentsAsync(filter);
                    if (result.TotalCount > 0)
                    {
                        SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                        SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                        var dictionaryDoList = await _dictionaryDBRepository.FindAllAsync(filter, new FindOptions<MongoTable> { NoCursorTimeout = true, Skip = (request.PageIndex - 1) * request.PageSize, Limit = request.PageSize, Sort = sort });
                        var dicDTOList = dictionaryDoList.MappingTo<List<DictionaryDTO>>();
                        result.PageData = dicDTOList;
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 根据code查询字典
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<DictionaryDTO> FindDictionaryByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindDictionaryByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dictionaryDO = await _dictionaryDBRepository.FindOneAsync("Code", code);
                return dictionaryDO.MappingTo<DictionaryDTO>();
            }
        }

        /// <summary>
        /// 查询子字典
        /// </summary>
        /// <param name="parentCode"></param>
        /// <returns></returns>
        public async Task<List<DictionaryDTO>> FindDictionaryItemsByParentCodeAsync(string parentCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindDictionaryItemsByParentCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filter = new Dictionary<string, string>();
                filter.Add("ParentCode", parentCode);
                var deviceTypeDOList = await _dictionaryDBRepository.FindListAsync(filter);
                deviceTypeDOList = deviceTypeDOList.Where(t => t.ParentCode != null).ToList();
                return deviceTypeDOList.MappingTo<List<DictionaryDTO>>();
            }
        }

        /// <summary>
        /// 根据设备编码删除设备
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDeviceInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteDeviceInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                return await _deviceInfoDBRepository.DeleteOneAsync("Code", code);
            }
        }

        /// <summary>
        /// 根据设备编码查询设备
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<DeviceInfoDTO> FindDeviceInfoByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", code);
                return deviceInfoDO.MappingTo<DeviceInfoDTO>();
            }
        }

        /// <summary>
        /// 根据名称查询设备
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public async Task<DeviceInfoDTO> FindDeviceInfoByNameAsync(string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoByNameAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Name", name);
                return deviceInfoDO.MappingTo<DeviceInfoDTO>();
            }
        }

        /// <summary>
        /// 根据唯一码查询设备
        /// </summary>
        /// <param name="shortCode"></param>
        /// <returns></returns>
        public async Task<DeviceInfoDTO> FindDeviceInfoByShortCodeAsync(string shortCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoByShortCodeAsync execute", $"{this.GetType().Name}"))
            {
                // MongoCustomerFilter<MongoTable> filter = new();
                // MongoCustomerFilter<MongoDataItem> subfilter = new();
                // subfilter.AppendEq((item => item.Key, "ShortCode"));
                // subfilter.AppendRegex(item => item.Value, shortCode, false, false);
                // filter.SubFilter(item => item.QueryData.QueryItems, subfilter.AndCombine());
                // subfilter.ClearFilter();
                // var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync(filter.OrCombine());
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("ShortCode",shortCode);
                if (deviceInfoDO != null)
                {
                    return deviceInfoDO.MappingTo<DeviceInfoDTO>();
                }
                return null;
            }
        }

        /// <summary>
        /// 查找设备列表（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageCollection<DeviceInfoDTO>> FindDeviceInfoPagesAsync(PageFilterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoPagesAsync execute", $"{this.GetType().Name}"))
            {
                var dbPage = GetDBPageRequest(request);
                var deviceInfoDOPage = await _deviceInfoDBRepository.GetPages(dbPage);
                return deviceInfoDOPage.MappingTo<PageCollection<DeviceInfoDTO>>();
            }
        }

        /// <summary>
        /// 查询设备列表（分页）
        /// </summary>
        /// <param name="request">查询设备列表（分页）请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<ManageDeviceInfoDTO>> GetDevicePagesAsync(GetDevicePagesRequest queryRequest)
        {
            using (new Performance(_performanceThreshold, "{0} GetDevicePagesAsync execute", $"{this.GetType().Name}"))
            {
                var deviceTypeList = await FindDictionaryItemsAsync(new FindDictionaryItemsDBRequest { DictionaryType = (int)DictionaryTypeEnum.DeviceType });
                if (deviceTypeList?.Count > 0) 
                {
                    deviceTypeList = deviceTypeList.Where(c => c.DictionaryCode == "US" || c.DictionaryCode == "sonopost")?.ToList() ?? new List<DictionaryDTO>();
                }
                MongoCustomerFilter<MongoTable> customerFilter = await GetFilter(queryRequest, deviceTypeList);
                var dbRequest = new DBPageRequest();
                dbRequest.CurrentPage = queryRequest.PageIndex;
                dbRequest.PageSize = queryRequest.PageSize;
                dbRequest.Filter = customerFilter.AndCombine();
                var deviceListResult = await _deviceInfoDBRepository.GetPages(dbRequest);
                var organizations = new List<OrganizationDO>();
                var users = new List<UserInfoDO>();
                //var directorUsers = new List<UserDTO>();
                var organizationCodes = deviceListResult.PageData.Select(device => device.OrganizationCode).ToList();
                if (organizationCodes != null && organizationCodes.Count() > 0)
                {
                    organizations = FindOrganizationDOListByCodesAsync(organizationCodes).Result;
                    var userCodes = new List<string>();
                    foreach (var item in organizations.Select(o => o.Directors))
                    {
                        if (item != null && item.Count > 0)
                        {
                            userCodes.AddRange(item.Where(i => !string.IsNullOrWhiteSpace(i)));
                        }
                    }
                    users = FindUserDOListByUserCodesAsync(userCodes).Result;
                    var deviceCodes = deviceListResult.PageData.Select(d => d.Code).ToList();
                    var deviceShareUsers = GetUserDOsByDevicesAsync(deviceCodes).Result;
                    users.AddRange(deviceShareUsers.Where(du => !userCodes.Contains(du.UserCode)));
                    //users = FindUsers(new GetUsersDBRequest()).Result; //todo 非超级管理员需要加查询条件(机构)
                    // var directorUserCodes = organizations.Where(o => o.Directors != null && o.Directors.Count > 0 && !string.IsNullOrWhiteSpace(o.Directors[0])).Select(o => o.Directors[0]).ToList();
                    // if (directorUserCodes != null && directorUserCodes.Count > 0)
                    // {
                    //     directorUsers = FindUserListByUserCodes(directorUserCodes).Result;
                    // }
                }
                var departments = new List<OrganizationDO>();
                var departmentCodes = deviceListResult.PageData.Select(device => device.DepartmentCode).ToList();
                if (departmentCodes != null && departmentCodes.Count() > 0)
                {
                    departments = FindOrganizationDOListByCodesAsync(departmentCodes).Result;
                }
                var pageResult = PageResultConvert<DeviceInfoDO, ManageDeviceInfoDTO>(deviceListResult);
                foreach (var item in pageResult.PageData)
                {
                    var organization = organizations.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                    item.HospitalName = organization?.OrganizationName;
                    var department = departments.FirstOrDefault(o => o.OrganizationCode == item.DepartmentCode);
                    item.DepartmentName = department?.OrganizationName;
                    item.ShareUserNum = users.Count(u => u.BindDevices.Contains(item.DeviceCode)).ToString();
                    item.LanguageConfigs = deviceTypeList.FirstOrDefault(t => t.DictionaryCode == item.DeviceType)?.LanguageConfigs;
                    item.DeviceModelName = item.DeviceModel;
                    if (organization != null && organization.Directors != null && organization.Directors.Count > 0 && !string.IsNullOrWhiteSpace(organization.Directors[0]))
                    {
                        item.DirectorName = users.FirstOrDefault(u => u.UserCode == organization.Directors[0])?.FullName;
                    }
                }
                return pageResult;
            }
        }

        /// <summary>
        /// 查询设备列表（分页）条件拼接
        /// </summary>
        /// <param name="queryRequest"></param>
        /// <param name="deviceTypeList"></param>
        /// <param name="deviceModelList"></param>
        /// <returns></returns>
        private async Task<MongoCustomerFilter<MongoTable>> GetFilter(GetDevicePagesRequest queryRequest, List<DictionaryDTO> deviceTypeList)
        {
            List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

            //todo 超级管理员和普通管理员查询条件应该不一样
            if (!string.IsNullOrWhiteSpace(queryRequest.Keyword))
            {
                var items = new List<string> { "Name", "DeviceModel", "Description", "ShortCode", "DeviceType" };
                foreach (var item in items)
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, item));
                    queryItemsFilter.AppendRegex(item => item.Value, queryRequest.Keyword, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
                //根据关键字查询到机构
                var orgs = await GetOrganizationDOListByKeyword(queryRequest.Keyword);
                if (orgs.Count == 0)
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                    queryItemsFilter.AppendEq((item => item.Value, "-1"));
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
                else
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                    queryItemsFilter.AppendIn(item => item.Value, orgs.Select(x => x.Code).ToList());
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                }
            }
            if (!string.IsNullOrWhiteSpace(queryRequest.QueryType))
            {
                switch (queryRequest.QueryType)
                {
                    case "1":
                        var deviceTypeWhereList = deviceTypeList.Where(t => t.Value.Contains(queryRequest.Keyword));
                        if (deviceTypeWhereList != null && deviceTypeWhereList.Count() > 0)
                        {
                            queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                            queryItemsFilter.AppendIn(item => item.Value, deviceTypeWhereList.Select(t => t.DictionaryCode).ToList());
                        }
                        else
                        {
                            queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                        }
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                        break;
                    case "4":
                        var deviceTypeStrList = new List<string>() { "sonopost", "US" };
                        if (deviceTypeStrList != null && deviceTypeStrList.Count() > 0)
                        {
                            queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                            queryItemsFilter.AppendIn(item => item.Value, deviceTypeStrList);
                        }
                        else
                        {
                            queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                            queryItemsFilter.AppendEq((item => item.Value, "-1"));
                        }
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                        break;
                    default:
                        break;
                }
            }
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            if (queryItemsFilterList.Count > 0)
            {
                var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
            }
            if (queryRequest.Hospitals != null && queryRequest.Hospitals.Count > 0)
            {
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                queryItemsFilter.AppendIn(item => item.Value, queryRequest.Hospitals);
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            queryItemsFilter = new MongoCustomerFilter<MongoDataItem>();
            if (!string.IsNullOrWhiteSpace(queryRequest.UserName))
            {
                var filter = new Dictionary<string, string>();
                filter.Add("FullName", queryRequest.UserName);
                var userListResult = await FindUserPagesAsync(new PageFilterRequest
                {
                    CurrentPage = 1,
                    PageSize = 999999,
                    Filter = filter,
                    IsFuzzy = true
                });
                if (userListResult != null && userListResult.DataCount > 0)
                {
                    var userBindDevices = userListResult.PageData.Where(u => u.BindDevices != null && u.BindDevices.Count > 0)?.Select(u => u.BindDevices);
                    var deviceCodeList = new List<string>();
                    foreach (var item in userBindDevices)
                    {
                        deviceCodeList.AddRange(item.Where(i => !string.IsNullOrWhiteSpace(i)));
                    }
                    queryItemsFilter.AppendEq((item => item.Key, "Code"));
                    queryItemsFilter.AppendIn(item => item.Value, deviceCodeList);
                }
                else
                {
                    queryItemsFilter.AppendEq((item => item.Key, "Code"));
                    queryItemsFilter.AppendEq((item => item.Value, "-1"));
                }
            }
            if (!string.IsNullOrWhiteSpace(queryRequest.AdminName))
            {
                var adminList = await _adminDBRepository.FindAdminsByNameAsync(queryRequest.AdminName);
                if (adminList.Any())
                {
                    var organizationList = await _organizationDBRepository.FindOrganizationsAsync(adminList.Select(x => x.AdminCode).ToList());
                    queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                    queryItemsFilter.AppendIn(item => item.Value, organizationList.Select(x => x.OrganizationCode).ToList());
                }
                else
                {
                    queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                    queryItemsFilter.AppendEq((item => item.Value, "-1"));
                }
            }
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            return customerFilter;
        }

        /// <summary>
        /// 新增设备
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> InsertDeviceInfoAsync(CreateDeviceInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} InsertDeviceInfoAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = request.Data.MappingTo<DeviceInfoDO>();
                if (!string.IsNullOrEmpty(request.Data.HeadPicUrl))
                {
                    deviceInfoDO.HeadPicUrl = request.Data.HeadPicUrl.ToUrlToken();
                }
                deviceInfoDO.LastLoginTime = DateTime.UtcNow;
                deviceInfoDO.DeviceType = !string.IsNullOrWhiteSpace(deviceInfoDO.DeviceType) ? deviceInfoDO.DeviceType : "US";
                var result = await _deviceInfoDBRepository.InsertOneAsync(deviceInfoDO, request.PlatformDatas, request.ExtensionData);
                await AddPushDeviceAsync(deviceInfoDO);
                return result;
            }
        }

        /// <summary>
        /// 修改设备密码
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ModifyPasswordAsync(ModifyPasswordDBRequest request)
        {
            var deviceCode = request.DeviceCode;
            var password = request.Password;
            using (new Performance(_performanceThreshold, "{0} ModifyPasswordAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", deviceCode);
                deviceInfoDO.Password = password;
                var result = await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceCode, deviceInfoDO, "", false);
                CacheMaintenance.Instance.Get<IDeviceInfosManager>().Remove(deviceCode);
                return result;
            }
        }

        /// <summary>
        /// 删除字典项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveDictionaryItemAsync(RemoveDictionaryItemDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveDictionaryItemAsync execute", $"{this.GetType().Name}"))
            {
                return await _dictionaryDBRepository.DeleteOneAsync("Code", request.Code);
            }
        }

        /// <summary>
        /// 根据设备编码更新设备
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <param name="extensionData"></param>
        /// <param name="canNull"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDeviceInfoByCodeAsync(string code, DeviceInfoDTO data, string extensionData, bool canNull = false, bool isSync = false, bool isUpdateImg = true)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateDeviceInfoByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = data.MappingTo<DeviceInfoDO>();
                if (!string.IsNullOrEmpty(data.HeadPicUrl) && isUpdateImg)
                {
                    deviceInfoDO.HeadPicUrl = data.HeadPicUrl.ToUrlToken();
                }
                var oldDeviceInfo = await _deviceInfoDBRepository.FindOneAsync("Code", code);
                if (oldDeviceInfo.OrganizationCode != deviceInfoDO.OrganizationCode)
                {
                    var diagnosisModules = new List<string>();
                    if (!string.IsNullOrWhiteSpace(deviceInfoDO.OrganizationCode))
                    {
                        var organizationDiagnosisModules = await GetOrganizationDiagnosisModulesAsync(new GetOrganizationDiagnosisModulesDBRequest
                        {
                            OrganizationCode = deviceInfoDO.OrganizationCode
                        });
                        diagnosisModules = organizationDiagnosisModules.Where(x => x.Enabled)?.Select(x => x.DiagnosisModule)?.ToList() ?? new List<string>();
                    }
                    deviceInfoDO.DiagnosisModules = diagnosisModules;
                }
                var result = false;
                if (isSync)
                {
                    result = await _deviceInfoDBRepository.UpdateOneForSyncDBAsync("Code", code, deviceInfoDO, extensionData, canNull);
                }
                else
                {
                    result = await _deviceInfoDBRepository.UpdateOneAsync("Code", code, deviceInfoDO, extensionData, canNull);
                }
                if (oldDeviceInfo.OrganizationCode != deviceInfoDO.OrganizationCode)
                {
                    await AddPushDeviceAsync(deviceInfoDO);
                }
                CacheMaintenance.Instance.Get<IDeviceInfosManager>().Remove(code);
                return result;
            }
        }

        /// <summary>
        /// 根据类型查找字典项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<DictionaryDTO>> FindDictionaryItemsAsync(FindDictionaryItemsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDictionaryItemsAsync execute", $"{this.GetType().Name}"))
            {
                var paras = new Dictionary<string, string>();
                paras.Add("DictionaryType", request.DictionaryType.ToString());
                if (!string.IsNullOrWhiteSpace(request.ParentCode))
                {
                    paras.Add("ParentCode", request.ParentCode);
                }
                if (!string.IsNullOrWhiteSpace(request.Value))
                {
                    paras.Add("Value", request.Value);
                }
                var dictionaryDTOList = await _dictionaryDBRepository.FindListAsync(paras);
                if (dictionaryDTOList?.Count > 0)
                {
                    if (request.ShowDeviceCategory == 0)
                    {
                        dictionaryDTOList = dictionaryDTOList.Where(c => c.Code == "US" || c.Code == "sonopost" || c.ParentCode == "US" || c.ParentCode == "sonopost")?.ToList() ?? new List<DictionaryDO>();
                    }
                    else if (request.ShowDeviceCategory == 1)
                    {
                        dictionaryDTOList = dictionaryDTOList.Where(c => c.Code == "Breath" || c.Code == "Oxygen" || c.ParentCode == "Breath" || c.ParentCode == "Oxygen")?.ToList() ?? new List<DictionaryDO>();
                    }
                }
                return dictionaryDTOList.MappingTo<List<DictionaryDTO>>();
            }
        }

        /// <summary>
        /// 根据设备code列表获取设备信息列表
        /// </summary>
        /// <param name="request">移除设备请求实体</param>
        /// <returns></returns>
        public async Task<List<DeviceInfoDTO>> FindDeviceInfoListByCodeListAsync(List<string> codeList)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoListByCodeListAsync execute", $"{this.GetType().Name}"))
            {
                if (codeList == null || codeList.Count == 0)
                {
                    return new List<DeviceInfoDTO>();
                }
                var filter = new Dictionary<string, List<string>>();
                filter.Add("Code", codeList);
                //查询设备基础信息
                var dataList = await _deviceInfoDBRepository.FindInAsync(filter);
                return dataList.MappingTo<List<DeviceInfoDTO>>();
            }
        }

        /// <summary>
        /// 根据设备code列表获取设备信息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<ManageDeviceInfoDTO>> FindDevicesByPageDBAsync(FindDeviceInfoPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDevicesByPageDBAsync execute", $"{this.GetType().Name}"))
            {
                IList<string> deviceCodes = request.DeviceCodes;
                var docList = new List<BsonDocument>();
                var matchFilter = new List<BsonDocument>();

                if (deviceCodes?.Count > 0)
                {
                    var deviceCodesFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                        new BsonDocument("Key", "Code"),
                        new BsonDocument("Value", new BsonDocument("$in", new BsonArray(deviceCodes))),
                    }));
                    matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceCodesFilter));
                }
                if (request.OrganizationCodes?.Count > 0)
                {
                    var deviceCodesFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                        new BsonDocument("Key", "OrganizationCode"),
                        new BsonDocument("Value", new BsonDocument("$in", new BsonArray(request.OrganizationCodes))),
                    }));
                    matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceCodesFilter));
                }
                var isDeleteFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                    new BsonDocument("Key", "IsDelete"),
                    new BsonDocument("Value", "False"),
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", isDeleteFilter));

                if (!string.IsNullOrWhiteSpace(request.KeyWord?.Trim()))
                {
                    var pattern = ReplaceSpecialCharacter(request.KeyWord.Trim().ToLower());
                    BsonRegularExpression regularExp = new BsonRegularExpression($".*{pattern}.*", "ig");
                    var keywordFilter = new BsonDocument("$elemMatch",
                        new BsonDocument("$or", new BsonArray
                        {
                            new BsonDocument("$and", new BsonArray
                            {
                                new BsonDocument("Key", "Description"),
                                new BsonDocument("Value", regularExp),
                            }),
                            new BsonDocument("$and", new BsonArray
                            {
                                new BsonDocument("Key", "Name"),
                                new BsonDocument("Value", regularExp),
                            }),
                            new BsonDocument("$and", new BsonArray
                            {
                                new BsonDocument("Key", "ShortCode"),
                                new BsonDocument("Value", regularExp),
                            })
                        })
                    );
                    matchFilter.Add(new BsonDocument("QueryData.QueryItems", keywordFilter));
                }

                var matchFilterDoc = new BsonDocument("$and", new BsonArray(matchFilter));
                var matchDoc = new BsonDocument("$match", new BsonDocument(matchFilterDoc));
                docList.Add(matchDoc);
                var sortFilter = new List<KeyValuePair<string, object>>();
                sortFilter.Add(new KeyValuePair<string, object>("BaseData.CreateTime", -1));
                var sortDoc = new BsonDocument("$sort", new BsonDocument(sortFilter));
                docList.Add(sortDoc);

                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Pipeline = docList;
                var recordResult = await _deviceInfoDBRepository.GetAggregateTablePageAsync(dbPage);
                var dataList = PageResultConvert<DeviceInfoDO, ManageDeviceInfoDTO>(recordResult);
                if (dataList != null && dataList?.PageData.Count > 0)
                {
                    var organizations = new List<OrganizationDO>();
                    var users = new List<UserInfoDO>();
                    var organizationCodes = dataList.PageData.Select(device => device.OrganizationCode).ToList();
                    if (organizationCodes != null && organizationCodes.Count() > 0)
                    {
                        organizations = FindOrganizationDOListByCodesAsync(organizationCodes).Result;
                        var userCodes = new List<string>();
                        foreach (var item in organizations.Select(o => o.Directors))
                        {
                            if (item != null && item.Count > 0)
                            {
                                userCodes.AddRange(item.Where(i => !string.IsNullOrWhiteSpace(i)));
                            }
                        }
                        users = FindUserDOListByUserCodesAsync(userCodes).Result;
                    }
                    var deviceTypeList = await FindDictionaryItemsAsync(new FindDictionaryItemsDBRequest { DictionaryType = (int)DictionaryTypeEnum.DeviceType });
                    foreach (var item in dataList.PageData)
                    {
                        var organization = organizations.FirstOrDefault(o => o.OrganizationCode == item.OrganizationCode);
                        item.HospitalName = organization?.OrganizationName;
                        item.LanguageConfigs = deviceTypeList.FirstOrDefault(t => t.DictionaryCode == item.DeviceType)?.LanguageConfigs;
                        item.DeviceModelName = item.DeviceModel;
                        if (organization != null && organization.Directors != null && organization.Directors.Count > 0 && !string.IsNullOrWhiteSpace(organization.Directors[0]))
                        {
                            item.DirectorName = users.FirstOrDefault(u => u.UserCode == organization.Directors[0])?.FullName;
                        }
                    }
                }
                return dataList;
            }
        }

        /// <summary>
        /// 查询个人机构内设备列表分页列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<DeviceInfoDTO>> FindMyOrgDeviceInfoListPageDBAsync(FindDeviceInfoPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindMyOrgDeviceInfoListPageDBAsync execute", $"{this.GetType().Name}"))
            {
                var user = await FindUserByCodeAsync(request.UserCode);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                queryItemsFilter = new();
                queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                queryItemsFilter.AppendEq((item => item.Value, user.RootOrganizationCode));
                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }
                if (!user.IsDirector)
                {
                    queryItemsFilter = new();
                    if (user.BindDevices == null || user.BindDevices.Count == 0)
                    {
                        queryItemsFilter.AppendEq((item => item.Key, "Code"));
                        queryItemsFilter.AppendEq((item => item.Value, "-1"));
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                    else
                    {
                        queryItemsFilter.AppendEq((item => item.Key, "Code"));
                        queryItemsFilter.AppendIn(item => item.Value, user.BindDevices);
                        if (queryItemsFilter.HasFilter)
                        {
                            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                        }
                    }
                }
                dbPage.Filter = customerFilter.AndCombine();
                var result = await _deviceInfoDBRepository.GetPages(dbPage);
                return PageResultConvert<DeviceInfoDO, DeviceInfoDTO>(result);
            }

        }

        /// <summary>
        /// 查询个人设备列表分页列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<PageResult<DeviceInfoDTO>> FindDeviceInfoListPageDBAsync(FindDeviceInfoPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoListPageDBAsync execute", $"{this.GetType().Name}"))
            {
                var aggregateFilter = GetDeviceInfoPagesAggregateFilter(request);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Pipeline = aggregateFilter;
                var recordResult = await _deviceInfoDBRepository.GetAggregateTablePageAsync(dbPage);
                //Logger.WriteLineWarn($"DeviceInfoDBService FindDeviceInfoListPageDBAsync recordResult: {JsonConvert.SerializeObject(recordResult)}");
                return PageResultConvert<DeviceInfoDO, DeviceInfoDTO>(recordResult);
            }
        }

        /// <summary>
        /// 获取设备列表（分页）条件拼接
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private List<BsonDocument> GetDeviceInfoPagesAggregateFilter(FindDeviceInfoPageRequest request)
        {
            //Logger.WriteLineWarn($"DeviceInfoDBService GetDeviceInfoPagesAggregateFilter request: {JsonConvert.SerializeObject(request)}");
            bool isIncloudReferral = request.IsIncloudReferral;//是否包含转诊申请设备
            IList<string> isOnlineCodes = request.IsOnlineCodes;
            List<string> deviceCodes = request.DeviceCodes?.ToList();
            if (isIncloudReferral && !string.IsNullOrWhiteSpace(request.UserCode))
            {
                var userInfo = FindUserByCodeAsync(request.UserCode).Result;
                //转诊申请机构   机构负责人、普通员工
                List<ReferralDataEntity> referralDatas = new List<ReferralDataEntity>();
                if (userInfo.IsDirector)
                {
                    FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInOrganizationCode == userInfo.RootOrganizationCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                    referralDatas = _referralDataDBRepository.FindAllAsync(referralDataFliter).Result;
                }
                else
                {
                    FilterDefinition<ReferralDataEntity> referralDataFliter = Builders<ReferralDataEntity>.Filter.Where(x => x.ReferralInUserCode == userInfo.UserCode && x.ReferralStatus != RecordReferralStatusEnum.Withdrawn);
                    referralDatas = _referralDataDBRepository.FindAllAsync(referralDataFliter).Result;
                }
                if (referralDatas.Count > 0)
                {
                    var recordCodes = referralDatas.Select(x => x.RecordCode).ToList();
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                    queryItemsFilter.ClearFilter();
                    queryItemsFilter.AppendEq((item => item.Key, "Code"));
                    queryItemsFilter.AppendIn(item => item.Value, recordCodes);
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                    var records = _recordInfoDBRepository.FindAllAsync(customerFilter.AndCombine()).Result;
                    if (records.Count > 0)
                    {
                        var referralDeviceCodes = records.Where(x => !string.IsNullOrWhiteSpace(x.DeviceCode)).Select(x => x.DeviceCode);
                        deviceCodes.AddRange(referralDeviceCodes);
                    }
                }
            }
            IList<string> orgCodes = request.OrgCodes;
            IList<string> restrictOrgCodes = request.RestrictOrgCodes;
            var docList = new List<BsonDocument>();
            var matchFilter = new List<BsonDocument>();
            if (orgCodes?.Count > 0)
            {
                if (deviceCodes?.Count > 0)
                {
                    var orgCodesFilter = new BsonDocument("$elemMatch",
                        new BsonDocument("$or", new BsonArray
                        {
                            new BsonDocument("$and", new BsonArray
                            {
                                new BsonDocument("Key", "OrganizationCode"),
                                new BsonDocument("Value", new BsonDocument("$in", new BsonArray(orgCodes))),
                            }),
                            new BsonDocument("$and", new BsonArray
                            {
                                new BsonDocument("Key", "Code"),
                                new BsonDocument("Value", new BsonDocument("$in", new BsonArray(deviceCodes))),
                            })
                        })
                    );
                    matchFilter.Add(new BsonDocument("QueryData.QueryItems", orgCodesFilter));
                }
                else
                {
                    var deviceCodesFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                        new BsonDocument("Key", "OrganizationCode"),
                        new BsonDocument("Value", new BsonDocument("$in", new BsonArray(orgCodes))),
                    }));
                    matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceCodesFilter));
                }
            }
            else
            {
                var deviceCodesFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                    new BsonDocument("Key", "Code"),
                    new BsonDocument("Value", new BsonDocument("$in", new BsonArray(deviceCodes))),
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceCodesFilter));
            }
            var isDeleteFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                new BsonDocument("Key", "IsDelete"),
                new BsonDocument("Value", "False"),
            }));
            matchFilter.Add(new BsonDocument("QueryData.QueryItems", isDeleteFilter));
            if (request.OrganizationCodes?.Count > 0)
            {
                var organizationCodeFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                    new BsonDocument("Key", "OrganizationCode"),
                    new BsonDocument("Value", new BsonDocument("$in", new BsonArray(request.OrganizationCodes))),
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", organizationCodeFilter));
            }
            if (restrictOrgCodes?.Count > 0)
            {
                var restrictOrgCodesQuery = new BsonDocument("$elemMatch", new BsonDocument("$or", new BsonArray {
                    new BsonDocument("$and", new BsonArray
                    {
                        new BsonDocument("Key", "OrganizationCode"),
                        new BsonDocument("Value", new BsonDocument("$in", new BsonArray(restrictOrgCodes)))
                    })
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", restrictOrgCodesQuery));
            }
            if (!string.IsNullOrWhiteSpace(request.DeviceType))
            {
                var deviceTypeFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                    new BsonDocument("Key", "DeviceType"),
                    new BsonDocument("Value", request.DeviceType),
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceTypeFilter));
            }
            if (!string.IsNullOrWhiteSpace(request.DeviceModel))
            {
                var deviceModelFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                    new BsonDocument("Key", "DeviceModel"),
                    new BsonDocument("Value", request.DeviceModel),
                }));
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", deviceModelFilter));
            }
            if (!string.IsNullOrWhiteSpace(request.KeyWord?.Trim()))
            {
                var pattern = ReplaceSpecialCharacter(request.KeyWord.Trim().ToLower());
                BsonRegularExpression regularExp = new BsonRegularExpression($".*{pattern}.*", "ig");
                var keywordFilter = new BsonDocument("$elemMatch",
                    new BsonDocument("$or", new BsonArray
                    {
                        new BsonDocument("$and", new BsonArray
                        {
                            new BsonDocument("Key", "Description"),
                            new BsonDocument("Value", regularExp),
                        }),
                        new BsonDocument("$and", new BsonArray
                        {
                            new BsonDocument("Key", "Name"),
                            new BsonDocument("Value", regularExp),
                        }),
                        new BsonDocument("$and", new BsonArray
                        {
                            new BsonDocument("Key", "ShortCode"),
                            new BsonDocument("Value", regularExp),
                        })
                    })
                );
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", keywordFilter));
            }
            if (request.IsOnline != null && isOnlineCodes?.Count > 0)
            {
                BsonDocument isOnlineFilter;
                if (request.IsOnline.Value == true)
                {
                    isOnlineFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                        new BsonDocument("Key", "Code"),
                        new BsonDocument("Value", new BsonDocument("$in", new BsonArray(isOnlineCodes))),
                    }));
                }
                else
                {
                    isOnlineFilter = new BsonDocument("$elemMatch", new BsonDocument("$and", new BsonArray {
                        new BsonDocument("Key", "Code"),
                        new BsonDocument("Value", new BsonDocument("$nin", new BsonArray(isOnlineCodes))),
                    }));
                }
                matchFilter.Add(new BsonDocument("QueryData.QueryItems", isOnlineFilter));
            }
            var matchFilterDoc = new BsonDocument("$and", new BsonArray(matchFilter));
            var matchDoc = new BsonDocument("$match", new BsonDocument(matchFilterDoc));
            docList.Add(matchDoc);

            var sortFilter = new List<KeyValuePair<string, object>>();
            //定义一个项，让指定值显示最上面
            if (isOnlineCodes?.Count > 0)
            {
                var bDoc = new BsonDocument("isSpecifiedValue", new BsonDocument{
                    { "$cond", new BsonArray
                        {
                            new BsonDocument("$in", new BsonArray { "$BaseData.Code", new BsonArray(isOnlineCodes) }), // 判断条件
                            0, // 满足条件时的返回值
                            1 // 不满足条件时的返回值
                        }
                    }
                });
                var addFieldDoc = new BsonDocument("$addFields", bDoc);
                docList.Add(addFieldDoc);

                sortFilter.Add(new KeyValuePair<string, object>("isSpecifiedValue", 1));
                sortFilter.Add(new KeyValuePair<string, object>("BaseData.CreateTime", -1));
            }
            else
            {
                sortFilter.Add(new KeyValuePair<string, object>("BaseData.CreateTime", -1));
            }
            var sortDoc = new BsonDocument("$sort", new BsonDocument(sortFilter));
            docList.Add(sortDoc);
            return docList;
        }

        /// <summary>
        /// 根据组织编码查询所属全部设备信息
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        public async Task<List<DeviceInfoDTO>> FindDeviceInfoListByOrganizationCodeAsync(string organizationCode)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoListByOrganizationCodeAsync execute", $"{this.GetType().Name}"))
            {
                var filter = new Dictionary<string, string>();
                filter.Add("OrganizationCode", organizationCode);
                //查询设备基础信息
                var dataList = await _deviceInfoDBRepository.FindListAsync(filter);
                return dataList.MappingTo<List<DeviceInfoDTO>>();
            }
        }

        /// <summary>
        /// 根据组织编码列表查询所属全部设备信息
        /// </summary>
        /// <param name="organizationCode"></param>
        /// <returns></returns>
        public async Task<List<DeviceInfoDTO>> FindDeviceInfoListByOrganizationCodesAsync(IList<string> organizationCodes)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoListByOrganizationCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (organizationCodes == null || organizationCodes.Count == 0)
                {
                    return new List<DeviceInfoDTO>();
                }
                var filter = new Dictionary<string, List<string>>();
                filter.Add("OrganizationCode", organizationCodes.ToList());
                //查询设备基础信息
                var dataList = await _deviceInfoDBRepository.FindInAsync(filter);
                return dataList.MappingTo<List<DeviceInfoDTO>>();
            }
        }

        /// <summary>
        /// 根据设备序列号查询设备
        /// </summary>
        /// <param name="serialNumber">设备序列号</param>
        /// <returns>设备信息</returns>
        public async Task<DeviceInfoDTO> FindDeviceInfoBySerialNumberAsync(string serialNumber)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoBySerialNumberAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("SerialNumber", serialNumber);
                return deviceInfoDO.MappingTo<DeviceInfoDTO>();
            }
        }

        /// <summary>
        /// 根据设备型号列表获取设备信息列表
        /// </summary>
        /// <param name="request">移除设备请求实体</param>
        /// <returns></returns>
        public async Task<List<DeviceInfoDTO>> FindDeviceInfoListByModelCodeListAsync(List<string> modelCodeList)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoListByModelCodeListAsync execute", $"{this.GetType().Name}"))
            {
                var filter = new Dictionary<string, List<string>>();
                filter.Add("DeviceModel", modelCodeList);
                //查询设备基础信息
                var dataList = await _deviceInfoDBRepository.FindInAsync(filter);
                return dataList.MappingTo<List<DeviceInfoDTO>>();
            }
        }

        public async Task<List<DeviceInfoDTO>> FindDeviceInfoByDeviceType(FindDeviceInfoByDeviceTypeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindDeviceInfoByDeviceType execute", $"{this.GetType().Name}"))
            {
                string deviceTypeCode = request.DeviceTypeCode;
                MongoCustomerFilter<MongoTable> customerFilter = new();
                customerFilter.AppendEq(("BaseData.DeviceType", deviceTypeCode));
                var dataList = await _deviceInfoDBRepository.FindAllAsync(customerFilter.AndCombine());
                return dataList.MappingTo<List<DeviceInfoDTO>>();
            }
        }

        /// <summary>
        /// 更新最近登录时间
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<bool> UpdateLoginTimeAsync(UpdateLoginTimeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateLoginTimeAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                deviceInfoDO.LastLoginTime = request.LastLoginTime ?? DateTime.UtcNow;
                if (string.IsNullOrWhiteSpace(deviceInfoDO.ShortCode))
                {
                    deviceInfoDO.ShortCode = request.ShortCode;
                }
                deviceInfoDO.DeviceModel = request.DeviceModel;
                deviceInfoDO.DeviceType = request.DeviceType;
                deviceInfoDO.DeviceSoftwareVersion = request.SoftwareVersion;
                deviceInfoDO.SystemVersion = request.SystemVersion;
                deviceInfoDO.CPUModel = request.CPUModel;
                // deviceInfoDO.Description = request.Description;
                // deviceInfoDO.Name = request.Name;
                // deviceInfoDO.OrganizationCode = request.OrganizationCode;
                deviceInfoDO.SystemLanguage = request.SystemLanguage;
                return await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfoDO.Code, deviceInfoDO, "", false);
            }
        }

        /// <summary>
        /// 删除设备
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<bool> DeleteDeviceDBAsync(string deviceCode)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteDeviceDBAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", deviceCode);
                deviceInfoDO.IsDelete = true;
                return await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfoDO.Code, deviceInfoDO, "", false);
            }
        }

        public async Task<long> GetActiveDeviceCount()
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("IsDelete", "False"));
            var count = await _deviceInfoDBRepository.CountDocumentsAsync(customerFilter.AndCombine());
            return count;
        }

        /// <summary>
        /// 更新视频设备画面信息
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<bool> UpdateVideoDeviceInfosAsync(UpdateVideoDeviceInfosDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateVideoDeviceInfosAsync execute", $"{this.GetType().Name}"))
            {
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                if (deviceInfoDO != null && !string.IsNullOrEmpty(deviceInfoDO.Code))
                {
                    deviceInfoDO.LiveOpened = request.LiveOpened;
                    deviceInfoDO.VideoDeviceInfos = request.VideoDeviceInfos?.MappingTo<List<VideoDeviceDO>>() ?? new List<VideoDeviceDO>();
                    if (deviceInfoDO.MergedVideoOutputWidth > 0 && deviceInfoDO.MergedVideoOutputHeight > 0)
                    {
                        await AddDeviceOutputConfigAsync(new AddDeviceOutputConfigDBRequest
                        {
                            DeviceVideoOutputType = DeviceVideoOutputTypeEnum.MergedVideo,
                            VideoWidth = deviceInfoDO.MergedVideoOutputWidth,
                            VideoHeight = deviceInfoDO.MergedVideoOutputHeight
                        });
                    }
                    if (deviceInfoDO.VideoDeviceInfos.Any())
                    {
                        foreach (var videoDeviceInfo in deviceInfoDO.VideoDeviceInfos)
                        {
                            await AddDeviceOutputConfigAsync(new AddDeviceOutputConfigDBRequest
                            {
                                DeviceVideoOutputType = videoDeviceInfo.VideoDeviceSourceType == VideoDeviceSourceTypeEnum.Camera ? DeviceVideoOutputTypeEnum.Camera : DeviceVideoOutputTypeEnum.Desktop,
                                VideoWidth = videoDeviceInfo.OutputWidth,
                                VideoHeight = videoDeviceInfo.OutputHeight
                            });
                        }
                    }
                    if (request.IsSync)
                    {
                        return await _deviceInfoDBRepository.UpdateOneForSyncDBAsync("Code", deviceInfoDO.Code, deviceInfoDO, "", false);
                    }
                    else
                    {
                        return await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfoDO.Code, deviceInfoDO, "", false);
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 新增设备画面尺寸
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<bool> AddDeviceOutputConfigAsync(AddDeviceOutputConfigDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddDeviceOutputConfigAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<DeviceOutputConfig>.Filter.Where(f =>
                    f.VideoWidth == request.VideoWidth
                    && f.VideoHeight == request.VideoHeight
                    && f.DeviceVideoOutputType == request.DeviceVideoOutputType
                );
                var deviceOutputConfig = await _deviceOutputConfigDBRepository.FindOneAsync(filter);
                if (deviceOutputConfig == null)
                {
                    var result = await _deviceOutputConfigDBRepository.InsertOneAsync(new DeviceOutputConfig
                    {
                        Name = $"{request.VideoWidth}x{request.VideoHeight}",
                        DeviceVideoOutputType = request.DeviceVideoOutputType,
                        VideoWidth = request.VideoWidth,
                        VideoHeight = request.VideoHeight
                    });
                    return !string.IsNullOrWhiteSpace(result);
                }
                return true;
            }
        }

        /// <summary>
        /// 获取设备画面尺寸
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<List<DeviceOutputConfigDTO>> GetDeviceOutputConfigAsync(GetDeviceOutputConfigDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDeviceOutputConfigAsync execute", $"{this.GetType().Name}"))
            {
                var filter = Builders<DeviceOutputConfig>.Filter.Where(f =>
                    f.DeviceVideoOutputType == request.DeviceVideoOutputType
                );
                var deviceOutputConfigs = await _deviceOutputConfigDBRepository.FindAllAsync(filter);
                if (deviceOutputConfigs != null)
                {
                    var configList = new List<DeviceOutputConfigDTO>();
                    foreach (var deviceOutput in deviceOutputConfigs)
                    {
                        var config = new DeviceOutputConfigDTO();
                        {
                            config.Code = deviceOutput.Code;
                            config.Name = deviceOutput.Name;
                            config.VideoWidth = deviceOutput.VideoWidth;
                            config.VideoHeight = deviceOutput.VideoHeight;
                            config.DeviceVideoOutputType = deviceOutput.DeviceVideoOutputType;
                        };
                        configList.Add(config);
                    }
                    return configList;
                }
            }
            return new List<DeviceOutputConfigDTO>();
        }

        /// <summary>
        /// 删除品牌型号
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteBrandModelOutputConfigAsync(DeleteBrandModelOutputConfigDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteBrandModelOutputConfigAsync execute", $"{this.GetType().Name}"))
            {
                var filter = request.BrandModelCodes.Select(s => ("Code", s)).ToList();
                var brandModelOutputConfig = await _brandModelOutputConfigDBRepository.DeleteAllAsync(filter);
                return brandModelOutputConfig > 0;
            }
        }

        /// <summary>
        /// 新增设备画面尺寸
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<bool> AddBrandModelOutputConfigAsync(AddBrandModelOutputConfigDBRequest request)
        {
            var filter = Builders<BrandModelOutputConfig>.Filter.Where(f => f.Brand == request.Brand && f.Model == request.Model);
            var brandModelOutputConfig = await _brandModelOutputConfigDBRepository.FindOneAsync(filter);
            if (brandModelOutputConfig != null)
            {
                var updateFilter = Builders<BrandModelOutputConfig>.Filter.Eq("Code", brandModelOutputConfig.Code);
                if (brandModelOutputConfig.Resolutions != null && brandModelOutputConfig.Resolutions.Count > 0)
                {
                    var resolution = brandModelOutputConfig.Resolutions.FirstOrDefault(w => w.ShortCode == request.ShortCode);
                    if (resolution != null)
                    {
                        var subFilter = Builders<Resolution>.Filter.Eq(e => e.ShortCode, request.ShortCode);
                        var updatePull = Builders<BrandModelOutputConfig>.Update.PullFilter(f => f.Resolutions, subFilter);
                        var updatePullResult = await _brandModelOutputConfigDBRepository.UpdateOneAsync(updateFilter, updatePull);
                        if (updatePullResult > 0)
                        {
                            var updatePush = Builders<BrandModelOutputConfig>.Update.Push(f => f.Resolutions, new Entities.Resolution
                            {
                                ShortCode = request.ShortCode,
                                VideoWidth = request.VideoWidth,
                                VideoHeight = request.VideoHeight
                            });
                            return await _brandModelOutputConfigDBRepository.UpdateOneAsync(updateFilter, updatePush) > 0;
                        }
                        return false;
                    }
                    else
                    {
                        var config = new Entities.Resolution
                        {
                            ShortCode = request.ShortCode,
                            VideoWidth = request.VideoWidth,
                            VideoHeight = request.VideoHeight
                        };
                        var updateAddToSet = Builders<BrandModelOutputConfig>.Update.AddToSet(s => s.Resolutions, config);
                        return await _brandModelOutputConfigDBRepository.UpdateOneAsync(updateFilter, updateAddToSet) > 0;
                    }
                }
                else
                {
                    var config = new Entities.Resolution
                    {
                        ShortCode = request.ShortCode,
                        VideoWidth = request.VideoWidth,
                        VideoHeight = request.VideoHeight
                    };
                    var updateAddToSet = Builders<BrandModelOutputConfig>.Update.Set(s => s.Resolutions, new List<Entities.Resolution> { config });
                    return await _brandModelOutputConfigDBRepository.UpdateOneAsync(updateFilter, updateAddToSet) > 0;
                }
            }
            else
            {
                var entity = new BrandModelOutputConfig();
                entity.Code = Guid.NewGuid().ToString("N");
                entity.Brand = request.Brand;
                entity.Model = request.Model;
                entity.Resolutions = new List<Entities.Resolution>();
                entity.Resolutions.Add(new Entities.Resolution
                {
                    ShortCode = request.ShortCode,
                    VideoWidth = request.VideoWidth,
                    VideoHeight = request.VideoHeight
                });
                if (request.ShortCode != "DEFAULT")
                {
                    entity.Resolutions.Add(new Entities.Resolution
                    {
                        ShortCode = "DEFAULT",
                        VideoWidth = request.VideoWidth,
                        VideoHeight = request.VideoHeight
                    });
                }
                return !string.IsNullOrWhiteSpace(await _brandModelOutputConfigDBRepository.InsertOneAsync(entity));
            }
        }

        /// <summary>
        /// 获取设备画面尺寸
        /// </summary>
        /// <returns>是否操作成功</returns>
        public async Task<List<BrandModelOutputConfigDTO>> GetBrandModelOutputConfigAsync(GetBrandModelOutputConfigDBRequest request)
        {
            var filter = Builders<BrandModelOutputConfig>.Filter.Where(f => f.Brand == request.Brand && f.Model == request.Model && f.IsDelete == false);
            var brandModelOutput = await _brandModelOutputConfigDBRepository.FindOneAsync(filter);
            if (brandModelOutput != null && brandModelOutput.Resolutions != null && brandModelOutput.Resolutions.Count > 0)
            {
                var resolution = brandModelOutput.Resolutions.FirstOrDefault(w => w.ShortCode == request.ShortCode);
                if (resolution == null)
                {
                    resolution = brandModelOutput.Resolutions.FirstOrDefault(w => w.ShortCode == "DEFAULT");
                }
                return brandModelOutput.Resolutions.Select(s => new BrandModelOutputConfigDTO
                {
                    IsSelect = s.ShortCode == resolution.ShortCode,
                    VideoWidth = s.VideoWidth,
                    VideoHeight = s.VideoHeight,
                }).ToList();
            }
            else
            {
                ThrowCustomerException(CustomerRpcCode.ConfigNotExists, "ConfigNotExists");
                return null;
            }
        }

        /// <summary>
        /// 获取品牌
        /// </summary>
        /// <returns></returns>
        public async Task<List<string>> GetBrandsAsync()
        {
            var projection = Builders<BrandModelOutputConfig>.Projection.Include(f => f.Brand);
            var filter = Builders<BrandModelOutputConfig>.Filter.Where(f => f.IsDelete == false);
            var entitys = await _brandModelOutputConfigDBRepository.FindAllAsync(filter, new FindOptions<BrandModelOutputConfig>()
            {
                Projection = projection
            });
            var brands = entitys.Select(s => s.Brand).Distinct().ToList();
            return brands;
        }

        /// <summary>
        /// 获取品牌型号列表
        /// </summary>
        /// <returns></returns>
        public async Task<List<string>> GetModelsAsync(GetModelsDBRequest request)
        {
            var projection = Builders<BrandModelOutputConfig>.Projection.Include(f => f.Model);
            var filter = Builders<BrandModelOutputConfig>.Filter.Where(f => f.IsDelete == false && f.Brand == request.Brand);
            var entitys = await _brandModelOutputConfigDBRepository.FindAllAsync(filter, new FindOptions<BrandModelOutputConfig>()
            {
                Projection = projection
            });
            var models = entitys.Select(s => s.Model).Distinct().ToList();
            return models;
        }

        /// <summary>
        /// 获取品牌型号分页
        /// </summary>
        /// <param name="queryRequest"></param>
        /// <returns></returns>
        public async Task<PageResult<BrandModelDetailDTO>> GetDeviceBrandModelPageAsync(GetDeviceBrandModelPageDBRequest queryRequest)
        {
            var filterKeyword = Builders<BrandModelOutputConfig>.Filter.Where(f => f.IsDelete == false);
            if (!string.IsNullOrWhiteSpace(queryRequest.Keyword))
            {
                var pattern = ReplaceSpecialCharacter(queryRequest.Keyword);
                BsonRegularExpression regularExp = new BsonRegularExpression($".*{pattern}.*", "ig");
                filterKeyword = filterKeyword & Builders<BrandModelOutputConfig>.Filter.Or(
                    Builders<BrandModelOutputConfig>.Filter.Regex(f => f.Brand, regularExp),
                    Builders<BrandModelOutputConfig>.Filter.Regex(f => f.Model, regularExp)
                );
            }
            var brandModel = new DBPageRequest<BrandModelOutputConfig>();
            brandModel.CurrentPage = queryRequest.PageIndex;
            brandModel.PageSize = queryRequest.PageSize;
            brandModel.Filter = filterKeyword;
            var pageResult = await _brandModelOutputConfigDBRepository.GetTablePageAsync(brandModel);
            if (pageResult != null)
            {
                var result = new PageResult<BrandModelDetailDTO>();
                result.PageIndex = pageResult.CurrentPage;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.DataCount;
                result.PageData = pageResult.PageData?.MappingTo<List<BrandModelDetailDTO>>();
                return result;
            }
            return null;
        }

        /// <summary>
        /// 通过名称获取设备更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DevicePatchDTO> GetDevicePatchDBbyNameAsync(GetDevicePatchDBbyNameRequest request)
        {
            var filter = Builders<DevicePatchEntity>.Filter.Where(f =>
               (f.Name == request.Name || f.Code == request.Code)
               && f.IsDelete == false);
            var patch = await _devicePatchDBRepository.FindOneAsync(filter);
            return patch?.MappingTo<DevicePatchDTO>();
        }

        /// <summary>
        /// 增加设备更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDevicePatchDBAsync(AddDevicePatchDBRequest request)
        {
            var result = await _devicePatchDBRepository.InsertOneAsync(new DevicePatchEntity
            {
                Name = request.Name,
                Description = request.Description,
                DeviceType = request.DeviceType,
                SoftwareVersion = request.SoftwareVersion,
                OsVersion = request.OsVersion,
                DeviceFileInfoList = request.DeviceFileInfoList,
                FileSize = request.FileSize,
                FileName = request.FileName
            });
            return !string.IsNullOrWhiteSpace(result);
        }
        /// <summary>
        /// 更新设备更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDevicePatchDBAsync(UpdateDevicePatchDBRequest request)
        {
            var filter = Builders<DevicePatchEntity>.Filter.Eq(f => f.Code, request.Code);
            var update = Builders<DevicePatchEntity>.Update
            .Set(s => s.Name, request.Name)
            .Set(s => s.Description, request.Description)
            .Set(s => s.SoftwareVersion, request.SoftwareVersion)
            .Set(s => s.OsVersion, request.OsVersion)
            .Set(s => s.UpdateTime, DateTime.UtcNow);
            var modifiedCount = await _devicePatchDBRepository.UpdateOneAsync(filter, update);
            return modifiedCount > 0;
        }
        /// <summary>
        /// 删除设备更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDevicePatchDBbyCodeAsync(DeleteDevicePatchDBbyCodeRequest request)
        {
            var filter = Builders<DevicePatchEntity>.Filter.Eq(f => f.Code, request.Code);
            var update = Builders<DevicePatchEntity>.Update
            .Set(s => s.IsDelete, true)
            .Set(s => s.UpdateTime, DateTime.UtcNow);
            var modifiedCount = await _devicePatchDBRepository.UpdateOneAsync(filter, update);
            return modifiedCount > 0;
        }

        /// <summary>
        /// 分页查询设备更新包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<DevicePatchDTO>> FindDevicePatchPageDBAsync(FindDevicePatchPageDBRequest request)
        {
            var builder = Builders<DevicePatchEntity>.Filter;
            var filterKeyword = builder.Where(f => f.IsDelete == false);

            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {
                var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                var regularExp = new BsonRegularExpression(keyword, "i");
                var nameFilter = builder.Regex(c => c.Name, regularExp);
                var descriptionFilter = builder.Regex(c => c.Description, regularExp);
                var softwareVersionFilter = builder.Regex(c => c.SoftwareVersion, regularExp);
                var osVersionFilter = builder.Regex(c => c.OsVersion, regularExp);
                var deviceTypeFilter = builder.Where(f => f.DeviceType == keyword);
                filterKeyword = filterKeyword & (nameFilter | descriptionFilter | softwareVersionFilter | osVersionFilter | deviceTypeFilter);
            }

            if (request.FindOnlyDeviceTypes != null && request.FindOnlyDeviceTypes.Count > 0)
            {
                filterKeyword = filterKeyword & (builder.Where(f => request.FindOnlyDeviceTypes.Contains(f.DeviceType)));
            }
            var patch = new DBPageRequest<DevicePatchEntity>();
            patch.CurrentPage = request.PageIndex;
            patch.PageSize = request.PageSize;
            patch.Filter = filterKeyword;
            var pageResult = await _devicePatchDBRepository.GetTablePageAsync(patch);
            if (pageResult != null)
            {
                var result = new PageResult<DevicePatchDTO>();
                result.PageIndex = pageResult.CurrentPage;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.DataCount;
                result.PageData = pageResult.PageData?.MappingTo<List<DevicePatchDTO>>();
                return result;
            }
            return null;
        }

        /// <summary>
        /// 增加设备版本
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDevicePatchVersionDBAsync(AddDevicePatchVersionDBRequest request)
        {
            var result = await _devicePatchVersionDBRepository.InsertOneAsync(new DevicePatchVersionEntity
            {
                Version = request.Version,
                VersionTypeEnum = request.VersionTypeEnum,
                DeviceType = request.DeviceType,
            });
            return !string.IsNullOrWhiteSpace(result);
        }
        /// <summary>
        /// 查询设备版本
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<DevicePatchVersionDTO>> FindDevicePatchVersionListDBAsync(FindDevicePatchVersionListDBRequest request)
        {
            var filterContent =
              Builders<DevicePatchVersionEntity>.Filter.Where(f => f.DeviceType == request.DeviceType);
            if (request.VersionTypeEnum != DeviceVersionEnum.All)
            {
                filterContent = filterContent & Builders<DevicePatchVersionEntity>.Filter.Where(f => f.VersionTypeEnum == request.VersionTypeEnum);
            }
            var patchList = await _devicePatchVersionDBRepository.FindAllAsync(filterContent);
            return patchList.MappingTo<List<DevicePatchVersionDTO>>();
        }

        /// <summary>
        /// 删除设备版本
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDevicePatchVersionDBAsync(DeleteDevicePatchVersionDBRequest request)
        {
            return await _devicePatchVersionDBRepository.DeleteOneAsync("Code", request.Code);
        }

        /// <summary>
        /// 增加打印机驱动包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddDevicePrinterDBAsync(AddDevicePrinterDBRequest request)
        {
            var filter = Builders<DevicePrinterEntity>.Filter.Eq(w => w.Name, request.Name);
            var existPrinter = await _devicePrinterDBRepository.FindOneAsync(filter);
            if (existPrinter == null)
            {
                var result = await _devicePrinterDBRepository.InsertOneAsync(new DevicePrinterEntity
                {
                    Name = request.Name,
                    Description = request.Description,
                    OsVersion = request.OsVersion,
                    FileUploadInfoList = request.FileUploadInfoList,
                    FileSize = request.FileSize,
                    PrinterBrands = request.PrinterBrands,
                    PrinterModels = request.PrinterModels
                });
                return !string.IsNullOrWhiteSpace(result);
            }
            else
            {
                existPrinter.FileName = request.FileName;
                existPrinter.FileSize = request.FileSize;
                existPrinter.FileUploadInfoList = request.FileUploadInfoList;
                return await _devicePrinterDBRepository.UpdateOneAsync("Code", existPrinter.Code, existPrinter);
            }
        }

        /// <summary>
        /// 查询打印机驱动包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<DevicePrinterDTO>> FindDevicePrinterPageDBAsync(FindDevicePrinterPageDBRequest request)
        {
            var filterKeyword = Builders<DevicePrinterEntity>.Filter.Where(f => f.IsDelete == false);
            if (!string.IsNullOrWhiteSpace(request.Keyword))
            {
                var pattern = ReplaceSpecialCharacter(request.Keyword);
                BsonRegularExpression regularExp = new BsonRegularExpression($".*{pattern}.*", "ig");
                filterKeyword = filterKeyword & Builders<DevicePrinterEntity>.Filter.Or(
                    Builders<DevicePrinterEntity>.Filter.Regex(f => f.Name, regularExp),
                    Builders<DevicePrinterEntity>.Filter.Regex(f => f.Description, regularExp),
                    Builders<DevicePrinterEntity>.Filter.Regex(f => f.OsVersion, regularExp),
                    Builders<DevicePrinterEntity>.Filter.Regex(f => f.PrinterBrands, regularExp)
                );
            }
            var patch = new DBPageRequest<DevicePrinterEntity>();
            patch.CurrentPage = request.PageIndex;
            patch.PageSize = request.PageSize;
            patch.Filter = filterKeyword;
            var pageResult = await _devicePrinterDBRepository.GetTablePageAsync(patch);
            if (pageResult != null)
            {
                var result = new PageResult<DevicePrinterDTO>();
                result.PageIndex = pageResult.CurrentPage;
                result.PageSize = pageResult.PageSize;
                result.TotalCount = pageResult.DataCount;
                result.PageData = pageResult.PageData?.MappingTo<List<DevicePrinterDTO>>();
                return result;
            }
            return null;
        }

        /// <summary>
        /// 查询打印机驱动是否存在
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ExistsDevicePrinterDBAsync(ExistsDevicePrinterDBRequest request)
        {
            var filter = Builders<DevicePrinterEntity>.Filter.Eq("Name", request.Name);
            return await _devicePrinterDBRepository.Exists(filter);
        }

        /// <summary>
        /// 删除打印机驱动包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDevicePrinterDBAsync(DeleteDevicePrinterDBRequest request)
        {
            return await _devicePrinterDBRepository.DeleteOneAsync("Code", request.Code);
        }

        /// <summary>
        /// 获取打印机驱动包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DevicePrinterDTO> GetDevicePrinterDBAsync(GetDevicePrinterDBRequest request)
        {
            var filter = Builders<DevicePrinterEntity>.Filter.Where(c => c.IsDelete == false);
            if (!string.IsNullOrEmpty(request.Code))
            {
                filter = filter & Builders<DevicePrinterEntity>.Filter.Where(
                f => f.Code == request.Code);
            }
            else if (!string.IsNullOrEmpty(request.Name))
            {
                filter = filter & Builders<DevicePrinterEntity>.Filter.Where(
               f => f.Name == request.Name);
            }
            // if (!string.IsNullOrEmpty(request.PrinterModel))
            // {
            //     filter = filter | Builders<DevicePrinterEntity>.Filter.Where(
            //    f => f. PrinterModels.Contains( request.PrinterModel));
            // }
            var result = await _devicePrinterDBRepository.FindOneAsync(filter);
            return result?.MappingTo<DevicePrinterDTO>();
        }

        /// <summary>
        /// 更新打印机驱动包
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDevicePrinterDBAsync(UpdateDevicePrinterDBRequest request)
        {
            var filter = Builders<DevicePrinterEntity>.Filter.Eq(f => f.Code, request.Code);
            var update = Builders<DevicePrinterEntity>.Update
            .Set(s => s.Name, request.Name)
            .Set(s => s.Description, request.Description)
            .Set(s => s.OsVersion, request.OsVersion)
            .Set(s => s.UpdateTime, DateTime.UtcNow);
            var modifiedCount = await _devicePrinterDBRepository.UpdateOneAsync(filter, update);
            return modifiedCount > 0;
        }

        /// <summary>
        /// 批量新增打印机驱动
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertDevicePrintersDataAsync(MigrateDevicePrinterRequest request)
        {
            long res = 0;
            if (request?.MigrateDevicePrinterInfos?.Count > 0)
            {
                //找到相同的
                var ids = request.MigrateDevicePrinterInfos.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<DevicePrinterEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var devicePrinterList = await _devicePrinterDBRepository.FindAllAsync(filter);
                //求同存异
                if (devicePrinterList?.Count > 0)
                {
                    var existCodes = devicePrinterList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.MigrateDevicePrinterInfos.Where(c => existCodes.Contains(c.Code));
                    var insertData = request.MigrateDevicePrinterInfos.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<DevicePrinterEntity>();
                                data.Code = f.Code.Trim();
                                return data;
                            }
                        ).ToList();
                        res = await _devicePrinterDBRepository.InsertAllAsync(req);
                    }
                    if (updateData?.Count() > 0)
                    {
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<DevicePrinterEntity>();
                            updateEntity.Code = f.Code.Trim();
                            var r = await _devicePrinterDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity);
                        }

                    }
                }
                else
                {
                    //新增
                    var req = request.MigrateDevicePrinterInfos.Select(
                        f =>
                        {
                            var data = f.MappingTo<DevicePrinterEntity>();
                            data.Code = f.Code.Trim();
                            return data;
                        }
                    ).ToList();
                    res = await _devicePrinterDBRepository.InsertAllAsync(req);
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 批量新增补丁包
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertDevicePatchsDataAsync(MigrateDevicePatchRequest request)
        {
            var patchInfos = request.MigrateDevicePatchInfos;
            long res = 0;
            if (patchInfos != null && patchInfos.Any())
            {
                //找到相同的
                var ids = patchInfos.Select(c => c.Code).Distinct().ToList();
                var builder = Builders<DevicePatchEntity>.Filter;
                var filter = builder.In(c => c.Code, ids);
                var devicePatchList = await _devicePatchDBRepository.FindAllAsync(filter);
                //求同存异
                if (devicePatchList != null && devicePatchList.Any())
                {
                    var existCodes = devicePatchList.Select(c => c.Code).Distinct().ToList();
                    var updateData = patchInfos.Where(c => existCodes.Contains(c.Code));
                    var insertData = patchInfos.Where(c => !existCodes.Contains(c.Code));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<DevicePatchEntity>();
                                data.Code = f.Code.Trim();
                                return data;
                            }
                        ).ToList();
                        res = await _devicePatchDBRepository.InsertAllAsync(req);
                    }
                    if (updateData?.Count() > 0)
                    {
                        foreach (var f in updateData)
                        {
                            var updateEntity = f.MappingTo<DevicePatchEntity>();
                            updateEntity.Code = f.Code.Trim();
                            var r = await _devicePatchDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity);
                        }

                    }
                }
                else
                {
                    //新增
                    var req = patchInfos.Select(
                        f =>
                        {
                            var data = f.MappingTo<DevicePatchEntity>();
                            data.Code = f.Code.Trim();
                            return data;
                        }
                    ).ToList();
                    res = await _devicePatchDBRepository.InsertAllAsync(req);
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 批量新增超声机数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertDeviceDataDBAsync(BatchInsertDeviceRequest request)
        {
            bool res = false;
            if (request?.DeviceMigratorys?.Count > 0)
            {
                //找到相同的
                var ids = request.DeviceMigratorys.Select(c => c.DeviceCode).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var deviceList = await _deviceInfoDBRepository.FindInAsync(dic);
                //求同存异
                var moduleList = await GetAllDiagnosisModuleList();
                if (deviceList?.Count > 0)
                {
                    var existCodes = deviceList.Select(c => c.Code).Distinct().ToList();
                    var updateData = request.DeviceMigratorys.Where(c => existCodes.Contains(c.DeviceCode));
                    var insertData = request.DeviceMigratorys.Where(c => !existCodes.Contains(c.DeviceCode));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var shortCodes = insertData.Where(c => !string.IsNullOrEmpty(c.ShortCode)).Select(c => c.ShortCode).Distinct().ToList();
                        dic = new Dictionary<string, List<string>>();
                        dic.Add("ShortCode", shortCodes);
                        var shortCodeList = await _deviceInfoDBRepository.FindInAsync(dic);
                        if (shortCodeList?.Count > 0)
                        {
                            var existShortCodes = shortCodeList.Select(c => c.ShortCode).Distinct().ToList();
                            var updateShortCodeData = insertData.Where(c => existShortCodes.Contains(c.ShortCode));
                            var insertShortCodeData = insertData.Where(c => !existShortCodes.Contains(c.ShortCode));
                            if (insertShortCodeData?.Count() > 0)
                            {
                                res = await BatchAddDeviceData(insertShortCodeData, moduleList, request.SyncType);
                            }
                            if (updateShortCodeData?.Count() > 0)
                            {
                                Logger.WriteLineWarn($"DeviceInfoDBService BatchInsertDeviceDataDBAsync Repeat ShortCode, insertData: {JsonConvert.SerializeObject(updateShortCodeData)}");
                            }
                        }
                        else
                        {
                            res = await BatchAddDeviceData(insertData, moduleList, request.SyncType);
                        }
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(f =>
                        {
                            var updateEntity = f.MappingTo<DeviceInfoDO>();
                            updateEntity.Code = updateEntity.DeviceCode.Trim();
                            updateEntity.IsDelete = f.IsDelete;
                            updateEntity.DiagnosisModules = DealWithDiagnosisModules(moduleList, f.DiagnosisModule);
                            if (request.SyncType == SyncDBEnum.Synchronize)
                            {
                                var r = _deviceInfoDBRepository.UpdateOneForSyncDBAsync("Code", updateEntity.Code, updateEntity, null).Result;
                            }
                            return updateEntity;
                        }).ToList();
                        if (request.SyncType == SyncDBEnum.Migrate)
                        {
                            res = await _deviceInfoDBRepository.UpdateAllWithoutOplogLogAsync(req);
                        }
                        foreach (var item in updateData)
                        {
                            CacheMaintenance.Instance.Get<IDeviceInfosManager>().Remove(item.DeviceCode);
                        }
                    }
                }
                else
                {
                    //新增
                    var shortCodes = request.DeviceMigratorys.Where(c => !string.IsNullOrEmpty(c.ShortCode)).Select(c => c.ShortCode).Distinct().ToList();
                    dic = new Dictionary<string, List<string>>();
                    dic.Add("ShortCode", shortCodes);
                    var shortCodeList = await _deviceInfoDBRepository.FindInAsync(dic);
                    if (shortCodeList?.Count > 0)
                    {
                        var existShortCodes = shortCodeList.Select(c => c.ShortCode).Distinct().ToList();
                        var updateShortCodeData = request.DeviceMigratorys.Where(c => existShortCodes.Contains(c.ShortCode));
                        var insertShortCodeData = request.DeviceMigratorys.Where(c => !existShortCodes.Contains(c.ShortCode));
                        if (insertShortCodeData?.Count() > 0)
                        {
                            res = await BatchAddDeviceData(insertShortCodeData, moduleList, request.SyncType);
                        }
                        if (updateShortCodeData?.Count() > 0)
                        {
                            Logger.WriteLineWarn($"DeviceInfoDBService BatchInsertDeviceDataDBAsync Repeat ShortCode, DeviceMigratorys: {JsonConvert.SerializeObject(updateShortCodeData)}");
                        }
                    }
                    else
                    {
                        res = await BatchAddDeviceData(request.DeviceMigratorys, moduleList, request.SyncType);
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 设备数据列表-分页 
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<DeviceInfoDTO>> QueryIOTDeviceInfoDBByPageAsync(QueryDeviceInfoByPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryIOTDeviceInfoDBByPageAsync execute", $"{this.GetType().Name}"))
            {
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

                //todo 超级管理员和普通管理员查询条件应该不一样
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    var items = new List<string> { "SerialNumber", "ShortCode" };
                    foreach (var item in items)
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, item));
                        queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                        queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                    }
                }
                if (queryItemsFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                var deviceTypeList = new List<string>() { "Breath", "Oxygen" };
                if (!string.IsNullOrWhiteSpace(request.DeviceType))
                {
                    if (request.DeviceType == "1")
                    {
                        deviceTypeList = new List<string>() { "Breath" };
                    }
                    else if (request.DeviceType == "2")
                    {
                        deviceTypeList = new List<string>() { "Oxygen" };
                    }
                }
                queryItemsFilter.AppendEq((item => item.Key, "DeviceType"));
                queryItemsFilter.AppendIn(item => item.Value, deviceTypeList);

                if (queryItemsFilter.HasFilter)
                {
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
                }

                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
                var dbRequest = new DBPageRequest();
                dbRequest.CurrentPage = request.PageIndex;
                dbRequest.PageSize = request.PageSize;
                dbRequest.Filter = customerFilter.AndCombine();
                dbRequest.Sort = sort;
                var deviceListResult = await _deviceInfoDBRepository.GetPages(dbRequest);
                var pageResult = PageResultConvert<DeviceInfoDO, DeviceInfoDTO>(deviceListResult);
                if (pageResult?.PageData?.Count > 0)
                {
                    var dicReq = new FindDictionaryItemsDBRequest { 
                        DictionaryType = (int)DictionaryTypeEnum.DeviceType,
                        ShowDeviceCategory = 1
                    };                    
                    var deviceTypeInfoList = await FindDictionaryItemsAsync(dicReq);
                    if (deviceTypeInfoList?.Count > 0)
                    {
                        foreach(var item in pageResult.PageData)
                        {
                            item.LanguageConfigs = deviceTypeInfoList.FirstOrDefault(t => t.DictionaryCode == item.DeviceType)?.LanguageConfigs;
                        }
                    }
                }
                return pageResult;
            }
        }

        /// <summary>
        /// 设备报告数据列表-分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<DeviceReportDTO>> QueryIOTDeviceReportInfoDBByPageAsync(QueryDeviceReportByPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryIOTDeviceReportInfoDBByPageAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.Keyword;
                var builder = Builders<DeviceReportEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.DeviceCode == request.DeviceCode);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var keyFilter = builder.Regex(c => c.DeviceType, new BsonRegularExpression(keyword, "i"));
                    filter = filter & keyFilter;
                }
                if (request.StartTime > Convert.ToDateTime("1900-2-1"))
                {
                     filter = filter & builder.Where(c => c.Timestamp >= request.StartTime);
                }
                if (request.EndTime > Convert.ToDateTime("1900-2-1"))
                {
                    filter = filter & builder.Where(c => c.Timestamp < request.EndTime.AddDays(1));
                }
                var sort = Builders<DeviceReportEntity>.Sort.Descending(v => v.CreateTime);
                var dbPage = new DBPageRequest<DeviceReportEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _deviceReportDBRepository.GetPages(dbPage);
                var dataList = PageResultConvert<DeviceReportEntity, DeviceReportDTO>(pageResult);
                return dataList;
            }
        }

        /// <summary>
        /// 新增数据
        /// </summary>
        private async Task<bool> BatchAddDeviceData(IEnumerable<DeviceMigratoryInfo> insertData, IList<DiagnosisModuleDO> moduleList, SyncDBEnum syncType)
        {
            var req = insertData.Select(
                f =>
                {
                    var data = f.MappingTo<DeviceInfoDO>();
                    data.Code = data.DeviceCode.Trim();
                    data.IsDelete = f.IsDelete;
                    if (string.IsNullOrEmpty(data.ShortCode))
                    {
                        var shortCode = "";
                        for (var i = 0; i < 5; i++)
                        {
                            shortCode = GeneralShortCode();
                            if (!string.IsNullOrWhiteSpace(shortCode))
                            {
                                var deviceInfo = FindDeviceInfoByShortCodeAsync(shortCode).Result;
                                if (deviceInfo == null || string.IsNullOrWhiteSpace(deviceInfo.DeviceCode))
                                {
                                    break;
                                }
                            }
                            shortCode = string.Empty;
                        }
                        data.ShortCode = shortCode;
                    }
                    data.DiagnosisModules = DealWithDiagnosisModules(moduleList, f.DiagnosisModule);
                    if (syncType == SyncDBEnum.Synchronize)
                    {
                        var r = _deviceInfoDBRepository.InsertOneForAsyncDBAsync(data, new Dictionary<string, string>(), "").Result;
                        var addResult = AddPushDeviceAsync(data).Result;
                    }
                    return data;
                }
            ).ToList();
            var res = true;
            if (syncType == SyncDBEnum.Migrate)
            {
                res = await _deviceInfoDBRepository.InsertAllWithoutOplogLogAsync(req);
            }
            return res;
        }

        /// <summary>
        /// AI模块选中值数据转换
        /// </summary>
        private List<string> DealWithDiagnosisModules(IList<DiagnosisModuleDO> moduelList, IList<DiagnosisModuleEnum> diagnosisModule)
        {
            var res = new List<string>();
            if (diagnosisModule?.Count > 0 && moduelList?.Count > 0)
            {
                var selectedList = diagnosisModule.Select(c => c.ToString()).Distinct().ToList();
                var isExistLiverLesionDetect = diagnosisModule.ToList().Exists(c => c == DiagnosisModuleEnum.LiverLesionDetect);
                var isExistBreastLesionDetectBIRADS = diagnosisModule.ToList().Exists(c => c == DiagnosisModuleEnum.BreastLesionDetectBIRADS);
                foreach (var item in selectedList)
                {
                    var findEntity = moduelList.FirstOrDefault(c => c.DiagnosisModule == item) ?? new DiagnosisModuleDO();
                    if (!string.IsNullOrEmpty(findEntity.Code))
                    {
                        res.Add(findEntity.Code);
                    }
                }
                if (isExistBreastLesionDetectBIRADS && isExistLiverLesionDetect)
                {
                    //增加甲状腺
                    var findEntity = moduelList.FirstOrDefault(c => c.DiagnosisModule == DiagnosisModuleEnum.ThyroidLesionDetect.ToString()) ?? new DiagnosisModuleDO();
                    if (!string.IsNullOrEmpty(findEntity.Code))
                    {
                        res.Add(findEntity.Code);
                    }
                }
            }
            return res;
        }

        /// <summary>
        /// 查询所有的AI模块
        /// </summary>
        private async Task<IList<DiagnosisModuleDO>> GetAllDiagnosisModuleList()
        {
            IList<DiagnosisModuleDO> res = new List<DiagnosisModuleDO>();
            res = await _diagnosisModuleDBRepository.FindAllAsync();
            return res;
        }

        /// <summary>
        /// 清理重复短码设备
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> ClearRepeatDeviceInfoDBAsync(ShareDeviceRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindCoursesByCodesAsync execute", "DatabaseService"))
            {
                var res = false;
                if (request.DeviceCodes?.Count > 0)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("ShortCode", request.DeviceCodes);
                    var shortCodeList = await _deviceInfoDBRepository.FindInAsync(dic);
                    if (shortCodeList?.Count > 0)
                    {
                        foreach (var item in request.DeviceCodes)
                        {
                            var findList = shortCodeList.Where(c => c.ShortCode == item);
                            var queryList = findList.Where(c => c.VideoDeviceInfos?.Count <= 0);
                            if (queryList?.Count() > 1)
                            {
                                var findEntity = queryList.FirstOrDefault();
                                var shortCode = "";
                                for (var i = 0; i < 5; i++)
                                {
                                    shortCode = GeneralShortCode();
                                    if (!string.IsNullOrWhiteSpace(shortCode))
                                    {
                                        var orgInfo = await _organizationDBRepository.FindOneAsync("ShortCode", shortCode);
                                        if (orgInfo == null || string.IsNullOrWhiteSpace(orgInfo.Code))
                                        {
                                            break;
                                        }
                                    }
                                    shortCode = string.Empty;
                                }
                                findEntity.ShortCode = shortCode;
                                res = await _deviceInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity, string.Empty);
                            }
                            // else
                            // {
                            //     var findEntity = queryList.FirstOrDefault();
                            //     MongoCustomerFilter<MongoTable> customerFilter = new();
                            //     customerFilter.AppendEq(("BaseData.ShortCode", item));
                            //     customerFilter.AppendEq(("BaseData.MergedChannel", false));
                            //     MongoCustomerFilter<MongoDataItem> subFilter = new();
                            //     MongoCustomerUpdate<MongoTable> update = new();
                            //     subFilter.AppendEq((item => item.Key, nameof(findEntity.IsDelete)));
                            //     update.SetField("BaseData.IsDelete", true);
                            //     update.RemoveSubItemFilter(item => item.QueryData.QueryItems, subFilter.AndCombine());
                            //     res = await _userExtendsDBRepository.UpdateAllAsync(customerFilter.AndCombine(), update.AllUpdateDefinition) > 0;
                            // }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 初始化数据
        /// </summary>
        public async Task<bool> InitDeviceModelDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} InitDeviceModelDBAsync execute", $"{this.GetType().Name}"))
            {
                try
                {
                    var filter = new Dictionary<string, string>();
                    filter.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceModel).ToString());
                    filter.Add("IsDelete", "False");
                    var deviceTypeDOList = await _dictionaryDBRepository.FindListAsync(filter);
                    if (deviceTypeDOList?.Count > 0)
                    {
                        foreach (var dictionaryDO in deviceTypeDOList)
                        {
                            dictionaryDO.IsDelete = true;
                            var res = await _dictionaryDBRepository.UpdateOneAsync("Code", dictionaryDO.Code, dictionaryDO, string.Empty, true);
                        }
                    }

                    filter = new Dictionary<string, string>();
                    filter.Add("IsDelete", "False");
                    var deviceDOList = await _deviceInfoDBRepository.FindListAsync(filter);
                    if (deviceDOList?.Count > 0)
                    {
                        var typeFilter = new Dictionary<string, string>();
                        typeFilter.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceModel).ToString());
                        typeFilter.Add("IsDelete", "False");
                        deviceTypeDOList = await _dictionaryDBRepository.FindListAsync(typeFilter) ?? new List<DictionaryDO>();
                        var groupedDeviceType = deviceDOList.GroupBy(p => new { p.DeviceType, p.DeviceModel });
                        foreach (var group in groupedDeviceType)
                        {
                            if (!string.IsNullOrEmpty(group.Key.DeviceType) && !string.IsNullOrEmpty(group.Key.DeviceModel)
                                && !deviceTypeDOList.Exists(c => c.ParentCode == group.Key.DeviceType && c.Value == group.Key.DeviceModel))
                            {
                                //新增
                                var dicInfo = new DictionaryDO()
                                {
                                    Code = Guid.NewGuid().ToString("N"),
                                    DictionaryType = (int)DictionaryTypeEnum.DeviceModel,
                                    Value = group.Key.DeviceModel,
                                    ParentCode = group.Key.DeviceType,
                                    LanguageConfigs = new List<DictionaryLanguageConfigDTO>()
                                };
                                var res = await _dictionaryDBRepository.InsertOneForAsyncDBAsync(dicInfo, new Dictionary<string, string>(), string.Empty);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DeviceDBService InitDeviceModelDBAsync Error: {ex}");
                }
            }
            return true;
        }

        /// <summary>
        /// 升级设备型号数据
        /// </summary>
        private async Task<bool> UpgradeDeviceModelForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeDeviceModelForESDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var typeFilter = new Dictionary<string, string>();
                    typeFilter.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceType).ToString());
                    typeFilter.Add("IsDelete", "False");
                    var deviceTypeDOList = await _dictionaryDBRepository.FindListAsync(typeFilter) ?? new List<DictionaryDO>();
                    if (deviceTypeDOList?.Count > 0)
                    {
                        foreach (var item in deviceTypeDOList)
                        {
                            if (!item.LanguageConfigs.Exists(c => c.Language == "es-ES"))
                            {
                                if (item.Code == "US")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "es-ES",
                                        Value = "US"
                                    });
                                }
                                else if (item.Code == "sonopost")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "es-ES",
                                        Value = "sonopost"
                                    });
                                }
                                else
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "es-ES",
                                        Value = ""
                                    });
                                }
                                res = await _dictionaryDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                            }
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DeviceDBService UpgradeDeviceModelForESDBAsync Error: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// 升级设备型号数据
        /// </summary>
        private async Task<bool> UpgradeDeviceModelForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeDeviceModelForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                try
                {
                    var typeFilter = new Dictionary<string, string>();
                    typeFilter.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceType).ToString());
                    typeFilter.Add("IsDelete", "False");
                    var deviceTypeDOList = await _dictionaryDBRepository.FindListAsync(typeFilter) ?? new List<DictionaryDO>();
                    if (deviceTypeDOList?.Count > 0)
                    {
                        foreach (var item in deviceTypeDOList)
                        {
                            if (!item.LanguageConfigs.Exists(c => c.Language == "ro-RO"))
                            {
                                if (item.Code == "US")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "ro-RO",
                                        Value = "US"
                                    });
                                }
                                else if (item.Code == "sonopost")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "ro-RO",
                                        Value = "sonopost"
                                    });
                                }
                                else if (item.Code == "Breath")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "ro-RO",
                                        Value = "respirator"
                                    });
                                }
                                else if (item.Code == "Oxygen")
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "ro-RO",
                                        Value = "Concentrator de oxigen"
                                    });
                                }
                                else
                                {
                                    item.LanguageConfigs.Add(new DictionaryLanguageConfigDTO()
                                    {
                                        Language = "ro-RO",
                                        Value = ""
                                    });
                                }
                                res = await _dictionaryDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true, false);
                            }
                        }
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DeviceDBService UpgradeDeviceModelForRODBAsync Error: {ex}");
                    return res;
                }
            }
        }


        /// <summary>
        /// 添加设备类型
        /// </summary>
        private async Task<string> UpdateIOTDeviceTypeDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateIOTDeviceTypeDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = "";
                try
                {
                    var breathInfo = await _dictionaryDBRepository.FindOneAsync("Code", "Breath") ?? new DictionaryDO();
                    if (string.IsNullOrEmpty(breathInfo?.Code))
                    {
                        //新增
                        var model = new DictionaryDO()
                        {
                            Code = "Breath",
                            DictionaryType = 1,
                            Value = "呼吸机",
                            ParentCode = "",
                            LanguageConfigs = new List<DictionaryLanguageConfigDTO>()
                            {
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "en-US",
                                    Value = "Ventilator"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "es-ES",
                                    Value = "Ventilador"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "ru-RU",
                                    Value = "Вентилятор"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "zh-CN",
                                    Value = "呼吸机"
                                }
                            }
                        };
                        res = await _dictionaryDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty);
                    }
                    var oxygenInfo = await _dictionaryDBRepository.FindOneAsync("Code", "Oxygen") ?? new DictionaryDO();
                    if (string.IsNullOrEmpty(oxygenInfo?.Code))
                    {
                        //新增
                        var model = new DictionaryDO()
                        {
                            Code = "Oxygen",
                            DictionaryType = 1,
                            Value = "制氧机",
                            ParentCode = "",
                            LanguageConfigs = new List<DictionaryLanguageConfigDTO>()
                            {
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "en-US",
                                    Value = "Oxygenerator"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "es-ES",
                                    Value = "Generador de oxígeno"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "ru-RU",
                                    Value = "Кислородный генератор"
                                },
                                new DictionaryLanguageConfigDTO() {                                    
                                    Language = "zh-CN",
                                    Value = "制氧机"
                                }
                            }
                        };
                        res = await _dictionaryDBRepository.InsertOneAsync(model, new Dictionary<string, string>(), string.Empty, false);
                    }
                    return res;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DeviceDBService UpdateIOTDeviceTypeDBAsync Error: {ex}");
                    return res;
                }
            }
        }

        /// <summary>
        /// IOT设备注册,保存，有可能是编辑，每次登录都会提示，然后有更新
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<string> IOTDeviceRegisterDBAsync(IOTDeviceRegisterRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} IOTDeviceRegisterDBAsync execute", $"{this.GetType().Name}"))
            {
                var serialNumber = request.DeviceSN + "_" + request.DeviceMac;
                var existData = await _deviceInfoDBRepository.FindOneAsync("SerialNumber", serialNumber);
                if (string.IsNullOrEmpty(existData?.Code))
                {
                    var data = new DeviceInfoDO();
                    data.Code = Guid.NewGuid().ToString("N");
                    data.IsDelete = false;

                    var shortCode = "";
                    for (var i = 0; i < 10; i++)
                    {
                        shortCode = GeneralShortCode();
                        if (!string.IsNullOrWhiteSpace(shortCode))
                        {
                            var deviceInfo = FindDeviceInfoByShortCodeAsync(shortCode).Result;
                            if (deviceInfo == null || string.IsNullOrWhiteSpace(deviceInfo.DeviceCode))
                            {
                                break;
                            }
                        }
                        shortCode = string.Empty;
                    }
                    data.Name = request.Name;
                    data.Description = request.Description;
                    data.ShortCode = shortCode;
                    if (string.IsNullOrEmpty(data.Name))
                    {
                        data.Name = data.ShortCode;
                    }
                    data.SerialNumber = serialNumber;
                    data.SystemVersion = request.DeviceMac;     
                    data.DeviceModel = request.DeviceModel;     
                    data.DeviceSoftwareVersion = request.DeviceSN;
                    data.Password = "Symmetry_mk9xah8N";       
                    if (!string.IsNullOrEmpty(request.DeviceModel) && request.DeviceModel.Length > 2)
                    {
                        //SC-30BS
                        var typeKey = request.DeviceModel.Substring(0, 2).ToUpper();
                        if (typeKey == "SC" || typeKey == "BC")
                        {
                            data.DeviceType = "Breath";
                        }
                        else 
                        {
                            var firstChar = typeKey.Substring(0, 1).ToUpper();
                            if (firstChar == "K" || firstChar == "Z")
                            {
                                data.DeviceType = "Oxygen";
                            }
                        }
                    }
                    data.DeviceAttributes = string.Empty;
                    var res = await _deviceInfoDBRepository.InsertOneAsync(data, new Dictionary<string, string>(), string.Empty, true);
                    if (string.IsNullOrEmpty(res))
                    {
                        Logger.WriteLineWarn($"DeviceDBService IOTDeviceRegisterDBAsync Insert Fail: {JsonConvert.SerializeObject(data)}");
                    }
                    return res;
                }
                else
                {
                    //更新其他信息，然后在返回
                    existData.Description = request.Description;
                    if (string.IsNullOrEmpty(existData.Name))
                    {
                        existData.Name = existData.ShortCode;
                    }
                    existData.DeviceModel = request.DeviceModel;     
                    if (!string.IsNullOrEmpty(request.DeviceModel) && request.DeviceModel.Length > 2)
                    {
                        //SC-30BS
                        var typeKey = request.DeviceModel.Substring(0, 2).ToUpper();
                        if (typeKey == "SC" || typeKey == "BC")
                        {
                            existData.DeviceType = "Breath";
                        }
                        else 
                        {
                            var firstChar = typeKey.Substring(0, 1).ToUpper();
                            if (firstChar == "K" || firstChar == "Z")
                            {
                                existData.DeviceType = "Oxygen";
                            }
                        }
                    }
                    await _deviceInfoDBRepository.UpdateOneAsync("Code", existData.Code, existData, string.Empty, true);
                    return existData.Code;
                }
            }
        }
    
        /// <summary>
        /// 新增版本数据
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddIOTDeviceVersionInfoDBAsync(AddIOTDeviceVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddIOTDeviceVersionInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<IOTDeviceVersionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.VersionNo == request.VersionNo && c.IOTDeviceVersionType == request.IOTDeviceVersionType 
                && c.Type == request.Type && c.DeviceType == request.DeviceType && c.BaseVersionType == request.BaseVersionType && c.FileBuildNum == request.FileBuildNum);
                var findEntity = await _iotDeviceVersionDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code))
                {
                    return false;
                }
                else 
                {
                    var deviceVersionInfo = request.MappingTo<IOTDeviceVersionEntity>();
                    var insertCode = await _iotDeviceVersionDBRepository.InsertOneAsync(deviceVersionInfo);
                    if (string.IsNullOrEmpty(insertCode))
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
        }
        
        /// <summary>
        /// 设备报告数据列表-分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> UpdateIOTDeviceVersionInfoDBAsync(UpdateIOTDeviceVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateIOTDeviceVersionInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<IOTDeviceVersionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.VersionNo == request.VersionNo && c.IOTDeviceVersionType == request.IOTDeviceVersionType 
                && c.Type == request.Type && c.DeviceType == request.DeviceType && c.BaseVersionType == request.BaseVersionType && c.FileBuildNum == request.FileBuildNum);
                var findEntity = await _iotDeviceVersionDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(findEntity?.Code) && findEntity.Code != request.Code)
                {
                    //表示找到另外一个其他的对象，有相同的版本信息
                    return false;
                }
                else
                {
                    filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);
                    findEntity = await _iotDeviceVersionDBRepository.FindOneAsync(filter);
                    if(string.IsNullOrEmpty(findEntity?.Code))
                    {
                        return false;
                    }
                    else
                    {
                        findEntity.VersionCode = request.VersionCode;
                        findEntity.VersionNo = request.VersionNo;
                        findEntity.DeviceType = request.DeviceType;
                        findEntity.IOTDeviceVersionType = request.IOTDeviceVersionType;
                        findEntity.Type = request.Type;
                        findEntity.FileToken = request.FileToken;
                        findEntity.FileBuildNum = request.FileBuildNum;
                        findEntity.BaseVersionType = request.BaseVersionType;
                        if (request.ChildDeviceVersions?.Count > 0)
                        {
                            findEntity.ChildDeviceVersions = request.ChildDeviceVersions.MappingTo<List<IOTDeviceVersionEntity>>();
                        }
                        else
                        {
                            findEntity.ChildDeviceVersions = new List<IOTDeviceVersionEntity>();
                        }
                        findEntity.UpdateTime = DateTime.UtcNow;
                        var res = await _iotDeviceVersionDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                        return res;
                    }
                }
            }
        }
        
        /// <summary>
        /// 设备报告数据列表-分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> DeleteIOTDeviceVersionInfoDBAsync(QueryIOTDeviceVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteIOTDeviceVersionInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _iotDeviceVersionDBRepository.DeleteOneAsync("Code", request.Code);
                return result;
            }
        }

        /// <summary>
        /// 根据查询实体查询设备报告数据
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IOTDeviceVersionDTO> QueryIOTDeviceVersionInfoDBAsync(QueryIOTDeviceVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryIOTDeviceVersionInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<IOTDeviceVersionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Code))
                {
                    filter = filter & builder.Where(c => c.Code == request.Code);
                }
                if (((int)request.IOTDeviceVersionType) > 0)
                {
                    filter = filter & builder.Where(c => c.IOTDeviceVersionType == request.IOTDeviceVersionType);
                }
                if (((int)request.Type) > 0)
                {
                    filter = filter & builder.Where(c => c.Type == request.Type);
                }
                if (!string.IsNullOrEmpty(request.VersionNo))
                {
                    filter = filter & builder.Where(c => c.VersionNo == request.VersionNo);
                }
                var findEntity = await _iotDeviceVersionDBRepository.FindOneAsync(filter);
                var result = findEntity.MappingTo<IOTDeviceVersionDTO>();
                return result;
            }
        }

        /// <summary>
        /// IOT设备版本分页信息列表-分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<IOTDeviceVersionDTO>> QueryIOTDeviceVersionInfoDBByPageAsync(QueryDeviceVersionByPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} QueryIOTDeviceVersionInfoDBByPageAsync execute", $"{this.GetType().Name}"))
            {
                var keyword = request.Keyword;
                var builder = Builders<IOTDeviceVersionEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(keyword))
                {
                    keyword = EscapeExprSpecialWord(keyword);//字符串转义
                    var deviceTypeCodes = new List<string>();
                    var queryDic = new Dictionary<string, string>();
                    queryDic.Add("DictionaryType", ((int)DictionaryTypeEnum.DeviceType).ToString());
                    var dictionaryDatas = await _dictionaryDBRepository.FindListAsync(queryDic) ?? new List<DictionaryDO>();
                    if (dictionaryDatas?.Count > 0)
                    {
                        if (!string.IsNullOrWhiteSpace(keyword))
                        {
                            var filterDatas = dictionaryDatas.Where(dicData =>
                            {
                                if (dicData.LanguageConfigs == null || !dicData.LanguageConfigs.Any())
                                {
                                    return dicData.Value.Contains(keyword, StringComparison.OrdinalIgnoreCase);
                                }
                                else
                                {
                                    return dicData.LanguageConfigs.Any(x => x.Value.Contains(keyword, StringComparison.OrdinalIgnoreCase));
                                }
                            })?.ToList() ?? new();
                            if (filterDatas?.Count > 0)
                            {
                                deviceTypeCodes = filterDatas.Select(c => c.DictionaryCode).Distinct().ToList();
                            }
                        }
                    }
                    
                    var versionCodeFilter = builder.Regex(c => c.VersionCode, new BsonRegularExpression(keyword, "i"));
                    var versionNoFilter = builder.Regex(c => c.VersionNo, new BsonRegularExpression(keyword, "i"));
                    var fileBuildNumVersionNoFilter = builder.Regex(c => c.FileBuildNum, new BsonRegularExpression(keyword, "i"));
                    var orFilter = (versionNoFilter | versionCodeFilter | fileBuildNumVersionNoFilter);
                    if (deviceTypeCodes?.Count > 0)
                    {
                        var deviceTypeFilter = builder.In(c => c.DeviceType,deviceTypeCodes);
                        orFilter = orFilter | deviceTypeFilter;
                    }
                    filter = filter & orFilter;
                }
                if (request.QueryIOTDeviceVersionType == 1)
                {
                    filter = filter & builder.Where(c => c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.BaseVersion);
                }
                else if (request.QueryIOTDeviceVersionType == 2)
                {
                    filter = filter & builder.Where(c => c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.DevelopVersion | c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.ReleaseVersion);
                }
                else if (request.QueryIOTDeviceVersionType == 3)
                {
                    filter = filter & builder.Where(c => c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.DevelopVersion);
                }
                else if (request.QueryIOTDeviceVersionType == 4)
                {
                    filter = filter & builder.Where(c => c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.ReleaseVersion);
                }
                if (((int)request.Type) > 0)
                {
                    filter = filter & builder.Where(c => c.Type == request.Type);
                }
                var sort = Builders<IOTDeviceVersionEntity>.Sort.Descending(v => v.CreateTime);
                var dbPage = new DBPageRequest<IOTDeviceVersionEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _iotDeviceVersionDBRepository.GetPages(dbPage);
                var dataList = PageResultConvert<IOTDeviceVersionEntity, IOTDeviceVersionDTO>(pageResult);
                return dataList;
            }
        }

        /// <summary>
        /// 绑定设备 
        /// 用户:设备 = 1：N；设备同时只能被一个用户绑定
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SaveUserDevicesDBAsync(UserBindDeviceRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveUserDevicesDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //Oxygen 制氧机 ； Breath 呼吸机
                var dic = new Dictionary<string, string>();
                var serialNumber = request.MachineNo + "_" + request.MachineAddress;
                dic.Add("SerialNumber", serialNumber);
                var deviceType = "";
                if (request.Type == 0)
                {
                    deviceType = "Breath";
                    dic.Add("DeviceType", "Breath");
                }
                else 
                {
                    deviceType = "Oxygen";
                    dic.Add("DeviceType", "Oxygen");
                }
                var deviceList = await _deviceInfoDBRepository.FindListAsync(dic);
                if (deviceList?.Count > 0)
                {
                    //表示存在设备信息
                    var deviceCode = deviceList.OrderByDescending(c => c.UpdateTime).FirstOrDefault()?.Code ?? string.Empty;
                    //验证是否其他账号已经绑定了此设备
                    var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.DeviceCode == deviceCode && c.UserCode != request.UserCode);
                    var deviceExist = await _userDeviceExtendInfoDBRepository.Exists(filter);
                    if (deviceExist)
                    {
                        ThrowCustomerException(CustomerRpcCode.DeviceDuplicateBind, "Device duplicate bind");
                    }
                    filter = builder.Where(c => c.IsDelete == false && c.DeviceCode == deviceCode && c.UserCode == request.UserCode);
                    var findEntity = await _userDeviceExtendInfoDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(findEntity?.Code))
                    {
                        //修改
                        findEntity.UserDeviceName = request.UserDeviceName;
                        findEntity.WifiName = request.WifiName;
                        findEntity.IsAuto = request.IsAuto;
                        findEntity.AreaCode = request.AreaCode;
                        findEntity.MachineAddress = serialNumber;
                        findEntity.DeviceSN = request.MachineNo;
                        findEntity.DeviceMAC = request.MachineAddress;
                        findEntity.ModeType = request.ModeType;
                        findEntity.UpdateTime = DateTime.UtcNow;
                        res = await _userDeviceExtendInfoDBRepository.UpdateOneAsync("Code", findEntity.Code, findEntity);
                    }
                    else
                    {
                        //新增
                        findEntity = new UserDeviceExtendInfoEntity();
                        findEntity.UserDeviceName = request.UserDeviceName;
                        findEntity.WifiName = request.WifiName;
                        findEntity.IsAuto = request.IsAuto;
                        findEntity.AreaCode = request.AreaCode;
                        findEntity.MachineAddress = serialNumber;
                        findEntity.DeviceSN = request.MachineNo;
                        findEntity.DeviceMAC = request.MachineAddress;
                        findEntity.ModeType = request.ModeType;
                        findEntity.UserCode = request.UserCode;
                        findEntity.DeviceCode = deviceCode;
                        findEntity.DeviceType = deviceType;
                        var result = await _userDeviceExtendInfoDBRepository.InsertOneAsync(findEntity);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 删除用户绑定设备
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> DeleteUserDeviceDBAsync(DeleteDeviceInfoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteUserDeviceDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _userDeviceExtendInfoDBRepository.DeleteOneAsync("Code", request.Id);
                return result;
            }
        }

        /// <summary>
        /// 获取用户设备数据列表通过设备code
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<UserDeviceInfoResult> GetUserDevicesByDeviceCodeDBAsync(string deviceCode)
         {
            using (new Performance(_performanceThreshold, "{0} GetUserDevicesByDeviceCodeDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<UserDeviceInfoResult>();
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.DeviceCode == deviceCode);
                var res = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                if (res?.Count > 0)
                {
                    var findEntity = res.OrderByDescending(c => c.UpdateTime).FirstOrDefault();
                    var model = new UserDeviceInfoResult()
                    {
                        Id = findEntity.Code,
                        UserId = findEntity.UserCode,
                        DeviceId = findEntity.DeviceCode,
                        UserDeviceName = findEntity.UserDeviceName,
                        WifiName = findEntity.WifiName,
                        IsAuto = findEntity.IsAuto,
                        AreaCode = findEntity.AreaCode,
                        ProductNo = "",
                        CreatedTime = findEntity.CreateTime
                    };
                    if (!string.IsNullOrEmpty(findEntity.MachineAddress) && findEntity.MachineAddress.Split('_').Length > 1)
                    {
                        model.MachineAddress = findEntity.MachineAddress.Split('_')[1];
                        model.MachineNo =  findEntity.MachineAddress.Split('_')[0];
                    }
                    if (findEntity.DeviceType == "Breath")
                    {
                        model.Type = 0;
                    }
                    else if (findEntity.DeviceType == "Oxygen")
                    {
                        model.Type = 1;
                    }
                    return model;
                }
                return null;
            }
        }

        /// <summary>
        /// 获取用户设备数据列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IList<UserDeviceInfoResult>> GetUserDevicesDBAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserDevicesDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<UserDeviceInfoResult>();
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserCode == userCode && !string.IsNullOrEmpty(c.DeviceCode));
                var res = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                if (res?.Count > 0)
                {
                    var deviceCodes = res.Select(c => c.DeviceCode).Distinct().ToList();
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", deviceCodes);
                    var diviceInfos = await _deviceInfoDBRepository.FindInAsync(dic);
                    if (diviceInfos?.Count > 0)
                    {
                        foreach(var item in res)
                        {
                            var deviceInfo = diviceInfos.FirstOrDefault(c => c.Code == item.DeviceCode) ?? new DeviceInfoDO();
                            var model = new UserDeviceInfoResult()
                            {
                                Id = item.Code,
                                UserId = item.UserCode,
                                DeviceId = item.DeviceCode,
                                UserDeviceName = item.UserDeviceName,
                                WifiName = item.WifiName,
                                IsAuto = item.IsAuto,
                                AreaCode = item.AreaCode,
                                MachineAddress = deviceInfo.SystemVersion,
                                MachineNo = deviceInfo.DeviceSoftwareVersion,
                                ProductNo = "",
                                CreatedTime = item.CreateTime
                            };
                            if (deviceInfo.DeviceType == "Breath")
                            {
                                model.Type = 0;
                            }
                            else if (deviceInfo.DeviceType == "Oxygen")
                            {
                                model.Type = 1;
                            }

                            var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                            var lastFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == item.DeviceCode && c.ReportType == ReportType.Day);
                            
                            var entity = await _deviceReportDBRepository.FindOneAsync(lastFilter, new FindOptions<DeviceReportEntity> { Sort = Builders<DeviceReportEntity>.Sort.Descending(f => f.UpdateTime) });
                            if (!string.IsNullOrEmpty(entity?.Code))
                            {
                                model.LastTreatTime = entity.Timestamp;     
                                var useHourStr = entity.ReportAttributes.FirstOrDefault(c => c.Key == "use_hour")?.Value ?? string.Empty;
                                decimal totalUseHour = 0;
                                if (!string.IsNullOrEmpty(useHourStr))
                                {
                                    totalUseHour = decimal.Round(Decimal.Parse(useHourStr), 2);
                                }
                                var avgUseHour = entity.ReportAttributes.FirstOrDefault(c => c.Key == "use_ahour")?.Value ?? string.Empty;
                                model.Score = GetReportScore(entity, totalUseHour);
                                model.UseHour = totalUseHour;
                                model.ModeType = (int)entity.Mode;
                            }
                            result.Add(model);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 保存设备指定升级版本
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SaveDeviceUpgradeVersionDBAsync(SaveDeviceUpgradeVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveDeviceUpgradeVersionDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                if (!string.IsNullOrEmpty(deviceInfoDO?.Code))
                {
                    deviceInfoDO.UpgradeVersionCode = request.DeviceUpgradeVersionCode;
                    res = await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfoDO.Code, deviceInfoDO, string.Empty, true);
                }
                return res;
            }
        }

        /// <summary>
        /// 获取设备首页总结数据
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IOTDeviceSumaryResult> GetUserDevicesSummaryDBAsync(GetSummaryRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetUserDevicesSummaryDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = new IOTDeviceSumaryResult();
                var deviceType = "";
                if (request.Type == 0)
                {
                    deviceType = "Breath";
                }
                else 
                {
                    deviceType = "Oxygen";
                }
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserCode == request.UserCode && c.DeviceType == deviceType);
                var findEntity = await _userDeviceExtendInfoDBRepository.FindOneAsync(filter);
                if (string.IsNullOrEmpty(findEntity?.Code))
                {
                    return res;
                }
                //获取最近的治疗日期
                var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                var lastFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == findEntity.DeviceCode && c.ReportType == ReportType.Day);
                
                var entity = await _deviceReportDBRepository.FindOneAsync(lastFilter, new FindOptions<DeviceReportEntity> { Sort = Builders<DeviceReportEntity>.Sort.Descending(f => f.UpdateTime) });
                if (!string.IsNullOrEmpty(entity?.Code))
                {
                    res.LastTreatTime = entity.Timestamp;
                }

                var startDate = DateTime.Now.GetCurrentWeek();
                var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == findEntity.DeviceCode && c.Timestamp >= startDate && c.ReportType == ReportType.Day);
                var deviceReports = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                var dicts = new Dictionary<DateTime, decimal>();
                var totalUseDays = 0;
                var reportCodes = new List<string>();
                var firstDay = DateTime.Now.GetCurrentWeek();
                for (int i = 0; i <= 6; i++)
                {
                    var dt = firstDay.Date.AddDays(i);
                    var deviceReportInfo = deviceReports?.OrderByDescending(c => c.Timestamp).FirstOrDefault(c => c.Timestamp.Date == dt);
                    decimal useHour = 0;
                    if (!string.IsNullOrEmpty(deviceReportInfo?.Code) && deviceReportInfo?.ReportAttributes?.Count > 0)
                    {
                        var val = deviceReportInfo.ReportAttributes.FirstOrDefault(c => c.Key == "use_hour")?.Value ?? string.Empty;
                        if (!string.IsNullOrEmpty(val))
                        {
                            useHour = decimal.Round(Decimal.Parse(val), 2);
                            if (useHour > 0)
                            {
                                reportCodes.Add(deviceReportInfo.Code);
                            }
                        }

                        var useDayStr = deviceReportInfo.ReportAttributes.FirstOrDefault(c => c.Key == "use_day")?.Value ?? string.Empty;
                        if (!string.IsNullOrEmpty(useDayStr))
                        {
                            var tempTotalUseDay = Convert.ToInt32(useDayStr);
                            if (tempTotalUseDay > 0)
                            {
                                totalUseDays += tempTotalUseDay;
                            }
                        }
                    }
                    dicts.Add(dt, useHour);
                }

                deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == findEntity.DeviceCode && c.ReportType == ReportType.Year);
                var yearReports = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                var yearReport = yearReports?.OrderByDescending(c => c.Timestamp)?.FirstOrDefault() ?? new DeviceReportEntity();
                //年统计数据
                var yearInfo = new IOTDeviceSumaryYearInfo();
                if (!string.IsNullOrEmpty(yearReport?.Code) && yearReport?.ReportAttributes?.Count > 0)
                {
                    var useDayStr = yearReport.ReportAttributes.FirstOrDefault(c => c.Key == "use_day")?.Value ?? string.Empty;
                    if (!string.IsNullOrEmpty(useDayStr))
                    {
                        yearInfo.TotalUseDay = Convert.ToInt32(useDayStr);
                    }
                    var useHourStr = yearReport.ReportAttributes.FirstOrDefault(c => c.Key == "use_hour")?.Value ?? string.Empty;
                    if (!string.IsNullOrEmpty(useHourStr))
                    {
                        yearInfo.TotalUseHour = decimal.Round(Decimal.Parse(useHourStr), 2);
                    }
                    yearInfo.AvgUseHour = yearReport.ReportAttributes.FirstOrDefault(c => c.Key == "use_ahour")?.Value ?? string.Empty;
                    yearInfo.AvgSocre = GetReportScore(yearReport, yearInfo.TotalUseHour);
                }
                else
                {
                    startDate = DateTime.Now.GetCurrentYear();
                    deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == findEntity.DeviceCode && c.Timestamp >= startDate && c.ReportType == ReportType.Day);
                    deviceReports = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);

                    var totalUseDayYear = 0;
                    decimal totalUseHourYear = 0;
                    var avgSocreYear = 0;
                    var tempTwice = 0;
                    for (int i = 0; i < 365; i++)
                    {
                        var dt = startDate.Date.AddDays(i);
                        var deviceReportInfo = deviceReports?.OrderByDescending(c => c.Timestamp).FirstOrDefault(c => c.Timestamp.Date == dt);
                        decimal useHour = 0;
                        if (!string.IsNullOrEmpty(deviceReportInfo?.Code) && deviceReportInfo?.ReportAttributes?.Count > 0)
                        {
                            var val = deviceReportInfo.ReportAttributes.FirstOrDefault(c => c.Key == "use_hour")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(val))
                            {
                                useHour = decimal.Round(Decimal.Parse(val), 2);
                                if (useHour > 0)
                                {
                                    tempTwice++;
                                }
                                avgSocreYear += GetReportScore(deviceReportInfo, useHour);
                                totalUseHourYear += useHour;
                            }

                            var useDayStr = deviceReportInfo.ReportAttributes.FirstOrDefault(c => c.Key == "use_day")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(useDayStr))
                            {
                                var tempTotalUseDay = Convert.ToInt32(useDayStr);
                                if (tempTotalUseDay > 0)
                                {
                                    totalUseDayYear += tempTotalUseDay;
                                }
                            }
                        }
                    }
                    if (tempTwice > 0)
                    {
                        yearInfo.AvgUseHour = Math.Round((totalUseHourYear / tempTwice), 2).ToString();
                        yearInfo.TotalUseHour = totalUseHourYear;
                        yearInfo.AvgSocre = avgSocreYear / tempTwice;
                        yearInfo.TotalUseDay = totalUseDayYear;
                    }
                }
                //周统计数据
                var weekInfo = new IOTDeviceSumaryWeekInfo();
                weekInfo.TotalUseDay = deviceReports?.Count ?? 0;
                if (deviceReports?.Count > 0)
                {
                    var twice = 0;
                    var curAvgSocre = 0;
                    foreach(var reportCode in reportCodes)
                    {
                        var item = deviceReports.FirstOrDefault(x => x.Code == reportCode);
                        if (item?.ReportAttributes?.Count > 0)
                        {
                            var val = item.ReportAttributes.FirstOrDefault(c => c.Key == "use_hour")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(val))
                            {
                                var useHourVal = decimal.Round(Decimal.Parse(val), 2);
                                if (useHourVal > 0)
                                {
                                    twice++;
                                }
                                curAvgSocre += GetReportScore(item, useHourVal);
                                weekInfo.TotalUseHour += useHourVal;
                            }
                        }
                    }
                    if (twice > 0 && yearInfo.TotalUseHour <= 0)
                    {
                        yearInfo.AvgUseHour = Math.Round((weekInfo.TotalUseHour / twice), 2).ToString();
                        yearInfo.TotalUseHour = weekInfo.TotalUseHour;
                        yearInfo.AvgSocre = curAvgSocre / twice;
                        yearInfo.TotalUseDay = totalUseDays;
                    }
                }
                weekInfo.DayHours = dicts;
                //绑定统计数据
                res.Year = yearInfo;
                res.Week = weekInfo;
                return res;
            }
        }

        /// <summary>
        /// 计算当天报告得分
        /// </summary>
        /// <param name="report"></param>
        /// <returns></returns>
        private int GetReportScore(DeviceReportEntity yearReport, decimal useHour)
        {
            //使用时间
            var useHoutScore = GetUseHourScore(useHour);
            //漏气量
            int leak95 = 0;
            decimal ahi = 0;
            if (!string.IsNullOrEmpty(yearReport?.Code) && yearReport?.ReportAttributes?.Count > 0)
            {
                var leak95Str = yearReport.ReportAttributes.FirstOrDefault(c => c.Key == "leak_95")?.Value ?? string.Empty;
                if (!string.IsNullOrEmpty(leak95Str))
                {
                    leak95 = Convert.ToInt32(leak95Str);
                }
                var ahiStr = yearReport.ReportAttributes.FirstOrDefault(c => c.Key == "ahi")?.Value ?? string.Empty;
                if (!string.IsNullOrEmpty(ahiStr))
                {
                    ahi = decimal.Round(Decimal.Parse(ahiStr), 2);
                }
            }
            var leakScore = GetLeakScore(leak95);
            //Ahi
            var ahiScore = GetAhiScore(ahi);
            return useHoutScore + leakScore.score + ahiScore;
        }

        /// <summary>
        /// 使用时间得分
        /// </summary>
        private int GetUseHourScore(decimal useHour)
        {
            int score = default;
            if (useHour < 1)
            {
                score = 0;
            }
            else if (useHour >= 1 && useHour < 1.5M)
            {
                score = 10;
            }
            else if (useHour >= 1.5M && useHour < 2)
            {
                score = 20;
            }
            else if (useHour >= 2 && useHour < 2.5M)
            {
                score = 30;
            }
            else if (useHour >= 2.5M && useHour < 3)
            {
                score = 40;
            }
            else if (useHour >= 3 && useHour < 3.5M)
            {
                score = 50;
            }
            else if (useHour >= 3.5M && useHour < 4)
            {
                score = 60;
            }
            else if (useHour >= 4)
            {
                score = 70;
            }
            return score;
        }

        /// <summary>
        /// 漏气量结果
        /// </summary>
        private (int score, string level) GetLeakScore(int leak)
        {
            var leakResult = (0, string.Empty);
            if (leak > 34)
            {
                leakResult = (0, "高");
            }
            else if (leak > 33 && leak <= 34)
            {
                leakResult = (1, "高");
            }
            else if (leak > 32 && leak <= 33)
            {
                leakResult = (2, "高");
            }
            else if (leak > 31 && leak <= 32)
            {
                leakResult = (3, "高");
            }
            else if (leak > 30 && leak <= 31)
            {
                leakResult = (4, "高");
            }
            else if (leak > 29 && leak <= 30)
            {
                leakResult = (5, "高");
            }
            else if (leak > 28 && leak <= 29)
            {
                leakResult = (6, "高");
            }
            else if (leak > 27 && leak <= 28)
            {
                leakResult = (7, "高");
            }
            else if (leak > 26 && leak <= 27)
            {
                leakResult = (8, "高");
            }
            else if (leak > 25 && leak <= 26)
            {
                leakResult = (9, "高");
            }
            else if (leak > 24 && leak <= 25)
            {
                leakResult = (10, "中");
            }
            else if (leak > 23 && leak <= 24)
            {
                leakResult = (11, "中");
            }
            else if (leak > 22 && leak <= 23)
            {
                leakResult = (12, "中");
            }
            else if (leak > 21 && leak <= 22)
            {
                leakResult = (13, "中");
            }
            else if (leak > 20 && leak <= 21)
            {
                leakResult = (14, "中");
            }
            else if (leak > 19 && leak <= 20)
            {
                leakResult = (15, "中");
            }
            else if (leak > 18 && leak <= 19)
            {
                leakResult = (16, "中");
            }
            else if (leak > 17 && leak <= 18)
            {
                leakResult = (17, "中");
            }
            else if (leak > 16 && leak <= 17)
            {
                leakResult = (18, "中");
            }
            else if (leak > 15 && leak <= 16)
            {
                leakResult = (19, "中");
            }
            else if (leak <= 15)
            {
                leakResult = (20, "低");
            }
            return leakResult;
        }

        /// <summary>
        /// Ahi得分
        /// </summary>
        private int GetAhiScore(decimal ahi)
        {
            int score = default;
            if (ahi >= 25)
            {
                score = 0;
            }
            else if (ahi >= 20 && ahi < 25)
            {
                score = 2;
            }
            else if (ahi >= 15 && ahi < 20)
            {
                score = 4;
            }
            else if (ahi >= 10 && ahi < 15)
            {
                score = 6;
            }
            else if (ahi >= 5 && ahi < 10)
            {
                score = 8;
            }
            else if (ahi < 5)
            {
                score = 10;
            }
            return score;
        }

        /// <summary>
        /// 获取 呼吸机|制氧机 最新版本号数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<IOTDeviceVersionResult> GetLasterVersionDBAsync(GetDeviceVersionRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLasterVersionDBAsync execute", $"{this.GetType().Name}"))
            {
                //先查询设备中有没有指定版本，有则使用，否则找release版本最新版本
                var res = new IOTDeviceVersionResult()
                {
                    ota = new DeviceOTAResult()
                    {
                        pack_type = 0,
                        pack_version = "",
                        firmware = new List<FirmwareInfoResult>()
                    }
                };
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.Id);
                var deviceVersionInfo = new IOTDeviceVersionEntity();
                if (!string.IsNullOrEmpty(deviceInfoDO?.Code))
                {
                    var upgradeVersionCode = deviceInfoDO.UpgradeVersionCode;
                    var builder = Builders<IOTDeviceVersionEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.Code == upgradeVersionCode);
                    deviceVersionInfo = await _iotDeviceVersionDBRepository.FindOneAsync(filter);
                    if (string.IsNullOrEmpty(deviceVersionInfo?.Code))
                    {
                        var deviceType = "";
                        if (request.Type == 0)
                        {
                            deviceType = "Breath";
                        }
                        else 
                        {
                            deviceType = "Oxygen";
                        }
                        filter = builder.Where(c => c.IsDelete == false && c.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.ReleaseVersion && 
                                                        c.DeviceType == deviceType);
                        var deviceVersionList = await _iotDeviceVersionDBRepository.FindAllAsync(filter);
                        if (deviceVersionList?.Count > 0)
                        {
                            deviceVersionInfo = deviceVersionList.OrderByDescending(c => c.VersionNo).FirstOrDefault();
                        }
                    }
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.DeviceNotExist, "Device not exist");                    
                }
                
                if (deviceVersionInfo?.ChildDeviceVersions?.Count > 0)
                {
                    int packType = 0;
                    if (deviceVersionInfo.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.DevelopVersion)
                    {
                        packType = 1;
                    }
                    else if (deviceVersionInfo.IOTDeviceVersionType == IOTDeviceVersionTypeEnum.ReleaseVersion)
                    {
                        packType = 0;
                    }
                    res.ota.pack_type = packType;
                    res.ota.pack_version = deviceVersionInfo.VersionNo;
                    foreach(var item in deviceVersionInfo.ChildDeviceVersions)
                    {
                        var fwType = 0;
                        if (item.BaseVersionType == 2)
                        {
                            fwType = 1;
                        }
                        else if (item.BaseVersionType == 3)
                        {
                            fwType = 0;
                        }
                        var fwHwType = 0;
                        if (item.Type == DeviceBaseVersionBinTypeEnum.Control)
                        {
                            fwHwType = 1;
                        }
                        else if (item.Type == DeviceBaseVersionBinTypeEnum.UI)
                        {
                            fwHwType = 0;
                        }
                        else if (item.Type == DeviceBaseVersionBinTypeEnum.Motor)
                        {
                            fwHwType = 2;
                        }
                        var model = new FirmwareInfoResult() 
                        {
                            fw_type = fwType,
                            fw_version = item.VersionNo,
                            fw_hw_type = fwHwType,
                            fw_url = item.FileToken,
                            fw_build = Convert.ToInt32(item.FileBuildNum)
                        };
                        res.ota.firmware.Add(model);
                    }
                }
                return res;
            }
        }
    
        /// <summary>
        /// IOT设备属性上报
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> DeviceAttributePostDBAsync(IOTDeviceAttributeEscalationRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeviceAttributePostDBAsync execute", $"{this.GetType().Name}"))
            {    
                var res = false;
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.Id);
                if (!string.IsNullOrEmpty(deviceInfoDO?.Code)) 
                {
                    if (!string.IsNullOrEmpty(request.DeviceAttributesJson))         
                    {
                        var deviceAttributesObject = JObject.Parse(request.DeviceAttributesJson);                       
                        if (deviceAttributesObject != null)
                        {
                            if (deviceAttributesObject.ContainsKey("consumable_property"))
                            {
                                deviceInfoDO.DeviceConsumableAttributes = request.DeviceAttributesJson;
                            }
                            else if (deviceAttributesObject.ContainsKey("treat_property"))
                            {
                                deviceInfoDO.DeviceAttributes = request.DeviceAttributesJson;
                            }
                            res = await _deviceInfoDBRepository.UpdateOneAsync("Code", deviceInfoDO.Code, deviceInfoDO, string.Empty, true);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// IOT设备属性查询
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<string> DeviceAttributeGetDBAsync(IOTDeviceAttributeEscalationRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeviceAttributeGetDBAsync execute", $"{this.GetType().Name}"))
            {    
                var res = "";
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.Id);
                if (!string.IsNullOrEmpty(deviceInfoDO?.Code) )
                {
                    if (!string.IsNullOrEmpty(deviceInfoDO?.DeviceAttributes))
                    {
                        res = deviceInfoDO.DeviceAttributes;
                    }
                    if (!string.IsNullOrEmpty(deviceInfoDO?.DeviceConsumableAttributes))
                    {
                        if (string.IsNullOrEmpty(res))
                        {
                            res = deviceInfoDO.DeviceConsumableAttributes;
                        }
                        else
                        {
                            var dataObj = JObject.Parse(res);
                            if (dataObj != null)
                            {
                                var consumableObj = JObject.Parse(deviceInfoDO.DeviceConsumableAttributes);
                                var keyItem = "consumable_property";
                                if (dataObj.ContainsKey(keyItem))
                                {
                                    dataObj[keyItem] = consumableObj[keyItem];
                                }
                                else
                                {
                                    dataObj.Add(keyItem, consumableObj[keyItem]);
                                }
                                res = JsonConvert.SerializeObject(dataObj);
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 获取IOT设备详细数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<IOTDeviceDetailResult> GetIOTUserDeviceDetailDBAsync(GetIOTDeviceDetailRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetIOTUserDeviceDetailDBAsync execute", $"{this.GetType().Name}"))
            { 
                var result = new IOTDeviceDetailResult();
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceId);
                if (string.IsNullOrEmpty(deviceInfoDO?.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.DeviceNotExist, "Device not exist");
                }
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserCode == request.UserCode && c.DeviceCode == deviceInfoDO.Code);
                var userDeviceInfo = await _userDeviceExtendInfoDBRepository.FindOneAsync(filter);
                var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                // if (!string.IsNullOrEmpty(userDeviceInfo?.Code))
                // {
                //     request.Mode = (ModeType)userDeviceInfo.ModeType;
                // }
                // else
                // {
                //     var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == deviceInfoDO.Code && c.ReportType == ReportType.Day);
                //     var findReportList = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                //     if (findReportList?.Count > 0)
                //     {
                //         var findReportInfo = findReportList.OrderByDescending(c => c.Timestamp).FirstOrDefault();
                //         request.Mode = findReportInfo.Mode;
                //     }
                // }
                if (deviceInfoDO.DeviceType == "Breath")
                {
                    result.ModeType = (int)request.Mode;
                    var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == deviceInfoDO.Code && c.Mode == request.Mode);
                    if (request.Date != DateTime.MinValue)
                    {
                        var startTime = Convert.ToDateTime(request.Date.ToString("yyyy-MM-dd"));
                        var endTime = Convert.ToDateTime(request.Date.AddDays(1).ToString("yyyy-MM-dd"));
                        var timeFilter = deviceReportBuilder.Where(c => c.Timestamp >= startTime && c.Timestamp < endTime);
                        deviceReportFilter = deviceReportFilter & timeFilter;
                    }
                    DeviceReportEntity findReportEntity = null;
                    var findList = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                    if (findList?.Count > 0)
                    {
                        findReportEntity = findList.OrderByDescending(c => c.UpdateTime).FirstOrDefault();
                    }
                    if (!string.IsNullOrEmpty(findReportEntity?.Code))
                    {
                        result.Id = deviceInfoDO.Code;
                        //使用时间
                        if (findReportEntity.ReportAttributes?.Count > 0)
                        {
                            var useHourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                            decimal totalUseHour = 0;
                            if (!string.IsNullOrEmpty(useHourStr))
                            {
                                totalUseHour = decimal.Round(Decimal.Parse(useHourStr), 2);
                            }
                            var useHoutScore = GetUseHourScore(totalUseHour);
                            //漏气量
                            var leak95Str = findReportEntity.ReportAttributes.Find(c => c.Key == "leak_95")?.Value ?? string.Empty;
                            int totalLeak95 = 0;
                            if (!string.IsNullOrEmpty(leak95Str))
                            {
                                totalLeak95 = Convert.ToInt32(leak95Str);
                            }
                            var leakScore = GetLeakScore(totalLeak95);
                            //Ahi
                            var ahiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                            decimal totalAhi = 0;
                            if (!string.IsNullOrEmpty(ahiStr))
                            {
                                totalAhi = decimal.Round(Decimal.Parse(ahiStr), 2);
                            }
                            var ahiScore = GetAhiScore(totalAhi);
                            result.UseHour = totalUseHour;
                            result.LeakLevel = leakScore.level;
                            result.Ahi = totalAhi;
                            result.TotalScore = GetReportScore(findReportEntity, totalUseHour);
                        }
                        result.ModeType = (int)findReportEntity.Mode;
                        result.UpdatedTime = findReportEntity.UpdateTime;
                        if (!string.IsNullOrEmpty(deviceInfoDO.DeviceConsumableAttributes))
                        {
                            var deviceAttributesObject = JObject.Parse(deviceInfoDO.DeviceConsumableAttributes);                       
                            if (deviceAttributesObject != null)
                            {
                                if (deviceAttributesObject.ContainsKey("consumable_property"))
                                {   
                                    var secondObj = JObject.Parse(deviceAttributesObject["consumable_property"].ToString());
                                    if (secondObj != null)
                                    {
                                        if (secondObj.ContainsKey("cartridge_life"))
                                        {
                                            result.Cartridge = secondObj["cartridge_life"].ToString();
                                        }
                                        if (secondObj.ContainsKey("cartridge_life"))
                                        {
                                            result.Mask = secondObj["cartridge_life"].ToString();
                                        }
                                        if (secondObj.ContainsKey("tube_type"))
                                        {
                                            result.Pip = secondObj["tube_type"].ToString();
                                        }
                                    }
                                }
                                // else if (deviceAttributesObject.ContainsKey("treat_property"))
                                // { 
                                //     var secondObj = JObject.Parse(deviceAttributesObject["treat_property"].ToString());
                                //     if (secondObj != null)
                                //     {
                                //         var keys = Enum.GetNames(typeof(ModeType));
                                //         foreach(var key in keys)
                                //         {
                                //             var keyItem = key.ToLower();
                                //             if (secondObj.ContainsKey(keyItem))
                                //             {   
                                //                 var thirdObj = JObject.Parse(secondObj[keyItem].ToString());
                                //                 if (thirdObj.ContainsKey("pip_heat"))
                                //                 { 
                                //                     result.Pip = thirdObj["pip_heat"].ToString();
                                //                 }
                                //             }
                                //         }
                                //     }
                                // }
                            }
                        }
                    }
                }
                else if (deviceInfoDO.DeviceType == "Oxygen")
                {
                    if (!string.IsNullOrEmpty(deviceInfoDO.DeviceConsumableAttributes))
                    {
                        var deviceAttributesObject = JObject.Parse(deviceInfoDO.DeviceConsumableAttributes);                       
                        if (deviceAttributesObject != null)
                        {
                            result.Id = deviceInfoDO.Code;
                            var concAttrValue = deviceAttributesObject["consumable_property"]["oxygen_conc"].ToString();
                            if (Int64.TryParse(concAttrValue, out long conc))
                            {
                                result.OxygenConc = conc;
                            }
                            var treatPwrAttrValue = deviceAttributesObject["consumable_property"]["treat_pwr"].ToString();
                            if (Int64.TryParse(treatPwrAttrValue, out long treatPwr))
                            {
                                if (treatPwr == 90) 
                                { 
                                    result.TreatPwr = true;
                                }
                                else if (treatPwr == 165) 
                                {
                                    result.TreatPwr = false;
                                }
                                else 
                                {
                                    result.TreatPwr = null;
                                }
                            }
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 获取IOT设备阶段报告
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<string> GetIOTDeviceReportDBAsync(GetIOTDeviceReportRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetIOTDeviceReportDBAsync execute", $"{this.GetType().Name}"))
            { 
                var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == request.DeviceId && c.ReportType == request.ReportType && c.Mode == request.Mode);
                var findReportEntity = await _deviceReportDBRepository.FindOneAsync(deviceReportFilter);
                if (!string.IsNullOrEmpty(findReportEntity?.Code) && findReportEntity?.ReportAttributes?.Count > 0)
                {
                    try
                    {
                        var tempDic = new Dictionary<string, object>();          
                        tempDic.Add("Code", findReportEntity.Code);
                        tempDic.Add("DeviceType", findReportEntity.DeviceType);
                        tempDic.Add("ReportType", (int)findReportEntity.ReportType);
                        tempDic.Add("DeviceCode", findReportEntity.DeviceCode);
                        tempDic.Add("Mode", (int)findReportEntity.Mode);
                        tempDic.Add("Timestamp", findReportEntity.Timestamp);
                        tempDic.Add("IsReport", findReportEntity.IsReport.Value);                    
                        foreach(var childItem in findReportEntity.ReportAttributes)
                        {
                            tempDic.Add(childItem.Key, childItem.Value);
                        }               
                        dynamic obj = new System.Dynamic.ExpandoObject();
                        foreach (KeyValuePair<string, object> keyValuePair in tempDic)
                        {
                            ((IDictionary<string, object>)obj).Add(keyValuePair.Key, keyValuePair.Value);
                        }
                        var json = JsonConvert.SerializeObject(obj);
                        return json;
                    }
                    catch(Exception ex)
                    {
                        Logger.WriteLineWarn($"DeviceInfoDBService GetIOTDeviceReportDBAsync error, DeviceCode: {request.DeviceId}, errorMsg: {ex}");
                        return string.Empty;
                    }
                }
                else
                {
                    return string.Empty;
                }
            }
        }

        /// <summary>
        /// 获取IOT设备阶段时间内有使用记录日期
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<IList<DateTime>> GetDeviceUseDatesDBAsync(GetIOTDeviceUseDateRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDeviceUseDatesDBAsync execute", $"{this.GetType().Name}"))
            {
                var datas = new List<DateTime>();
                if (request.Start >= Convert.ToDateTime("2000-1-1") && request.End >= Convert.ToDateTime("2000-1-1"))
                {
                    var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                    var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == request.DeviceId && c.Mode == request.Mode && c.Timestamp >= request.Start && c.Timestamp <= request.End);
                    var findReportList = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                    if (findReportList?.Count > 0)
                    {
                        foreach(var item in findReportList)
                        {
                            item.Timestamp = UTCToBeijing(item.Timestamp);
                        }
                        datas = findReportList?.Select(c => c.Timestamp.Date)?.Distinct()?.ToList() ?? new List<DateTime>();
                    }
                }
                return datas;
            }
        }
        
        /// <summary>
        /// utc转bj时间
        /// </summary>
        private DateTime UTCToBeijing(DateTime utcTime)
        {
            // 获取中国标准时间 (CST) 的时区信息
            TimeZoneInfo cstZone = TimeZoneInfo.FindSystemTimeZoneById("China Standard Time");
            // 将UTC时间转换为北京时间
            DateTime beijingTime = TimeZoneInfo.ConvertTimeFromUtc(utcTime, cstZone);
            return beijingTime;
        }

        /// <summary>
        /// 获取IOT设备使用模式数据源
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<IList<ModeType>> GetDeviceModeTypesDBAsync(GetIOTDevicBaseRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDeviceModeTypesDBAsync execute", $"{this.GetType().Name}"))
            {
                var modeTypes = new List<ModeType>();                
                var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == request.DeviceId);
                var findReportList = await _deviceReportDBRepository.FindAllAsync(deviceReportFilter);
                if (findReportList?.Count > 0)
                {
                    modeTypes = findReportList.Select(c => c.Mode).Distinct().ToList();
                }
                return modeTypes;
            }
        }

        /// <summary>
        /// 设备报告上报接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        public async Task<bool> DeviceEventPostDBAsync(IOTReportAttributeEscalationRequest request) 
        {
            using (new Performance(_performanceThreshold, "{0} DeviceEventPostDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var deviceInfoDO = await _deviceInfoDBRepository.FindOneAsync("Code", request.Id);
                if (!string.IsNullOrEmpty(deviceInfoDO?.Code)) 
                {
                    ModeType mode = ModeType.APAP;
                    DateTime timestamp = DateTime.MinValue;

                    var deviceReportAttributes = new List<WingInterfaceLibrary.DTO.Common.DataItemDTO>();
                    if (!string.IsNullOrEmpty(request.ReportAttributesJson))         
                    {
                        var deviceReportAttributesObject = JObject.Parse(request.ReportAttributesJson);
                        var jsonContent = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, $"Resource\\IOT", "DeviceReport.json"));
                        JObject json = JObject.Parse(jsonContent);
                        JArray array = null;
                        if (deviceInfoDO.DeviceType == "Breath")
                        {
                            array = (JArray)json["BreathReportInfo"];
                        }
                        else 
                        {
                            array = (JArray)json["OxygenReportInfo"];
                        }
                        foreach (JToken jObject in array)
                        {
                            var attributeModel = new WingInterfaceLibrary.DTO.Common.DataItemDTO();
                            //赋值属性
                            attributeModel.Key = jObject["Name"].ToString();//获取字符串中id值
                            if (deviceReportAttributesObject.ContainsKey("user_report") && request.ReportAttributesJson.Contains(attributeModel.Key))
                            {
                                attributeModel.Value = deviceReportAttributesObject["user_report"][attributeModel.Key].ToString();//获取字符串中id值
                                if (attributeModel.Key == "report_mode")
                                {
                                    mode = (ModeType)Enum.ToObject(typeof(ModeType), Convert.ToInt32(attributeModel.Value));
                                }

                                if (attributeModel.Key == "report_timestamp")
                                {
                                    int timestampValue = Convert.ToInt32(attributeModel.Value);
                                    DateTime baseTime = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
                                    timestamp = baseTime.AddSeconds(timestampValue).ToLocalTime();
                                }
                                deviceReportAttributes.Add(attributeModel);
                            }
                        }
                    }
                    var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                    var deviceReportFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.DeviceCode == request.Id 
                                                && c.ReportType == ReportType.Day && c.Mode == mode && c.Timestamp == timestamp);
                    var deviceReportInfoDO = await _deviceReportDBRepository.FindOneAsync(deviceReportFilter);
                    if (!string.IsNullOrEmpty(deviceReportInfoDO?.Code))
                    {
                        //存在，修改
                        if (deviceReportAttributes?.Count > 0)
                        {
                            deviceReportInfoDO.IsReport = true;
                            deviceReportInfoDO.ReportAttributes = deviceReportAttributes;
                        }
                        else
                        {
                            deviceReportInfoDO.IsReport = false;
                            deviceReportInfoDO.ReportAttributes = new List<WingInterfaceLibrary.DTO.Common.DataItemDTO>();
                        }
                        res = await _deviceReportDBRepository.UpdateOneAsync("Code", deviceReportInfoDO.Code, deviceReportInfoDO, true);
                    }
                    else
                    {
                        var deviceType = "";
                        if (request.Type == 0)
                         {
                            deviceType = "Breath";
                        }
                        else 
                        {
                            deviceType = "Oxygen";
                        }
                        //新增
                        var model = new DeviceReportEntity()
                        {
                            Code = Guid.NewGuid().ToString("N"),
                            DeviceType = deviceType,
                            ReportType = ReportType.Day,
                            DeviceCode = request.Id, 
                            Mode = mode, 
                            Timestamp = timestamp
                        };
                        if (deviceReportAttributes?.Count > 0)
                        {
                            model.IsReport = true;
                            model.ReportAttributes = deviceReportAttributes;
                        }
                        else
                        {
                            model.IsReport = false;
                            model.ReportAttributes = new List<WingInterfaceLibrary.DTO.Common.DataItemDTO>();
                        }
                        var result = await _deviceReportDBRepository.InsertOneAsync(model);
                        if (!string.IsNullOrEmpty(result))
                        {
                            res = true;
                        }
                        else 
                        {
                            res = false;
                        }
                    }
                    
                }
                return res;
            }
        }

    
        #region IOT设备通用字典
        /// <summary>
        /// 保存通用字典数据
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SaveCommonDictionaryInfoDBAsync(SaveCommonDictionaryRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveCommonDictionaryInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var builder = Builders<CommonDictionaryEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Key == request.Key);
                var keyInfo = await _commonDictionaryDBRepository.FindOneAsync(filter);
                if (string.IsNullOrEmpty(request.Code))
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code))
                    {
                        return res;
                    }
                    //新增
                    var model = request.MappingTo<CommonDictionaryEntity>();
                    model.Code = Guid.NewGuid().ToString("N");
                    var insertIdentity = await _commonDictionaryDBRepository.InsertOneAsync(model);
                    if (!string.IsNullOrEmpty(insertIdentity))
                    {
                        res = true;
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code) && keyInfo.Code != request.Code)
                    {
                        return res;
                    }
                    var commonDicInfo = await _commonDictionaryDBRepository.FindOneAsync("Code", request.Code);
                    if (!string.IsNullOrEmpty(commonDicInfo?.Code))
                    {
                        //修改
                        commonDicInfo.Name = request.Name;
                        commonDicInfo.Description = request.Description;
                        commonDicInfo.ParentCode = request.ParentCode;
                        commonDicInfo.UpdateTime = DateTime.UtcNow;
                        res = await _commonDictionaryDBRepository.UpdateOneAsync("Code", commonDicInfo.Code, commonDicInfo, true);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 移除字典
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> RemoveCommonDictionaryInfoDBAsync(CommonDictionaryRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveCommonDictionaryInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var commonDicInfo = await _commonDictionaryDBRepository.FindOneAsync("Code", request.Code);
                if (!string.IsNullOrEmpty(commonDicInfo?.Code))
                {
                    res = await _commonDictionaryDBRepository.DeleteOneAsync("Code", commonDicInfo.Code);
                }
                return res;
            }
        }

        /// <summary>
        /// 获取字典
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<IList<CommonDictionaryDTO>> GetCommonDictionaryInfosDBAsync(CommonDictionaryRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetCommonDictionaryInfosDBAsync execute", $"{this.GetType().Name}"))
            {
                var dataList = new List<CommonDictionaryDTO>();
                var builder = Builders<CommonDictionaryEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if ((int)request.Type > 0)
                {
                    filter = filter & builder.Where(c => c.Type == request.Type);
                }
                if (!string.IsNullOrEmpty(request.Code))
                {
                    filter = filter & builder.Where(c => c.Code == request.Code);
                }
                if (!string.IsNullOrEmpty(request.Key))
                {
                    filter = filter & builder.Where(c => c.Key == request.Key);
                }
                if (!string.IsNullOrEmpty(request.Name))
                {
                    filter = filter & builder.Where(c => c.Name == request.Name);
                }
                var list = await _commonDictionaryDBRepository.FindAllAsync(filter);
                if (list?.Count > 0)
                {
                    dataList = list.MappingTo<List<CommonDictionaryDTO>>();
                }
                return dataList;
            }
        }

        /// <summary>
        /// 分页查询设备字典
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<CommonDictionaryDTO>> FindCommonDictionaryPagesDBAsync(GetCommonDictionaryPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindCommonDictionaryPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<CommonDictionaryEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if ((int)request.Type > 0)
                {
                    filter = filter & builder.Where(c => c.Type == request.Type);
                }
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var nameFilter = builder.Regex(c => c.Name, new BsonRegularExpression(keyword, "i"));
                    var keyFilter = builder.Regex(c => c.Key, new BsonRegularExpression(keyword, "i"));
                    var DescFilter = builder.Regex(c => c.Description, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (nameFilter | keyFilter | DescFilter);
                }
                var sort = Builders<CommonDictionaryEntity>.Sort.Descending(v => v.CreateTime);
                var dbPage = new DBPageRequest<CommonDictionaryEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _commonDictionaryDBRepository.GetPages(dbPage);
                return PageResultConvert<CommonDictionaryEntity, CommonDictionaryDTO>(pageResult);
            }
        }

        /// <summary>
        /// 保存设备绑定用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SaveIOTDeviceBindUserInfoDBAsync(IOTDeviceBindUserDataRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveIOTDeviceBindUserInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.UserCode == request.UserCode && c.DeviceCode == request.DeviceCode);
                var keyInfo = await _userDeviceExtendInfoDBRepository.FindOneAsync(filter);
                if (string.IsNullOrEmpty(request.DeviceBindUserCode))
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code))
                    {
                        return res;
                    }
                    var deviceInfo = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                    //新增
                    var model = request.MappingTo<UserDeviceExtendInfoEntity>();
                    model.Code = Guid.NewGuid().ToString("N");
                    model.DeviceMAC = deviceInfo?.SystemVersion ?? string.Empty;
                    model.DeviceSN = deviceInfo?.DeviceSoftwareVersion ?? string.Empty;
                    var insertIdentity = await _userDeviceExtendInfoDBRepository.InsertOneAsync(model);
                    if (!string.IsNullOrEmpty(insertIdentity))
                    {
                        res = true;
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code) && keyInfo.Code != request.DeviceBindUserCode)
                    {
                        return res;
                    }
                    var deviceUserInfo = await _userDeviceExtendInfoDBRepository.FindOneAsync("Code", request.DeviceBindUserCode);
                    if (!string.IsNullOrEmpty(deviceUserInfo?.Code))
                    {
                        var deviceInfo = await _deviceInfoDBRepository.FindOneAsync("Code", request.DeviceCode);
                        //修改
                        deviceUserInfo.UserCode = request.UserCode;
                        deviceUserInfo.DeviceCode = request.DeviceCode;
                        deviceUserInfo.DeviceMAC = deviceInfo?.SystemVersion ?? string.Empty;
                        deviceUserInfo.DeviceSN = deviceInfo?.DeviceSoftwareVersion ?? string.Empty;
                        deviceUserInfo.UpdateTime = DateTime.UtcNow;
                        res = await _userDeviceExtendInfoDBRepository.UpdateOneAsync("Code", deviceUserInfo.Code, deviceUserInfo, true);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 分页查询设备用户数据列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<IOTDeviceUserInfoDTO>> FindIOTDeviceUseListPageDBAsync(GetIOTDeviceBindUserDataPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindIOTDeviceUseListPageDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.DeviceSN))
                {
                    var userInfo = await FindIOTDeviceUserInfoDBAsync("", request.DeviceSN);
                    if (!string.IsNullOrEmpty(userInfo?.Code))
                    {
                        filter = filter & builder.Where(c => c.UserCode == userInfo.Code || c.DeviceSN.Contains(request.DeviceSN));
                    }
                    else
                    {
                        filter = filter & builder.Where(c => c.DeviceSN.Contains(request.DeviceSN));
                    }
                }
                var sort = Builders<UserDeviceExtendInfoEntity>.Sort.Descending(v => v.UpdateTime);
                var dbPage = new DBPageRequest<UserDeviceExtendInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _userDeviceExtendInfoDBRepository.GetPages(dbPage);
                var res = PageResultConvert<UserDeviceExtendInfoEntity, IOTDeviceUserInfoDTO>(pageResult);
                if (res.PageData?.Count > 0)
                {
                    var userCodes = res.PageData.Select(c => c.UserCode).Distinct().ToList();
                    if (userCodes?.Count > 0)
                    {
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", userCodes);
                        var userInfos = await _userDBRepository.FindInAsync(dic);
                        foreach(var item in res.PageData)
                        {
                            var userInfo = userInfos.FirstOrDefault(c => c.Code == item.UserCode) ?? new UserInfoDO();
                            item.Name = (userInfo.OpenId == "-") ? userInfo.UserName : userInfo.Phone;
                            if (userInfo.BirthDate != null && userInfo.BirthDate > Convert.ToDateTime("1900-1-1"))
                            {
                                var newBirthDate = UTCToBeijing(userInfo.BirthDate.Value);
                                item.Age = CalculateAge(newBirthDate);
                                item.Birthday = newBirthDate;
                            }
                            else if (!string.IsNullOrEmpty(userInfo.Birthday))
                            {
                                item.Birthday = Convert.ToDateTime(userInfo.Birthday);
                                item.Age = CalculateAge(item.Birthday);
                            }
                            else
                            {
                                item.Age = 0;
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 获取设备用户信息详情接口
        /// </summary>
        /// <param name="code">请求对象</param>
        /// <param name="name">请求对象</param>
        /// <returns></returns>
        public async Task<IOTDeviceUserDTO> FindIOTDeviceUserInfoDBAsync(string code, string name)
        {
            using (new Performance(_performanceThreshold, "{0} FindIOTDeviceUserDTOPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new IOTDeviceUserDTO();
                var dic = new Dictionary<string, string>();
                if (!string.IsNullOrEmpty(code))
                {
                    dic.Add("Code", code);
                }
                if (!string.IsNullOrEmpty(name))
                {
                    dic.Add("UserName", name);
                }
                var dataList = await _userDBRepository.FindListAsync(dic);
                if (dataList?.Count > 0)
                {
                    var dataItem = dataList.FirstOrDefault() ?? new UserInfoDO();  
                    var newDate = dataItem.BirthDate == null ? DateTime.MinValue : dataItem.BirthDate.Value;                                     
                    var model = new IOTDeviceUserDTO()
                    {
                        Code = dataItem.UserCode,
                        Name = dataItem.UserCode,
                        Gender = dataItem.Gender,
                        Birthday = UTCToBeijing(newDate),
                        Height = Convert.ToInt32(dataItem.Height),
                        Weight = Convert.ToInt32(dataItem.Weight),
                        Neck = Convert.ToInt32(dataItem.NeckGirth),
                        BMI = dataItem.BMI,
                        HeadImageUrl = dataItem.HeadImageToken,
                        FullName = dataItem.FullName,
                        NickName = dataItem.NickName,
                        OpenId = dataItem.OpenId
                    };
                    if (dataItem.Gender == 1)
                    {
                        model.Sex = "男";
                    }
                    else if (dataItem.Gender == 2)
                    {
                        model.Sex = "女";
                    }
                    else 
                    {
                        model.Sex = "未知";
                    }                    
                }
                return result;
            }
        }

        /// <summary>
        /// 保存通用设备用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SaveIOTDeviceUserInfoDBAsync(SaveIOTDeviceUserDataRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveIOTDeviceUserInfoDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                // 
                var keyInfo = await _userDBRepository.FindOneAsync("UserName", request.Name);
                if (string.IsNullOrEmpty(request.DeviceUserCode))
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code))
                    {
                        return res;
                    }
                    //新增
                    var userDO = new UserInfoDO();
                    userDO.SecretPassword = "Symmetry_mk9xah8N";
                    userDO.Code = Guid.NewGuid().ToString("N");
                    userDO.UserName = request.Name;
                    userDO.Gender = request.Gender;
                    userDO.Birthday = request.Birthday.ToString("yyyy-MM-dd");
                    userDO.BirthDate = request.Birthday;
                    userDO.Height = request.Height;
                    userDO.Weight = (float)request.Weight;
                    userDO.NeckGirth = (float)request.Neck;
                    userDO.OpenId = "-";
                    userDO.BMI = (float)Math.Round(request.BMI, 2);
                    var insertIdentity = await _userDBRepository.InsertOneAsync(userDO, new Dictionary<string, string>(), string.Empty);
                    if (!string.IsNullOrEmpty(insertIdentity))
                    {
                        res = true;
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(keyInfo?.Code) && keyInfo.Code != request.DeviceUserCode)
                    {
                        return res;
                    }
                    var deviceUserInfo = await _userDBRepository.FindOneAsync("Code", request.DeviceUserCode);
                    if (!string.IsNullOrEmpty(deviceUserInfo?.Code))
                    {
                        //修改
                        deviceUserInfo.UserName = request.Name;
                        deviceUserInfo.Birthday = request.Birthday.ToString("yyyy-MM-dd");
                        deviceUserInfo.BirthDate = request.Birthday;
                        deviceUserInfo.BMI = (float)Math.Round(request.BMI, 2);
                        deviceUserInfo.Gender = request.Gender;
                        deviceUserInfo.Height = request.Height;
                        deviceUserInfo.Weight = (float)request.Weight;
                        deviceUserInfo.NeckGirth = (float)request.Neck;
                        deviceUserInfo.UpdateTime = DateTime.UtcNow;
                        res = await _userDBRepository.UpdateOneAsync("Code", deviceUserInfo.Code, deviceUserInfo, string.Empty, true);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 分页查询设备用户信息
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<PageResult<IOTDeviceUserDTO>> FindIOTDeviceUserPagesDBAsync(GetIOTDeviceUserDataPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} FindIOTDeviceUserDTOPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                PageResult<IOTDeviceUserDTO> convertResult = new();
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                MongoCustomerFilter<MongoTable> customerFilter = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                var items = new List<string> { "UserName", "FullName" };
                if (!string.IsNullOrWhiteSpace(request.Name))
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "UserName"));
                    queryItemsFilter.AppendRegex(item => item.Value, request.Name, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());                    
                }
                if (queryItemsFilter.HasFilter)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                    customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                var tempEmptyList = new List<string>() { null, "" };
                customerFilter.AppendAnyNotIn("BaseData.OpenId", tempEmptyList);
                if (request.Gender > 0)
                {
                    customerFilter.AppendEq(("BaseData.Gender", request.Gender));
                }
                if (request.BeginAge > 0)
                {
                    var beginBirthDay = CalculateBirthDate(request.BeginAge);
                    customerFilter.AppendLt("BaseData.BirthDate", beginBirthDay.AddDays(1));
                }
                if (request.EndAge > 0)
                {
                    var endBirthDay = CalculateBirthDate(request.EndAge);
                    customerFilter.AppendGte("BaseData.BirthDate", endBirthDay);
                }
                if (request.Height > 0)
                {
                    customerFilter.AppendEq(("BaseData.Height", request.Height));
                }
                if (request.Weight > 0)
                {
                    customerFilter.AppendEq(("BaseData.Weight", request.Weight));
                }
                if (request.Neck > 0)
                {
                    customerFilter.AppendEq(("BaseData.NeckGirth", request.Neck));
                }
                if (request.BeginBMI > 0)
                {
                    customerFilter.AppendGte("BaseData.BMI", request.BeginBMI);
                }
                if (request.EndBMI > 0)
                {
                    customerFilter.AppendLte("BaseData.BMI", request.EndBMI);
                }
                var filter = customerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var result = await _userDBRepository.GetPages(dbPage);
                if (result.DataCount > 0)
                {
                    convertResult.PageIndex = result.CurrentPage;
                    convertResult.PageSize = result.PageSize;
                    convertResult.TotalCount = result.DataCount;
                    if (result.PageData?.Count > 0)
                    {
                        var tempList = new List<IOTDeviceUserDTO>();
                        foreach(var dataItem in result.PageData)
                        {
                            var newData = dataItem.BirthDate == null ? DateTime.MinValue : dataItem.BirthDate.Value;
                            var model = new IOTDeviceUserDTO()
                            {
                                Code = dataItem.UserCode,
                                Name = (dataItem.OpenId == "-") ? dataItem.UserName : dataItem.Phone,
                                Gender = dataItem.Gender,
                                Birthday = UTCToBeijing(newData),
                                Height = Convert.ToInt32(dataItem.Height),
                                Weight = Convert.ToInt32(dataItem.Weight),
                                Neck = Convert.ToInt32(dataItem.NeckGirth),
                                BMI = Math.Round(dataItem.BMI, 2),
                                HeadImageUrl = dataItem.HeadImageToken,
                                FullName = dataItem.FullName,
                                NickName = dataItem.NickName,
                                OpenId = dataItem.OpenId
                            };
                            if (dataItem.Gender == 1)
                            {
                                model.Sex = "男";
                            }
                            else if (dataItem.Gender == 2)
                            {
                                model.Sex = "女";
                            }
                            else 
                            {
                                model.Sex = "未知";
                            }
                            tempList.Add(model);
                        }
                        convertResult.PageData = tempList;
                    }
                }
                return convertResult;
            }
        }

        /// <summary>
        /// 根据年龄计算生日
        /// </summary>
        private DateTime CalculateBirthDate(int age)
        {
            DateTime today = DateTime.Today;
            int year = today.Year - age;
            DateTime birthday = new DateTime(year, today.Month, today.Day);

            // 如果生日已经过了，则将年份减去1
            if (birthday > today)
            {
                birthday = birthday.AddYears(-1);
            }

            return birthday;
        }


        /// <summary>
        /// 分页查询呼吸机设备原始数据列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<IList<IOTDeviceUserListDTO>> GetIOTDeviceDataListDBAsync(GetIOTDeviceFileDataRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetIOTDeviceDataListDBAsync execute", $"{this.GetType().Name}"))
            {
                var dataList = new List<IOTDeviceUserListDTO>();
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.DeviceSN))
                {
                    filter = filter & builder.Where(c => c.DeviceSN == request.DeviceSN);
                    var deviceUseRecordList = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                    if (deviceUseRecordList?.Count > 0)
                    {
                        var findDeviceUseInfos = deviceUseRecordList.OrderBy(c => c.CreateTime).ToList();
                        var tempData = await FilterDeviceUserFileData(findDeviceUseInfos, request);
                        if (tempData != null)
                        {
                            dataList.Add(tempData);
                        }
                    }
                }
                else
                {
                    filter = filter & builder.Where(c => c.IsDelete == false && c.UpdateTime >= DateTime.UtcNow.AddMonths(-3) && c.UpdateTime < DateTime.UtcNow);
                    var deviceUseRecordList = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                    var userInfos = deviceUseRecordList.GroupBy(c => c.DeviceCode).Select(c => new { DeviceCode = c.Key, IOTDeviceUseRecords = c.OrderByDescending(p => p.CreateTime).ToList() }).ToList();
                    foreach(var item in userInfos)
                    {
                        var tempData = await FilterDeviceUserFileData(item.IOTDeviceUseRecords, request);                        
                        if (tempData != null)
                        {
                            dataList.Add(tempData);
                        }
                    }
                    // var tempDeviceUseRecordList = userInfos.Select(c => c.IOTDeviceUseRecord).ToList();
                    // if (tempDeviceUseRecordList?.Count > 0)
                    // {
                    //     foreach(var item in tempDeviceUseRecordList)
                    //     {
                    //         var tempDataList = FilterDeviceUserFileData(item, request);
                    //         if (tempDataList?.Count > 0)
                    //         {
                    //             dataList.AddRange(tempDataList);
                    //         }
                    //     }
                    // }
                }
                return dataList;
            }
        }

        /// <summary>
        /// 处理数据
        /// </summary>
        private async Task<IOTDeviceUserListDTO> FilterDeviceUserFileData(List<UserDeviceExtendInfoEntity> findDeviceUseInfos, GetIOTDeviceFileDataRequest request)
        {
            var firstDeviceInfo = findDeviceUseInfos.FirstOrDefault() ?? new UserDeviceExtendInfoEntity();            
            var dataInfo = new IOTDeviceUserListDTO()
            {
                Code = firstDeviceInfo.Code,
                DeviceCode = firstDeviceInfo.DeviceCode,
                DeviceMAC = firstDeviceInfo.DeviceMAC,
                DeviceSN = firstDeviceInfo.DeviceSN,
                UserDataList = new List<IOTDeviceUserExtendInfoDTO>()
            };
            var userCodes = findDeviceUseInfos.Select(c => c.UserCode).Distinct().ToList();            
            var dic = new Dictionary<string, List<string>>();
            dic.Add("Code", userCodes);
            var deviceUserList = await _userDBRepository.FindInAsync(dic);
            foreach(var findDeviceUseInfo in findDeviceUseInfos) 
            {
                if (!string.IsNullOrEmpty(findDeviceUseInfo?.Code))
                {
                    var curUserInfo = deviceUserList.FirstOrDefault(cx => cx.Code == findDeviceUseInfo.UserCode) ?? new UserInfoDO();
                    var userInfo = ConvertUserToIOTDeviceUserExtendInfoDTO(curUserInfo);//curUserInfo.MappingTo<IOTDeviceUserExtendInfoDTO>();
                    userInfo.FolderList = new List<IOTDeviceDataFolderDTO>();  
                    if (findDeviceUseInfo.FolderList?.Count > 0)
                    {
                        foreach(var item in findDeviceUseInfo.FolderList)
                        {
                            var foldInfo = new IOTDeviceDataFolderDTO() {
                                FolderName = item.FolderName,
                                CreateTime = item.CreateTime
                            };
                            var existFileInfos = new List<IOTDeviceFileDataRecordEntity>();
                            //治疗时间的维度
                            if (request.BeginTreatTime > DateTime.MinValue && request.EndTreatTime > DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.TreatTime >= request.BeginTreatTime && c.TreatTime <= request.EndTreatTime && c.IsCompleted)?.ToList();
                            }
                            else if (request.BeginTreatTime > DateTime.MinValue && request.EndTreatTime == DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.TreatTime >= request.BeginTreatTime && c.IsCompleted)?.ToList();
                            }
                            else if (request.BeginTreatTime == DateTime.MinValue && request.EndTreatTime > DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.TreatTime <= request.EndTreatTime && c.IsCompleted)?.ToList();
                            }
                            //创建时间的维度
                            else if (request.CreateTimeFrom > DateTime.MinValue && request.CreateTimeTo > DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.CreateTime >= request.CreateTimeFrom && c.CreateTime <= request.CreateTimeTo && c.IsCompleted)?.ToList();
                            }
                            else if (request.CreateTimeFrom > DateTime.MinValue && request.CreateTimeTo == DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.CreateTime >= request.CreateTimeFrom && c.IsCompleted)?.ToList();
                            }
                            else if (request.CreateTimeFrom == DateTime.MinValue && request.CreateTimeTo > DateTime.MinValue)
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.CreateTime <= request.CreateTimeTo && c.IsCompleted)?.ToList();
                            }
                            else
                            {
                                existFileInfos = item.DataFileList?.Where(c => c.IsCompleted)?.ToList();
                            }
                            if (existFileInfos?.Count > 0)
                            {
                                foldInfo.DataFileList = existFileInfos.MappingTo<List<IOTDeviceFileDataRecordDTO>>();
                                userInfo.FolderList.Add(foldInfo);
                            }
                        }
                    }
                    dataInfo.UserDataList.Add(userInfo);
                }
            }
            return dataInfo;
        }

        /// <summary>
        /// 保存设备用户原始数据db接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SaveIOTDeviceDataStatusDBAsync(UpdateIOTDeviceDataResult request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveIOTDeviceDataStatusDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.Code == request.UseRecordCode);
                var deviceUseRecordInfo = await _userDeviceExtendInfoDBRepository.FindOneAsync(filter);
                if (!string.IsNullOrEmpty(deviceUseRecordInfo?.Code))
                {
                    if (deviceUseRecordInfo.FolderList?.Count > 0)
                    {
                        //看下是否存在这个文件夹
                        var findDeviceUseEntity = deviceUseRecordInfo.FolderList.FirstOrDefault(c => c.Id == request.FolderCode);
                        if (!string.IsNullOrEmpty(findDeviceUseEntity?.Id))
                        {
                            if (findDeviceUseEntity?.DataFileList?.Count > 0)
                            {
                                var dataInfo = findDeviceUseEntity.DataFileList.FirstOrDefault(c => c.Id == request.DataCode);
                                if (!string.IsNullOrEmpty(dataInfo?.Id))
                                { 
                                    dataInfo.IsCompleted = true;
                                    if (!string.IsNullOrEmpty(request.FileUrl))
                                    {
                                        dataInfo.FileToken = request.FileUrl;
                                    }
                                    res = await _userDeviceExtendInfoDBRepository.UpdateOneAsync("Code", deviceUseRecordInfo.Code, deviceUseRecordInfo, true);
                                }
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 保存设备用户原始数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<UpdateIOTDeviceDataResult> SaveIOTDeviceDataDBAsync(SaveIOTDeviceDataDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveIOTDeviceDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = new UpdateIOTDeviceDataResult();
                var treatTime = request.TreatTime;
                var dataList = new List<IOTDeviceUserInfoDTO>();
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (request.IsCompleted)
                {
                    filter = filter & builder.Where(c => c.DeviceSN == request.DeviceSN);
                }
                else
                {
                    filter = filter & builder.Where(c => c.DeviceCode == request.DeviceSN);
                }
                if (!string.IsNullOrEmpty(request.UserCode))
                {
                    filter = filter & builder.Where(c => c.UserCode == request.UserCode);
                }
                var deviceUseRecordList = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                if (deviceUseRecordList?.Count > 0)
                {
                    var findDeviceUseInfo = deviceUseRecordList.OrderBy(c => c.CreateTime).FirstOrDefault();
                    if (findDeviceUseInfo.FolderList?.Count > 0)
                    {
                        //看下是否存在这个文件夹
                        var findDeviceUseEntity = findDeviceUseInfo.FolderList.FirstOrDefault(c => c.FolderName == request.FolderName);
                        if (string.IsNullOrEmpty(findDeviceUseEntity?.FolderName))
                        {
                            //没找到
                            var newFoldInfo = new IOTDeviceDataFolderEntity()
                            {
                                FolderName = request.FolderName,
                                DataFileList = new List<IOTDeviceFileDataRecordEntity>(),
                            };
                            var deviceFileInfo = new IOTDeviceFileDataRecordEntity() 
                            {
                                TreatTime = treatTime,
                                DataFileType = request.DataFileType,
                                OriginalFileName = request.OriginalFileName,
                                DataDesc = request.DataDesc,
                                FileToken = request.FileToken,
                                IsCompleted = request.IsCompleted
                            };
                            newFoldInfo.DataFileList.Add(deviceFileInfo);
                            findDeviceUseInfo.FolderList.Add(newFoldInfo);
                            res.DataCode = deviceFileInfo.Id;
                            res.FolderCode = newFoldInfo.Id;
                        }
                        else
                        {
                            res.FolderCode = findDeviceUseEntity.Id;
                            //找到文件夹
                            var findFileInfoEntity = findDeviceUseEntity.DataFileList.FirstOrDefault(c => c.OriginalFileName == request.OriginalFileName && c.TreatTime == treatTime);
                            if (string.IsNullOrEmpty(findFileInfoEntity?.OriginalFileName))
                            {
                                //没找到文件
                                var deviceFileInfo = new IOTDeviceFileDataRecordEntity() 
                                {
                                    TreatTime = treatTime,
                                    DataFileType = request.DataFileType,
                                    OriginalFileName = request.OriginalFileName,
                                    DataDesc = request.DataDesc,
                                    FileToken = request.FileToken,
                                    IsCompleted = request.IsCompleted
                                };
                                findDeviceUseEntity.DataFileList.Add(deviceFileInfo);
                                res.DataCode = deviceFileInfo.Id;
                            }
                            else
                            {
                                //找到文件
                                findFileInfoEntity.FileToken = request.FileToken;
                                res.DataCode = findFileInfoEntity.Id;
                            }
                        }
                    }
                    else
                    {
                        findDeviceUseInfo.FolderList = new List<IOTDeviceDataFolderEntity>();
                        var newFoldInfo = new IOTDeviceDataFolderEntity()
                        {
                            FolderName = request.FolderName,
                            DataFileList = new List<IOTDeviceFileDataRecordEntity>(),
                        };
                        var deviceFileInfo = new IOTDeviceFileDataRecordEntity() 
                        {
                            TreatTime = treatTime,
                            DataFileType = request.DataFileType,
                            OriginalFileName = request.OriginalFileName,
                            DataDesc = request.DataDesc,
                            FileToken = request.FileToken,
                            IsCompleted = request.IsCompleted
                        };
                        newFoldInfo.DataFileList.Add(deviceFileInfo);
                        findDeviceUseInfo.FolderList.Add(newFoldInfo);
                        res.DataCode = deviceFileInfo.Id;
                        res.FolderCode = newFoldInfo.Id;
                    }
                    findDeviceUseInfo.UpdateTime = DateTime.UtcNow;
                    res.UseRecordCode = findDeviceUseInfo.Code;
                    res.FileUrl = request.FileToken;
                    //更新数据
                    await _userDeviceExtendInfoDBRepository.UpdateOneAsync("Code", findDeviceUseInfo.Code, findDeviceUseInfo, true);
                }
                return res;
            }
        }

        /// <summary>
        /// 北京时间转UTC时间
        /// </summary>
        private DateTime BeijingToUtc(DateTime beijingTime)
        {
            TimeZoneInfo chinaZone = TimeZoneInfo.FindSystemTimeZoneById("China Standard Time");
            return TimeZoneInfo.ConvertTimeToUtc(beijingTime, chinaZone);
        }

        /// <summary>
        /// 查询呼吸机设备绑定用户信息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<List<IOTDeviceUserListDTO>> GetIOTDeviceUserListDBAsync(GetIOTDeviceUsersRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetIOTDeviceUserListDBAsync execute", $"{this.GetType().Name}"))
            {
                var dataList = new List<IOTDeviceUserListDTO>();
                var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (request.DeviceSNList?.Count > 0)
                {
                    filter = filter & builder.Where(c => request.DeviceSNList.Contains(c.DeviceSN));            
                    var deviceUseRecordList = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                    if (deviceUseRecordList?.Count > 0)
                    {
                        var userInfos = deviceUseRecordList.GroupBy(c => c.DeviceCode).Select(c => new { DeviceCode = c.Key, IOTDeviceUseRecords = c.OrderByDescending(p => p.CreateTime).ToList() }).ToList();
                        foreach(var item in userInfos)
                        {
                            var firstEntity = item.IOTDeviceUseRecords.FirstOrDefault() ?? new UserDeviceExtendInfoEntity();
                            var model = new IOTDeviceUserListDTO();
                            model.DeviceCode = item.DeviceCode;
                            model.DeviceSN = firstEntity.DeviceSN;                            
                            model.DeviceMAC = firstEntity.DeviceMAC;                        
                            model.Code = firstEntity.Code;   
                            model.UserDataList = new List<IOTDeviceUserExtendInfoDTO>();  
                            var userCodes = item.IOTDeviceUseRecords.Select(c => c.UserCode).Distinct().ToList();
                            var dic = new Dictionary<string, List<string>>();
                            dic.Add("Code", userCodes);
                            var deviceUserList = await _userDBRepository.FindInAsync(dic);
                            foreach(var userItem in item.IOTDeviceUseRecords)
                            {
                                var curUserInfo = deviceUserList.FirstOrDefault(cx => cx.Code == userItem.UserCode) ?? new UserInfoDO();
                                var userInfo = ConvertUserToIOTDeviceUserExtendInfoDTO(curUserInfo);//curUserInfo.MappingTo<IOTDeviceUserExtendInfoDTO>();
                                userInfo.FolderList = null;  
                                model.UserDataList.Add(userInfo);
                            }
                            dataList.Add(model);
                        }
                    }
                }
                return dataList;
            }
        }
        #endregion 

        /// <summary>
        /// model实体转换
        /// </summary>
        private IOTDeviceUserExtendInfoDTO ConvertUserToIOTDeviceUserExtendInfoDTO(UserInfoDO curUserInfo)
        {
            if (curUserInfo == null)
            {
                return new IOTDeviceUserExtendInfoDTO();
            }
            var model = curUserInfo.MappingTo<IOTDeviceUserExtendInfoDTO>() ?? new IOTDeviceUserExtendInfoDTO();
            model.Name = (curUserInfo.OpenId == "-") ? curUserInfo.UserName : curUserInfo.Phone;
            model.UserCode = curUserInfo.UserCode;
            if (curUserInfo.BirthDate != null && curUserInfo.BirthDate > Convert.ToDateTime("1900-1-1"))
            {
                var newDate = UTCToBeijing(curUserInfo.BirthDate.Value);
                model.Age = CalculateAge(newDate);
                model.Birthday = newDate;
            }
            else if (!string.IsNullOrEmpty(curUserInfo.Birthday))
            {
                var newDate = Convert.ToDateTime(curUserInfo.Birthday);
                model.Age = CalculateAge(newDate);
                model.Birthday = newDate;
            }
            else
            {
                model.Age = 0;
            }
            model.Neck = curUserInfo.NeckGirth;
            //性别：0-未知；1-男；2-女
            if (curUserInfo.Gender == 1)
            {
                model.Sex = "男";
            }
            else if (curUserInfo.Gender == 1)
            {
                model.Sex = "女";
            }
            else
            {
                model.Sex = "未知";
            }
            return model;
        }

        /// <summary>
        /// 获取license信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<LicenseInfoBasicDTO>> GetLicenseInfoListDBAsync(TokenRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLicenseInfoListDBAsync execute", $"{this.GetType().Name}"))
            {
                var resData = new List<LicenseInfoBasicDTO>();
                var builder = Builders<LicenseInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);      
                var licenseInfos = await _licenseInfoDBRepository.FindAllAsync(filter);
                resData = licenseInfos.MappingTo<List<LicenseInfoBasicDTO>>();
                return resData;
            }
        }

        /// <summary>
        /// 保存License信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SaveLicenseInfoListDBAsync(SaveLicenseInfoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveLicenseInfoListDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                if (string.IsNullOrEmpty(request.LicenseId) || string.IsNullOrEmpty(request.AgencyName) || string.IsNullOrEmpty(request.LicenseContent))
                {
                    ThrowCustomerException(CustomerRpcCode.LicenseIdOrDongleIdOrAgencyNameOrContentEmptyError, "LicenseId Or DongleId Or AgencyName Or Content Empty Error");
                }
                if (string.IsNullOrEmpty(request.Code))
                {
                    //新增
                    //查找用户名和licenseId是否存在
                    var builder = Builders<LicenseInfoEntity>.Filter;
                    //var filter = builder.Where(c => c.IsDelete == false && c.LicenseId == request.LicenseId && c.AgencyName == request.AgencyName);  
                    var filter = builder.Where(c => c.IsDelete == false && c.LicenseId == request.LicenseId);      
                    var licenseInfo = await _licenseInfoDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(licenseInfo?.Code))
                    {
                        //修改
                        licenseInfo.LicenseContent = request.LicenseContent;
                        licenseInfo.AgencyName = request.AgencyName;
                        licenseInfo.UpdateTime = DateTime.UtcNow;
                        res = await _licenseInfoDBRepository.UpdateOneAsync("Code", licenseInfo.Code, licenseInfo, true);
                    }
                    else
                    {
                        // filter = builder.Where(c => c.IsDelete == false && c.LicenseId != request.LicenseId && c.AgencyName == request.AgencyName);      
                        // licenseInfo = await _licenseInfoDBRepository.FindOneAsync(filter);
                        // if (!string.IsNullOrEmpty(licenseInfo?.Code))
                        // {
                        //     //异常
                        //     ThrowCustomerException(CustomerRpcCode.LicenseBindAgencyExist, "License Bind Agency Exist");
                        //     return false;
                        // }
                        // filter = builder.Where(c => c.IsDelete == false && c.LicenseId == request.LicenseId && c.AgencyName != request.AgencyName);      
                        // licenseInfo = await _licenseInfoDBRepository.FindOneAsync(filter);
                        // if (!string.IsNullOrEmpty(licenseInfo?.Code))
                        // {
                        //     //异常
                        //     ThrowCustomerException(CustomerRpcCode.LicenseBindAgencyExist, "License Bind Agency Exist");
                        //     return false;
                        // }
                        //新增
                        licenseInfo = new LicenseInfoEntity();
                        licenseInfo.LicenseId = request.LicenseId;
                        licenseInfo.AgencyName = request.AgencyName;
                        licenseInfo.LicenseContent = request.LicenseContent;
                        var insertIdentity = await _licenseInfoDBRepository.InsertOneAsync(licenseInfo);
                        if (!string.IsNullOrEmpty(insertIdentity))
                        {
                            res = true;
                        }
                        
                    }
                }
                else
                {
                    //修改
                    var builder = Builders<LicenseInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);      
                    var licenseInfo = await _licenseInfoDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(licenseInfo?.Code))
                    {
                        //修改
                        licenseInfo.LicenseContent = request.LicenseContent;
                        licenseInfo.LicenseId = request.LicenseId;
                        licenseInfo.AgencyName = request.AgencyName;
                        licenseInfo.UpdateTime = DateTime.UtcNow;
                        res = await _licenseInfoDBRepository.UpdateOneAsync("Code", licenseInfo.Code, licenseInfo, true);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 保存Dongle信息数据
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SaveDongleInfoListDBAsync(SaveDongleInfoRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveDongleInfoListDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                if (string.IsNullOrEmpty(request.LicenseId) || string.IsNullOrEmpty(request.DongleId) || string.IsNullOrEmpty(request.DongleContent))
                {
                    ThrowCustomerException(CustomerRpcCode.LicenseIdOrDongleIdOrAgencyNameOrContentEmptyError, "LicenseId Or DongleId Or AgencyName Or Content Empty Error");
                }
                if (string.IsNullOrEmpty(request.Code))
                {
                    //新增
                    //查找用户名和licenseId是否存在
                    var builder = Builders<DongleInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.LicenseId == request.LicenseId && c.DongleId == request.DongleId);      
                    var dongleInfo = await _dongleInfoDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(dongleInfo?.Code))
                    {
                        //修改
                        dongleInfo.DongleContent = request.DongleContent;
                        dongleInfo.DongleType = request.DongleType;
                        dongleInfo.UpdateTime = DateTime.UtcNow;
                        res = await _dongleInfoDBRepository.UpdateOneAsync("Code", dongleInfo.Code, dongleInfo, true);
                    }
                    else
                    {
                        //新增
                        dongleInfo = new DongleInfoEntity();
                        dongleInfo.DongleContent = request.DongleContent;
                        dongleInfo.DongleType = request.DongleType;
                        dongleInfo.DongleId = request.DongleId;
                        dongleInfo.LicenseId = request.LicenseId;
                        var insertIdentity = await _dongleInfoDBRepository.InsertOneAsync(dongleInfo);
                        if (!string.IsNullOrEmpty(insertIdentity))
                        {
                            res = true;
                        }
                    }
                }
                else
                {
                    //修改
                    var builder = Builders<DongleInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.Code == request.Code);      
                    var dongleInfo = await _dongleInfoDBRepository.FindOneAsync(filter);
                    if (!string.IsNullOrEmpty(dongleInfo?.Code))
                    {
                        //修改
                        dongleInfo.DongleContent = request.DongleContent;
                        dongleInfo.DongleType = request.DongleType;
                        dongleInfo.LicenseId = request.LicenseId;
                        dongleInfo.DongleId = request.DongleId;
                        dongleInfo.UpdateTime = DateTime.UtcNow;
                        res = await _dongleInfoDBRepository.UpdateOneAsync("Code", dongleInfo.Code, dongleInfo, true);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 分页获取license信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<LicenseDetailInfoDTO>> GetLicenseInfoPagesDBAsync(GetLicensePageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetLicenseInfoPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<LicenseInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var agencyNameFilter = builder.Regex(c => c.AgencyName, new BsonRegularExpression(keyword, "i"));
                    var licenseIdFilter = builder.Regex(c => c.LicenseId, new BsonRegularExpression(keyword, "i"));
                    filter = filter & (agencyNameFilter | licenseIdFilter);
                }
                var sort = Builders<LicenseInfoEntity>.Sort.Descending(v => v.UpdateTime);
                var dbPage = new DBPageRequest<LicenseInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _licenseInfoDBRepository.GetPages(dbPage);
                var res = PageResultConvert<LicenseInfoEntity, LicenseDetailInfoDTO>(pageResult);
                if (res?.PageData?.Count > 0)
                {
                    foreach(var item in res.PageData)
                    {
                        if (!string.IsNullOrEmpty(item.LicenseContent))
                        {
                            var xmlText = LicenseHelper.Decrypt(item.LicenseContent);
                            var license = LicenseHelper.FromFile(xmlText);
                            if (license != null)
                            {
                                item.LicenseType = (int)license.LicenseType;
                                item.FunctionalItemValue = license.FunctionalItemValue;
                                item.ExpiredTime = license.ExpirationUtcDate;
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 分页获取Dongle信息列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<DongleDetailInfoDTO>> GetDongleInfoPagesDBAsync(GetDonglePageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDongleInfoPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<DongleInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false);
                if (!string.IsNullOrEmpty(request.Keyword))
                {
                    var keyword = EscapeExprSpecialWord(request.Keyword);//字符串转义
                    var dongleIdFilter = builder.Regex(c => c.DongleId, new BsonRegularExpression(keyword, "i"));

                    var lBuilder = Builders<LicenseInfoEntity>.Filter;
                    var lFilter = lBuilder.Where(c => c.IsDelete == false); 
                    var agencyNameFilter = lBuilder.Regex(c => c.AgencyName, new BsonRegularExpression(keyword, "i"));
                    var lIdFilter = lBuilder.Regex(c => c.LicenseId, new BsonRegularExpression(keyword, "i"));
                    lFilter = lFilter & (agencyNameFilter | lIdFilter);
                    var allLicenseList = await _licenseInfoDBRepository.FindAllAsync(lFilter);
                    if (allLicenseList?.Count > 0)
                    {
                        var licenseIdList = allLicenseList.Select(c => c.LicenseId).Distinct().ToList() ?? new List<string>();
                        var anyLicenseFilter = builder.Where(c => licenseIdList.Contains(c.LicenseId));
                        filter = filter & (dongleIdFilter | anyLicenseFilter);
                    }
                    else 
                    {
                        var licenseIdFilter = builder.Regex(c => c.LicenseId, new BsonRegularExpression(keyword, "i"));
                        filter = filter & (dongleIdFilter | licenseIdFilter);
                    }
                }
                if (request.DongleType > 0)
                {
                    var dongleTypeFilter = builder.Where(c => c.DongleType == request.DongleType);
                    filter = filter & dongleTypeFilter;
                }
                var sort = Builders<DongleInfoEntity>.Sort.Descending(v => v.UpdateTime);
                var dbPage = new DBPageRequest<DongleInfoEntity>();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _dongleInfoDBRepository.GetPages(dbPage);
                var res = PageResultConvert<DongleInfoEntity, DongleDetailInfoDTO>(pageResult);
                if (res?.PageData?.Count > 0)
                {
                    var licenseIds = res.PageData.Select(c => c.LicenseId).Distinct().ToList();
                    var licenseBuilder = Builders<LicenseInfoEntity>.Filter;
                    var licenseFilter = licenseBuilder.Where(c => c.IsDelete == false && licenseIds.Contains(c.LicenseId));
                    var resData = await _licenseInfoDBRepository.FindAllAsync(licenseFilter) ?? new List<LicenseInfoEntity>();
                    foreach(var item in res.PageData)
                    {
                        var findLicenseInfo = resData.FirstOrDefault(c => c.LicenseId == item.LicenseId) ?? new LicenseInfoEntity();
                        item.AgencyName = findLicenseInfo.AgencyName;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 根据licenseId获取license详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<LicenseDetailInfoDTO> GetLicenseDetailDBAsync(GetLicenseOrDongleRequest request)
        {            
            using (new Performance(_performanceThreshold, "{0} GetLicenseDetailDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<LicenseInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.LicenseId == request.LicenseId);
                var res = await _licenseInfoDBRepository.FindOneAsync(filter);
                return res.MappingTo<LicenseDetailInfoDTO>();
            }
        }

        /// <summary>
        /// 根据dongleid或者licenseId获取Dongle详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<DongleDetailInfoDTO> GetDongleDetailDBAsync(GetLicenseOrDongleRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDongleInfoPagesDBAsync execute", $"{this.GetType().Name}"))
            {
                var builder = Builders<DongleInfoEntity>.Filter;
                var filter = builder.Where(c => c.IsDelete == false && c.DongleId == request.DongleId && c.LicenseId == request.LicenseId);
                var res = await _dongleInfoDBRepository.FindOneAsync(filter);
                return res.MappingTo<DongleDetailInfoDTO>();
            }
        }

        /// <summary>
        /// 验证设备是否已经被绑定
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<List<CheckIOTDeviceBeenBoundResult>> CheckIOTDeviceBeenBoundDBAsync(CheckIOTDeviceBeenBoundRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CheckIOTDeviceBeenBoundDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new List<CheckIOTDeviceBeenBoundResult>();
                var deviceDataList = new List<string>();
                if (request.DeviceList?.Count > 0)
                {
                    var macList = request.DeviceList.Select(c => c.DeviceMac).Distinct().ToList();
                    var builder = Builders<UserDeviceExtendInfoEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && macList.Contains(c.DeviceMAC));
                    var dataList = await _userDeviceExtendInfoDBRepository.FindAllAsync(filter);
                    if (dataList?.Count > 0)
                    {
                        foreach(var item in request.DeviceList)
                        {
                            var findEntity = dataList.FirstOrDefault(c => c.MachineAddress.Contains(item.DeviceMac)) ?? new UserDeviceExtendInfoEntity();
                            var model = new CheckIOTDeviceBeenBoundResult()
                            {
                                DeviceCode = findEntity.DeviceCode,
                                DeviceSN = item.DeviceSN,
                                DeviceMac = item.DeviceMac,
                                IsBound = !string.IsNullOrEmpty(findEntity.DeviceCode) ? true : false
                            };
                            result.Add(model);
                        }
                    }
                    else
                    {
                        foreach(var item in request.DeviceList)
                        {
                            var model = new CheckIOTDeviceBeenBoundResult()
                            {
                                DeviceCode = "",
                                DeviceSN = item.DeviceSN,
                                DeviceMac = item.DeviceMac,
                                IsBound = false
                            };
                            result.Add(model);
                        }
                    }
                }
                return result;
            }
        }

        /// <summary>
        /// 设备阶段报告日周月年查询接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<IOTDeviceStageReportResult> GetIOTDeviceStageReportDBAsync(IOTDeviceStageReportRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CheckIOTDeviceBeenBoundDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new IOTDeviceStageReportResult();
                try
                {
                    //获取所有的名词解释
                    var dataList = new List<CommonDictionaryDTO>();
                    var builder = Builders<CommonDictionaryEntity>.Filter;
                    var filter = builder.Where(c => c.IsDelete == false && c.Type == CommonDictionaryTypeEnum.NounInterpretation);
                    var nounInterpretationList = (await _commonDictionaryDBRepository.FindAllAsync(filter)) ?? new List<CommonDictionaryEntity>();
                    var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                    if (request.ReportType == IOTDeviceStageReportTypeEnum.Day)
                    {
                        var beginTime = Convert.ToDateTime(request.ReportDate.ToString("yyyy-MM-dd"));
                        var endTime = Convert.ToDateTime(request.ReportDate.ToString("yyyy-MM-dd")).AddDays(1);
                        var lastFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.Mode == request.Mode && c.DeviceCode == request.DeviceCode && c.ReportType == ReportType.Day && c.Timestamp >= beginTime && c.Timestamp < endTime);
                        var entity = await _deviceReportDBRepository.FindOneAsync(lastFilter, new FindOptions<DeviceReportEntity> { Sort = Builders<DeviceReportEntity>.Sort.Descending(f => f.UpdateTime) });
                        if (!string.IsNullOrEmpty(entity?.Code))
                        {
                            result.Mode = request.Mode;
                            var useHourStr = entity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(useHourStr))
                            {
                                result.TotalUseHour = Math.Round(Convert.ToDouble(useHourStr), 1);
                            }
                            var tpress_aveStr = entity.ReportAttributes.Find(c => c.Key == "tpress_ave")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(tpress_aveStr))
                            {
                                result.AvgTreatmentPressure = Math.Round(Convert.ToDouble(tpress_aveStr), 1);
                            }
                            var tpress_95Str = entity.ReportAttributes.Find(c => c.Key == "tpress_95")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(tpress_95Str))
                            {
                                result.TreatmentPressure95 = Math.Round(Convert.ToDouble(tpress_95Str), 1);
                            }
                            var tpress_90Str = entity.ReportAttributes.Find(c => c.Key == "tpress_90")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(tpress_90Str))
                            {
                                result.TreatmentPressure90 = Math.Round(Convert.ToDouble(tpress_90Str), 1);
                            }

                            var spo2_aveStr = entity.ReportAttributes.Find(c => c.Key == "spo2_ave")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(spo2_aveStr))
                            {
                                result.Spo2Ave = Math.Round(Convert.ToDouble(spo2_aveStr), 1);
                            }
                            var spo2_minStr = entity.ReportAttributes.Find(c => c.Key == "spo2_min")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(spo2_minStr))
                            {
                                result.Spo2Min = Math.Round(Convert.ToDouble(spo2_minStr), 1);
                            }
                            var pr_aveStr = entity.ReportAttributes.Find(c => c.Key == "pr_ave")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(pr_aveStr))
                            {
                                result.PrAve = Math.Round(Convert.ToDouble(pr_aveStr), 1);
                            }

                            var ahiStr = entity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(ahiStr))
                            {
                                var ahi = Math.Round(Convert.ToDouble(ahiStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "ahi".ToUpper()) ?? new CommonDictionaryEntity();
                                result.AHI = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = ahi,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var hiStr = entity.ReportAttributes.Find(c => c.Key == "hi")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(hiStr))
                            {
                                var hi = Math.Round(Convert.ToDouble(hiStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "hi".ToUpper()) ?? new CommonDictionaryEntity();
                                result.HI = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = hi,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var aiStr = entity.ReportAttributes.Find(c => c.Key == "ai")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(aiStr))
                            {                                
                                var ai = Math.Round(Convert.ToDouble(aiStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "ai".ToUpper()) ?? new CommonDictionaryEntity();
                                result.AI = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = ai,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var oaiStr = entity.ReportAttributes.Find(c => c.Key == "oai")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(oaiStr))
                            {
                                var oai = Math.Round(Convert.ToDouble(oaiStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "oai".ToUpper()) ?? new CommonDictionaryEntity();
                                result.OAI = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = oai,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var caiStr = entity.ReportAttributes.Find(c => c.Key == "cai")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(caiStr))
                            {
                                var cai = Math.Round(Convert.ToDouble(caiStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "cai") ?? new CommonDictionaryEntity();
                                result.CAI = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = cai,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var snore_eStr = entity.ReportAttributes.Find(c => c.Key == "snore_e")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(snore_eStr))
                            {
                                var snore_e = Math.Round(Convert.ToDouble(snore_eStr), 1);
                                var tempExplainInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "打鼾事件") ?? new CommonDictionaryEntity();
                                result.SnoringEvent = new IOTDeviceReportAttributeDTO()
                                {
                                    Value = snore_e,
                                    Explain = tempExplainInfo.Description
                                };
                            }
                            var leak_aveStr = entity.ReportAttributes.Find(c => c.Key == "leak_ave")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(leak_aveStr))
                            {
                                result.AvgLeak = Math.Round(Convert.ToDouble(leak_aveStr), 1);
                            }
                            var leak_95Str = entity.ReportAttributes.Find(c => c.Key == "leak_95")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(leak_95Str))
                            {
                                result.Leak95 = Math.Round(Convert.ToDouble(leak_95Str), 1);
                            }
                            var leak_90Str = entity.ReportAttributes.Find(c => c.Key == "leak_90")?.Value ?? string.Empty;
                            if (!string.IsNullOrEmpty(leak_90Str))
                            {
                                result.Leak90 = Math.Round(Convert.ToDouble(leak_90Str), 1);
                            }
                            result.AvgSocre = GetReportScore(entity, Convert.ToDecimal(result.TotalUseHour));
                        }
                        if (result.AvgSocre < 60)
                        {
                            result.TreatmentEffect = "不佳";
                        }
                        else if (result.AvgSocre >= 60 && result.AvgSocre < 80)
                        {
                            result.TreatmentEffect = "一般";
                        }
                        else if (result.AvgSocre >= 80 && result.AvgSocre < 90)
                        {
                            result.TreatmentEffect = "良好";
                        }
                        else if (result.AvgSocre >= 90 && result.AvgSocre <= 100)
                        {
                            result.TreatmentEffect = "优秀";
                        }
                    }
                    else
                    {
                        //按照时间范围查询多条数据，合并计算
                        var beginTime = DateTime.Today;
                        var endTime = DateTime.Today;
                        var dayCount = 0;
                        var cycle = 0;
                        if (request.ReportType == IOTDeviceStageReportTypeEnum.Week)
                        {
                            beginTime = request.ReportDate.AddHours(12).GetCurrentWeek();
                            endTime = beginTime.AddDays(7);
                            dayCount = 7;
                        }
                        else if (request.ReportType == IOTDeviceStageReportTypeEnum.Month)
                        {
                            beginTime = request.ReportDate.AddHours(12).GetCurrentMonth().Item1;
                            endTime = beginTime.AddMonths(1);
                            dayCount = Convert.ToInt32((endTime - beginTime).TotalDays);
                            cycle = 4;
                        }
                        else if (request.ReportType == IOTDeviceStageReportTypeEnum.Year)
                        {
                            beginTime = request.ReportDate.AddHours(12).GetCurrentYear();
                            endTime = beginTime.AddYears(1);
                            dayCount = 365;
                            cycle = 12;
                        }
                        var lastFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.Mode == request.Mode && c.DeviceCode == request.DeviceCode && c.ReportType == ReportType.Day && c.Timestamp >= beginTime && c.Timestamp < endTime);
                        var reportList = await _deviceReportDBRepository.FindAllAsync(lastFilter);
                        if (reportList?.Count > 0)
                        {
                            result.Mode = request.Mode;
                            var validUseDays = 0;
                            int totalUseDay = 0;
                            int goodStateUseDay = 0;
                            int gteFourHoursUseDay = 0;
                            double totalUseHour = 0;
                            double avgUseHour = 0;
                            double avgSocre = 0;
                            double avgTreatmentPressure = 0;
                            double treatmentPressure95 = 0;
                            double treatmentPressure90 = 0;
                            double avgLeak = 0;
                            double leak95 = 0;
                            double leak90 = 0;
                            double ahi = 0;
                            double hi = 0;
                            double ai = 0;
                            double oai = 0;
                            double cai = 0;
                            double snoringEvent = 0;
                            double spo2Ave = 0;
                            double spo2Min = 0;
                            double prAve = 0;
                            for(var i = 0; i < dayCount; i++)
                            {
                                var tempStartTime = beginTime.AddDays(i);
                                var tempEndTime = tempStartTime.AddDays(1);
                                var findReportEntity = reportList.OrderBy(c => c.Timestamp).FirstOrDefault(c => c.Timestamp >= tempStartTime && c.Timestamp < tempEndTime) ?? new DeviceReportEntity();
                                if (!string.IsNullOrEmpty(findReportEntity?.Code))
                                {
                                    var use_ahourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_ahour")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(use_ahourStr))
                                    {
                                        avgUseHour += Convert.ToInt32(use_ahourStr);
                                    }
                                    var use_4hdayStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_4hday")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(use_4hdayStr))
                                    {
                                        gteFourHoursUseDay += Convert.ToInt32(use_4hdayStr);
                                    }
                                    var curUseDay = 0;
                                    var use_dayStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_day")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(use_dayStr))
                                    {
                                        totalUseDay += Convert.ToInt32(use_dayStr);
                                        curUseDay = Convert.ToInt32(use_dayStr);
                                    }
                                    double curTotalUseHour = 0;
                                    var useHourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(useHourStr))
                                    {
                                        totalUseHour += Math.Round(Convert.ToDouble(useHourStr), 1);
                                        curTotalUseHour = Math.Round(Convert.ToDouble(useHourStr), 1);
                                    }
                                    var tpress_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "tpress_ave")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(tpress_aveStr))
                                    {
                                        avgTreatmentPressure = Math.Round(Convert.ToDouble(tpress_aveStr), 1);
                                    }
                                    var tpress_95Str = findReportEntity.ReportAttributes.Find(c => c.Key == "tpress_95")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(tpress_95Str))
                                    {
                                        treatmentPressure95 = Math.Round(Convert.ToDouble(tpress_95Str), 1);
                                    }
                                    var tpress_90Str = findReportEntity.ReportAttributes.Find(c => c.Key == "tpress_90")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(tpress_90Str))
                                    {
                                        treatmentPressure90 = Math.Round(Convert.ToDouble(tpress_90Str), 1);
                                    }

                                    var spo2_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "spo2_ave")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(spo2_aveStr))
                                    {
                                        spo2Ave = Math.Round(Convert.ToDouble(spo2_aveStr), 1);
                                    }
                                    var spo2_minStr = findReportEntity.ReportAttributes.Find(c => c.Key == "spo2_min")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(spo2_minStr))
                                    {
                                        spo2Min = Math.Round(Convert.ToDouble(spo2_minStr), 1);
                                    }
                                    var pr_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "pr_ave")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(pr_aveStr))
                                    {
                                        prAve = Math.Round(Convert.ToDouble(pr_aveStr), 1);
                                    }

                                    var ahiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(ahiStr))
                                    {
                                        ahi += Math.Round(Convert.ToDouble(ahiStr), 1);
                                    }
                                    var hiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "hi")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(hiStr))
                                    {
                                        hi += Math.Round(Convert.ToDouble(hiStr), 1);
                                    }
                                    var aiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(aiStr))
                                    {                                
                                        ai += Math.Round(Convert.ToDouble(aiStr), 1);
                                    }
                                    var oaiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "oai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(oaiStr))
                                    {
                                        oai += Math.Round(Convert.ToDouble(oaiStr), 1);
                                    }
                                    var caiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "cai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(caiStr))
                                    {
                                        cai += Math.Round(Convert.ToDouble(caiStr), 1);
                                    }
                                    var snore_eStr = findReportEntity.ReportAttributes.Find(c => c.Key == "snore_e")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(snore_eStr))
                                    {
                                        snoringEvent += Math.Round(Convert.ToDouble(snore_eStr), 1);
                                    }
                                    var leak_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "leak_ave")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(leak_aveStr))
                                    {
                                        avgLeak = Math.Round(Convert.ToDouble(leak_aveStr), 1);
                                    }
                                    var leak_95Str = findReportEntity.ReportAttributes.Find(c => c.Key == "leak_95")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(leak_95Str))
                                    {
                                        leak95 = Math.Round(Convert.ToDouble(leak_95Str), 1);
                                    }
                                    var leak_90Str = findReportEntity.ReportAttributes.Find(c => c.Key == "leak_90")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(leak_90Str))
                                    {
                                        leak90 = Math.Round(Convert.ToDouble(leak_90Str), 1);
                                    }
                                    avgSocre += GetReportScore(findReportEntity, Convert.ToDecimal(curTotalUseHour));
                                    if (result.AvgSocre >= 80 && result.AvgSocre < 90)
                                    {
                                        goodStateUseDay += curUseDay;
                                    }
                                    else if (result.AvgSocre >= 90 && result.AvgSocre <= 100)
                                    {
                                        goodStateUseDay += curUseDay;
                                    }
                                    validUseDays++;
                                }
                            }
                            result.TotalUseDay = totalUseDay;
                            result.GoodStateUseDay = goodStateUseDay;
                            result.GteFourHoursUseDay = gteFourHoursUseDay;
                            result.TotalUseHour = totalUseHour;
                            if (validUseDays > 0) 
                            {
                                result.AvgTreatmentPressure = Math.Round((avgTreatmentPressure / validUseDays), 2);
                                result.TreatmentPressure95 = Math.Round((treatmentPressure95 / validUseDays), 2);
                                result.TreatmentPressure90 = Math.Round((treatmentPressure90 / validUseDays), 2);
                                result.AvgLeak = Math.Round((avgLeak / validUseDays), 2);
                                result.Leak95 = Math.Round((leak95 / validUseDays), 2);
                                result.Leak90 = Math.Round((leak90 / validUseDays), 2);
                                result.Spo2Ave = Math.Round((spo2Ave / validUseDays), 2);
                                result.Spo2Min = Math.Round((spo2Min / validUseDays), 2);
                                result.PrAve = Math.Round((prAve / validUseDays), 2);
                                result.AvgSocre = Math.Round((avgSocre / validUseDays), 2);
                            }
                            
                            if (cycle > 0)
                            {
                                result.AvgUseHour = Math.Round((avgUseHour / cycle), 2);
                            }
                            else
                            {
                                if (validUseDays > 0) 
                                {
                                    result.AvgUseHour = Math.Round((avgUseHour / validUseDays), 2);
                                }
                            }
                            var tempExplainHiInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "hi".ToUpper()) ?? new CommonDictionaryEntity();
                            result.HI = new IOTDeviceReportAttributeDTO()
                            {
                                Value = hi,
                                Explain = tempExplainHiInfo.Description
                            };
                            var tempExplainAiInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "ai".ToUpper()) ?? new CommonDictionaryEntity();
                            result.AI = new IOTDeviceReportAttributeDTO()
                            {
                                Value = ai,
                                Explain = tempExplainAiInfo.Description
                            };
                            var tempExplainOaiInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "oai".ToUpper()) ?? new CommonDictionaryEntity();
                            result.OAI = new IOTDeviceReportAttributeDTO()
                            {
                                Value = oai,
                                Explain = tempExplainOaiInfo.Description
                            };
                            var tempExplainCaiInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "cai".ToUpper()) ?? new CommonDictionaryEntity();
                            result.CAI = new IOTDeviceReportAttributeDTO()
                            {
                                Value = cai,
                                Explain = tempExplainCaiInfo.Description
                            };
                            var tempExplainAhiInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "ahi".ToUpper()) ?? new CommonDictionaryEntity();
                            result.AHI = new IOTDeviceReportAttributeDTO()
                            {
                                Value = ahi,
                                Explain = tempExplainAhiInfo.Description
                            };
                            var tempExplainSnoringInfo = nounInterpretationList.FirstOrDefault(c => c.Key == "打鼾事件".ToUpper()) ?? new CommonDictionaryEntity();
                            result.SnoringEvent = new IOTDeviceReportAttributeDTO()
                            {
                                Value = snoringEvent,
                                Explain = tempExplainSnoringInfo.Description
                            };
                            if (result.AvgSocre < 60)
                            {
                                result.TreatmentEffect = "不佳";
                            }
                            else if (result.AvgSocre >= 60 && result.AvgSocre < 80)
                            {
                                result.TreatmentEffect = "一般";
                            }
                            else if (result.AvgSocre >= 80 && result.AvgSocre < 90)
                            {
                                result.TreatmentEffect = "良好";
                            }
                            else if (result.AvgSocre >= 90 && result.AvgSocre <= 100)
                            {
                                result.TreatmentEffect = "优秀";
                            }
                        }
                    }
                    return result;
                }
                catch(Exception ex)
                {
                    Logger.WriteLineError($"DeviceInfoDBService GetIOTDeviceStageReportDBAsync error, request: {JsonConvert.SerializeObject(request)}, error: {ex}");
                    return result;
                }
            }
        }
        
        /// <summary>
        /// 设备阶段报告趋势图查询接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<IOTDeviceStageReportTrendChartResult> GetIOTDeviceStageReportTrendChartDBAsync(IOTDeviceStageReportTrendChartRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetIOTDeviceStageReportTrendChartDBAsync execute", $"{this.GetType().Name}"))
            {
                var result = new IOTDeviceStageReportTrendChartResult()
                {
                    UseTimeChartData = new List<DataItemDTO>(),
                    AHIChartData = new List<DataItemDTO>(),
                    HIChartData = new List<DataItemDTO>(),
                    AIChartData = new List<DataItemDTO>(),
                    OAIChartData = new List<DataItemDTO>(),
                    CAIChartData = new List<DataItemDTO>(),
                    SnoreEventChartData = new List<DataItemDTO>(),
                    Spo2AveChartData = new List<DataItemDTO>()
                };
                try
                {
                    var beginTime = DateTime.Today;
                    var endTime = DateTime.Today;
                    var dayCount = 0;
                    var dic = new Dictionary<int, List<DateTime>>();
                    if (request.ReportType == IOTDeviceStageReportTypeEnum.Week)
                    {
                        beginTime = request.ReportDate.AddHours(12).GetCurrentWeek();
                        endTime = beginTime.AddDays(7);
                        dayCount = 7;
                    }
                    else if (request.ReportType == IOTDeviceStageReportTypeEnum.Month)
                    {
                        beginTime = request.ReportDate.AddHours(12).GetCurrentMonth().Item1;
                        endTime = beginTime.AddMonths(1);
                        dic = request.ReportDate.AddHours(12).GetCurrentMonthAllWeek();
                    }
                    else if (request.ReportType == IOTDeviceStageReportTypeEnum.Year)
                    {
                        beginTime = request.ReportDate.AddHours(12).GetCurrentYear();
                        endTime = beginTime.AddYears(1);
                        dic = request.ReportDate.AddHours(12).GetCurrentYearAllMonth();
                    }
                    var deviceReportBuilder = Builders<DeviceReportEntity>.Filter;
                    var lastFilter = deviceReportBuilder.Where(c => c.IsDelete == false && c.Mode == request.Mode && c.DeviceCode == request.DeviceCode && c.ReportType == ReportType.Day && c.Timestamp >= beginTime && c.Timestamp < endTime);
                    var reportList = await _deviceReportDBRepository.FindAllAsync(lastFilter);
                    if (reportList?.Count > 0)
                    {
                        if (request.ReportType == IOTDeviceStageReportTypeEnum.Week)
                        {
                            for(var i = 0; i < dayCount; i++)
                            {
                                var model = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelAHI = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelHI = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelAI = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelOAI = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelCAI = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelSnoreEvent = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var modelSpo2Ave = new DataItemDTO()
                                {
                                    Value = "0"
                                };
                                var tempStartTime = beginTime.AddDays(i);
                                var tempEndTime = tempStartTime.AddDays(1);
                                model.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelAHI.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelHI.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelAI.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelOAI.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelCAI.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelSnoreEvent.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                modelSpo2Ave.Key = tempStartTime.Month + "." + tempStartTime.Day;
                                var findReportEntity = reportList.OrderBy(c => c.Timestamp).FirstOrDefault(c => c.Timestamp >= tempStartTime && c.Timestamp < tempEndTime) ?? new DeviceReportEntity();
                                if (!string.IsNullOrEmpty(findReportEntity?.Code))
                                {
                                    var useHourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(useHourStr))
                                    {
                                        model.Value = Math.Round(Convert.ToDouble(useHourStr), 1).ToString();
                                    }
                                    var ahiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(ahiStr))
                                    {
                                        modelAHI.Value = Math.Round(Convert.ToDouble(ahiStr), 1).ToString();
                                    }
                                    var hiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "hi")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(hiStr))
                                    {
                                        modelHI.Value += Math.Round(Convert.ToDouble(hiStr), 1);
                                    }
                                    var aiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(aiStr))
                                    {                                
                                        modelAI.Value += Math.Round(Convert.ToDouble(aiStr), 1);
                                    }
                                    var oaiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "oai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(oaiStr))
                                    {
                                        modelOAI.Value += Math.Round(Convert.ToDouble(oaiStr), 1);
                                    }
                                    var caiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "cai")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(caiStr))
                                    {
                                        modelCAI.Value += Math.Round(Convert.ToDouble(caiStr), 1);
                                    }
                                    var snore_eStr = findReportEntity.ReportAttributes.Find(c => c.Key == "snore_e")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(snore_eStr))
                                    {
                                        modelSnoreEvent.Value += Math.Round(Convert.ToDouble(snore_eStr), 1);
                                    }
                                    var spo2_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "spo2_ave")?.Value ?? string.Empty;
                                    if (!string.IsNullOrEmpty(spo2_aveStr))
                                    {
                                        modelSpo2Ave.Value += Math.Round(Convert.ToDouble(spo2_aveStr), 1);
                                    }
                                }
                                result.UseTimeChartData.Add(model);
                                result.AHIChartData.Add(modelAHI);
                                result.HIChartData.Add(modelHI);
                                result.AIChartData.Add(modelAI);
                                result.OAIChartData.Add(modelOAI);
                                result.CAIChartData.Add(modelCAI);
                                result.SnoreEventChartData.Add(modelSnoreEvent);
                                result.Spo2AveChartData.Add(modelSpo2Ave);
                            }
                        }
                        else if (request.ReportType == IOTDeviceStageReportTypeEnum.Month)
                        {
                            foreach(var key in dic.Keys)
                            {
                                var model = new DataItemDTO();
                                var modelAHI = new DataItemDTO();
                                var modelHI = new DataItemDTO();
                                var modelAI = new DataItemDTO();
                                var modelOAI = new DataItemDTO();
                                var modelCAI = new DataItemDTO();
                                var modelSnoreEvent = new DataItemDTO();
                                var modelSpo2Ave = new DataItemDTO();
                                model.Key = "第" + key + "周";
                                modelAHI.Key = "第" + key + "周";
                                modelHI.Key = "第" + key + "周";
                                modelAI.Key = "第" + key + "周";
                                modelOAI.Key = "第" + key + "周";
                                modelCAI.Key = "第" + key + "周";
                                modelSnoreEvent.Key = "第" + key + "周";
                                modelSpo2Ave.Key = "第" + key + "周";
                                var dateList = dic[key];
                                int validUseHourCount = 0;
                                int validAhiCount = 0;
                                int validHiCount = 0;
                                int validAICount = 0;
                                int validOAICount = 0;
                                int validCAICount = 0;
                                int validSnoreEventCount = 0;
                                int validSpo2AveCount = 0;
                                double useHour = 0;
                                double ahi = 0;
                                double hi = 0;
                                double ai = 0;
                                double oai = 0;
                                double cai = 0;
                                double snoreEvent = 0;
                                double spo2Ave = 0;
                                foreach(var dateItem in dateList)
                                {
                                    var tempStartTime = dateItem;
                                    var tempEndTime = dateItem.AddDays(1);

                                    var findReportEntity = reportList.OrderBy(c => c.Timestamp).FirstOrDefault(c => c.Timestamp >= tempStartTime && c.Timestamp < tempEndTime) ?? new DeviceReportEntity();
                                    if (!string.IsNullOrEmpty(findReportEntity?.Code))
                                    {
                                        var useHourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(useHourStr))
                                        {
                                            useHour += Math.Round(Convert.ToDouble(useHourStr), 1);
                                            validUseHourCount++;
                                        }
                                        var ahiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(ahiStr))
                                        {
                                            ahi = Math.Round(Convert.ToDouble(ahiStr), 1);
                                            validAhiCount++;
                                        }
                                        var hiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "hi")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(hiStr))
                                        {
                                            hi += Math.Round(Convert.ToDouble(hiStr), 1);
                                            validHiCount++;
                                        }
                                        var aiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(aiStr))
                                        {                                
                                            ai += Math.Round(Convert.ToDouble(aiStr), 1);
                                            validAICount++;
                                        }
                                        var oaiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "oai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(oaiStr))
                                        {
                                            oai += Math.Round(Convert.ToDouble(oaiStr), 1);
                                            validOAICount++;
                                        }
                                        var caiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "cai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(caiStr))
                                        {
                                            cai += Math.Round(Convert.ToDouble(caiStr), 1);
                                            validCAICount++;
                                        }
                                        var snore_eStr = findReportEntity.ReportAttributes.Find(c => c.Key == "snore_e")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(snore_eStr))
                                        {
                                            snoreEvent += Math.Round(Convert.ToDouble(snore_eStr), 1);
                                            validSnoreEventCount++;
                                        }
                                        var spo2_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "spo2_ave")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(spo2_aveStr))
                                        {
                                            spo2Ave += Math.Round(Convert.ToDouble(spo2_aveStr), 1);
                                            validSpo2AveCount++;
                                        }
                                    }
                                }
                                if (validUseHourCount != 0)
                                {
                                    model.Value = Math.Round((useHour / validUseHourCount), 2).ToString();
                                }
                                else
                                {
                                    model.Value = "0";
                                }
                                if (validAhiCount != 0)
                                {
                                    modelAHI.Value = Math.Round((ahi / validAhiCount), 2).ToString();
                                }
                                else
                                { 
                                    modelAHI.Value = "0"; 
                                }
                                if (validHiCount != 0)
                                {
                                    modelHI.Value = Math.Round((hi / validHiCount), 2).ToString();
                                }
                                else
                                { 
                                    modelHI.Value = "0"; 
                                }
                                if (validAICount != 0)
                                {
                                    modelAI.Value = Math.Round((ai / validAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelAI.Value = "0"; 
                                }
                                if (validOAICount != 0)
                                {
                                    modelOAI.Value = Math.Round((oai / validOAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelOAI.Value = "0"; 
                                }
                                if (validCAICount != 0)
                                {
                                    modelCAI.Value = Math.Round((cai / validCAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelCAI.Value = "0"; 
                                }
                                if (validSnoreEventCount != 0)
                                {
                                    modelSnoreEvent.Value = Math.Round((snoreEvent / validSnoreEventCount), 2).ToString();
                                }
                                else
                                { 
                                    modelSnoreEvent.Value = "0"; 
                                }
                                if (validSpo2AveCount != 0)
                                {
                                    modelSpo2Ave.Value = Math.Round((spo2Ave / validSpo2AveCount), 2).ToString();
                                }
                                else
                                { 
                                    modelSpo2Ave.Value = "0"; 
                                }
                                result.UseTimeChartData.Add(model);
                                result.AHIChartData.Add(modelAHI);
                                result.HIChartData.Add(modelHI);
                                result.AIChartData.Add(modelAI);
                                result.OAIChartData.Add(modelOAI);
                                result.CAIChartData.Add(modelCAI);
                                result.SnoreEventChartData.Add(modelSnoreEvent);
                                result.Spo2AveChartData.Add(modelSpo2Ave);
                            }
                        }
                        else if (request.ReportType == IOTDeviceStageReportTypeEnum.Year)
                        {
                            foreach(var key in dic.Keys)
                            {
                                var model = new DataItemDTO();
                                var modelAHI = new DataItemDTO();
                                var modelHI = new DataItemDTO();
                                var modelAI = new DataItemDTO();
                                var modelOAI = new DataItemDTO();
                                var modelCAI = new DataItemDTO();
                                var modelSnoreEvent = new DataItemDTO();
                                var modelSpo2Ave = new DataItemDTO();
                                model.Key = key + "月";
                                modelAHI.Key = key + "月";
                                modelHI.Key = key + "月";
                                modelAI.Key = key + "月";
                                modelOAI.Key = key + "月";
                                modelCAI.Key = key + "月";
                                modelSnoreEvent.Key = key + "月"; 
                                modelSpo2Ave.Key = key + "月";
                                var dateList = dic[key];
                                int validUseHourCount = 0;
                                int validAhiCount = 0;
                                int validHiCount = 0;
                                int validAICount = 0;
                                int validOAICount = 0;
                                int validCAICount = 0;
                                int validSnoreEventCount = 0;
                                int validSpo2AveCount = 0;
                                double useHour = 0;
                                double ahi = 0;
                                double hi = 0;
                                double ai = 0;
                                double oai = 0;
                                double cai = 0;
                                double snoreEvent = 0;
                                double spo2Ave = 0;
                                foreach(var dateItem in dateList)
                                {
                                    var tempStartTime = dateItem;
                                    var tempEndTime = dateItem.AddDays(1);

                                    var findReportEntity = reportList.OrderBy(c => c.Timestamp).FirstOrDefault(c => c.Timestamp >= tempStartTime && c.Timestamp < tempEndTime) ?? new DeviceReportEntity();
                                    if (!string.IsNullOrEmpty(findReportEntity?.Code))
                                    {
                                        var useHourStr = findReportEntity.ReportAttributes.Find(c => c.Key == "use_hour")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(useHourStr))
                                        {
                                            useHour += Math.Round(Convert.ToDouble(useHourStr), 1);
                                            validUseHourCount++;
                                        }
                                        var ahiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ahi")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(ahiStr))
                                        {
                                            ahi = Math.Round(Convert.ToDouble(ahiStr), 1);
                                            validAhiCount++;
                                        }
                                        var hiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "hi")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(hiStr))
                                        {
                                            hi += Math.Round(Convert.ToDouble(hiStr), 1);
                                            validHiCount++;
                                        }
                                        var aiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "ai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(aiStr))
                                        {                                
                                            ai += Math.Round(Convert.ToDouble(aiStr), 1);
                                            validAICount++;
                                        }
                                        var oaiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "oai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(oaiStr))
                                        {
                                            oai += Math.Round(Convert.ToDouble(oaiStr), 1);
                                            validOAICount++;
                                        }
                                        var caiStr = findReportEntity.ReportAttributes.Find(c => c.Key == "cai")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(caiStr))
                                        {
                                            cai += Math.Round(Convert.ToDouble(caiStr), 1);
                                            validCAICount++;
                                        }
                                        var snore_eStr = findReportEntity.ReportAttributes.Find(c => c.Key == "snore_e")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(snore_eStr))
                                        {
                                            snoreEvent += Math.Round(Convert.ToDouble(snore_eStr), 1);
                                            validSnoreEventCount++;
                                        }
                                        var spo2_aveStr = findReportEntity.ReportAttributes.Find(c => c.Key == "spo2_ave")?.Value ?? string.Empty;
                                        if (!string.IsNullOrEmpty(spo2_aveStr))
                                        {
                                            spo2Ave += Math.Round(Convert.ToDouble(spo2_aveStr), 1);
                                            validSpo2AveCount++;
                                        }
                                    }
                                }
                                if (validUseHourCount != 0)
                                {
                                    model.Value = Math.Round((useHour / validUseHourCount), 2).ToString();
                                }
                                else
                                {
                                    model.Value = "0";
                                }
                                if (validAhiCount != 0)
                                {
                                    modelAHI.Value = Math.Round((ahi / validAhiCount), 2).ToString();
                                }
                                else
                                {
                                    modelAHI.Value = "0";
                                }
                                if (validHiCount != 0)
                                {
                                    modelHI.Value = Math.Round((hi / validHiCount), 2).ToString();
                                }
                                else
                                { 
                                    modelHI.Value = "0"; 
                                }
                                if (validAICount != 0)
                                {
                                    modelAI.Value = Math.Round((ai / validAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelAI.Value = "0"; 
                                }
                                if (validOAICount != 0)
                                {
                                    modelOAI.Value = Math.Round((oai / validOAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelOAI.Value = "0"; 
                                }
                                if (validCAICount != 0)
                                {
                                    modelCAI.Value = Math.Round((cai / validCAICount), 2).ToString();
                                }
                                else
                                { 
                                    modelCAI.Value = "0"; 
                                }
                                if (validSnoreEventCount != 0)
                                {
                                    modelSnoreEvent.Value = Math.Round((snoreEvent / validSnoreEventCount), 2).ToString();
                                }
                                else
                                { 
                                    modelSnoreEvent.Value = "0"; 
                                }
                                if (validSpo2AveCount != 0)
                                {
                                    modelSpo2Ave.Value = Math.Round((spo2Ave / validSpo2AveCount), 2).ToString();
                                }
                                else
                                { 
                                    modelSpo2Ave.Value = "0"; 
                                }
                                result.UseTimeChartData.Add(model);
                                result.AHIChartData.Add(modelAHI);
                                result.HIChartData.Add(modelHI);
                                result.AIChartData.Add(modelAI);
                                result.OAIChartData.Add(modelOAI);
                                result.CAIChartData.Add(modelCAI);
                                result.SnoreEventChartData.Add(modelSnoreEvent);
                                result.Spo2AveChartData.Add(modelSpo2Ave);
                            }
                        }
                    }
                    return result;
                }
                catch(Exception ex)
                {
                    Logger.WriteLineError($"DeviceInfoDBService GetIOTDeviceStageReportTrendChartDBAsync error, request: {JsonConvert.SerializeObject(request)}, error: {ex}");
                    return result;
                }
            }
        }


    }
}