using System.Collections.Generic;
using System.Threading.Tasks;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using System.Linq;
using JsonRpcLite.Services;
using System;
using WingMongoDBModule.DO;
using WingInterfaceLibrary.DB.Request;
using WingMongoDBModule.Mongodb.Driver;
using WingMongoDBModule.Entity;
using WingInterfaceLibrary.Result;
using WingMongoDBModule.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Request.DBRequest;
using MongoDB.Driver;
using WingServerCommon.Log;
using Newtonsoft.Json.Linq;
using WingServerCommon.Utilities;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Config;
using WingInterfaceLibrary.Request.Management;
using WingServerCommon.Interfaces.FileTransfer;
using System.IO;
using WingInterfaceLibrary.DTO.DataCenter;
using System;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.DTO.Patient;
using WingInterfaceLibrary.Request.DBCopy;
using WingMongoDBModule.Entities;
using WingMongoDBModule.Common;
using MongoDB.Bson;
using MongoDB.Driver.Core.Misc;
using WingInterfaceLibrary.Result.Management;
using System.Text.RegularExpressions;
using MongoDB.Driver.GeoJsonObjectModel;

namespace WingMongoDBModule.Service
{
    public partial class DatabaseService : JsonRpcService, IReportDBService
    {
        /// <summary>
        /// 查询报告信息详情
        /// </summary>
        /// <returns></returns>
        public async Task<ReportDTO> GetReportByCodeAsync(GetReportByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportDBRepository.FindOneAsync("Code", request.ReportCode);
                var reportInfo = entity.MappingTo<ReportDTO>();

                if (reportInfo != null && !string.IsNullOrWhiteSpace(reportInfo.ReportDatasJson))
                {
                    reportInfo.ReportDatasJson = DOShare.ReplaceAll(reportInfo.ReportDatasJson);
                }
                if (!string.IsNullOrWhiteSpace(reportInfo.ReportUserCode))
                {
                    var user = await _userDBRepository.FindOneAsync("Code", reportInfo.ReportUserCode);
                    var reportUserName = string.Empty;
                    if (!string.IsNullOrWhiteSpace(user?.FullName))
                    {
                        reportUserName = user.FullName;
                    }
                    else if (!string.IsNullOrWhiteSpace(user?.UserName))
                    {
                        reportUserName = user.UserName;
                    }
                    reportInfo.ReportUserName = reportUserName;
                }
                reportInfo.EncryptPatientName = reportInfo.PatientName;
                if (!string.IsNullOrWhiteSpace(reportInfo.DeviceCode))
                {
                    var deviceInfo = await _deviceInfoDBRepository.FindOneAsync("Code", reportInfo.DeviceCode);
                    reportInfo.DeviceName = deviceInfo?.Name;
                }
                if (!string.IsNullOrWhiteSpace(reportInfo.ReportTemplateJson))
                {
                    try
                    {
                        dynamic templateInfo = JObject.Parse(reportInfo.ReportTemplateJson);
                        reportInfo.ReportTemplateName = templateInfo?.Name;
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn($"Parse ReportTemplateJson error {ex}");
                    }
                }
                if (!string.IsNullOrWhiteSpace(reportInfo.RecordCode))
                {
                    var remedicalInfo = await _remedicalDBRepository.FindOneAsync("RecordCode", reportInfo.RecordCode);
                    if (remedicalInfo != null)
                    {
                        reportInfo.ExamDate = remedicalInfo.UpdateTime;
                    }
                }
                reportInfo.ReportTemplateCode = reportInfo.ReportTemplateCode ?? string.Empty;
                return reportInfo;
            }
        }

        /// <summary>
        /// 查询报告信息列表
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportDTO>> GetReportListByCodesAsync(QueryReportsByCodesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportListByCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (request.ReportCodes == null || request.ReportCodes.Count <= 0)
                {
                    return new List<ReportDTO>();
                }
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", request.ReportCodes);
                var list = await _reportDBRepository.FindInAsync(dic);
                var reportList = list.MappingTo<List<ReportDTO>>();
                if (reportList?.Count > 0)
                {
                    foreach (var reportInfo in reportList)
                    {
                        if (!string.IsNullOrWhiteSpace(reportInfo.ReportUserCode))
                        {
                            var user = await _userDBRepository.FindOneAsync("Code", reportInfo.ReportUserCode);
                            var reportUserName = string.Empty;
                            if (!string.IsNullOrWhiteSpace(user?.FullName))
                            {
                                reportUserName = user.FullName;
                            }
                            else if (!string.IsNullOrWhiteSpace(user?.UserName))
                            {
                                reportUserName = user.UserName;
                            }
                            reportInfo.ReportUserName = reportUserName;
                        }
                    }
                }
                return reportList;
            }
        }

        /// <summary>
        /// 查询检查关联的报告集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportDTO>> GetReportByRecordCodeAsync(GetReportByRecordCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportByRecordCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                //dic.Add("IsDelete", "False");
                dic.Add("RecordCode", request.RecordCode);
                var entities = await _reportDBRepository.FindListAsync(dic);
                entities = entities.OrderByDescending(e => e.CreateTime).ToList();
                entities.ForEach(report =>
                {
                    report.ReportDatasJson = string.Empty;
                    report.ReportTemplateJson = string.Empty;
                });
                var reportList = entities.MappingTo<List<ReportDTO>>();
                if (reportList?.Count > 0)
                {
                    reportList = await SetReportUserName(reportList);
                }
                return reportList;
            }
        }

        /// <summary>
        /// 查询病人关联的报告集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportDTO>> GetReportByPatientCodeAsync(string patientCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportByPatientCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                //dic.Add("IsDelete", "False");
                dic.Add("PatientCode", patientCode);
                var entities = await _reportDBRepository.FindListAsync(dic);
                entities = entities.OrderByDescending(e => e.CreateTime).ToList();
                entities.ForEach(report =>
                {
                    report.ReportDatasJson = string.Empty;
                    report.ReportTemplateJson = string.Empty;
                });
                var reportList = entities.MappingTo<List<ReportDTO>>();
                if (reportList?.Count > 0)
                {
                    reportList = await SetReportUserName(reportList);
                }
                return reportList;
            }
        }

        private async Task<List<ReportDTO>> SetReportUserName(List<ReportDTO> reportList)
        {
            var dicUser = new Dictionary<string, List<string>>();
            dicUser.Add("Code", reportList.Select(r => r.ReportUserCode).ToList());
            var users = await _userDBRepository.FindInAsync(dicUser);
            foreach (var reportInfo in reportList)
            {
                if (!string.IsNullOrWhiteSpace(reportInfo.ReportUserCode))
                {
                    var user = users.FirstOrDefault(u => u.Code == reportInfo.ReportUserCode);
                    if (!string.IsNullOrWhiteSpace(reportInfo.RecordCode))
                    {
                        var consultationInfo = await _consultationRecordDBRepository.FindOneAsync("Code", reportInfo.RecordCode);
                        if (consultationInfo != null && !string.IsNullOrWhiteSpace(consultationInfo.ExpertUserCode) && consultationInfo.ExpertUserCode != reportInfo.ReportUserCode)
                        {
                            var expertInfo = await _userDBRepository.FindOneAsync("Code", consultationInfo.ExpertUserCode);
                            if (expertInfo != null && expertInfo.BindAssistantDoctorUserCode == reportInfo.ReportUserCode)
                            {
                                user = expertInfo;
                            }
                        }
                    }
                    reportInfo.ReportUserName = GetUserDisplayName(user);
                }
            }
            return reportList;
        }

        private string GetUserDisplayName(UserInfoDO user)
        {
            if (user != null)
            {
                var name = string.IsNullOrWhiteSpace(user.FullName) ? user.UserName : user.FullName;
                if (Regex.IsMatch(name, "[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(user.Phone))
                {
                    name = user.Phone;
                }
                if (Regex.IsMatch(name, "[0-9A-Z]{32}") && !string.IsNullOrWhiteSpace(user.Email))
                {
                    name = user.Email;
                }
                return name;
            }
            return string.Empty;
        }

        /// <summary>
        /// 查询病人关联的报告集合
        /// </summary>
        /// <returns></returns>
        internal async Task<List<ReportDTO>> GetReportByPatientCodesAsync(List<string> patientCodes, DateTime startTime, DateTime endTime)
        {
            if (patientCodes == null || patientCodes.Count == 0)
            {
                return new List<ReportDTO>();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
            queryItemsFilter.AppendEq((item => item.Key, "PatientCode"));
            queryItemsFilter.AppendIn(item => item.Value, patientCodes);
            if (queryItemsFilter.HasFilter)
            {
                customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());
            }
            if (startTime > DateTime.MinValue)
            {
                customerFilter.AppendGte("BaseData.CreateTime", startTime);
            }
            if (endTime > DateTime.MinValue)
            {
                customerFilter.AppendLt("BaseData.CreateTime", endTime);
            }
            var entities = await _reportDBRepository.FindAllAsync(customerFilter.AndCombine());
            return entities.MappingTo<List<ReportDTO>>();
        }

        /// <summary>
        /// 查询设备关联的报告信息
        /// </summary>
        /// <returns></returns>
        public async Task<ReportDTO> GetReportByDeviceReportIdAsync(string deviceReportId)
        {
            if (string.IsNullOrEmpty(deviceReportId))
            {
                return new ReportDTO();
            }
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendEq(("BaseData.DeviceReportId", deviceReportId));
            var entities = await _reportDBRepository.FindOneAsync(customerFilter.AndCombine());
            return entities.MappingTo<ReportDTO>();
        }


        /// <summary>
        /// 查询检查关联的报告集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportDTO>> GetReportsByRecordCodesAsync(List<string> recordCodes)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportsByRecordCodesAsync execute", $"{this.GetType().Name}"))
            {
                if (recordCodes == null || recordCodes.Count == 0)
                {
                    return new List<ReportDTO>();
                }
                var dic = new Dictionary<string, List<string>>();
                dic.Add("RecordCode", recordCodes);
                var entities = await _reportDBRepository.FindInAsync(dic);
                entities.ForEach(report =>
                {
                    report.ReportDatasJson = string.Empty;
                    report.ReportTemplateJson = string.Empty;
                });
                entities = entities.Where(r => r.IsDelete == false).OrderByDescending(r => r.CreateTime).ToList();
                return entities.MappingTo<List<ReportDTO>>();
            }
        }

        /// <summary>
        /// 保存报告
        /// </summary>
        /// <returns></returns>
        public async Task<string> SaveReportAsync(SaveReportDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportAsync execute", $"{this.GetType().Name}"))
            {
                if (request.DiagnosisLabels?.Contains("VAid") == true)
                {
                    var aiReports = await GetAIReportAsync(request.RecordCode);
                    var reportInfo = aiReports.FirstOrDefault(x => x.ReportOrgan == request.ReportOrgan);
                    if (reportInfo != null)
                    {
                        request.ReportCode = reportInfo.ReportCode;
                    }
                }
                if (string.IsNullOrWhiteSpace(request.ReportCode))
                {
                    var entity = new ReportDO();
                    entity.RecordCode = request.RecordCode;
                    entity.ReportUserCode = request.ReportUserCode;
                    entity.DeviceCode = request.DeviceCode;
                    entity.PatientCode = request.PatientCode;
                    entity.ReportTemplateJson = request.ReportTemplateJson;
                    entity.ReportDatasJson = request.ReportDatasJson;
                    entity.PatientId = request.PatientId;
                    entity.PatientName = request.PatientName;
                    entity.ReportLabels = request.ReportLabels ?? new List<string>();
                    entity.PlatformLabels = request.PlatformLabels ?? new List<string>();
                    entity.DiagnosisLabels = request.DiagnosisLabels ?? new List<string>();
                    entity.ReportOrgan = request.ReportOrgan;
                    entity.ReferralRecordCode = request.ReferralRecordCode ?? string.Empty;
                    entity.ReportType = request.ReportType;
                    entity.IsReferral = request.IsReferral;
                    entity.ReportTemplateCode = request.ReportTemplateCode;
                    entity.ReportMeasureDatasJson = request.ReportMeasureDatasJson;
                    entity.DeviceReportId = request.DeviceReportId;
                    entity.ReportExamDataJson = request.ReportExamDataJson;
                    entity.HealthDiagnosis = request.HealthDiagnosis;
                    
                    if (!string.IsNullOrEmpty(request.ReportPDFUrl))
                    {
                        entity.ReportPreviewList = new List<ReportPreviewDO>() {
                            new ReportPreviewDO() {
                                FileToken = request.ReportPDFUrl,
                                ThumbnailToken = "",
                                Language = request.LanguageCode,
                                FileType = UploadFileTypeEnum.PDF
                            }
                        };
                        //更新检查状态
                        var recordInfo = await _recordInfoDBRepository.FindOneAsync("Code", entity.RecordCode);
                        if (!string.IsNullOrEmpty(recordInfo?.Code))
                        {
                            recordInfo.RecordStatus = RecordStatusEnum.Completed;
                            var updateRes = await _recordInfoDBRepository.UpdateOneAsync("Code", entity.RecordCode, recordInfo, string.Empty, true);
                        }
                    }
                    else
                    {
                        entity.ReportPreviewList = new List<ReportPreviewDO>();
                    }

                    var reportCode = await _reportDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
                    if (!string.IsNullOrWhiteSpace(entity.ReferralRecordCode) && !string.IsNullOrWhiteSpace(reportCode))
                    {
                        await SaveReferralReportAsync(entity.ReferralRecordCode, entity.ReportUserCode, reportCode);
                        await RefreshPatientUpdateTime(entity.PatientCode);
                    }
                    return reportCode;
                }
                else
                {
                    var entity = await _reportDBRepository.FindOneAsync("Code", request.ReportCode);
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        entity.RecordCode = request.RecordCode;
                        entity.ReportUserCode = request.ReportUserCode;
                        entity.DeviceCode = request.DeviceCode;
                        entity.PatientCode = request.PatientCode;
                        entity.ReportTemplateJson = request.ReportTemplateJson;
                        entity.ReportDatasJson = request.ReportDatasJson;
                        entity.ReportMeasureDatasJson = request.ReportMeasureDatasJson;
                        entity.PatientId = request.PatientId;
                        entity.PatientName = request.PatientName;
                        entity.ReportTemplateCode = request.ReportTemplateCode;
                        entity.ReportLabels = request.ReportLabels ?? new List<string>();
                        entity.ReportExamDataJson = request.ReportExamDataJson;
                        entity.HealthDiagnosis = request.HealthDiagnosis;
                        //来源标签做追加
                        var platfromLabels = entity.PlatformLabels;
                        if (request.PlatformLabels != null && request.PlatformLabels.Any())
                        {
                            foreach (var label in request.PlatformLabels)
                            {
                                if (!platfromLabels.Contains(label))
                                {
                                    platfromLabels.Add(label);
                                }
                            }
                        }
                        entity.PlatformLabels = platfromLabels;
                        entity.DiagnosisLabels = request.DiagnosisLabels ?? new List<string>();
                        entity.ReportOrgan = request.ReportOrgan;
                        //entity.ReferralRecordCode = request.ReferralRecordCode ?? string.Empty;
                        entity.UpdateTime = DateTime.UtcNow;

                        if (entity.ReportPreviewList?.Exists(c => c.FileType == UploadFileTypeEnum.PDF && c.Language == request.LanguageCode) == true)
                        {
                            foreach (var item in entity.ReportPreviewList)
                            {
                                if (item.FileType == UploadFileTypeEnum.PDF && item.Language == request.LanguageCode && !string.IsNullOrEmpty(request.ReportPDFUrl))
                                {
                                    item.FileToken = request.ReportPDFUrl;
                                }
                            }
                        }
                        else
                        {
                            if (entity.ReportPreviewList?.Count > 0)
                            {
                                if (!string.IsNullOrEmpty(request.ReportPDFUrl))
                                {
                                    entity.ReportPreviewList.Add(new ReportPreviewDO()
                                    {
                                        FileToken = request.ReportPDFUrl,
                                        ThumbnailToken = "",
                                        Language = request.LanguageCode,
                                        FileType = UploadFileTypeEnum.PDF
                                    });
                                }
                            }
                            else
                            {
                                if (!string.IsNullOrEmpty(request.ReportPDFUrl))
                                {
                                    entity.ReportPreviewList = new List<ReportPreviewDO>() {
                                        new ReportPreviewDO() {
                                            FileToken = request.ReportPDFUrl,
                                            ThumbnailToken = "",
                                            Language = request.LanguageCode,
                                            FileType = UploadFileTypeEnum.PDF
                                        }
                                    };
                                }
                                else
                                {
                                    entity.ReportPreviewList = new List<ReportPreviewDO>();
                                }
                            }
                        }

                        await _reportDBRepository.UpdateOneAsync("Code", request.ReportCode, entity, string.Empty);
                        if (!string.IsNullOrWhiteSpace(entity.ReferralRecordCode))
                        {
                            await RefreshPatientUpdateTime(entity.PatientCode);
                        }
                        return request.ReportCode;
                    }
                }
                return string.Empty;
            }
        }




        public async Task<string> SaveReportQualityInfoAsync(SaveReportQualityInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportQualityInfo execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportDBRepository.FindOneAsync("Code", request.ReportCode);

                entity.QualifiedState = request.QualifiedState;
                entity.QualityType = request.QualityType;
                entity.QualityControlDatas = request.QualityControlDatas;
                entity.UpdateTime = DateTime.UtcNow;
                await _reportDBRepository.UpdateOneAsync("Code", request.ReportCode, entity, string.Empty);
                return request.ReportCode;
            }
        }


        /// <summary>
        /// 保存转诊报告关联信息
        /// </summary>
        /// <param name="referralCode"></param>
        /// <param name="userCode"></param>
        /// <param name="reportCode"></param>
        /// <returns></returns>
        private async Task SaveReferralReportAsync(string referralCode, string userCode, string reportCode)
        {
            try
            {
                var referralRecordList = new List<ReferralRecordDO>();
                var referralEntity = await _referralRecordDBRepository.FindOneAsync("Code", referralCode);
                if (referralEntity.ReferralCodeList?.Any() == true)
                {
                    var dic = new Dictionary<string, List<string>>();
                    dic.Add("Code", referralEntity.ReferralCodeList);
                    var referralRecords = await _referralRecordDBRepository.FindInAsync(dic);
                    if (referralRecords.Any())
                    {
                        referralRecordList.AddRange(referralRecords);
                    }
                }
                foreach (var entity in referralRecordList)
                {
                    var createReportCodeList = entity.CreateReportCodeList;
                    createReportCodeList.Add(new DO.CreateReportClass
                    {
                        ReferralCode = referralCode,
                        CreatorCode = userCode,
                        ReportCode = reportCode,
                    });
                    await _referralRecordDBRepository.UpdateOneAsync("Code", entity.Code, entity, string.Empty);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"save referral record CreateReportCodeList failed, err:{ex}");
            }
        }

        /// <summary>
        /// 修改患者更新时间
        /// </summary>
        /// <param name="patientCode"></param>
        /// <returns></returns>
        private async Task RefreshPatientUpdateTime(string patientCode)
        {
            try
            {
                var entity = await _patientInfoDBRepository.FindOneAsync("Code", patientCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.UpdateTime = DateTime.UtcNow;
                    await _patientInfoDBRepository.UpdateOneAsync("Code", patientCode, entity, string.Empty);
                    // await AddPushPatientAsync(entity);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"save referral record PatientUpdateTime failed, err:{ex}");
            }
        }

        private async Task<List<ReportDO>> GetAIReportAsync(string recordCode)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();

            queryItemsFilter.AppendEq((item => item.Key, "RecordCode"));
            queryItemsFilter.AppendEq((item => item.Value, recordCode));
            customerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemsFilter.AndCombine());

            customerFilter.AppendArrayEq("BaseData.DiagnosisLabels", "VAid");

            SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
            SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime");
            var entities = await _reportDBRepository.FindAllAsync(customerFilter.AndCombine(), new FindOptions<MongoTable> { NoCursorTimeout = true, Sort = sort });

            return entities;
        }

        /// <summary>
        /// 保存报告关联标签
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveReportLabelsAsync(SaveReportLabelsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportLabelsAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportDBRepository.FindOneAsync("Code", request.ReportCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    entity.ReportLabels = request.ReportLabels;
                    return await _reportDBRepository.UpdateOneAsync("Code", request.ReportCode, entity, string.Empty);
                }
                return false;
            }
        }

        /// <summary>
        /// 保存报告预览图
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveReportPreviewImageAsync(SaveReportPreviewImageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportPreviewImageAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportDBRepository.FindOneAsync("Code", request.ReportCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    var reportPreviewList = entity.ReportPreviewList ?? new List<ReportPreviewDO>();
                    var thisMode = reportPreviewList.FirstOrDefault(x => x.Language == request.Language && x.FileType == request.FileType);
                    if (thisMode != null)
                    {
                        reportPreviewList.Remove(thisMode);
                    }
                    reportPreviewList.Add(new ReportPreviewDO
                    {
                        Language = request.Language,
                        FileType = request.FileType,
                        FileToken = request.FileToken,
                        ThumbnailToken = request.ThumbnailToken,
                    });
                    entity.ReportPreviewList = reportPreviewList;
                    var result = await _reportDBRepository.UpdateOneAsync("Code", request.ReportCode, entity, string.Empty);
                    if (request.FileType == UploadFileTypeEnum.JPG)
                    {
                        await AddPushReportAsync(entity);
                    }
                    return result;
                }
                return false;
            }
        }

        /// <summary>
        /// 保存报告模板预览图
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveReportTemplatePreviewImageAsync(SaveReportPreviewImageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportTemplatePreviewImageAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportTemplateDBRepository.FindOneAsync("Code", request.ReportCode);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    var reportPreviewList = entity.TemplatePreviewList ?? new List<ReportPreviewDO>();
                    var thisMode = reportPreviewList.FirstOrDefault(x => x.Language == request.Language && x.FileType == request.FileType);
                    if (thisMode != null)
                    {
                        reportPreviewList.Remove(thisMode);
                    }
                    reportPreviewList.Add(new ReportPreviewDO
                    {
                        Language = request.Language,
                        FileType = request.FileType,
                        FileToken = request.FileToken,
                        ThumbnailToken = request.ThumbnailToken,
                    });
                    entity.TemplatePreviewList = reportPreviewList;
                    return await _reportTemplateDBRepository.UpdateOneAsync("Code", request.ReportCode, entity, string.Empty, true);
                }
                return false;
            }
        }

        /// <summary>
        /// 分页查询报告标签
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ReportLabelDTO>> GetReportLabelByPageAsync(GetReportLabelByPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportLabelByPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();

                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                //eqFilters.Add(("IsDelete", "False"));
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    regexFilters.Add(("LabelName", request.Keyword, true, false));
                }

                if (regexFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                var pageResult = await _reportLabelDBRepository.GetPages(dbPage);
                return PageResultConvert<ReportLabelDO, ReportLabelDTO>(pageResult);
            }
        }

        /// <summary>
        /// 查询报告标签详情
        /// </summary>
        /// <returns></returns>
        public async Task<ReportLabelDTO> GetReportLabelByCodeAsync(GetReportLabelByCodeDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportLabelByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportLabelDBRepository.FindOneAsync("Code", request.ReportLabelCode);
                return entity.MappingTo<ReportLabelDTO>();
            }
        }

        /// <summary>
        /// 查询报告标签集合
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportLabelDTO>> GetReportLabelsAsync(GetReportLabelsDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportLabelsAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("IsDelete", "False");
                dic.Add("LanguageCode", request.LanguageCode);
                dic.Add("UsageType", request.UsageType.ToString());
                var entities = await _reportLabelDBRepository.FindListAsync(dic);
                return entities.OrderBy(x => x.CreateTime).MappingTo<List<ReportLabelDTO>>();
            }
        }

        /// <summary>
        /// 删除报告标签
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeleteReportLabelAsync(DeleteReportLabelDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} DeleteReportLabelAsync execute", $"{this.GetType().Name}"))
            {
                return await _reportLabelDBRepository.DeleteOneAsync("Code", request.ReportLabelCode);
            }
        }

        /// <summary>
        /// 保存报告标签
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveReportLabelAsync(SaveReportLabelDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} SaveReportLabelAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.ReportLabelCode))
                {
                    var entity = new ReportLabelDO();
                    entity.LabelName = request.LabelName;
                    entity.LanguageCode = request.LanguageCode;
                    entity.UsageType = request.UsageType;
                    entity.LabelItems = request.LabelItems;
                    var reportLabelCode = await _reportLabelDBRepository.InsertOneAsync(entity, new Dictionary<string, string>(), string.Empty);
                    return !string.IsNullOrWhiteSpace(reportLabelCode);
                }
                else
                {
                    var entity = await _reportLabelDBRepository.FindOneAsync("Code", request.ReportLabelCode);
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        entity.LabelName = request.LabelName;
                        entity.LanguageCode = request.LanguageCode;
                        entity.UsageType = request.UsageType;
                        entity.LabelItems = request.LabelItems;
                        return await _reportLabelDBRepository.UpdateOneAsync("Code", request.ReportLabelCode, entity, string.Empty);
                    }
                }
                return false;
            }
        }

        /// <summary>
        /// 拷贝报告模板
        /// </summary>
        /// <param name="reportTemplateCode">报告模板编码</param>
        /// <param name="userCode">用户编码</param>
        /// <param name="organizationCode">机构编码</param>
        /// <returns></returns>
        public async Task<string> CopyReportTemplateAsync(string reportTemplateCode, string userCode, string organizationCode, bool isDefault = false)
        {
            using (new Performance(_performanceThreshold, "{0} CopyReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(reportTemplateCode))
                {
                    return string.Empty;
                }
                if (string.IsNullOrWhiteSpace(userCode) && string.IsNullOrWhiteSpace(organizationCode))
                {
                    return string.Empty;
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", reportTemplateCode));
                var filter = mongoCustomerFilter.AndCombine();
                var doData = await _reportTemplateDBRepository.FindTableOneAsync(filter);
                if (doData != null && doData.BaseData != null)
                {
                    var newData = (ReportTemplateDO)doData.BaseData;
                    newData.Code = string.Empty;
                    newData.IsDefault = isDefault;
                    newData.OrganizationCode = organizationCode;
                    newData.PersonalCode = userCode;
                    newData.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                    if (!string.IsNullOrWhiteSpace(organizationCode))
                    {
                        newData.ReportTemplateType = ReportTemplateTypeEnum.Organization;
                    }
                    else if (!string.IsNullOrWhiteSpace(userCode))
                    {
                        newData.ReportTemplateType = ReportTemplateTypeEnum.Personal;
                    }
                    newData.ReferenceReportTemplateCode = reportTemplateCode;
                    MongoCustomerFilter<MongoTable> copyListFilter = new();
                    copyListFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", newData.ReportTemplateName + "-", true, false));
                    var copyfilter = copyListFilter.AndCombine();
                    var copyList = await _reportTemplateDBRepository.FindAllAsync(copyfilter);
                    var maxCopyNum = 1;
                    if (copyList != null && copyList.Count > 0)
                    {
                        maxCopyNum = copyList.Max(f =>
                        {
                            var splitArray = f.ReportTemplateName.Split("-");
                            if (splitArray.Length > 1)
                            {
                                int.TryParse(splitArray.LastOrDefault(), out int copyCount);
                                return copyCount + 1;
                            }
                            return 1;
                        });
                    }
                    newData.ReportTemplateName = $"{newData.ReportTemplateName}-{maxCopyNum}";
                    var platDic = new Dictionary<string, string>();
                    if (doData.PlatformData != null && doData.PlatformData.PlatformItems != null)
                    {
                        foreach (var plat in doData.PlatformData.PlatformItems)
                        {
                            platDic.Add(plat.Key, plat.Value);
                        }
                    }
                    try
                    {
                        JObject json = JObject.Parse(newData.ReportTemplateJson);
                        json["Name"] = newData.ReportTemplateName;
                        newData.ReportTemplateJson = Newtonsoft.Json.JsonConvert.SerializeObject(json);

                        var dataJson = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportTemplateItemDO>>(newData.ReportDatasJson);
                        if (newData.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal && dataJson?.Count > 0)
                        {
                            dataJson.ForEach(c =>
                            {
                                if (c.Name == "")
                                {
                                    c.Value = LanguageSetting.GetParammeter<StringParameter>(c.AILanguage, "Language", "AIPetSexFemale").Value;
                                }
                            });
                        }
                        newData.ReportDatasJson = Newtonsoft.Json.JsonConvert.SerializeObject(dataJson);
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineError($"Serialize ReportTemplateJson error {ex}");
                    }
                    return await _reportTemplateDBRepository.InsertOneAsync(newData, platDic, doData.ExtendsData);
                }
                return string.Empty;
            }
        }

        /// <summary>
        /// 获取个人报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> ManagementReportTemplatePageAsync(FindReportTemplatePagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} ManagementReportTemplatePageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                List<FilterDefinition<MongoDataItem>> orFilterList = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                List<(string, string)> eqFilters = new();
                if (!string.IsNullOrWhiteSpace(request.ReportTemplateName))
                {
                    // regexFilters.Add(("ReportTemplateName", request.ReportTemplateName, true, false));
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ReportTemplateName"));
                    queryItemsFilter.AppendRegex(item => item.Value, request.ReportTemplateName, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());

                    var orgs = await GetOrganizationDOListByKeyword(request.ReportTemplateName);
                    if (orgs?.Count > 0)
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                        queryItemsFilter.AppendIn(item => item.Value, orgs.Select(x => x.Code).ToList());
                        queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                    }

                    var users = await GetUserDOListByKeyword(request.ReportTemplateName);
                    if (users?.Count > 0)
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "PersonalCode"));
                        queryItemsFilter.AppendIn(item => item.Value, users.Select(x => x.Code).ToList());
                        queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                    }
                }
                if (request.ReportTemplateStateType != null)
                {
                    eqFilters.Add(("ReportTemplateStateType", Enum.GetName(request.ReportTemplateStateType.Value)));
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    eqFilters.Add(("OrganizationCode", request.OrganizationCode));
                }
                if (request.ReportTemplateType != null)
                {
                    eqFilters.Add(("ReportTemplateType", Enum.GetName(request.ReportTemplateType.Value)));
                }
                else
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ReportTemplateType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ReportTemplateTypeEnum.Default)));
                    orFilterList.Add(queryItemsFilter.AndCombine());

                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ReportTemplateType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ReportTemplateTypeEnum.Organization)));
                    orFilterList.Add(queryItemsFilter.AndCombine());

                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ReportTemplateType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ReportTemplateTypeEnum.Personal)));
                    orFilterList.Add(queryItemsFilter.AndCombine());
                }
                if (!string.IsNullOrWhiteSpace(request.LanguageCode))
                {
                    mongoCustomerFilter.AppendEq(("BaseData.LanguageCode", request.LanguageCode));
                }
                if (request.IsDefault == true)
                {
                    eqFilters.Add(("IsDefault", request.IsDefault.Value.ToString()));
                }
                if (queryItemsFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                    mongoCustomerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                if (orFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(orFilterList.ToArray());
                    mongoCustomerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime").Ascending("BaseData.ReportTemplateUsageType").Descending("BaseData.Code");
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _reportTemplateDBRepository.GetPages(dbPage);
                pageResult.PageData.ForEach(data =>
                {
                    //列表接口不返回Json
                    data.ReportTemplateJson = string.Empty;
                });
                return PageResultConvert<ReportTemplateDO, ReportTemplateDTO>(pageResult);
            }
        }

        /// <summary>
        /// 获取报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetReportTemplatePageAsync(FindReportTemplatePagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportTemplatePageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoDataItem> mongoCustomerSubFilter = new();
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                List<(string, string)> orEqFilters = new();
                List<(string, string)> orEqFilters2 = new();

                if (!string.IsNullOrWhiteSpace(request.ReportTemplateName))
                {
                    regexFilters.Add(("ReportTemplateName", request.ReportTemplateName, true, false));
                }
                else
                {
                    if (request.BusinessType == ReportTemplateBusinessTypeEnum.HEIUltrasonic)
                    {
                        regexFilters.Add(("ReportTemplateName", "Ultrasonic", true, false));
                    }
                    else if (request.BusinessType == ReportTemplateBusinessTypeEnum.HEIECG)
                    {
                        regexFilters.Add(("ReportTemplateName", "ECG", true, false));
                    }
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    var organizationInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                    orEqFilters.Add(("OrganizationCode", request.OrganizationCode));
                    orEqFilters.Add(("OrganizationCode", organizationInfo.RootCode));
                }
                orEqFilters.Add(("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Default)));
                eqFilters.Add(("LanguageCode", request.LanguageCode));
                if (!string.IsNullOrWhiteSpace(request.PersonalCode))
                {
                    orEqFilters.Add(("PersonalCode", request.PersonalCode));
                }
                if (request.IsDefault == true || request.IsDefault == null)
                {
                    eqFilters.Add(("IsDefault", bool.TrueString));
                }
                if (request.ReportTemplateStateType != null)
                {
                    eqFilters.Add(("ReportTemplateStateType", Enum.GetName(request.ReportTemplateStateType.Value)));
                }
                else
                {
                    eqFilters.Add(("ReportTemplateStateType", Enum.GetName(ReportTemplateStatusTypeEnum.Published)));
                }
                orEqFilters2.Add(("ReportTemplateUsageType", Enum.GetName(ReportTemplateUsageTypeEnum.People)));
                orEqFilters2.Add(("ReportTemplateUsageType", Enum.GetName(ReportTemplateUsageTypeEnum.Animal)));

                if (request.ReportTemplateType != null)
                {
                    eqFilters.Add(("ReportTemplateType", Enum.GetName(request.ReportTemplateType.Value)));
                }
                eqFilters.Add(("IsDelete", "False"));
                mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters2.ToArray());
                var filter = mongoCustomerFilter.AndCombine();
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ReportTemplateType").Descending("BaseData.CreateTime").Ascending("BaseData.ReportTemplateUsageType").Descending("BaseData.Code");
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _reportTemplateDBRepository.GetPages(dbPage);
                pageResult.PageData.ForEach(data =>
                {
                    //列表接口不返回Json
                    data.ReportTemplateJson = string.Empty;
                });
                return PageResultConvert<ReportTemplateDO, ReportTemplateDTO>(pageResult);

            }
        }

        /// <summary>
        /// 获取机构报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetOrgReportTemplatePageAsync(FindReportTemplatePagesDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrgReportTemplatePageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoDataItem> mongoCustomerSubFilter = new();
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                List<(string, string)> orEqFilters = new();
                List<(string, string)> orEqFilters2 = new();

                if (!string.IsNullOrWhiteSpace(request.ReportTemplateName))
                {
                    regexFilters.Add(("ReportTemplateName", request.ReportTemplateName, true, false));
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    var organizationInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                    orEqFilters.Add(("OrganizationCode", request.OrganizationCode));
                    orEqFilters.Add(("OrganizationCode", organizationInfo.RootCode));
                }
                orEqFilters.Add(("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Default)));
                eqFilters.Add(("LanguageCode", request.LanguageCode));
                if (!string.IsNullOrWhiteSpace(request.PersonalCode))
                {
                    orEqFilters.Add(("PersonalCode", request.PersonalCode));
                }
                if (request.IsDefault == true || request.IsDefault == null)
                {
                    eqFilters.Add(("IsDefault", bool.TrueString));
                }
                if (request.ReportTemplateStateType != null)
                {
                    eqFilters.Add(("ReportTemplateStateType", Enum.GetName(request.ReportTemplateStateType.Value)));
                }
                else
                {
                    eqFilters.Add(("ReportTemplateStateType", Enum.GetName(ReportTemplateStatusTypeEnum.Published)));
                }
                orEqFilters2.Add(("ReportTemplateUsageType", Enum.GetName(ReportTemplateUsageTypeEnum.People)));
                orEqFilters2.Add(("ReportTemplateUsageType", Enum.GetName(ReportTemplateUsageTypeEnum.Animal)));

                if (request.ReportTemplateType != null)
                {
                    eqFilters.Add(("ReportTemplateType", Enum.GetName(request.ReportTemplateType.Value)));
                }
                eqFilters.Add(("IsDelete", "False"));
                mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters2.ToArray());
                var filter = mongoCustomerFilter.AndCombine();
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ReportTemplateType").Descending("BaseData.CreateTime").Ascending("BaseData.ReportTemplateUsageType").Descending("BaseData.Code");
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _reportTemplateDBRepository.GetPages(dbPage);
                pageResult.PageData.ForEach(data =>
                {
                    //列表接口不返回Json
                    data.ReportTemplateJson = string.Empty;
                });
                return PageResultConvert<ReportTemplateDO, ReportTemplateDTO>(pageResult);
            }
        }

        /// <summary>
        /// 添加报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<GetReportTemplateDBRequest> AddReportTemplateAsync(AddReportTemplateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                var resp = new GetReportTemplateDBRequest();
                var reportTemplate = request.Data.MappingTo<ReportTemplateDO>();
                reportTemplate.ReportTemplateName = reportTemplate.ReportTemplateName.Trim();
                reportTemplate.ReportDatasJson = reportTemplate.ReportDatasJson ?? "{}";
                if (reportTemplate.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal && !string.IsNullOrEmpty(reportTemplate.ReportDatasJson) && reportTemplate.ReportDatasJson != "{}")
                {
                    var dataJson = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportTemplateItemDO>>(reportTemplate.ReportDatasJson);
                    if (dataJson?.Count > 0)
                    {
                        dataJson.ForEach(c =>
                        {
                            if (c.Name == "")
                            {
                                c.Value = LanguageSetting.GetParammeter<StringParameter>(c.AILanguage, "Language", "AIPetSexFemale").Value;
                            }
                        });
                    }
                    reportTemplate.ReportDatasJson = Newtonsoft.Json.JsonConvert.SerializeObject(dataJson);
                }
                reportTemplate.ReportTemplateJson = reportTemplate.ReportTemplateJson ?? "{}";
                MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
                filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", reportTemplate.ReportTemplateName), ("IsDelete", "False"), ("ReportTemplateStateType", Enum.GetName(ReportTemplateStatusTypeEnum.Published)), ("IsDefault", "True"));
                var list = await _reportTemplateDBRepository.FindAllAsync(filter.AndCombine());
                if (list == null || list.Count == 0)
                {
                    if (!string.IsNullOrEmpty(reportTemplate.PersonalCode) && reportTemplate.ReportTemplateType == ReportTemplateTypeEnum.Personal)
                    {
                        reportTemplate.OrganizationCode = "";
                    }
                    var result = await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, request.PlatformDatas, request.ExtensionData);
                    return new GetReportTemplateDBRequest()
                    {
                        ReportTemplateCode = result,
                        LanguageCode = reportTemplate.LanguageCode,
                    };
                }
                else
                {
                    if (list.Exists(a => (a.PersonalCode == request.Data.PersonalCode && !string.IsNullOrEmpty(request.Data.PersonalCode)) || (!string.IsNullOrEmpty(request.Data.OrganizationCode) && a.OrganizationCode == (request.Data.OrganizationCode ?? "")) || a.ReportTemplateType == ReportTemplateTypeEnum.Default && a.ReportTemplateStateType == ReportTemplateStatusTypeEnum.Published))
                    {
                        ThrowCustomerException(CustomerRpcCode.ReportTemplateNameDuplication, "ReportTemplateName duplication");
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(reportTemplate.PersonalCode) && reportTemplate.ReportTemplateType == ReportTemplateTypeEnum.Personal)
                        {
                            reportTemplate.OrganizationCode = "";
                        }
                        var result = await _reportTemplateDBRepository.InsertOneAsync(reportTemplate, request.PlatformDatas, request.ExtensionData);
                        return new GetReportTemplateDBRequest()
                        {
                            ReportTemplateCode = result,
                            LanguageCode = reportTemplate.LanguageCode,
                        };
                    }
                }
                ThrowCustomerException(CustomerRpcCode.ReportTemplateNameDuplication, "ReportTemplateName duplication");
                return resp;
            }
        }

        /// <summary>
        /// 验证报告模板名称是否重复
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CheckReportTemplateName(UpdateReportTemplateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} CheckReportTemplateName execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.Data.ReportTemplateCode))
                {
                    return false;
                }
                JObject json = JObject.Parse(request.Data.ReportTemplateJson);
                var reportTemplateName = json["Name"].ToString();
                MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
                filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", reportTemplateName), ("IsDelete", "False"));
                var list = await _reportTemplateDBRepository.FindAllAsync(filter.AndCombine());
                if (list == null)//没找到这个名字，返回true
                {
                    return true;
                }
                else if (list.Count == 0 || !list.Any(a => a.ReportTemplateCode != request.Data.ReportTemplateCode && a.PersonalCode == request.Data.PersonalCode && a.OrganizationCode == (request.Data.OrganizationCode ?? "")))
                {
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// 更新报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<GetReportTemplateDBRequest> UpdateReportTemplateAsync(UpdateReportTemplateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                var resp = new GetReportTemplateDBRequest();
                bool needUpdate = false;
                if (string.IsNullOrWhiteSpace(request.Data.ReportTemplateCode))
                {
                    return resp;
                }
                var reportTemplate = await _reportTemplateDBRepository.FindOneAsync("Code", request.Data.ReportTemplateCode);
                if (reportTemplate == null)
                {
                    return resp;
                }
                if (reportTemplate.IsDefault != request.Data.IsDefault)
                {
                    reportTemplate.IsDefault = request.Data.IsDefault;
                    needUpdate = true;
                }
                if (reportTemplate.LanguageCode != request.Data.LanguageCode && !string.IsNullOrEmpty(request.Data.LanguageCode))
                {
                    reportTemplate.LanguageCode = request.Data.LanguageCode;
                    needUpdate = true;
                }
                if (reportTemplate.ReportTemplateName != null)
                {
                    reportTemplate.ReportTemplateName = reportTemplate.ReportTemplateName.Trim();
                    if (!string.IsNullOrWhiteSpace(request.Data.ReportTemplateName) && reportTemplate.ReportTemplateName != request.Data.ReportTemplateName)
                    {
                        needUpdate = true;
                    }
                }
                else
                {
                    if (!string.IsNullOrWhiteSpace(request.Data.ReportTemplateName))
                    {
                        reportTemplate.ReportTemplateName = request.Data.ReportTemplateName;
                        needUpdate = true;
                    }
                }
                bool needUpdateTempateName = false;
                if (!string.IsNullOrWhiteSpace(request.Data.ReportTemplateJson) && reportTemplate.ReportTemplateJson != request.Data.ReportTemplateJson)
                {
                    reportTemplate.ReportTemplateJson = request.Data.ReportTemplateJson ?? "{}";
                    reportTemplate.ReportDatasJson = reportTemplate.ReportDatasJson ?? "{}";
                    if (!string.IsNullOrEmpty(request.Data.ReportDatasJson))
                    {
                        var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementStringDTO>>(request.Data.ReportDatasJson);
                        if (reportElements?.Count > 0)
                        {
                            // var cosHost = ConfigurationManager.GetParammeter<StringParameter>("Storage", "CDNServerUrl").Value;
                            reportElements.ForEach(d =>
                            {
                                if (d.Name == "ReportImages")
                                {
                                    var tempList = d.Value.ToString().Split(';').ToList();
                                    var valList = new List<string>();
                                    tempList.ForEach(e =>
                                    {
                                        //valList.Add((cosHost + "/" + e.Replace("img/", "")));
                                        valList.Add(e);
                                    });
                                    d.Value = valList;
                                }
                                if (reportTemplate.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                                {
                                    if (d.Name == "PatientGender")
                                    {
                                        d.Value = LanguageSetting.GetParammeter<StringParameter>(d.AILanguage, "Language", "AIPetSexFemale").Value;
                                    }
                                }
                            });
                            reportTemplate.ReportDatasJson = Newtonsoft.Json.JsonConvert.SerializeObject(reportElements);
                        }
                    }

                    JObject json = JObject.Parse(reportTemplate.ReportTemplateJson);
                    if (json["Name"].ToString() != reportTemplate.ReportTemplateName)
                    {
                        needUpdateTempateName = true;
                        reportTemplate.ReportTemplateName = json["Name"].ToString();
                    }
                    reportTemplate.ReportTemplateUsageType = json["Applicable"].ToString() == "Person" ? ReportTemplateUsageTypeEnum.People : ReportTemplateUsageTypeEnum.Animal;

                    needUpdate = true;
                }
                if (string.IsNullOrEmpty(request.Data.ReportDatasJson) && string.IsNullOrEmpty(request.Data.ReportTemplateJson))
                {
                    reportTemplate.ReportTemplateName = request.Data.ReportTemplateName;
                    reportTemplate.ReportTemplateType = request.Data.ReportTemplateType;
                    reportTemplate.ReportTemplateUsageType = request.Data.ReportTemplateUsageType;
                    reportTemplate.OrganizationCode = request.Data.OrganizationCode;
                    reportTemplate.PersonalCode = request.Data.PersonalCode;
                    reportTemplate.LanguageCode = request.Data.LanguageCode;
                    reportTemplate.IsDefault = request.Data.IsDefault;
                    var res = await _reportTemplateDBRepository.UpdateOneAsync("Code", request.Data.ReportTemplateCode, reportTemplate, request.ExtensionData, true);
                    return new GetReportTemplateDBRequest()
                    {
                        ReportTemplateCode = reportTemplate.ReportTemplateCode,
                        LanguageCode = reportTemplate.LanguageCode,
                    };
                }
                if (needUpdate)
                {
                    if (needUpdateTempateName)
                    {
                        MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
                        filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", reportTemplate.ReportTemplateName), ("IsDelete", "False"));
                        var list = await _reportTemplateDBRepository.FindAllAsync(filter.AndCombine());
                        if (list == null)
                        {
                            var result = await _reportTemplateDBRepository.UpdateOneAsync("Code", request.Data.ReportTemplateCode, reportTemplate, request.ExtensionData, true);
                            return new GetReportTemplateDBRequest()
                            {
                                ReportTemplateCode = reportTemplate.ReportTemplateCode,
                                LanguageCode = reportTemplate.LanguageCode,
                            };
                        }
                        else if (list.Count == 0 || !list.Any(a => a.PersonalCode == request.Data.PersonalCode && a.OrganizationCode == (request.Data.OrganizationCode ?? "")))
                        {
                            var result = await _reportTemplateDBRepository.UpdateOneAsync("Code", request.Data.ReportTemplateCode, reportTemplate, request.ExtensionData, true);
                            return new GetReportTemplateDBRequest()
                            {
                                ReportTemplateCode = reportTemplate.ReportTemplateCode,
                                LanguageCode = reportTemplate.LanguageCode,
                            };
                        }
                        ThrowCustomerException(CustomerRpcCode.ReportTemplateNameDuplication, "ReportTemplateName duplication");
                    }
                    var res = await _reportTemplateDBRepository.UpdateOneAsync("Code", request.Data.ReportTemplateCode, reportTemplate, request.ExtensionData, true);
                    return new GetReportTemplateDBRequest()
                    {
                        ReportTemplateCode = reportTemplate.ReportTemplateCode,
                        LanguageCode = reportTemplate.LanguageCode,
                    };
                }
                return resp;
            }
        }

        /// <summary>
        /// 更新报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveReportTemplateAsync(RemoveReportTemplateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                var res = await _reportTemplateDBRepository.DeleteOneAsync("Code", request.ReportTemplateCode);
                //根据词条code和用户code查询相关数据，有关系的则清空
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                if (!string.IsNullOrEmpty(request.UserCode))
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", request.UserCode));
                }
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultReportTemplateCode", request.ReportTemplateCode));
                var filter = mongoCustomerFilter.AndCombine();
                var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
                if (findTable != null)
                {
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.SetField("BaseData.DefaultReportTemplateCode", "");
                    var queryItems = findTable.QueryData.QueryItems;
                    queryItems.RemoveWhere(r => r.Key == "DefaultReportTemplateCode");
                    queryItems.Add(new MongoDataItem { Key = "DefaultReportTemplateCode", Value = "" });
                    update.SetField("QueryData.QueryItems", queryItems);
                    res = await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                }
                if (!string.IsNullOrEmpty(request.OrganizationCode))
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.OrganizationCode));
                    customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultReportTemplateCode", request.ReportTemplateCode));
                    var orgFilter = customerFilter.AndCombine();
                    var queryTable = await _organizationDBRepository.FindTableOneAsync(orgFilter);
                    if (queryTable != null)
                    {
                        MongoCustomerUpdate<MongoTable> update = new();
                        update.SetField("BaseData.DefaultReportTemplateCode", "");
                        var queryItems = queryTable.QueryData.QueryItems;
                        queryItems.RemoveWhere(r => r.Key == "DefaultReportTemplateCode");
                        queryItems.Add(new MongoDataItem { Key = "DefaultReportTemplateCode", Value = "" });
                        update.SetField("QueryData.QueryItems", queryItems);
                        res = await _organizationDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 获取报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetReportTemplateAsync(GetReportTemplateDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                var result = await _reportTemplateDBRepository.FindOneAsync("Code", request.ReportTemplateCode);
                return result.MappingTo<ReportTemplateDTO>();
            }
        }

        /// <summary>
        /// 根据名称获取vital报告模板
        /// </summary>
        /// <param name="reportTemplateName"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetVitalReportTemplateByNameAsync(string reportTemplateName)
        {
            using (new Performance(_performanceThreshold, "{0} GetVitalReportTemplateByNameAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("ReportTemplateName", reportTemplateName);
                dic.Add("ReportTemplateStateType", "Published");
                dic.Add("ReportTemplateType", "Default");
                dic.Add("ReportTemplateUsageType", "People");
                dic.Add("LanguageCode", "zh-CN");
                dic.Add("IsDelete", "False");
                var result = await _reportTemplateDBRepository.FindListAsync(dic);
                if (result?.Count > 0)
                {
                    return result.FirstOrDefault().MappingTo<ReportTemplateDTO>();
                }
                else
                {
                    return new ReportTemplateDTO();
                }
            }
        }

        /// <summary>
        /// 发布报告模板
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> PublishReportTemplateAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} PublishReportTemplateAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportTemplateDBRepository.FindOneAsync("Code", code);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (string.IsNullOrWhiteSpace(entity.ReportTemplateJson) || entity.ReportTemplateJson == "{}")
                    {
                        ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJsonEmpty");
                    }
                    if (entity.ReportTemplateStateType == ReportTemplateStatusTypeEnum.Unpublished)
                    {
                        entity.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                        return await _reportTemplateDBRepository.UpdateOneAsync("Code", code, entity, string.Empty, true);
                    }
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// 查询报告模板
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetReportTemplateByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportTemplateByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _reportTemplateDBRepository.FindOneAsync("Code", code);
                return entity.MappingTo<ReportTemplateDTO>();
            }
        }
        /// <summary>
        /// 添加词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusAsync(AddThesaurusDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddThesaurusAsync execute", $"{this.GetType().Name}"))
            {
                var thesaurus = request.Data.MappingTo<ThesaurusDO>();
                thesaurus.ThesaurusName = thesaurus.ThesaurusName.Trim();
                MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
                filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ThesaurusName", thesaurus.ThesaurusName), ("IsDelete", "False"), ("LanguageCode", request.Data.LanguageCode));
                var list = await _thesaurusDBRepository.FindAllAsync(filter.AndCombine());
                if (list == null)
                {
                    if (!string.IsNullOrEmpty(thesaurus.PersonalCode) && thesaurus.ThesaurusType == ThesaurusTypeEnum.Personal)
                    {
                        thesaurus.OrganizationCode = "";
                    }
                    var result = await _thesaurusDBRepository.InsertOneAsync(thesaurus, request.PlatformDatas, request.ExtensionData);
                    return !string.IsNullOrWhiteSpace(result);
                }
                else if (list.Count == 0 || !ValidateIsAddThesaurus(list, request.Data.PersonalCode, request.Data.OrganizationCode))
                {
                    if (!string.IsNullOrEmpty(thesaurus.PersonalCode) && thesaurus.ThesaurusType == ThesaurusTypeEnum.Personal)
                    {
                        thesaurus.OrganizationCode = "";
                    }
                    var result = await _thesaurusDBRepository.InsertOneAsync(thesaurus, request.PlatformDatas, request.ExtensionData);
                    return !string.IsNullOrWhiteSpace(result);
                }
                ThrowCustomerException(CustomerRpcCode.ThesaurusNameDuplication, "ThesaurusName duplication");
                return false;
            }
        }

        /// <summary>
        /// 验证词条重复
        /// </summary>
        /// <param name="request"></param>
        private bool ValidateIsAddThesaurus(List<ThesaurusDO> list, string personalCode, string organizationCode)
        {
            bool res = false;
            if (list?.Count > 0)
            {
                var isExist = list.Exists(c => c.PersonalCode == personalCode && c.ThesaurusType == ThesaurusTypeEnum.Personal && c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.IsDefault == true);
                if (isExist)
                {
                    res = true;
                    return res;
                }
                isExist = list.Exists(c => c.OrganizationCode == organizationCode && c.ThesaurusType == ThesaurusTypeEnum.Organization && c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.IsDefault == true);
                if (isExist)
                {
                    res = true;
                    return res;
                }
                isExist = list.Exists(c => c.ThesaurusType == ThesaurusTypeEnum.Default && c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.IsDefault == true);
                if (isExist)
                {
                    res = true;
                    return res;
                }
            }
            return res;
        }

        /// <summary>
        /// 更新词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusAsync(UpdateThesaurusDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateThesaurusAsync execute", $"{this.GetType().Name}"))
            {
                var thesaurus = await _thesaurusDBRepository.FindOneAsync("Code", request.Data.ThesaurusCode);
                if (thesaurus == null)
                {
                    return false;
                }
                request.Data.ThesaurusName = request.Data.ThesaurusName.Trim();
                if (string.IsNullOrWhiteSpace(request.Data.ThesaurusName))
                {
                    ThrowCustomerException(CustomerRpcCode.ThesaurusNameEmpty, "ThesaurusName is empty");
                }
                if (thesaurus.ThesaurusName != request.Data.ThesaurusName)
                {
                    var first = await _thesaurusDBRepository.FindOneAsync("ThesaurusName", request.Data.ThesaurusName);
                    if (first == null)
                    {
                        thesaurus.ThesaurusName = request.Data.ThesaurusName;
                    }
                    else
                    {
                        ThrowCustomerException(CustomerRpcCode.ThesaurusNameDuplication, "ThesaurusName duplication");
                    }
                }
                return await _thesaurusDBRepository.UpdateOneAsync("Code", thesaurus.ThesaurusCode, thesaurus, request.ExtensionData, true);
            }
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusItemAsync(AddThesaurusItemDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} AddThesaurusItemAsync execute", $"{this.GetType().Name}"))
            {
                if (request.ThesaurusItem == null)
                {
                    return false;
                }
                if (request.ThesaurusItem.ThesaurusItemType == ThesaurusItemTypeEnum.Classification)
                {
                    if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemName))
                    {
                        throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemName error", "ThesaurusItemName error");
                    }
                }
                if (request.ThesaurusItem.ThesaurusItemType == ThesaurusItemTypeEnum.Contents)
                {
                    if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemConclusion) && string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemDescription))
                    {
                        throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemConclusion ThesaurusItemDescription error", "ThesaurusItemConclusion ThesaurusItemDescription error");
                    }
                }
                if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemCode))
                {
                    request.ThesaurusItem.ThesaurusItemCode = "ThesaurusItem" + Guid.NewGuid().ToString("N");
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
                var filter = mongoCustomerFilter.AndCombine();
                MongoCustomerUpdate<MongoTable> update = new();
                var thesaurus = await _thesaurusDBRepository.FindOneAsync(filter);
                if (thesaurus?.ThesaurusItems != null && thesaurus.ThesaurusItems.Count(w => w.ThesaurusItemType == request.ThesaurusItem.ThesaurusItemType) > 0)
                {
                    if (!thesaurus.ThesaurusItems.Any(w => w.ThesaurusItemType == request.ThesaurusItem.ThesaurusItemType && w.ParentItemCode == request.ThesaurusItem.ParentItemCode))
                    {
                        request.ThesaurusItem.Sort = 1;
                    }
                    else
                    {
                        var maxSort = thesaurus.ThesaurusItems.Where(w => w.ThesaurusItemType == request.ThesaurusItem.ThesaurusItemType && w.ParentItemCode == request.ThesaurusItem.ParentItemCode).Max(f => f.Sort);
                        if (maxSort > 0)
                        {
                            request.ThesaurusItem.Sort = maxSort + 1;
                        }
                        else
                        {
                            request.ThesaurusItem.Sort = 1;
                        }
                    }
                }
                update.AddSubItem("BaseData.ThesaurusItems", request.ThesaurusItem);
                return await _thesaurusDBRepository.UpdateOneAsync(filter, update.AllUpdateDefinition) > 0;
            }
        }

        /// <summary>
        /// 更新词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusItemAsync(UpdateThesaurusItemDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} UpdateThesaurusItemAsync execute", $"{this.GetType().Name}"))
            {
                if (request.ThesaurusItem == null)
                {
                    return false;
                }
                if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemCode))
                {
                    throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemCode can not be empty", "ThesaurusItemCode can not be empty");
                }
                if (request.ThesaurusItem.ThesaurusItemType == ThesaurusItemTypeEnum.Classification)
                {
                    if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemName))
                    {
                        throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemName error", "ThesaurusItemName error");
                    }
                }
                if (request.ThesaurusItem.ThesaurusItemType == ThesaurusItemTypeEnum.Contents)
                {
                    if (string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemConclusion) && string.IsNullOrWhiteSpace(request.ThesaurusItem.ThesaurusItemDescription))
                    {
                        throw new RpcException((int)ErorrCodeEnum.ParameterError, "ThesaurusItemConclusion ThesaurusItemDescription error", "ThesaurusItemConclusion ThesaurusItemDescription error");
                    }
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
                var filter = mongoCustomerFilter.AndCombine();
                MongoCustomerUpdate<MongoTable> remove = new();
                MongoCustomerFilter<ThesaurusItemDO> thesaurusItemFilter = new();
                thesaurusItemFilter.AppendEq(("ThesaurusItemCode", request.ThesaurusItem.ThesaurusItemCode));
                var thesaurus = await _thesaurusDBRepository.FindOneAsync(filter);
                if (thesaurus.ThesaurusItems != null && thesaurus.ThesaurusItems.Count > 0)
                {
                    var findThesaurusItems = thesaurus.ThesaurusItems?.Where(w => w.ThesaurusItemType == request.ThesaurusItem.ThesaurusItemType && w.ParentItemCode == request.ThesaurusItem.ParentItemCode);
                    if (findThesaurusItems?.Any() == true)
                    {
                        var maxSort = findThesaurusItems.Max(m => m.Sort);
                        if (maxSort > 0)
                        {
                            request.ThesaurusItem.Sort = maxSort + 1;
                        }
                        else
                        {
                            request.ThesaurusItem.Sort = 1;
                        }
                    }
                    else
                    {
                        request.ThesaurusItem.Sort = 1;
                    }
                }
                remove.RemoveSubItemFilter("BaseData.ThesaurusItems", thesaurusItemFilter.AndCombine());
                if (await _thesaurusDBRepository.UpdateOneAsync(filter, remove.AllUpdateDefinition) > 0)
                {
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.AddSubItem("BaseData.ThesaurusItems", request.ThesaurusItem);
                    return await _thesaurusDBRepository.UpdateOneAsync(filter, update.AllUpdateDefinition) > 0;
                }
                return false;
            }
        }

        /// <summary>
        /// 删除词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusAsync(RemoveTemplateInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveThesaurusAsync execute", $"{this.GetType().Name}"))
            {
                var res = await _thesaurusDBRepository.DeleteOneAsync("Code", request.Code);
                //根据词条code和用户code查询相关数据，有关系的则清空
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                if (!string.IsNullOrEmpty(request.UserCode))
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", request.UserCode));
                }
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultThesaurusCode", request.Code));
                var filter = mongoCustomerFilter.AndCombine();
                var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
                if (findTable != null)
                {
                    MongoCustomerUpdate<MongoTable> update = new();
                    update.SetField("BaseData.DefaultThesaurusCode", "");
                    var queryItems = findTable.QueryData.QueryItems;
                    queryItems.RemoveWhere(r => r.Key == "DefaultThesaurusCode");
                    queryItems.Add(new MongoDataItem { Key = "DefaultThesaurusCode", Value = "" });
                    update.SetField("QueryData.QueryItems", queryItems);
                    res = await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                }
                if (!string.IsNullOrEmpty(request.OrganizationCode))
                {
                    MongoCustomerFilter<MongoTable> customerFilter = new();
                    customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.OrganizationCode));
                    customerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultThesaurusCode", request.Code));
                    var orgFilter = customerFilter.AndCombine();
                    var queryTable = await _organizationDBRepository.FindTableOneAsync(orgFilter);
                    if (queryTable != null)
                    {
                        MongoCustomerUpdate<MongoTable> update = new();
                        update.SetField("BaseData.DefaultThesaurusCode", "");
                        var queryItems = queryTable.QueryData.QueryItems;
                        queryItems.RemoveWhere(r => r.Key == "DefaultThesaurusCode");
                        queryItems.Add(new MongoDataItem { Key = "DefaultThesaurusCode", Value = "" });
                        update.SetField("QueryData.QueryItems", queryItems);
                        res = await _organizationDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 清理默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ClearDefaultTemplateInfoAsync(RemoveTemplateInfoDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} ClearDefaultTemplateInfoAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                if (request.Type == TemplateQueryTypeEnum.Thesaurus)
                {
                    //根据词条code和用户code查询相关数据，有关系的则清空
                    MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", request.UserCode));
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultThesaurusCode", request.Code));
                    var filter = mongoCustomerFilter.AndCombine();
                    var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
                    if (findTable != null)
                    {
                        MongoCustomerUpdate<MongoTable> update = new();
                        update.SetField("BaseData.DefaultThesaurusCode", "");
                        var queryItems = findTable.QueryData.QueryItems;
                        queryItems.RemoveWhere(r => r.Key == "DefaultThesaurusCode");
                        queryItems.Add(new MongoDataItem { Key = "DefaultThesaurusCode", Value = "" });
                        update.SetField("QueryData.QueryItems", queryItems);
                        res = await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                    }
                }
                else if (request.Type == TemplateQueryTypeEnum.ReportTemplate)
                {
                    //根据词条code和用户code查询相关数据，有关系的则清空
                    MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("UserCode", request.UserCode));
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("DefaultReportTemplateCode", request.Code));
                    var filter = mongoCustomerFilter.AndCombine();
                    var findTable = await _userExtendsDBRepository.FindTableOneAsync(filter);
                    if (findTable != null)
                    {
                        MongoCustomerUpdate<MongoTable> update = new();
                        update.SetField("BaseData.DefaultReportTemplateCode", "");
                        var queryItems = findTable.QueryData.QueryItems;
                        queryItems.RemoveWhere(r => r.Key == "DefaultReportTemplateCode");
                        queryItems.Add(new MongoDataItem { Key = "DefaultReportTemplateCode", Value = "" });
                        update.SetField("QueryData.QueryItems", queryItems);
                        res = await _userExtendsDBRepository.UpdateAllAsync(filter, update.AllUpdateDefinition) > 0;
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 删除词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusItemAsync(RemoveThesaurusItemDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} RemoveThesaurusItemAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.ThesaurusItemCode))
                {
                    return false;
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                MongoCustomerFilter<ThesaurusItemDO> queryItemsFilter = new();
                queryItemsFilter.AppendEq((q => q.ThesaurusItemCode, request.ThesaurusItemCode));
                queryItemsFilter.AppendEq((q => q.ParentItemCode, request.ThesaurusItemCode));
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
                var subFilter = queryItemsFilter.OrCombine();
                mongoCustomerFilter.SubFilter("BaseData.ThesaurusItems", subFilter);
                var filter = mongoCustomerFilter.AndCombine();
                MongoCustomerUpdate<MongoTable> update = new();
                update.RemoveSubItemFilter("BaseData.ThesaurusItems", subFilter);
                return await _thesaurusDBRepository.UpdateOneAsync(filter, update.AllUpdateDefinition) > 0;
            }
        }

        /// <summary>
        /// 获取词条库分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoDataItem> mongoCustomerSubFilter = new();
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                List<(string, string)> orEqFilters3 = new();
                List<(string, string)> orEqFilters4 = new();

                if (!string.IsNullOrWhiteSpace(request.ThesaurusName))
                {
                    regexFilters.Add(("ThesaurusName", request.ThesaurusName, true, false));
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    var organizationInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                    orEqFilters4.Add(("OrganizationCode", request.OrganizationCode));
                    orEqFilters4.Add(("OrganizationCode", organizationInfo.RootCode));
                }
                if (!string.IsNullOrWhiteSpace(request.PersonalCode))
                {
                    orEqFilters4.Add(("PersonalCode", request.PersonalCode));
                }
                orEqFilters4.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Default)));
                eqFilters.Add(("LanguageCode", request.LanguageCode));
                if (request.IsDefault == true || request.IsDefault == null)
                {
                    eqFilters.Add(("IsDefault", bool.TrueString));
                }
                if (request.ThesaurusStateType != null)
                {
                    eqFilters.Add(("ThesaurusStateType", Enum.GetName(request.ThesaurusStateType.Value)));
                }
                else
                {
                    eqFilters.Add(("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)));
                }
                orEqFilters3.Add(("ThesaurusUsageType", Enum.GetName(ThesaurusUsageTypeEnum.People)));
                orEqFilters3.Add(("ThesaurusUsageType", Enum.GetName(ThesaurusUsageTypeEnum.Animal)));
                if (request.ThesaurusType != null)
                {
                    eqFilters.Add(("ThesaurusType", Enum.GetName(request.ThesaurusType.Value)));
                }

                mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters3.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters4.ToArray());
                var filter = mongoCustomerFilter.AndCombine();
                //业务类型
                MongoCustomerFilter<MongoTable> thesaurusBusinessTypeFilter = GetThesaurusBusinessTypeFilter(request.ThesaurusBusinessType);
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ThesaurusType").Descending("BaseData.CreateTime").Ascending("BaseData.ThesaurusUsageType").Descending("BaseData.Code");
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter & thesaurusBusinessTypeFilter.AndCombine();
                dbPage.Sort = sort;
                var pageResult = await _thesaurusDBRepository.GetPages(dbPage);
                return PageResultConvert<ThesaurusDO, ThesaurusDTO>(pageResult);

            }
        }

        /// <summary>
        /// 获取机构词条库分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetOrgThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetOrgThesaurusPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoDataItem> mongoCustomerSubFilter = new();
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<(string, string, bool, bool)> regexFilters = new();
                List<(string, string)> eqFilters = new();
                List<(string, string)> orEqFilters = new();
                List<(string, string)> orEqFilters2 = new();

                if (!string.IsNullOrWhiteSpace(request.ThesaurusName))
                {
                    regexFilters.Add(("ThesaurusName", request.ThesaurusName, true, false));
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    var organizationInfo = await _organizationDBRepository.GetOrganizationByCodeAsync(request.OrganizationCode);
                    orEqFilters.Add(("OrganizationCode", request.OrganizationCode));
                    orEqFilters.Add(("OrganizationCode", organizationInfo.RootCode));
                }
                if (!string.IsNullOrWhiteSpace(request.PersonalCode))
                {
                    orEqFilters.Add(("PersonalCode", request.PersonalCode));
                }
                orEqFilters.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Default)));
                eqFilters.Add(("LanguageCode", request.LanguageCode));
                if (request.IsDefault == true || request.IsDefault == null)
                {
                    eqFilters.Add(("IsDefault", bool.TrueString));
                }
                if (request.ThesaurusStateType != null)
                {
                    eqFilters.Add(("ThesaurusStateType", Enum.GetName(request.ThesaurusStateType.Value)));
                }
                else
                {
                    eqFilters.Add(("ThesaurusStateType", Enum.GetName(ThesaurusStatusTypeEnum.Published)));
                }
                orEqFilters2.Add(("ThesaurusUsageType", Enum.GetName(ThesaurusUsageTypeEnum.People)));
                orEqFilters2.Add(("ThesaurusUsageType", Enum.GetName(ThesaurusUsageTypeEnum.Animal)));
                if (request.ThesaurusType != null)
                {
                    eqFilters.Add(("ThesaurusType", Enum.GetName(request.ThesaurusType.Value)));
                }
                // else
                // {
                //     orEqFilters2.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Organization)));
                //     orEqFilters2.Add(("ThesaurusType", Enum.GetName(ThesaurusTypeEnum.Default)));
                // }
                mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, regexFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters.ToArray());
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, orEqFilters2.ToArray());
                var filter = mongoCustomerFilter.AndCombine();
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.ThesaurusType").Descending("BaseData.CreateTime").Ascending("BaseData.ThesaurusUsageType").Descending("BaseData.Code");
                //业务类型
                MongoCustomerFilter<MongoTable> thesaurusBusinessTypeFilter = GetThesaurusBusinessTypeFilter(request.ThesaurusBusinessType);
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.PageIndex;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter & thesaurusBusinessTypeFilter.AndCombine();
                dbPage.Sort = sort;
                var pageResult = await _thesaurusDBRepository.GetPages(dbPage);
                return PageResultConvert<ThesaurusDO, ThesaurusDTO>(pageResult);

            }
        }

        /// <summary>
        /// 词条库业务类型条件
        /// </summary>
        /// <param name="thesaurusBusinessType"></param>
        /// <returns></returns>
        private MongoCustomerFilter<MongoTable> GetThesaurusBusinessTypeFilter(ThesaurusBusinessTypeEnum thesaurusBusinessType)
        {
            //业务类型
            MongoCustomerFilter<MongoTable> thesaurusBusinessTypeFilter = new();
            thesaurusBusinessTypeFilter.AppendEq(("BaseData.IsDelete", false));
            //当查询超声类型时
            if (thesaurusBusinessType == ThesaurusBusinessTypeEnum.Ultrasonic)
            {
                var notUltrasonicTypes = new List<int>();
                var thesaurusBusinessTypeEnums = Enum.GetValues(typeof(ThesaurusBusinessTypeEnum));
                foreach (var item in thesaurusBusinessTypeEnums)
                {
                    if (item.GetHashCode() == ThesaurusBusinessTypeEnum.Ultrasonic.GetHashCode())
                    {
                        continue;
                    }
                    notUltrasonicTypes.Add(item.GetHashCode());
                }
                thesaurusBusinessTypeFilter.AppendNotIn("BaseData.ThesaurusBusinessType", notUltrasonicTypes);
            }
            else
            {
                thesaurusBusinessTypeFilter.AppendEq(("BaseData.ThesaurusBusinessType", thesaurusBusinessType.GetHashCode()));
            }
            return thesaurusBusinessTypeFilter;
        }

        /// <summary>
        /// 获取词条项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusItemsAsync(GetThesaurusItemsRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusItemsAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
                {
                    return new List<ThesaurusItemDTO>();
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
                var filter = mongoCustomerFilter.AndCombine();
                var doData = await _thesaurusDBRepository.FindOneAsync(filter);
                if (doData == null || doData.ThesaurusItems == null || doData.ThesaurusItems.Count <= 0)
                {
                    return new List<ThesaurusItemDTO>();
                }
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    doData.ThesaurusItems = doData.ThesaurusItems.Where(f => !string.IsNullOrEmpty(f.ThesaurusItemName) && f.ThesaurusItemName.ToLower().IndexOf(request.Keyword.ToLower()) > -1)?.ToList() ?? new List<ThesaurusItemDO>();
                }
                if (doData.ThesaurusItems?.Count > 0)
                {
                    foreach (var item in doData.ThesaurusItems)
                    {
                        item.HasChildren = doData.ThesaurusItems.Exists(f => f.ParentItemCode == item.ThesaurusItemCode && f.ThesaurusItemType == ThesaurusItemTypeEnum.Classification);
                    }
                    return doData.ThesaurusItems.MappingTo<List<ThesaurusItemDTO>>();
                }
                else
                {
                    return new List<ThesaurusItemDTO>();
                }
            }
        }

        /// <summary>
        /// 获取词条项详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusItemDTO> GetThesaurusItemByCodeAsync(GetThesaurusItemByCodeRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusItemByCodeAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
                {
                    return null;
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
                var filter = mongoCustomerFilter.AndCombine();
                var doData = await _thesaurusDBRepository.FindOneAsync(filter);
                var result = doData?.ThesaurusItems?.FirstOrDefault(w => w.ThesaurusItemCode == request.ThesaurusItemCode);
                return result?.MappingTo<ThesaurusItemDTO>();
            }
        }

        /// <summary>
        /// 分页查询系统词库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetDefaultThesaurusPageAsync(GetDefaultThesaurusPageDBRequest request)
        {
            using (new Performance(_performanceThreshold, "{0} GetDefaultThesaurusPageAsync execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                List<FilterDefinition<MongoDataItem>> queryItemsFilterList = new();
                List<FilterDefinition<MongoDataItem>> orFilterList = new();
                MongoCustomerFilter<MongoDataItem> queryItemsFilter = new();
                List<(string, string)> eqFilters = new();
                if (!string.IsNullOrWhiteSpace(request.Keyword))
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ThesaurusName"));
                    queryItemsFilter.AppendRegex(item => item.Value, request.Keyword, true, false);
                    queryItemsFilterList.Add(queryItemsFilter.AndCombine());

                    var orgs = await GetOrganizationDOListByKeyword(request.Keyword);
                    if (orgs?.Count > 0)
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "OrganizationCode"));
                        queryItemsFilter.AppendIn(item => item.Value, orgs.Select(x => x.Code).ToList());
                        queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                    }

                    var users = await GetUserDOListByKeyword(request.Keyword);
                    if (users?.Count > 0)
                    {
                        queryItemsFilter = new();
                        queryItemsFilter.AppendEq((item => item.Key, "PersonalCode"));
                        queryItemsFilter.AppendIn(item => item.Value, users.Select(x => x.Code).ToList());
                        queryItemsFilterList.Add(queryItemsFilter.AndCombine());
                    }
                }
                if (!string.IsNullOrWhiteSpace(request.OrganizationCode))
                {
                    eqFilters.Add(("OrganizationCode", request.OrganizationCode));
                }
                if (request.IsDefault)
                {
                    eqFilters.Add(("IsDefault", request.IsDefault.ToString()));
                }
                if (request.ThesaurusType != null)
                {
                    eqFilters.Add(("ThesaurusType", Enum.GetName(request.ThesaurusType.Value)));
                }
                else
                {
                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ThesaurusType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ThesaurusTypeEnum.Default)));
                    orFilterList.Add(queryItemsFilter.AndCombine());

                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ThesaurusType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ThesaurusTypeEnum.Organization)));
                    orFilterList.Add(queryItemsFilter.AndCombine());

                    queryItemsFilter = new();
                    queryItemsFilter.AppendEq((item => item.Key, "ThesaurusType"));
                    queryItemsFilter.AppendEq((item => item.Value, Enum.GetName(ThesaurusTypeEnum.Personal)));
                    orFilterList.Add(queryItemsFilter.AndCombine());
                }
                if (request.ThesaurusStateType != null)
                {
                    eqFilters.Add(("ThesaurusStateType", Enum.GetName(request.ThesaurusStateType.Value)));
                }
                if (queryItemsFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(queryItemsFilterList.ToArray());
                    mongoCustomerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                if (orFilterList.Count > 0)
                {
                    var queryItemAllFilter = queryItemsFilter.OrCombine(orFilterList.ToArray());
                    mongoCustomerFilter.SubFilter(item => item.QueryData.QueryItems, queryItemAllFilter);
                }
                if (eqFilters.Count > 0)
                {
                    mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, eqFilters.ToArray());
                }
                SortDefinitionBuilder<MongoTable> builderSort = Builders<MongoTable>.Sort;
                SortDefinition<MongoTable> sort = builderSort.Descending("BaseData.CreateTime").Ascending("BaseData.ThesaurusUsageType").Descending("BaseData.Code");
                var filter = mongoCustomerFilter.AndCombine();
                var dbPage = new DBPageRequest();
                dbPage.CurrentPage = request.CurrentPage;
                dbPage.PageSize = request.PageSize;
                dbPage.Filter = filter;
                dbPage.Sort = sort;
                var pageResult = await _thesaurusDBRepository.GetPages(dbPage);
                return PageResultConvert<ThesaurusDO, ThesaurusDTO>(pageResult);
            }
        }

        /// <summary>
        /// 查询词库
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<ThesaurusDTO> GetThesaurusByCodeAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusByCodeAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _thesaurusDBRepository.FindOneAsync("Code", code);
                return entity.MappingTo<ThesaurusDTO>();
            }
        }

        /// <summary>
        /// 更新词库
        /// </summary>
        /// <param name="code"></param>
        /// <param name="data"></param>
        /// <param name="extensionData"></param>
        /// <returns></returns>
        public async Task<bool> ModifyThesaurusAsync(string code, ThesaurusDTO data, string extensionData)
        {
            try
            {
                using (new Performance(_performanceThreshold, "{0} ModifyThesaurusAsync execute", $"{this.GetType().Name}"))
                {
                    var entity = await _thesaurusDBRepository.FindOneAsync("Code", code);
                    if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                    {
                        entity.ThesaurusName = data.ThesaurusName;
                        entity.ThesaurusType = data.ThesaurusType;
                        entity.ThesaurusUsageType = data.ThesaurusUsageType;
                        entity.IsDefault = data.IsDefault;
                        entity.LanguageCode = data.LanguageCode;
                        entity.OrganizationCode = data.OrganizationCode;
                        entity.PersonalCode = data.PersonalCode;
                        entity.ThesaurusBusinessType = data.ThesaurusBusinessType;
                        return await _thesaurusDBRepository.UpdateOneAsync("Code", code, entity, string.Empty, true);
                    }
                    return false;
                }
            }
            catch (System.Exception ex)
            {

            }
            return false;

        }

        /// <summary>
        /// 发布词库
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public async Task<bool> PublishThesaurusAsync(string code)
        {
            using (new Performance(_performanceThreshold, "{0} PublishThesaurusAsync execute", $"{this.GetType().Name}"))
            {
                var entity = await _thesaurusDBRepository.FindOneAsync("Code", code);
                if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                {
                    if (entity.ThesaurusStateType == ThesaurusStatusTypeEnum.Unpublished)
                    {
                        entity.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
                        return await _thesaurusDBRepository.UpdateOneAsync("Code", code, entity, string.Empty, true);
                    }
                    return true;
                }
                return false;
            }
        }

        /// <summary>
        /// 判断词条库是否存在
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ExistThesaurusInfoDBAsync(ImportThesaurusInfoRequest request)
        {
            var res = false;
            MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
            filter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ThesaurusName", request.ThesaurusName), ("IsDelete", "False"), ("LanguageCode", request.LanguageCode));
            var list = await _thesaurusDBRepository.FindAllAsync(filter.AndCombine());
            if (list == null || list.Count == 0)
            {
                res = false;
                return res;
            }
            res = ValidateIsAddThesaurus(list, request.PersonalCode, request.OrganizationCode);
            return res;
            // if (doDatas?.Count > 0)
            // {
            //     if (request.ThesaurusType == ThesaurusTypeEnum.Default)
            //     {
            //         res = true;
            //     }
            //     else
            //     {
            //         if (request.ThesaurusType == ThesaurusTypeEnum.Organization)
            //         {
            //             res = doDatas.Exists(c => c.OrganizationCode == request.OrganizationCode && c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.LanguageCode == request.LanguageCode);
            //         }
            //         else if (request.ThesaurusType == ThesaurusTypeEnum.Personal)
            //         {
            //             res = doDatas.Exists(c => c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.ThesaurusUsageType == request.ThesaurusUsageType && c.LanguageCode == request.LanguageCode 
            //             && (c.OrganizationCode == request.OrganizationCode || c.PersonalCode == request.PersonalCode));
            //         }
            //     }
            // }
            // return res;
        }

        /// <summary>
        /// 获取词条列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusItemsDetailListAsync(string code, string parentItemCode, string keyword = null, ThesaurusItemTypeEnum thesaurusItemType = ThesaurusItemTypeEnum.Contents)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusItemsDetailListAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(code))
                {
                    return new List<ThesaurusItemDTO>();
                }
                if (parentItemCode == null)
                {
                    parentItemCode = string.Empty;
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", code));
                if (!string.IsNullOrWhiteSpace(keyword))
                {
                    mongoCustomerFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, ("ThesaurusItemName", keyword, true, false));
                }
                var filter = mongoCustomerFilter.AndCombine();
                var doData = await _thesaurusDBRepository.FindOneAsync(filter);
                foreach (var item in doData.ThesaurusItems)
                {
                    item.HasChildren = doData.ThesaurusItems.Exists(f => f.ParentItemCode == item.ThesaurusItemCode && f.ThesaurusItemType == ThesaurusItemTypeEnum.Classification);
                }
                var resList = doData.ThesaurusItems.FindAll(c => ((string.IsNullOrWhiteSpace(parentItemCode) && string.IsNullOrWhiteSpace(c.ParentItemCode)) || c.ParentItemCode == parentItemCode) && c.ThesaurusItemType == thesaurusItemType) ?? new List<ThesaurusItemDO>();
                return resList.MappingTo<List<ThesaurusItemDTO>>();
            }
        }

        /// <summary>
        /// 拷贝词条库
        /// </summary>
        /// <param name="thesaurusCode">词条库编码</param>
        /// <param name="userCode">用户编码</param>
        /// <param name="organizationCode">组织编码</param>
        /// <returns></returns>
        public async Task<bool> CopyThesaurusAsync(string thesaurusCode, string userCode, string organizationCode)
        {
            using (new Performance(_performanceThreshold, "{0} CopyThesaurusAsync execute", $"{this.GetType().Name}"))
            {
                if (string.IsNullOrWhiteSpace(thesaurusCode))
                {
                    return false;
                }
                if (string.IsNullOrWhiteSpace(userCode) && string.IsNullOrWhiteSpace(organizationCode))
                {
                    return false;
                }
                MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
                mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", thesaurusCode));
                var filter = mongoCustomerFilter.AndCombine();
                var doData = await _thesaurusDBRepository.FindTableOneAsync(filter);
                if (doData != null && doData.BaseData != null)
                {
                    var newData = (ThesaurusDO)doData.BaseData;
                    newData.Code = string.Empty;
                    newData.IsDefault = true;
                    newData.OrganizationCode = organizationCode;
                    newData.PersonalCode = userCode;
                    newData.ReferenceThesaurusCode = thesaurusCode;
                    newData.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
                    if (!string.IsNullOrWhiteSpace(organizationCode))
                    {
                        newData.ThesaurusType = ThesaurusTypeEnum.Organization;
                    }
                    else if (!string.IsNullOrWhiteSpace(userCode))
                    {
                        newData.ThesaurusType = ThesaurusTypeEnum.Personal;
                    }
                    if (newData.ThesaurusType == ThesaurusTypeEnum.Personal && !string.IsNullOrEmpty(newData.PersonalCode))
                    {
                        newData.OrganizationCode = "";
                    }
                    MongoCustomerFilter<MongoTable> copyListFilter = new();
                    copyListFilter.AppendQueryItemRegexFilter(MongoCombineTypeEnum.And, ("ThesaurusName", newData.ThesaurusName + "-", true, false));
                    var copyfilter = copyListFilter.AndCombine();
                    var copyList = await _thesaurusDBRepository.FindAllAsync(copyfilter);
                    var maxCopyNum = 1;
                    if (copyList != null && copyList.Count > 0)
                    {
                        maxCopyNum = copyList.Max(f =>
                        {
                            var splitArray = f.ThesaurusName.Split("-");
                            if (splitArray.Length > 1)
                            {
                                int.TryParse(splitArray.LastOrDefault(), out int copyCount);
                                return copyCount + 1;
                            }
                            return 1;
                        });
                    }
                    newData.ThesaurusName = $"{newData.ThesaurusName}-{maxCopyNum}";
                    var platDic = new Dictionary<string, string>();
                    if (doData.PlatformData != null && doData.PlatformData.PlatformItems != null)
                    {
                        foreach (var plat in doData.PlatformData.PlatformItems)
                        {
                            platDic.Add(plat.Key, plat.Value);
                        }
                    }
                    return !string.IsNullOrWhiteSpace(await _thesaurusDBRepository.InsertOneAsync(newData, platDic, doData.ExtendsData));
                }
                return false;
            }
        }

        /// <summary>
        /// 获取报告模板引用用户编码列表
        /// </summary>
        /// <param name="reportTemplateCode">报告模版编码</param>
        /// <returns></returns>
        public async Task<List<string>> GetReportTemplateUserCodesAsync(string reportTemplateCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportTemplateUserCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("ReferenceReportTemplateCode", reportTemplateCode);
                var reportTemplateList = await _reportTemplateDBRepository.FindListAsync(dic);
                if (reportTemplateList != null && reportTemplateList.Count > 0)
                {
                    return reportTemplateList.Where(f => !string.IsNullOrWhiteSpace(f.PersonalCode)).Select(s => s.PersonalCode).ToList();
                }
                return new List<string>();
            }
        }

        /// <summary>
        /// 获取词条库引用用户编码列表
        /// </summary>
        /// <param name="thesaurusCode">词条库编码</param>
        /// <returns></returns>
        public async Task<List<string>> GetThesaurusUserCodesAsync(string thesaurusCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetThesaurusUserCodesAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                dic.Add("ReferenceThesaurusCode", thesaurusCode);
                var thesaurusTemplateList = await _thesaurusDBRepository.FindListAsync(dic);
                if (thesaurusTemplateList != null && thesaurusTemplateList.Count > 0)
                {
                    return thesaurusTemplateList.Where(f => !string.IsNullOrWhiteSpace(f.PersonalCode)).Select(s => s.PersonalCode).ToList();
                }
                return new List<string>();
            }
        }

        /// <summary>
        /// 根据userCode查询报告列表
        /// </summary>
        /// <returns></returns>
        public async Task<List<ReportDTO>> GetReportsByUserCodeAsync(string userCode)
        {
            using (new Performance(_performanceThreshold, "{0} GetReportsByUserCodeAsync execute", $"{this.GetType().Name}"))
            {
                var dic = new Dictionary<string, string>();
                //dic.Add("IsDelete", "False");
                dic.Add("ReportUserCode", userCode);
                var entities = await _reportDBRepository.FindListAsync(dic);
                entities.ForEach(report =>
                {
                    report.ReportDatasJson = string.Empty;
                    report.ReportTemplateJson = string.Empty;
                });
                var reportList = entities.MappingTo<List<ReportDTO>>();
                return reportList;
            }
        }

        /// <summary>
        /// 批量新增报告模板数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertReportTemplateDataDBAsync(BatchInsertReportTemplateRequest request)
        {
            long res = 0;
            if (request?.ReportTemplateMigratorys?.Count > 0)
            {
                var defaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";
                var animalDefaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";

                //找到相同的
                var ids = request.ReportTemplateMigratorys.Select(c => c.ReportTemplateCode).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var reportTemplateList = await _reportTemplateDBRepository.FindInAsync(dic);
                //求同存异
                if (reportTemplateList?.Count > 0)
                {
                    var existCodes = reportTemplateList.Select(c => c.ReportTemplateCode).Distinct().ToList();
                    var updateData = request.ReportTemplateMigratorys.Where(c => existCodes.Contains(c.ReportTemplateCode));
                    var insertData = request.ReportTemplateMigratorys.Where(c => !existCodes.Contains(c.ReportTemplateCode));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<ReportTemplateDO>();
                                data.Code = f.ReportTemplateCode.Trim();
                                //data.isdef
                                data.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                                if (data.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                                {
                                    data.ReportDatasJson = animalDefaultReportDatasJson;
                                }
                                else
                                {
                                    data.ReportDatasJson = defaultReportDatasJson;
                                }
                                return (data, new Dictionary<string, string>(), string.Empty);
                            }
                        ).ToList();
                        res = await _reportTemplateDBRepository.InsertAllAsync(req);
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(f =>
                        {
                            var updateEntity = f.MappingTo<ReportTemplateDO>();
                            updateEntity.Code = f.ReportTemplateCode.Trim();
                            updateEntity.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                            if (updateEntity.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                            {
                                updateEntity.ReportDatasJson = animalDefaultReportDatasJson;
                            }
                            else
                            {
                                updateEntity.ReportDatasJson = defaultReportDatasJson;
                            }
                            return ("Code", f.ReportTemplateCode, updateEntity, string.Empty);
                        }).ToList();
                        res = await _reportTemplateDBRepository.UpdateAllAsync(req);
                    }
                }
                else
                {
                    //新增
                    var req = request.ReportTemplateMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<ReportTemplateDO>();
                            data.Code = f.ReportTemplateCode.Trim();
                            data.IsDelete = false;
                            data.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                            if (data.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                            {
                                data.ReportDatasJson = animalDefaultReportDatasJson;
                            }
                            else
                            {
                                data.ReportDatasJson = defaultReportDatasJson;
                            }
                            return (data, new Dictionary<string, string>(), string.Empty);
                        }
                    ).ToList();
                    res = await _reportTemplateDBRepository.InsertAllAsync(req);
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// json模板数据转换
        /// </summary>
        private string TransferReportTemplateInfos(string fileUrl)
        {
            FileTransferService _fileTransfer = new FileTransferService();
            var fileList = new List<string>() { fileUrl };
            var fileToken = "";
            var subInfoList = fileUrl.Split('/');
            if (subInfoList != null && subInfoList.Length > 1)
            {
                var fileName = subInfoList[(subInfoList.Length - 1)];
                if (!string.IsNullOrEmpty(fileName))
                {
                    fileToken = Path.GetFileNameWithoutExtension(fileName);
                }
            }
            var sourceFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Temp", fileToken);
            var downloadRes = _fileTransfer.DownloadFileAsync(fileList, sourceFilePath, (fileToken + ".json")).GetAwaiter().GetResult();
            if (string.IsNullOrEmpty(downloadRes))
            {
                Logger.WriteLineInfo($"Download File {fileUrl} Fail");
                return "";
            }
            var path = sourceFilePath + "\\" + fileToken + ".json";
            var json = File.ReadAllText(path);
            File.Delete(path);
            Directory.Delete(sourceFilePath);
            return json;
        }

        public async Task<bool> SyncBatchReportInfoDBAsync(SyncBatchReportInfoRequest syncBatchReportInfoRequest)
        {
            try
            {
                SyncDBEnum syncType = syncBatchReportInfoRequest.SyncType;
                var insertList = new List<ReportDO>();
                var updateList = new List<ReportDO>();
                var records = new List<RecordInfoDO>();
                var consultationRecords = new List<ConsultationRecordEntity>();
                var patients = new List<PatientInfoDO>();
                var relationCodes = syncBatchReportInfoRequest?.SyncReportInfos?.Select(x => x.RecordCode).Distinct().ToList();
                if (relationCodes != null && relationCodes.Count > 0)
                {
                    Dictionary<string, List<string>> filterRecord = new Dictionary<string, List<string>>();
                    filterRecord.Add("Code", relationCodes);
                    records = await _recordInfoDBRepository.FindInAsync(filterRecord);
                    var filterConsultationRecords = Builders<ConsultationRecordEntity>.Filter.Where(c => relationCodes.Contains(c.Code));
                    consultationRecords = await _consultationRecordDBRepository.FindAllAsync(filterConsultationRecords);
                    var patientCodes = records.Select(x => x.PatientCode).ToList();
                    if (consultationRecords != null && consultationRecords.Count > 0)
                    {
                        patientCodes.AddRange(consultationRecords.Select(x => x.ConsultationCode));
                    }
                    patientCodes = patientCodes.Distinct().ToList();
                    var filterPatients = new Dictionary<string, List<string>>();
                    filterPatients.Add("Code", relationCodes);
                    patients = await _patientInfoDBRepository.FindInAsync(filterPatients);
                }
                var reportCodes = syncBatchReportInfoRequest.SyncReportInfos.Select(x => x.ReportCode);
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", reportCodes.ToList());
                var reports = await _reportDBRepository.FindInAsync(dic);
                foreach (var request in syncBatchReportInfoRequest.SyncReportInfos)
                {
                    try
                    {
                        var entity = reports.FirstOrDefault(x => x.Code == request.ReportCode);
                        var isUpdate = false;
                        if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
                        {
                            isUpdate = true;
                        }
                        else
                        {
                            entity = new ReportDO();
                            entity.Code = request.ReportCode;
                        }
                        entity.RecordCode = request.RecordCode;
                        entity.CreateTime = request.CreateTime;
                        entity.UpdateTime = request.UpdateTime;
                        entity.ReportPreviewList = request.ReportPreviewList.MappingTo<List<ReportPreviewDO>>();
                        entity.ReportUserCode = request.ReportUserCode;
                        //根据关联code查询检查或会诊记录 取里面的病人code、设备code、PatientName
                        var record = records.FirstOrDefault(x => x.RecordCode == request.RecordCode);
                        var consultationRecord = consultationRecords.FirstOrDefault(x => x.ConsultationCode == request.RecordCode);
                        if (record != null)
                        {
                            entity.DeviceCode = record.DeviceCode;
                            entity.PatientCode = record.PatientCode;
                            var patient = patients.FirstOrDefault(x => x.PatientCode == record.PatientCode);
                            entity.PatientName = patient?.Name;
                        }
                        else if (consultationRecord != null)
                        {
                            entity.DeviceCode = consultationRecord.DeviceCode;
                            entity.PatientCode = consultationRecord.PatientCode;
                            var patient = patients.FirstOrDefault(x => x.PatientCode == consultationRecord.PatientCode);
                            entity.PatientName = patient?.Name;
                        }
                        else
                        {
                            continue;
                        }
                        entity.ReportTemplateJson = request.ReportTemplateJson;
                        entity.ReportDatasJson = request.ReportDatasJson;
                        entity.ReportLabels = request.ReportLabels ?? new List<string>();
                        entity.PlatformLabels = request.PlatformLabels ?? new List<string>();
                        entity.DiagnosisLabels = request.DiagnosisLabels ?? new List<string>();
                        entity.ReportOrgan = request.ReportOrgan;
                        if (isUpdate)
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                entity.QualifiedState = request.QualifiedState;
                                entity.QualityType = request.QualityType;
                                entity.QualityControlDatas = request.QualityControlDatas;
                                updateList.Add(entity);
                            }
                            else
                            {
                                await _reportDBRepository.UpdateOneForSyncDBAsync("Code", entity.Code, entity, string.Empty);
                                await AddPushReportAsync(entity);
                                if (record != null)
                                {
                                    var update = false;
                                    if (!entity.DiagnosisLabels.Contains("AI") && !entity.DiagnosisLabels.Contains("VAid"))
                                    {
                                        if (!record.IsUserReport)
                                        {
                                            record.IsUserReport = true;
                                            update = true;
                                        }
                                    }
                                    if (record.RecordStatus != RecordStatusEnum.Completed)
                                    {
                                        record.RecordStatus = RecordStatusEnum.Completed;
                                        update = true;
                                    }
                                    if (update)
                                    {
                                        await _recordInfoDBRepository.UpdateOneAsync("Code", record.Code, record, string.Empty);
                                    }
                                }
                                else if (consultationRecord != null)
                                {
                                    if (consultationRecord.ConsultationStatus != TransactionStatusEnum.End)
                                    {
                                        var filter = Builders<ConsultationRecordEntity>.Filter.Eq(f => f.Code, consultationRecord.Code);
                                        var update = Builders<ConsultationRecordEntity>.Update.Set(f => f.ConsultationStatus, TransactionStatusEnum.End);
                                        await _consultationRecordDBRepository.UpdateOneAsync(filter, update);
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (syncType == SyncDBEnum.Migrate)
                            {
                                entity.QualifiedState = request.QualifiedState;
                                entity.QualityType = request.QualityType;
                                entity.QualityControlDatas = request.QualityControlDatas;
                                insertList.Add(entity);
                            }
                            else
                            {
                                await _reportDBRepository.InsertOneForAsyncDBAsync(entity, new Dictionary<string, string>(), string.Empty);
                                await AddPushReportAsync(entity);
                                if (record != null)
                                {
                                    var update = false;
                                    if (!entity.DiagnosisLabels.Contains("AI") && !entity.DiagnosisLabels.Contains("VAid"))
                                    {
                                        if (!record.IsUserReport)
                                        {
                                            record.IsUserReport = true;
                                            update = true;
                                        }
                                    }
                                    if (record.RecordStatus != RecordStatusEnum.Completed)
                                    {
                                        record.RecordStatus = RecordStatusEnum.Completed;
                                        update = true;
                                    }
                                    if (update)
                                    {
                                        await _recordInfoDBRepository.UpdateOneAsync("Code", record.Code, record, string.Empty);
                                    }
                                }
                                else if (consultationRecord != null)
                                {
                                    if (consultationRecord.ConsultationStatus != TransactionStatusEnum.End)
                                    {
                                        var filter = Builders<ConsultationRecordEntity>.Filter.Eq(f => f.Code, consultationRecord.Code);
                                        var update = Builders<ConsultationRecordEntity>.Update.Set(f => f.ConsultationStatus, TransactionStatusEnum.End);
                                        await _consultationRecordDBRepository.UpdateOneAsync(filter, update);
                                    }
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn("foreach SyncBatchReportInfoDBAsync," + ex);
                    }
                }
                //迁移or同步
                if (syncType == SyncDBEnum.Migrate)
                {
                    if (updateList.Count > 0)
                    {
                        await _reportDBRepository.UpdateAllWithoutOplogLogAsync(updateList);
                    }
                    if (insertList.Count > 0)
                    {
                        await _reportDBRepository.InsertAllWithoutOplogLogAsync(insertList);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn("SyncBatchReportInfoDBAsync," + ex);
            }
            return true;
        }

        /// <summary>
        /// 清洗数据-清理新平台报告数据是否用户报告
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> SynIsUserReportInfoDBAsync(BatchModifyRecordReportDataRequest request)
        {
            bool res = false;
            var docList = new List<BsonDocument>();
            var matchFilter = new List<BsonDocument>();
            //过滤指定record
            if (request.RecordCodes?.Count > 0)
            {
                matchFilter.Add(new BsonDocument("BaseData.Code", new BsonDocument("$in", new BsonArray(request.RecordCodes))));
            }
            if (request.IsExistPropertyIsUserReport)
            {
                matchFilter.Add(new BsonDocument("BaseData.IsUserReport", new BsonDocument("$exists", false)));
            }
            if (request.IsUserReport > 0)
            {
                if (request.IsUserReport == 1)
                {
                    matchFilter.Add(new BsonDocument("BaseData.IsUserReport", true));
                }
                else
                {
                    matchFilter.Add(new BsonDocument("BaseData.IsUserReport", false));
                }
            }
            var matchFilterDoc = new BsonDocument("$and", new BsonArray(matchFilter));
            var matchDoc = new BsonDocument("$match", new BsonDocument(matchFilterDoc));
            docList.Add(matchDoc);
            var recordResult = await _recordInfoDBRepository.FindListByAggregateAsync(docList);

            if (recordResult?.Count > 0)
            {
                Logger.WriteLineInfo($"SynIsUserReportInfoDBAsync record Result Count: {recordResult.Count}");
                var waitReportRecordList = recordResult.Where(c => c.RecordStatus == RecordStatusEnum.Completed)?.Select(c => c.Code)?.Distinct()?.ToList() ?? new List<string>();
                var reportList = new List<ReportDO>();
                if (waitReportRecordList?.Count > 0)
                {
                    var pageIndex = 0;
                    var pageSize = 1000;
                    var reportRecordCount = waitReportRecordList.Count;
                    while (true)
                    {
                        var pendingRecords = waitReportRecordList.Skip(pageIndex * pageSize).Take(pageSize)?.ToList() ?? new List<string>();
                        var pendingCount = pendingRecords.Count;
                        if (pendingCount > 0)
                        {
                            var dic = new Dictionary<string, List<string>>();
                            dic.Add("RecordCode", pendingRecords);
                            var reports = await _reportDBRepository.FindInAsync(dic);
                            if (reports?.Count > 0)
                            {
                                //reportList.AddRange(reports);
                                foreach (var itemCode in pendingRecords)
                                {
                                    var item = recordResult.FirstOrDefault(c => c.Code == itemCode);
                                    if (!string.IsNullOrEmpty(item?.Code))
                                    {
                                        var entityList = reports.FindAll(d => d.RecordCode == item.Code);
                                        if (entityList?.Count > 0 && entityList.Exists(c => !c.DiagnosisLabels.Contains("AI") && !c.DiagnosisLabels.Contains("VAid")))
                                        {
                                            item.IsUserReport = true;
                                        }
                                        else
                                        {
                                            item.IsUserReport = false;
                                        }
                                        res = await _recordInfoDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true);
                                    }
                                }
                            }
                            pageIndex++;
                        }
                        if (pageIndex * pageSize >= reportRecordCount)
                        {
                            Logger.WriteLineInfo($"SynIsUserReportInfoDBAsync record processing: 100%");
                            break;
                        }
                        else
                        {
                            var tempValue = reportRecordCount == 0 ? 1 : reportRecordCount;
                            Logger.WriteLineInfo($"SynIsUserReportInfoDBAsync record processing: {(pageIndex * pageSize * 100 / tempValue)}%");
                        }
                    }
                }
            }
            else
            {
                Logger.WriteLineWarn($"ReportDBService SynIsUserReportInfoDBAsync request: {Newtonsoft.Json.JsonConvert.SerializeObject(request)}");
            }
            Logger.WriteLineWarn($"ReportDBService SynIsUserReportInfoDBAsync Completed");
            return res;
        }

        /// <summary>
        /// 查询报告增量数据
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        /// <returns></returns>
        public async Task<IList<ReportInfoResultDTO>> FindReportsDataCenterAsync(DateTime startTime, DateTime endTime)
        {
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendGte("BaseData.CreateTime", startTime);
            customerFilter.AppendGte("BaseData.UpdateTime", startTime);
            customerFilter.AppendLte("BaseData.CreateTime", endTime);
            customerFilter.AppendLte("BaseData.UpdateTime", endTime);
            var reportResult = await _reportDBRepository.FindAllAsync(customerFilter.AndCombine());
            var result = new List<ReportInfoResultDTO>();
            if (reportResult.Count > 0)
            {
                //获取设备信息
                var deviceCodeCodes = reportResult.Select(v => v.DeviceCode).ToList();
                var filter = new Dictionary<string, List<string>>();
                filter.Add("Code", deviceCodeCodes);
                var deviceList = await _deviceInfoDBRepository.FindInAsync(filter);
                //获取病人信息
                var patientCodes = reportResult.Select(v => v.PatientCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", patientCodes);
                var patientList = await _patientInfoDBRepository.FindInAsync(filter);
                //获取医院信息
                var orgCodeList = patientList?.Select(v => v.RootOrganizationCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", orgCodeList);
                var orgList = await _organizationDBRepository.FindInAsync(filter);
                //用户信息
                var userCodes = reportResult.Select(v => v.ReportUserCode).ToList();
                filter = new Dictionary<string, List<string>>();
                filter.Add("Code", userCodes);
                var userList = await _userDBRepository.FindInAsync(filter);

                foreach (var item in reportResult)
                {
                    var patient = patientList.FirstOrDefault(c => c.Code == item.PatientCode);
                    var org = orgList.FirstOrDefault(c => c.Code == patient?.RootOrganizationCode);
                    var device = deviceList.FirstOrDefault(c => c.Code == item.DeviceCode);
                    var user = userList.FirstOrDefault(c => c.Code == item.ReportUserCode);
                    var deviceOrg = orgList.FirstOrDefault(c => c.Code == device?.OrganizationCode);

                    result.Add(new ReportInfoResultDTO()
                    {
                        Code = item.Code,
                        CreateTime = item.CreateTime,
                        UpdateTime = item.UpdateTime,
                        OrganizationCode = org?.Code,
                        OrganizationName = org?.OrganizationName,
                        RecordCode = item.RecordCode,
                        ReportUserCode = item.ReportUserCode,
                        ReportUserName = user?.NickName,
                        DeviceInfo = new DeviceView()
                        {
                            Code = device?.Code,
                            Name = device?.Name,
                            Description = device?.Description,
                            UniqueCode = device?.ShortCode,
                            OrganizationCode = device?.OrganizationCode,
                            OrganizationName = deviceOrg?.OrganizationName,
                            DeviceType = device?.DeviceType,
                            DeviceModel = device?.DeviceModel,
                        },
                        PatientCode = item.PatientCode,
                        PatientId = item.PatientId,
                        ReportTemplateJson = item.ReportTemplateJson,
                        ReportDatasJson = item.ReportDatasJson,
                        PatientName = item.PatientName,
                        ReportLabels = item.ReportLabels,
                        PlatformLabels = item.PlatformLabels,
                        DiagnosisLabels = item.DiagnosisLabels,
                        ReportOrgan = item.ReportOrgan,
                        ReportPreviewList = item.ReportPreviewList?.MappingTo<List<ReportPreviewDTO>>(),
                        ReferralRecordCode = item.ReferralRecordCode,
                        ReportType = item.ReportType,
                    });

                }
            }

            return result;
        }


        /// <summary>
        /// 批量新增报告标签数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        public async Task<bool> BatchInsertReportLabelAsync(BatchInsertReportLabelRequest request)
        {
            var syncType = request.SyncType;
            if (request?.ReportLabelData?.Count > 0)
            {
                var insertList = new List<ReportLabelDO>();
                var updateList = new List<ReportLabelDO>();
                //找到相同的
                var ids = request.ReportLabelData.Select(c => c.ReportLabelCode).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var reportLabelList = await _reportLabelDBRepository.FindInAsync(dic);

                foreach (var reportLabel in request.ReportLabelData)
                {
                    var isUpdate = reportLabelList.Any(x => x.ReportLabelCode == reportLabel.ReportLabelCode);
                    var data = reportLabel.MappingTo<ReportLabelDO>();
                    data.Code = reportLabel.ReportLabelCode.Trim();
                    data.IsDelete = reportLabel.IsDelete;
                    if (syncType == SyncDBEnum.Migrate)
                    {
                        if (isUpdate)
                        {
                            updateList.Add(data);
                        }
                        else
                        {
                            insertList.Add(data);
                        }
                    }
                    else
                    {
                        if (isUpdate)
                        {
                            await _reportLabelDBRepository.UpdateOneForSyncDBAsync("Code", data.ReportLabelCode, data, null);
                        }
                        else
                        {
                            await _reportLabelDBRepository.InsertOneForAsyncDBAsync(data, new Dictionary<string, string>(), "");
                        }
                    }
                }
                if (syncType == SyncDBEnum.Migrate)
                {
                    if (insertList.Any())
                    {
                        await _reportLabelDBRepository.InsertAllWithoutOplogLogAsync(insertList);
                    }
                    if (updateList.Any())
                    {
                        await _reportLabelDBRepository.UpdateAllWithoutOplogLogAsync(updateList);
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// 根据记录编码将报告中的病人编码更新为指定编码
        /// </summary>
        /// <param name="patientCode">病人编码</param>
        /// <param name="recordCodes">记录编码集合</param>
        /// <returns></returns>
        public async Task<bool> UpdateReportPatientCodeByRecordCodes(string patientCode, List<string> recordCodes)
        {
            try
            {
                MongoCustomerFilter<MongoTable> filter = new MongoCustomerFilter<MongoTable>();
                filter.AppendQueryItemInFilter(MongoCombineTypeEnum.Or, "RecordCode", recordCodes);
                var reports = await _reportDBRepository.FindAllAsync(filter.AndCombine());
                foreach (var report in reports)
                {
                    report.PatientCode = patientCode;
                    await _reportDBRepository.UpdateOneAsync("Code", report.Code, report, string.Empty);
                }
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"UpdateReportPatientCodeByRecordCodes error {ex}");
                return false;
            }
        }

        /// <summary>
        /// 导出词条信息
        /// </summary>
        /// <param name="request">查询词条详情请求实体</param>
        /// <returns></returns>
        public async Task<ExportThesaurusTemplateResult> ExportThesaurusDBAsync(FindThesaurusByCodeRequest request)
        {
            var response = new ExportThesaurusTemplateResult()
            {
                ThesaurusTemplateInfo = new ThesaurusTemplateInfoDTO()
            };
            MongoCustomerFilter<MongoTable> mongoCustomerFilter = new();
            mongoCustomerFilter.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", request.ThesaurusCode));
            var filter = mongoCustomerFilter.AndCombine();
            var doData = await _thesaurusDBRepository.FindOneAsync(filter);
            if (doData != null && doData.ThesaurusItems?.Count > 0)
            {
                var thesaurusList = new List<ThesaurusInfoDTO>();
                var firstList = doData.ThesaurusItems.FindAll(c => string.IsNullOrEmpty(c.ParentItemCode) && c.ThesaurusItemType == ThesaurusItemTypeEnum.Classification) ?? new List<ThesaurusItemDO>();
                if (firstList?.Count > 0)
                {
                    foreach (var firstEntity in firstList)
                    {
                        var thesaurusInfo = new ThesaurusInfoDTO()
                        {
                            Name = firstEntity.ThesaurusItemName,
                            Applications = new List<ApplicationInfoDTO>()
                        };
                        var secondList = doData.ThesaurusItems.FindAll(c => c.ParentItemCode == firstEntity.ThesaurusItemCode && c.ThesaurusItemType == ThesaurusItemTypeEnum.Classification) ?? new List<ThesaurusItemDO>();
                        if (secondList?.Count > 0)
                        {
                            foreach (var secondEntity in secondList)
                            {
                                var application = new ApplicationInfoDTO()
                                {
                                    Name = secondEntity.ThesaurusItemName,
                                    Items = new List<ApplicationItemDTO>()
                                };
                                var thirdList = doData.ThesaurusItems.FindAll(c => c.ParentItemCode == secondEntity.ThesaurusItemCode && c.ThesaurusItemType == ThesaurusItemTypeEnum.Contents) ?? new List<ThesaurusItemDO>();
                                if (thirdList?.Count > 0)
                                {
                                    foreach (var thirdEntity in thirdList)
                                    {
                                        var applicationItem = new ApplicationItemDTO()
                                        {
                                            Description = thirdEntity.ThesaurusItemDescription,
                                            Summary = thirdEntity.ThesaurusItemConclusion
                                        };
                                        application.Items.Add(applicationItem);
                                    }
                                }
                                thesaurusInfo.Applications.Add(application);
                            }
                        }
                        thesaurusList.Add(thesaurusInfo);
                    }
                }
                response.ThesaurusName = doData.ThesaurusName;
                response.ThesaurusTemplateInfo.ThesaurusList = thesaurusList;
            }
            return response;
        }

        /// <summary>
        /// 导入词条
        /// </summary>
        /// <param name="thesaurusName"></param>
        /// <param name="json"></param>
        /// <param name="thesaurusUsageTypeEnum"></param>
        /// <returns></returns>
        public async Task<bool> ImportThesaurusDBAsync(ImportThesaurusInfoRequest request)
        {
            Logger.WriteLineWarn($"WingMongoDBModule ImportThesaurusDBAsync init request: {request.LanguageCode}-{request.ThesaurusType}-{request.ThesaurusUsageType}-{request.ThesaurusStateType}-{request.ThesaurusName}");
            var thesaurusCode = CodeCreator.CreateCode("ThesaurusDO");
            var thesaurusDO = new ThesaurusDO();
            thesaurusDO.Code = thesaurusCode;
            thesaurusDO.IsDefault = true;
            thesaurusDO.ThesaurusName = request.ThesaurusName;
            thesaurusDO.ThesaurusStateType = request.ThesaurusStateType;
            thesaurusDO.ThesaurusUsageType = request.ThesaurusUsageType;
            thesaurusDO.ThesaurusType = request.ThesaurusType;
            if (!string.IsNullOrEmpty(request.PersonalCode) && request.ThesaurusType == ThesaurusTypeEnum.Personal)
            {
                thesaurusDO.OrganizationCode = "";
            }
            else
            {
                thesaurusDO.OrganizationCode = request.OrganizationCode;
            }
            thesaurusDO.PersonalCode = request.PersonalCode;
            thesaurusDO.LanguageCode = request.LanguageCode;
            thesaurusDO.ThesaurusItems = new List<ThesaurusItemDO>();
            foreach (var item in request.ThesaurusTemplateInfo.ThesaurusList)
            {
                var itemDO = new ThesaurusItemDO();
                itemDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                itemDO.ThesaurusItemType = ThesaurusItemTypeEnum.Classification;
                itemDO.ThesaurusItemName = item.Name;
                itemDO.ThesaurusItemDescription = "";
                itemDO.ThesaurusItemConclusion = "";
                itemDO.ParentItemCode = "";
                thesaurusDO.ThesaurusItems.Add(itemDO);
                foreach (var application in item.Applications)
                {
                    var applicationDO = new ThesaurusItemDO();
                    applicationDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                    applicationDO.ThesaurusItemType = ThesaurusItemTypeEnum.Classification;
                    applicationDO.ThesaurusItemName = application.Name;
                    applicationDO.ThesaurusItemDescription = "";
                    applicationDO.ThesaurusItemConclusion = "";
                    applicationDO.ParentItemCode = itemDO.ThesaurusItemCode;
                    thesaurusDO.ThesaurusItems.Add(applicationDO);
                    foreach (var applicationItem in application.Items)
                    {
                        var applicationItemDO = new ThesaurusItemDO();
                        applicationItemDO.ThesaurusItemCode = CodeCreator.CreateCode("ThesaurusItem");
                        applicationItemDO.ThesaurusItemType = ThesaurusItemTypeEnum.Contents;
                        applicationItemDO.ThesaurusItemName = "";
                        applicationItemDO.ParentItemCode = applicationDO.ThesaurusItemCode;
                        applicationItemDO.ThesaurusItemDescription = applicationItem.Description;
                        applicationItemDO.ThesaurusItemConclusion = applicationItem.Summary;
                        thesaurusDO.ThesaurusItems.Add(applicationItemDO);
                    }
                }
            }
            var result = await _thesaurusDBRepository.InsertOneAsync(thesaurusDO, new Dictionary<string, string>(), string.Empty, true);
            Logger.WriteLineWarn($"WingMongoDBModule ImportThesaurusDBAsync init res: {result}");

            return !string.IsNullOrWhiteSpace(result);
        }

        /// <summary>
        /// 清洗数据-重新清洗报告模板数据及个人绑定数据
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> ModifyReportTemplateHistoryDataDBAsync(BatchModifyReportTemplateRequest request)
        {
            long res = 0;
            var defaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";
            var animalDefaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"雌性\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Самка\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";

            if (request?.ReportTemplateMigratorys?.Count > 0)
            {
                //全部置为false
                MongoCustomerFilter<MongoDataItem> subFilter = new();
                MongoCustomerFilter<MongoTable> filter = new();

                var reportTemplateCodes = request.ReportTemplateMigratorys.Select(c => c.ReportTemplateCode).Distinct().ToList();
                subFilter.AppendEq((item => item.Key, "ReferenceReportTemplateCode"));
                subFilter.AppendIn(item => item.Value, reportTemplateCodes);
                filter.SubFilter<MongoDataItem>(item => item.QueryData.QueryItems, subFilter.AndCombine());

                subFilter = new();
                subFilter.AppendEq((item => item.Key, "Code"));
                subFilter.AppendIn(item => item.Value, reportTemplateCodes);
                filter.SubFilter<MongoDataItem>(item => item.QueryData.QueryItems, subFilter.AndCombine());

                var mongoCustomerFilter = filter.OrCombine();
                var findList = await _reportTemplateDBRepository.FindAllAsync(mongoCustomerFilter);
                if (findList?.Count > 0)
                {
                    foreach (var findItem in findList)
                    {
                        findItem.IsDelete = true;
                        var updateResult = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", findItem.Code, findItem, string.Empty, true);
                    }
                }

                //找到相同的
                var ids = request.ReportTemplateMigratorys.Select(c => c.ReportTemplateCode).Distinct().ToList();
                var dic = new Dictionary<string, List<string>>();
                dic.Add("Code", ids);
                var reportTemplateList = await _reportTemplateDBRepository.FindInAsync(dic);
                //求同存异
                if (reportTemplateList?.Count > 0)
                {
                    var existCodes = reportTemplateList.Select(c => c.ReportTemplateCode).Distinct().ToList();
                    var updateData = request.ReportTemplateMigratorys.Where(c => existCodes.Contains(c.ReportTemplateCode));
                    var insertData = request.ReportTemplateMigratorys.Where(c => !existCodes.Contains(c.ReportTemplateCode));
                    if (insertData?.Count() > 0)
                    {
                        //新增
                        var req = insertData.Select(
                            f =>
                            {
                                var data = f.MappingTo<ReportTemplateDO>();
                                data.Code = f.ReportTemplateCode.Trim();
                                data.IsDelete = false;
                                data.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                                if (data.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                                {
                                    data.ReportDatasJson = animalDefaultReportDatasJson;
                                }
                                else
                                {
                                    data.ReportDatasJson = defaultReportDatasJson;
                                }
                                return (data, new Dictionary<string, string>(), string.Empty);
                            }
                        ).ToList();
                        res = await _reportTemplateDBRepository.InsertAllAsync(req);
                    }
                    if (updateData?.Count() > 0)
                    {
                        var req = updateData.Select(f =>
                        {
                            var updateEntity = f.MappingTo<ReportTemplateDO>();
                            updateEntity.Code = f.ReportTemplateCode.Trim();
                            updateEntity.IsDelete = false;
                            updateEntity.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                            if (updateEntity.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                            {
                                updateEntity.ReportDatasJson = animalDefaultReportDatasJson;
                            }
                            else
                            {
                                updateEntity.ReportDatasJson = defaultReportDatasJson;
                            }
                            return ("Code", f.ReportTemplateCode, updateEntity, string.Empty);
                        }).ToList();
                        res = await _reportTemplateDBRepository.UpdateAllAsync(req);
                    }
                }
                else
                {
                    //新增
                    var req = request.ReportTemplateMigratorys.Select(
                        f =>
                        {
                            var data = f.MappingTo<ReportTemplateDO>();
                            data.Code = f.ReportTemplateCode.Trim();
                            data.IsDelete = false;
                            data.ReportTemplateJson = TransferReportTemplateInfos(f.ReportTemplateJson);
                            if (data.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                            {
                                data.ReportDatasJson = animalDefaultReportDatasJson;
                            }
                            else
                            {
                                data.ReportDatasJson = defaultReportDatasJson;
                            }
                            return (data, new Dictionary<string, string>(), string.Empty);
                        }
                    ).ToList();
                    res = await _reportTemplateDBRepository.InsertAllAsync(req);
                }
            }
            //更新org和用户
            if (request.OrganizationDefaultTemplates?.Count > 0)
            {
                foreach (var item in request.OrganizationDefaultTemplates)
                {
                    if (!string.IsNullOrEmpty(item.DefaultTemplateId))
                    {
                        if (item.ReportTemplateIds?.Count > 0)
                        {
                            if (!item.ReportTemplateIds.Contains(item.DefaultTemplateId))
                            {
                                item.ReportTemplateIds.Add(item.DefaultTemplateId);
                            }
                            else
                            {
                                //包含。不处理
                            }
                        }
                        else
                        {
                            item.ReportTemplateIds = new List<string>() { item.DefaultTemplateId };
                        }
                    }
                    if (item.ReportTemplateIds?.Count > 0)
                    {
                        var reportDic = new Dictionary<string, string>();
                        //获取所有的报告数据
                        var ReportTemplateIdsDic = new Dictionary<string, List<string>>();
                        ReportTemplateIdsDic.Add("Code", item.ReportTemplateIds.ToList());
                        var entities = await _reportTemplateDBRepository.FindInAsync(ReportTemplateIdsDic);
                        foreach (var idItem in item.ReportTemplateIds)
                        {
                            bool isDefault = false;
                            if (idItem == item.DefaultTemplateId)
                            {
                                isDefault = true;
                            }
                            //检查是否存在
                            reportDic = new Dictionary<string, string>();
                            reportDic.Add("ReferenceReportTemplateCode", idItem);
                            reportDic.Add("OrganizationCode", item.OrganizationCode.Trim());
                            reportDic.Add("ReportTemplateType", ReportTemplateTypeEnum.Organization.ToString());
                            var reportTemplateList = await _reportTemplateDBRepository.FindListAsync(reportDic);
                            if (reportTemplateList?.Count > 0)
                            {
                                //找到了。不更新
                            }
                            else
                            {
                                var findEntity = entities?.FirstOrDefault(d => d.ReportTemplateCode == idItem) ?? new ReportTemplateDO();
                                if (!string.IsNullOrEmpty(findEntity.Code))
                                {
                                    //新增
                                    var dbRequest = new AddReportTemplateDBRequest();
                                    dbRequest.Data = new WingInterfaceLibrary.DTO.ReportTemplate.ReportTemplateDTO();
                                    dbRequest.Data.IsDefault = isDefault;
                                    dbRequest.Data.ReportTemplateName = findEntity.ReportTemplateName;
                                    dbRequest.Data.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                                    dbRequest.Data.ReferenceReportTemplateCode = findEntity.Code;
                                    if (findEntity.TemplatePreviewList?.Count > 0)
                                    {
                                        dbRequest.Data.TemplatePreviewList = findEntity.TemplatePreviewList.MappingTo<List<ReportPreviewDTO>>();
                                    }
                                    dbRequest.Data.ReportTemplateJson = findEntity.ReportTemplateJson ?? "{}";
                                    if (findEntity.ReportTemplateUsageType == ReportTemplateUsageTypeEnum.Animal)
                                    {
                                        dbRequest.Data.ReportDatasJson = animalDefaultReportDatasJson;
                                    }
                                    else
                                    {
                                        dbRequest.Data.ReportDatasJson = findEntity.ReportDatasJson ?? "{}";
                                    }
                                    dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Organization;
                                    dbRequest.Data.LanguageCode = findEntity.LanguageCode;
                                    dbRequest.Data.OrganizationCode = item.OrganizationCode.Trim();
                                    dbRequest.Data.ReportTemplateUsageType = findEntity.ReportTemplateUsageType;
                                    await AddReportTemplateAsync(dbRequest);
                                }
                                else
                                {
                                    Logger.WriteLineWarn($"OrganizationDBService BatchInsertOrganizationDataDBAsync ReportTemplateSyn Error, ReportTemplateCode: {idItem}");
                                }
                            }
                        }

                        //更新用户
                        reportDic = new Dictionary<string, string>();
                        reportDic.Add("OrganizationCode", item.OrganizationCode.Trim());
                        reportDic.Add("ReportTemplateType", ReportTemplateTypeEnum.Organization.ToString());
                        var reportTemplateResultList = await _reportTemplateDBRepository.FindListAsync(reportDic);
                        if (item.UserCodes?.Count > 0 && reportTemplateResultList?.Count > 0)
                        {
                            var reportTemplateInfo = reportTemplateResultList.FirstOrDefault() ?? new ReportTemplateDO();
                            if (!string.IsNullOrEmpty(reportTemplateInfo.Code))
                            {
                                foreach (var userItem in item.UserCodes)
                                {
                                    var setResult = SetUserDefaultReportTemplateCodeAsync(userItem, reportTemplateInfo.Code).Result;
                                }
                            }
                        }
                    }
                }
            }

            //重新绑定预览图数据
            MongoCustomerFilter<MongoTable> customerFilter = new();
            customerFilter.AppendSizeLt("BaseData.TemplatePreviewList", 1);
            var allReportTemplateList = await _reportTemplateDBRepository.FindAllAsync(customerFilter.AndCombine());
            if (allReportTemplateList?.Count > 0)
            {
                foreach (var item in allReportTemplateList)
                {
                    item.ReportDatasJson = defaultReportDatasJson;
                    var r = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", item.Code, item, string.Empty, true);
                }
            }
            var result = res > 0 ? true : false;
            return result;
        }

        /// <summary>
        /// 报告模板预览图数据清洗
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> UpgradeDefaultReportTemplateToOtherNodeAsync(SaveOrgTemplateDataRequest request)
        {
            var res = false;
            var dicInfo = new Dictionary<string, string>();
            dicInfo.Add("ReportTemplateType", Enum.GetName(ReportTemplateTypeEnum.Default));
            dicInfo.Add("IsDefault", "True");
            if (!string.IsNullOrEmpty(request.DefaultReportTemplateCode))
            {
                dicInfo.Add("Code", request.DefaultReportTemplateCode);
            }
            var list = await _reportTemplateDBRepository.FindListAsync(dicInfo);
            if (list?.Count > 0)
            {
                var hostDic = new Dictionary<string, string>();
                hostDic.Add("https://cdn-bj.fis.plus", "https://flyinsono-bj-1300984704.cos.ap-beijing.myqcloud.com");
                hostDic.Add("https://cdn-hk.fis.plus", "https://flyinsono-hk-1300984704.cos.ap-hongkong.myqcloud.com");
                hostDic.Add("https://cdn-fra.fis.plus", "https://flyinsono-fra-1300984704.cos.eu-frankfurt.myqcloud.com");
                hostDic.Add("http://cdn-bj.fis.plus", "https://flyinsono-bj-1300984704.cos.ap-beijing.myqcloud.com");
                hostDic.Add("http://cdn-hk.fis.plus", "https://flyinsono-hk-1300984704.cos.ap-hongkong.myqcloud.com");
                hostDic.Add("http://cdn-fra.fis.plus", "https://flyinsono-fra-1300984704.cos.eu-frankfurt.myqcloud.com");
                foreach (var item in list)
                {
                    if (item.TemplatePreviewList?.Count > 0)
                    {
                        foreach (var priviewItem in item.TemplatePreviewList)
                        {
                            var fileTokenUrlStr = priviewItem.FileToken;
                            var thumbnailTokenUrl = priviewItem.ThumbnailToken;

                            try
                            {
                                if (!string.IsNullOrEmpty(fileTokenUrlStr) && fileTokenUrlStr.Split(';').Length > 0)
                                {
                                    var tempList = fileTokenUrlStr.Split(';').ToList();
                                    foreach (var fileTokenUrl in tempList)
                                    {
                                        var tempFileTokenUrl = "";
                                        foreach (var dicKey in hostDic.Keys)
                                        {
                                            if (!string.IsNullOrEmpty(fileTokenUrl) && fileTokenUrl.Contains(dicKey))
                                            {
                                                tempFileTokenUrl = fileTokenUrl.Replace(dicKey, hostDic[dicKey]);
                                            }
                                        }
                                        var _fileTransfer = new FileTransferService();
                                        var fileName = Path.GetFileName(tempFileTokenUrl);
                                        var result = await _fileTransfer.CopyToOtherNode(tempFileTokenUrl, fileName);
                                    }
                                }
                                if (!string.IsNullOrEmpty(thumbnailTokenUrl))
                                {
                                    foreach (var dicKey in hostDic.Keys)
                                    {
                                        if (!string.IsNullOrEmpty(thumbnailTokenUrl) && thumbnailTokenUrl.Contains(dicKey))
                                        {
                                            thumbnailTokenUrl = thumbnailTokenUrl.Replace(dicKey, hostDic[dicKey]);
                                        }
                                    }

                                    var _fileTransfer = new FileTransferService();
                                    var fileName = Path.GetFileName(thumbnailTokenUrl);
                                    var result = await _fileTransfer.CopyToOtherNode(thumbnailTokenUrl, fileName);
                                }
                                res = true;
                            }
                            catch (Exception ex)
                            {
                                Logger.WriteLineWarn($"CopyToOtherNodeAysnc:{fileTokenUrlStr},{thumbnailTokenUrl},{ex.ToString()}");
                            }
                        }
                    }
                }
            }
            return res;
        }


        /// <summary>
        /// 分页查询增量报告集合
        /// </summary>
        /// <returns></returns>
        public async Task<PageResult<ReportDTO>> FindReportsByUpdateTimeDBAsync(FindCollectionByTimeDBRequest request)
        {
            var startTime = request.StartTime;
            var pageIndex = request.PageIndex;
            var pageSize = request.PageSize;
            MongoCustomerFilter<MongoTable> customerFilter = new();//常规查询

            customerFilter.AppendGt("BaseData.UpdateTime", startTime);
            var filter = customerFilter.AndCombine();
            var pagedResult = await _reportDBRepository.GetPages(new DBPageRequest
            {
                CurrentPage = pageIndex,
                PageSize = pageSize,
                Filter = filter,
                Sort = Builders<MongoTable>.Sort.Ascending("BaseData.UpdateTime"),
            }, onlyNotDeleted: false);
            var result = new PageResult<ReportDTO>();
            result.PageIndex = pagedResult.CurrentPage;
            result.PageSize = pagedResult.PageSize;
            result.TotalCount = pagedResult.DataCount;
            result.PageData = pagedResult.PageData?.MappingTo<List<ReportDTO>>() ?? new List<ReportDTO>();
            return result;
        }

        /// <summary>
        /// 保存报告记录
        /// </summary>
        /// <returns></returns>
        public async Task<bool> SaveReportInfoDBAsync(ReportDTO request)
        {
            var code = request.ReportCode;
            var reportDO = request.MappingTo<ReportDO>();
            var entity = await _reportDBRepository.FindOneAsync("Code", code);
            if (entity != null && !string.IsNullOrWhiteSpace(entity.Code))
            {
                await _reportDBRepository.UpdateOneAsync("Code", code, reportDO, "");
            }
            else
            {
                await _reportDBRepository.InsertOneAsync(reportDO, new Dictionary<string, string>(), "");
            }
            return true;
        }



        #region db 1.1 version upgrade
        /// <summary>
        /// 词条数据清洗，只针对个人和机构词条数据
        /// isdefault：个人和机构必须是默认的
        /// publish：个人和机构必须要发布的
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpgradeThesaurusDataDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeThesaurusDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //查询个人和机构词条数据
                var dic = new Dictionary<string, List<string>>();
                var typeList = new List<string>() { Enum.GetName(ThesaurusTypeEnum.Organization), Enum.GetName(ThesaurusTypeEnum.Personal) };
                dic.Add("ThesaurusType", typeList);
                var list = await _thesaurusDBRepository.FindInAsync(dic);
                if (list?.Count > 0)
                {
                    foreach (var item in list)
                    {
                        item.IsDefault = true;
                        item.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
                        if (item.ThesaurusType == ThesaurusTypeEnum.Organization)
                        {
                            item.PersonalCode = "";
                        }
                        if (item.ThesaurusType == ThesaurusTypeEnum.Personal)
                        {
                            item.OrganizationCode = "";
                        }
                        res = await _thesaurusDBRepository.UpdateOneForSyncDBAsync("Code", item.ThesaurusCode, item, string.Empty, true, false);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 新增西班牙的两个默认词条
        /// isdefault：个人和机构必须是默认的
        /// publish：个人和机构必须要发布的
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateThesaurusForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateThesaurusForESDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                MongoCustomerFilter<MongoTable> filterThesaurus = new MongoCustomerFilter<MongoTable>();
                filterThesaurus.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", "Entrada humana"));
                if (!await _thesaurusDBRepository.Exists(filterThesaurus.AndCombine()))
                {
                    await AddThesaurusDO("Entrada humana", "HumanGeneral-ES.json", "es-ES", ThesaurusUsageTypeEnum.People, "C43515A8D1E3428095EBADC4369CEE87");
                }
                filterThesaurus = new MongoCustomerFilter<MongoTable>();
                filterThesaurus.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("ReportTemplateName", "Entrada para animales"));
                if (!await _thesaurusDBRepository.Exists(filterThesaurus.AndCombine()))
                {
                    await AddThesaurusDO("Entrada para animales", "AnimalGeneral-ES.json", "es-ES", ThesaurusUsageTypeEnum.Animal, "E4872A35EC1941308001CF08C7279EA7");
                }
                return res;
            }
        }

        /// <summary>
        /// 报告模板数据清洗，只针对个人和机构模板数据
        /// isdefault：个人和机构必须是默认的
        /// publish：个人和机构必须要发布的
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpgradeReportTemplateDataDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeReportTemplateDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //查询个人和机构模板数据
                var dic = new Dictionary<string, List<string>>();
                var typeList = new List<string>() { Enum.GetName(ReportTemplateTypeEnum.Organization), Enum.GetName(ReportTemplateTypeEnum.Personal) };
                dic.Add("ReportTemplateType", typeList);
                var list = await _reportTemplateDBRepository.FindInAsync(dic);
                if (list?.Count > 0)
                {
                    foreach (var item in list)
                    {
                        item.IsDefault = true;
                        item.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                        if (item.ReportTemplateType == ReportTemplateTypeEnum.Organization)
                        {
                            item.PersonalCode = "";
                        }
                        if (item.ReportTemplateType == ReportTemplateTypeEnum.Personal)
                        {
                            item.OrganizationCode = "";
                        }
                        res = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", item.ReportTemplateCode, item, string.Empty, true, false);
                    }
                }

                var reportDic = new Dictionary<string, string>();
                reportDic.Add("e9d763614fd6a01cf35653c0cdae3d51", "辅助咨询单");
                reportDic.Add("c21a2ff88dfc51859918ad0c085c653a", "Auxiliary advice");
                reportDic.Add("03DED5E023C5419696F34CB1DF9F53FC", "Сопроводительный консультационный лист");
                reportDic.Add("db7007f993914b32963408bc2a074a85", "杏聆荟颈动脉模板");
                dic = new Dictionary<string, List<string>>();
                dic.Add("Code", reportDic.Keys.ToList());
                list = await _reportTemplateDBRepository.FindInAsync(dic);
                if (list == null)
                {
                    list = new List<ReportTemplateDO>();
                }
                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                var defaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"}]";

                foreach (var key in reportDic.Keys)
                {
                    var item = list.FirstOrDefault(c => c.Code == key);
                    if (!string.IsNullOrEmpty(item?.Code))
                    {
                        //修改
                        item.IsDefault = true;
                        item.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                        item.ReportTemplateName = reportDic[key];
                        res = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", item.ReportTemplateCode, item, string.Empty, true, false);
                    }
                    else
                    {
                        //新增
                        if (key == "03DED5E023C5419696F34CB1DF9F53FC")
                        {
                            await InsertAIRussiaReportTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                        else if (key == "e9d763614fd6a01cf35653c0cdae3d51")
                        {
                            await InsertAIZHTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                        else if (key == "c21a2ff88dfc51859918ad0c085c653a")
                        {
                            await InsertAIENTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                        else if (key == "db7007f993914b32963408bc2a074a85")
                        {
                            await InsertAICarotidTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 插入颈动脉模板
        /// </summary>
        private async Task InsertAICarotidTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "db7007f993914b32963408bc2a074a85";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter6 = new MongoCustomerFilter<MongoTable>();
            filter6.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "db7007f993914b32963408bc2a074a85"));
            if (!await _reportTemplateDBRepository.Exists(filter6.AndCombine()))
            {
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var fileRUPreviewPath1 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_1.jpg");
                    var fileRUPreviewPath2 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_2.jpg");
                    var fileRUPreviewPath3 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_3.jpg");
                    var fileRUPreviewPath4 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_4.jpg");
                    var fileRUPreviewPath5 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_5.jpg");
                    var fileRUPreviewPath6 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Preview_6.jpg");
                    var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Carotid_Thumbnail.jpg");

                    var fileRUThumbnailUrl = "";
                    string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                    var factName = oriFilename;
                    var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                    var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                    fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl1 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath1);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath1);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath1, waitCopyFilePath, true);
                    fileRUPreviewUrl1 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl2 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath2);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath2);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath2, waitCopyFilePath, true);
                    fileRUPreviewUrl2 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl3 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath3);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath3);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath3, waitCopyFilePath, true);
                    fileRUPreviewUrl3 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl4 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath4);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath4);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath4, waitCopyFilePath, true);
                    fileRUPreviewUrl4 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl5 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath5);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath5);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath5, waitCopyFilePath, true);
                    fileRUPreviewUrl5 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    var fileRUPreviewUrl6 = "";
                    oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath6);
                    factName = oriFilename;
                    factNameExtension = Path.GetExtension(fileRUPreviewPath6);
                    waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                    File.Copy(fileRUPreviewPath6, waitCopyFilePath, true);
                    fileRUPreviewUrl6 = vcsStorageUrlPrefix + (factName + factNameExtension);

                    reportTemplate.TemplatePreviewList.Add(
                    new ReportPreviewDO
                    {
                        FileToken = fileRUPreviewUrl1 + ";" + fileRUPreviewUrl2 + ";" + fileRUPreviewUrl3 + ";" + fileRUPreviewUrl4 + ";" + fileRUPreviewUrl5 + ";" + fileRUPreviewUrl6,
                        FileType = UploadFileTypeEnum.JPG,
                        ThumbnailToken = fileRUThumbnailUrl,
                        Language = "zh-CN"
                    });
                }
                else
                {
                    reportTemplate.TemplatePreviewList.Add(
                    new ReportPreviewDO
                    {
                        FileToken = "http://cdn-bj.fis.plus/3ED6CC18AA7D4553A4352FF01BA2F6F5.jpg;http://cdn-bj.fis.plus/F1244071D78C42D3B1624032A00B902A.jpg;http://cdn-bj.fis.plus/521232516EF048A7B6FD1D2E9FDE5EC2.jpg;http://cdn-bj.fis.plus/E0AC69139E3A4E87B73F38496084660C.jpg;http://cdn-bj.fis.plus/B5922B5625A2483DA1DDD06D23CDE664.jpg;http://cdn-bj.fis.plus/81AE213C2EBC4BFB9EF8233291CF0E56.jpg",
                        FileType = UploadFileTypeEnum.JPG,
                        ThumbnailToken = "http://cdn-bj.fis.plus/8E536E8E8CD64364891D54C6AC4E503E.jpg",
                        Language = "zh-CN"
                    });
                    reportTemplate.TemplatePreviewList.Add(
                    new ReportPreviewDO
                    {
                        FileToken = "http://cdn-bj.fis.plus/9E111FD12D8C428980CF0B7E4D0F40A7.jpg;http://cdn-bj.fis.plus/EB56FE9868944BD4AC9EE71E4946AD0F.jpg;http://cdn-bj.fis.plus/410C84AAA9BD45E2A531BDC016D94B51.jpg;http://cdn-bj.fis.plus/08B79375047048E688292CFC8F50ECA5.jpg;http://cdn-bj.fis.plus/E02D6AE34F9F4ABBAA2597E2830D69E7.jpg;http://cdn-bj.fis.plus/1E256FBC96D4409085F938550916325B.jpg",
                        FileType = UploadFileTypeEnum.JPG,
                        ThumbnailToken = "http://cdn-bj.fis.plus/781F0FF7B1324ECFAC7D9D9A5EDAA877.jpg",
                        Language = "en-US"
                    });
                    reportTemplate.TemplatePreviewList.Add(
                    new ReportPreviewDO
                    {
                        FileToken = "http://cdn-bj.fis.plus/521F94011A894E1DAEAE416143DD45AD.jpg;http://cdn-bj.fis.plus/917DBEF153CE4A3A934BA5B425D7D856.jpg;http://cdn-bj.fis.plus/973D4B4DDFC6493CBDC6DD049E4F544D.jpg;http://cdn-bj.fis.plus/AAB5ECB92A7D4150842FB486917711C6.jpg;http://cdn-bj.fis.plus/0A6E42BB9B074553A72FB846EC422D89.jpg;http://cdn-bj.fis.plus/3F725CB5DF8D487CA0D536C1449423A5.jpg",
                        FileType = UploadFileTypeEnum.JPG,
                        ThumbnailToken = "http://cdn-bj.fis.plus/23D35BE9940B4A76878EA7E10314950D.jpg",
                        Language = "ru-RU"
                    });
                }
                await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 插入AI中文模板
        /// </summary>
        private async Task InsertAIZHTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "e9d763614fd6a01cf35653c0cdae3d51";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "zh-CN";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter5 = new MongoCustomerFilter<MongoTable>();
            filter5.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "e9d763614fd6a01cf35653c0cdae3d51"));
            if (!await _reportTemplateDBRepository.Exists(filter5.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "AI");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/A79DAD84D03246119AA59D4476677D87.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/D5747F04B6D243D8A9EB45ABE8446C3F.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/3E1DB516859A4AC4B607E24A12C51E61.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/592C2352CCDF43349169D7AA7D7724F8.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 插入AI英文模板
        /// </summary>
        private async Task InsertAIENTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "c21a2ff88dfc51859918ad0c085c653a";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI_en.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "en-US";
            reportTemplate.TemplatePreviewList.Clear();

            MongoCustomerFilter<MongoTable> filter6 = new MongoCustomerFilter<MongoTable>();
            filter6.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "c21a2ff88dfc51859918ad0c085c653a"));
            if (!await _reportTemplateDBRepository.Exists(filter6.AndCombine()))
            {
                var tuple = await UploadReportTemplatePreviewImage(storageFold, vcsStorageUrlPrefix, "AI_en");
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item1,//"https://cdn-bj.fis.plus/A79DAD84D03246119AA59D4476677D87.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item2,//"https://cdn-bj.fis.plus/D5747F04B6D243D8A9EB45ABE8446C3F.jpg",
                    Language = "zh-CN"
                });
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = tuple.Item3,//"https://cdn-bj.fis.plus/3E1DB516859A4AC4B607E24A12C51E61.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = tuple.Item4,//"https://cdn-bj.fis.plus/592C2352CCDF43349169D7AA7D7724F8.jpg",
                    Language = "en-US"
                });
                await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 插入AI俄文模板
        /// </summary>
        private async Task InsertAIRussiaReportTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "03DED5E023C5419696F34CB1DF9F53FC";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "AI_ru.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "ru-RU";

            MongoCustomerFilter<MongoTable> filterAI = new MongoCustomerFilter<MongoTable>();
            filterAI.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "03DED5E023C5419696F34CB1DF9F53FC"));
            if (!await _reportTemplateDBRepository.Exists(filterAI.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "ИИ_Preview_RU.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "ИИ_Thumbnail.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ru-RU"
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 修复默认模板数据
        /// isdefault：个人和机构必须是默认的
        /// publish：个人和机构必须要发布的
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpgradeDefaultTemplateDataDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeDefaultTemplateDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //查询Extend表数据
                var list = await _userExtendsDBRepository.FindAllAsync();
                if (list?.Count > 0)
                {
                    //处理词条
                    var thesaurusCodes = list.Where(c => !string.IsNullOrEmpty(c.DefaultThesaurusCode) && c.IsDelete == false && !string.IsNullOrEmpty(c.UserCode))?.Select(d => d.DefaultThesaurusCode)?.Distinct()?.ToList();
                    if (thesaurusCodes?.Count > 0)
                    {
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", thesaurusCodes);
                        var thesaurusDatas = await _thesaurusDBRepository.FindInAsync(dic);
                        if (thesaurusDatas?.Count > 0)
                        {
                            foreach (var item in thesaurusCodes)
                            {
                                var isExist = thesaurusDatas.Exists(c => c.IsDelete == false && c.IsDefault == true && c.ThesaurusStateType == ThesaurusStatusTypeEnum.Published && c.Code == item);
                                if (!isExist)
                                {
                                    var userExtendInfo = list.FirstOrDefault(c => c.DefaultThesaurusCode == item) ?? new UserExtendsDO();
                                    //不存在，清理
                                    var dbRequest = new RemoveTemplateInfoDBRequest()
                                    {
                                        Code = item,
                                        UserCode = userExtendInfo.UserCode,
                                        Type = TemplateQueryTypeEnum.Thesaurus
                                    };
                                    var result = await ClearDefaultTemplateInfoAsync(dbRequest);
                                }
                            }
                        }
                    }
                    //处理报告模板
                    var reportTemplateCodes = list.Where(c => !string.IsNullOrEmpty(c.DefaultReportTemplateCode) && c.IsDelete == false && !string.IsNullOrEmpty(c.UserCode))?.Select(d => d.DefaultReportTemplateCode)?.Distinct()?.ToList();
                    if (reportTemplateCodes?.Count > 0)
                    {
                        var dic = new Dictionary<string, List<string>>();
                        dic.Add("Code", reportTemplateCodes);
                        var reportTemplateDatas = await _reportTemplateDBRepository.FindInAsync(dic);
                        if (reportTemplateDatas?.Count > 0)
                        {
                            foreach (var item in reportTemplateCodes)
                            {
                                var isExist = reportTemplateDatas.Exists(c => c.IsDelete == false && c.IsDefault == true && c.ReportTemplateStateType == ReportTemplateStatusTypeEnum.Published && c.Code == item);
                                if (!isExist)
                                {
                                    var userExtendInfo = list.FirstOrDefault(c => c.DefaultReportTemplateCode == item) ?? new UserExtendsDO();
                                    //不存在，清理
                                    var dbRequest = new RemoveTemplateInfoDBRequest()
                                    {
                                        Code = item,
                                        UserCode = userExtendInfo.UserCode,
                                        Type = TemplateQueryTypeEnum.ReportTemplate
                                    };
                                    var result = await ClearDefaultTemplateInfoAsync(dbRequest);
                                }
                            }
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 添加西班牙语词条
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateReportTemplateForESDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpgradeReportTemplateDataDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //查询个人和机构模板数据
                var dic = new Dictionary<string, List<string>>();
                var reportDic = new Dictionary<string, string>();
                reportDic.Add("0107BF5EC4E345E58F310C7358157BC45", "Rutina");
                reportDic.Add("94215D6BA3EB4A6C93BEE684CEF96C8E5", "Mascota");
                dic = new Dictionary<string, List<string>>();
                dic.Add("Code", reportDic.Keys.ToList());
                var list = await _reportTemplateDBRepository.FindInAsync(dic);
                if (list == null)
                {
                    list = new List<ReportTemplateDO>();
                }
                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                var defaultReportDatasJson = "[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Consulta externa\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Corazón\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dueño\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dueño\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Perro\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"}]";

                foreach (var key in reportDic.Keys)
                {
                    var item = list.FirstOrDefault(c => c.Code == key);
                    if (!string.IsNullOrEmpty(item?.Code))
                    {
                        //修改
                        item.IsDefault = true;
                        item.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                        item.ReportTemplateName = reportDic[key];
                        res = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", item.ReportTemplateCode, item, string.Empty, true, false);
                    }
                    else
                    {
                        //新增
                        if (key == "0107BF5EC4E345E58F310C7358157BC45")
                        {
                            await InsertGeneralESReportTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                        else if (key == "94215D6BA3EB4A6C93BEE684CEF96C8E5")
                        {
                            await InsertPetESReportTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 插入General 西班牙语模板
        /// </summary>
        private async Task InsertGeneralESReportTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "0107BF5EC4E345E58F310C7358157BC45";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Default_es.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "es-ES";

            MongoCustomerFilter<MongoTable> filterGeneral = new MongoCustomerFilter<MongoTable>();
            filterGeneral.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "0107BF5EC4E345E58F310C7358157BC45"));
            if (!await _reportTemplateDBRepository.Exists(filterGeneral.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "General_Preview_ES.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "General_Thum_ES.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "es-ES"
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 插入Pet 西班牙语模板
        /// </summary>
        private async Task InsertPetESReportTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "94215D6BA3EB4A6C93BEE684CEF96C8E5";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_es.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            reportTemplate.LanguageCode = "es-ES";

            MongoCustomerFilter<MongoTable> filterPet = new MongoCustomerFilter<MongoTable>();
            filterPet.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "94215D6BA3EB4A6C93BEE684CEF96C8E5"));
            if (!await _reportTemplateDBRepository.Exists(filterPet.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_Preview_ES.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_Thum_ES.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "es-ES"
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 添加西班牙语词条
        /// </summary>
        /// <returns>是否成功</returns>
        private async Task<bool> UpdateReportTemplateForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} UpdateReportTemplateForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;
                //查询个人和机构模板数据
                var dic = new Dictionary<string, List<string>>();
                var reportDic = new Dictionary<string, string>();
                reportDic.Add("0107BF5EC4E345E58F310C7358157BC451", "Șablon uman");
                reportDic.Add("94215D6BA3EB4A6C93BEE684CEF96C8E51", "Șablon animal");
                dic = new Dictionary<string, List<string>>();
                dic.Add("Code", reportDic.Keys.ToList());
                var list = await _reportTemplateDBRepository.FindInAsync(dic);
                if (list == null)
                {
                    list = new List<ReportTemplateDO>();
                }
                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                var defaultReportDatasJson = "{\"General\":{\"Version\":\"1.0\"},\"DataConfig\":[{\"Value\":\"XX市XXXXX医院\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"门诊\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"OutPatient\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"心脏\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Heart\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"zh-CN\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"XX市XXXXX医院\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XX City XXXXX Hospital\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"en-US\"},{\"Value\":\"主人\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Pet Owner\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"en-US\"},{\"Value\":\"张三\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Han MeiMei\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"en-US\"},{\"Value\":\"女\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Female\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"en-US\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"en-US\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"en-US\"},{\"Value\":\"狗\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dog\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"en-US\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"zh-CN\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"en-US\"},{\"Value\":\"1.左房内径正常，二尖瓣形态未见明显异常，彩色多普勒未测及返流。\\n2.左室内径正常，室间隔及左室壁厚度正常，节段运动未见异常。\\n3.主动脉内径正常，主动脉瓣形态未见异常，彩色多普勒未测及返流。\\n4.右房室内径正常，三尖瓣形态未见异常，彩色多普勒未测及返流。\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.Left atrial diameter was normal, mitral valve morphology was not abnormal, color Doppler did not detect reflux.\\r\\n2.Left ventricular diameter was normal, septal and left ventricular wall thickness were normal, and segmental motion was normal.\\r\\n3.Aortic diameter was normal, aortic valve morphology was normal, color Doppler flow was not detected and regurgitation was not observed.\\r\\n4.The right atrial ventricular diameter was normal, the shape of tricuspid valve was normal, the color Doppler flow was not detected and the regurgitation was not observed.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"en-US\"},{\"Value\":\"静息状态下超声心动未见明显异常\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"No obvious abnormality of echocardiography was found in resting state\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"en-US\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"王医生\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"Dr.Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"en-US\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Поликлиника\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Сердце\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Больница XX города XXXXX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Хозяин\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Иосиф Сталин\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Женский\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"30\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"ru-RU\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"1. Нормальный внутрипредсердный диаметр, нет явных аномалий митрального клапана, цветной допплер не выявил обратного потока.\\n2. Нормальный внутрижелудочковый диаметр, нормальная толщина межжелудочковой перегородки и стенки левого желудочка, нет нарушений сегментарного движения.\\n3. Нормальный диаметр аорты, нет аномалий аортального клапана, цветной допплер не выявил обратного потока.\\n4. Нормальный внутрипредсердно-желудочковый диаметр, нет аномалий трикуспидального клапана, цветной допплер не выявил обратного потока.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"При покое ультразвуковое сердечное исследование без явных аномалий\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Доктор Ван\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ru-RU\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Consulta externa\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Corazón\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"017ce934-446f-423d-be66-664aeb89c217\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"98d9600b-3d4e-4413-ad81-d12133c9d817\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"f6c50df4-06e2-4c1f-a21e-ffef3833f234\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"15162cff-f94d-44e9-9912-e77585256a59\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1949-10-1\",\"Key\":\"07c963b8-4961-46eb-91c2-ba089971cbe9\",\"TagId\":\"9aabaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientBirthday\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"ac472a5a-9d45-46c4-8362-1ad7ba51e90d\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":null,\"Key\":\"2a2b6e6e-4f85-4537-a8e4-6ee11c696fef\",\"TagId\":\"12340A8E-16FA-4528-B69D-C160F9B24AD5\",\"Name\":\"History\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"5cc4fbbe-9729-4577-a087-30f8eafcfed0\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"33e272e5-c9a9-4ebb-a813-389de1f48481\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"9af5ee42-1ff7-4fa0-8bcd-1abee897661c\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"6bca4c98-5087-451c-9aa6-9e103d2919ca\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"630a046d-2d60-4fd9-9117-ac27f802f016\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dueño\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Собака\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"f1a372bf-552f-4bd4-984b-d614e5f5e783\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Hospital XXXXX de la ciudad XX\",\"Key\":\"b2870a61-4d9c-4710-947f-f9743df09581\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dueño\",\"Key\":\"49c4ad30-f6b0-4b00-b86d-7aa31c4ce6b9\",\"TagId\":\"DB1202FB-51DF-4EB7-903A-2F6740CAD74B\",\"Name\":\"Owner\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Tom\",\"Key\":\"880c3847-d8d2-4dcb-9c83-b3d3b6f2233b\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Mujer\",\"Key\":\"5534673f-ffc5-4cd7-b397-0f35b8808f97\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"es-ES\"},{\"Value\":\"30 años\",\"Key\":\"644600c0-7f5e-4ddf-82e5-94d1495f530a\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"es-ES\"},{\"Value\":\"20190101\",\"Key\":\"4c06b52c-b854-41d7-9edb-31ea89169b44\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Perro\",\"Key\":\"0e26ede8-49c7-4fa3-896c-181264cf5557\",\"TagId\":\"FC5F7564-B63F-4185-8E1C-FC4BC5046332\",\"Name\":\"Species\",\"AILanguage\":\"es-ES\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"es-ES\"},{\"Value\":\"1. Diámetro auricular izquierdo normal, sin anormalidades evidentes en la morfología de la válvula mitral, sin regurgitación detectada mediante Doppler en color.\\n2. Diámetro ventricular izquierdo normal, grosor normal del septo interventricular y de la pared ventricular izquierda, sin anomalías en el movimiento segmentario.\\n3. Diámetro normal de la aorta, sin anormalidades en la morfología de la válvula aórtica, sin regurgitación detectada mediante Doppler en color.\\n4. Diámetro atrioventricular derecho normal, sin anormalidades evidentes en la morfología de la válvula tricúspide, sin regurgitación detectada mediante Doppler en color.\\n\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"es-ES\"},{\"Value\":\"No se observaron anomalías evidentes en el ecocardiograma en reposo.\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"es-ES\"},{\"Value\":\"2023-06-20\",\"Key\":\"679d8db3-aa5c-4101-abaa-2f1e02ce572e\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"es-ES\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c8960cd-028b-4157-8ccd-0fb827b055ca\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"es-ES\"},{\"Value\":\"25000059863\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362w\",\"Name\":\"PhysicalExamNumber\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"36.3\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3611\",\"Name\":\"PatientTemperature\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"73\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3612\",\"Name\":\"PatientPulseRate\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"19\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3613\",\"Name\":\"PatientRespiratoryRate\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"23.37\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3614\",\"Name\":\"PatientWeightIndex\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"82\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3615\",\"Name\":\"PatientWaist\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"144/86\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3616\",\"Name\":\"PatientLeftBloodPressure\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"136/90\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3617\",\"Name\":\"PatientRightBloodPressure\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"130\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3618\",\"Name\":\"PatientHemoglobin\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"4.96\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3619\",\"Name\":\"PatientLeukocyte\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"200\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3610\",\"Name\":\"PatientPlatelet\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2.108\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361a\",\"Name\":\"PatientLymphocyte\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2.373\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361b\",\"Name\":\"PatientNeutrophils\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"0.3\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361c\",\"Name\":\"PatientMonocyte\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"42.51\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361d\",\"Name\":\"PatientLymphocyteRatio\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"47.84\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361e\",\"Name\":\"PatientNeutrophilRatio\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"6.05\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361f\",\"Name\":\"PatientMonocyteRatio\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"4.53\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361g\",\"Name\":\"PatientErythrocyte\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"28.6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361h\",\"Name\":\"PatientMch\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"87.5\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361i\",\"Name\":\"PatientMcV\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"328\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361j\",\"Name\":\"PatientMchc\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"56.6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361k\",\"Name\":\"PatientErythrocyteDistributionWidth\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"39.6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361l\",\"Name\":\"PatientHematocrit\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"7.4\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361m\",\"Name\":\"PatientMeanPlateletVolume\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"12.6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361n\",\"Name\":\"PatientPlateletDistributionWidth\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"0.14\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361o\",\"Name\":\"PatientPlateletPct\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361p\",\"Name\":\"PatientUrinaryProtein\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361q\",\"Name\":\"PatientGlucose\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361r\",\"Name\":\"PatientUrinaryKetoneBody\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"+\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361s\",\"Name\":\"PatientBld\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361t\",\"Name\":\"PatientUrobilinogen\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361u\",\"Name\":\"PatientNitrite\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361v\",\"Name\":\"PatientPHValue\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.02\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361w\",\"Name\":\"PatientProportion\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"0.6\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361x\",\"Name\":\"PatientVitaminC\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"-\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361y\",\"Name\":\"PatientBilirubin\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"10.4\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3621\",\"Name\":\"PatientSerumAlanineAminotransferase\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"18.5\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3622\",\"Name\":\"PatientSGOT\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"23.9\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3623\",\"Name\":\"PatientTotalBilirubin\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3624\",\"Name\":\"PatientAlbumin\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3625\",\"Name\":\"PatientConjugatedBilirubin\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"71\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3626\",\"Name\":\"PatientCreatinine\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"3.96\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3627\",\"Name\":\"PatientUreaNitrogen\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3628\",\"Name\":\"PatientBloodPotassiumConcentration\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3629\",\"Name\":\"PatientBloodSodiumConcentration\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"6.64\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A3620\",\"Name\":\"PatientTotalCholesterol\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.54\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362a\",\"Name\":\"PatientTriglyceride\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"4.1\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362b\",\"Name\":\"PatientLowDensityCholesterol\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.38\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362c\",\"Name\":\"PatientHighDensityCholesterol\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"5.93\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362d\",\"Name\":\"PatientDextrose\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"**正常心电图**\\n窦性心律\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362e\",\"Name\":\"PatientEcgDiagnosticResult\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"40 deg\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362f\",\"Name\":\"PatientPAxis\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"20 deg\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362g\",\"Name\":\"PatientQRSAxis\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"44 deg\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362h\",\"Name\":\"PatientTAxis\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"114 ms\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362i\",\"Name\":\"PatientPWidth\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"166 ms\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362j\",\"Name\":\"PatientPRInterval\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"98 ms\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362k\",\"Name\":\"PatientQRSDuration\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"448/420 ms\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362l\",\"Name\":\"PatientQTQTCInterval\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.368/0.000mv\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362m\",\"Name\":\"PatientRV5SV\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"1.368mv\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362n\",\"Name\":\"PatientRV5SV1\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"河北省XX市XX县XX乡镇XX村\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362o\",\"Name\":\"PatientResidentialAddress\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"河北省XX市\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362p1\",\"Name\":\"PatientDomicileAddress\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XXX乡镇\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362q\",\"Name\":\"PatientStressName\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XXX村\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362r\",\"Name\":\"PatientVillage\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XXX卫生室\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362s\",\"Name\":\"PatientFilingUnit\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"张XX\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362t\",\"Name\":\"PatientArchivist\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"张XX\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362u\",\"Name\":\"PatientResponsibleDoctor\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"2023-06-20\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362v\",\"Name\":\"PatientCreatedDate\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"平和质（基本是） 阳虚质（倾向是）\\n要保持平和的心态\\n饮食清淡粗细搭配，忌辛辣油腻，戒烟限酒\\n规律起居，劳逸结合\\n多运动多保健\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362o\",\"Name\":\"PatientChineseMedicineResults\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"胆固醇偏高\\n红细胞分布宽度偏高\\n淋巴细胞比率偏高\\n中性粒细胞比率偏低\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362p\",\"Name\":\"PatientExceptionalResults\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"https://cdn-hk.fis.plus/ECGDemo.jpg\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A361z\",\"Name\":\"PatientECGImage\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"53 bpm\",\"Key\":\"\",\"TagId\":\"33D46E3F-A089-4B0A-8D1D-F4C7556BD62D\",\"Name\":\"HR\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"↑↓\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362x\",\"Name\":\"ResultPrompt\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"【B超指导】\\n建议定期按时进行B超筛查\\n【心电指导】\\n建议定期按时进行心电筛查\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362z\",\"Name\":\"PatientHealthGuidance\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"4.2\",\"Key\":\"\",\"TagId\":\"A1843C2E-B60A-483F-A5F0-B26F475A362y\",\"Name\":\"PatientBloodSugar\",\"AILanguage\":\"zh-CN\"},{\"Value\":\"XXXXX Hospital in XX City\",\"Key\":\"1790e450-19a6-4000-8895-3dda6a43aef9\",\"TagId\":\"7abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"HospitalName\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Zhang San\",\"Key\":\"97ec1395-3e8d-4ca9-a36a-0baace5646da\",\"TagId\":\"91bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientName\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Femeie\",\"Key\":\"f2e65e15-1a55-47ca-9e6c-42f9e8659daf\",\"TagId\":\"93bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientGender\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"30\",\"Key\":\"b84a9cbb-1b06-49b4-b8c5-6b2a57e24a58\",\"TagId\":\"92bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientAge\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Paznic\",\"Key\":\"135ec3fe-1a2b-46dd-98d7-cfbee06e2d66\",\"TagId\":\"9a3baa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientApplicationDepartment\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"20190101\",\"Key\":\"384adcda-7209-4ee8-b4fa-94dd27c3899f\",\"TagId\":\"90bbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientId\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Inima\",\"Key\":\"457086a8-1129-4847-8e7a-761aae984afd\",\"TagId\":\"9bbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"PatientInspectionSite\",\"AILanguage\":\"ro_RO\"},{\"Value\":[\"https://cdn-bj.fis.plus/DefaultUltrasonicImage2.png\",\"https://cdn-bj.fis.plus/DefaultUltrasonicImage3.png\"],\"Key\":\"cc81bba9-8d32-4389-a18e-f64ac3890ec9\",\"TagId\":\"9fbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportImages\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"1. Diametrul atrial stâng este normal și nu există anormalități evidente în forma valvei mitrale. Color Doppler nu a măsurat regurgitarea. \\n2. Diametrul ventriculului stâng este normal, septul interventricular și grosimea peretelui ventricularului stâng sunt normale și nu există anormalități în mișcarea segmentală. \\n3. Diametrul aortic este normal și nu există anormalități în forma valvei aortice. Color Doppler nu a măsurat regurgitarea. \\n4. Diametrul atrial drept este normal, și nu există anormalități în forma valvei tricuspide. Color Doppler nu a măsurat regurgitarea.\",\"Key\":\"e9e4b446-2f64-479c-8e15-cdda067cd00b\",\"TagId\":\"9dbbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportDescription\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Nu s-au observat anomalii semnificative în ecocardiografia în stare de repaus\",\"Key\":\"8b498af9-fbe4-4bfa-b3eb-2b02ebd785a1\",\"TagId\":\"9ebbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportSummary\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"2023-06-20\",\"Key\":\"415df52f-3fef-46c5-a39f-c3e3090e79f2\",\"TagId\":\"4abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportTime\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Dr. Wang\",\"Key\":\"35fbecf3-7900-479d-8d97-2daf23150be6\",\"TagId\":\"6abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"InspectionPhysician\",\"AILanguage\":\"ro_RO\"},{\"Value\":\"Dr. Wang\",\"Key\":\"5c51bdd6-6a10-40d3-8bcf-5a6a952d8ec7\",\"TagId\":\"5abbaa61-a91d-4399-a3f4-df29117d2d59\",\"Name\":\"ReportPhysician\",\"AILanguage\":\"ro_RO\"}]}";

                foreach (var key in reportDic.Keys)
                {
                    var item = list.FirstOrDefault(c => c.Code == key);
                    if (!string.IsNullOrEmpty(item?.Code))
                    {
                        //修改
                        item.IsDefault = true;
                        item.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
                        item.ReportTemplateName = reportDic[key];
                        res = await _reportTemplateDBRepository.UpdateOneForSyncDBAsync("Code", item.ReportTemplateCode, item, string.Empty, true, false);
                    }
                    else
                    {
                        //新增
                        if (key == "0107BF5EC4E345E58F310C7358157BC451")
                        {
                            await InsertGeneralForROReportTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                        else if (key == "94215D6BA3EB4A6C93BEE684CEF96C8E51")
                        {
                            await InsertPetForROReportTemplate(reportDic[key], defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                        }
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 插入General 罗马尼亚语模板
        /// </summary>
        private async Task InsertGeneralForROReportTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "0107BF5EC4E345E58F310C7358157BC451";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Default_ro.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            reportTemplate.LanguageCode = "ro-RO";

            MongoCustomerFilter<MongoTable> filterGeneral = new MongoCustomerFilter<MongoTable>();
            filterGeneral.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "0107BF5EC4E345E58F310C7358157BC451"));
            if (!await _reportTemplateDBRepository.Exists(filterGeneral.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "General_Preview_RO.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "General_Thum_RO.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ro-RO"
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 插入Pet 罗马尼亚语模板
        /// </summary>
        private async Task InsertPetForROReportTemplate(string name, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = "94215D6BA3EB4A6C93BEE684CEF96C8E51";
            reportTemplate.ReportTemplateName = name;
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_ro.json"));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            reportTemplate.LanguageCode = "ro-RO";

            MongoCustomerFilter<MongoTable> filterPet = new MongoCustomerFilter<MongoTable>();
            filterPet.AppendQueryItemEqFilter(MongoCombineTypeEnum.And, ("Code", "94215D6BA3EB4A6C93BEE684CEF96C8E51"));
            if (!await _reportTemplateDBRepository.Exists(filterPet.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_Preview_RO.jpg");
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Pet_Thum_RO.jpg");
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = "ro-RO"
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }

        /// <summary>
        /// 更新RO模板
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> InsertTemplateFromTemplateForRODBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} InsertTemplateFromTemplateForRODBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;

                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                //读取报告模板数据
                var reportDataPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "DefaultReportTemplateData.json");
                JObject reportDataJson = JObject.Parse(File.ReadAllText(reportDataPath));
                var defaultReportDatasJson = reportDataJson["DataConfig"].ToString();
                //读取模板配置
                var templateConfig = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "TemplateConfig.json");
                JObject json = JObject.Parse(File.ReadAllText(templateConfig));
                JArray array = (JArray)json["TemplateConfig"];
                foreach (JToken jObject in array)
                {
                    //读取分类
                    var category = jObject["Category"].ToString();//获取字符串中Category
                    var languageCode = jObject["LanguageCode"].ToString();//获取字符串中LanguageCode
                    if (category == "ThesaurusTemplate" && languageCode == "ro-RO")
                    {
                        await InsertThesaurusTemplateFromConfig(jObject);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 更新模板配置，包括词条和报告模板 
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> InsertTemplateFromTemplateDBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} InsertTemplateFromTemplateDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;

                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                //读取报告模板数据
                var reportDataPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "DefaultReportTemplateData.json");
                JObject reportDataJson = JObject.Parse(File.ReadAllText(reportDataPath));
                var defaultReportDatasJson = reportDataJson["DataConfig"].ToString();
                //读取模板配置
                var templateConfig = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "TemplateConfig.json");
                JObject json = JObject.Parse(File.ReadAllText(templateConfig));
                JArray array = (JArray)json["TemplateConfig"];
                foreach (JToken jObject in array)
                {
                    //读取分类
                    var category = jObject["Category"].ToString();//获取字符串中Category
                    if (category == "ReportTemplate")
                    {
                        await InsertReportTemplateFromConfig(jObject, defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                    }
                    else if (category == "ThesaurusTemplate")
                    {
                        await InsertThesaurusTemplateFromConfig(jObject);
                    }
                }
                return res;
            }
        }

        /// <summary>
        /// 插入配置的词条模板
        /// </summary>
        private async Task InsertThesaurusTemplateFromConfig(JToken jObject)
        {
            using (new Performance(_performanceThreshold, "{0} InsertThesaurusTemplateFromConfig execute", $"{this.GetType().Name}"))
            {
                MongoCustomerFilter<MongoTable> filterThesaurus = new MongoCustomerFilter<MongoTable>();
                List<(string, string)> eqFilters = new();
                eqFilters.Add(("ThesaurusName", jObject["Name"].ToString()));
                eqFilters.Add(("Code", jObject["Code"].ToString()));
                filterThesaurus.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, eqFilters.ToArray());
                if (!await _thesaurusDBRepository.Exists(filterThesaurus.AndCombine()))
                {
                    ThesaurusUsageTypeEnum thesaurusUsageTypeEnum = ThesaurusUsageTypeEnum.People;
                    if (jObject["UsageType"].ToString() == "Animal")
                    {
                        thesaurusUsageTypeEnum = ThesaurusUsageTypeEnum.Animal;
                    }
                    await AddThesaurusDO(jObject["Name"].ToString(), jObject["SourceFilePath"].ToString(), jObject["LanguageCode"].ToString(), thesaurusUsageTypeEnum, jObject["Code"].ToString());
                }
            }
        }

        /// <summary>
        /// 插入配置的报告模板
        /// </summary>
        private async Task InsertReportTemplateFromConfig(JToken jObject, string defaultReportDatasJson, string storageFold, string vcsStorageUrlPrefix)
        {
            var reportTemplate = new ReportTemplateDO();
            reportTemplate.Code = jObject["Code"].ToString();
            reportTemplate.ReportTemplateName = jObject["Name"].ToString();
            reportTemplate.IsDefault = true;
            reportTemplate.ReportTemplateJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", jObject["SourceFilePath"].ToString()));
            reportTemplate.ReportDatasJson = defaultReportDatasJson;
            reportTemplate.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            reportTemplate.ReportTemplateType = ReportTemplateTypeEnum.Default;
            if (jObject["UsageType"].ToString() == "Animal")
            {
                reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
            }
            else
            {
                reportTemplate.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            }
            reportTemplate.LanguageCode = jObject["LanguageCode"].ToString();

            MongoCustomerFilter<MongoTable> filterGeneral = new MongoCustomerFilter<MongoTable>();
            List<(string, string)> eqFilters = new();
            eqFilters.Add(("ReportTemplateName", reportTemplate.ReportTemplateName));
            eqFilters.Add(("Code", reportTemplate.Code));
            filterGeneral.AppendQueryItemEqFilter(MongoCombineTypeEnum.Or, eqFilters.ToArray());
            if (!await _reportTemplateDBRepository.Exists(filterGeneral.AndCombine()))
            {
                reportTemplate.TemplatePreviewList.Clear();
                var fileRUPreviewPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", jObject["PreviewPath"].ToString());
                var fileRUThumbnailPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", jObject["ThumbnailPath"].ToString());
                var fileRUPreviewUrl = "";
                var fileRUThumbnailUrl = "";
                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                if (File.Exists(fileRUPreviewPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUPreviewPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUPreviewPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUPreviewPath, waitCopyFilePath, true);
                        fileRUPreviewUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUPreviewPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUPreviewPath} Fail");
                        }
                        fileRUPreviewUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUPreviewUrl))
                        {
                            fileRUPreviewUrl = fileRUPreviewUrl.ToUrlToken();
                        }
                    }
                }
                if (File.Exists(fileRUThumbnailPath))
                {
                    if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                    {
                        string oriFilename = Path.GetFileNameWithoutExtension(fileRUThumbnailPath);
                        var factName = oriFilename;
                        var factNameExtension = Path.GetExtension(fileRUThumbnailPath);
                        var waitCopyFilePath = Path.Combine(storageFold, (factName + factNameExtension));
                        File.Copy(fileRUThumbnailPath, waitCopyFilePath, true);
                        fileRUThumbnailUrl = vcsStorageUrlPrefix + (factName + factNameExtension);
                    }
                    else
                    {
                        var fileList = _fileTransfer.FileTransferAsync(fileRUThumbnailPath, false).GetAwaiter().GetResult();
                        if (fileList == null || fileList.Count <= 0)
                        {
                            Logger.WriteLineInfo($"Upload ReportTemplate Preview Image {fileRUThumbnailPath} Fail");
                        }
                        fileRUThumbnailUrl = fileList.FirstOrDefault() ?? string.Empty;
                        if (!string.IsNullOrEmpty(fileRUThumbnailUrl))
                        {
                            fileRUThumbnailUrl = fileRUThumbnailUrl.ToUrlToken();
                        }
                    }
                }
                reportTemplate.TemplatePreviewList.Add(
                new ReportPreviewDO
                {
                    FileToken = fileRUPreviewUrl,//"https://cdn-bj.fis.plus/2CE2C5E324464135858D255D8A506DBF.jpg",
                    FileType = UploadFileTypeEnum.JPG,
                    ThumbnailToken = fileRUThumbnailUrl,//"https://cdn-bj.fis.plus/BEE3082883A942D1BACA7D46C5AEDD47.jpg",
                    Language = reportTemplate.LanguageCode
                });
                var res = await _reportTemplateDBRepository.InsertOneForAsyncDBAsync(reportTemplate, new Dictionary<string, string>(), string.Empty, false);
            }
        }
        #endregion


        /// <summary>
        /// 更新模板配置，包括词条和报告模板 
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<bool> InsertTemplateFromTemplateForSignBAsync()
        {
            using (new Performance(_performanceThreshold, "{0} InsertTemplateFromTemplateDBAsync execute", $"{this.GetType().Name}"))
            {
                var res = false;

                var storageFold = "";
                var vcsStorageUrlPrefix = "";
                if (_storageServer == WingInterfaceLibrary.Enum.StorageServerEnum.VCS.ToString())
                {
                    var secondDirectory = "";
                    var tempServerUrl = "";
                    if (!string.IsNullOrEmpty(_serverUrl))
                    {
                        tempServerUrl = _serverUrl.TrimEnd('/');
                        if (tempServerUrl.Split('/').Length > 0)
                        {
                            var arrayList = tempServerUrl.Split('/');
                            secondDirectory = arrayList[(arrayList.Length - 1)];
                        }
                    }
                    var curTime = DateTime.UtcNow.ToString("yyyyMMdd");
                    storageFold = Path.Combine(_storageDisk, secondDirectory, curTime);
                    if (!Directory.Exists(storageFold))
                    {
                        Directory.CreateDirectory(storageFold);
                    }
                    vcsStorageUrlPrefix = tempServerUrl + "/" + curTime + "/";
                }
                //读取报告模板数据
                var reportDataPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "DefaultReportTemplateData.json");
                JObject reportDataJson = JObject.Parse(File.ReadAllText(reportDataPath));
                var defaultReportDatasJson = reportDataJson["DataConfig"].ToString();
                //读取模板配置
                var templateConfig = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", "TemplateConfig.json");
                JObject json = JObject.Parse(File.ReadAllText(templateConfig));
                JArray array = (JArray)json["TemplateConfig"];
                foreach (JToken jObject in array)
                {
                    //读取分类
                    var category = jObject["Category"].ToString();//获取字符串中Category
                    var name = jObject["Name"].ToString();//获取字符串中Category
                    if (category == "ReportTemplate" && name == "系统默认签名模板")
                    {
                        await InsertReportTemplateFromConfig(jObject, defaultReportDatasJson, storageFold, vcsStorageUrlPrefix);
                    }
                }
                return res;
            }
        }

    }
}